@@ -2,13 +2,18 @@
 """Determine the number of resmoke jobs to run."""
 
 import argparse
+import logging
 import platform
 import re
 import sys
+from collections import defaultdict
 
 import psutil
+import structlog
 import yaml
 
+LOGGER = structlog.get_logger(__name__)
+
 CPU_COUNT = psutil.cpu_count()
 PLATFORM_MACHINE = platform.machine()
 SYS_PLATFORM = sys.platform
@@ -25,22 +30,38 @@ SYS_PLATFORM = sys.platform
 
 # Apply factor for a task based on the build variant it is running on.
 VARIANT_TASK_FACTOR_OVERRIDES = {
-    "enterprise-rhel-62-64-bit": [{"task": r"logical_session_cache_replication.*", "factor": 0.75}],
-    "enterprise-rhel-62-64-bit-inmem": [{"task": "secondary_reads_passthrough", "factor": 0.3}]
+    "enterprise-rhel-80-64-bit": [{"task": r"logical_session_cache_replication.*", "factor": 0.75}],
+    "enterprise-rhel-80-64-bit-inmem": [{"task": "secondary_reads_passthrough", "factor": 0.3}]
 }
 
 TASKS_FACTORS = [{"task": r"replica_sets.*", "factor": 0.5}, {"task": r"sharding.*", "factor": 0.5}]
 
+DISTRO_MULTIPLIERS = {"rhel80-large": 1.618}
+
 # Apply factor for a task based on the machine type it is running on.
-MACHINE_TASK_FACTOR_OVERRIDES = {"aarch64": TASKS_FACTORS}
+MACHINE_TASK_FACTOR_OVERRIDES = {
+    "aarch64":
+        TASKS_FACTORS,
+    "ppc64le": [
+        dict(task=r"causally_consistent_hedged_reads_jscore_passthrough.*", factor=0.125),
+        dict(task=r"causally_consistent_read_concern_snapshot_passthrough.*", factor=0.125),
+        dict(task=r"sharded_causally_consistent_read_concern_snapshot_passthrough.*", factor=0.125),
+    ],
+}
 
 # Apply factor for a task based on the platform it is running on.
 PLATFORM_TASK_FACTOR_OVERRIDES = {"win32": TASKS_FACTORS, "cygwin": TASKS_FACTORS}
 
 # Apply factor for a task everywhere it is run.
 GLOBAL_TASK_FACTOR_OVERRIDES = {
+    r"causally_consistent_hedged_reads_jscore_passthrough.*": 0.25,
     r"logical_session_cache.*_refresh_jscore_passthrough.*": 0.25,
     r"multi_shard_.*multi_stmt_txn_.*jscore_passthrough.*": 0.125,
+    r"replica_sets_reconfig_jscore_passthrough.*": 0.25,
+    r"replica_sets_reconfig_jscore_stepdown_passthrough.*": 0.25,
+    r"replica_sets_reconfig_kill_primary_jscore_passthrough.*": 0.25,
+    r"sharded_causally_consistent_jscore_passthrough.*": 0.75,
+    r"sharded_collections_jscore_passthrough.*": 0.75,
 }
 
 
@@ -68,7 +89,13 @@ def get_task_factor(task_name, overrides, override_type, factor):
     return factor
 
 
-def determine_factor(task_name, variant, factor):
+def determine_final_multiplier(distro):
+    """Determine the final multiplier."""
+    multipliers = defaultdict(lambda: 1, DISTRO_MULTIPLIERS)
+    return multipliers[distro]
+
+
+def determine_factor(task_name, variant, distro, factor):
     """Determine the job factor."""
     factors = [
         get_task_factor(task_name, MACHINE_TASK_FACTOR_OVERRIDES, PLATFORM_MACHINE, factor),
@@ -76,16 +103,16 @@ def determine_factor(task_name, variant, factor):
         get_task_factor(task_name, VARIANT_TASK_FACTOR_OVERRIDES, variant, factor),
         global_task_factor(task_name, GLOBAL_TASK_FACTOR_OVERRIDES, factor),
     ]
-    return min(factors)
+    return min(factors) * determine_final_multiplier(distro)
 
 
-def determine_jobs(task_name, variant, jobs_max=0, job_factor=1.0):
+def determine_jobs(task_name, variant, distro, jobs_max=0, job_factor=1.0):
     """Determine the resmoke jobs."""
     if jobs_max < 0:
         raise ValueError("The jobs_max must be >= 0.")
     if job_factor <= 0:
         raise ValueError("The job_factor must be > 0.")
-    factor = determine_factor(task_name, variant, job_factor)
+    factor = determine_factor(task_name, variant, distro, job_factor)
     jobs_available = int(round(CPU_COUNT * factor))
     if jobs_max == 0:
         return max(1, jobs_available)
@@ -110,6 +137,8 @@ def main():
     parser.add_argument("--taskName", dest="task", required=True, help="Task being executed.")
     parser.add_argument("--buildVariant", dest="variant", required=True,
                         help="Build variant task is being executed on.")
+    parser.add_argument("--distro", dest="distro", required=True,
+                        help="Distro task is being executed on.")
     parser.add_argument(
         "--jobFactor", dest="jobs_factor", type=float, default=1.0,
         help=("Job factor to use as a mulitplier with the number of CPUs. Defaults"
@@ -125,7 +154,14 @@ def main():
 
     options = parser.parse_args()
 
-    jobs = determine_jobs(options.task, options.variant, options.jobs_max, options.jobs_factor)
+    logging.basicConfig(stream=sys.stdout, level=logging.INFO)
+    structlog.configure(logger_factory=structlog.stdlib.LoggerFactory())
+
+    LOGGER.info("Finding job count", task=options.task, variant=options.variant,
+                platform=PLATFORM_MACHINE, sys=SYS_PLATFORM, cpu_count=CPU_COUNT)
+
+    jobs = determine_jobs(options.task, options.variant, options.distro, options.jobs_max,
+                          options.jobs_factor)
     if jobs < CPU_COUNT:
         print("Reducing number of jobs to run from {} to {}".format(CPU_COUNT, jobs))
     output_jobs(jobs, options.outfile)
