@@ -30,23 +30,26 @@ def make_process(*args, **kwargs):
 
     # Add the current working directory and /data/multiversion to the PATH.
     env_vars = kwargs.get("env_vars", {}).copy()
-    path = [
-        os.getcwd(),
-        config.DEFAULT_MULTIVERSION_DIR,
-    ]
+    path = get_path_env_var(env_vars)
 
-    # If installDir is provided, add it early to the path
     if config.INSTALL_DIR is not None:
-        path.append(config.INSTALL_DIR)
         env_vars["INSTALL_DIR"] = config.INSTALL_DIR
 
-    path.append(env_vars.get("PATH", os.environ.get("PATH", "")))
-
     env_vars["PATH"] = os.pathsep.join(path)
     kwargs["env_vars"] = env_vars
     return process_cls(*args, **kwargs)
 
 
+def get_path_env_var(env_vars):
+    """Return the path base on provided environment variable."""
+    path = [os.getcwd()] + config.DEFAULT_MULTIVERSION_DIRS
+    # If installDir is provided, add it early to the path
+    if config.INSTALL_DIR is not None:
+        path.append(config.INSTALL_DIR)
+    path.append(env_vars.get("PATH", os.environ.get("PATH", "")))
+    return path
+
+
 def mongod_program(logger, job_num, executable, process_kwargs, mongod_options):
     """
     Return a Process instance that starts mongod arguments constructed from 'mongod_options'.
@@ -145,9 +148,6 @@ def mongo_shell_program(  # pylint: disable=too-many-arguments,too-many-branches
         "wiredTigerCollectionConfigString": (config.WT_COLL_CONFIG, ""),
         "wiredTigerEngineConfigString": (config.WT_ENGINE_CONFIG, ""),
         "wiredTigerIndexConfigString": (config.WT_INDEX_CONFIG, ""),
-
-        # Evergreen variables.
-        "evergreenDebugSymbolsUrl": (config.DEBUG_SYMBOLS_URL, ""),
     }
 
     test_data = global_vars.get("TestData", {}).copy()
@@ -163,6 +163,7 @@ def mongo_shell_program(  # pylint: disable=too-many-arguments,too-many-branches
 
     if config.EVERGREEN_TASK_ID is not None:
         test_data["inEvergreen"] = True
+        test_data["evergreenTaskId"] = config.EVERGREEN_TASK_ID
 
     # Initialize setParameters for mongod and mongos, to be passed to the shell via TestData. Since
     # they are dictionaries, they will be converted to JavaScript objects when passed to the shell
@@ -191,8 +192,8 @@ def mongo_shell_program(  # pylint: disable=too-many-arguments,too-many-branches
 
     # If the 'logComponentVerbosity' setParameter for mongod was not already specified, we set its
     # value to a default.
-    mongod_set_parameters.setdefault("logComponentVerbosity",
-                                     mongod_launcher.default_mongod_log_component_verbosity())
+    mongod_set_parameters.setdefault(
+        "logComponentVerbosity", mongod_launcher.get_default_log_component_verbosity_for_mongod())
 
     # If the 'enableFlowControl' setParameter for mongod was not already specified, we set its value
     # to a default.
@@ -260,12 +261,6 @@ def mongo_shell_program(  # pylint: disable=too-many-arguments,too-many-branches
     args.append("--eval")
     args.append(eval_str)
 
-    if config.SHELL_READ_MODE is not None:
-        kwargs["readMode"] = config.SHELL_READ_MODE
-
-    if config.SHELL_WRITE_MODE is not None:
-        kwargs["writeMode"] = config.SHELL_WRITE_MODE
-
     if connection_string is not None:
         # The --host and --port options are ignored by the mongo shell when an explicit connection
         # string is specified. We remove these options to avoid any ambiguity with what server the
