@@ -36,6 +36,7 @@ from buildscripts.resmokelib.powercycle import powercycle_config, powercycle_con
 
 # See https://docs.python.org/2/library/sys.html#sys.platform
 from buildscripts.resmokelib.powercycle.lib.services import WindowsService, PosixService
+from buildscripts.resmokelib.utils.filesystem import build_hygienic_bin_path
 
 _IS_WINDOWS = sys.platform == "win32" or sys.platform == "cygwin"
 _IS_LINUX = sys.platform.startswith("linux")
@@ -1543,7 +1544,9 @@ def main(parser_actions, options):  # pylint: disable=too-many-branches,too-many
         host_port = f"{mongod_host}:{secret_port}"
         new_config_file = NamedTempFile.create(suffix=".yml", directory="tmp")
         temp_client_files.append(new_config_file)
-        validation_test_data = {"skipValidationOnNamespaceNotFound": True}
+        validation_test_data = {
+            "skipValidationOnNamespaceNotFound": True, "allowUncleanShutdowns": True
+        }
         new_resmoke_config(with_external_server, new_config_file, validation_test_data)
         ret, output = resmoke_client(mongo_repo_root_dir, mongo_path, host_port,
                                      "jstests/hooks/run_validate_collections.js", new_config_file)
