@@ -1,10 +1,9 @@
 // Tests that text search requires a text index.
-// TODO: Reenable test on passthroughs with sharded collections as part of SERVER-38996.
-// @tags: [assumes_unsharded_collection]
 (function() {
 "use strict";
 
 load("jstests/aggregation/extras/utils.js");  // For "assertErrorCode".
+load("jstests/libs/fixture_helpers.js");      // For isSharded.
 
 const coll = db.coll;
 const from = db.from;
@@ -12,6 +11,15 @@ const from = db.from;
 coll.drop();
 from.drop();
 
+// Do not run the rest of the tests if the foreign collection is implicitly sharded but the flag to
+// allow $lookup/$graphLookup into a sharded collection is disabled.
+const getShardedLookupParam = db.adminCommand({getParameter: 1, featureFlagShardedLookup: 1});
+const isShardedLookupEnabled = getShardedLookupParam.hasOwnProperty("featureFlagShardedLookup") &&
+    getShardedLookupParam.featureFlagShardedLookup.value;
+if (FixtureHelpers.isSharded(from) && !isShardedLookupEnabled) {
+    return;
+}
+
 const textPipeline = [{$match: {$text: {$search: "foo"}}}];
 
 const pipeline = [
