@@ -3,9 +3,9 @@
  * @tags: [requires_replication, requires_sharding]
  */
 
-// TODO SERVER-35447: This test logs in users on the admin database, but doesn't log them out, which
-// can fail with implicit sessions and ReplSetTest when the fixture attempts to verify data hashes
-// at shutdown by authenticating as the __system user.
+// This test logs in users on the admin database, but doesn't log them out, which can fail with
+// implicit sessions and ReplSetTest when the fixture attempts to verify data hashes at shutdown by
+// authenticating as the __system user.
 TestData.disableImplicitSessions = true;
 
 function setup_users(granter) {
@@ -28,16 +28,14 @@ function setup_dbs_and_cols(db) {
     var test_db_a = db.getSiblingDB("a");
     var test_db_b = db.getSiblingDB("b");
 
-    test_db_a.dropDatabase();
-    test_db_b.dropDatabase();
+    assert.commandWorked(test_db_a.dropDatabase({w: 'majority'}));
+    assert.commandWorked(test_db_b.dropDatabase({w: 'majority'}));
 
-    test_db_a.createCollection("a");
-    test_db_a.createCollection("b");
+    assert.commandWorked(test_db_a.createCollection("a", {writeConcern: {w: 'majority'}}));
+    assert.commandWorked(test_db_a.createCollection("b", {writeConcern: {w: 'majority'}}));
 
-    test_db_b.createCollection("a");
-    test_db_b.createCollection("b");
-
-    db.getLastError('majority');
+    assert.commandWorked(test_db_b.createCollection("a", {writeConcern: {w: 'majority'}}));
+    assert.commandWorked(test_db_b.createCollection("b", {writeConcern: {w: 'majority'}}));
 }
 
 function grant_privileges(granter, privileges) {
@@ -85,7 +83,8 @@ function run_test(name, granter, verifier, privileges, collections) {
     grant_privileges(granter, privileges);
     invalidateUserCache(verifier);
 
-    verifier.getSiblingDB('admin').auth("test_user", "password");
+    const verifierDB = verifier.getSiblingDB('admin');
+    verifierDB.auth("test_user", "password");
 
     for (var key in collections) {
         var parts = key.split(".");
@@ -98,6 +97,7 @@ function run_test(name, granter, verifier, privileges, collections) {
     }
 
     revoke_privileges(granter, privileges);
+    verifierDB.logout();
 }
 
 function run_test_bad_resource(name, granter, resource) {
