@@ -1,21 +1,24 @@
-
 /**
  * @tags: [
  *   uses_multiple_connections,
  * ]
  */
-t = db.bench_test2;
+(function() {
+"use strict";
+
+const t = db.bench_test2;
 t.drop();
 
-for (i = 0; i < 100; i++)
-    t.insert({_id: i, x: 0});
+for (let i = 0; i < 100; i++)
+    assert.commandWorked(t.insert({_id: i, x: 0}));
 
-benchArgs = {
+const benchArgs = {
     ops: [{
         ns: t.getFullName(),
         op: "update",
         query: {_id: {"#RAND_INT": [0, 100]}},
-        update: {$inc: {x: 1}}
+        update: {$inc: {x: 1}},
+        writeCmd: true
     }],
     parallel: 2,
     seconds: 1,
@@ -28,14 +31,14 @@ if (jsTest.options().auth) {
     benchArgs['password'] = jsTest.options().authPassword;
 }
 
-res = benchRun(benchArgs);
+const res = benchRun(benchArgs);
 printjson(res);
 
-sumsq = 0;
-sum = 0;
+let sumsq = 0;
+let sum = 0;
 
-min = 1000;
-max = 0;
+let min = 1000;
+let max = 0;
 t.find().forEach(function(z) {
     sum += z.x;
     sumsq += Math.pow((res.update / 100) - z.x, 2);
@@ -43,10 +46,11 @@ t.find().forEach(function(z) {
     max = Math.max(z.x, max);
 });
 
-avg = sum / 100;
-std = Math.sqrt(sumsq / 100);
+const avg = sum / 100;
+const std = Math.sqrt(sumsq / 100);
 
 print("Avg: " + avg);
 print("Std: " + std);
 print("Min: " + min);
 print("Max: " + max);
+})();
