@@ -2,11 +2,13 @@
  * @tags: [
  *   assumes_superuser_permissions,
  * ]
+ * fcv49 for the change to error code in createIndexes invalid field reply.
  */
 (function() {
 'use strict';
 
-var isMongos = ("isdbgrid" == db.runCommand("ismaster").msg);
+const kUnknownIDLFieldError = 40415;
+var isMongos = ("isdbgrid" == db.runCommand("hello").msg);
 
 var extractResult = function(obj) {
     if (!isMongos)
@@ -37,12 +39,11 @@ var extractResult = function(obj) {
     return result;
 };
 
-var checkImplicitCreate = function(createIndexResult, isMongos) {
-    let allowImplicit = !isMongos;
-    assert.eq(allowImplicit, createIndexResult.createdCollectionAutomatically);
+var checkImplicitCreate = function(createIndexResult) {
+    assert.eq(true, createIndexResult.createdCollectionAutomatically);
 };
 
-var dbTest = db.getSisterDB('create_indexes_db');
+var dbTest = db.getSiblingDB('create_indexes_db');
 dbTest.dropDatabase();
 
 // Database does not exist
@@ -50,7 +51,7 @@ var collDbNotExist = dbTest.create_indexes_no_db;
 var res = assert.commandWorked(
     collDbNotExist.runCommand('createIndexes', {indexes: [{key: {x: 1}, name: 'x_1'}]}));
 res = extractResult(res);
-checkImplicitCreate(res, isMongos);
+checkImplicitCreate(res);
 assert.eq(1, res.numIndexesBefore);
 assert.eq(2, res.numIndexesAfter);
 assert.isnull(
@@ -62,7 +63,7 @@ var t = dbTest.create_indexes;
 var res =
     assert.commandWorked(t.runCommand('createIndexes', {indexes: [{key: {x: 1}, name: 'x_1'}]}));
 res = extractResult(res);
-checkImplicitCreate(res, isMongos);
+checkImplicitCreate(res);
 assert.eq(1, res.numIndexesBefore);
 assert.eq(2, res.numIndexesAfter);
 assert.isnull(
@@ -149,7 +150,7 @@ assert.commandWorked(res, 'v1 index creation should succeed');
 
 // Test that index creation fails with an invalid top-level field.
 res = t.runCommand('createIndexes', {indexes: [{key: {e: 1}, name: 'e_1'}], 'invalidField': 1});
-assert.commandFailedWithCode(res, ErrorCodes.BadValue);
+assert.commandFailedWithCode(res, kUnknownIDLFieldError);
 
 // Test that index creation fails with an invalid field in the index spec for index version V2.
 res = t.runCommand('createIndexes',
@@ -169,6 +170,15 @@ assert.commandFailedWithCode(res, ErrorCodes.BadValue);
 res = t.runCommand('createIndexes', {indexes: [{key: {f: ""}, name: 'f_1'}]});
 assert.commandFailedWithCode(res, ErrorCodes.CannotCreateIndex);
 
+// Test that index creation fails with duplicate index names in the index specs.
+res = t.runCommand('createIndexes', {
+    indexes: [
+        {key: {g: 1}, name: 'myidx'},
+        {key: {h: 1}, name: 'myidx'},
+    ],
+});
+assert.commandFailedWithCode(res, ErrorCodes.IndexKeySpecsConflict);
+
 // Test that user is not allowed to create indexes in config.transactions.
 var configDB = db.getSiblingDB('config');
 res =
