@@ -2,13 +2,11 @@
  * Cannot implicitly shard accessed collections because the explain output from a mongod when run
  * against a sharded collection is wrapped in a "shards" object with keys for each shard.
  *
- * @tags: [assumes_unsharded_collection,
- *         does_not_support_stepdowns,
- *         requires_fastcount,
- *         # Projection push down works differently between 4.2 and 4.4. explain() provides a
- *         # different output between the two versions. This test expects only the 4.4 version
- *         # output.
- *         requires_fcv_44]
+ * @tags: [
+ *   assumes_unsharded_collection,
+ *   does_not_support_stepdowns,
+ *   requires_fastcount,
+ * ]
  */
 
 // Tests for the .explain() shell helper, which provides syntactic sugar for the explain command.
@@ -22,7 +20,7 @@ load("jstests/libs/analyze_plan.js");
 var explain;
 var stage;
 
-t.ensureIndex({a: 1});
+t.createIndex({a: 1});
 for (var i = 0; i < 10; i++) {
     t.insert({_id: i, a: i, b: 1});
 }
@@ -126,40 +124,40 @@ assert.commandWorked(explain);
 // .sort()
 explain = t.explain().find().sort({b: -1}).finish();
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "SORT"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "SORT"));
 explain = t.find().sort({b: -1}).explain();
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "SORT"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "SORT"));
 
 // .hint()
 explain = t.explain().find().hint({a: 1}).finish();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 explain = t.explain().find().hint("a_1").finish();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 explain = t.find().hint({a: 1}).explain();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 explain = t.find().hint("a_1").explain();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 
 // .min()
 explain = t.explain().find().min({a: 1}).hint({a: 1}).finish();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 explain = t.find().min({a: 1}).hint({a: 1}).explain();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 
 // .max()
 explain = t.explain().find().max({a: 1}).hint({a: 1}).finish();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 explain = t.find().max({a: 1}).hint({a: 1}).explain();
 assert.commandWorked(explain);
-assert(isIxscan(db, explain.queryPlanner.winningPlan));
+assert(isIxscan(db, getWinningPlan(explain.queryPlanner)));
 
 // .allowDiskUse()
 explain = t.explain().find().allowDiskUse().finish();
@@ -174,15 +172,17 @@ explain = t.find().showDiskLoc().explain();
 assert.commandWorked(explain);
 
 // .maxTimeMS()
-explain = t.explain().find().maxTimeMS(200).finish();
+// Provide longer maxTime when the test runs in suites which can affect query execution time.
+const numConn = db.serverStatus().connections.current;
+explain = t.explain().find().maxTimeMS(200 * numConn).finish();
 assert.commandWorked(explain);
-explain = t.find().maxTimeMS(200).explain();
+explain = t.find().maxTimeMS(200 * numConn).explain();
 assert.commandWorked(explain);
 
 // .readPref()
-explain = t.explain().find().readPref("secondary").finish();
+explain = t.explain().find().readPref("secondaryPreferred").finish();
 assert.commandWorked(explain);
-explain = t.find().readPref("secondary").explain();
+explain = t.find().readPref("secondaryPreferred").explain();
 assert.commandWorked(explain);
 
 // .comment()
@@ -243,7 +243,7 @@ assert("queryPlanner" in explain.stages[0].$cursor);
 // Basic count.
 explain = t.explain().count();
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "RECORD_STORE_FAST_COUNT"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "RECORD_STORE_FAST_COUNT"));
 
 // Tests for applySkipLimit argument to .count. When we don't apply the skip, we
 // count one result. When we do apply the skip we count zero.
@@ -263,15 +263,15 @@ assert.eq(0, stage.nCounted);
 // Count with hint.
 explain = t.explain().find({a: 3}).hint({a: 1}).count();
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "COUNT"));
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "COUNT_SCAN"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "COUNT"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "COUNT_SCAN"));
 
 // Explainable count with hint.
-assert.commandWorked(t.ensureIndex({c: 1}, {sparse: true}));
+assert.commandWorked(t.createIndex({c: 1}, {sparse: true}));
 explain = t.explain().count({c: {$exists: false}}, {hint: "c_1"});
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "IXSCAN"));
-assert.eq(getPlanStage(explain.queryPlanner.winningPlan, "IXSCAN").indexName, "c_1");
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "IXSCAN"));
+assert.eq(getPlanStage(getWinningPlan(explain.queryPlanner), "IXSCAN").indexName, "c_1");
 assert.commandWorked(t.dropIndex({c: 1}));
 
 //
@@ -280,17 +280,17 @@ assert.commandWorked(t.dropIndex({c: 1}));
 
 explain = t.explain().distinct('_id');
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "PROJECTION_COVERED"));
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "DISTINCT_SCAN"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "PROJECTION_COVERED"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "DISTINCT_SCAN"));
 
 explain = t.explain().distinct('a');
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "PROJECTION_COVERED"));
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "DISTINCT_SCAN"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "PROJECTION_COVERED"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "DISTINCT_SCAN"));
 
 explain = t.explain().distinct('b');
 assert.commandWorked(explain);
-assert(planHasStage(db, explain.queryPlanner.winningPlan, "COLLSCAN"));
+assert(planHasStage(db, getWinningPlan(explain.queryPlanner), "COLLSCAN"));
 
 //
 // .remove()
@@ -343,7 +343,7 @@ if ("SHARD_WRITE" === stage.stage) {
     stage = stage.shards[0].executionStages;
 }
 assert.eq(stage.stage, "UPDATE");
-assert(stage.wouldInsert);
+assert(stage.nWouldUpsert == 1);
 
 // Make sure that the insert didn't actually happen.
 assert.eq(10, t.count());
@@ -356,7 +356,7 @@ if ("SHARD_WRITE" === stage.stage) {
     stage = stage.shards[0].executionStages;
 }
 assert.eq(stage.stage, "UPDATE");
-assert(stage.wouldInsert);
+assert(stage.nWouldUpsert == 1);
 assert.eq(0, stage.nMatched);
 
 // Make sure that the insert didn't actually happen.
@@ -370,7 +370,7 @@ if ("SHARD_WRITE" === stage.stage) {
     stage = stage.shards[0].executionStages;
 }
 assert.eq(stage.stage, "UPDATE");
-assert(!stage.wouldInsert);
+assert(stage.nWouldUpsert == 0);
 assert.eq(3, stage.nMatched);
 assert.eq(3, stage.nWouldModify);
 
@@ -382,7 +382,7 @@ if ("SHARD_WRITE" === stage.stage) {
     stage = stage.shards[0].executionStages;
 }
 assert.eq(stage.stage, "UPDATE");
-assert(!stage.wouldInsert);
+assert(stage.nWouldUpsert == 0);
 assert.eq(3, stage.nMatched);
 assert.eq(3, stage.nWouldModify);
 
@@ -415,7 +415,7 @@ if ("SINGLE_SHARD" === stage.stage) {
     stage = stage.shards[0].executionStages;
 }
 assert.eq(stage.stage, "UPDATE");
-assert(stage.wouldInsert);
+assert(stage.nWouldUpsert == 1);
 
 // Make sure that the insert didn't actually happen.
 assert.eq(10, t.count());
