@@ -7,6 +7,7 @@
 //   assumes_read_preference_unchanged,
 //   does_not_support_stepdowns,
 // ]
+load("jstests/libs/logv2_helpers.js");
 
 (function() {
 'use strict';
@@ -29,8 +30,18 @@ function contains(arr, func) {
     return false;
 }
 
+function stringContains(haystack, needle) {
+    if (isJsonLog(db.getMongo())) {
+        if (needle.indexOf(":"))
+            needle = '"' + needle.replace(':', "\":");
+        needle = needle.replace(/ /g, "");
+    }
+
+    return haystack.indexOf(needle) != -1;
+}
+
 // test doesn't work when talking to mongos
-if (db.isMaster().msg === "isdbgrid") {
+if (db.hello().msg === "isdbgrid") {
     return;
 }
 
@@ -51,10 +62,8 @@ assert.gt(query.log.length, 0, "no log lines");
 // Ensure that slow query is logged in detail.
 assert(contains(query.log, function(v) {
     print(v);
-    const opString = db.getMongo().useReadCommands() ? " find " : " query ";
-    const filterString = db.getMongo().useReadCommands() ? "filter:" : "command:";
-    return v.indexOf(opString) != -1 && v.indexOf(filterString) != -1 &&
-        v.indexOf("keysExamined:") != -1 && v.indexOf("docsExamined:") != -1 &&
+    return stringContains(v, " find ") && stringContains(v, "filter:") &&
+        stringContains(v, "keysExamined:") && stringContains(v, "docsExamined:") &&
         v.indexOf("SENTINEL") != -1;
 }));
 
@@ -74,9 +83,8 @@ assert.gt(update.log.length, 0, "no log lines");
 
 // Ensure that slow update is logged in deail.
 assert(contains(update.log, function(v) {
-    print(v);
-    return v.indexOf(" update ") != -1 && v.indexOf("command") != -1 &&
-        v.indexOf("keysExamined:") != -1 && v.indexOf("docsExamined:") != -1 &&
+    return stringContains(v, " update ") != -1 && stringContains(v, "command") &&
+        stringContains(v, "keysExamined:") && stringContains(v, "docsExamined:") &&
         v.indexOf("SENTINEL") != -1;
 }));
 })();
