@@ -1,14 +1,8 @@
 /**
  * Tests execution of pipeline-style update.
- *
- * 'requires_find_command' needed to prevent this test from running with 'compatibility' write mode
- * as pipeline-style update is not supported by OP_UPDATE.
- *
  * @tags: [
- *   requires_find_command,
  *   requires_multi_updates,
  *   requires_non_retryable_writes,
- *   requires_fcv_50,
  * ]
  */
 (function() {
@@ -186,6 +180,7 @@ assert.commandFailedWithCode(coll.update({x: 1}, [{$match: {x: 1}}]), ErrorCodes
 assert.commandFailedWithCode(coll.update({x: 1}, [{$sort: {x: 1}}]), ErrorCodes.InvalidOptions);
 assert.commandFailedWithCode(coll.update({x: 1}, [{$facet: {a: [{$match: {x: 1}}]}}]),
                              ErrorCodes.InvalidOptions);
+assert.commandFailedWithCode(coll.update({x: 1}, [{$indexStats: {}}]), ErrorCodes.InvalidOptions);
 assert.commandFailedWithCode(
     coll.update(
         {x: 1}, [{
@@ -203,11 +198,6 @@ assert.commandFailedWithCode(
         }]),
     ErrorCodes.InvalidOptions);
 
-// $indexStats is not supported in a transaction passthrough and will fail with a different error.
-assert.commandFailedWithCode(
-    coll.update({x: 1}, [{$indexStats: {}}]),
-    [ErrorCodes.InvalidOptions, ErrorCodes.OperationNotSupportedInTransaction]);
-
 // Update fails when supported agg stage is specified outside of pipeline.
 assert.commandFailedWithCode(coll.update({_id: 1}, {$addFields: {x: 1}}), ErrorCodes.FailedToParse);
 
@@ -266,23 +256,12 @@ testUpdate({
     nModified: 1
 });
 
-const isDotsAndDollarsEnabled = db.adminCommand({getParameter: 1, featureFlagDotsAndDollars: 1})
-                                    .featureFlagDotsAndDollars.value;
-if (isDotsAndDollarsEnabled) {
-    // Test that expressions within constants are treated as field names instead of expressions.
-    db.runCommand({
-        update: collName,
-        updates: [{q: {_id: 1}, u: [{$set: {x: "$$foo"}}], c: {foo: {$add: [1, 2]}}}]
-    });
-    assert.eq([{_id: 1, x: {$add: [1, 2]}, foo: "$x"}], coll.find({_id: 1}).toArray());
-} else {
-    // Cannot use expressions in constants.
-    assert.commandFailedWithCode(db.runCommand({
-        update: collName,
-        updates: [{q: {_id: 1}, u: [{$set: {x: "$$foo"}}], c: {foo: {$add: [1, 2]}}}]
-    }),
-                                 ErrorCodes.DollarPrefixedFieldName);
-}
+// Test that expressions within constants are treated as field names instead of expressions.
+db.runCommand({
+    update: collName,
+    updates: [{q: {_id: 1}, u: [{$set: {x: "$$foo"}}], c: {foo: {$add: [1, 2]}}}]
+});
+assert.eq([{_id: 1, x: {$add: [1, 2]}, foo: "$x"}], coll.find({_id: 1}).toArray());
 
 // Cannot use constants with regular updates.
 assert.commandFailedWithCode(
