@@ -115,7 +115,6 @@ if (typeof _threadInject != "undefined") {
     // Helper class for running tests in parallel.  It assembles a set of tests
     // and then calls assert.parallelests to run them.
     ParallelTester = function() {
-        assert.neq(db.getMongo().writeMode(), "legacy", "wrong shell write mode");
         this.params = new Array();
     };
 
@@ -164,12 +163,6 @@ if (typeof _threadInject != "undefined") {
             "run_program1.js",
             "bench_test1.js",
 
-            // These tests use the getLastError command, which is unsafe to use in this environment,
-            // since a previous test's cursors could be garbage collected in the middle of the next
-            // test, which would reset the last error associated with the shell's client.
-            "dropdb_race.js",
-            "bulk_legacy_enforce_gle.js",
-
             // These tests use getLog to examine the logs. Tests which do so shouldn't be run in
             // this suite because any test being run at the same time could conceivably spam the
             // logs so much that the line they are looking for has been rotated off the server's
@@ -207,10 +200,6 @@ if (typeof _threadInject != "undefined") {
             "views/views_all_commands.js",        // Drops test DB.
             "views/view_with_invalid_dbname.js",  // Puts invalid view definitions in system.views.
 
-            // This test works close to the BSON document limit for entries in the durable catalog,
-            // so running it in parallel with other tests will cause failures.
-            "long_collection_names.js",
-
             // This test causes collMod commands to hang, which interferes with other tests running
             // collMod.
             "crud_ops_do_not_throw_locktimeout.js",
@@ -228,7 +217,15 @@ if (typeof _threadInject != "undefined") {
             // Assumes that other tests are not creating API version 1 incompatible data.
             "validate_db_metadata_command.js",
 
-            // This test causes delete commands to hang, which may affect other tests running
+            // The tests in 'bench_test*.js' files use 'benchRun()'. The main purpose of
+            // 'benchRun()' is for performance testing and the 'benchRun()' implementation itself
+            // launches multiple threads internally, it's not necessary to keep 'bench_test*.js'
+            // within the parallel test job.
+            "bench_test1.js",
+            "bench_test2.js",
+            "bench_test3.js",
+
+            // These tests cause deletes and updates to hang, which may affect other tests running
             // concurrently.
             "timeseries/timeseries_delete_hint.js",
             "timeseries/timeseries_update_hint.js",
@@ -250,41 +247,8 @@ if (typeof _threadInject != "undefined") {
             return fileList;
         };
 
-        // The following tests cannot run when shell readMode is legacy.
-        if (db.getMongo().readMode() === "legacy") {
-            var requires_find_command = [
-                "apply_ops_system_dot_views.js",
-                "command_let_variables.js",
-                "doc_validation_error.js",
-                "internal_apply_oplog_update.js",
-                "merge_sort_collation.js",
-                "explode_for_sort_fetch.js",
-                "or_to_in.js",
-                "update_pipeline_shell_helpers.js",
-                "update_with_pipeline.js",
-                "verify_update_mods.js",
-                "views/dbref_projection.js",
-                "views/views_aggregation.js",
-                "views/views_change.js",
-                "views/views_drop.js",
-                "views/views_find.js",
-                "wildcard_index_collation.js"
-            ];
-            Object.assign(skipTests, makeKeys(requires_find_command));
-
-            // Time-series collections require support for views, so are incompatible with legacy
-            // readMode.
-            const timeseriesTestFiles =
-                getFilesRecursive('jstests/core/timeseries').map(f => ('timeseries/' + f.baseName));
-            Object.assign(skipTests, makeKeys(timeseriesTestFiles));
-        }
-
         // Transactions are not supported on standalone nodes so we do not run them here.
-        // NOTE: We need to take substring of the full test path to ensure that 'jstests/core/' is
-        // not included.
-        const txnsTestFiles =
-            getFilesRecursive("jstests/core/txns/")
-                .map(fullPathToTest => fullPathToTest.name.substring("jstests/core/".length));
+        let txnsTestFiles = getFilesRecursive("jstests/core/txns").map(f => ("txns/" + f.baseName));
         Object.assign(skipTests, makeKeys(txnsTestFiles));
 
         var parallelFilesDir = "jstests/core";
