@@ -38,16 +38,16 @@ const expectedParamDefaults = {
     internalLookupStageIntermediateDocumentMaxSizeBytes: 100 * 1024 * 1024,
     internalDocumentSourceGroupMaxMemoryBytes: 100 * 1024 * 1024,
     internalDocumentSourceSetWindowFieldsMaxMemoryBytes: 100 * 1024 * 1024,
-    internalPipelineLengthLimit: 1000,
     internalQueryMaxJsEmitBytes: 100 * 1024 * 1024,
     internalQueryMaxPushBytes: 100 * 1024 * 1024,
+    internalQueryMaxRangeBytes: 100 * 1024 * 1024,
     internalQueryMaxAddToSetBytes: 100 * 1024 * 1024,
     // Should be half the value of 'internalQueryExecYieldIterations' parameter.
     internalInsertMaxBatchSize: 500,
     internalQueryPlannerGenerateCoveredWholeIndexScans: false,
     internalQueryIgnoreUnknownJSONSchemaKeywords: false,
     internalQueryProhibitBlockingMergeOnMongoS: false,
-    internalQuerySlotBasedExecutionMaxStaticIndexScanIntervals: 1000,
+    internalQuerySlotBasedExecutionMaxStaticIndexScanIntervals: 1000
 };
 
 function assertDefaultParameterValues() {
@@ -74,6 +74,13 @@ function assertSetParameterFails(paramName, value) {
                                  ErrorCodes.BadValue);
 }
 
+// InternalPipelineLengthLimit is different is lowered in a debug build so its expected value
+// depends on that
+const getParamRes =
+    assert.commandWorked(testDB.adminCommand({getParameter: 1, internalPipelineLengthLimit: 1}));
+assert.eq(getParamRes["internalPipelineLengthLimit"],
+          testDB.adminCommand("buildInfo").debug ? 200 : 1000);
+
 // Verify that the default values are set as expected when the server starts up.
 assertDefaultParameterValues();
 
@@ -192,8 +199,8 @@ assertSetParameterSucceeds("internalQuerySlotBasedExecutionMaxStaticIndexScanInt
 assertSetParameterFails("internalQuerySlotBasedExecutionMaxStaticIndexScanIntervals", 0);
 assertSetParameterFails("internalQuerySlotBasedExecutionMaxStaticIndexScanIntervals", -1);
 
-assertSetParameterSucceeds("internalQueryEnableSlotBasedExecutionEngine", true);
-assertSetParameterSucceeds("internalQueryEnableSlotBasedExecutionEngine", false);
+assertSetParameterSucceeds("internalQueryForceClassicEngine", true);
+assertSetParameterSucceeds("internalQueryForceClassicEngine", false);
 
 MongoRunner.stopMongod(conn);
 })();
