@@ -40,7 +40,10 @@ class MockOCSPServer {
      * @param {number} next_update_secs
      * @param {object} responder_certificate_set
      */
-    constructor(fault_type, next_update_secs, responder_certificate_set = OCSP_DELEGATE_RESPONDER) {
+    constructor(fault_type,
+                next_update_secs,
+                responder_certificate_set = OCSP_DELEGATE_RESPONDER,
+                response_delay_secs = 0) {
         this.python = "python3";
         this.fault_type = fault_type;
 
@@ -57,6 +60,7 @@ class MockOCSPServer {
         // responder in the certificates.
         this.port = 8100;
         this.next_update_secs = next_update_secs;
+        this.response_delay_secs = response_delay_secs;
     }
 
     start() {
@@ -65,10 +69,11 @@ class MockOCSPServer {
             this.python,
             "-u",
             OCSP_PROGRAM,
-            "-p=" + this.port,
+            "--port=" + this.port,
             "--ca_file=" + this.ca_file,
             "--ocsp_responder_cert=" + this.ocsp_cert_file,
-            "--ocsp_responder_key=" + this.ocsp_cert_key
+            "--ocsp_responder_key=" + this.ocsp_cert_key,
+            "--verbose",
         ];
 
         if (this.fault_type) {
@@ -79,6 +84,10 @@ class MockOCSPServer {
             args.push("--next_update_seconds=" + this.next_update_secs);
         }
 
+        if (this.response_delay_secs) {
+            args.push("--response_delay_seconds=" + this.response_delay_secs);
+        }
+
         clearRawMongoProgramOutput();
 
         this.pid = _startMongoProgram({args: args});
@@ -105,6 +114,23 @@ class MockOCSPServer {
      * Stop the web server
      */
     stop() {
-        stopMongoProgramByPid(this.pid);
+        if (!this.pid) {
+            print("Not stopping Mock OCSP Server, it was never started");
+            return;
+        }
+
+        print("Stopping Mock OCSP Server");
+
+        if (_isWindows()) {
+            // we use taskkill because we need to kill children
+            waitProgram(_startMongoProgram("taskkill", "/F", "/T", "/PID", this.pid));
+            // waitProgram to ignore error code
+            waitProgram(this.pid);
+        } else {
+            const kSIGINT = 2;
+            stopMongoProgramByPid(this.pid, kSIGINT);
+        }
+
+        print("Mock OCSP Server stop complete");
     }
 }
\ No newline at end of file
