@@ -34,29 +34,22 @@ const allCommands = {
     _configsvrBalancerStop: {skip: isPrimaryOnly},
     _configsvrCleanupReshardCollection: {skip: isPrimaryOnly},
     _configsvrClearJumboFlag: {skip: isPrimaryOnly},
-    _configsvrCommitChunkMerge: {skip: isPrimaryOnly},
     _configsvrCommitChunksMerge: {skip: isPrimaryOnly},
     _configsvrCommitChunkMigration: {skip: isPrimaryOnly},
     _configsvrCommitChunkSplit: {skip: isPrimaryOnly},
-    _configsvrCommitMovePrimary: {skip: isPrimaryOnly},
     _configsvrCommitReshardCollection: {skip: isPrimaryOnly},
+    _configsvrConfigureAutoSplit: {skip: isPrimaryOnly},
     _configsvrCreateDatabase: {skip: isPrimaryOnly},
-    _configsvrDropCollection: {skip: isPrimaryOnly},
-    _configsvrDropDatabase: {skip: isPrimaryOnly},
-    _configsvrEnableSharding: {skip: isPrimaryOnly},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isPrimaryOnly},
     _configsvrMoveChunk: {skip: isPrimaryOnly},
-    _configsvrMovePrimary: {skip: isPrimaryOnly},
     _configsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
     _configsvrRemoveChunks: {skip: isPrimaryOnly},
     _configsvrRemoveShard: {skip: isPrimaryOnly},
     _configsvrRemoveShardFromZone: {skip: isPrimaryOnly},
     _configsvrRemoveTags: {skip: isPrimaryOnly},
-    _configsvrRepairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
-    _configsvrRenameCollectionMetadata: {skip: isPrimaryOnly},
+    _configsvrRenameCollectionMetadata: {skip: isAnInternalCommand},
     _configsvrReshardCollection: {skip: isPrimaryOnly},
     _configsvrSetAllowMigrations: {skip: isPrimaryOnly},
-    _configsvrShardCollection: {skip: isPrimaryOnly},
     _configsvrUpdateZoneKeyRange: {skip: isPrimaryOnly},
     _flushDatabaseCacheUpdates: {skip: isPrimaryOnly},
     _flushDatabaseCacheUpdatesWithWriteConcern: {skip: isPrimaryOnly},
@@ -90,11 +83,9 @@ const allCommands = {
     _shardsvrRenameCollectionParticipantUnblock: {skip: isAnInternalCommand},
     _shardsvrDropDatabase: {skip: isPrimaryOnly},
     _shardsvrDropDatabaseParticipant: {skip: isPrimaryOnly},
-    _shardsvrShardCollection: {skip: isPrimaryOnly},
     _shardsvrReshardCollection: {skip: isPrimaryOnly},
     _shardsvrReshardingOperationTime: {skip: isPrimaryOnly},
     _shardsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
-    _shardsvrSetAllowMigrations: {skip: isPrimaryOnly},
     _transferMods: {skip: isPrimaryOnly},
     _vectorClockPersist: {skip: isPrimaryOnly},
     abortReshardCollection: {skip: isPrimaryOnly},
@@ -107,7 +98,7 @@ const allCommands = {
     appendOplogNote: {skip: isPrimaryOnly},
     applyOps: {skip: isPrimaryOnly},
     authenticate: {skip: isNotAUserDataRead},
-    autoSplitVector: {skip: isNotAUserDataRead},
+    autoSplitVector: {skip: isPrimaryOnly},
     availableQueryOptions: {skip: isNotAUserDataRead},
     buildInfo: {skip: isNotAUserDataRead},
     captrunc: {skip: isPrimaryOnly},
@@ -126,6 +117,7 @@ const allCommands = {
     commitTransaction: {skip: isPrimaryOnly},
     compact: {skip: isNotAUserDataRead},
     configureFailPoint: {skip: isNotAUserDataRead},
+    configureCollectionAutoSplitter: {skip: isPrimaryOnly},
     connPoolStats: {skip: isNotAUserDataRead},
     connPoolSync: {skip: isNotAUserDataRead},
     connectionStatus: {skip: isNotAUserDataRead},
@@ -166,6 +158,9 @@ const allCommands = {
     donorForgetMigration: {skip: isPrimaryOnly},
     donorStartMigration: {skip: isPrimaryOnly},
     donorWaitForMigrationToCommit: {skip: isPrimaryOnly},
+    donorAbortSplit: {skip: isPrimaryOnly},
+    donorForgetSplit: {skip: isPrimaryOnly},
+    donorStartSplit: {skip: isPrimaryOnly},
     driverOIDTest: {skip: isNotAUserDataRead},
     drop: {skip: isPrimaryOnly},
     dropAllRolesFromDatabase: {skip: isPrimaryOnly},
@@ -279,7 +274,7 @@ const allCommands = {
     refreshSessions: {skip: isNotAUserDataRead},
     reIndex: {skip: isNotAUserDataRead},
     renameCollection: {skip: isPrimaryOnly},
-    repairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
+    repairDatabase: {skip: isNotAUserDataRead},
     replSetAbortPrimaryCatchUp: {skip: isNotAUserDataRead},
     replSetFreeze: {skip: isNotAUserDataRead},
     replSetGetConfig: {skip: isNotAUserDataRead},
