@@ -15,19 +15,14 @@ var getLastOpTime;
 var getFirstOplogEntry;
 var setLogVerbosity;
 var stopReplicationAndEnforceNewPrimaryToCatchUp;
-var setFailPoint;
-var clearFailPoint;
 var isConfigCommitted;
 var assertSameConfigContent;
 var getConfigWithNewlyAdded;
 var isMemberNewlyAdded;
-var replConfigHasNewlyAddedMembers;
 var waitForNewlyAddedRemovalForNodeToBeCommitted;
 var assertVoteCount;
 var disconnectSecondaries;
 var reconnectSecondaries;
-var selectDelayFieldName;
-var isDefaultReadConcernLocalFlagEnabled;
 
 (function() {
 "use strict";
@@ -701,6 +696,11 @@ stopReplicationAndEnforceNewPrimaryToCatchUp = function(rst, node) {
     const oldSecondaries = rst.getSecondaries();
     const oldPrimary = rst.getPrimary();
 
+    // In the case that the old primary has just stepped up and is running internal writes from
+    // PrimaryOnlyService, wait for those to be replicated. This is because there could be a race
+    // between stopping one of the secondaries, while the other secondary is still able to replicate
+    // the internal writes from PrimaryOnlyService before stopping replication.
+    rst.awaitReplication();
     stopServerReplication(oldSecondaries);
     for (let i = 0; i < 3; i++) {
         assert.commandWorked(oldPrimary.getDB("test").foo.insert({x: i}));
@@ -725,26 +725,6 @@ stopReplicationAndEnforceNewPrimaryToCatchUp = function(rst, node) {
     };
 };
 
-/**
- * Sets the specified failpoint to 'alwaysOn' on the node and returns the number of
- * times the fail point has been entered so far.
- */
-setFailPoint = function(node, failpoint, data = {}) {
-    jsTestLog("Setting fail point " + failpoint);
-    let configureFailPointRes =
-        node.adminCommand({configureFailPoint: failpoint, mode: "alwaysOn", data: data});
-    assert.commandWorked(configureFailPointRes);
-    return configureFailPointRes.count;
-};
-
-/**
- * Sets the specified failpoint to 'off' on the node.
- */
-clearFailPoint = function(node, failpoint) {
-    jsTestLog("Clearing fail point " + failpoint);
-    assert.commandWorked(node.adminCommand({configureFailPoint: failpoint, mode: "off"}));
-};
-
 /**
  * Returns the replSetGetConfig field 'commitmentStatus', which is true or false.
  */
@@ -815,11 +795,6 @@ isMemberNewlyAdded = function(node, memberIndex) {
     return hasNewlyAdded(memberIndex);
 };
 
-// Returns true if at least one member in the repl set config contains "newlyAdded" field.
-replConfigHasNewlyAddedMembers = function(conn) {
-    return isMemberNewlyAdded(conn);
-};
-
 waitForNewlyAddedRemovalForNodeToBeCommitted = function(node, memberIndex, force = false) {
     jsTestLog("Waiting for member " + memberIndex + " to no longer be 'newlyAdded'");
     assert.soonNoExcept(function() {
@@ -861,33 +836,4 @@ reconnectSecondaries = function(rst) {
         }
     }
 };
-
-/**
- * If featureFlagUseSecondaryDelaySecs is enabled, we must use the 'secondaryDelaySecs' field name
- * in our config. Otherwise, we use 'slaveDelay'. This helper implements the logic to determine
- * which field to use.
- */
-selectDelayFieldName = function(rst) {
-    const useSecondaryDelaySecs =
-        rst.nodes[0]
-            .adminCommand({getParameter: 1, featureFlagUseSecondaryDelaySecs: 1})
-            .featureFlagUseSecondaryDelaySecs.value;
-    return useSecondaryDelaySecs ? "secondaryDelaySecs" : "slaveDelay";
-};
-
-/**
- * Returns whether featureFlagDefaultReadConcernLocal is enabled. Returns false if the node is
- * running an older version with no knowledge of the flag.
- */
-isDefaultReadConcernLocalFlagEnabled = function(conn) {
-    let res = conn.adminCommand({getParameter: 1, featureFlagDefaultReadConcernLocal: 1});
-    if (!res.ok) {
-        // Running with old version which doesn't have the flag.
-        if (res.errmsg == "no option found to get")
-            return false;
-        assert(false);
-    }
-
-    return res.featureFlagDefaultReadConcernLocal.value;
-};
 }());
