@@ -7,8 +7,15 @@
  * migration state to "committed" and "aborted" to be majority committed but it cannot do that on
  * ephemeralForTest.
  *
- * @tags: [requires_fcv_47, requires_majority_read_concern, incompatible_with_eft,
- * incompatible_with_windows_tls, incompatible_with_macos, requires_persistence]
+ * @tags: [
+ *   incompatible_with_eft,
+ *   incompatible_with_macos,
+ *   incompatible_with_windows_tls,
+ *   requires_majority_read_concern,
+ *   requires_persistence,
+ *   # TODO SERVER-59090: Remove this tag.
+ *   backport_required_multiversion,
+ * ]
  */
 (function() {
 'use strict';
@@ -27,10 +34,7 @@ const tenantMigrationTest = new TenantMigrationTest({
             {'failpoint.tenantMigrationDonorAllowsNonTimestampedReads': tojson({mode: 'alwaysOn'})}
     }
 });
-if (!tenantMigrationTest.isFeatureFlagEnabled()) {
-    jsTestLog("Skipping test because the tenant migrations feature flag is disabled");
-    return;
-}
+
 const donorRst = tenantMigrationTest.getDonorRst();
 
 const primary = donorRst.getPrimary();
@@ -483,22 +487,18 @@ const testCases = {
     _configsvrBalancerStatus: {skip: isNotRunOnUserDatabase},
     _configsvrBalancerStop: {skip: isNotRunOnUserDatabase},
     _configsvrClearJumboFlag: {skip: isNotRunOnUserDatabase},
-    _configsvrCommitChunkMerge: {skip: isNotRunOnUserDatabase},
     _configsvrCommitChunksMerge: {skip: isNotRunOnUserDatabase},
     _configsvrCommitChunkMigration: {skip: isNotRunOnUserDatabase},
     _configsvrCommitChunkSplit: {skip: isNotRunOnUserDatabase},
-    _configsvrCommitMovePrimary: {skip: isNotRunOnUserDatabase},
+    _configsvrCommitMovePrimary:
+        {skip: isNotRunOnUserDatabase},  // Can be removed once 6.0 is last LTS
     _configsvrCreateDatabase: {skip: isNotRunOnUserDatabase},
-    _configsvrDropCollection: {skip: isNotRunOnUserDatabase},
-    _configsvrDropDatabase: {skip: isNotRunOnUserDatabase},
-    _configsvrEnableSharding: {skip: isNotRunOnUserDatabase},
     _configsvrEnsureChunkVersionIsGreaterThan: {skip: isNotRunOnUserDatabase},
-    _configsvrMoveChunk: {skip: isNotRunOnUserDatabase},
+    _configsvrMoveChunk: {skip: isNotRunOnUserDatabase},  // Can be removed once 6.0 is last LTS
     _configsvrMovePrimary: {skip: isNotRunOnUserDatabase},
     _configsvrRefineCollectionShardKey: {skip: isNotRunOnUserDatabase},
     _configsvrRemoveShard: {skip: isNotRunOnUserDatabase},
     _configsvrRemoveShardFromZone: {skip: isNotRunOnUserDatabase},
-    _configsvrShardCollection: {skip: isNotRunOnUserDatabase},
     _configsvrUpdateZoneKeyRange: {skip: isNotRunOnUserDatabase},
     _flushDatabaseCacheUpdates: {skip: isNotRunOnUserDatabase},
     _flushDatabaseCacheUpdatesWithWriteConcern: {skip: isNotRunOnUserDatabase},
@@ -520,8 +520,8 @@ const testCases = {
     _shardsvrCreateCollection: {skip: isOnlySupportedOnShardedCluster},
     _shardsvrCreateCollectionParticipant: {skip: isOnlySupportedOnShardedCluster},
     _shardsvrMovePrimary: {skip: isNotRunOnUserDatabase},
-    _shardsvrSetAllowMigrations: {skip: isOnlySupportedOnShardedCluster},
-    _shardsvrShardCollection: {skip: isNotRunOnUserDatabase},
+    _shardsvrShardCollection:
+        {skip: isNotRunOnUserDatabase},  // TODO SERVER-58843: Remove once 6.0 becomes last LTS
     _shardsvrRenameCollection: {skip: isOnlySupportedOnShardedCluster},
     _transferMods: {skip: isNotRunOnUserDatabase},
     abortTransaction: {
@@ -921,7 +921,6 @@ const testCases = {
     saslStart: {skip: isAuthCommand},
     sbe: {skip: isNotRunOnUserDatabase},
     serverStatus: {skip: isNotRunOnUserDatabase},
-    setAllowMigrations: {skip: isNotRunOnUserDatabase},
     setCommittedSnapshot: {skip: isNotRunOnUserDatabase},
     setDefaultRWConcern: {skip: isNotRunOnUserDatabase},
     setFeatureCompatibilityVersion: {skip: isNotRunOnUserDatabase},
