@@ -67,20 +67,6 @@ assert.commandFailed(mongos.adminCommand({shardCollection: 'foo', key: {_id: 1}}
 
 assert.commandFailed(mongos.adminCommand({shardCollection: kDbName + '.foo', key: "aaa"}));
 
-const testDB = st.rs0.getPrimary().getDB(kDbName);
-const fcvDoc = testDB.adminCommand({getParameter: 1, featureCompatibilityVersion: 1});
-if (MongoRunner.compareBinVersions(fcvDoc.featureCompatibilityVersion.version, '5.0') >= 0) {
-    jsTestLog('Verify namespace length limit.');
-
-    const longEnoughNs = kDbName + '.' +
-        'x'.repeat(235 - kDbName.length - 1);
-    assert.commandWorked(mongos.adminCommand({shardCollection: longEnoughNs, key: {_id: 1}}));
-
-    const tooLongNs = longEnoughNs + 'x';
-    assert.commandFailedWithCode(mongos.adminCommand({shardCollection: tooLongNs, key: {_id: 1}}),
-                                 ErrorCodes.InvalidNamespace);
-}
-
 jsTestLog('shardCollection may only be run against admin database.');
 assert.commandFailed(
     mongos.getDB('test').runCommand({shardCollection: kDbName + '.foo', key: {_id: 1}}));
@@ -105,12 +91,6 @@ jsTestLog('Shard key can contain dotted path to embedded element.');
 assert.commandWorked(
     mongos.adminCommand({shardCollection: kDbName + '.shard_key_dotted_path', key: {'_id.a': 1}}));
 
-jsTestLog('Command should still verify index even if implicitlyCreateIndex is false.');
-assert.commandFailedWithCode(
-    mongos.adminCommand(
-        {shardCollection: kDbName + '.foo', key: {x: 1}, implicitlyCreateIndex: false}),
-    6373200);
-
 //
 // Test shardCollection's idempotency
 //
@@ -133,31 +113,6 @@ assert.commandFailed(
 
 assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
 
-jsTestLog('Allow non-unique index if enforceUniquenessCheck is false');
-assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
-assert.commandWorked(mongos.getDB(kDbName).foo.createIndex({x: 1}));
-assert.commandWorked(mongos.adminCommand(
-    {shardCollection: kDbName + '.foo', key: {x: 1}, unique: true, enforceUniquenessCheck: false}));
-let collDoc = mongos.getDB('config').collections.findOne({_id: `${kDbName}.foo`});
-assert(collDoc);
-assert(collDoc.unique);
-assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
-
-jsTestLog('mongosync unique key pattern use case');
-assert.commandWorked(mongos.adminCommand({enableSharding: kDbName}));
-assert.commandWorked(mongos.getDB(kDbName).foo.createIndex({x: 1}));
-assert.commandWorked(mongos.adminCommand({
-    shardCollection: kDbName + '.foo',
-    key: {x: 1},
-    unique: true,
-    implicitlyCreateIndex: false,
-    enforceUniquenessCheck: false
-}));
-collDoc = mongos.getDB('config').collections.findOne({_id: `${kDbName}.foo`});
-assert(collDoc);
-assert(collDoc.unique);
-assert.commandWorked(mongos.getDB(kDbName).dropDatabase());
-
 jsTestLog('Shard empty collections no index required.');
 testAndClenaupWithKeyNoIndexOK({_id: 1});
 testAndClenaupWithKeyNoIndexOK({_id: 'hashed'});
