@@ -1,7 +1,10 @@
-/*
+/**
  * Tests that changing the shard key value of a document using update and findAndModify works
  * correctly when the new shard key value belongs to the same shard.
- * @tags: [requires_find_command, uses_transactions, uses_multi_shard_transaction]
+ * @tags: [
+ *   uses_multi_shard_transaction,
+ *   uses_transactions,
+ * ]
  */
 
 (function() {
@@ -10,12 +13,7 @@
 load("jstests/sharding/libs/sharded_transactions_helpers.js");
 load("jstests/sharding/libs/update_shard_key_helpers.js");
 
-const st = new ShardingTest({
-    mongos: 1,
-    shards: {rs0: {nodes: 3}, rs1: {nodes: 3}},
-    shardOptions:
-        {setParameter: {maxTransactionLockRequestTimeoutMillis: ReplSetTest.kDefaultTimeoutMS}}
-});
+const st = new ShardingTest({mongos: 1, shards: {rs0: {nodes: 3}, rs1: {nodes: 3}}});
 const kDbName = 'db';
 const ns = kDbName + '.foo';
 const mongos = st.s0;
