@@ -2,14 +2,9 @@
  * Test simple updates issued through mongos. Updates have different constraints through mongos,
  * since shard key is immutable.
  *
- * Updating a shard key in a single statement may use a multi shard transaction. Uses the
- * 'requires_find_command' tag to skip this test in sharding_op_query suite. Otherwise,
- * sessionDb.coll.find() will throw "Cannot run a legacy query on a session".
- *
  * @tags: [
- *  requires_find_command,
- *  uses_multi_shard_transaction,
- *  uses_transactions,
+ *   uses_multi_shard_transaction,
+ *   uses_transactions,
  * ]
  */
 (function() {
@@ -45,14 +40,6 @@ for (let i = 0; i < 2; i++) {
     assert.commandWorked(coll.update({_id: 2, key: 2}, {key: 2, foo: 'bar'}, {upsert: true}));
     assert.commandWorked(coll.update({_id: 3, key: 3}, {$set: {foo: 'bar'}}, {upsert: true}));
 
-    // Mixing operator & non-operator fields in updates is not allowed.
-    assert.commandFailedWithCode(
-        coll.update({_id: 4, key: 4}, {key: 4, $baz: {foo: 'bar'}}, {upsert: true}),
-        ErrorCodes.UnsupportedFormat);
-    assert.commandFailedWithCode(
-        coll.update({_id: 5, key: 5}, {$baz: {foo: 'bar'}, key: 5}, {upsert: true}),
-        ErrorCodes.UnsupportedFormat);
-
     assert.eq(coll.count(), 3, "count A");
     assert.eq(coll.findOne({_id: 3}).key, 3, "findOne 3 key A");
     assert.eq(coll.findOne({_id: 3}).foo, 'bar', "findOne 3 foo A");
