@@ -56,24 +56,6 @@ COMMAND_TYPES = (
     SCons.Action.CommandGeneratorAction,
 )
 
-ninja_compdb_adjust = """\
-import json
-import sys
-
-compdb = {}
-with open(sys.argv[1]) as f:
-    compdb = json.load(f)
-
-for command in compdb:
-    if command['output'].endswith('.compdb'):
-        command['output'] = command['output'][:-(len('.compdb'))]
-    else:
-        print(f"compdb entry does not contain '.compdb': {command['output']}")
-
-with open(sys.argv[1], 'w') as f:
-    json.dump(compdb, f, indent=2)
-"""
-
 
 def _install_action_function(_env, node):
     """Install files using the install or copy commands"""
@@ -82,7 +64,6 @@ def _install_action_function(_env, node):
         "rule": "INSTALL",
         "inputs": [get_path(src_file(s)) for s in node.sources],
         "implicit": get_dependencies(node),
-        "variables": {"precious": node.precious},
     }
 
 
@@ -97,11 +78,9 @@ def _mkdir_action_function(env, node):
         # to an invalid ninja file.
         "variables": {
             # On Windows mkdir "-p" is always on
-            "cmd":
-                "mkdir {args}".format(
-                    args=' '.join(get_outputs(node)) + " & exit /b 0"
-                    if env["PLATFORM"] == "win32" else "-p " + ' '.join(get_outputs(node)), ),
-            "variables": {"precious": node.precious},
+            "cmd": "{mkdir} $out".format(
+                mkdir="mkdir" if env["PLATFORM"] == "win32" else "mkdir -p",
+            ),
         },
     }
 
@@ -121,7 +100,6 @@ def _lib_symlink_action_function(_env, node):
         "inputs": inputs,
         "rule": "SYMLINK",
         "implicit": get_dependencies(node),
-        "variables": {"precious": node.precious},
     }
 
 
@@ -326,11 +304,6 @@ class SConsToNinjaTranslator:
             if callable(node_callback):
                 node_callback(env, node, build)
 
-        if build is not None and node.precious:
-            if not build.get('variables'):
-                build['variables'] = {}
-            build['variables']['precious'] = node.precious
-
         return build
 
     def handle_func_action(self, node, action):
@@ -435,6 +408,14 @@ class SConsToNinjaTranslator:
                 "implicit": dependencies,
             }
 
+        elif results[0]["rule"] == "INSTALL":
+            return {
+                "outputs": all_outputs,
+                "rule": "INSTALL",
+                "inputs": [get_path(src_file(s)) for s in node.sources],
+                "implicit": dependencies,
+            }
+
         raise Exception("Unhandled list action with rule: " + results[0]["rule"])
 
 
@@ -464,21 +445,13 @@ class NinjaState:
         scons_escape = env.get("ESCAPE", lambda x: x)
 
         self.variables = {
-            # The /b option here will make sure that windows updates the mtime
-            # when copying the file. This allows to not need to use restat for windows
-            # copy commands.
-            "COPY":
-                "cmd.exe /c 1>NUL copy /b" if sys.platform == "win32" else "cp",
-            "NOOP":
-                "cmd.exe /c 1>NUL echo 0" if sys.platform == "win32" else "echo 0 >/dev/null",
-            "SCONS_INVOCATION":
-                "{} {} __NINJA_NO=1 $out".format(
-                    sys.executable,
-                    " ".join([
-                        ninja_syntax.escape(scons_escape(arg)) for arg in sys.argv
-                        if arg not in COMMAND_LINE_TARGETS
-                    ]),
+            "COPY": "cmd.exe /c 1>NUL copy" if sys.platform == "win32" else "cp",
+            "SCONS_INVOCATION": "{} {} __NINJA_NO=1 $out".format(
+                sys.executable,
+                " ".join(
+                    [ninja_syntax.escape(scons_escape(arg)) for arg in sys.argv if arg not in COMMAND_LINE_TARGETS]
                 ),
+            ),
             "SCONS_INVOCATION_W_TARGETS": "{} {}".format(
                 sys.executable, " ".join([ninja_syntax.escape(scons_escape(arg)) for arg in sys.argv])
             ),
@@ -512,18 +485,6 @@ class NinjaState:
                 "rspfile": "$out.rsp",
                 "rspfile_content": "$rspc",
             },
-            "COMPDB_CC": {
-                "command": "$CC @$out.rsp",
-                "description": "Compiling $out",
-                "rspfile": "$out.rsp",
-                "rspfile_content": "$rspc",
-            },
-            "COMPDB_CXX": {
-                "command": "$CXX @$out.rsp",
-                "description": "Compiling $out",
-                "rspfile": "$out.rsp",
-                "rspfile_content": "$rspc",
-            },
             "LINK": {
                 "command": "$env$LINK @$out.rsp",
                 "description": "Linking $out",
@@ -531,9 +492,17 @@ class NinjaState:
                 "rspfile_content": "$rspc",
                 "pool": "local_pool",
             },
+            # Ninja does not automatically delete the archive before
+            # invoking ar. The ar utility will append to an existing archive, which
+            # can cause duplicate symbols if the symbols moved between object files.
+            # Native SCons will perform this operation so we need to force ninja
+            # to do the same. See related for more info:
+            # https://jira.mongodb.org/browse/SERVER-49457
             "AR": {
-                "command": "$env$AR @$out.rsp",
-                "description": "Archived $out",
+                "command": "{}$env$AR @$out.rsp".format(
+                    '' if sys.platform == "win32" else "rm -f $out && "
+                ),
+                "description": "Archiving $out",
                 "rspfile": "$out.rsp",
                 "rspfile_content": "$rspc",
                 "pool": "local_pool",
@@ -550,6 +519,15 @@ class NinjaState:
                 "command": "$COPY $in $out",
                 "description": "Install $out",
                 "pool": "install_pool",
+                # On Windows cmd.exe /c copy does not always correctly
+                # update the timestamp on the output file. This leads
+                # to a stuck constant timestamp in the Ninja database
+                # and needless rebuilds.
+                #
+                # Adding restat here ensures that Ninja always checks
+                # the copy updated the timestamp and that Ninja has
+                # the correct information.
+                "restat": 1,
             },
             "TEMPLATE": {
                 "command": "$SCONS_INVOCATION $out",
@@ -662,9 +640,6 @@ class NinjaState:
 
         ninja.comment("Generated by scons. DO NOT EDIT.")
 
-        # This version is needed because it is easy to get from pip and it support compile_commands.json
-        ninja.variable("ninja_required_version", "1.10")
-
         ninja.variable("builddir", get_path(self.env['NINJA_BUILDDIR']))
 
         for pool_name, size in self.pools.items():
@@ -673,127 +648,34 @@ class NinjaState:
         for var, val in self.variables.items():
             ninja.variable(var, val)
 
-        # This is the command that is used to clean a target before building it,
-        # excluding precious targets.
-        if sys.platform == "win32":
-            rm_cmd = f'cmd.exe /c del /q $rm_outs >nul 2>&1 &'
-        else:
-            rm_cmd = 'rm -f $rm_outs;'
-
-        precious_rule_suffix = "_PRECIOUS"
-
-        # Make two sets of rules to honor scons Precious setting. The build nodes themselves
-        # will then reselect their rule according to the precious being set for that node.
-        precious_rules = {}
         for rule, kwargs in self.rules.items():
             if self.env.get('NINJA_MAX_JOBS') is not None and 'pool' not in kwargs:
                 kwargs['pool'] = 'local_pool'
-            # Do not worry about precious for commands that don't have targets (phony)
-            # or that will callback to scons (which maintains its own precious).
-            if rule not in ['phony', 'TEMPLATE', 'REGENERATE', 'COMPDB_CC', 'COMPDB_CXX']:
-                precious_rule = rule + precious_rule_suffix
-                precious_rules[precious_rule] = kwargs.copy()
-                ninja.rule(precious_rule, **precious_rules[precious_rule])
+            ninja.rule(rule, **kwargs)
 
-                kwargs['command'] = f"{rm_cmd} " + kwargs['command']
-                ninja.rule(rule, **kwargs)
-            else:
+        generated_source_files = sorted({
+            output
+            # First find builds which have header files in their outputs.
+            for build in self.builds.values()
+            if self.has_generated_sources(build["outputs"])
+            for output in build["outputs"]
+            # Collect only the header files from the builds with them
+            # in their output. We do this because is_generated_source
+            # returns True if it finds a header in any of the outputs,
+            # here we need to filter so we only have the headers and
+            # not the other outputs.
+            if self.is_generated_source(output)
+        })
 
-                ninja.rule(rule, **kwargs)
-        self.rules.update(precious_rules)
-
-        # If the user supplied an alias to determine generated sources, use that, otherwise
-        # determine what the generated sources are dynamically.
-        generated_sources_alias = self.env.get('NINJA_GENERATED_SOURCE_ALIAS_NAME')
-        generated_sources_build = None
-
-        if generated_sources_alias:
-            generated_sources_build = self.builds.get(generated_sources_alias)
-            if generated_sources_build is None or generated_sources_build["rule"] != 'phony':
-                raise Exception(
-                    "ERROR: 'NINJA_GENERATED_SOURCE_ALIAS_NAME' set, but no matching Alias object found."
-                )
-
-        if generated_sources_alias and generated_sources_build:
-            generated_source_files = sorted(
-                [] if not generated_sources_build else generated_sources_build['implicit']
+        if generated_source_files:
+            ninja.build(
+                outputs="_generated_sources",
+                rule="phony",
+                implicit=generated_source_files
             )
-            def check_generated_source_deps(build):
-                return (
-                    build != generated_sources_build
-                    and set(build["outputs"]).isdisjoint(generated_source_files)
-                )
-        else:
-            generated_sources_build = None
-            generated_source_files = sorted({
-                output
-                # First find builds which have header files in their outputs.
-                for build in self.builds.values()
-                if self.has_generated_sources(build["outputs"])
-                for output in build["outputs"]
-                # Collect only the header files from the builds with them
-                # in their output. We do this because is_generated_source
-                # returns True if it finds a header in any of the outputs,
-                # here we need to filter so we only have the headers and
-                # not the other outputs.
-                if self.is_generated_source(output)
-            })
-
-            if generated_source_files:
-                generated_sources_alias = "_ninja_generated_sources"
-                ninja.build(
-                    outputs=generated_sources_alias,
-                    rule="phony",
-                    implicit=generated_source_files
-                )
-                def check_generated_source_deps(build):
-                    return (
-                        not build["rule"] == "INSTALL"
-                        and set(build["outputs"]).isdisjoint(generated_source_files)
-                        and set(build.get("implicit", [])).isdisjoint(generated_source_files)
-                    )
 
         template_builders = []
 
-        # If we ever change the name/s of the rules that include
-        # compile commands (i.e. something like CC) we will need to
-        # update this build to reflect that complete list.
-        compile_commands = "compile_commands.json"
-        compdb_expand = '-x ' if self.env.get('NINJA_COMPDB_EXPAND') else ''
-        adjust_script_out = os.path.join(
-            get_path(self.env['NINJA_BUILDDIR']), 'ninja_compdb_adjust.py')
-        os.makedirs(os.path.dirname(adjust_script_out), exist_ok=True)
-        with open(adjust_script_out, 'w') as f:
-            f.write(ninja_compdb_adjust)
-        self.builds[compile_commands] = {
-            'rule': "CMD",
-            'outputs': [compile_commands],
-            'pool': "console",
-            'implicit': [ninja_file],
-            'variables': {
-                "cmd":
-                    f"ninja -f {ninja_file} -t compdb {compdb_expand}COMPDB_CC COMPDB_CXX > {compile_commands};"
-                    + f"{sys.executable} {adjust_script_out} {compile_commands}"
-            },
-        }
-        self.builds["compiledb"] = {
-            'rule': "phony",
-            "outputs": ["compiledb"],
-            'implicit': [compile_commands],
-        }
-
-        # Now for all build nodes, we want to select the precious rule or not.
-        # If it's not precious, we need to save all the outputs into a variable
-        # on that node. Later we will be removing outputs and switching them to
-        # phonies so that we can generate response and depfiles correctly.
-        for build, kwargs in self.builds.items():
-            if kwargs.get('variables') and kwargs['variables'].get('precious'):
-                kwargs['rule'] = kwargs['rule'] + precious_rule_suffix
-            elif kwargs['rule'] not in ['phony', 'TEMPLATE', 'REGENERATE']:
-                if not kwargs.get('variables'):
-                    kwargs['variables'] = {}
-                kwargs['variables']['rm_outs'] = kwargs['outputs'].copy()
-
         for build in [self.builds[key] for key in sorted(self.builds.keys())]:
             if build["rule"] == "TEMPLATE":
                 template_builders.append(build)
@@ -809,7 +691,9 @@ class NinjaState:
             # cycle.
             if (
                 generated_source_files
-                and check_generated_source_deps(build)
+                and not build["rule"] == "INSTALL"
+                and set(build["outputs"]).isdisjoint(generated_source_files)
+                and set(build.get("implicit", [])).isdisjoint(generated_source_files)
             ):
 
                 # Make all non-generated source targets depend on
@@ -819,7 +703,7 @@ class NinjaState:
                 # sure that all of these sources are generated before
                 # other builds.
                 order_only = build.get("order_only", [])
-                order_only.append(generated_sources_alias)
+                order_only.append("_generated_sources")
                 build["order_only"] = order_only
             if "order_only" in build:
                 build["order_only"].sort()
@@ -884,21 +768,7 @@ class NinjaState:
 
             ninja.build(**build)
 
-        for build, kwargs in self.builds.items():
-            if kwargs['rule'] in [
-                    'CC', f'CC{precious_rule_suffix}', 'CXX', f'CXX{precious_rule_suffix}'
-            ]:
-                rule = kwargs['rule'].replace(
-                    precious_rule_suffix
-                ) if precious_rule_suffix in kwargs['rule'] else kwargs['rule']
-                rule = "COMPDB_" + rule
-                compdb_build = kwargs.copy()
-
-                compdb_build['rule'] = rule
-                compdb_build['outputs'] = [kwargs['outputs'] + ".compdb"]
-                ninja.build(**compdb_build)
-
-        template_builds = {'rule': "TEMPLATE"}
+        template_builds = dict()
         for template_builder in template_builders:
 
             # Special handling for outputs and implicit since we need to
@@ -915,6 +785,9 @@ class NinjaState:
                     new_val.append(cur_val)
                 template_builds[agg_key] = new_val
 
+            # Collect all other keys
+            template_builds.update(template_builder)
+
         if template_builds.get("outputs", []):
             ninja.build(**template_builds)
 
@@ -945,6 +818,25 @@ class NinjaState:
             implicit=[__file__],
         )
 
+        # If we ever change the name/s of the rules that include
+        # compile commands (i.e. something like CC) we will need to
+        # update this build to reflect that complete list.
+        ninja.build(
+            "compile_commands.json",
+            rule="CMD",
+            pool="console",
+            implicit=[ninja_file],
+            variables={
+                "cmd": "ninja -f {} -t compdb {}CC CXX > compile_commands.json".format(
+                    ninja_file, '-x ' if self.env.get('NINJA_COMPDB_EXPAND') else ''
+                )
+            },
+        )
+
+        ninja.build(
+            "compiledb", rule="phony", implicit=["compile_commands.json"],
+        )
+
         # Look in SCons's list of DEFAULT_TARGETS, find the ones that
         # we generated a ninja build rule for.
         scons_default_targets = [
@@ -1243,10 +1135,7 @@ def get_command(env, node, action):  # pylint: disable=too-many-branches
         # Possibly these could be ignore and the build would still work, however it may not always
         # rebuild correctly, so we hard stop, and force the user to fix the issue with the provided
         # ninja rule.
-        err_msg = f"Could not resolve path for '{provider_dep}' dependency on node '{node}', you may need to setup your shell environment for ninja builds."
-        if os.name == "nt":
-            err_msg += " On Windows, please ensure that you have run the necessary Visual Studio environment setup scripts (e.g. vcvarsall.bat ...,  or launching a Visual Studio Command Prompt) before invoking SCons."
-        raise Exception(err_msg)
+        raise Exception(f"Could not resolve path for {provider_dep} dependency on node '{node}'")
 
     ninja_build = {
         "order_only": get_order_only(node),
@@ -1315,9 +1204,7 @@ def register_custom_rule_mapping(env, pre_subst_string, rule):
     __NINJA_RULE_MAPPING[pre_subst_string] = rule
 
 
-def register_custom_rule(env, rule, command, description="", deps=None, pool=None,
-                         use_depfile=False, use_response_file=False, response_file_content="$rspc",
-                         restat=False):
+def register_custom_rule(env, rule, command, description="", deps=None, pool=None, use_depfile=False, use_response_file=False, response_file_content="$rspc"):
     """Allows specification of Ninja rules from inside SCons files."""
     rule_obj = {
         "command": command,
@@ -1335,15 +1222,8 @@ def register_custom_rule(env, rule, command, description="", deps=None, pool=Non
 
     if use_response_file:
         rule_obj["rspfile"] = "$out.rsp"
-        if rule_obj["rspfile"] not in command:
-            raise Exception(
-                f'Bad Ninja Custom Rule: response file requested, but {rule_obj["rspfile"]} not in in command: {command}'
-            )
         rule_obj["rspfile_content"] = response_file_content
 
-    if restat:
-        rule_obj["restat"] = 1
-
     env[NINJA_RULES][rule] = rule_obj
 
 
@@ -1520,6 +1400,7 @@ def generate(env):
     ninja_file_name = env.subst("${NINJA_PREFIX}.${NINJA_SUFFIX}")
     ninja_file = env.Ninja(target=ninja_file_name, source=[])
     env.AlwaysBuild(ninja_file)
+    env.Alias("$NINJA_ALIAS_NAME", ninja_file)
 
     # TODO: API for getting the SConscripts programmatically
     # exists upstream: https://github.com/SCons/scons/issues/3625
@@ -1545,15 +1426,6 @@ def generate(env):
     env.AddMethod(gen_get_response_file_command, "NinjaGenResponseFileProvider")
     env.AddMethod(set_build_node_callback, "NinjaSetBuildNodeCallback")
 
-    # Expose ninja node path converstion functions to make writing
-    # custom function action handlers easier.
-    env.AddMethod(lambda _env, node: get_outputs(node), "NinjaGetOutputs")
-    env.AddMethod(lambda _env, node, skip_unknown_types=False: get_inputs(node, skip_unknown_types),
-                  "NinjaGetInputs")
-    env.AddMethod(lambda _env, node, skip_sources=False: get_dependencies(node),
-                  "NinjaGetDependencies")
-    env.AddMethod(lambda _env, node: get_order_only(node), "NinjaGetOrderOnly")
-
     # Provides a way for users to handle custom FunctionActions they
     # want to translate to Ninja.
     env[NINJA_CUSTOM_HANDLERS] = {}
@@ -1648,12 +1520,6 @@ def generate(env):
     if not exists(env):
         return
 
-    # There is a target called generate-ninja which needs to be included
-    # with the --ninja flag in order to generate the ninja file. Because the --ninja
-    # flag is ONLY used with generate-ninja, we have combined the two by making the --ninja flag
-    # implicitly build the generate-ninja target.
-    SCons.Script.BUILD_TARGETS = SCons.Script.TargetList(env.Alias("$NINJA_ALIAS_NAME", ninja_file))
-
     # Set a known variable that other tools can query so they can
     # behave correctly during ninja generation.
     env["GENERATING_NINJA"] = True
