@@ -80,7 +80,12 @@ public:
     /**
      * Represents next steps of fetcher.
      */
-    enum class NextAction : int { kInvalid = 0, kNoAction = 1, kGetMore = 2 };
+    enum class NextAction : int {
+        kInvalid = 0,
+        kNoAction = 1,
+        kGetMore = 2,
+        kExitAndKeepCursorAlive = 3
+    };
 
     /**
      * Type of a fetcher callback function.
@@ -230,7 +235,7 @@ private:
      *
      * Note: Errors are ignored and no retry is done
      */
-    void _sendKillCursors(const CursorId id, const NamespaceString& nss);
+    void _sendKillCursors(CursorId id, const NamespaceString& nss);
 
     /**
      * Returns whether the fetcher is in shutdown.
