@@ -167,14 +167,20 @@ int32_t pingTimeMillis(const Node& node) {
 const Seconds kDefaultRefreshPeriod(30);
 }  // namespace
 
-ScanningReplicaSetMonitor::ScanningReplicaSetMonitor(const SetStatePtr& initialState)
-    : _state(initialState) {}
+// Defaults to random selection as required by the spec
+bool ScanningReplicaSetMonitor::useDeterministicHostSelection = false;
 
-ScanningReplicaSetMonitor::ScanningReplicaSetMonitor(const MongoURI& uri)
+ScanningReplicaSetMonitor::ScanningReplicaSetMonitor(const SetStatePtr& initialState,
+                                                     std::function<void()> cleanupCallback)
+    : ReplicaSetMonitor(cleanupCallback), _state(initialState) {}
+
+ScanningReplicaSetMonitor::ScanningReplicaSetMonitor(const MongoURI& uri,
+                                                     std::function<void()> cleanupCallback)
     : ScanningReplicaSetMonitor(
           std::make_shared<SetState>(uri,
                                      &ReplicaSetMonitorManager::get()->getNotifier(),
-                                     ReplicaSetMonitorManager::get()->getExecutor().get())) {}
+                                     ReplicaSetMonitorManager::get()->getExecutor().get()),
+          cleanupCallback) {}
 
 void ScanningReplicaSetMonitor::init() {
     if (areRefreshRetriesDisabledForTest()) {
@@ -198,6 +204,8 @@ void ScanningReplicaSetMonitor::drop() {
 }
 
 ScanningReplicaSetMonitor::~ScanningReplicaSetMonitor() {
+    // `drop` is idempotent and a duplicate call from ReplicaSetMonitorManager::removeMonitor() is
+    // safe.
     drop();
 }
 
@@ -205,7 +213,7 @@ template <typename Callback>
 auto ScanningReplicaSetMonitor::SetState::scheduleWorkAt(Date_t when, Callback&& cb) const {
     auto wrappedCallback = [cb = std::forward<Callback>(cb),
                             anchor = shared_from_this()](const CallbackArgs& cbArgs) mutable {
-        if (ErrorCodes::isCancelationError(cbArgs.status)) {
+        if (ErrorCodes::isCancellationError(cbArgs.status)) {
             // Do no more work if we're removed or canceled
             return;
         }
@@ -238,7 +246,7 @@ void ScanningReplicaSetMonitor::SetState::rescheduleRefresh(SchedulingStrategy s
         return;
     }
 
-    Milliseconds period = refreshPeriod;
+    Milliseconds period = _getRefreshPeriod();  // Supports fail injection.
     if (isExpedited) {
         period = std::min<Milliseconds>(period, kExpeditedRefreshPeriod);
     }
@@ -299,8 +307,11 @@ void ScanningReplicaSetMonitor::SetState::rescheduleRefresh(SchedulingStrategy s
 }
 
 SemiFuture<HostAndPort> ScanningReplicaSetMonitor::getHostOrRefresh(
-    const ReadPreferenceSetting& criteria, Milliseconds maxWait) {
-    return _getHostsOrRefresh(criteria, maxWait)
+    const ReadPreferenceSetting& criteria,
+    const std::vector<HostAndPort>& excludedHosts,
+    const CancellationToken&) {
+    return _getHostsOrRefresh(
+               criteria, ReplicaSetMonitorInterface::kDefaultFindHostTimeout, excludedHosts)
         .then([](const auto& hosts) {
             invariant(hosts.size());
             return hosts[0];
@@ -309,21 +320,27 @@ SemiFuture<HostAndPort> ScanningReplicaSetMonitor::getHostOrRefresh(
 }
 
 SemiFuture<std::vector<HostAndPort>> ScanningReplicaSetMonitor::getHostsOrRefresh(
-    const ReadPreferenceSetting& criteria, Milliseconds maxWait) {
-    return _getHostsOrRefresh(criteria, maxWait).semi();
+    const ReadPreferenceSetting& criteria,
+    const std::vector<HostAndPort>& excludedHosts,
+    const CancellationToken&) {
+    return _getHostsOrRefresh(
+               criteria, ReplicaSetMonitorInterface::kDefaultFindHostTimeout, excludedHosts)
+        .semi();
 }
 
 Future<std::vector<HostAndPort>> ScanningReplicaSetMonitor::_getHostsOrRefresh(
-    const ReadPreferenceSetting& criteria, Milliseconds maxWait) {
+    const ReadPreferenceSetting& criteria,
+    Milliseconds maxWait,
+    const std::vector<HostAndPort>& excludedHosts) {
 
     stdx::lock_guard<Latch> lk(_state->mutex);
     if (_state->isDropped) {
-        return Status(ErrorCodes::ReplicaSetMonitorRemoved,
+        return Status(ErrorCodes::ShutdownInProgress,
                       str::stream()
                           << "ScanningReplicaSetMonitor for set " << getName() << " is removed");
     }
 
-    auto out = _state->getMatchingHosts(criteria);
+    auto out = _state->getMatchingHosts(criteria, excludedHosts);
     if (!out.empty())
         return {std::move(out)};
 
@@ -335,7 +352,7 @@ Future<std::vector<HostAndPort>> ScanningReplicaSetMonitor::_getHostsOrRefresh(
     // dealing with maxWait.
     auto pf = makePromiseFuture<decltype(out)>();
     _state->waiters.emplace_back(
-        SetState::Waiter{_state->now() + maxWait, criteria, std::move(pf.promise)});
+        SetState::Waiter{_state->now() + maxWait, criteria, excludedHosts, std::move(pf.promise)});
 
     // This must go after we set up the wait state to correctly handle unittests using
     // MockReplicaSet.
@@ -347,8 +364,11 @@ Future<std::vector<HostAndPort>> ScanningReplicaSetMonitor::_getHostsOrRefresh(
 
     return std::move(pf.future);
 }
-HostAndPort ScanningReplicaSetMonitor::getMasterOrUassert() {
-    return getHostOrRefresh(kPrimaryOnlyReadPreference).get();
+
+HostAndPort ScanningReplicaSetMonitor::getPrimaryOrUassert() {
+    return ReplicaSetMonitorInterface::getHostOrRefresh(kPrimaryOnlyReadPreference,
+                                                        CancellationToken::uncancelable())
+        .get();
 }
 
 void ScanningReplicaSetMonitor::failedHost(const HostAndPort& host, const Status& status) {
@@ -475,14 +495,20 @@ bool ScanningReplicaSetMonitor::isKnownToHaveGoodPrimary() const {
     return false;
 }
 
-Seconds ScanningReplicaSetMonitor::getDefaultRefreshPeriod() {
-    Seconds r = kDefaultRefreshPeriod;
+Seconds ScanningReplicaSetMonitor::_getRefreshPeriod() {
+    boost::optional<Seconds> failInjected = _getFailInjectedRefreshPeriod();
+    return failInjected ? *failInjected : kDefaultRefreshPeriod;
+}
+
+boost::optional<Seconds> ScanningReplicaSetMonitor::_getFailInjectedRefreshPeriod() {
+    boost::optional<Seconds> result;
     static constexpr auto kPeriodField = "period"_sd;
     globalFailPointRegistry()
         .find("modifyReplicaSetMonitorDefaultRefreshPeriod")
-        ->executeIf([&r](const BSONObj& data) { r = Seconds{data.getIntField(kPeriodField)}; },
-                    [](const BSONObj& data) { return data.hasField(kPeriodField); });
-    return r;
+        ->executeIf(
+            [&result](const BSONObj& data) { result = Seconds{data.getIntField(kPeriodField)}; },
+            [](const BSONObj& data) { return data.hasField(kPeriodField); });
+    return result;
 }
 
 void ScanningReplicaSetMonitor::runScanForMockReplicaSet() {
@@ -494,6 +520,18 @@ void ScanningReplicaSetMonitor::runScanForMockReplicaSet() {
     invariant(_state->currentScan == nullptr);
 }
 
+namespace {
+AtomicWord<bool> refreshRetriesDisabledForTest{false};
+}  // namespace
+
+void ScanningReplicaSetMonitor::disableRefreshRetries_forTest() {
+    refreshRetriesDisabledForTest.store(true);
+}
+
+bool ScanningReplicaSetMonitor::areRefreshRetriesDisabledForTest() {
+    return refreshRetriesDisabledForTest.load();
+}
+
 void ScanningReplicaSetMonitor::_ensureScanInProgress(const SetStatePtr& state) {
     Refresher(state).scheduleNetworkRequests();
 }
@@ -559,7 +597,7 @@ void Refresher::scheduleIsMaster(const HostAndPort& host) {
             auto timer = Timer();
             auto reply = BSONObj();
             bool ignoredOutParam = false;
-            conn->isMaster(ignoredOutParam, &reply);
+            conn->isPrimary(ignoredOutParam, &reply);
             conn.done();  // return to pool on success.
 
             receivedIsMaster(host, timer.micros(), reply);
@@ -572,8 +610,8 @@ void Refresher::scheduleIsMaster(const HostAndPort& host) {
 
     BSONObjBuilder bob;
     bob.append("isMaster", 1);
-    if (WireSpec::instance().isInternalClient) {
-        WireSpec::appendInternalClientWireVersion(WireSpec::instance().outgoing, &bob);
+    if (auto wireSpec = WireSpec::instance().get(); wireSpec->isInternalClient) {
+        WireSpec::appendInternalClientWireVersion(wireSpec->outgoing, &bob);
     }
     auto request = executor::RemoteCommandRequest(host, "admin", bob.obj(), nullptr, kCheckTimeout);
     request.sslMode = _set->setUri.getSSLMode();
@@ -956,7 +994,7 @@ void IsMasterReply::parse(const BSONObj& obj) {
         maxWireVersion = raw["maxWireVersion"].numberInt();
 
         // hidden nodes can't be master, even if they claim to be.
-        isMaster = !hidden && raw["ismaster"].trueValue();
+        isMaster = !hidden && (raw["isWritablePrimary"].trueValue() || raw["ismaster"].trueValue());
 
         if (isMaster && raw.hasField("electionId")) {
             electionId = raw["electionId"].OID();
@@ -1106,7 +1144,7 @@ SetState::SetState(const MongoURI& uri,
       seedNodes(setUri.getServers().begin(), setUri.getServers().end()),
       latencyThresholdMicros(serverGlobalParams.defaultLocalThresholdMillis * int64_t(1000)),
       rand(std::random_device()()),
-      refreshPeriod(getDefaultRefreshPeriod()) {
+      refreshPeriod(_getRefreshPeriod()) {
     uassert(13642, "Replica set seed list can't be empty", !seedNodes.empty());
 
     if (name.empty())
@@ -1135,8 +1173,9 @@ SetState::SetState(const MongoURI& uri,
         checkInvariants();
 }
 
-HostAndPort SetState::getMatchingHost(const ReadPreferenceSetting& criteria) const {
-    auto hosts = getMatchingHosts(criteria);
+HostAndPort SetState::getMatchingHost(const ReadPreferenceSetting& criteria,
+                                      const std::vector<HostAndPort>& excludedHosts) const {
+    auto hosts = getMatchingHosts(criteria, excludedHosts);
 
     if (hosts.empty()) {
         return HostAndPort();
@@ -1145,27 +1184,32 @@ HostAndPort SetState::getMatchingHost(const ReadPreferenceSetting& criteria) con
     return hosts[0];
 }
 
-std::vector<HostAndPort> SetState::getMatchingHosts(const ReadPreferenceSetting& criteria) const {
+std::vector<HostAndPort> SetState::getMatchingHosts(
+    const ReadPreferenceSetting& criteria, const std::vector<HostAndPort>& excludedHosts) const {
     switch (criteria.pref) {
         // "Prefered" read preferences are defined in terms of other preferences
         case ReadPreference::PrimaryPreferred: {
-            std::vector<HostAndPort> out =
-                getMatchingHosts(ReadPreferenceSetting(ReadPreference::PrimaryOnly, criteria.tags));
+            std::vector<HostAndPort> out = getMatchingHosts(
+                ReadPreferenceSetting(ReadPreference::PrimaryOnly, criteria.tags), excludedHosts);
             // NOTE: the spec says we should use the primary even if tags don't match
             if (!out.empty())
                 return out;
-            return getMatchingHosts(ReadPreferenceSetting(
-                ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessSeconds));
+            return getMatchingHosts(ReadPreferenceSetting(ReadPreference::SecondaryOnly,
+                                                          criteria.tags,
+                                                          criteria.maxStalenessSeconds),
+                                    excludedHosts);
         }
 
         case ReadPreference::SecondaryPreferred: {
-            std::vector<HostAndPort> out = getMatchingHosts(ReadPreferenceSetting(
-                ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessSeconds));
+            std::vector<HostAndPort> out = getMatchingHosts(
+                ReadPreferenceSetting(
+                    ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessSeconds),
+                excludedHosts);
             if (!out.empty())
                 return out;
             // NOTE: the spec says we should use the primary even if tags don't match
             return getMatchingHosts(
-                ReadPreferenceSetting(ReadPreference::PrimaryOnly, criteria.tags));
+                ReadPreferenceSetting(ReadPreference::PrimaryOnly, criteria.tags), excludedHosts);
         }
 
         case ReadPreference::PrimaryOnly: {
@@ -1173,7 +1217,15 @@ std::vector<HostAndPort> SetState::getMatchingHosts(const ReadPreferenceSetting&
             Nodes::const_iterator it = std::find_if(nodes.begin(), nodes.end(), isMaster);
             if (it == nodes.end())
                 return {};
-            return {it->host};
+            const auto& primaryHost = it->host;
+
+            // If the primary is excluded and we have specified readPreference primaryOnly, return
+            // empty host.
+            if (std::find(excludedHosts.begin(), excludedHosts.end(), primaryHost) !=
+                excludedHosts.end()) {
+                return {};
+            }
+            return {primaryHost};
         }
 
         // The difference between these is handled by Node::matches
@@ -1227,20 +1279,23 @@ std::vector<HostAndPort> SetState::getMatchingHosts(const ReadPreferenceSetting&
 
                 std::vector<const Node*> matchingNodes;
                 for (size_t i = 0; i < nodes.size(); i++) {
+                    auto isNotExcluded =
+                        (std::find(excludedHosts.begin(), excludedHosts.end(), nodes[i].host) ==
+                         excludedHosts.end());
                     if (nodes[i].matches(criteria.pref) && nodes[i].matches(tag) &&
-                        matchNode(nodes[i])) {
+                        matchNode(nodes[i]) && isNotExcluded) {
                         matchingNodes.push_back(&nodes[i]);
                     }
                 }
 
-                // Only consider nodes that satisfy the minOpTime
-                if (!criteria.minOpTime.isNull()) {
+                // Only consider nodes that satisfy the minClusterTime
+                if (!criteria.minClusterTime.isNull()) {
                     std::sort(matchingNodes.begin(), matchingNodes.end(), opTimeGreater);
                     for (size_t i = 0; i < matchingNodes.size(); i++) {
-                        if (matchingNodes[i]->opTime < criteria.minOpTime) {
+                        if (matchingNodes[i]->opTime.getTimestamp() < criteria.minClusterTime) {
                             if (i == 0) {
-                                // If no nodes satisfy the minOpTime criteria, we ignore the
-                                // minOpTime requirement.
+                                // If no nodes satisfy the minClusterTime criteria, we ignore the
+                                // minClusterTime requirement.
                                 break;
                             }
                             matchingNodes.erase(matchingNodes.begin() + i, matchingNodes.end());
@@ -1261,8 +1316,8 @@ std::vector<HostAndPort> SetState::getMatchingHosts(const ReadPreferenceSetting&
                 return {allMatchingNodes.front()->host};
             }
 
-            // If there are multiple nodes satisfying the minOpTime, next order by latency
-            // and don't consider hosts further than a threshold from the closest.
+            // If there are multiple nodes satisfying the minClusterTime, next order by latency and
+            // don't consider hosts further than a threshold from the closest.
             std::sort(allMatchingNodes.begin(), allMatchingNodes.end(), compareLatencies);
 
             if (!MONGO_unlikely(scanningServerSelectorIgnoreLatencyWindow.shouldFail())) {
@@ -1376,8 +1431,7 @@ void SetState::notify() {
             waiters.erase(it++);
             continue;
         }
-
-        auto match = getMatchingHosts(it->criteria);
+        auto match = getMatchingHosts(it->criteria, it->excludedHosts);
         if (!match.empty()) {
             // match;
             it->promise.emplaceValue(std::move(match));
@@ -1419,6 +1473,7 @@ void SetState::init() {
 }
 
 void SetState::drop() {
+    // This is invoked from ScanningReplicaSetMonitor::drop() under lock.
     if (std::exchange(isDropped, true)) {
         // If a SetState calls drop() from destruction after the RSMM calls shutdown(), then the
         // RSMM's executor may no longer exist. Thus, only drop once.
