@@ -127,7 +127,7 @@ Status makeUnsatisfiedReadPrefError(const std::string& name,
 }
 
 Status makeReplicaSetMonitorRemovedError(const std::string& name) {
-    return Status(ErrorCodes::ReplicaSetMonitorRemoved,
+    return Status(ErrorCodes::ShutdownInProgress,
                   str::stream() << "ReplicaSetMonitor for set " << name << " is removed");
 }
 
@@ -185,14 +185,16 @@ StreamableReplicaSetMonitor::StreamableReplicaSetMonitor(
     const MongoURI& uri,
     std::shared_ptr<TaskExecutor> executor,
     std::shared_ptr<executor::EgressTagCloser> connectionManager,
-    std::function<void()> cleanupCallback)
+    std::function<void()> cleanupCallback,
+    std::shared_ptr<ReplicaSetMonitorManagerStats> managerStats)
     : ReplicaSetMonitor(cleanupCallback),
       _errorHandler(std::make_unique<SdamErrorHandler>(uri.getSetName())),
       _queryProcessor(std::make_shared<StreamableReplicaSetMonitorQueryProcessor>()),
       _uri(uri),
       _connectionManager(connectionManager),
       _executor(executor),
-      _random(PseudoRandom(SecureRandom().nextInt64())) {
+      _random(PseudoRandom(SecureRandom().nextInt64())),
+      _stats(std::make_shared<ReplicaSetMonitorStats>(managerStats)) {
     // Maintain order of original seed list
     std::vector<HostAndPort> seedsNoDups;
     std::set<HostAndPort> alreadyAdded;
@@ -205,7 +207,11 @@ StreamableReplicaSetMonitor::StreamableReplicaSetMonitor(
         }
     }
 
-    _sdamConfig = SdamConfiguration(seedsNoDups);
+    // StreamableReplicaSetMonitor cannot be used with kSingle type, thus we know that the type is
+    // kReplicaSetNoPrimary. We need to save the expected set name to avoid the case when the
+    // provided seed address contains a ReplicaSet with different name (deployment mistake).
+    _sdamConfig =
+        SdamConfiguration(seedsNoDups, TopologyType::kReplicaSetNoPrimary, _uri.getSetName());
     _serverSelector = std::make_unique<SdamServerSelector>(_sdamConfig);
 }
 
@@ -219,10 +225,12 @@ ReplicaSetMonitorPtr StreamableReplicaSetMonitor::make(
     const MongoURI& uri,
     std::shared_ptr<TaskExecutor> executor,
     std::shared_ptr<executor::EgressTagCloser> connectionManager,
-    std::function<void()> cleanupCallback) {
+    std::function<void()> cleanupCallback,
+    std::shared_ptr<ReplicaSetMonitorManagerStats> managerStats) {
     auto result = std::make_shared<StreamableReplicaSetMonitor>(
-        uri, executor, connectionManager, cleanupCallback);
+        uri, executor, connectionManager, cleanupCallback, managerStats);
     result->init();
+    invariant(managerStats);
     return result;
 }
 
@@ -248,8 +256,13 @@ void StreamableReplicaSetMonitor::init() {
         _uri, _eventsPublisher.get(), _sdamConfig.getHeartBeatFrequency(), _executor);
     _eventsPublisher->registerListener(_pingMonitor);
 
-    _serverDiscoveryMonitor = std::make_unique<ServerDiscoveryMonitor>(
-        _uri, _sdamConfig, _eventsPublisher, _topologyManager->getTopologyDescription(), _executor);
+    _serverDiscoveryMonitor =
+        std::make_unique<ServerDiscoveryMonitor>(_uri,
+                                                 _sdamConfig,
+                                                 _eventsPublisher,
+                                                 _topologyManager->getTopologyDescription(),
+                                                 _stats,
+                                                 _executor);
     _eventsPublisher->registerListener(_serverDiscoveryMonitor);
 
     _eventsPublisher->registerListener(_queryProcessor);
@@ -390,7 +403,7 @@ SemiFuture<std::vector<HostAndPort>> StreamableReplicaSetMonitor::_enqueueOutsta
     const CancellationToken& cancelToken,
     const Date_t& deadline) {
 
-    auto query = std::make_shared<HostQuery>();
+    auto query = std::make_shared<HostQuery>(_stats);
     query->criteria = criteria;
     query->excludedHosts = excludedHosts;
 
@@ -714,6 +727,21 @@ void StreamableReplicaSetMonitor::onTopologyDescriptionChangedEvent(
             }
         }
     }
+
+    if (auto previousMaxElectionIdSetVersionPair =
+            previousDescription->getMaxElectionIdSetVersionPair(),
+        newMaxElectionIdSetVersionPair = newDescription->getMaxElectionIdSetVersionPair();
+        setVersionWentBackwards(previousMaxElectionIdSetVersionPair,
+                                newMaxElectionIdSetVersionPair)) {
+        // The previous primary was unable to reach consensus for the config with
+        // higher version and it was abandoned after failover.
+        LOGV2(5940902,
+              "Max known Set version coming from new primary forces to rollback it backwards",
+              "replicaSet"_attr = getName(),
+              "newElectionIdSetVersion"_attr = newMaxElectionIdSetVersionPair.setVersion,
+              "previousMaxElectionIdSetVersion"_attr =
+                  previousMaxElectionIdSetVersionPair.setVersion);
+    }
 }
 
 void StreamableReplicaSetMonitor::onServerHeartbeatSucceededEvent(const HostAndPort& hostAndPort,
