@@ -455,9 +455,6 @@ void addClusterManagerPrivileges(PrivilegeVector* privileges) {
     Privilege::addPrivilegeToPrivilegeVector(
         privileges,
         Privilege(ResourcePattern::forAnyNormalResource(), clusterManagerRoleDatabaseActions));
-    Privilege::addPrivilegeToPrivilegeVector(
-        privileges,
-        Privilege(ResourcePattern::forAnySystemBuckets(), clusterManagerRoleDatabaseActions));
     Privilege::addPrivilegeToPrivilegeVector(
         privileges,
         Privilege(ResourcePattern::forDatabaseName("config"), clusterManagerRoleDatabaseActions));
@@ -470,13 +467,6 @@ void addClusterManagerPrivileges(PrivilegeVector* privileges) {
                   readRoleActions));
     addReadOnlyDbPrivileges(privileges, "config");
 
-    Privilege::addPrivilegeToPrivilegeVector(
-        privileges, Privilege(ResourcePattern::forAnyResource(), ActionType::dbCheck));
-    Privilege::addPrivilegeToPrivilegeVector(
-        privileges,
-        Privilege(ResourcePattern::forExactNamespace(NamespaceString("local", "system.healthlog")),
-                  readRoleActions));
-
     ActionSet writeActions;
     writeActions << ActionType::insert << ActionType::update << ActionType::remove;
     Privilege::addPrivilegeToPrivilegeVector(
@@ -677,13 +667,7 @@ void addRestorePrivileges(PrivilegeVector* privileges) {
     Privilege::addPrivilegeToPrivilegeVector(
         privileges,
         Privilege(ResourcePattern::forClusterResource(),
-                  {
-                      // Need to be able to force UUID consistency in sharded restores
-                      ActionType::forceUUID,
-                      ActionType::useUUID,
-                      // Needed for `mongorestore --preserveUUID`
-                      ActionType::applyOps,
-                  }));
+                  {ActionType::forceUUID, ActionType::useUUID}));
 }
 
 void addRootRolePrivileges(PrivilegeVector* privileges) {
@@ -757,21 +741,9 @@ const std::map<StringData, BuiltinRoleDefinition> kBuiltinRoles({
     {BUILTIN_ROLE_INTERNAL, {true, addInternalRolePrivileges}},
 });
 
-// $external is a virtual database used for X509, LDAP,
-// and other authentication mechanisms and not used for storage.
-// Therefore, granting privileges on this database does not make sense.
-bool isValidDB(StringData dbname) {
-    return NamespaceString::validDBName(dbname, NamespaceString::DollarInDbNameBehavior::Allow) &&
-        (dbname != NamespaceString::kExternalDb);
-}
-
 }  // namespace
 
 stdx::unordered_set<RoleName> auth::getBuiltinRoleNamesForDB(StringData dbname) {
-    if (!isValidDB(dbname)) {
-        return {};
-    }
-
     const bool isAdmin = dbname == ADMIN_DBNAME;
 
     stdx::unordered_set<RoleName> roleNames;
@@ -787,7 +759,8 @@ bool auth::addPrivilegesForBuiltinRole(const RoleName& roleName, PrivilegeVector
     auto role = roleName.getRole();
     auto dbname = roleName.getDB();
 
-    if (!isValidDB(dbname)) {
+    if (!NamespaceString::validDBName(dbname, NamespaceString::DollarInDbNameBehavior::Allow) ||
+        dbname == "$external") {
         return false;
     }
 
@@ -814,7 +787,8 @@ void auth::generateUniversalPrivileges(PrivilegeVector* privileges) {
 
 bool auth::isBuiltinRole(const RoleName& role) {
     auto dbname = role.getDB();
-    if (!isValidDB(dbname)) {
+    if (!NamespaceString::validDBName(dbname, NamespaceString::DollarInDbNameBehavior::Allow) ||
+        dbname == "$external") {
         return false;
     }
 
