@@ -95,9 +95,10 @@ struct CollectionUpdateArgs {
 
     StoreDocOption storeDocOption = StoreDocOption::None;
     bool preImageRecordingEnabledForCollection = false;
+    bool changeStreamPreAndPostImagesEnabledForCollection = false;
 
-    // Set if OpTimes were reserved for the update ahead of time.
-    std::vector<OplogSlot> oplogSlots;
+    // Set if an OpTime was reserved for the update ahead of time.
+    boost::optional<OplogSlot> oplogSlot = boost::none;
 
     // When true, store the pre- or post- image for findAndModify commands in the side collection.
     // When false, store the image in the oplog.
@@ -333,29 +334,29 @@ public:
 
     virtual bool requiresIdIndex() const = 0;
 
-    virtual Snapshotted<BSONObj> docFor(OperationContext* const opCtx, RecordId loc) const = 0;
+    virtual Snapshotted<BSONObj> docFor(OperationContext* opCtx, RecordId loc) const = 0;
 
     /**
      * @param out - contents set to the right docs if exists, or nothing.
      * @return true iff loc exists
      */
-    virtual bool findDoc(OperationContext* const opCtx,
+    virtual bool findDoc(OperationContext* opCtx,
                          RecordId loc,
-                         Snapshotted<BSONObj>* const out) const = 0;
+                         Snapshotted<BSONObj>* out) const = 0;
 
-    virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* const opCtx,
-                                                            const bool forward = true) const = 0;
+    virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
+                                                            bool forward = true) const = 0;
 
     /**
      * Deletes the document with the given RecordId from the collection. For a description of the
      * parameters, see the overloaded function below.
      */
-    virtual void deleteDocument(OperationContext* const opCtx,
+    virtual void deleteDocument(OperationContext* opCtx,
                                 StmtId stmtId,
                                 RecordId loc,
-                                OpDebug* const opDebug,
-                                const bool fromMigrate = false,
-                                const bool noWarn = false,
+                                OpDebug* opDebug,
+                                bool fromMigrate = false,
+                                bool noWarn = false,
                                 StoreDeletedDoc storeDeletedDoc = StoreDeletedDoc::Off) const = 0;
 
     /**
@@ -370,13 +371,13 @@ public:
      * 'noWarn' if unindexing the record causes an error, if noWarn is true the error
      * will not be logged.
      */
-    virtual void deleteDocument(OperationContext* const opCtx,
+    virtual void deleteDocument(OperationContext* opCtx,
                                 Snapshotted<BSONObj> doc,
                                 StmtId stmtId,
                                 RecordId loc,
-                                OpDebug* const opDebug,
-                                const bool fromMigrate = false,
-                                const bool noWarn = false,
+                                OpDebug* opDebug,
+                                bool fromMigrate = false,
+                                bool noWarn = false,
                                 StoreDeletedDoc storeDeletedDoc = StoreDeletedDoc::Off) const = 0;
 
     /*
@@ -386,11 +387,11 @@ public:
      *
      * 'opDebug' Optional argument. When not null, will be used to record operation statistics.
      */
-    virtual Status insertDocuments(OperationContext* const opCtx,
-                                   const std::vector<InsertStatement>::const_iterator begin,
-                                   const std::vector<InsertStatement>::const_iterator end,
-                                   OpDebug* const opDebug,
-                                   const bool fromMigrate = false) const = 0;
+    virtual Status insertDocuments(OperationContext* opCtx,
+                                   std::vector<InsertStatement>::const_iterator begin,
+                                   std::vector<InsertStatement>::const_iterator end,
+                                   OpDebug* opDebug,
+                                   bool fromMigrate = false) const = 0;
 
     /**
      * this does NOT modify the doc before inserting
@@ -398,16 +399,16 @@ public:
      *
      * 'opDebug' Optional argument. When not null, will be used to record operation statistics.
      */
-    virtual Status insertDocument(OperationContext* const opCtx,
+    virtual Status insertDocument(OperationContext* opCtx,
                                   const InsertStatement& doc,
-                                  OpDebug* const opDebug,
-                                  const bool fromMigrate = false) const = 0;
+                                  OpDebug* opDebug,
+                                  bool fromMigrate = false) const = 0;
 
     /**
      * Callers must ensure no document validation is performed for this collection when calling
      * this method.
      */
-    virtual Status insertDocumentsForOplog(OperationContext* const opCtx,
+    virtual Status insertDocumentsForOplog(OperationContext* opCtx,
                                            std::vector<Record>* records,
                                            const std::vector<Timestamp>& timestamps) const = 0;
 
@@ -419,7 +420,7 @@ public:
      * NOTE: It is up to caller to commit the indexes.
      */
     virtual Status insertDocumentForBulkLoader(
-        OperationContext* const opCtx,
+        OperationContext* opCtx,
         const BSONObj& doc,
         const OnRecordInsertedFn& onRecordInserted) const = 0;
 
@@ -432,13 +433,13 @@ public:
      * 'opDebug' Optional argument. When not null, will be used to record operation statistics.
      * @return the post update location of the doc (may or may not be the same as oldLocation)
      */
-    virtual RecordId updateDocument(OperationContext* const opCtx,
+    virtual RecordId updateDocument(OperationContext* opCtx,
                                     RecordId oldLocation,
                                     const Snapshotted<BSONObj>& oldDoc,
                                     const BSONObj& newDoc,
-                                    const bool indexesAffected,
-                                    OpDebug* const opDebug,
-                                    CollectionUpdateArgs* const args) const = 0;
+                                    bool indexesAffected,
+                                    OpDebug* opDebug,
+                                    CollectionUpdateArgs* args) const = 0;
 
     virtual bool updateWithDamagesSupported() const = 0;
 
@@ -450,12 +451,12 @@ public:
      * @return the contents of the updated record.
      */
     virtual StatusWith<RecordData> updateDocumentWithDamages(
-        OperationContext* const opCtx,
+        OperationContext* opCtx,
         RecordId loc,
         const Snapshotted<RecordData>& oldRec,
-        const char* const damageSource,
+        const char* damageSource,
         const mutablebson::DamageVector& damages,
-        CollectionUpdateArgs* const args) const = 0;
+        CollectionUpdateArgs* args) const = 0;
 
     // -----------
 
@@ -467,7 +468,7 @@ public:
      * The caller should hold a collection X lock and ensure there are no index builds in progress
      * on the collection.
      */
-    virtual Status truncate(OperationContext* const opCtx) = 0;
+    virtual Status truncate(OperationContext* opCtx) = 0;
 
     /**
      * Truncate documents newer than the document at 'end' from the capped
@@ -478,19 +479,18 @@ public:
      * The caller should hold a collection X lock and ensure there are no index builds in progress
      * on the collection.
      */
-    virtual void cappedTruncateAfter(OperationContext* const opCtx,
+    virtual void cappedTruncateAfter(OperationContext* opCtx,
                                      RecordId end,
-                                     const bool inclusive) const = 0;
+                                     bool inclusive) const = 0;
 
     /**
      * Returns a non-ok Status if validator is not legal for this collection.
      */
-    virtual Validator parseValidator(
-        OperationContext* opCtx,
-        const BSONObj& validator,
-        MatchExpressionParser::AllowedFeatureSet allowedFeatures,
-        boost::optional<ServerGlobalParams::FeatureCompatibility::Version>
-            maxFeatureCompatibilityVersion) const = 0;
+    virtual Validator parseValidator(OperationContext* opCtx,
+                                     const BSONObj& validator,
+                                     MatchExpressionParser::AllowedFeatureSet allowedFeatures,
+                                     boost::optional<multiversion::FeatureCompatibilityVersion>
+                                         maxFeatureCompatibilityVersion) const = 0;
 
     /**
      * Sets the validator for this collection.
@@ -498,12 +498,10 @@ public:
      * An empty validator removes all validation.
      * Requires an exclusive lock on the collection.
      */
-    virtual void setValidator(OperationContext* const opCtx, Validator validator) = 0;
+    virtual void setValidator(OperationContext* opCtx, Validator validator) = 0;
 
-    virtual Status setValidationLevel(OperationContext* const opCtx,
-                                      ValidationLevelEnum newLevel) = 0;
-    virtual Status setValidationAction(OperationContext* const opCtx,
-                                       ValidationActionEnum newAction) = 0;
+    virtual Status setValidationLevel(OperationContext* opCtx, ValidationLevelEnum newLevel) = 0;
+    virtual Status setValidationAction(OperationContext* opCtx, ValidationActionEnum newAction) = 0;
 
     virtual boost::optional<ValidationLevelEnum> getValidationLevel() const = 0;
     virtual boost::optional<ValidationActionEnum> getValidationAction() const = 0;
@@ -518,30 +516,17 @@ public:
     virtual bool getRecordPreImages() const = 0;
     virtual void setRecordPreImages(OperationContext* opCtx, bool val) = 0;
 
+    virtual bool isChangeStreamPreAndPostImagesEnabled() const = 0;
+    virtual void setChangeStreamPreAndPostImages(OperationContext* opCtx, bool val) = 0;
+
     /**
      * Returns true if this is a temporary collection.
      */
     virtual bool isTemporary() const = 0;
 
     /**
-     * Returns true if the time-series collection may have dates outside the standard range (roughly
-     * 1970-2038). The value may be updated in the background by another thread between calls, even
-     * if the caller holds a lock on the collection. The value may only transition from false to
-     * true.
-     */
-    virtual bool getRequiresTimeseriesExtendedRangeSupport() const = 0;
-
-    /**
-     * Sets the in-memory flag for this collection. This value can be retrieved by
-     * 'getRequiresTimeseriesExtendedRangeSupport'.
-     *
-     * Throws if this is not a time-series collection.
-     */
-    virtual void setRequiresTimeseriesExtendedRangeSupport(OperationContext* opCtx) const = 0;
-
-    /*
-     * Returns true if this collection is clustered. That is, its RecordIds store the value of the
-     * cluster key. If the collection is clustered on _id, there is no separate _id index.
+     * Returns true if this collection is clustered on _id values. That is, its RecordIds are _id
+     * values and has no separate _id index.
      */
     virtual bool isClustered() const = 0;
 
@@ -560,9 +545,10 @@ public:
 
     /**
      * Checks that the metadata for the index exists and matches the given spec.
+     * Returns offset in metadata on success.
      */
-    virtual Status checkMetaDataForIndex(const std::string& indexName,
-                                         const BSONObj& spec) const = 0;
+    virtual StatusWith<int> checkMetaDataForIndex(const std::string& indexName,
+                                                  const BSONObj& spec) const = 0;
 
     /*
      * Updates the expireAfterSeconds field of the given index to the value in newExpireSecs.
@@ -625,7 +611,8 @@ public:
      */
     virtual bool isIndexMultikey(OperationContext* opCtx,
                                  StringData indexName,
-                                 MultikeyPaths* multikeyPaths) const = 0;
+                                 MultikeyPaths* multikeyPaths,
+                                 int indexOffset = -1) const = 0;
 
     /**
      * Sets the index identified by 'indexName' to be multikey.
@@ -638,7 +625,8 @@ public:
      */
     virtual bool setIndexIsMultikey(OperationContext* opCtx,
                                     StringData indexName,
-                                    const MultikeyPaths& multikeyPaths) const = 0;
+                                    const MultikeyPaths& multikeyPaths,
+                                    int indexOffset = -1) const = 0;
 
     /**
      * Sets the index to be multikey with the provided paths. This performs minimal validation of
@@ -695,26 +683,26 @@ public:
      */
     virtual std::shared_ptr<CappedInsertNotifier> getCappedInsertNotifier() const = 0;
 
-    virtual long long numRecords(OperationContext* const opCtx) const = 0;
+    virtual long long numRecords(OperationContext* opCtx) const = 0;
 
-    virtual long long dataSize(OperationContext* const opCtx) const = 0;
+    virtual long long dataSize(OperationContext* opCtx) const = 0;
 
 
     /**
      * Returns true if the collection does not contain any records.
      */
-    virtual bool isEmpty(OperationContext* const opCtx) const = 0;
+    virtual bool isEmpty(OperationContext* opCtx) const = 0;
 
-    virtual int averageObjectSize(OperationContext* const opCtx) const = 0;
+    virtual int averageObjectSize(OperationContext* opCtx) const = 0;
 
-    virtual uint64_t getIndexSize(OperationContext* const opCtx,
-                                  BSONObjBuilder* const details = nullptr,
-                                  const int scale = 1) const = 0;
+    virtual uint64_t getIndexSize(OperationContext* opCtx,
+                                  BSONObjBuilder* details = nullptr,
+                                  int scale = 1) const = 0;
 
     /**
      * Returns the number of unused, free bytes used by all indexes on disk.
      */
-    virtual uint64_t getIndexFreeStorageBytes(OperationContext* const opCtx) const = 0;
+    virtual uint64_t getIndexFreeStorageBytes(OperationContext* opCtx) const = 0;
 
     /**
      * If return value is not boost::none, reads with majority read concern using an older snapshot
@@ -722,7 +710,7 @@ public:
      */
     virtual boost::optional<Timestamp> getMinimumVisibleSnapshot() const = 0;
 
-    virtual void setMinimumVisibleSnapshot(const Timestamp name) = 0;
+    virtual void setMinimumVisibleSnapshot(Timestamp name) = 0;
 
     /**
      * Returns the time-series options for this buckets collection, or boost::none if not a
@@ -846,6 +834,14 @@ public:
     void yield() const override;
     void restore() const override;
 
+    RestoreFn detachRestoreFn() {
+        return std::move(_restoreFn);
+    }
+
+    void attachRestoreFn(RestoreFn newRestoreFn) {
+        _restoreFn = std::move(newRestoreFn);
+    }
+
     friend std::ostream& operator<<(std::ostream& os, const CollectionPtr& coll);
 
     void setShardKeyPattern(const BSONObj& shardKeyPattern) {
