@@ -34,8 +34,8 @@
 
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/uncommitted_collections.h"
-#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/snapshot_helper.h"
@@ -339,8 +339,7 @@ CollectionCatalog::iterator::value_type CollectionCatalog::iterator::operator*()
         return CollectionPtr();
     }
 
-    return {
-        _opCtx, _mapIter->second.get(), LookupCollectionForYieldRestore(_mapIter->second->ns())};
+    return {_opCtx, _mapIter->second.get(), LookupCollectionForYieldRestore()};
 }
 
 Collection* CollectionCatalog::iterator::getWritableCollection(OperationContext* opCtx,
@@ -379,7 +378,7 @@ CollectionCatalog::iterator CollectionCatalog::iterator::operator++(int) {
     return oldPosition;
 }
 
-bool CollectionCatalog::iterator::operator==(const iterator& other) {
+bool CollectionCatalog::iterator::operator==(const iterator& other) const {
     invariant(_catalog == other._catalog);
     if (other._mapIter == _catalog->_orderedCollections.end()) {
         return _uuid == boost::none;
@@ -388,7 +387,7 @@ bool CollectionCatalog::iterator::operator==(const iterator& other) {
     return _uuid == other._uuid;
 }
 
-bool CollectionCatalog::iterator::operator!=(const iterator& other) {
+bool CollectionCatalog::iterator::operator!=(const iterator& other) const {
     return !(*this == other);
 }
 
@@ -586,17 +585,16 @@ void CollectionCatalog::onCloseDatabase(OperationContext* opCtx, std::string dbN
     removeResource(rid, dbName);
 }
 
-void CollectionCatalog::onCloseCatalog() {
-    if (_shadowCatalog) {
-        return;
-    }
-
+void CollectionCatalog::onCloseCatalog(OperationContext* opCtx) {
+    invariant(opCtx->lockState()->isW());
+    invariant(!_shadowCatalog);
     _shadowCatalog.emplace();
     for (auto& entry : _catalog)
         _shadowCatalog->insert({entry.first, entry.second->ns()});
 }
 
-void CollectionCatalog::onOpenCatalog() {
+void CollectionCatalog::onOpenCatalog(OperationContext* opCtx) {
+    invariant(opCtx->lockState()->isW());
     invariant(_shadowCatalog);
     _shadowCatalog.reset();
     ++_epoch;
@@ -667,12 +665,12 @@ CollectionPtr CollectionCatalog::lookupCollectionByUUID(OperationContext* opCtx,
     }
 
     if (auto coll = UncommittedCollections::getForTxn(opCtx, uuid)) {
-        return {opCtx, coll.get(), LookupCollectionForYieldRestore(coll->ns())};
+        return {opCtx, coll.get(), LookupCollectionForYieldRestore()};
     }
 
     auto coll = _lookupCollectionByUUID(uuid);
     return (coll && coll->isCommitted())
-        ? CollectionPtr(opCtx, coll.get(), LookupCollectionForYieldRestore(coll->ns()))
+        ? CollectionPtr(opCtx, coll.get(), LookupCollectionForYieldRestore())
         : CollectionPtr();
 }
 
@@ -752,7 +750,7 @@ CollectionPtr CollectionCatalog::lookupCollectionByNamespace(OperationContext* o
     // If found=true above but we don't have a Collection pointer it is a drop or rename. But first
     // check UncommittedCollections in case we find a new collection there.
     if (auto coll = UncommittedCollections::getForTxn(opCtx, nss)) {
-        return {opCtx, coll.get(), LookupCollectionForYieldRestore(coll->ns())};
+        return {opCtx, coll.get(), LookupCollectionForYieldRestore()};
     }
 
     // Report the drop or rename as nothing new was created.
@@ -763,7 +761,7 @@ CollectionPtr CollectionCatalog::lookupCollectionByNamespace(OperationContext* o
     auto it = _collections.find(nss);
     auto coll = (it == _collections.end() ? nullptr : it->second);
     return (coll && coll->isCommitted())
-        ? CollectionPtr(opCtx, coll.get(), LookupCollectionForYieldRestore(coll->ns()))
+        ? CollectionPtr(opCtx, coll.get(), LookupCollectionForYieldRestore())
         : nullptr;
 }
 
@@ -1199,28 +1197,21 @@ void CollectionCatalogStasher::reset() {
 
 const Collection* LookupCollectionForYieldRestore::operator()(OperationContext* opCtx,
                                                               CollectionUUID uuid) const {
-    auto collection = CollectionCatalog::get(opCtx)->lookupCollectionByUUIDForRead(opCtx, uuid);
-
-    // Collection dropped during yielding.
-    if (!collection) {
+    auto collection = CollectionCatalog::get(opCtx)->lookupCollectionByUUID(opCtx, uuid).get();
+    if (!collection)
         return nullptr;
-    }
-
-    // Collection renamed during yielding.
-    // This check ensures that we are locked on the same namespace and that it is safe to return
-    // the C-style pointer to the Collection.
-    if (collection->ns() != _nss) {
-        return nullptr;
-    }
 
     // After yielding and reacquiring locks, the preconditions that were used to select our
     // ReadSource initially need to be checked again. We select a ReadSource based on replication
     // state. After a query yields its locks, the replication state may have changed, invalidating
     // our current choice of ReadSource. Using the same preconditions, change our ReadSource if
     // necessary.
-    SnapshotHelper::changeReadSourceIfNeeded(opCtx, collection->ns());
+    auto [newReadSource, _] = SnapshotHelper::shouldChangeReadSource(opCtx, collection->ns());
+    if (newReadSource) {
+        opCtx->recoveryUnit()->setTimestampReadSource(*newReadSource);
+    }
 
-    return collection.get();
+    return collection;
 }
 
 BatchedCollectionCatalogWriter::BatchedCollectionCatalogWriter(OperationContext* opCtx)
