@@ -89,8 +89,8 @@ public:
          * Equality operators == and != do not attempt to reposition the iterators being compared.
          * The behavior for comparing invalid iterators is undefined.
          */
-        bool operator==(const iterator& other);
-        bool operator!=(const iterator& other);
+        bool operator==(const iterator& other) const;
+        bool operator!=(const iterator& other) const;
 
     private:
         bool _exhausted();
@@ -118,7 +118,7 @@ public:
 
         ProfileSettings() = default;
 
-        bool operator==(const ProfileSettings& other) {
+        bool operator==(const ProfileSettings& other) const {
             return level == other.level && filter == other.filter;
         }
     };
@@ -208,9 +208,18 @@ public:
 
     /**
      * This function gets the Collection pointer that corresponds to the CollectionUUID.
+     *
      * The required locks must be obtained prior to calling this function, or else the found
      * Collection pointer might no longer be valid when the call returns.
      *
+     * 'lookupCollectionByUUIDForMetadataWrite' requires a MODE_X collection lock, returns a copy to
+     * the caller because catalog updates are copy-on-write.
+     *
+     * 'lookupCollectionByUUID' requires a MODE_IS collection lock.
+     *
+     * 'lookupCollectionByUUIDForRead' does not require locks and should only be used in the context
+     * of a lock-free read wherein we also have a consistent storage snapshot.
+     *
      * Returns nullptr if the 'uuid' is not known.
      */
     Collection* lookupCollectionByUUIDForMetadataWrite(OperationContext* opCtx,
@@ -227,10 +236,19 @@ public:
     bool isCollectionAwaitingVisibility(CollectionUUID uuid) const;
 
     /**
-     * This function gets the Collection pointer that corresponds to the NamespaceString.
+     * These functions fetch a Collection pointer that corresponds to the NamespaceString.
+     *
      * The required locks must be obtained prior to calling this function, or else the found
      * Collection pointer may no longer be valid when the call returns.
      *
+     * 'lookupCollectionByNamespaceForMetadataWrite' requires a MODE_X collection lock, returns a
+     * copy to the caller because catalog updates are copy-on-write.
+     *
+     * 'lookupCollectionByNamespace' requires a MODE_IS collection lock.
+     *
+     * 'lookupCollectionByNamespaceForRead' does not require locks and should only be used in the
+     * context of a lock-free read wherein we also have a consistent storage snapshot.
+     *
      * Returns nullptr if the namespace is unknown.
      */
     Collection* lookupCollectionByNamespaceForMetadataWrite(OperationContext* opCtx,
@@ -359,14 +377,14 @@ public:
      *
      * Must be called with the global lock acquired in exclusive mode.
      */
-    void onCloseCatalog();
+    void onCloseCatalog(OperationContext* opCtx);
 
     /**
      * Puts the catatlog back in open state, removing the pre-close state. See onCloseCatalog.
      *
      * Must be called with the global lock acquired in exclusive mode.
      */
-    void onOpenCatalog();
+    void onOpenCatalog(OperationContext* opCtx);
 
     /**
      * The epoch is incremented whenever the catalog is closed and re-opened.
@@ -501,11 +519,7 @@ private:
  * restore implementation for CollectionPtr when acquired from the catalog.
  */
 struct LookupCollectionForYieldRestore {
-    explicit LookupCollectionForYieldRestore(const NamespaceString& nss) : _nss(nss) {}
     const Collection* operator()(OperationContext* opCtx, CollectionUUID uuid) const;
-
-private:
-    const NamespaceString _nss;
 };
 
 /**
