@@ -35,7 +35,7 @@ namespace mongo {
 
 class DatabaseImpl final : public Database {
 public:
-    explicit DatabaseImpl(StringData name, uint64_t epoch);
+    explicit DatabaseImpl(StringData name);
 
     void init(OperationContext*) const final;
 
@@ -45,18 +45,6 @@ public:
 
     void clearTmpCollections(OperationContext* opCtx) const final;
 
-    /**
-     * Sets a new profiling level for the database and returns the outcome.
-     *
-     * @param opCtx Operation context which to use for creating the profiling collection.
-     * @param newLevel New profiling level to use.
-     */
-    Status setProfilingLevel(OperationContext* opCtx, int newLevel) final;
-
-    int getProfilingLevel() const final {
-        return _profile.load();
-    }
-
     void setDropPending(OperationContext* opCtx, bool dropPending) final;
 
     bool isDropPending(OperationContext* opCtx) const final;
@@ -78,7 +66,8 @@ public:
                           repl::OpTime dropOpTime) const final;
     Status dropCollectionEvenIfSystem(OperationContext* opCtx,
                                       NamespaceString nss,
-                                      repl::OpTime dropOpTime) const final;
+                                      repl::OpTime dropOpTime,
+                                      bool markFromMigrate = false) const final;
 
     Status dropView(OperationContext* opCtx, NamespaceString viewName) const final;
 
@@ -124,21 +113,10 @@ public:
 
     void checkForIdIndexesAndDropPendingCollections(OperationContext* opCtx) const final;
 
-    CollectionCatalog::iterator begin(OperationContext* opCtx) const final {
-        return CollectionCatalog::get(opCtx).begin(_name);
-    }
-
-    CollectionCatalog::iterator end(OperationContext* opCtx) const final {
-        return CollectionCatalog::get(opCtx).end();
-    }
-
-    uint64_t epoch() const {
-        return _epoch;
-    }
-
 private:
     /**
-     * Throws if there is a reason 'ns' cannot be created as a user collection.
+     * Throws if there is a reason 'ns' cannot be created as a user collection. Namespace pattern
+     * matching checks should be added to userAllowedCreateNS().
      */
     void _checkCanCreateCollection(OperationContext* opCtx,
                                    const NamespaceString& nss,
@@ -163,12 +141,8 @@ private:
 
     const std::string _name;  // "dbname"
 
-    const uint64_t _epoch;
-
     const NamespaceString _viewsName;  // "dbname.system.views"
 
-    AtomicWord<int> _profile{0};  // 0=off
-
     // If '_dropPending' is true, this Database is in the midst of a two-phase drop. No new
     // collections may be created in this Database.
     // This variable may only be read/written while the database is locked in MODE_X.
