@@ -82,9 +82,7 @@ void IndexBuildBlock::_completeInit(OperationContext* opCtx, Collection* collect
     auto desc = getEntry(opCtx, collection)->descriptor();
     CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, collection);
     CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
-        .registerIndex(desc->indexName(),
-                       desc->keyPattern(),
-                       IndexFeatures::make(desc, collection->ns().isOnInternalDb()));
+        .registerIndex(desc->indexName(), desc->keyPattern());
 }
 
 Status IndexBuildBlock::initForResume(OperationContext* opCtx,
@@ -268,10 +266,7 @@ void IndexBuildBlock::success(OperationContext* opCtx, Collection* collection) {
             // Add the index to the TTLCollectionCache upon successfully committing the index build.
             // TTL indexes are not compatible with capped collections.
             if (spec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName) && !coll->isCapped()) {
-                TTLCollectionCache::get(svcCtx).registerTTLInfo(
-                    coll->uuid(),
-                    TTLCollectionCache::Info{
-                        indexName, spec[IndexDescriptor::kExpireAfterSecondsFieldName].isNaN()});
+                TTLCollectionCache::get(svcCtx).registerTTLInfo(coll->uuid(), indexName);
             }
         });
 }
