@@ -39,7 +39,7 @@
 #include "mongo/base/init.h"
 #include "mongo/db/catalog/collection.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/matcher/expression.h"
@@ -71,19 +71,13 @@ IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
       _ordering(Ordering::make(_descriptor->keyPattern())),
       _isReady(false),
       _isFrozen(isFrozen),
-      _isDropped(false) {
+      _isDropped(false),
+      _indexOffset(invariantStatusOK(
+          collection->checkMetaDataForIndex(_descriptor->indexName(), _descriptor->infoObj()))) {
 
     _descriptor->_entry = this;
     _isReady = collection->isIndexReady(_descriptor->indexName());
 
-    {
-        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        const bool isMultikey = _catalogIsMultikey(opCtx, collection, &_indexMultikeyPathsForRead);
-        _isMultikeyForRead.store(isMultikey);
-        _isMultikeyForWrite.store(isMultikey);
-        _indexTracksMultikeyPathsInCatalog = !_indexMultikeyPathsForRead.empty();
-    }
-
     auto nss = DurableCatalog::get(opCtx)->getEntry(_catalogId).nss;
     const BSONObj& collation = _descriptor->collation();
     if (!collation.isEmpty()) {
@@ -147,13 +141,17 @@ bool IndexCatalogEntryImpl::isFrozen() const {
     return _isFrozen;
 }
 
-bool IndexCatalogEntryImpl::isMultikey() const {
-    return _isMultikeyForRead.load();
+bool IndexCatalogEntryImpl::isMultikey(OperationContext* const opCtx,
+                                       const CollectionPtr& collection) const {
+    return _catalogIsMultikey(opCtx, collection, nullptr);
 }
 
-MultikeyPaths IndexCatalogEntryImpl::getMultikeyPaths(OperationContext* opCtx) const {
-    stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-    return _indexMultikeyPathsForRead;
+MultikeyPaths IndexCatalogEntryImpl::getMultikeyPaths(OperationContext* opCtx,
+                                                      const CollectionPtr& collection) const {
+    MultikeyPaths indexMultikeyPathsForRead;
+    [[maybe_unused]] const bool isMultikeyInCatalog =
+        _catalogIsMultikey(opCtx, collection, &indexMultikeyPathsForRead);
+    return indexMultikeyPathsForRead;
 }
 
 // ---
@@ -174,18 +172,18 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
                                         const MultikeyPaths& multikeyPaths) const {
     // An index can either track path-level multikey information in the catalog or as metadata keys
     // in the index itself, but not both.
-    invariant(!(_indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.size() > 0));
+    MultikeyPaths indexMultikeyPathsForWrite;
+    auto isMultikeyForWrite = _catalogIsMultikey(opCtx, collection, &indexMultikeyPathsForWrite);
+    auto indexTracksMultikeyPathsInCatalog = !indexMultikeyPathsForWrite.empty();
+    invariant(!(indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.size() > 0));
     // If the index is already set as multikey and we don't have any path-level information to
     // update, then there's nothing more for us to do.
-    bool hasNoPathLevelInfo = (!_indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.empty());
-    if (hasNoPathLevelInfo && _isMultikeyForWrite.load()) {
+    bool hasNoPathLevelInfo = (!indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.empty());
+    if (hasNoPathLevelInfo && isMultikeyForWrite) {
         return;
     }
 
-    if (_indexTracksMultikeyPathsInCatalog) {
-        MultikeyPaths indexMultikeyPathsForWrite;
-        [[maybe_unused]] const bool isMultikeyInCatalog =
-            _catalogIsMultikey(opCtx, collection, &indexMultikeyPathsForWrite);
+    if (indexTracksMultikeyPathsInCatalog) {
         invariant(multikeyPaths.size() == indexMultikeyPathsForWrite.size());
 
         bool newPathIsMultikey = false;
@@ -208,7 +206,7 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
         }
     }
 
-    MultikeyPaths paths = _indexTracksMultikeyPathsInCatalog ? multikeyPaths : MultikeyPaths{};
+    MultikeyPaths paths = indexTracksMultikeyPathsInCatalog ? multikeyPaths : MultikeyPaths{};
 
     // On a primary, we can simply assign this write the same timestamp as the index creation,
     // insert, or update that caused this index to become multikey. This is because if two
@@ -273,18 +271,6 @@ void IndexCatalogEntryImpl::forceSetMultikey(OperationContext* const opCtx,
     // currently support path-level multikey path tracking.
     coll->forceSetIndexIsMultikey(opCtx, _descriptor.get(), isMultikey, multikeyPaths);
 
-    // The prior call to set the multikey metadata in the catalog does some validation and clean up
-    // based on the inputs, so reset the multikey variables based on what is actually in the durable
-    // catalog entry.
-    {
-        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        const bool isMultikeyInCatalog =
-            _catalogIsMultikey(opCtx, coll, &_indexMultikeyPathsForRead);
-        _isMultikeyForRead.store(isMultikeyInCatalog);
-        _isMultikeyForWrite.store(isMultikeyInCatalog);
-        _indexTracksMultikeyPathsInCatalog = !_indexMultikeyPathsForRead.empty();
-    }
-
     // Since multikey metadata has changed, invalidate the query cache.
     CollectionQueryInfo::get(coll).clearQueryCacheForSetMultikey(coll);
 }
@@ -375,7 +361,8 @@ bool IndexCatalogEntryImpl::isPresentInMySnapshot(OperationContext* opCtx) const
 bool IndexCatalogEntryImpl::_catalogIsMultikey(OperationContext* opCtx,
                                                const CollectionPtr& collection,
                                                MultikeyPaths* multikeyPaths) const {
-    return collection->isIndexMultikey(opCtx, _descriptor->indexName(), multikeyPaths);
+    return collection->isIndexMultikey(
+        opCtx, _descriptor->indexName(), multikeyPaths, _indexOffset);
 }
 
 void IndexCatalogEntryImpl::_catalogSetMultikey(OperationContext* opCtx,
@@ -386,25 +373,9 @@ void IndexCatalogEntryImpl::_catalogSetMultikey(OperationContext* opCtx,
     // CollectionCatalogEntry::setIndexIsMultikey() requires that we discard the path-level
     // multikey information in order to avoid unintentionally setting path-level multikey
     // information on an index created before 3.4.
-    auto indexMetadataHasChanged =
-        collection->setIndexIsMultikey(opCtx, _descriptor->indexName(), multikeyPaths);
+    auto indexMetadataHasChanged = collection->setIndexIsMultikey(
+        opCtx, _descriptor->indexName(), multikeyPaths, _indexOffset);
 
-    // In the absence of using the storage engine to read from the catalog, we must set multikey
-    // prior to the storage engine transaction committing.
-    //
-    // Moreover, there must not be an `onRollback` handler to reset this back to false. Given a long
-    // enough pause in processing `onRollback` handlers, a later writer that successfully flipped
-    // multikey can be undone. Alternatively, one could use a counter instead of a boolean to avoid
-    // that problem.
-    _isMultikeyForRead.store(true);
-    if (_indexTracksMultikeyPathsInCatalog) {
-        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        [[maybe_unused]] const bool isMultikeyInCatalog =
-            _catalogIsMultikey(opCtx, collection, &_indexMultikeyPathsForRead);
-        for (size_t i = 0; i < multikeyPaths.size(); ++i) {
-            _indexMultikeyPathsForRead[i].insert(multikeyPaths[i].begin(), multikeyPaths[i].end());
-        }
-    }
     if (indexMetadataHasChanged) {
         LOGV2_DEBUG(4718705,
                     1,
@@ -413,13 +384,6 @@ void IndexCatalogEntryImpl::_catalogSetMultikey(OperationContext* opCtx,
                     "keyPattern"_attr = _descriptor->keyPattern());
         CollectionQueryInfo::get(collection).clearQueryCacheForSetMultikey(collection);
     }
-
-    opCtx->recoveryUnit()->onCommit([this](boost::optional<Timestamp>) {
-        // Writers must attempt to flip multikey until it's confirmed a storage engine
-        // transaction successfully commits. Only after this point may a writer optimize out
-        // flipping multikey.
-        _isMultikeyForWrite.store(true);
-    });
 }
 
 }  // namespace mongo
