@@ -124,7 +124,7 @@ public:
     /**
      * Returns true if this index is multikey, and returns false otherwise.
      */
-    bool isMultikey() const final;
+    bool isMultikey(OperationContext* opCtx, const CollectionPtr& collection) const final;
 
     /**
      * Returns the path components that cause this index to be multikey if this index supports
@@ -135,7 +135,8 @@ public:
      * returns a vector with size equal to the number of elements in the index key pattern where
      * each element in the vector is an empty set.
      */
-    MultikeyPaths getMultikeyPaths(OperationContext* opCtx) const final;
+    MultikeyPaths getMultikeyPaths(OperationContext* opCtx,
+                                   const CollectionPtr& collection) const final;
 
     /**
      * Sets this index to be multikey. Information regarding which newly detected path components
@@ -157,7 +158,7 @@ public:
                      const KeyStringSet& multikeyMetadataKeys,
                      const MultikeyPaths& multikeyPaths) const final;
 
-    void forceSetMultikey(OperationContext* const opCtx,
+    void forceSetMultikey(OperationContext* opCtx,
                           const CollectionPtr& coll,
                           bool isMultikey,
                           const MultikeyPaths& multikeyPaths) const final;
@@ -233,38 +234,12 @@ private:
     bool _isFrozen;
     AtomicWord<bool> _isDropped;  // Whether the index drop is committed.
 
-    // Members for multikey are mutable so they can be changed in const functions. They are
-    // synchronized with the '_indexMultikeyPathsMutex' mutex or are atomic. We don't have the ABA
-    // problem as multikey may only go from disabled to enabled. When multikey it stays multikey.
-
-    // Set to true if this index can track path-level multikey information in the catalog. This
-    // member is effectively const after IndexCatalogEntry::init() is called.
-    mutable bool _indexTracksMultikeyPathsInCatalog = false;
-
-    // Set to true if this index may contain multikey data.
-    mutable AtomicWord<bool> _isMultikeyForRead;
-
-    // Set to true after a transaction commit successfully updates multikey on the catalog data. At
-    // this point, future writers do not need to update the catalog.
-    mutable AtomicWord<bool> _isMultikeyForWrite;
-
-    // Controls concurrent access to '_indexMultikeyPathsForRead'.
-    // We acquire this mutex rather than the RESOURCE_METADATA lock as a performance optimization
-    // so that it is cheaper to detect whether there is actually any path-level multikey
-    // information to update or not.
-    mutable Mutex _indexMultikeyPathsMutex =
-        MONGO_MAKE_LATCH("IndexCatalogEntryImpl::_indexMultikeyPathsMutex");
-
-    // Non-empty only if '_indexTracksPathLevelMultikeyInfo' is true.
-    //
-    // If non-empty, '_indexMultikeyPaths*' is a vector with size equal to the number of elements
-    // in the index key pattern. Each element in the vector is an ordered set of positions (starting
-    // at 0) into the corresponding indexed field that represent what prefixes of the indexed field
-    // causes the index to be multikey.
-    mutable MultikeyPaths
-        _indexMultikeyPathsForRead;  // May include paths not committed to catalog.
-
     // The earliest snapshot that is allowed to read this index.
     boost::optional<Timestamp> _minVisibleSnapshot;
+
+    // Offset of this index within the Collection metadata.
+    // Used to improve lookups without having to search for the index name
+    // accessing the collection metadata.
+    int _indexOffset;
 };
 }  // namespace mongo
