@@ -46,13 +46,14 @@
 #include "mongo/db/catalog/uncommitted_collections.h"
 #include "mongo/db/client.h"
 #include "mongo/db/clientcursor.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/fts/fts_spec.h"
 #include "mongo/db/index/index_access_method.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/index/s2_access_method.h"
+#include "mongo/db/index/s2_bucket_access_method.h"
 #include "mongo/db/index_names.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/keypattern.h"
@@ -64,6 +65,7 @@
 #include "mongo/db/query/collection_index_usage_tracker_decoration.h"
 #include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/query/internal_plans.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl_set_member_in_standalone_mode.h"
 #include "mongo/db/server_options.h"
@@ -72,6 +74,7 @@
 #include "mongo/db/storage/execution_context.h"
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/storage_engine_init.h"
+#include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/storage/storage_util.h"
 #include "mongo/db/ttl_collection_cache.h"
 #include "mongo/db/vector_clock.h"
@@ -118,40 +121,13 @@ Status IndexCatalogImpl::init(OperationContext* opCtx, Collection* collection) {
         BSONObj spec = collection->getIndexSpec(indexName).getOwned();
         BSONObj keyPattern = spec.getObjectField("key");
 
-        // TODO SERVER-51871: Delete this block once 5.0 becomes last-lts.
-        if (spec.hasField(IndexDescriptor::kGeoHaystackBucketSize)) {
-            LOGV2_OPTIONS(4670602,
-                          {logv2::LogTag::kStartupWarnings},
-                          "Found an existing geoHaystack index in the catalog. Support for "
-                          "geoHaystack indexes has been removed. Instead create a 2d index. See "
-                          "https://dochub.mongodb.org/core/4.4-deprecate-geoHaystack");
-        }
         auto descriptor = std::make_unique<IndexDescriptor>(_getAccessMethodName(keyPattern), spec);
 
-        // TTL indexes with NaN 'expireAfterSeconds' used to cause problems in multiversion
-        // settings. However, we continue to issue a warning message at startup to draw attention
-        // to the presence of these misconfigured indexes.
-        if (spec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName)) {
-            if (spec[IndexDescriptor::kExpireAfterSecondsFieldName].isNaN()) {
-                LOGV2_OPTIONS(6852200,
-                              {logv2::LogTag::kStartupWarnings},
-                              "Found an existing TTL index with NaN 'expireAfterSeconds' in the "
-                              "catalog.",
-                              "ns"_attr = collection->ns(),
-                              "uuid"_attr = collection->uuid(),
-                              "index"_attr = indexName,
-                              "spec"_attr = spec);
-            }
-        }
-
         // TTL indexes are not compatible with capped collections.
         if (spec.hasField(IndexDescriptor::kExpireAfterSecondsFieldName) &&
             !collection->isCapped()) {
             TTLCollectionCache::get(opCtx->getServiceContext())
-                .registerTTLInfo(
-                    collection->uuid(),
-                    TTLCollectionCache::Info{
-                        indexName, spec[IndexDescriptor::kExpireAfterSecondsFieldName].isNaN()});
+                .registerTTLInfo(collection->uuid(), indexName);
         }
 
         bool ready = collection->isIndexReady(indexName);
@@ -354,15 +330,6 @@ StatusWith<BSONObj> IndexCatalogImpl::prepareSpecForCreate(
 
     auto validatedSpec = swValidatedAndFixed.getValue();
 
-    // TODO SERVER-51871: Delete this block once 5.0 becomes last-lts.
-    if (validatedSpec.hasField(IndexDescriptor::kGeoHaystackBucketSize)) {
-        LOGV2_OPTIONS(4670601,
-                      {logv2::LogTag::kStartupWarnings},
-                      "Support for "
-                      "geoHaystack indexes has been removed. Instead create a 2d index. See "
-                      "https://dochub.mongodb.org/core/4.4-deprecate-geoHaystack");
-    }
-
     // Check whether this is a non-_id index and there are any settings disallowing this server
     // from building non-_id indexes.
     Status status = _isNonIDIndexAndNotAllowedToBuild(opCtx, validatedSpec);
@@ -475,7 +442,7 @@ IndexCatalogEntry* IndexCatalogImpl::createIndexEntry(OperationContext* opCtx,
         engine->getEngine()->getSortedDataInterface(opCtx, collOptions, ident, desc);
 
     std::unique_ptr<IndexAccessMethod> accessMethod =
-        IndexAccessMethod::make(entry.get(), std::move(sdi));
+        IndexAccessMethodFactory::get(opCtx)->make(entry.get(), std::move(sdi));
 
     entry->init(std::move(accessMethod));
 
@@ -548,22 +515,73 @@ namespace {
 
 constexpr int kMaxNumIndexesAllowed = 64;
 
-// While technically recursive, only current possible with 2 levels.
-Status _checkValidFilterExpressions(MatchExpression* expression, int level = 0) {
+/**
+ * Recursive function which confirms whether 'expression' is valid for use in partial indexes.
+ * Recursion is restricted to 'internalPartialFilterExpressionMaxDepth' levels.
+ */
+Status _checkValidFilterExpressions(const MatchExpression* expression,
+                                    bool timeseriesMetricIndexesFeatureFlagEnabled,
+                                    int level = 0) {
     if (!expression)
         return Status::OK();
 
+    const auto kMaxDepth = internalPartialFilterExpressionMaxDepth.load();
+    if (timeseriesMetricIndexesFeatureFlagEnabled && (level + 1) > kMaxDepth) {
+        return Status(ErrorCodes::CannotCreateIndex,
+                      str::stream()
+                          << "partialFilterExpression depth may not exceed " << kMaxDepth);
+    }
+
     switch (expression->matchType()) {
         case MatchExpression::AND:
-            if (level > 0)
-                return Status(ErrorCodes::CannotCreateIndex,
-                              "$and only supported in partialFilterExpression at top level");
+            if (!timeseriesMetricIndexesFeatureFlagEnabled) {
+                if (level > 0)
+                    return Status(ErrorCodes::CannotCreateIndex,
+                                  "$and only supported in partialFilterExpression at top level");
+            }
             for (size_t i = 0; i < expression->numChildren(); i++) {
-                Status status = _checkValidFilterExpressions(expression->getChild(i), level + 1);
+                Status status = _checkValidFilterExpressions(
+                    expression->getChild(i), timeseriesMetricIndexesFeatureFlagEnabled, level + 1);
                 if (!status.isOK())
                     return status;
             }
             return Status::OK();
+
+        case MatchExpression::OR:
+            if (!timeseriesMetricIndexesFeatureFlagEnabled) {
+                return Status(ErrorCodes::CannotCreateIndex,
+                              str::stream() << "Expression not supported in partial index: "
+                                            << expression->debugString());
+            }
+            for (size_t i = 0; i < expression->numChildren(); i++) {
+                Status status = _checkValidFilterExpressions(
+                    expression->getChild(i), timeseriesMetricIndexesFeatureFlagEnabled, level + 1);
+                if (!status.isOK()) {
+                    return status;
+                }
+            }
+            return Status::OK();
+        case MatchExpression::GEO:
+            if (timeseriesMetricIndexesFeatureFlagEnabled) {
+                return Status::OK();
+            }
+            return Status(ErrorCodes::CannotCreateIndex,
+                          str::stream() << "Expression not supported in partial index: "
+                                        << expression->debugString());
+        case MatchExpression::INTERNAL_BUCKET_GEO_WITHIN:
+            if (timeseriesMetricIndexesFeatureFlagEnabled) {
+                return Status::OK();
+            }
+            return Status(ErrorCodes::CannotCreateIndex,
+                          str::stream() << "Expression not supported in partial index: "
+                                        << expression->debugString());
+        case MatchExpression::MATCH_IN:
+            if (timeseriesMetricIndexesFeatureFlagEnabled) {
+                return Status::OK();
+            }
+            return Status(ErrorCodes::CannotCreateIndex,
+                          str::stream() << "Expression not supported in partial index: "
+                                        << expression->debugString());
         case MatchExpression::EQ:
         case MatchExpression::LT:
         case MatchExpression::LTE:
@@ -574,7 +592,7 @@ Status _checkValidFilterExpressions(MatchExpression* expression, int level = 0)
             return Status::OK();
         default:
             return Status(ErrorCodes::CannotCreateIndex,
-                          str::stream() << "unsupported expression in partial index: "
+                          str::stream() << "Expression not supported in partial index: "
                                         << expression->debugString());
     }
 }
@@ -597,11 +615,20 @@ StatusWith<BSONObj> adjustIndexSpecObject(const BSONObj& obj) {
         return S2AccessMethod::fixSpec(obj);
     }
 
+    if (IndexNames::GEO_2DSPHERE_BUCKET == pluginName) {
+        return S2BucketAccessMethod::fixSpec(obj);
+    }
+
     return obj;
 }
 
 }  // namespace
 
+Status IndexCatalogImpl::checkValidFilterExpressions(
+    const MatchExpression* expression, bool timeseriesMetricIndexesFeatureFlagEnabled) {
+    return _checkValidFilterExpressions(expression, timeseriesMetricIndexesFeatureFlagEnabled);
+}
+
 Status IndexCatalogImpl::_isSpecOk(OperationContext* opCtx,
                                    const CollectionPtr& collection,
                                    const BSONObj& spec) const {
@@ -757,7 +784,10 @@ Status IndexCatalogImpl::_isSpecOk(OperationContext* opCtx,
         }
         const std::unique_ptr<MatchExpression> filterExpr = std::move(statusWithMatcher.getValue());
 
-        Status status = _checkValidFilterExpressions(filterExpr.get());
+        Status status =
+            _checkValidFilterExpressions(filterExpr.get(),
+                                         feature_flags::gTimeseriesMetricIndexes.isEnabled(
+                                             serverGlobalParams.featureCompatibility));
         if (!status.isOK()) {
             return status;
         }
@@ -1067,11 +1097,9 @@ namespace {
 class IndexRemoveChange final : public RecoveryUnit::Change {
 public:
     IndexRemoveChange(IndexCatalogEntryContainer* entries,
-                      const NamespaceString& nss,
                       std::shared_ptr<IndexCatalogEntry> entry,
                       SharedCollectionDecorations* collectionDecorations)
         : _entries(entries),
-          _nss(nss),
           _entry(std::move(entry)),
           _collectionDecorations(collectionDecorations) {}
 
@@ -1085,14 +1113,11 @@ public:
         // Refresh the CollectionIndexUsageTrackerDecoration's knowledge of what indices are
         // present as it is shared state across Collection copies.
         CollectionIndexUsageTrackerDecoration::get(_collectionDecorations)
-            .registerIndex(indexDescriptor->indexName(),
-                           indexDescriptor->keyPattern(),
-                           IndexFeatures::make(indexDescriptor, _nss.isOnInternalDb()));
+            .registerIndex(indexDescriptor->indexName(), indexDescriptor->keyPattern());
     }
 
 private:
     IndexCatalogEntryContainer* _entries;
-    const NamespaceString _nss;
     std::shared_ptr<IndexCatalogEntry> _entry;
     SharedCollectionDecorations* _collectionDecorations;
 };
@@ -1111,19 +1136,13 @@ Status IndexCatalogImpl::dropIndexEntry(OperationContext* opCtx,
     auto released = _readyIndexes.release(entry->descriptor());
     if (released) {
         invariant(released.get() == entry);
-        opCtx->recoveryUnit()->registerChange(
-            std::make_unique<IndexRemoveChange>(&_readyIndexes,
-                                                collection->ns(),
-                                                std::move(released),
-                                                collection->getSharedDecorations()));
+        opCtx->recoveryUnit()->registerChange(std::make_unique<IndexRemoveChange>(
+            &_readyIndexes, std::move(released), collection->getSharedDecorations()));
     } else {
         released = _buildingIndexes.release(entry->descriptor());
         invariant(released.get() == entry);
-        opCtx->recoveryUnit()->registerChange(
-            std::make_unique<IndexRemoveChange>(&_buildingIndexes,
-                                                collection->ns(),
-                                                std::move(released),
-                                                collection->getSharedDecorations()));
+        opCtx->recoveryUnit()->registerChange(std::make_unique<IndexRemoveChange>(
+            &_buildingIndexes, std::move(released), collection->getSharedDecorations()));
     }
 
     CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, collection);
@@ -1239,6 +1258,7 @@ void IndexCatalogImpl::findIndexesByKeyPattern(OperationContext* opCtx,
 }
 
 const IndexDescriptor* IndexCatalogImpl::findShardKeyPrefixedIndex(OperationContext* opCtx,
+                                                                   const CollectionPtr& collection,
                                                                    const BSONObj& shardKey,
                                                                    bool requireSingleKey) const {
     const IndexDescriptor* best = nullptr;
@@ -1255,7 +1275,7 @@ const IndexDescriptor* IndexCatalogImpl::findShardKeyPrefixedIndex(OperationCont
         if (!shardKey.isPrefixOf(desc->keyPattern(), SimpleBSONElementComparator::kInstance))
             continue;
 
-        if (!entry->isMultikey() && hasSimpleCollation)
+        if (!entry->isMultikey(opCtx, collection) && hasSimpleCollation)
             return desc;
 
         if (!requireSingleKey && hasSimpleCollation)
@@ -1308,7 +1328,7 @@ const IndexDescriptor* IndexCatalogImpl::refreshEntry(OperationContext* opCtx,
     auto oldEntry = _readyIndexes.release(oldDesc);
     invariant(oldEntry);
     opCtx->recoveryUnit()->registerChange(std::make_unique<IndexRemoveChange>(
-        &_readyIndexes, collection->ns(), std::move(oldEntry), collection->getSharedDecorations()));
+        &_readyIndexes, std::move(oldEntry), collection->getSharedDecorations()));
     CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
         .unregisterIndex(indexName);
 
@@ -1324,9 +1344,7 @@ const IndexDescriptor* IndexCatalogImpl::refreshEntry(OperationContext* opCtx,
     invariant(newEntry->isReady(opCtx, collection));
     auto desc = newEntry->descriptor();
     CollectionIndexUsageTrackerDecoration::get(collection->getSharedDecorations())
-        .registerIndex(desc->indexName(),
-                       desc->keyPattern(),
-                       IndexFeatures::make(desc, collection->ns().isOnInternalDb()));
+        .registerIndex(desc->indexName(), desc->keyPattern());
 
     // Last rebuild index data for CollectionQueryInfo for this Collection.
     CollectionQueryInfo::get(collection).rebuildIndexData(opCtx, collection);
@@ -1377,16 +1395,6 @@ Status IndexCatalogImpl::_indexKeys(OperationContext* opCtx,
             *keysInsertedOut += inserted;
         }
     } else {
-        // Ensure that our snapshot is compatible with the index's minimum visibile snapshot.
-        const auto minVisibleTimestamp = index->getMinimumVisibleSnapshot();
-        const auto readTimestamp =
-            opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx).value_or(
-                opCtx->recoveryUnit()->getCatalogConflictingTimestamp());
-        if (minVisibleTimestamp && !readTimestamp.isNull() &&
-            readTimestamp < *minVisibleTimestamp) {
-            throw WriteConflictException();
-        }
-
         int64_t numInserted = 0;
         status = index->accessMethod()->insertKeysAndUpdateMultikeyPaths(
             opCtx,
@@ -1430,7 +1438,9 @@ Status IndexCatalogImpl::_indexFilteredRecords(OperationContext* opCtx,
         auto multikeyMetadataKeys = executionCtx.multikeyMetadataKeys();
         auto multikeyPaths = executionCtx.multikeyPaths();
 
-        index->accessMethod()->getKeys(pooledBuilder,
+        index->accessMethod()->getKeys(opCtx,
+                                       coll,
+                                       pooledBuilder,
                                        *bsonRecord.docPtr,
                                        options.getKeysMode,
                                        IndexAccessMethod::GetKeysContext::kAddingKeys,
@@ -1495,7 +1505,7 @@ Status IndexCatalogImpl::_updateRecord(OperationContext* const opCtx,
 
     UpdateTicket updateTicket;
 
-    iam->prepareUpdate(opCtx, index, oldDoc, newDoc, recordId, options, &updateTicket);
+    iam->prepareUpdate(opCtx, coll, index, oldDoc, newDoc, recordId, options, &updateTicket);
 
     int64_t keysInserted = 0;
     int64_t keysDeleted = 0;
@@ -1571,14 +1581,6 @@ void IndexCatalogImpl::_unindexKeys(OperationContext* opCtx,
     // duplicates. See SERVER-17487 for more details.
     options.dupsAllowed = options.dupsAllowed || !index->isReady(opCtx, collection);
 
-    // Ensure that our snapshot is compatible with the index's minimum visibile snapshot.
-    const auto minVisibleTimestamp = index->getMinimumVisibleSnapshot();
-    const auto readTimestamp = opCtx->recoveryUnit()->getPointInTimeReadTimestamp(opCtx).value_or(
-        opCtx->recoveryUnit()->getCatalogConflictingTimestamp());
-    if (minVisibleTimestamp && !readTimestamp.isNull() && readTimestamp < *minVisibleTimestamp) {
-        throw WriteConflictException();
-    }
-
     int64_t removed = 0;
     Status status = index->accessMethod()->removeKeys(opCtx, keys, loc, options, &removed);
 
@@ -1610,7 +1612,9 @@ void IndexCatalogImpl::_unindexRecord(OperationContext* opCtx,
     // multikey when removing a document since the index metadata isn't updated when keys are
     // deleted.
     auto keys = executionCtx.keys();
-    entry->accessMethod()->getKeys(pooledBuilder,
+    entry->accessMethod()->getKeys(opCtx,
+                                   collection,
+                                   pooledBuilder,
                                    obj,
                                    IndexAccessMethod::GetKeysMode::kRelaxConstraintsUnfiltered,
                                    IndexAccessMethod::GetKeysContext::kRemovingKeys,
@@ -1789,11 +1793,7 @@ void IndexCatalogImpl::indexBuildSuccess(OperationContext* opCtx,
     invariant(releasedEntry.get() == index);
     _readyIndexes.add(std::move(releasedEntry));
 
-    // Wait to unset the interceptor until the index actually commits. If a write conflict is
-    // encountered and the index commit process is restated, the multikey information from the
-    // interceptor may still be needed.
-    opCtx->recoveryUnit()->onCommit(
-        [index](boost::optional<Timestamp>) { index->setIndexBuildInterceptor(nullptr); });
+    index->setIndexBuildInterceptor(nullptr);
     index->setIsReady(true);
 }
 
