@@ -68,9 +68,34 @@ namespace {
 // specification.
 MONGO_FAIL_POINT_DEFINE(skipIndexCreateFieldNameValidation);
 
-// When the skipTTLIndexNaNExpireAfterSecondsValidation failpoint is enabled, validation for
-// TTL index 'expireAfterSeconds' will be disabled.
-MONGO_FAIL_POINT_DEFINE(skipTTLIndexNaNExpireAfterSecondsValidation);
+static std::set<StringData> allowedFieldNames = {
+    IndexDescriptor::k2dIndexBitsFieldName,
+    IndexDescriptor::k2dIndexMaxFieldName,
+    IndexDescriptor::k2dIndexMinFieldName,
+    IndexDescriptor::k2dsphereCoarsestIndexedLevel,
+    IndexDescriptor::k2dsphereFinestIndexedLevel,
+    IndexDescriptor::k2dsphereVersionFieldName,
+    IndexDescriptor::kBackgroundFieldName,
+    IndexDescriptor::kCollationFieldName,
+    IndexDescriptor::kDefaultLanguageFieldName,
+    IndexDescriptor::kDropDuplicatesFieldName,
+    IndexDescriptor::kExpireAfterSecondsFieldName,
+    IndexDescriptor::kHiddenFieldName,
+    IndexDescriptor::kIndexNameFieldName,
+    IndexDescriptor::kIndexVersionFieldName,
+    IndexDescriptor::kKeyPatternFieldName,
+    IndexDescriptor::kLanguageOverrideFieldName,
+    IndexDescriptor::kNamespaceFieldName,
+    IndexDescriptor::kPartialFilterExprFieldName,
+    IndexDescriptor::kPathProjectionFieldName,
+    IndexDescriptor::kSparseFieldName,
+    IndexDescriptor::kStorageEngineFieldName,
+    IndexDescriptor::kTextVersionFieldName,
+    IndexDescriptor::kUniqueFieldName,
+    IndexDescriptor::kWeightsFieldName,
+    IndexDescriptor::kOriginalSpecFieldName,
+    // Index creation under legacy writeMode can result in an index spec with an _id field.
+    "_id"};
 
 static const std::set<StringData> allowedIdIndexFieldNames = {
     IndexDescriptor::kCollationFieldName,
@@ -95,27 +120,6 @@ Status isIndexVersionAllowedForCreation(IndexVersion indexVersion, const BSONObj
             str::stream() << "Invalid index specification " << indexSpec
                           << "; cannot create an index with v=" << static_cast<int>(indexVersion)};
 }
-
-BSONObj buildRepairedIndexSpec(
-    const NamespaceString& ns,
-    const BSONObj& indexSpec,
-    const std::set<StringData>& allowedFieldNames,
-    std::function<void(const BSONElement&, BSONObjBuilder*)> indexSpecHandleFn) {
-    BSONObjBuilder builder;
-    for (const auto& indexSpecElem : indexSpec) {
-        StringData fieldName = indexSpecElem.fieldNameStringData();
-        if (allowedFieldNames.count(fieldName)) {
-            indexSpecHandleFn(indexSpecElem, &builder);
-        } else {
-            LOGV2_WARNING(23878,
-                          "Removing unknown field from index spec",
-                          "namespace"_attr = redact(ns.toString()),
-                          "fieldName"_attr = redact(fieldName),
-                          "indexSpec"_attr = redact(indexSpec));
-        }
-    }
-    return builder.obj();
-}
 }  // namespace
 
 Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion indexVersion) {
@@ -129,6 +133,10 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
 
     auto pluginName = IndexNames::findPluginName(key);
     if (pluginName.size()) {
+        if (pluginName == IndexNames::GEO_HAYSTACK)
+            return {
+                ErrorCodes::CannotCreateIndex,
+                str::stream() << "GeoHaystack indexes cannot be created in version 5.0 and above"};
         if (!IndexNames::isKnownName(pluginName))
             return Status(code, str::stream() << "Unknown index plugin '" << pluginName << '\'');
     }
@@ -246,55 +254,31 @@ Status validateKeyPattern(const BSONObj& key, IndexDescriptor::IndexVersion inde
     return Status::OK();
 }
 
-BSONObj removeUnknownFields(const NamespaceString& ns, const BSONObj& indexSpec) {
-    auto appendIndexSpecFn = [](const BSONElement& indexSpecElem, BSONObjBuilder* builder) {
-        builder->append(indexSpecElem);
-    };
-    return buildRepairedIndexSpec(ns, indexSpec, allowedFieldNames, appendIndexSpecFn);
-}
-
-BSONObj repairIndexSpec(const NamespaceString& ns,
-                        const BSONObj& indexSpec,
-                        const std::set<StringData>& allowedFieldNames) {
-    auto fixIndexSpecFn = [&indexSpec, &ns](const BSONElement& indexSpecElem,
-                                            BSONObjBuilder* builder) {
+BSONObj removeUnknownFields(const BSONObj& indexSpec) {
+    BSONObjBuilder builder;
+    for (const auto& indexSpecElem : indexSpec) {
         StringData fieldName = indexSpecElem.fieldNameStringData();
-        if ((IndexDescriptor::kBackgroundFieldName == fieldName ||
-             IndexDescriptor::kUniqueFieldName == fieldName ||
-             IndexDescriptor::kSparseFieldName == fieldName ||
-             IndexDescriptor::kDropDuplicatesFieldName == fieldName) &&
-            !indexSpecElem.isNumber() && !indexSpecElem.isBoolean() && indexSpecElem.trueValue()) {
-            LOGV2_WARNING(6444400,
-                          "Fixing boolean field from index spec",
-                          "namespace"_attr = redact(ns.toString()),
-                          "fieldName"_attr = redact(fieldName),
-                          "indexSpec"_attr = redact(indexSpec));
-            builder->appendBool(fieldName, true);
-        } else if (IndexDescriptor::kExpireAfterSecondsFieldName == fieldName &&
-                   !(indexSpecElem.isNumber() && !indexSpecElem.isNaN())) {
-            LOGV2_WARNING(6835900,
-                          "Fixing expire field from TTL index spec",
-                          "namespace"_attr = redact(ns.toString()),
-                          "fieldName"_attr = redact(fieldName),
-                          "indexSpec"_attr = redact(indexSpec));
-            builder->appendNumber(fieldName,
-                                  durationCount<Seconds>(kExpireAfterSecondsForInactiveTTLIndex));
+        if (allowedFieldNames.count(fieldName)) {
+            builder.append(indexSpecElem);
         } else {
-            builder->append(indexSpecElem);
+            LOGV2_WARNING(23878,
+                          "Removing field '{fieldName}' from index spec: {indexSpec}",
+                          "Removing unknown field from index spec",
+                          "fieldName"_attr = redact(fieldName),
+                          "indexSpec"_attr = redact(indexSpec));
         }
-    };
-
-    return buildRepairedIndexSpec(ns, indexSpec, allowedFieldNames, fixIndexSpecFn);
+    }
+    return builder.obj();
 }
 
 StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& indexSpec) {
     bool hasKeyPatternField = false;
     bool hasIndexNameField = false;
     bool hasNamespaceField = false;
-    bool isTTLIndexWithNaNExpireAfterSeconds = false;
     bool hasVersionField = false;
     bool hasCollationField = false;
     bool hasWeightsField = false;
+    bool hasOriginalSpecField = false;
     bool apiStrict = opCtx && APIParameters::get(opCtx).getAPIStrict().value_or(false);
 
     auto fieldNamesValidStatus = validateIndexSpecFieldNames(indexSpec);
@@ -314,30 +298,20 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                             << "The field '" << IndexDescriptor::kKeyPatternFieldName
                             << "' must be an object, but got " << typeName(indexSpecElem.type())};
             }
-
+            auto keyPattern = indexSpecElem.Obj();
             std::vector<StringData> keys;
-            for (auto&& keyElem : indexSpecElem.Obj()) {
+            for (auto&& keyElem : keyPattern) {
                 auto keyElemFieldName = keyElem.fieldNameStringData();
                 if (std::find(keys.begin(), keys.end(), keyElemFieldName) != keys.end()) {
                     return {ErrorCodes::BadValue,
                             str::stream() << "The field '" << keyElemFieldName
                                           << "' appears multiple times in the index key pattern "
-                                          << indexSpecElem.Obj()};
+                                          << keyPattern};
                 }
                 keys.push_back(keyElemFieldName);
             }
 
-            // TODO SERVER-51871: When 5.0 becomes last-lts, this check should be moved into
-            // 'validateKeyPattern()'. It must currently be done here so that haystack indexes
-            // continue to replicate correctly before the upgrade to FCV "4.9" is complete.
-            const auto keyPattern = indexSpecElem.Obj();
             indexType = IndexNames::findPluginName(keyPattern);
-            if (indexType == IndexNames::GEO_HAYSTACK) {
-                return {ErrorCodes::CannotCreateIndex,
-                        str::stream()
-                            << "GeoHaystack indexes cannot be created in version 4.9 and above"};
-            }
-
             if (apiStrict && indexType == IndexNames::TEXT) {
                 return {ErrorCodes::APIStrictError,
                         str::stream()
@@ -352,7 +326,7 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                 return keyPatternValidateStatus;
             }
 
-            for (const auto& keyElement : indexSpecElem.Obj()) {
+            for (const auto& keyElement : keyPattern) {
                 if (keyElement.type() == String && keyElement.str().empty()) {
                     return {ErrorCodes::CannotCreateIndex,
                             str::stream()
@@ -406,6 +380,21 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
 
             hasVersionField = true;
             resolvedIndexVersion = requestedIndexVersion;
+        } else if (IndexDescriptor::kOriginalSpecFieldName == indexSpecElemFieldName) {
+            if (indexSpecElem.type() != BSONType::Object) {
+                return {ErrorCodes::TypeMismatch,
+                        str::stream()
+                            << "The field '" << IndexDescriptor::kOriginalSpecFieldName
+                            << "' must be an object, but got " << typeName(indexSpecElem.type())};
+            }
+
+            if (indexSpecElem.Obj().isEmpty()) {
+                return {ErrorCodes::BadValue,
+                        str::stream() << "The field '" << IndexDescriptor::kOriginalSpecFieldName
+                                      << "' cannot be an empty object."};
+            }
+
+            hasOriginalSpecField = true;
         } else if (IndexDescriptor::kCollationFieldName == indexSpecElemFieldName) {
             if (indexSpecElem.type() != BSONType::Object) {
                 return {ErrorCodes::TypeMismatch,
@@ -493,11 +482,6 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                             << "' must be an object, but got " << typeName(indexSpecElem.type())};
             }
             hasWeightsField = true;
-        } else if (IndexDescriptor::kGeoHaystackBucketSize == indexSpecElemFieldName) {
-            return {ErrorCodes::CannotCreateIndex,
-                    str::stream()
-                        << "The 'bucketSize' parameter is disallowed because "
-                           "geoHaystack indexes are no longer supported in version 4.9 and above"};
         } else if ((IndexDescriptor::kBackgroundFieldName == indexSpecElemFieldName ||
                     IndexDescriptor::kUniqueFieldName == indexSpecElemFieldName ||
                     IndexDescriptor::kSparseFieldName == indexSpecElemFieldName ||
@@ -526,8 +510,6 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
                     str::stream() << "The field '" << indexSpecElemFieldName
                                   << "' must be a number, but got "
                                   << typeName(indexSpecElem.type())};
-        } else if (IndexDescriptor::kExpireAfterSecondsFieldName == indexSpecElemFieldName) {
-            isTTLIndexWithNaNExpireAfterSeconds = indexSpecElem.isNaN();
         } else {
             // We can assume field name is valid at this point. Validation of fieldname is handled
             // prior to this in validateIndexSpecFieldNames().
@@ -576,19 +558,6 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
         modifiedSpec = modifiedSpec.removeField(IndexDescriptor::kNamespaceFieldName);
     }
 
-    if (isTTLIndexWithNaNExpireAfterSeconds &&
-        !skipTTLIndexNaNExpireAfterSecondsValidation.shouldFail()) {
-        // We create a new index specification with the 'expireAfterSeconds' field set as
-        // kExpireAfterSecondsForInactiveTTLIndex if the current value is NaN. A similar
-        // treatment is done in repairIndexSpec(). This rewrites the 'expireAfterSeconds'
-        // value to be compliant with the 'safeInt' IDL type for the listIndexes response.
-        BSONObjBuilder builder;
-        builder.appendNumber(IndexDescriptor::kExpireAfterSecondsFieldName,
-                             durationCount<Seconds>(kExpireAfterSecondsForInactiveTTLIndex));
-        auto obj = builder.obj();
-        modifiedSpec = modifiedSpec.addField(obj.firstElement());
-    }
-
     if (!hasVersionField) {
         // We create a new index specification with the 'v' field set as 'defaultIndexVersion' if
         // the field was omitted.
@@ -597,6 +566,18 @@ StatusWith<BSONObj> validateIndexSpec(OperationContext* opCtx, const BSONObj& in
         modifiedSpec = modifiedSpec.addField(versionObj.firstElement());
     }
 
+    if (hasOriginalSpecField) {
+        StatusWith<BSONObj> modifiedOriginalSpec = validateIndexSpec(
+            opCtx, indexSpec.getObjectField(IndexDescriptor::kOriginalSpecFieldName));
+        if (!modifiedOriginalSpec.isOK()) {
+            return modifiedOriginalSpec.getStatus();
+        }
+
+        BSONObj specToAdd =
+            BSON(IndexDescriptor::kOriginalSpecFieldName << modifiedOriginalSpec.getValue());
+        modifiedSpec = modifiedSpec.addField(specToAdd.firstElement());
+    }
+
     return modifiedSpec;
 }
 
@@ -709,8 +690,7 @@ StatusWith<BSONObj> validateIndexSpecCollation(OperationContext* opCtx,
     return indexSpec;
 }
 
-Status validateExpireAfterSeconds(std::int64_t expireAfterSeconds,
-                                  ValidateExpireAfterSecondsMode mode) {
+Status validateExpireAfterSeconds(std::int64_t expireAfterSeconds) {
     if (expireAfterSeconds < 0) {
         return {ErrorCodes::InvalidOptions,
                 str::stream() << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
@@ -721,31 +701,16 @@ Status validateExpireAfterSeconds(std::int64_t expireAfterSeconds,
         << "TTL index '" << IndexDescriptor::kExpireAfterSecondsFieldName
         << "' option must be within an acceptable range, try a lower number";
 
-    if (mode == ValidateExpireAfterSecondsMode::kSecondaryTTLIndex) {
-        // Relax epoch restriction on TTL indexes. This allows us to export and import existing
-        // TTL indexes with large values or NaN for the 'expireAfterSeconds' field.
-        // Additionally, the 'expireAfterSeconds' for TTL indexes is defined as safeInt (int32_t)
-        // in the IDL for listIndexes and collMod. See list_indexes.idl and coll_mod.idl.
-        if (expireAfterSeconds > std::numeric_limits<std::int32_t>::max()) {
-            return {ErrorCodes::InvalidOptions, tooLargeErr};
-        }
-    } else {
-        // Clustered collections with TTL.
-        // Note that 'expireAfterSeconds' is defined as safeInt64 in the IDL for the create and
-        // collMod commands. See create.idl and coll_mod.idl.
-        // There are two cases where we can encounter an issue here.
-        // The first case is when we try to cast to millseconds from seconds, which could cause an
-        // overflow. The second case is where 'expireAfterSeconds' is larger than the current epoch
-        // time. This isn't necessarily problematic for the general case, but for the specific case
-        // of time series collections, we cluster the collection by an OID value, where the
-        // timestamp portion is only a 32-bit unsigned integer offset of seconds since the epoch.
-        if (expireAfterSeconds > std::numeric_limits<std::int64_t>::max() / 1000) {
-            return {ErrorCodes::InvalidOptions, tooLargeErr};
-        }
-        auto expireAfterMillis = duration_cast<Milliseconds>(Seconds(expireAfterSeconds));
-        if (expireAfterMillis > Date_t::now().toDurationSinceEpoch()) {
-            return {ErrorCodes::InvalidOptions, tooLargeErr};
-        }
+    // There are two cases where we can encounter an issue here.
+    // The first case is when we try to cast to millseconds from seconds, which could cause an
+    // overflow. The second case is where 'expireAfterSeconds' is larger than the current epoch
+    // time.
+    if (expireAfterSeconds > std::numeric_limits<std::int64_t>::max() / 1000) {
+        return {ErrorCodes::InvalidOptions, tooLargeErr};
+    }
+    auto expireAfterMillis = duration_cast<Milliseconds>(Seconds(expireAfterSeconds));
+    if (expireAfterMillis > Date_t::now().toDurationSinceEpoch()) {
+        return {ErrorCodes::InvalidOptions, tooLargeErr};
     }
     return Status::OK();
 }
@@ -765,12 +730,10 @@ Status validateIndexSpecTTL(const BSONObj& indexSpec) {
                               << "'. Index spec: " << indexSpec};
     }
 
-    if (auto status =
-            validateExpireAfterSeconds(expireAfterSecondsElt.safeNumberLong(),
-                                       ValidateExpireAfterSecondsMode::kSecondaryTTLIndex);
+    if (auto status = validateExpireAfterSeconds(expireAfterSecondsElt.safeNumberLong());
         !status.isOK()) {
         return {ErrorCodes::CannotCreateIndex,
-                str::stream() << status.reason() << "index spec: " << indexSpec};
+                str::stream() << status.reason() << ". Index spec: " << indexSpec};
     }
 
     const BSONObj key = indexSpec["key"].Obj();
