@@ -42,7 +42,7 @@
 #include "mongo/db/catalog/list_indexes.h"
 #include "mongo/db/catalog/local_oplog_info.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_descriptor.h"
@@ -106,7 +106,7 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
     auto catalog = CollectionCatalog::get(opCtx);
     const auto sourceColl = catalog->lookupCollectionByNamespace(opCtx, source);
     if (!sourceColl) {
-        if (ViewCatalog::get(db)->lookup(opCtx, source.ns()))
+        if (ViewCatalog::get(db)->lookup(opCtx, source))
             return Status(ErrorCodes::CommandNotSupportedOnView,
                           str::stream() << "cannot rename view: " << source);
         return Status(ErrorCodes::NamespaceNotFound,
@@ -118,7 +118,7 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
     const auto targetColl = catalog->lookupCollectionByNamespace(opCtx, target);
 
     if (!targetColl) {
-        if (ViewCatalog::get(db)->lookup(opCtx, target.ns()))
+        if (ViewCatalog::get(db)->lookup(opCtx, target))
             return Status(ErrorCodes::NamespaceExists,
                           str::stream() << "a view already exists with that name: " << target);
     } else {
@@ -496,7 +496,7 @@ Status renameBetweenDBs(OperationContext* opCtx,
     auto catalog = CollectionCatalog::get(opCtx);
     const auto sourceColl = catalog->lookupCollectionByNamespace(opCtx, source);
     if (!sourceColl) {
-        if (sourceDB && ViewCatalog::get(sourceDB)->lookup(opCtx, source.ns()))
+        if (sourceDB && ViewCatalog::get(sourceDB)->lookup(opCtx, source))
             return Status(ErrorCodes::CommandNotSupportedOnView,
                           str::stream() << "cannot rename view: " << source);
         return Status(ErrorCodes::NamespaceNotFound, "source namespace does not exist");
@@ -525,7 +525,7 @@ Status renameBetweenDBs(OperationContext* opCtx,
             return Status(ErrorCodes::NamespaceExists, "target namespace exists");
         }
 
-    } else if (targetDB && ViewCatalog::get(targetDB)->lookup(opCtx, target.ns())) {
+    } else if (targetDB && ViewCatalog::get(targetDB)->lookup(opCtx, target)) {
         return Status(ErrorCodes::NamespaceExists,
                       str::stream() << "a view already exists with that name: " << target);
     }
@@ -571,7 +571,7 @@ Status renameBetweenDBs(OperationContext* opCtx,
     }
 
     // Dismissed on success
-    auto tmpCollectionDropper = makeGuard([&] {
+    ScopeGuard tmpCollectionDropper([&] {
         Status status = Status::OK();
         try {
             status = dropCollectionForApplyOps(
