@@ -45,7 +45,7 @@
 #include "mongo/db/catalog/uncommitted_collections.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/create_indexes_gen.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/index/index_descriptor.h"
@@ -109,7 +109,7 @@ std::vector<BSONObj> parseAndValidateIndexSpecs(OperationContext* opCtx,
     for (const auto& index : cmd.getIndexes()) {
         BSONObj parsedIndexSpec = index;
         if (ignoreUnknownIndexOptions) {
-            parsedIndexSpec = index_key_validate::removeUnknownFields(ns, parsedIndexSpec);
+            parsedIndexSpec = index_key_validate::removeUnknownFields(parsedIndexSpec);
         }
 
         auto indexSpecStatus = index_key_validate::validateIndexSpec(opCtx, parsedIndexSpec);
@@ -296,7 +296,7 @@ CreateIndexesReply runCreateIndexesOnNewCollection(
     auto db = databaseHolder->getDb(opCtx, ns.db());
     uassert(ErrorCodes::CommandNotSupportedOnView,
             "Cannot create indexes on a view",
-            !db || !ViewCatalog::get(db)->lookup(opCtx, ns.ns()));
+            !db || !ViewCatalog::get(db)->lookup(opCtx, ns));
 
     if (createCollImplicitly) {
         // We need to create the collection.
@@ -338,7 +338,7 @@ CreateIndexesReply runCreateIndexesOnNewCollection(
     uassert(ErrorCodes::OperationNotSupportedInTransaction,
             str::stream() << "Cannot create new indexes on non-empty collection " << ns
                           << " in a multi-document transaction.",
-            collection->numRecords(opCtx) == 0);
+            collection->isEmpty(opCtx));
 
     const int numIndexesBefore =
         IndexBuildsCoordinator::getNumIndexesTotal(opCtx, collection.get());
@@ -472,7 +472,8 @@ CreateIndexesReply runCreateIndexesWithCoordinator(OperationContext* opCtx,
           "namespace"_attr = ns,
           "collectionUUID"_attr = *collectionUUID,
           "indexes"_attr = specs.size(),
-          "firstIndex"_attr = specs[0][IndexDescriptor::kIndexNameFieldName]);
+          "firstIndex"_attr = specs[0][IndexDescriptor::kIndexNameFieldName],
+          "command"_attr = cmd.toBSON({}));
     hangCreateIndexesBeforeStartingIndexBuild.pauseWhileSet(opCtx);
 
     bool shouldContinueInBackground = false;
