@@ -37,6 +37,7 @@
 #include <string>
 
 #include "mongo/bson/util/bson_extract.h"
+#include "mongo/client/read_preference.h"
 #include "mongo/db/api_parameters.h"
 #include "mongo/db/auth/authorization_checks.h"
 #include "mongo/db/auth/authorization_session.h"
@@ -225,6 +226,7 @@ void setUpOperationContextStateForGetMore(OperationContext* opCtx,
                                           bool disableAwaitDataFailpointActive) {
     applyCursorReadConcern(opCtx, cursor.getReadConcernArgs());
     opCtx->setWriteConcern(cursor.getWriteConcernOptions());
+    ReadPreferenceSetting::get(opCtx) = cursor.getReadPreferenceSetting();
 
     auto apiParamsFromClient = APIParameters::get(opCtx);
     uassert(
@@ -239,6 +241,7 @@ void setUpOperationContextStateForGetMore(OperationContext* opCtx,
     // that if the 'getMore' command itself has a 'comment' field, we give precedence to it.
     auto comment = cursor.getOriginatingCommandObj()["comment"];
     if (!opCtx->getComment() && comment) {
+        stdx::lock_guard<Client> lk(*opCtx->getClient());
         opCtx->setComment(comment.wrap());
     }
 }
@@ -355,9 +358,7 @@ public:
                 // to the resume token of the invalidating event, and mark the cursor response as
                 // invalidated. We always expect to have ExtraInfo for this error code.
                 const auto extraInfo = ex.extraInfo<ChangeStreamInvalidationInfo>();
-                tassert(5493700,
-                        "Missing ChangeStreamInvalidationInfo on exception",
-                        extraInfo != nullptr);
+                tassert(5493700, "Missing ChangeStreamInvalidationInfo on exception", extraInfo);
 
                 nextBatch->setPostBatchResumeToken(extraInfo->getInvalidateResumeToken());
                 nextBatch->setInvalidated();
@@ -511,7 +512,7 @@ public:
             }
 
             // On early return, get rid of the cursor.
-            auto cursorFreer = makeGuard([&] { cursorPin.deleteUnderlying(); });
+            ScopeGuard cursorFreer([&] { cursorPin.deleteUnderlying(); });
 
             // If the 'waitAfterPinningCursorBeforeGetMoreBatch' fail point is enabled, set the
             // 'msg' field of this operation's CurOp to signal that we've hit this point and then
