@@ -50,6 +50,8 @@
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/collection_query_info.h"
+#include "mongo/db/query/plan_cache_key_factory.h"
+#include "mongo/db/query/query_settings_decoration.h"
 #include "mongo/logv2/log.h"
 #include "mongo/stdx/unordered_set.h"
 
@@ -64,17 +66,16 @@ using namespace mongo;
  * Retrieves a collection's query settings and plan cache from the database.
  */
 static Status getQuerySettingsAndPlanCache(OperationContext* opCtx,
-                                           Collection* collection,
-                                           const string& ns,
+                                           const CollectionPtr& collection,
                                            QuerySettings** querySettingsOut,
                                            PlanCache** planCacheOut) {
     *querySettingsOut = nullptr;
     *planCacheOut = nullptr;
-    if (nullptr == collection) {
+    if (!collection) {
         return Status(ErrorCodes::BadValue, "no such collection");
     }
 
-    QuerySettings* querySettings = CollectionQueryInfo::get(collection).getQuerySettings();
+    QuerySettings* querySettings = QuerySettingsDecoration::get(collection->getSharedDecorations());
     invariant(querySettings);
 
     *querySettingsOut = querySettings;
@@ -93,13 +94,11 @@ static Status getQuerySettingsAndPlanCache(OperationContext* opCtx,
 // available to the client.
 //
 
-MONGO_INITIALIZER_WITH_PREREQUISITES(SetupIndexFilterCommands, MONGO_NO_PREREQUISITES)
+MONGO_INITIALIZER_WITH_PREREQUISITES(SetupIndexFilterCommands, ())
 (InitializerContext* context) {
     new ListFilters();
     new ClearFilters();
     new SetFilter();
-
-    return Status::OK();
 }
 
 }  // namespace
@@ -164,7 +163,7 @@ Status ListFilters::runIndexFilterCommand(OperationContext* opCtx,
     QuerySettings* querySettings;
     PlanCache* unused;
     Status status =
-        getQuerySettingsAndPlanCache(opCtx, ctx.getCollection(), ns, &querySettings, &unused);
+        getQuerySettingsAndPlanCache(opCtx, ctx.getCollection(), &querySettings, &unused);
     if (!status.isOK()) {
         // No collection - return empty array of filters.
         BSONArrayBuilder hintsBuilder(bob->subarrayStart("filters"));
@@ -232,19 +231,19 @@ Status ClearFilters::runIndexFilterCommand(OperationContext* opCtx,
     QuerySettings* querySettings;
     PlanCache* planCache;
     Status status =
-        getQuerySettingsAndPlanCache(opCtx, ctx.getCollection(), ns, &querySettings, &planCache);
+        getQuerySettingsAndPlanCache(opCtx, ctx.getCollection(), &querySettings, &planCache);
     if (!status.isOK()) {
         // No collection - do nothing.
         return Status::OK();
     }
-    return clear(opCtx, querySettings, planCache, ns, cmdObj);
+    return clear(opCtx, ctx.getCollection(), querySettings, planCache, cmdObj);
 }
 
 // static
 Status ClearFilters::clear(OperationContext* opCtx,
+                           const CollectionPtr& collection,
                            QuerySettings* querySettings,
                            PlanCache* planCache,
-                           const std::string& ns,
                            const BSONObj& cmdObj) {
     invariant(querySettings);
 
@@ -253,7 +252,7 @@ Status ClearFilters::clear(OperationContext* opCtx,
     // - clear hints for single query shape when a query shape is described in the
     //   command arguments.
     if (cmdObj.hasField("query")) {
-        auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, ns, cmdObj);
+        auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, collection->ns().ns(), cmdObj);
         if (!statusWithCQ.isOK()) {
             return statusWithCQ.getStatus();
         }
@@ -262,7 +261,7 @@ Status ClearFilters::clear(OperationContext* opCtx,
         querySettings->removeAllowedIndices(cq->encodeKey());
 
         // Remove entry from plan cache
-        planCache->remove(*cq).transitional_ignore();
+        planCache->remove(plan_cache_key_factory::make<PlanCacheKey>(*cq, collection));
 
         LOGV2(20479,
               "Removed index filter on {query}",
@@ -287,7 +286,7 @@ Status ClearFilters::clear(OperationContext* opCtx,
     // OK to proceed with clearing entire cache.
     querySettings->clearAllowedIndices();
 
-    const NamespaceString nss(ns);
+    const NamespaceString nss(collection->ns());
     const ExtensionsCallbackReal extensionsCallback(opCtx, &nss);
 
     // Remove corresponding entries from plan cache.
@@ -304,15 +303,16 @@ Status ClearFilters::clear(OperationContext* opCtx,
         AllowedIndexEntry entry = *i;
 
         // Create canonical query.
-        auto qr = std::make_unique<QueryRequest>(nss);
-        qr->setFilter(entry.query);
-        qr->setSort(entry.sort);
-        qr->setProj(entry.projection);
-        qr->setCollation(entry.collation);
+        auto findCommand = std::make_unique<FindCommandRequest>(nss);
+        findCommand->setFilter(entry.query);
+        findCommand->setSort(entry.sort);
+        findCommand->setProjection(entry.projection);
+        findCommand->setCollation(entry.collation);
         const boost::intrusive_ptr<ExpressionContext> expCtx;
         auto statusWithCQ =
             CanonicalQuery::canonicalize(opCtx,
-                                         std::move(qr),
+                                         std::move(findCommand),
+                                         false,
                                          expCtx,
                                          extensionsCallback,
                                          MatchExpressionParser::kAllowAllSpecialFeatures);
@@ -320,13 +320,13 @@ Status ClearFilters::clear(OperationContext* opCtx,
         std::unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
 
         // Remove plan cache entry.
-        planCache->remove(*cq).transitional_ignore();
+        planCache->remove(plan_cache_key_factory::make<PlanCacheKey>(*cq, collection));
     }
 
     LOGV2(20480,
           "Removed all index filters for collection: {namespace}",
           "Removed all index filters for collection",
-          "namespace"_attr = ns);
+          "namespace"_attr = collection->ns().ns());
 
     return Status::OK();
 }
@@ -346,18 +346,18 @@ Status SetFilter::runIndexFilterCommand(OperationContext* opCtx,
     QuerySettings* querySettings;
     PlanCache* planCache;
     Status status =
-        getQuerySettingsAndPlanCache(opCtx, ctx.getCollection(), ns, &querySettings, &planCache);
+        getQuerySettingsAndPlanCache(opCtx, ctx.getCollection(), &querySettings, &planCache);
     if (!status.isOK()) {
         return status;
     }
-    return set(opCtx, querySettings, planCache, ns, cmdObj);
+    return set(opCtx, ctx.getCollection(), querySettings, planCache, cmdObj);
 }
 
 // static
 Status SetFilter::set(OperationContext* opCtx,
+                      const CollectionPtr& collection,
                       QuerySettings* querySettings,
                       PlanCache* planCache,
-                      const string& ns,
                       const BSONObj& cmdObj) {
     // indexes - required
     BSONElement indexesElt = cmdObj.getField("indexes");
@@ -391,7 +391,7 @@ Status SetFilter::set(OperationContext* opCtx,
         }
     }
 
-    auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, ns, cmdObj);
+    auto statusWithCQ = plan_cache_commands::canonicalize(opCtx, collection->ns().ns(), cmdObj);
     if (!statusWithCQ.isOK()) {
         return statusWithCQ.getStatus();
     }
@@ -401,7 +401,7 @@ Status SetFilter::set(OperationContext* opCtx,
     querySettings->setAllowedIndices(*cq, indexes, indexNames);
 
     // Remove entry from plan cache.
-    planCache->remove(*cq).transitional_ignore();
+    planCache->remove(plan_cache_key_factory::make<PlanCacheKey>(*cq, collection));
 
     LOGV2(20481,
           "Index filter set on {query} {indexes}",
