@@ -166,7 +166,9 @@ BSONObj buildViewBson(const ViewDefinition& view, bool nameOnly) {
     return b.obj();
 }
 
-BSONObj buildTimeseriesBson(const CollectionPtr& collection, bool nameOnly) {
+BSONObj buildTimeseriesBson(OperationContext* opCtx,
+                            const CollectionPtr& collection,
+                            bool nameOnly) {
     invariant(collection);
 
     BSONObjBuilder builder;
@@ -368,13 +370,14 @@ public:
                                         opCtx, collection, includePendingDrops, nameOnly);
                                 }
 
-                                auto view = viewCatalog->lookupWithoutValidatingDurableViews(
-                                    opCtx, nss.ns());
+                                auto view =
+                                    viewCatalog->lookupWithoutValidatingDurableViews(opCtx, nss);
                                 if (view && view->timeseries()) {
                                     if (auto bucketsCollection = CollectionCatalog::get(opCtx)
                                                                      ->lookupCollectionByNamespace(
                                                                          opCtx, view->viewOn())) {
-                                        return buildTimeseriesBson(bucketsCollection, nameOnly);
+                                        return buildTimeseriesBson(
+                                            opCtx, bucketsCollection, nameOnly);
                                     } else {
                                         // The buckets collection does not exist, so the time-series
                                         // view will be appended when we iterate through the view
@@ -393,22 +396,21 @@ public:
                     } else {
                         auto perCollectionWork = [&](const CollectionPtr& collection) {
                             if (collection && collection->getTimeseriesOptions() &&
-                                !collection->ns().isDropPendingNamespace()) {
-                                auto viewNss = collection->ns().getTimeseriesViewNamespace();
-                                auto view = viewCatalog->lookupWithoutValidatingDurableViews(
-                                    opCtx, viewNss.ns());
-                                if (view && view->timeseries() &&
-                                    (!authorizedCollections ||
-                                     as->isAuthorizedForAnyActionOnResource(
-                                         ResourcePattern::forExactNamespace(viewNss)))) {
-                                    // The time-series view for this buckets namespace exists, so
-                                    // add it here while we have the collection options.
-                                    _addWorkingSetMember(opCtx,
-                                                         buildTimeseriesBson(collection, nameOnly),
-                                                         matcher.get(),
-                                                         ws.get(),
-                                                         root.get());
-                                }
+                                !collection->ns().isDropPendingNamespace() &&
+                                viewCatalog->lookupWithoutValidatingDurableViews(
+                                    opCtx, collection->ns().getTimeseriesViewNamespace()) &&
+                                (!authorizedCollections ||
+                                 as->isAuthorizedForAnyActionOnResource(
+                                     ResourcePattern::forExactNamespace(
+                                         collection->ns().getTimeseriesViewNamespace())))) {
+                                // The time-series view for this buckets namespace exists, so add it
+                                // here while we have the collection options.
+                                _addWorkingSetMember(
+                                    opCtx,
+                                    buildTimeseriesBson(opCtx, collection, nameOnly),
+                                    matcher.get(),
+                                    ws.get(),
+                                    root.get());
                             }
 
                             if (authorizedCollections &&
@@ -544,10 +546,10 @@ public:
                  APIParameters::get(opCtx),
                  opCtx->getWriteConcern(),
                  repl::ReadConcernArgs::get(opCtx),
+                 ReadPreferenceSetting::get(opCtx),
                  cmdObj,
                  uassertStatusOK(AuthorizationSession::get(opCtx->getClient())
-                                     ->checkAuthorizedToListCollections(dbName, cmdObj)),
-                 false /*isOpQueryExhaust*/});
+                                     ->checkAuthorizedToListCollections(dbName, cmdObj))});
 
             pinnedCursor->incNBatches();
             pinnedCursor->incNReturnedSoFar(firstBatch.size());
