@@ -27,6 +27,8 @@
  *    it in the license file.
  */
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
+
 #include "mongo/base/checked_cast.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/mutable/document.h"
@@ -41,10 +43,10 @@
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/json.h"
-#include "mongo/db/lasterror.h"
 #include "mongo/db/matcher/doc_validation_error.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/not_primary_error_tracker.h"
 #include "mongo/db/ops/delete_request_gen.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/ops/parsed_update.h"
@@ -65,12 +67,13 @@
 #include "mongo/db/storage/duplicate_key_error_info.h"
 #include "mongo/db/storage/storage_parameters_gen.h"
 #include "mongo/db/timeseries/bucket_catalog.h"
+#include "mongo/db/timeseries/bucket_compression.h"
 #include "mongo/db/timeseries/timeseries_constants.h"
-#include "mongo/db/timeseries/timeseries_index_schema_conversion_functions.h"
-#include "mongo/db/timeseries/timeseries_update_delete_util.h"
+#include "mongo/db/timeseries/timeseries_stats.h"
 #include "mongo/db/transaction_participant.h"
 #include "mongo/db/views/view_catalog.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/logv2/log.h"
 #include "mongo/logv2/redaction.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/util/fail_point.h"
@@ -209,6 +212,20 @@ write_ops::UpdateOpEntry makeTimeseriesUpdateOpEntry(
     return update;
 }
 
+/**
+ * Transforms a single time-series insert to an update request on an existing bucket.
+ */
+write_ops::UpdateOpEntry makeTimeseriesCompressionOpEntry(
+    OperationContext* opCtx,
+    const OID& bucketId,
+    write_ops::UpdateModification::TransformFunc compressionFunc) {
+    write_ops::UpdateModification u(std::move(compressionFunc));
+    write_ops::UpdateOpEntry update(BSON("_id" << bucketId), std::move(u));
+    invariant(!update.getMulti(), bucketId.toString());
+    invariant(!update.getUpsert(), bucketId.toString());
+    return update;
+}
+
 /**
  * Returns the document for inserting a new bucket.
  */
@@ -549,7 +566,7 @@ public:
 
             return insertReply;
         } catch (const DBException& ex) {
-            LastError::get(opCtx->getClient()).setLastError(ex.code(), ex.reason());
+            NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(ex.code());
             throw;
         }
 
@@ -567,7 +584,7 @@ public:
                                             request().getBypassDocumentValidation(),
                                             request());
         } catch (const DBException& ex) {
-            LastError::get(opCtx->getClient()).setLastError(ex.code(), ex.reason());
+            NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(ex.code());
             throw;
         }
 
@@ -624,6 +641,27 @@ public:
             return op;
         }
 
+        write_ops::UpdateCommandRequest _makeTimeseriesCompressionOp(
+            OperationContext* opCtx,
+            const OID& bucketId,
+            write_ops::UpdateModification::TransformFunc compressionFunc) const {
+            write_ops::UpdateCommandRequest op(
+                makeTimeseriesBucketsNamespace(ns()),
+                {makeTimeseriesCompressionOpEntry(opCtx, bucketId, std::move(compressionFunc))});
+
+            write_ops::WriteCommandRequestBase base;
+            // The schema validation configured in the bucket collection is intended for direct
+            // operations by end users and is not applicable here.
+            base.setBypassDocumentValidation(true);
+
+            // Timeseries compression operation is not a user operation and should not use a
+            // statement id from any user op. Set to Uninitialized to bypass.
+            base.setStmtIds(std::vector<StmtId>{kUninitializedStmtId});
+
+            op.setWriteCommandRequestBase(std::move(base));
+            return op;
+        }
+
         /**
          * Returns the status and whether the request can continue.
          */
@@ -659,6 +697,59 @@ public:
                 OperationSource::kTimeseriesInsert));
         }
 
+        TimeseriesSingleWriteResult _performTimeseriesBucketCompression(
+            OperationContext* opCtx, const BucketCatalog::ClosedBucket& closedBucket) const {
+            if (!feature_flags::gTimeseriesBucketCompression.isEnabled(
+                    serverGlobalParams.featureCompatibility)) {
+                return {SingleWriteResult(), true};
+            }
+
+            // Buckets with just a single measurement is not worth compressing.
+            if (closedBucket.numMeasurements <= 1) {
+                return {SingleWriteResult(), true};
+            }
+
+            boost::optional<int> beforeSize;
+            boost::optional<int> afterSize;
+
+            auto bucketCompressionFunc = [&](const BSONObj& bucketDoc) -> boost::optional<BSONObj> {
+                beforeSize = bucketDoc.objsize();
+                // Reset every time we run to ensure we never use a stale value
+                afterSize = boost::none;
+                auto compressed = timeseries::compressBucket(bucketDoc, closedBucket.timeField);
+                // If compressed object size is larger than uncompressed, skip compression update.
+                if (compressed && compressed->objsize() >= *beforeSize) {
+                    LOGV2_DEBUG(5857802,
+                                1,
+                                "Skipping time-series bucket compression, compressed object is "
+                                "larger than original",
+                                "originalSize"_attr = bucketDoc.objsize(),
+                                "compressedSize"_attr = compressed->objsize());
+                    return boost::none;
+                }
+                afterSize = compressed->objsize();
+                return compressed;
+            };
+
+            auto compressionOp =
+                _makeTimeseriesCompressionOp(opCtx, closedBucket.bucketId, bucketCompressionFunc);
+            auto result = _getTimeseriesSingleWriteResult(
+                write_ops_exec::performUpdates(opCtx, compressionOp, OperationSource::kStandard));
+
+            // Report stats, if we fail before running the transform function then just skip
+            // reporting.
+            if (result.result.isOK() && beforeSize) {
+                auto coll = CollectionCatalog::get(opCtx)->lookupCollectionByNamespaceForRead(
+                    opCtx, compressionOp.getNamespace());
+                if (coll) {
+                    const auto& stats = TimeseriesStats::get(coll.get());
+                    stats.onBucketClosed(*beforeSize, afterSize);
+                }
+            }
+
+            return result;
+        }
+
         /**
          * Returns whether the request can continue.
          */
@@ -677,11 +768,20 @@ public:
             bool prepared = bucketCatalog.prepareCommit(batch);
             if (!prepared) {
                 invariant(batch->finished());
+                auto batchStatus = batch->getResult().getStatus();
+                tassert(5916402,
+                        str::stream() << "Got unexpected error (" << batchStatus
+                                      << ") preparing time-series bucket to be committed for "
+                                      << ns() << ": " << redact(request().toBSON({})),
+                        batchStatus == ErrorCodes::TimeseriesBucketCleared ||
+                            batchStatus.isA<ErrorCategory::Interruption>() ||
+                            batchStatus.isA<ErrorCategory::StaleShardVersionError>());
+
                 docsToRetry->push_back(index);
                 return true;
             }
             // Now that the batch is prepared, make sure we clean up if we throw.
-            auto batchGuard = makeGuard([&] { bucketCatalog.abort(batch); });
+            ScopeGuard batchGuard([&] { bucketCatalog.abort(batch); });
 
             hangTimeseriesInsertBeforeWrite.pauseWhileSet();
 
@@ -721,8 +821,20 @@ public:
 
             getOpTimeAndElectionId(opCtx, opTime, electionId);
 
-            bucketCatalog.finish(batch, BucketCatalog::CommitInfo{*opTime, *electionId});
+            auto closedBucket =
+                bucketCatalog.finish(batch, BucketCatalog::CommitInfo{*opTime, *electionId});
+
             batchGuard.dismiss();
+
+            if (closedBucket) {
+                // If this write closed a bucket, compress the bucket
+                auto output = _performTimeseriesBucketCompression(opCtx, *closedBucket);
+                if (auto error =
+                        generateError(opCtx, output.result, start + index, errors->size())) {
+                    errors->push_back(*error);
+                    return output.canContinue;
+                }
+            }
             return true;
         }
 
@@ -753,13 +865,13 @@ public:
             });
 
             boost::optional<Status> abortStatus;
-            auto batchGuard = makeGuard([&] {
+            ScopeGuard batchGuard{[&] {
                 for (auto batch : batchesToCommit) {
                     if (batch.get()) {
                         bucketCatalog.abort(batch, abortStatus);
                     }
                 }
-            });
+            }};
 
             std::vector<write_ops::InsertCommandRequest> insertOps;
             std::vector<write_ops::UpdateCommandRequest> updateOps;
@@ -790,22 +902,45 @@ public:
 
             getOpTimeAndElectionId(opCtx, opTime, electionId);
 
+            bool compressClosedBuckets = true;
             for (auto batch : batchesToCommit) {
-                bucketCatalog.finish(batch, BucketCatalog::CommitInfo{*opTime, *electionId});
+                auto closedBucket =
+                    bucketCatalog.finish(batch, BucketCatalog::CommitInfo{*opTime, *electionId});
                 batch.get().reset();
+
+                if (!closedBucket || !compressClosedBuckets) {
+                    continue;
+                }
+
+                // If this write closed a bucket, compress the bucket
+                auto ret = _performTimeseriesBucketCompression(opCtx, *closedBucket);
+                if (!ret.result.isOK()) {
+                    // Don't try to compress any other buckets if we fail. We're not allowed to do
+                    // more write operations.
+                    compressClosedBuckets = false;
+                }
+                if (!ret.canContinue) {
+                    if (!ret.result.isOK()) {
+                        abortStatus = ret.result.getStatus();
+                    }
+                    return false;
+                }
             }
 
             batchGuard.dismiss();
             return true;
         }
 
-        std::tuple<TimeseriesBatches, TimeseriesStmtIds, size_t> _insertIntoBucketCatalog(
-            OperationContext* opCtx,
-            size_t start,
-            size_t numDocs,
-            const std::vector<size_t>& indices,
-            std::vector<BSONObj>* errors,
-            bool* containsRetry) const {
+        std::tuple<TimeseriesBatches,
+                   TimeseriesStmtIds,
+                   size_t /* numInserted */,
+                   bool /* canContinue */>
+        _insertIntoBucketCatalog(OperationContext* opCtx,
+                                 size_t start,
+                                 size_t numDocs,
+                                 const std::vector<size_t>& indices,
+                                 std::vector<BSONObj>* errors,
+                                 bool* containsRetry) const {
             auto& bucketCatalog = BucketCatalog::get(opCtx);
 
             auto bucketsNs = makeTimeseriesBucketsNamespace(ns());
@@ -822,6 +957,7 @@ public:
 
             TimeseriesBatches batches;
             TimeseriesStmtIds stmtIds;
+            bool canContinue = true;
 
             auto insert = [&](size_t index) {
                 invariant(start + index < request().getDocuments().size());
@@ -850,13 +986,34 @@ public:
                     errors->push_back(*error);
                     return false;
                 } else {
-                    const auto& batch = result.getValue();
+                    const auto& batch = result.getValue().batch;
                     batches.emplace_back(batch, index);
                     if (isTimeseriesWriteRetryable(opCtx)) {
                         stmtIds[batch->bucketId()].push_back(stmtId);
                     }
                 }
 
+                // If this insert closed buckets, rewrite to be a compressed column. If we cannot
+                // perform write operations at this point the bucket will be left uncompressed.
+                for (const auto& closedBucket : result.getValue().closedBuckets) {
+                    if (!canContinue) {
+                        break;
+                    }
+
+                    // If this write closed a bucket, compress the bucket
+                    auto ret = _performTimeseriesBucketCompression(opCtx, closedBucket);
+                    if (auto error =
+                            generateError(opCtx, ret.result, start + index, errors->size())) {
+                        // Bucket compression only fail when we may not try to perform any other
+                        // write operation. When handleError() inside write_ops_exec.cpp return
+                        // false.
+                        errors->push_back(*error);
+                        canContinue = false;
+                        return false;
+                    }
+                    canContinue = ret.canContinue;
+                }
+
                 return true;
             };
 
@@ -865,12 +1022,15 @@ public:
             } else {
                 for (size_t i = 0; i < numDocs; i++) {
                     if (!insert(i) && request().getOrdered()) {
-                        return {std::move(batches), std::move(stmtIds), i};
+                        return {std::move(batches), std::move(stmtIds), i, canContinue};
                     }
                 }
             }
 
-            return {std::move(batches), std::move(stmtIds), request().getDocuments().size()};
+            return {std::move(batches),
+                    std::move(stmtIds),
+                    request().getDocuments().size(),
+                    canContinue};
         }
 
         BSONObj _cloneErrorWithIndex(BSONObj error, size_t index) const {
@@ -955,8 +1115,13 @@ public:
                                                        boost::optional<repl::OpTime>* opTime,
                                                        boost::optional<OID>* electionId,
                                                        bool* containsRetry) const {
-            auto [batches, stmtIds, numInserted] = _insertIntoBucketCatalog(
+            auto [batches, stmtIds, numInserted, canContinue] = _insertIntoBucketCatalog(
                 opCtx, 0, request().getDocuments().size(), {}, errors, containsRetry);
+            if (!canContinue) {
+                // If we are not allowed to continue with any write operation return true here to
+                // prevent the ordered inserts from being retried one by one.
+                return true;
+            }
 
             hangTimeseriesInsertBeforeCommit.pauseWhileSet();
 
@@ -1009,14 +1174,17 @@ public:
                                                               boost::optional<repl::OpTime>* opTime,
                                                               boost::optional<OID>* electionId,
                                                               bool* containsRetry) const {
-            auto [batches, bucketStmtIds, _] =
+            auto [batches, bucketStmtIds, _, canContinue] =
                 _insertIntoBucketCatalog(opCtx, start, numDocs, indices, errors, containsRetry);
-            bool canContinue = true;
 
             hangTimeseriesInsertBeforeCommit.pauseWhileSet();
 
             std::vector<size_t> docsToRetry;
 
+            if (!canContinue) {
+                return docsToRetry;
+            }
+
             size_t itr = 0;
             for (; itr < batches.size(); ++itr) {
                 auto& [batch, index] = batches[itr];
@@ -1130,7 +1298,6 @@ public:
             }
 
             curOp.debug().additiveMetrics.ninserted = baseReply.getN();
-            globalOpCounters.gotInserts(baseReply.getN());
         }
     };
 } cmdInsert;
@@ -1293,14 +1460,11 @@ public:
                 if (update.getArrayFilters()) {
                     CmdUpdate::updateMetrics.incrementExecutedWithArrayFilters();
                 }
-                if (update.getMulti()) {
-                    updateManyCount.increment(1);
-                }
             }
 
             return updateReply;
         } catch (const DBException& ex) {
-            LastError::get(opCtx->getClient()).setLastError(ex.code(), ex.reason());
+            NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(ex.code());
             throw;
         }
 
@@ -1310,7 +1474,7 @@ public:
                                             request().getBypassDocumentValidation(),
                                             request());
         } catch (const DBException& ex) {
-            LastError::get(opCtx->getClient()).setLastError(ex.code(), ex.reason());
+            NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(ex.code());
             throw;
         }
 
@@ -1431,16 +1595,9 @@ public:
                           std::move(reply),
                           &deleteReply);
 
-            // Collect metrics.
-            for (auto&& deletes : request().getDeletes()) {
-                if (deletes.getMulti()) {
-                    deleteManyCount.increment(1);
-                }
-            }
-
             return deleteReply;
         } catch (const DBException& ex) {
-            LastError::get(opCtx->getClient()).setLastError(ex.code(), ex.reason());
+            NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(ex.code());
             throw;
         }
 
@@ -1450,7 +1607,7 @@ public:
                                             request().getBypassDocumentValidation(),
                                             request());
         } catch (const DBException& ex) {
-            LastError::get(opCtx->getClient()).setLastError(ex.code(), ex.reason());
+            NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(ex.code());
             throw;
         }
 
