@@ -58,15 +58,6 @@ MONGO_FAIL_POINT_DEFINE(enableTestOnlyFlagforRSTL);
 
 namespace {
 
-// Ignore data races in certain functions when running with TSAN. For performance reasons,
-// diagnostic commands are expected to race with concurrent lock acquisitions while gathering
-// statistics.
-#if defined(__has_feature) && __has_feature(thread_sanitizer)
-#define MONGO_TSAN_IGNORE __attribute__((no_sanitize("thread")))
-#else
-#define MONGO_TSAN_IGNORE
-#endif
-
 /**
  * Tracks global (across all clients) lock acquisition statistics, partitioned into multiple
  * buckets to minimize concurrent access conflicts.
@@ -126,10 +117,6 @@ private:
     AlignedLockStats _partitions[NumPartitions];
 };
 
-
-// Global lock manager instance.
-LockManager globalLockManager;
-
 // How often (in millis) to check for deadlock if a lock has not been granted for some time
 const Milliseconds MaxWaitTime = Milliseconds(500);
 
@@ -148,6 +135,8 @@ bool LockerImpl::_shouldDelayUnlock(ResourceId resId, LockMode mode) const {
             return false;
 
         case RESOURCE_GLOBAL:
+        case RESOURCE_PBWM:
+        case RESOURCE_RSTL:
         case RESOURCE_DATABASE:
         case RESOURCE_COLLECTION:
         case RESOURCE_METADATA:
@@ -378,7 +367,7 @@ bool LockerImpl::_acquireTicket(OperationContext* opCtx, LockMode mode, Date_t d
         _clientState.store(reader ? kQueuedReader : kQueuedWriter);
 
         // If the ticket wait is interrupted, restore the state of the client.
-        auto restoreStateOnErrorGuard = makeGuard([&] { _clientState.store(kInactive); });
+        ScopeGuard restoreStateOnErrorGuard([&] { _clientState.store(kInactive); });
 
         // Acquiring a ticket is a potentially blocking operation. This must not be called after a
         // transaction timestamp has been set, indicating this transaction has created an oplog
@@ -439,7 +428,8 @@ bool LockerImpl::unlockGlobal() {
         // error for any lock used with multi-granularity locking to have more references than
         // the global lock, because every scope starts by calling lockGlobal.
         const auto resType = it.key().getType();
-        if (resType == RESOURCE_GLOBAL || resType == RESOURCE_MUTEX) {
+        if (resType == RESOURCE_GLOBAL || resType == RESOURCE_PBWM || resType == RESOURCE_RSTL ||
+            resType == RESOURCE_MUTEX) {
             it.next();
         } else {
             invariant(_unlockImpl(&it));
@@ -563,7 +553,7 @@ void LockerImpl::lock(OperationContext* opCtx, ResourceId resId, LockMode mode,
 
 void LockerImpl::downgrade(ResourceId resId, LockMode newMode) {
     LockRequestsMap::Iterator it = _requests.find(resId);
-    globalLockManager.downgrade(it.objAddr(), newMode);
+    getGlobalLockManager()->downgrade(it.objAddr(), newMode);
 }
 
 bool LockerImpl::unlock(ResourceId resId) {
@@ -702,7 +692,13 @@ ResourceId LockerImpl::getWaitingResource() const {
     return _waitingResource;
 }
 
-MONGO_TSAN_IGNORE
+// Ignore data races in this function when running with TSAN. For performance reasons, diagnostic
+// commands are expected to race with concurrent lock acquisitions while gathering statistics.
+#if defined(__has_feature)
+#if __has_feature(thread_sanitizer)
+__attribute__((no_sanitize("thread")))
+#endif
+#endif
 void LockerImpl::getLockerInfo(LockerInfo* lockerInfo,
                                const boost::optional<SingleThreadedLockStats> lockStatsBase) const {
     invariant(lockerInfo);
@@ -790,9 +786,8 @@ bool LockerImpl::saveLockStateAndUnlock(Locker::LockSnapshot* stateOut) {
 
         // We should never have to save and restore metadata locks.
         invariant(RESOURCE_DATABASE == resType || RESOURCE_COLLECTION == resType ||
-                  (resId == resourceIdParallelBatchWriterMode && isSharedLockMode(it->mode)) ||
-                  resId == resourceIdFeatureCompatibilityVersion ||
-                  (resId == resourceIdReplicationStateTransitionLock && it->mode == MODE_IX));
+                  (RESOURCE_PBWM == resType && isSharedLockMode(it->mode)) ||
+                  (RESOURCE_RSTL == resType && it->mode == MODE_IX));
 
         // And, stuff the info into the out parameter.
         OneLock info;
@@ -842,15 +837,17 @@ void LockerImpl::restoreLockState(OperationContext* opCtx, const Locker::LockSna
     invariant(_modeForTicket != MODE_NONE);
 }
 
-MONGO_TSAN_IGNORE
-FlowControlTicketholder::CurOp LockerImpl::getFlowControlStats() const {
-    return _flowControlStats;
-}
-
-MONGO_TSAN_IGNORE
 LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, LockMode mode) {
     dassert(!getWaitingResource().isValid());
 
+    // Operations which are holding open an oplog hole cannot block when acquiring locks.
+    if (opCtx && !shouldAllowLockAcquisitionOnTimestampedUnitOfWork()) {
+        invariant(!opCtx->recoveryUnit()->isTimestamped(),
+                  str::stream()
+                      << "Operation holding open an oplog hole tried to acquire locks. ResourceId: "
+                      << resId << ", mode: " << modeName(mode));
+    }
+
     LockRequest* request;
     bool isNew = true;
 
@@ -884,7 +881,7 @@ LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, Loc
     // Give priority to the full modes for Global, PBWM, and RSTL resources so we don't stall global
     // operations such as shutdown or stepdown.
     const ResourceType resType = resId.getType();
-    if (resType == RESOURCE_GLOBAL) {
+    if (resType == RESOURCE_GLOBAL || resType == RESOURCE_PBWM || resType == RESOURCE_RSTL) {
         if (mode == MODE_S || mode == MODE_X) {
             request->enqueueAtFront = true;
             request->compatibleFirst = true;
@@ -903,8 +900,8 @@ LockResult LockerImpl::_lockBegin(OperationContext* opCtx, ResourceId resId, Loc
     // otherwise we might reset state if the lock becomes granted very fast.
     _notify.clear();
 
-    LockResult result = isNew ? globalLockManager.lock(resId, request, mode)
-                              : globalLockManager.convert(resId, request, mode);
+    LockResult result = isNew ? getGlobalLockManager()->lock(resId, request, mode)
+                              : getGlobalLockManager()->convert(resId, request, mode);
 
     if (result == LOCK_WAITING) {
         globalStats.recordWait(_id, resId, mode);
@@ -929,9 +926,18 @@ void LockerImpl::_lockComplete(OperationContext* opCtx,
                                ResourceId resId,
                                LockMode mode,
                                Date_t deadline) {
+    // Operations which are holding open an oplog hole cannot block when acquiring locks. Lock
+    // requests entering this function have been queued up and will be granted the lock as soon as
+    // the lock is released, which is a blocking operation.
+    if (opCtx && !shouldAllowLockAcquisitionOnTimestampedUnitOfWork()) {
+        invariant(!opCtx->recoveryUnit()->isTimestamped(),
+                  str::stream()
+                      << "Operation holding open an oplog hole tried to acquire locks. ResourceId: "
+                      << resId << ", mode: " << modeName(mode));
+    }
 
     // Clean up the state on any failed lock attempts.
-    auto unlockOnErrorGuard = makeGuard([&] {
+    ScopeGuard unlockOnErrorGuard([&] {
         LockRequestsMap::Iterator it = _requests.find(resId);
         invariant(it);
         _unlockImpl(&it);
@@ -1031,7 +1037,7 @@ void LockerImpl::getFlowControlTicket(OperationContext* opCtx, LockMode lockMode
         // method must not exit with a side-effect on the clientState. That value is also used for
         // tracking whether other resources need to be released.
         _clientState.store(kQueuedWriter);
-        auto restoreState = makeGuard([&] { _clientState.store(kInactive); });
+        ScopeGuard restoreState([&] { _clientState.store(kInactive); });
         // Acquiring a ticket is a potentially blocking operation. This must not be called after a
         // transaction timestamp has been set, indicating this transaction has created an oplog
         // hole.
@@ -1069,7 +1075,7 @@ void LockerImpl::_releaseTicket() {
 }
 
 bool LockerImpl::_unlockImpl(LockRequestsMap::Iterator* it) {
-    if (globalLockManager.unlock(it->objAddr())) {
+    if (getGlobalLockManager()->unlock(it->objAddr())) {
         if (it->key() == resourceIdGlobal) {
             invariant(_modeForTicket != MODE_NONE);
 
@@ -1124,13 +1130,14 @@ public:
 } unusedLockCleaner;
 }  // namespace
 
-
 //
 // Standalone functions
 //
 
 LockManager* getGlobalLockManager() {
-    return &globalLockManager;
+    auto serviceContext = getGlobalServiceContext();
+    invariant(serviceContext);
+    return LockManager::get(serviceContext);
 }
 
 void reportGlobalLockingStats(SingleThreadedLockStats* outStats) {
