@@ -44,6 +44,43 @@ using boost::intrusive_ptr;
 using std::string;
 using std::vector;
 
+namespace {
+/**
+ * Assert that a given field path does not exceed the length limit.
+ */
+void assertFieldPathLengthOK(const FieldPath& path) {
+    uassert(5984700,
+            "Field path exceeds path length limit",
+            path.getPathLength() < BSONDepth::getMaxAllowableDepth());
+}
+void assertFieldPathLengthOK(const std::vector<Position>& path) {
+    uassert(5984701,
+            "Field path exceeds path length limit",
+            path.size() < BSONDepth::getMaxAllowableDepth());
+}
+
+stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
+getNestedFieldHelperBSON(BSONElement elt, const FieldPath& fp, size_t level) {
+    if (level == fp.getPathLength()) {
+        if (elt.ok()) {
+            return elt;
+        } else {
+            return stdx::monostate{};
+        }
+    }
+
+    if (elt.type() == BSONType::Array) {
+        return Document::TraversesArrayTag{};
+    } else if (elt.type() == BSONType::Object) {
+        auto subFieldElt = elt.embeddedObject()[fp.getFieldName(level)];
+        return getNestedFieldHelperBSON(subFieldElt, fp, level + 1);
+    }
+
+    // The path continues "past" a scalar, and therefore does not exist.
+    return stdx::monostate{};
+}
+}  // namespace
+
 const DocumentStorage DocumentStorage::kEmptyDoc;
 
 const StringDataSet Document::allMetadataFieldNames{Document::metaFieldTextScore,
@@ -362,6 +399,12 @@ void DocumentStorage::reset(const BSONObj& bson, bool stripMetadata) {
     _metadataFields = DocumentMetadataFields{};
 }
 
+void DocumentStorage::fillCache() const {
+    for (DocumentStorageIterator it = iterator(); !it.atEnd(); it.advance()) {
+        it->val.fillCache();
+    }
+}
+
 void DocumentStorage::loadLazyMetadata() const {
     if (_haveLazyLoadedMetadata) {
         return;
@@ -431,6 +474,13 @@ Document::Document(std::initializer_list<std::pair<StringData, ImplicitValue>> i
     *this = mutableDoc.freeze();
 }
 
+Document::Document(std::vector<std::pair<StringData, Value>> fields) {
+    MutableDocument mutableDoc(fields.size());
+    for (auto&& pair : fields)
+        mutableDoc.addField(pair.first, pair.second);
+    *this = mutableDoc.freeze();
+}
+
 BSONObjBuilder& operator<<(BSONObjBuilderValueStream& builder, const Document& doc) {
     BSONObjBuilder subobj(builder.subobjStart());
     doc.toBson(&subobj);
@@ -480,7 +530,7 @@ constexpr StringData Document::metaFieldSearchScore;
 constexpr StringData Document::metaFieldSearchHighlights;
 constexpr StringData Document::metaFieldSearchScoreDetails;
 
-BSONObj Document::toBsonWithMetaData(SortKeyFormat sortKeyFormat) const {
+BSONObj Document::toBsonWithMetaData() const {
     BSONObjBuilder bb;
     toBson(&bb);
     if (!metadata()) {
@@ -491,24 +541,10 @@ BSONObj Document::toBsonWithMetaData(SortKeyFormat sortKeyFormat) const {
         bb.append(metaFieldTextScore, metadata().getTextScore());
     if (metadata().hasRandVal())
         bb.append(metaFieldRandVal, metadata().getRandVal());
-    if (metadata().hasSortKey()) {
-        switch (sortKeyFormat) {
-            case SortKeyFormat::k42ChangeStreamSortKey:
-                invariant(metadata().isSingleElementKey());
-                metadata().getSortKey().addToBsonObj(&bb, metaFieldSortKey);
-                break;
-            case SortKeyFormat::k42SortKey:
-                bb.append(metaFieldSortKey,
-                          DocumentMetadataFields::serializeSortKeyAsObject(
-                              metadata().isSingleElementKey(), metadata().getSortKey()));
-                break;
-            case SortKeyFormat::k44SortKey:
-                bb.append(metaFieldSortKey,
-                          DocumentMetadataFields::serializeSortKeyAsArray(
-                              metadata().isSingleElementKey(), metadata().getSortKey()));
-                break;
-        }
-    }
+    if (metadata().hasSortKey())
+        bb.append(metaFieldSortKey,
+                  DocumentMetadataFields::serializeSortKey(metadata().isSingleElementKey(),
+                                                           metadata().getSortKey()));
     if (metadata().hasGeoNearDistance())
         bb.append(metaFieldGeoNearDistance, metadata().getGeoNearDistance());
     if (metadata().hasGeoNearPoint())
@@ -531,7 +567,7 @@ Document Document::fromBsonWithMetaData(const BSONObj& bson) {
     return md.freeze();
 }
 
-Document Document::getOwned() const {
+Document Document::getOwned() const& {
     if (isOwned()) {
         return *this;
     } else {
@@ -541,6 +577,16 @@ Document Document::getOwned() const {
     }
 }
 
+Document Document::getOwned() && {
+    if (isOwned()) {
+        return std::move(*this);
+    } else {
+        MutableDocument md(std::move(*this));
+        md.makeOwned();
+        return md.freeze();
+    }
+}
+
 MutableDocument::MutableDocument(size_t expectedFields)
     : _storageHolder(nullptr), _storage(_storageHolder) {
     if (expectedFields) {
@@ -559,6 +605,7 @@ MutableValue MutableDocument::getNestedFieldHelper(const FieldPath& dottedField,
 
 MutableValue MutableDocument::getNestedField(const FieldPath& dottedField) {
     fassert(16601, dottedField.getPathLength());
+    assertFieldPathLengthOK(dottedField);
     return getNestedFieldHelper(dottedField, 0);
 }
 
@@ -574,31 +621,10 @@ MutableValue MutableDocument::getNestedFieldHelper(const vector<Position>& posit
 
 MutableValue MutableDocument::getNestedField(const vector<Position>& positions) {
     fassert(16488, !positions.empty());
+    assertFieldPathLengthOK(positions);
     return getNestedFieldHelper(positions, 0);
 }
 
-namespace {
-stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
-getNestedFieldHelperBSON(BSONElement elt, const FieldPath& fp, size_t level) {
-    if (level == fp.getPathLength()) {
-        if (elt.ok()) {
-            return elt;
-        } else {
-            return stdx::monostate{};
-        }
-    }
-
-    if (elt.type() == BSONType::Array) {
-        return Document::TraversesArrayTag{};
-    } else if (elt.type() == BSONType::Object) {
-        auto subFieldElt = elt.embeddedObject()[fp.getFieldName(level)];
-        return getNestedFieldHelperBSON(subFieldElt, fp, level + 1);
-    }
-
-    // The path continues "past" a scalar, and therefore does not exist.
-    return stdx::monostate{};
-}
-}  // namespace
 
 stdx::variant<BSONElement, Value, Document::TraversesArrayTag, stdx::monostate>
 Document::getNestedFieldNonCachingHelper(const FieldPath& dottedField, size_t level) const {
@@ -664,6 +690,7 @@ static Value getNestedFieldHelper(const Document& doc,
 
 const Value Document::getNestedField(const FieldPath& path, vector<Position>* positions) const {
     fassert(16489, path.getPathLength());
+    assertFieldPathLengthOK(path);
     return getNestedFieldHelper(*this, path, positions, 0);
 }
 
