@@ -49,13 +49,20 @@ public:
     HashAccessMethod(IndexCatalogEntry* btreeState, std::unique_ptr<SortedDataInterface> btree);
 
 private:
+    void validateDocument(const CollectionPtr& collection,
+                          const BSONObj& obj,
+                          const BSONObj& keyPattern) const override;
+
     /**
      * Fills 'keys' with the keys that should be generated for 'obj' on this index.
      *
      * This function ignores the 'multikeyPaths' and 'multikeyMetadataKeys' pointers because hashed
      * indexes don't support tracking path-level multikey information.
      */
-    void doGetKeys(const BSONObj& obj,
+    void doGetKeys(OperationContext* opCtx,
+                   const CollectionPtr& collection,
+                   SharedBufferFragmentBuilder& pooledBufferBuilder,
+                   const BSONObj& obj,
                    GetKeysContext context,
                    KeyStringSet* keys,
                    KeyStringSet* multikeyMetadataKeys,
