@@ -31,7 +31,7 @@
 
 #include "mongo/platform/basic.h"
 
-#include "mongo/db/index/index_access_method.h"
+#include "mongo/db/index/btree_access_method.h"
 
 #include <utility>
 #include <vector>
@@ -41,28 +41,18 @@
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog/index_consistency.h"
 #include "mongo/db/client.h"
-#include "mongo/db/commands/server_status.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/index/2d_access_method.h"
-#include "mongo/db/index/btree_access_method.h"
-#include "mongo/db/index/fts_access_method.h"
-#include "mongo/db/index/hash_access_method.h"
-#include "mongo/db/index/haystack_access_method.h"
 #include "mongo/db/index/index_build_interceptor.h"
 #include "mongo/db/index/index_descriptor.h"
-#include "mongo/db/index/s2_access_method.h"
-#include "mongo/db/index/wildcard_access_method.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/keypattern.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/timestamp_block.h"
 #include "mongo/db/storage/execution_context.h"
-#include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/logv2/log.h"
-#include "mongo/platform/atomic_word.h"
 #include "mongo/util/progress_meter.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/stacktrace.h"
@@ -79,80 +69,8 @@ MONGO_FAIL_POINT_DEFINE(hangIndexBuildDuringBulkLoadPhaseSecond);
 MONGO_FAIL_POINT_DEFINE(hangDuringIndexBuildBulkLoadYield);
 MONGO_FAIL_POINT_DEFINE(hangDuringIndexBuildBulkLoadYieldSecond);
 
-/**
- * Static factory method that constructs and returns an appropriate IndexAccessMethod depending on
- * the type of the index.
- */
-std::unique_ptr<IndexAccessMethod> IndexAccessMethod::make(
-    IndexCatalogEntry* entry, std::unique_ptr<SortedDataInterface> sortedDataInterface) {
-    auto desc = entry->descriptor();
-    const std::string& type = desc->getAccessMethodName();
-    if ("" == type)
-        return std::make_unique<BtreeAccessMethod>(entry, std::move(sortedDataInterface));
-    else if (IndexNames::HASHED == type)
-        return std::make_unique<HashAccessMethod>(entry, std::move(sortedDataInterface));
-    else if (IndexNames::GEO_2DSPHERE == type)
-        return std::make_unique<S2AccessMethod>(entry, std::move(sortedDataInterface));
-    else if (IndexNames::TEXT == type)
-        return std::make_unique<FTSAccessMethod>(entry, std::move(sortedDataInterface));
-    else if (IndexNames::GEO_HAYSTACK == type)
-        return std::make_unique<HaystackAccessMethod>(entry, std::move(sortedDataInterface));
-    else if (IndexNames::GEO_2D == type)
-        return std::make_unique<TwoDAccessMethod>(entry, std::move(sortedDataInterface));
-    else if (IndexNames::WILDCARD == type)
-        return std::make_unique<WildcardAccessMethod>(entry, std::move(sortedDataInterface));
-    LOGV2(20688,
-          "Can't find index for keyPattern {keyPattern}",
-          "Can't find index for keyPattern",
-          "keyPattern"_attr = desc->keyPattern());
-    fassertFailed(31021);
-}
-
 namespace {
 
-/**
- * Metrics for index bulk builder operations. Intended to support index build diagnostics
- * during the following scenarios:
- * - createIndex commands;
- * - collection cloning during initial sync; and
- * - resuming index builds at startup.
- *
- * Also includes statistics for disk usage (by the external sorter) for index builds that
- * do not fit in memory.
- */
-class IndexBulkBuilderSSS : public ServerStatusSection {
-public:
-    IndexBulkBuilderSSS() : ServerStatusSection("indexBulkBuilder") {}
-
-    bool includeByDefault() const final {
-        return true;
-    }
-
-    void addRequiredPrivileges(std::vector<Privilege>* out) final {}
-
-    BSONObj generateSection(OperationContext* opCtx, const BSONElement& configElement) const final {
-        BSONObjBuilder builder;
-        builder.append("count", count.loadRelaxed());
-        builder.append("resumed", resumed.loadRelaxed());
-        builder.append("filesOpenedForExternalSort", sorterFileStats.opened.loadRelaxed());
-        builder.append("filesClosedForExternalSort", sorterFileStats.closed.loadRelaxed());
-        return builder.obj();
-    }
-
-    // Number of instances of the bulk builder created.
-    AtomicWord<long long> count;
-
-    // Number of times the bulk builder was created for a resumable index build.
-    // This value should not exceed 'count'.
-    AtomicWord<long long> resumed;
-
-    // Number of times the external sorter opened/closed a file handle to spill data to disk.
-    // This pair of counters in aggregate indicate the number of open file handles used by
-    // the external sorter and may be useful in diagnosing situations where the process is
-    // close to exhausting this finite resource.
-    SorterFileStats sorterFileStats;
-} indexBulkBuilderSSS;
-
 /**
  * Returns true if at least one prefix of any of the indexed fields causes the index to be
  * multikey, and returns false otherwise. This function returns false if the 'multikeyPaths'
@@ -169,7 +87,6 @@ SortOptions makeSortOptions(size_t maxMemoryUsageBytes, StringData dbName) {
         .TempDir(storageGlobalParams.dbpath + "/_tmp")
         .ExtSortAllowed()
         .MaxMemoryUsageBytes(maxMemoryUsageBytes)
-        .FileStats(&indexBulkBuilderSSS.sorterFileStats)
         .DBName(dbName.toString());
 }
 
@@ -218,7 +135,9 @@ Status AbstractIndexAccessMethod::insert(OperationContext* opCtx,
     auto multikeyMetadataKeys = executionCtx.multikeyMetadataKeys();
     auto multikeyPaths = executionCtx.multikeyPaths();
 
-    getKeys(pooledBufferBuilder,
+    getKeys(opCtx,
+            coll,
+            pooledBufferBuilder,
             obj,
             options.getKeysMode,
             GetKeysContext::kAddingKeys,
@@ -281,28 +200,19 @@ Status AbstractIndexAccessMethod::insertKeys(OperationContext* opCtx,
     // Add all new keys into the index. The RecordId for each is already encoded in the KeyString.
     for (const auto& keyString : keys) {
         bool unique = _descriptor->unique();
-        auto result = _newInterface->insert(opCtx, keyString, !unique /* dupsAllowed */);
+        Status status = _newInterface->insert(opCtx, keyString, !unique /* dupsAllowed */);
 
         // When duplicates are encountered and allowed, retry with dupsAllowed. Call
         // onDuplicateKey() with the inserted duplicate key.
-        if (ErrorCodes::DuplicateKey == result.getStatus().code() && options.dupsAllowed) {
+        if (ErrorCodes::DuplicateKey == status.code() && options.dupsAllowed) {
             invariant(unique);
+            status = _newInterface->insert(opCtx, keyString, true /* dupsAllowed */);
 
-            result = _newInterface->insert(opCtx, keyString, true /* dupsAllowed */);
-            if (!result.isOK()) {
-                return result.getStatus();
-            } else if (result.getValue() && onDuplicateKey) {
-                // Only run the duplicate key handler if we inserted the key ourselves. Someone else
-                // could have already inserted this exact key, but in that case we don't count it as
-                // a duplicate.
-                auto status = onDuplicateKey(keyString);
-                if (!status.isOK()) {
-                    return status;
-                }
-            }
-        } else if (!result.isOK()) {
-            return result.getStatus();
+            if (status.isOK() && onDuplicateKey)
+                status = onDuplicateKey(keyString);
         }
+        if (!status.isOK())
+            return status;
     }
     if (numInserted) {
         *numInserted = keys.size();
@@ -361,6 +271,7 @@ Status AbstractIndexAccessMethod::initializeAsEmpty(OperationContext* opCtx) {
 }
 
 RecordId AbstractIndexAccessMethod::findSingle(OperationContext* opCtx,
+                                               const CollectionPtr& collection,
                                                const BSONObj& requestedKey) const {
     // Generate the key for this index.
     KeyString::Value actualKey = [&]() {
@@ -373,7 +284,9 @@ RecordId AbstractIndexAccessMethod::findSingle(OperationContext* opCtx,
             KeyStringSet* multikeyMetadataKeys = nullptr;
             MultikeyPaths* multikeyPaths = nullptr;
 
-            getKeys(pooledBuilder,
+            getKeys(opCtx,
+                    collection,
+                    pooledBuilder,
                     requestedKey,
                     GetKeysMode::kEnforceConstraints,
                     GetKeysContext::kAddingKeys,
@@ -474,6 +387,7 @@ pair<KeyStringSet, KeyStringSet> AbstractIndexAccessMethod::setDifference(
 }
 
 void AbstractIndexAccessMethod::prepareUpdate(OperationContext* opCtx,
+                                              const CollectionPtr& collection,
                                               const IndexCatalogEntry* index,
                                               const BSONObj& from,
                                               const BSONObj& to,
@@ -492,7 +406,9 @@ void AbstractIndexAccessMethod::prepareUpdate(OperationContext* opCtx,
         // There's no need to compute the prefixes of the indexed fields that possibly caused the
         // index to be multikey when the old version of the document was written since the index
         // metadata isn't updated when keys are deleted.
-        getKeys(pooledBuilder,
+        getKeys(opCtx,
+                collection,
+                pooledBuilder,
                 from,
                 getKeysMode,
                 GetKeysContext::kRemovingKeys,
@@ -504,7 +420,9 @@ void AbstractIndexAccessMethod::prepareUpdate(OperationContext* opCtx,
     }
 
     if (!indexFilter || indexFilter->matchesBSON(to)) {
-        getKeys(pooledBuilder,
+        getKeys(opCtx,
+                collection,
+                pooledBuilder,
                 to,
                 options.getKeysMode,
                 GetKeysContext::kAddingKeys,
@@ -547,9 +465,9 @@ Status AbstractIndexAccessMethod::update(OperationContext* opCtx,
 
     // Add all new data keys into the index.
     for (const auto& keyString : ticket.added) {
-        auto result = _newInterface->insert(opCtx, keyString, ticket.dupsAllowed);
-        if (!result.isOK())
-            return result.getStatus();
+        Status status = _newInterface->insert(opCtx, keyString, ticket.dupsAllowed);
+        if (!status.isOK())
+            return status;
     }
 
     // If these keys should cause the index to become multikey, pass them into the catalog.
@@ -583,6 +501,7 @@ public:
                     StringData dbName);
 
     Status insert(OperationContext* opCtx,
+                  const CollectionPtr& collection,
                   SharedBufferFragmentBuilder& pooledBuilder,
                   const BSONObj& obj,
                   const RecordId& loc,
@@ -645,9 +564,7 @@ std::unique_ptr<IndexAccessMethod::BulkBuilder> AbstractIndexAccessMethod::initi
 AbstractIndexAccessMethod::BulkBuilderImpl::BulkBuilderImpl(const IndexCatalogEntry* index,
                                                             size_t maxMemoryUsageBytes,
                                                             StringData dbName)
-    : _indexCatalogEntry(index), _sorter(_makeSorter(maxMemoryUsageBytes, dbName)) {
-    indexBulkBuilderSSS.count.addAndFetch(1);
-}
+    : _indexCatalogEntry(index), _sorter(_makeSorter(maxMemoryUsageBytes, dbName)) {}
 
 AbstractIndexAccessMethod::BulkBuilderImpl::BulkBuilderImpl(const IndexCatalogEntry* index,
                                                             size_t maxMemoryUsageBytes,
@@ -658,13 +575,11 @@ AbstractIndexAccessMethod::BulkBuilderImpl::BulkBuilderImpl(const IndexCatalogEn
           _makeSorter(maxMemoryUsageBytes, dbName, stateInfo.getFileName(), stateInfo.getRanges())),
       _keysInserted(stateInfo.getNumKeys().value_or(0)),
       _isMultiKey(stateInfo.getIsMultikey()),
-      _indexMultikeyPaths(createMultikeyPaths(stateInfo.getMultikeyPaths())) {
-    indexBulkBuilderSSS.count.addAndFetch(1);
-    indexBulkBuilderSSS.resumed.addAndFetch(1);
-}
+      _indexMultikeyPaths(createMultikeyPaths(stateInfo.getMultikeyPaths())) {}
 
 Status AbstractIndexAccessMethod::BulkBuilderImpl::insert(
     OperationContext* opCtx,
+    const CollectionPtr& collection,
     SharedBufferFragmentBuilder& pooledBuilder,
     const BSONObj& obj,
     const RecordId& loc,
@@ -678,6 +593,8 @@ Status AbstractIndexAccessMethod::BulkBuilderImpl::insert(
 
     try {
         _indexCatalogEntry->accessMethod()->getKeys(
+            opCtx,
+            collection,
             pooledBuilder,
             obj,
             options.getKeysMode,
@@ -970,7 +887,9 @@ IndexAccessMethod::OnSuppressedErrorFn IndexAccessMethod::kNoopOnSuppressedError
             "obj"_attr = redact(obj));
     };
 
-void AbstractIndexAccessMethod::getKeys(SharedBufferFragmentBuilder& pooledBufferBuilder,
+void AbstractIndexAccessMethod::getKeys(OperationContext* opCtx,
+                                        const CollectionPtr& collection,
+                                        SharedBufferFragmentBuilder& pooledBufferBuilder,
                                         const BSONObj& obj,
                                         GetKeysMode mode,
                                         GetKeysContext context,
@@ -987,7 +906,16 @@ void AbstractIndexAccessMethod::getKeys(SharedBufferFragmentBuilder& pooledBuffe
                           id->toString()));
 
     try {
-        doGetKeys(pooledBufferBuilder, obj, context, keys, multikeyMetadataKeys, multikeyPaths, id);
+        validateDocument(collection, obj, _descriptor->keyPattern());
+        doGetKeys(opCtx,
+                  collection,
+                  pooledBufferBuilder,
+                  obj,
+                  context,
+                  keys,
+                  multikeyMetadataKeys,
+                  multikeyPaths,
+                  id);
     } catch (const AssertionException& ex) {
         // Suppress all indexing errors when mode is kRelaxConstraints.
         if (mode == GetKeysMode::kEnforceConstraints) {
@@ -1022,6 +950,10 @@ bool AbstractIndexAccessMethod::shouldMarkIndexAsMultikey(
     return numberOfKeys > 1 || isMultikeyFromPaths(multikeyPaths);
 }
 
+void AbstractIndexAccessMethod::validateDocument(const CollectionPtr& collection,
+                                                 const BSONObj& obj,
+                                                 const BSONObj& keyPattern) const {}
+
 SortedDataInterface* AbstractIndexAccessMethod::getSortedDataInterface() const {
     return _newInterface.get();
 }
