@@ -146,13 +146,7 @@ bool handlePreValidationMongodOptions(const moe::Environment& params,
 
     if (params.count("replication.enableMajorityReadConcern") &&
         params["replication.enableMajorityReadConcern"].as<bool>() == false) {
-        LOGV2_FATAL_CONTINUE(
-            5324700,
-            "** ERROR: Starting in 5.0, disabling enableMajorityReadConcern setting "
-            "is no longer available because of MongoDB's resiliency enhancements. If "
-            "your environment was previously running with eMRC=f, please see "
-            "https://docs.mongodb.com/v5.0/reference/read-concern-majority/"
-            "#disable-read-concern-majority");
+        LOGV2_FATAL_CONTINUE(5324700, "enableMajorityReadConcern:false is no longer supported");
         return false;
     }
 
@@ -493,11 +487,6 @@ Status storeMongodOptions(const moe::Environment& params) {
         storageGlobalParams.noTableScan.store(params["notablescan"].as<bool>());
     }
 
-    // Initialize lock-free reads support from feature flag. This may be adjusted later based on
-    // replica set config.
-    storageGlobalParams.disableLockFreeReads =
-        !feature_flags::gLockFreeReads.isEnabledAndIgnoreFCV();
-
     repl::ReplSettings replSettings;
     if (params.count("replication.replSet")) {
         /* seed list of hosts for the repl set */
@@ -548,10 +537,7 @@ Status storeMongodOptions(const moe::Environment& params) {
     }
 
     if (!serverGlobalParams.enableMajorityReadConcern) {
-        // Lock-free reads are not supported with enableMajorityReadConcern=false, so we disable
-        // them. If the user tries to explicitly enable lock-free reads by specifying
-        // disableLockFreeReads=false, log a warning so that the user knows these are not
-        // compatible settings.
+        // Lock-free reads is not supported with enableMajorityReadConcern=false, so we disable it.
         if (!storageGlobalParams.disableLockFreeReads) {
             LOGV2_WARNING(4788401,
                           "Lock-free reads is not compatible with "
@@ -617,6 +603,17 @@ Status storeMongodOptions(const moe::Environment& params) {
     }
     if (params.count("sharding.clusterRole")) {
         auto clusterRoleParam = params["sharding.clusterRole"].as<std::string>();
+        const bool replicationEnabled =
+            params.count("replication.replSet") || params.count("replication.replSetName");
+        // Force to set up the node as a replica set, unless we're a shard and we're using queryable
+        // backup mode.
+        if ((clusterRoleParam == "configsvr" || !params.count("storage.queryableBackupMode")) &&
+            !replicationEnabled) {
+            return Status(ErrorCodes::BadValue,
+                          str::stream() << "Cannot start a " << clusterRoleParam
+                                        << " as a standalone server. Please use the option "
+                                           "--replset to start the node as a replica set.");
+        }
         if (clusterRoleParam == "configsvr") {
             serverGlobalParams.clusterRole = ClusterRole::ConfigServer;
             // Config server requires majority read concern.
