@@ -234,6 +234,13 @@ public:
         return _txnNumber;
     }
 
+    /**
+     * Returns the txnRetryCounter associated with this operation.
+     */
+    boost::optional<TxnRetryCounter> getTxnRetryCounter() const {
+        return _txnRetryCounter;
+    }
+
     /**
      * Returns a CancellationToken that will be canceled when the OperationContext is killed via
      * markKilled (including for internal reasons, like the OperationContext deadline being
@@ -263,6 +270,13 @@ public:
      */
     void setTxnNumber(TxnNumber txnNumber);
 
+    /**
+     * Associates a txnRetryCounter with this operation context. May only be called once for the
+     * lifetime of the operation and the operation must have a logical session id and a transaction
+     * number assigned.
+     */
+    void setTxnRetryCounter(TxnRetryCounter txnRetryCounter);
+
     /**
      * Returns the top-level WriteUnitOfWork associated with this operation context, if any.
      */
@@ -443,6 +457,9 @@ public:
      */
     void setInMultiDocumentTransaction() {
         _inMultiDocumentTransaction = true;
+        if (!_txnRetryCounter.has_value()) {
+            _txnRetryCounter = 0;
+        }
     }
 
     /**
@@ -503,6 +520,7 @@ public:
         _isStartingMultiDocumentTransaction = false;
         _lsid = boost::none;
         _txnNumber = boost::none;
+        _txnRetryCounter = boost::none;
     }
 
     /**
@@ -526,10 +544,17 @@ public:
         _isStartingMultiDocumentTransaction = isStartingMultiDocumentTransaction;
     }
 
+    /**
+     * Sets '_comment'. The client lock must be acquired before calling this method.
+     */
     void setComment(const BSONObj& comment) {
         _comment = comment.getOwned();
     }
 
+    /**
+     * Gets '_comment'. The client lock must be acquired when calling from any thread that does
+     * not own the client associated with the operation.
+     */
     boost::optional<BSONElement> getComment() {
         // The '_comment' object, if present, will only ever have one field.
         return _comment ? boost::optional<BSONElement>(_comment->firstElement()) : boost::none;
@@ -655,6 +680,7 @@ private:
 
     boost::optional<LogicalSessionId> _lsid;
     boost::optional<TxnNumber> _txnNumber;
+    boost::optional<TxnRetryCounter> _txnRetryCounter;
 
     std::unique_ptr<Locker> _locker;
 
