@@ -36,18 +36,17 @@
 #include "mongo/db/pipeline/accumulation_statement.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/window_function/window_function_avg.h"
+#include "mongo/db/pipeline/window_function/window_function_expression.h"
 #include "mongo/platform/decimal128.h"
 
 namespace mongo {
 
 using boost::intrusive_ptr;
 
-REGISTER_ACCUMULATOR(avg, genericParseSingleExpressionAccumulator<AccumulatorAvg>);
-REGISTER_EXPRESSION(avg, ExpressionFromAccumulator<AccumulatorAvg>::parse);
-
-const char* AccumulatorAvg::getOpName() const {
-    return "$avg";
-}
+REGISTER_ACCUMULATOR(avg, genericParseSBEUnsupportedSingleExpressionAccumulator<AccumulatorAvg>);
+REGISTER_STABLE_EXPRESSION(avg, ExpressionFromAccumulator<AccumulatorAvg>::parse);
+REGISTER_REMOVABLE_WINDOW_FUNCTION(avg, AccumulatorAvg, WindowFunctionAvg);
 
 namespace {
 const char subTotalName[] = "subTotal";
@@ -83,6 +82,8 @@ void AccumulatorAvg::processInternal(const Value& input, bool merging) {
             _nonDecimalTotal.addLong(input.getLong());
             break;
         case NumberInt:
+            _nonDecimalTotal.addInt(input.getInt());
+            break;
         case NumberDouble:
             _nonDecimalTotal.addDouble(input.getDouble());
             break;
@@ -93,8 +94,7 @@ void AccumulatorAvg::processInternal(const Value& input, bool merging) {
     _count++;
 }
 
-intrusive_ptr<AccumulatorState> AccumulatorAvg::create(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx) {
+intrusive_ptr<AccumulatorState> AccumulatorAvg::create(ExpressionContext* const expCtx) {
     return new AccumulatorAvg(expCtx);
 }
 
@@ -122,7 +122,7 @@ Value AccumulatorAvg::getValue(bool toBeMerged) {
     return Value(_nonDecimalTotal.getDouble() / static_cast<double>(_count));
 }
 
-AccumulatorAvg::AccumulatorAvg(const boost::intrusive_ptr<ExpressionContext>& expCtx)
+AccumulatorAvg::AccumulatorAvg(ExpressionContext* const expCtx)
     : AccumulatorState(expCtx), _isDecimal(false), _count(0) {
     // This is a fixed size AccumulatorState so we never need to update this
     _memUsageBytes = sizeof(*this);
