@@ -42,8 +42,9 @@
 #include "mongo/bson/json.h"
 #include "mongo/db/exec/document_value/document_value_test_util.h"
 #include "mongo/db/exec/document_value/value_comparator.h"
+#include "mongo/db/pipeline/aggregate_command_gen.h"
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
-#include "mongo/db/pipeline/aggregation_request.h"
+#include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document_source_group.h"
 #include "mongo/db/pipeline/document_source_mock.h"
@@ -73,12 +74,12 @@ TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoading) {
     auto expCtx = getExpCtx();
     expCtx->inMongos = true;  // Disallow external sort.
                               // This is the only way to do this in a debug build.
-    auto&& parser = AccumulationStatement::getParser("$sum", boost::none);
+    auto&& [parser, _1, _2, _3] = AccumulationStatement::getParser("$sum");
     auto accumulatorArg = BSON("" << 1);
-    auto accExpr = parser(expCtx, accumulatorArg.firstElement(), expCtx->variablesParseState);
+    auto accExpr = parser(expCtx.get(), accumulatorArg.firstElement(), expCtx->variablesParseState);
     AccumulationStatement countStatement{"count", accExpr};
     auto group = DocumentSourceGroup::create(
-        expCtx, ExpressionConstant::create(expCtx, Value(BSONNULL)), {countStatement});
+        expCtx, ExpressionConstant::create(expCtx.get(), Value(BSONNULL)), {countStatement});
     auto mock =
         DocumentSourceMock::createForTest({DocumentSource::GetNextResult::makePauseExecution(),
                                            Document(),
@@ -86,7 +87,8 @@ TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoading) {
                                            Document(),
                                            Document(),
                                            DocumentSource::GetNextResult::makePauseExecution(),
-                                           Document()});
+                                           Document()},
+                                          expCtx);
     group->setSource(mock.get());
 
     // There were 3 pauses, so we should expect 3 paused results before any results can be returned.
@@ -109,13 +111,13 @@ TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoadingWhileSpilled) {
     expCtx->allowDiskUse = true;
     const size_t maxMemoryUsageBytes = 1000;
 
-    auto&& parser = AccumulationStatement::getParser("$push", boost::none);
+    auto&& [parser, _1, _2, _3] = AccumulationStatement::getParser("$push");
     auto accumulatorArg = BSON(""
                                << "$largeStr");
-    auto accExpr = parser(expCtx, accumulatorArg.firstElement(), expCtx->variablesParseState);
+    auto accExpr = parser(expCtx.get(), accumulatorArg.firstElement(), expCtx->variablesParseState);
     AccumulationStatement pushStatement{"spaceHog", accExpr};
     auto groupByExpression =
-        ExpressionFieldPath::parse(expCtx, "$_id", expCtx->variablesParseState);
+        ExpressionFieldPath::parse(expCtx.get(), "$_id", expCtx->variablesParseState);
     auto group = DocumentSourceGroup::create(
         expCtx, groupByExpression, {pushStatement}, maxMemoryUsageBytes);
 
@@ -125,7 +127,8 @@ TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoadingWhileSpilled) {
                                            DocumentSource::GetNextResult::makePauseExecution(),
                                            Document{{"_id", 1}, {"largeStr", largeStr}},
                                            DocumentSource::GetNextResult::makePauseExecution(),
-                                           Document{{"_id", 2}, {"largeStr", largeStr}}});
+                                           Document{{"_id", 2}, {"largeStr", largeStr}}},
+                                          expCtx);
     group->setSource(mock.get());
 
     // There were 2 pauses, so we should expect 2 paused results before any results can be returned.
@@ -151,19 +154,20 @@ TEST_F(DocumentSourceGroupTest, ShouldErrorIfNotAllowedToSpillToDiskAndResultSet
     expCtx->inMongos = true;  // Disallow external sort.
                               // This is the only way to do this in a debug build.
 
-    auto&& parser = AccumulationStatement::getParser("$push", boost::none);
+    auto&& [parser, _1, _2, _3] = AccumulationStatement::getParser("$push");
     auto accumulatorArg = BSON(""
                                << "$largeStr");
-    auto accExpr = parser(expCtx, accumulatorArg.firstElement(), expCtx->variablesParseState);
+    auto accExpr = parser(expCtx.get(), accumulatorArg.firstElement(), expCtx->variablesParseState);
     AccumulationStatement pushStatement{"spaceHog", accExpr};
     auto groupByExpression =
-        ExpressionFieldPath::parse(expCtx, "$_id", expCtx->variablesParseState);
+        ExpressionFieldPath::parse(expCtx.get(), "$_id", expCtx->variablesParseState);
     auto group = DocumentSourceGroup::create(
         expCtx, groupByExpression, {pushStatement}, maxMemoryUsageBytes);
 
     string largeStr(maxMemoryUsageBytes, 'x');
     auto mock = DocumentSourceMock::createForTest({Document{{"_id", 0}, {"largeStr", largeStr}},
-                                                   Document{{"_id", 1}, {"largeStr", largeStr}}});
+                                                   Document{{"_id", 1}, {"largeStr", largeStr}}},
+                                                  expCtx);
     group->setSource(mock.get());
 
     ASSERT_THROWS_CODE(
@@ -176,13 +180,13 @@ TEST_F(DocumentSourceGroupTest, ShouldCorrectlyTrackMemoryUsageBetweenPauses) {
     expCtx->inMongos = true;  // Disallow external sort.
                               // This is the only way to do this in a debug build.
 
-    auto&& parser = AccumulationStatement::getParser("$push", boost::none);
+    auto&& [parser, _1, _2, _3] = AccumulationStatement::getParser("$push");
     auto accumulatorArg = BSON(""
                                << "$largeStr");
-    auto accExpr = parser(expCtx, accumulatorArg.firstElement(), expCtx->variablesParseState);
+    auto accExpr = parser(expCtx.get(), accumulatorArg.firstElement(), expCtx->variablesParseState);
     AccumulationStatement pushStatement{"spaceHog", accExpr};
     auto groupByExpression =
-        ExpressionFieldPath::parse(expCtx, "$_id", expCtx->variablesParseState);
+        ExpressionFieldPath::parse(expCtx.get(), "$_id", expCtx->variablesParseState);
     auto group = DocumentSourceGroup::create(
         expCtx, groupByExpression, {pushStatement}, maxMemoryUsageBytes);
 
@@ -191,7 +195,8 @@ TEST_F(DocumentSourceGroupTest, ShouldCorrectlyTrackMemoryUsageBetweenPauses) {
         DocumentSourceMock::createForTest({Document{{"_id", 0}, {"largeStr", largeStr}},
                                            DocumentSource::GetNextResult::makePauseExecution(),
                                            Document{{"_id", 1}, {"largeStr", largeStr}},
-                                           Document{{"_id", 2}, {"largeStr", largeStr}}});
+                                           Document{{"_id", 2}, {"largeStr", largeStr}}},
+                                          expCtx);
     group->setSource(mock.get());
 
     // The first getNext() should pause.
@@ -205,7 +210,7 @@ TEST_F(DocumentSourceGroupTest, ShouldCorrectlyTrackMemoryUsageBetweenPauses) {
 TEST_F(DocumentSourceGroupTest, ShouldReportSingleFieldGroupKeyAsARename) {
     auto expCtx = getExpCtx();
     VariablesParseState vps = expCtx->variablesParseState;
-    auto groupByExpression = ExpressionFieldPath::parse(expCtx, "$x", vps);
+    auto groupByExpression = ExpressionFieldPath::parse(expCtx.get(), "$x", vps);
     auto group = DocumentSourceGroup::create(expCtx, groupByExpression, {});
     auto modifiedPathsRet = group->getModifiedPaths();
     ASSERT(modifiedPathsRet.type == DocumentSource::GetModPathsReturn::Type::kAllExcept);
@@ -217,9 +222,9 @@ TEST_F(DocumentSourceGroupTest, ShouldReportSingleFieldGroupKeyAsARename) {
 TEST_F(DocumentSourceGroupTest, ShouldReportMultipleFieldGroupKeysAsARename) {
     auto expCtx = getExpCtx();
     VariablesParseState vps = expCtx->variablesParseState;
-    auto x = ExpressionFieldPath::parse(expCtx, "$x", vps);
-    auto y = ExpressionFieldPath::parse(expCtx, "$y", vps);
-    auto groupByExpression = ExpressionObject::create(expCtx, {{"x", x}, {"y", y}});
+    auto x = ExpressionFieldPath::parse(expCtx.get(), "$x", vps);
+    auto y = ExpressionFieldPath::parse(expCtx.get(), "$y", vps);
+    auto groupByExpression = ExpressionObject::create(expCtx.get(), {{"x", x}, {"y", y}});
     auto group = DocumentSourceGroup::create(expCtx, groupByExpression, {});
     auto modifiedPathsRet = group->getModifiedPaths();
     ASSERT(modifiedPathsRet.type == DocumentSource::GetModPathsReturn::Type::kAllExcept);
@@ -232,7 +237,7 @@ TEST_F(DocumentSourceGroupTest, ShouldReportMultipleFieldGroupKeysAsARename) {
 TEST_F(DocumentSourceGroupTest, ShouldNotReportDottedGroupKeyAsARename) {
     auto expCtx = getExpCtx();
     VariablesParseState vps = expCtx->variablesParseState;
-    auto xDotY = ExpressionFieldPath::parse(expCtx, "$x.y", vps);
+    auto xDotY = ExpressionFieldPath::parse(expCtx.get(), "$x.y", vps);
     auto group = DocumentSourceGroup::create(expCtx, xDotY, {});
     auto modifiedPathsRet = group->getModifiedPaths();
     ASSERT(modifiedPathsRet.type == DocumentSource::GetModPathsReturn::Type::kAllExcept);
@@ -252,7 +257,7 @@ public:
     Base()
         : _opCtx(makeOperationContext()),
           _ctx(new ExpressionContextForTest(_opCtx.get(),
-                                            AggregationRequest(NamespaceString(ns), {}))),
+                                            AggregateCommandRequest(NamespaceString(ns), {}))),
           _tempDir("DocumentSourceGroupTest") {}
 
 protected:
@@ -260,8 +265,8 @@ protected:
         BSONObj namedSpec = BSON("$group" << spec);
         BSONElement specElement = namedSpec.firstElement();
 
-        intrusive_ptr<ExpressionContextForTest> expressionContext =
-            new ExpressionContextForTest(_opCtx.get(), AggregationRequest(NamespaceString(ns), {}));
+        intrusive_ptr<ExpressionContextForTest> expressionContext = new ExpressionContextForTest(
+            _opCtx.get(), AggregateCommandRequest(NamespaceString(ns), {}));
         // For $group, 'inShard' implies 'fromMongos' and 'needsMerge'.
         expressionContext->fromMongos = expressionContext->needsMerge = inShard;
         expressionContext->inMongos = inMongos;
@@ -321,7 +326,7 @@ public:
     virtual ~ExpressionBase() {}
     void _doTest() final {
         createGroup(spec());
-        auto source = DocumentSourceMock::createForTest(Document(doc()));
+        auto source = DocumentSourceMock::createForTest(Document(doc()), ctx());
         group()->setSource(source.get());
         // A group result is available.
         auto next = group()->getNext();
@@ -559,7 +564,7 @@ public:
     }
     void runSharded(bool sharded) {
         createGroup(groupSpec());
-        auto source = DocumentSourceMock::createForTest(inputData());
+        auto source = DocumentSourceMock::createForTest(inputData(), ctx());
         group()->setSource(source.get());
 
         intrusive_ptr<DocumentSource> sink = group();
@@ -785,7 +790,8 @@ public:
         auto source = DocumentSourceMock::createForTest({"{_id:0,list:[1,2]}",
                                                          "{_id:1,list:[3,4]}",
                                                          "{_id:0,list:[10,20]}",
-                                                         "{_id:1,list:[30,40]}]}"});
+                                                         "{_id:1,list:[30,40]}]}"},
+                                                        ctx());
 
         // Create a group source.
         createGroup(BSON("_id"
