@@ -55,9 +55,11 @@ TEST_F(InternalUnpackBucketExecTest, UnpackBasicIncludeAllMeasurementFields) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
     // This source will produce two buckets.
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}",
-         "{meta: {'m1': 9, 'm2': 9, 'm3': 9}, data: {_id: {'0':3, '1':4}, time: {'0':3, '1':4}, "
+         "{control: {'version': 1}, meta: {'m1': 9, 'm2': 9, 'm3': 9}, data: {_id: {'0':3, '1':4}, "
+         "time: {'0':3, '1':4}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -102,9 +104,11 @@ TEST_F(InternalUnpackBucketExecTest, UnpackExcludeASingleField) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}",
-         "{meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: {'0':3, '1':4}, "
+         "{control: {'version': 1}, meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: "
+         "{'0':3, '1':4}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -147,9 +151,11 @@ TEST_F(InternalUnpackBucketExecTest, UnpackEmptyInclude) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}",
-         "{meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: {'0':3, '1':4}, "
+         "{control: {'version': 1}, meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: "
+         "{'0':3, '1':4}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -182,9 +188,11 @@ TEST_F(InternalUnpackBucketExecTest, UnpackEmptyExclude) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}",
-         "{meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: {'0':3, '1':4}, "
+         "{control: {'version': 1}, meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: "
+         "{'0':3, '1':4}, "
          "a:{'0':1, '1':2}, b:{'1':1}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -230,6 +238,7 @@ TEST_F(InternalUnpackBucketExecTest, UnpackNeitherIncludeNorExcludeDefaultsToEmp
     auto source = DocumentSourceMock::createForTest(
         {
             R"({
+    control: {'version': 1}, 
     meta: {'m1': 999, 'm2': 9999},
     data: {
         _id: {'0':1, '1':2},
@@ -239,6 +248,7 @@ TEST_F(InternalUnpackBucketExecTest, UnpackNeitherIncludeNorExcludeDefaultsToEmp
     }
 })",
             R"({
+    control: {'version': 1}, 
     meta: {m1: 9, m2: 9, m3: 9},
     data: {
         _id: {'0':3, '1':4},
@@ -289,7 +299,8 @@ TEST_F(InternalUnpackBucketExecTest, SparseColumnsWhereOneColumnIsExhaustedBefor
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}, "
          "a:{'0':1}, b:{'1':1}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -322,9 +333,11 @@ TEST_F(InternalUnpackBucketExecTest, UnpackBasicIncludeWithDollarPrefix) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
     // This source will produce two buckets.
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}, "
          "$a:{'0':1, '1':2}, b:{'1':1}}}",
-         "{meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: {'0':3, '1':4}, "
+         "{control: {'version': 1}, meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: "
+         "{'0':3, '1':4}, "
          "$a:{'0':1, '1':2}, b:{'1':1}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -368,8 +381,10 @@ TEST_F(InternalUnpackBucketExecTest, UnpackMetadataOnly) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, time: {'0':1, '1':2}}}",
-         "{meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: {'0':3, '1':4}}}"},
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'0':1, '1':2}, "
+         "time: {'0':1, '1':2}}}",
+         "{control: {'version': 1}, meta: {m1: 9, m2: 9, m3: 9}, data: {_id: {'0':3, '1':4}, time: "
+         "{'0':3, '1':4}}}"},
         expCtx);
     unpack->setSource(source.get());
 
@@ -409,9 +424,9 @@ TEST_F(InternalUnpackBucketExecTest, UnpackWithStrangeTimestampOrdering) {
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
          "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
-         "{meta: {'m1': 9, 'm2': 9, 'm3': 9}, data: {_id: {'1':4, "
+         "{control: {'version': 1}, meta: {'m1': 9, 'm2': 9, 'm3': 9}, data: {_id: {'1':4, "
          "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
         expCtx);
     unpack->setSource(source.get());
@@ -460,9 +475,9 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerHandlesMissingMetadataWhenMet
     auto unpack =
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
     auto source =
-        DocumentSourceMock::createForTest({"{data: {_id: {'1':1, "
+        DocumentSourceMock::createForTest({"{control: {'version': 1}, data: {_id: {'1':1, "
                                            "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
-                                           "{data: {_id: {'1':4, "
+                                           "{control: {'version': 1}, data: {_id: {'1':4, "
                                            "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
                                           expCtx);
 
@@ -508,9 +523,9 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerHandlesExcludedMetadataWhenBu
     auto unpack =
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
     auto source = DocumentSourceMock::createForTest(
-        {"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
          "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
-         "{meta: {'m1': 9, 'm2': 9, 'm3': 9}, data: {_id: {'1':4, "
+         "{control: {'version': 1}, meta: {'m1': 9, 'm2': 9, 'm3': 9}, data: {_id: {'1':4, "
          "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
         expCtx);
 
@@ -555,10 +570,10 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerThrowsOnUndefinedMetadata) {
     auto unpack =
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
-    auto source =
-        DocumentSourceMock::createForTest({"{meta: undefined, data: {_id: {'1':1, "
-                                           "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}"},
-                                          expCtx);
+    auto source = DocumentSourceMock::createForTest(
+        {"{control: {'version': 1}, meta: undefined, data: {_id: {'1':1, "
+         "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}"},
+        expCtx);
     unpack->setSource(source.get());
     ASSERT_THROWS_CODE(unpack->getNext(), AssertionException, 5369600);
 }
@@ -572,12 +587,12 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerThrowsWhenMetadataIsPresentUn
     auto unpack =
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
-    auto source =
-        DocumentSourceMock::createForTest({"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
-                                           "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
-                                           "{meta: null, data: {_id: {'1':4, "
-                                           "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
-                                          expCtx);
+    auto source = DocumentSourceMock::createForTest(
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
+         "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
+         "{control: {'version': 1}, meta: null, data: {_id: {'1':4, "
+         "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
+        expCtx);
     unpack->setSource(source.get());
 
     ASSERT_THROWS_CODE(unpack->getNext(), AssertionException, 5369601);
@@ -593,12 +608,12 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerHandlesNullMetadata) {
     auto unpack =
         DocumentSourceInternalUnpackBucket::createFromBsonInternal(spec.firstElement(), expCtx);
 
-    auto source =
-        DocumentSourceMock::createForTest({"{meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
-                                           "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
-                                           "{meta: null, data: {_id: {'1':4, "
-                                           "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
-                                          expCtx);
+    auto source = DocumentSourceMock::createForTest(
+        {"{control: {'version': 1}, meta: {'m1': 999, 'm2': 9999}, data: {_id: {'1':1, "
+         "'0':2, '2': 3}, time: {'1':1, '0': 2, '2': 3}}}",
+         "{control: {'version': 1}, meta: null, data: {_id: {'1':4, "
+         "'0':5, '2':6}, time: {'1':4, '0': 5, '2': 6}}}"},
+        expCtx);
     unpack->setSource(source.get());
 
     auto next = unpack->getNext();
@@ -647,6 +662,7 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerHandlesMissingMetadata) {
         {
             R"(
 {
+    control: {'version': 1}, 
     meta: {
         'm1': 999, 'm2': 9999
     },
@@ -657,6 +673,7 @@ TEST_F(InternalUnpackBucketExecTest, BucketUnpackerHandlesMissingMetadata) {
 })",
             R"(
 {
+    control: {'version': 1}, 
     data: {
         _id: {'1':4, '0':5, '2':6},
         time: {'1':4, '0': 5, '2': 6}
