@@ -26,6 +26,7 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/commands/feature_compatibility_version_parser.h"
 #include "mongo/platform/basic.h"
@@ -41,6 +42,7 @@
 #include "mongo/db/matcher/expression_algo.h"
 #include "mongo/db/pipeline/aggregation_request_helper.h"
 #include "mongo/db/pipeline/document_path_support.h"
+#include "mongo/db/pipeline/document_source_documents.h"
 #include "mongo/db/pipeline/document_source_merge_gen.h"
 #include "mongo/db/pipeline/document_source_sort.h"
 #include "mongo/db/pipeline/expression.h"
@@ -48,6 +50,8 @@
 #include "mongo/db/pipeline/variable_validation.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/query_knobs_gen.h"
+#include "mongo/db/views/resolved_view.h"
+#include "mongo/logv2/log.h"
 #include "mongo/platform/overflow_arithmetic.h"
 #include "mongo/util/fail_point.h"
 
@@ -87,10 +91,6 @@ void lookupPipeValidator(const Pipeline& pipeline) {
     });
 }
 
-bool foreignShardedLookupAllowed() {
-    return getTestCommandsEnabled() && internalQueryAllowShardedLookup.load();
-}
-
 // Parses $lookup 'from' field. The 'from' field must be a string or one of the following
 // exceptions:
 // {from: {db: "config", coll: "cache.chunks.*"}, ...} or
@@ -190,6 +190,16 @@ DocumentSourceLookUp::DocumentSourceLookUp(
     initializeResolvedIntrospectionPipeline();
 }
 
+boost::optional<BSONObj> extractDocumentsStage(const std::vector<BSONObj>& pipeline) {
+    // TODO SERVER-59628 We should be able to check for any valid data source here, not just
+    // $documents.
+    if (pipeline.size() > 0 && pipeline[0].hasField(DocumentSourceDocuments::kStageName)) {
+        return {pipeline[0]};
+    } else {
+        return boost::none;
+    }
+}
+
 DocumentSourceLookUp::DocumentSourceLookUp(
     NamespaceString fromNs,
     std::string as,
@@ -207,19 +217,27 @@ DocumentSourceLookUp::DocumentSourceLookUp(
         std::tie(_localField, _foreignField) = *localForeignFields;
 
         // Append a BSONObj to '_resolvedPipeline' as a placeholder for the stage corresponding to
-        // the local/foreignField $match.
-        _resolvedPipeline.reserve(_resolvedPipeline.size() + 1);
+        // the local/foreignField $match. It must next after $documents if present.
+        auto docs = extractDocumentsStage(pipeline);
+        int offset = docs ? 1 : 0;
+        _resolvedPipeline.reserve(_resolvedPipeline.size() + 1 + offset);
+        if (docs) {
+            _resolvedPipeline.push_back(*docs);
+        }
         _resolvedPipeline.push_back(BSON("$match" << BSONObj()));
         _fieldMatchPipelineIdx = _resolvedPipeline.size() - 1;
+        // Add the user pipeline to '_resolvedPipeline' after any potential view prefix and $match
+        _resolvedPipeline.insert(
+            _resolvedPipeline.end(), pipeline.begin() + offset, pipeline.end());
     } else {
         // When local/foreignFields are included, we cannot enable the cache because the $match
         // is a correlated prefix that will not be detected. Here, local/foreignFields are absent,
         // so we enable the cache.
         _cache.emplace(internalDocumentSourceLookupCacheSizeBytes.load());
+        // Add the user pipeline to '_resolvedPipeline' after any potential view prefix and $match
+        _resolvedPipeline.insert(_resolvedPipeline.end(), pipeline.begin(), pipeline.end());
     }
 
-    // Add the user pipeline to '_resolvedPipeline' after any potential view prefix and $match
-    _resolvedPipeline.insert(_resolvedPipeline.end(), pipeline.begin(), pipeline.end());
     _userPipeline = std::move(pipeline);
 
     for (auto&& varElem : letVariables) {
@@ -235,6 +253,54 @@ DocumentSourceLookUp::DocumentSourceLookUp(
     initializeResolvedIntrospectionPipeline();
 }
 
+DocumentSourceLookUp::DocumentSourceLookUp(const DocumentSourceLookUp& original)
+    : DocumentSource(kStageName, original.pExpCtx->copyWith(original.pExpCtx->ns)),
+      _fromNs(original._fromNs),
+      _resolvedNs(original._resolvedNs),
+      _as(original._as),
+      _additionalFilter(original._additionalFilter),
+      _localField(original._localField),
+      _foreignField(original._foreignField),
+      _fieldMatchPipelineIdx(original._fieldMatchPipelineIdx),
+      _variables(original._variables),
+      _variablesParseState(original._variablesParseState.copyWith(_variables.useIdGenerator())),
+      _fromExpCtx(original._fromExpCtx->copyWith(_resolvedNs)),
+      _hasExplicitCollation(original._hasExplicitCollation),
+      _resolvedPipeline(original._resolvedPipeline),
+      _userPipeline(original._userPipeline),
+      _resolvedIntrospectionPipeline(original._resolvedIntrospectionPipeline->clone()),
+      _letVariables(original._letVariables) {
+    if (!_localField && !_foreignField) {
+        _cache.emplace(internalDocumentSourceCursorBatchSizeBytes.load());
+    }
+    if (original._matchSrc) {
+        _matchSrc = static_cast<DocumentSourceMatch*>(original._matchSrc->clone().get());
+    }
+    if (original._unwindSrc) {
+        _unwindSrc = static_cast<DocumentSourceUnwind*>(original._unwindSrc->clone().get());
+    }
+}
+
+boost::intrusive_ptr<DocumentSource> DocumentSourceLookUp::clone() const {
+    return make_intrusive<DocumentSourceLookUp>(*this);
+}
+
+void validateLookupCollectionlessPipeline(const vector<BSONObj>& pipeline) {
+    uassert(ErrorCodes::FailedToParse,
+            "$lookup stage without explicit collection must have a pipeline with $documents as "
+            "first stage",
+            pipeline.size() > 0 &&
+                // TODO SERVER-59628 We should be able to check for any valid data source here, not
+                // just $documents.
+                !pipeline[0].getField(DocumentSourceDocuments::kStageName).eoo());
+}
+
+void validateLookupCollectionlessPipeline(const BSONElement& pipeline) {
+    uassert(ErrorCodes::FailedToParse, "must specify 'pipeline' when 'from' is empty", pipeline);
+    auto parsedPipeline = parsePipelineFromBSON(pipeline);
+    validateLookupCollectionlessPipeline(parsedPipeline);
+}
+
 std::unique_ptr<DocumentSourceLookUp::LiteParsed> DocumentSourceLookUp::LiteParsed::parse(
     const NamespaceString& nss, const BSONElement& spec) {
     uassert(ErrorCodes::FailedToParse,
@@ -244,16 +310,19 @@ std::unique_ptr<DocumentSourceLookUp::LiteParsed> DocumentSourceLookUp::LitePars
 
     auto specObj = spec.Obj();
     auto fromElement = specObj["from"];
-    uassert(ErrorCodes::FailedToParse,
-            str::stream() << "missing 'from' option to $lookup stage specification: " << specObj,
-            fromElement);
-    auto fromNss = parseLookupFromAndResolveNamespace(fromElement, nss.db());
+    auto pipelineElem = specObj["pipeline"];
+    NamespaceString fromNss;
+    if (!fromElement) {
+        validateLookupCollectionlessPipeline(pipelineElem);
+        fromNss = NamespaceString::makeCollectionlessAggregateNSS(nss.db());
+    } else {
+        fromNss = parseLookupFromAndResolveNamespace(fromElement, nss.db());
+    }
     uassert(ErrorCodes::InvalidNamespace,
             str::stream() << "invalid $lookup namespace: " << fromNss.ns(),
             fromNss.isValid());
 
     // Recursively lite parse the nested pipeline, if one exists.
-    auto pipelineElem = specObj["pipeline"];
     boost::optional<LiteParsedPipeline> liteParsedPipeline;
     if (pipelineElem) {
         auto pipeline = parsePipelineFromBSON(pipelineElem);
@@ -300,15 +369,26 @@ const char* DocumentSourceLookUp::getSourceName() const {
     return kStageName.rawData();
 }
 
-StageConstraints DocumentSourceLookUp::constraints(Pipeline::SplitState) const {
+bool DocumentSourceLookUp::foreignShardedLookupAllowed() const {
+    return feature_flags::gFeatureFlagShardedLookup.isEnabled(
+               serverGlobalParams.featureCompatibility) &&
+        !pExpCtx->opCtx->inMultiDocumentTransaction();
+}
+
+StageConstraints DocumentSourceLookUp::constraints(Pipeline::SplitState pipeState) const {
     HostTypeRequirement hostRequirement;
     if (_fromNs.isConfigDotCacheDotChunks()) {
         // $lookup from config.cache.chunks* namespaces is permitted to run on each individual
         // shard, rather than just the primary, since each shard should have an identical copy of
         // the namespace.
         hostRequirement = HostTypeRequirement::kAnyShard;
+    } else if (pipeState == Pipeline::SplitState::kSplitForShards) {
+        // This stage will only be on the shards pipeline if $lookup on sharded foreign collections
+        // is allowed.
+        hostRequirement = HostTypeRequirement::kAnyShard;
     } else {
-        // When $lookup on sharded foreign collections is allowed, the foreign collection is
+        // If the pipeline is unsplit or this stage is on the merging part of the pipeline,
+        // when $lookup on sharded foreign collections is allowed, the foreign collection is
         // sharded, and the stage is executing on mongos, the stage can run on mongos or any shard.
         hostRequirement = (foreignShardedLookupAllowed() && pExpCtx->inMongos &&
                            pExpCtx->mongoProcessInterface->isSharded(pExpCtx->opCtx, _fromNs))
@@ -368,11 +448,17 @@ DocumentSource::GetNextResult DocumentSourceLookUp::doGetNext() {
     } catch (const ExceptionForCat<ErrorCategory::StaleShardVersionError>& ex) {
         // If lookup on a sharded collection is disallowed and the foreign collection is sharded,
         // throw a custom exception.
-        if (auto staleInfo = ex.extraInfo<StaleConfigInfo>()) {
+        if (auto staleInfo = ex.extraInfo<StaleConfigInfo>(); staleInfo &&
+            staleInfo->getVersionWanted() &&
+            staleInfo->getVersionWanted() != ChunkVersion::UNSHARDED()) {
+            uassert(3904800,
+                    "Cannot run $lookup with a sharded foreign collection in a transaction",
+                    !feature_flags::gFeatureFlagShardedLookup.isEnabled(
+                        serverGlobalParams.featureCompatibility) ||
+                        !pExpCtx->opCtx->inMultiDocumentTransaction());
             uassert(51069,
                     "Cannot run $lookup with sharded foreign collection",
-                    foreignShardedLookupAllowed() || !staleInfo->getVersionWanted() ||
-                        staleInfo->getVersionWanted() == ChunkVersion::UNSHARDED());
+                    foreignShardedLookupAllowed());
         }
         throw;
     }
@@ -400,6 +486,39 @@ DocumentSource::GetNextResult DocumentSourceLookUp::doGetNext() {
     return output.freeze();
 }
 
+std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipelineFromViewDefinition(
+    std::vector<BSONObj> serializedPipeline,
+    ExpressionContext::ResolvedNamespace resolvedNamespace) {
+    // We don't want to optimize or attach a cursor source here because we need to update
+    // _resolvedPipeline so we can reuse it on subsequent calls to getNext(), and we may need to
+    // update _fieldMatchPipelineIdx as well in the case of a field join.
+    MakePipelineOptions opts;
+    opts.optimize = false;
+    opts.attachCursorSource = false;
+    opts.validator = lookupPipeValidator;
+
+    // Resolve the view definition.
+    auto pipeline = Pipeline::makePipelineFromViewDefinition(
+        _fromExpCtx, resolvedNamespace, serializedPipeline, opts);
+
+    // Store the pipeline with resolved namespaces so that we only trigger this exception on the
+    // first input document.
+    _resolvedPipeline = pipeline->serializeToBson();
+
+    // The index of the field join match stage needs to be set to the length of the view
+    // pipeline, as it is no longer the first stage in the resolved pipeline.
+    if (hasLocalFieldForeignFieldJoin()) {
+        _fieldMatchPipelineIdx = resolvedNamespace.pipeline.size();
+    }
+
+    // Update the expression context with any new namespaces the resolved pipeline has introduced.
+    LiteParsedPipeline liteParsedPipeline(resolvedNamespace.ns, resolvedNamespace.pipeline);
+    _fromExpCtx = _fromExpCtx->copyWith(resolvedNamespace.ns);
+    _fromExpCtx->addResolvedNamespaces(liteParsedPipeline.getInvolvedNamespaces());
+
+    return pipeline;
+}
+
 std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     const Document& inputDoc) {
     // Copy all 'let' variables into the foreign pipeline's expression context.
@@ -408,7 +527,8 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     // Resolve the 'let' variables to values per the given input document.
     resolveLetVariables(inputDoc, &_fromExpCtx->variables);
 
-    if (!foreignShardedLookupAllowed()) {
+    const auto allowForeignShardedColl = foreignShardedLookupAllowed();
+    if (!allowForeignShardedColl) {
         // Enforce that the foreign collection must be unsharded for lookup.
         _fromExpCtx->mongoProcessInterface->setExpectedShardVersion(
             _fromExpCtx->opCtx, _fromExpCtx->ns, ChunkVersion::UNSHARDED());
@@ -421,8 +541,31 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
         pipelineOpts.attachCursorSource = true;
         pipelineOpts.validator = lookupPipeValidator;
         // By default, $lookup doesnt support sharded 'from' collections.
-        pipelineOpts.allowTargetingShards = internalQueryAllowShardedLookup.load();
-        return Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+        pipelineOpts.shardTargetingPolicy = allowForeignShardedColl
+            ? ShardTargetingPolicy::kAllowed
+            : ShardTargetingPolicy::kNotAllowed;
+        try {
+            return Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+        } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
+            // This exception returns the information we need to resolve a sharded view. Update the
+            // pipeline with the resolved view definition.
+            auto pipeline = buildPipelineFromViewDefinition(
+                _resolvedPipeline,
+                ExpressionContext::ResolvedNamespace{e->getNamespace(), e->getPipeline()});
+
+            LOGV2_DEBUG(3254800,
+                        3,
+                        "$lookup found view definition. ns: {ns}, pipeline: {pipeline}. New "
+                        "$lookup sub-pipeline: {new_pipe}",
+                        "ns"_attr = e->getNamespace(),
+                        "pipeline"_attr = Value(e->getPipeline()),
+                        "new_pipe"_attr = _resolvedPipeline);
+
+            // We can now safely optimize and reattempt attaching the cursor source.
+            pipeline = Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
+
+            return pipeline;
+        }
     }
 
     // Construct the basic pipeline without a cache stage. Avoid optimizing here since we need to
@@ -433,25 +576,45 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     pipelineOpts.validator = lookupPipeValidator;
     auto pipeline = Pipeline::makePipeline(_resolvedPipeline, _fromExpCtx, pipelineOpts);
 
-    // Add the cache stage at the end and optimize. During the optimization process, the cache will
-    // either move itself to the correct position in the pipeline, or will abandon itself if no
-    // suitable cache position exists. Do it only if pipeline optimization is enabled, otherwise
-    // Pipeline::optimizePipeline() will exit early and correct placement of the cache will not
-    // occur.
-    if (auto fp = globalFailPointRegistry().find("disablePipelineOptimization");
-        fp && fp->shouldFail()) {
-        _cache->abandon();
-    } else {
-        pipeline->addFinalSource(
-            DocumentSourceSequentialDocumentCache::create(_fromExpCtx, _cache.get_ptr()));
-    }
+    // We can store the unoptimized serialization of the pipeline so that if we need to resolve
+    // a sharded view later on, and we have a local-foreign field join, we will need to update
+    // metadata tracking the position of this join in the _resolvedPipeline.
+    auto serializedPipeline = pipeline->serializeToBson();
 
-    pipeline->optimizePipeline();
+    addCacheStageAndOptimize(*pipeline);
 
     if (!_cache->isServing()) {
         // The cache has either been abandoned or has not yet been built. Attach a cursor.
-        pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(
-            pipeline.release(), internalQueryAllowShardedLookup.load() /* allowTargetingShards*/);
+        auto shardTargetingPolicy = allowForeignShardedColl ? ShardTargetingPolicy::kAllowed
+                                                            : ShardTargetingPolicy::kNotAllowed;
+        try {
+            pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+                pipeline.release(), shardTargetingPolicy);
+        } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
+            // This exception returns the information we need to resolve a sharded view. Update the
+            // pipeline with the resolved view definition.
+            pipeline = buildPipelineFromViewDefinition(
+                serializedPipeline,
+                ExpressionContext::ResolvedNamespace{e->getNamespace(), e->getPipeline()});
+
+            // The serialized pipeline does not have a cache stage, so we will add it back to the
+            // pipeline here if the cache has not been abandoned.
+            if (_cache && !_cache->isAbandoned()) {
+                addCacheStageAndOptimize(*pipeline);
+            }
+
+            LOGV2_DEBUG(3254801,
+                        3,
+                        "$lookup found view definition. ns: {ns}, pipeline: {pipeline}. New "
+                        "$lookup sub-pipeline: {new_pipe}",
+                        "ns"_attr = e->getNamespace(),
+                        "pipeline"_attr = Value(e->getPipeline()),
+                        "new_pipe"_attr = _resolvedPipeline);
+
+            // Try to attach the cursor source again.
+            pipeline = pExpCtx->mongoProcessInterface->attachCursorSourceToPipeline(
+                pipeline.release(), shardTargetingPolicy);
+        }
     }
 
     // If the cache has been abandoned, release it.
@@ -463,6 +626,23 @@ std::unique_ptr<Pipeline, PipelineDeleter> DocumentSourceLookUp::buildPipeline(
     return pipeline;
 }
 
+void DocumentSourceLookUp::addCacheStageAndOptimize(Pipeline& pipeline) {
+    // Add the cache stage at the end and optimize. During the optimization process, the cache will
+    // either move itself to the correct position in the pipeline, or will abandon itself if no
+    // suitable cache position exists. Do it only if pipeline optimization is enabled, otherwise
+    // Pipeline::optimizePipeline() will exit early and correct placement of the cache will not
+    // occur.
+    if (auto fp = globalFailPointRegistry().find("disablePipelineOptimization");
+        fp && fp->shouldFail()) {
+        _cache->abandon();
+    } else {
+        pipeline.addFinalSource(
+            DocumentSourceSequentialDocumentCache::create(_fromExpCtx, _cache.get_ptr()));
+    }
+
+    pipeline.optimizePipeline();
+}
+
 DocumentSource::GetModPathsReturn DocumentSourceLookUp::getModifiedPaths() const {
     std::set<std::string> modifiedPaths{_as.fullPath()};
     if (_unwindSrc) {
@@ -797,10 +977,8 @@ void DocumentSourceLookUp::resolveLetVariables(const Document& localDoc, Variabl
 
 void DocumentSourceLookUp::initializeResolvedIntrospectionPipeline() {
     _variables.copyToExpCtx(_variablesParseState, _fromExpCtx.get());
-    _fromExpCtx->startExpressionCounters();
     _resolvedIntrospectionPipeline =
         Pipeline::parse(_resolvedPipeline, _fromExpCtx, lookupPipeValidator);
-    _fromExpCtx->stopExpressionCounters();
 }
 
 void DocumentSourceLookUp::recordPlanSummaryStats(const Pipeline& pipeline) {
@@ -824,7 +1002,7 @@ void DocumentSourceLookUp::appendSpecificExecStats(MutableDocument& doc) const {
     doc["indexesUsed"] = Value{std::move(indexesUsedVec)};
 }
 
-void DocumentSourceLookUp::serializeToArrayWithBothSyntaxes(
+void DocumentSourceLookUp::serializeToArray(
     std::vector<Value>& array, boost::optional<ExplainOptions::Verbosity> explain) const {
 
     // Support alternative $lookup from config.cache.chunks* namespaces.
@@ -884,87 +1062,6 @@ void DocumentSourceLookUp::serializeToArrayWithBothSyntaxes(
     }
 }
 
-void DocumentSourceLookUp::serializeToArray(
-    std::vector<Value>& array, boost::optional<ExplainOptions::Verbosity> explain) const {
-    if (serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-            FeatureCompatibilityParams::Version::kVersion49)) {
-        return serializeToArrayWithBothSyntaxes(array, explain);
-    }
-
-    Document doc;
-
-    // Support alternative $lookup from config.cache.chunks* namespaces.
-    auto fromValue = (pExpCtx->ns.db() == _fromNs.db())
-        ? Value(_fromNs.coll())
-        : Value(Document{{"db", _fromNs.db()}, {"coll", _fromNs.coll()}});
-
-    if (!hasLocalFieldForeignFieldJoin()) {
-        MutableDocument exprList;
-        for (auto letVar : _letVariables) {
-            exprList.addField(letVar.name,
-                              letVar.expression->serialize(static_cast<bool>(explain)));
-        }
-
-        auto pipeline = _userPipeline.get_value_or(std::vector<BSONObj>());
-        // With pipeline syntax, any '_additionalFilter' should be added to the user pipeline. With
-        // only field syntax, we add '_additionalFilter' or '_matchSrc' to the output below.
-        if (_additionalFilter) {
-            pipeline.emplace_back(BSON("$match" << *_additionalFilter));
-        }
-
-        doc = Document{{getSourceName(),
-                        Document{{"from", fromValue},
-                                 {"as", _as.fullPath()},
-                                 {"let", exprList.freeze()},
-                                 {"pipeline", pipeline}}}};
-    } else {
-        doc = Document{{getSourceName(),
-                        Document{{"from", fromValue},
-                                 {"as", _as.fullPath()},
-                                 {"localField", _localField->fullPath()},
-                                 {"foreignField", _foreignField->fullPath()}}}};
-    }
-
-    MutableDocument output(std::move(doc));
-
-    if (_hasExplicitCollation) {
-        output[getSourceName()]["_internalCollation"] = Value(_fromExpCtx->getCollatorBSON());
-    }
-
-    if (explain) {
-        if (_unwindSrc) {
-            const boost::optional<FieldPath> indexPath = _unwindSrc->indexPath();
-            output[getSourceName()]["unwinding"] =
-                Value(DOC("preserveNullAndEmptyArrays"
-                          << _unwindSrc->preserveNullAndEmptyArrays() << "includeArrayIndex"
-                          << (indexPath ? Value(indexPath->fullPath()) : Value())));
-        }
-
-        // Add '_additionalFilter' for explain when $lookup was constructed without pipeline syntax.
-        if (hasLocalFieldForeignFieldJoin() && _additionalFilter) {
-            // Our output does not have to be parseable, so include a "matching" field with the
-            // descended match expression.
-            output[getSourceName()]["matching"] = Value(*_additionalFilter);
-        }
-
-        array.push_back(Value(output.freeze()));
-    } else {
-        array.push_back(Value(output.freeze()));
-
-        if (_unwindSrc) {
-            _unwindSrc->serializeToArray(array);
-        }
-
-        if (hasLocalFieldForeignFieldJoin() && _matchSrc) {
-            // '_matchSrc' tracks the originally specified $match, before it is descended (modified
-            // so it can be moved into '_resolvedPipeline'). It is set in the first call to
-            // getNext(), at which point we are confident that we no longer need to serialize the
-            // $lookup again.
-            _matchSrc->serializeToArray(array);
-        }
-    }
-}
-
 DepsTracker::State DocumentSourceLookUp::getDependencies(DepsTracker* deps) const {
     if (hasPipeline() || _letVariables.size() > 0) {
         // We will use the introspection pipeline which we prebuilt during construction.
@@ -1004,6 +1101,16 @@ DepsTracker::State DocumentSourceLookUp::getDependencies(DepsTracker* deps) cons
 }
 
 boost::optional<DocumentSource::DistributedPlanLogic> DocumentSourceLookUp::distributedPlanLogic() {
+    // If $lookup into a sharded foreign collection is allowed and the foreign namespace is sharded,
+    // top-level $lookup stages can run in parallel on the shards.
+    //
+    // Note that this decision is inherently racy and subject to become stale. This is okay because
+    // either choice will work correctly; we are simply applying a heuristic optimization.
+    if (foreignShardedLookupAllowed() && pExpCtx->subPipelineDepth == 0 &&
+        pExpCtx->mongoProcessInterface->isSharded(_fromExpCtx->opCtx, _fromNs)) {
+        return boost::none;
+    }
+
     if (_fromExpCtx->ns.isConfigDotCacheDotChunks()) {
         // When $lookup reads from config.cache.chunks.* namespaces, it should run on each
         // individual shard in parallel. This is a special case, and atypical for standard $lookup
@@ -1106,17 +1213,13 @@ intrusive_ptr<DocumentSource> DocumentSourceLookUp::createFromBson(
         }
     }
 
-    uassert(
-        ErrorCodes::FailedToParse, "must specify 'from' field for a $lookup", !fromNs.ns().empty());
+    if (fromNs.ns().empty()) {
+        validateLookupCollectionlessPipeline(pipeline);
+        fromNs = NamespaceString::makeCollectionlessAggregateNSS(pExpCtx->ns.db());
+    }
     uassert(ErrorCodes::FailedToParse, "must specify 'as' field for a $lookup", !as.empty());
 
     if (hasPipeline) {
-        uassert(ErrorCodes::FailedToParse,
-                "$lookup with 'pipeline' may not specify 'localField' or 'foreignField'",
-                (localField.empty() && foreignField.empty()) ||
-                    serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-                        FeatureCompatibilityParams::Version::kVersion49));
-
         if (localField.empty() && foreignField.empty()) {
             // $lookup specified with only pipeline syntax.
             return new DocumentSourceLookUp(std::move(fromNs),
