@@ -41,6 +41,7 @@
 #include "mongo/db/pipeline/aggregation_context_fixture.h"
 #include "mongo/db/pipeline/document_source_match.h"
 #include "mongo/db/pipeline/document_source_mock.h"
+#include "mongo/db/pipeline/document_source_project.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/death_test.h"
@@ -487,9 +488,24 @@ TEST_F(DocumentSourceMatchTest, MultipleMatchStagesShouldCombineIntoOne) {
     container.push_back(match3);
     match1->optimizeAt(container.begin(), &container);
     ASSERT_EQUALS(container.size(), 1U);
-    ASSERT_BSONOBJ_EQ(match1->getQuery(),
-                      fromjson("{'$and': [{'$and': [{a:1}, {b:1}]},"
-                               "{c:1}]}"));
+    ASSERT_BSONOBJ_EQ(match1->getQuery(), fromjson("{'$and': [{a:1}, {b:1}, {c:1}]}"));
+}
+
+TEST_F(DocumentSourceMatchTest, DoesNotPushProjectBeforeSelf) {
+    Pipeline::SourceContainer container;
+    auto match = DocumentSourceMatch::create(BSON("_id" << 1), getExpCtx());
+    auto project =
+        DocumentSourceProject::create(BSON("fullDocument" << true), getExpCtx(), "$project"_sd);
+
+    container.push_back(match);
+    container.push_back(project);
+
+    match->optimizeAt(container.begin(), &container);
+
+    ASSERT_EQUALS(2U, container.size());
+    ASSERT(dynamic_cast<DocumentSourceMatch*>(container.begin()->get()));
+    ASSERT(dynamic_cast<DocumentSourceSingleDocumentTransformation*>(
+        std::next(container.begin())->get()));
 }
 
 TEST_F(DocumentSourceMatchTest, ShouldPropagatePauses) {
@@ -542,6 +558,35 @@ TEST_F(DocumentSourceMatchTest, ShouldCorrectlyJoinWithSubsequentMatch) {
     ASSERT_TRUE(match->getNext().isEOF());
 }
 
+TEST_F(DocumentSourceMatchTest, RepeatedJoinWithShouldNotNestAnds) {
+    auto match1 = DocumentSourceMatch::create(fromjson("{}"), getExpCtx());
+    Pipeline::SourceContainer container{
+        match1,
+        DocumentSourceMatch::create(fromjson("{}"), getExpCtx()),
+        DocumentSourceMatch::create(fromjson("{a: 1}"), getExpCtx()),
+        DocumentSourceMatch::create(fromjson("{b: 1}"), getExpCtx()),
+        DocumentSourceMatch::create(fromjson("{$and: [{c: 1}, {d: 1}]}"), getExpCtx()),
+        DocumentSourceMatch::create(fromjson("{$or: [{e: 1}, {f: 1}]}"), getExpCtx()),
+        DocumentSourceMatch::create(fromjson("{}"), getExpCtx()),
+        DocumentSourceMatch::create(
+            fromjson("{$and: [{g: 1}, {h: 1}], $or: [{i: 1}, {j: 1}], $and: [{k: 1}, {l: 1}]}"),
+            getExpCtx()),
+        DocumentSourceMatch::create(fromjson("{$and: [{m: 1}, {$and: [{n: 1}, {o: 1}]}]}"),
+                                    getExpCtx())};
+
+    // Call optimizeAt() repeatedly to trigger joinWith() behavior
+    for (size_t i = 0; i < 8; i++) {
+        match1->optimizeAt(container.begin(), &container);
+    }
+
+    ASSERT_EQUALS(container.size(), 1U);
+    ASSERT_BSONOBJ_EQ(
+        match1->getQuery(),
+        fromjson("{'$and': [{}, {}, {a: 1}, {b: 1}, {c: 1}, {d: 1}, {$or: [{e: 1}, {f: 1}]}, {}, "
+                 "{g: 1}, {h: 1}, {$or: [{i: 1}, {j: 1}]}, {k: 1}, {l: 1}, {m: 1}, {$and: [{n: 1}, "
+                 "{o: 1}]}]}"));
+}
+
 DEATH_TEST_REGEX_F(DocumentSourceMatchTest,
                    ShouldFailToDescendExpressionOnPathThatIsNotACommonPrefix,
                    "Invariant failure.*expression::isPathPrefixOf") {
