@@ -48,6 +48,7 @@
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/datetime/date_time_support.h"
 #include "mongo/db/query/explain_options.h"
+#include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/tailable_mode.h"
 #include "mongo/db/server_options.h"
 #include "mongo/util/intrusive_counter.h"
@@ -69,7 +70,6 @@ struct ExpressionCounters {
 
 class ExpressionContext : public RefCountable {
 public:
-    static constexpr size_t kMaxSubPipelineViewDepth = 20;
     struct ResolvedNamespace {
         ResolvedNamespace() = default;
         ResolvedNamespace(NamespaceString ns, std::vector<BSONObj> pipeline);
@@ -98,8 +98,7 @@ public:
          * This constructor is private, all CollatorStashes should be created by calling
          * ExpressionContext::temporarilyChangeCollator().
          */
-        CollatorStash(ExpressionContext* const expCtx,
-                      std::unique_ptr<CollatorInterface> newCollator);
+        CollatorStash(ExpressionContext* expCtx, std::unique_ptr<CollatorInterface> newCollator);
 
         friend class ExpressionContext;
 
@@ -108,15 +107,6 @@ public:
         std::unique_ptr<CollatorInterface> _originalCollator;
     };
 
-    /**
-     * The structure ExpressionCounters encapsulates counters for match, aggregate, and other
-     * expression types as seen in end-user queries.
-     */
-    struct ExpressionCounters {
-        StringMap<uint64_t> aggExprCountersMap;
-        StringMap<uint64_t> matchExprCountersMap;
-    };
-
     /**
      * Constructs an ExpressionContext to be used for Pipeline parsing and evaluation.
      * 'resolvedNamespaces' maps collection names (not full namespaces) to ResolvedNamespaces.
@@ -257,13 +247,21 @@ public:
         boost::optional<UUID> uuid = boost::none,
         boost::optional<std::unique_ptr<CollatorInterface>> updatedCollator = boost::none) const;
 
+    /**
+     * Returns an ExpressionContext that is identical to 'this' except for the 'subPipelineDepth'
+     * and 'needsMerge' fields.
+     */
     boost::intrusive_ptr<ExpressionContext> copyForSubPipeline(NamespaceString nss) const {
         uassert(ErrorCodes::MaxSubPipelineDepthExceeded,
                 str::stream() << "Maximum number of nested sub-pipelines exceeded. Limit is "
-                              << ExpressionContext::kMaxSubPipelineViewDepth,
-                subPipelineDepth < kMaxSubPipelineViewDepth);
+                              << internalMaxSubPipelineViewDepth.load(),
+                subPipelineDepth < internalMaxSubPipelineViewDepth.load());
         auto newCopy = copyWith(std::move(nss));
         newCopy->subPipelineDepth += 1;
+        // The original expCtx might have been attached to an aggregation pipeline running on the
+        // shards. We must reset 'needsMerge' in order to get fully merged results for the
+        // subpipeline.
+        newCopy->needsMerge = false;
         return newCopy;
     }
 
@@ -289,12 +287,11 @@ public:
         _resolvedNamespaces = std::move(resolvedNamespaces);
     }
 
-    void setIsCappedDelete() {
-        _isCappedDelete = true;
-    }
-
-    bool getIsCappedDelete() const {
-        return _isCappedDelete;
+    void addResolvedNamespaces(
+        mongo::stdx::unordered_set<mongo::NamespaceString> resolvedNamespaces) {
+        for (auto&& nss : resolvedNamespaces) {
+            _resolvedNamespaces.try_emplace(nss.coll(), nss, std::vector<BSONObj>{});
+        }
     }
 
     /**
@@ -346,30 +343,21 @@ public:
      */
     void incrementMatchExprCounter(StringData name);
 
-    /**
-     * Increment the counter for the aggregate expression with a given name.
-     */
-    void incrementAggExprCounter(StringData name);
-
     /**
      * Merge expression counters from the current expression context into the global maps
      * and stop counting.
      */
     void stopExpressionCounters();
 
-    bool expressionCountersAreActive() {
-        return _expressionCounters.is_initialized();
-    }
-
     // The explain verbosity requested by the user, or boost::none if no explain was requested.
     boost::optional<ExplainOptions::Verbosity> explain;
 
     bool fromMongos = false;
     bool needsMerge = false;
     bool inMongos = false;
-    bool forPerShardCursor = false;
     bool allowDiskUse = false;
     bool bypassDocumentValidation = false;
+    bool inMultiDocumentTransaction = false;
     bool hasWhereClause = false;
 
     NamespaceString ns;
@@ -403,11 +391,10 @@ public:
     BSONObj initialPostBatchResumeToken;
 
     // Tracks the depth of nested aggregation sub-pipelines. Used to enforce depth limits.
-    size_t subPipelineDepth = 0;
+    long long subPipelineDepth = 0;
 
     // If set, this will disallow use of features introduced in versions above the provided version.
-    boost::optional<ServerGlobalParams::FeatureCompatibility::Version>
-        maxFeatureCompatibilityVersion;
+    boost::optional<multiversion::FeatureCompatibilityVersion> maxFeatureCompatibilityVersion;
 
     // True if this ExpressionContext is used to parse a view definition pipeline.
     bool isParsingViewDefinition = false;
@@ -423,6 +410,11 @@ public:
     // SBE expressions.
     bool sbeCompatible = true;
 
+    // True if all accumulators in the $group stage currently being parsed using this expression
+    // context can be translated into equivalent SBE expressions. This value is transient and gets
+    // reset for every $group stage we parse. Each $group stage has their per-stage flag.
+    bool sbeGroupCompatible = true;
+
     // These fields can be used in a context when API version validations were not enforced during
     // parse time (Example creating a view or validator), but needs to be enforce while querying
     // later.
@@ -437,11 +429,6 @@ public:
     // When non-empty, contains the unmodified user provided aggregation command.
     BSONObj originalAggregateCommand;
 
-    // True if the expression context is the original one for a given pipeline.
-    // False if another context is created for the same pipeline. Used to disable duplicate
-    // expression counting.
-    bool enabledCounters = true;
-
 protected:
     static const int kInterruptCheckPeriod = 128;
 
@@ -464,8 +451,6 @@ protected:
 
     int _interruptCounter = kInterruptCheckPeriod;
 
-    bool _isCappedDelete = false;
-
 private:
     boost::optional<ExpressionCounters> _expressionCounters = boost::none;
 };
