@@ -40,6 +40,7 @@
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/exec/collection_scan.h"
 #include "mongo/db/exec/fetch.h"
@@ -68,12 +69,14 @@
 #include "mongo/db/pipeline/document_source_sample_from_random_cursor.h"
 #include "mongo/db/pipeline/document_source_single_document_transformation.h"
 #include "mongo/db/pipeline/document_source_sort.h"
+#include "mongo/db/pipeline/inner_pipeline_stage_impl.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/pipeline/skip_and_limit.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/get_executor.h"
 #include "mongo/db/query/plan_executor_factory.h"
 #include "mongo/db/query/plan_summary_stats.h"
+#include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/db/query/query_planner.h"
 #include "mongo/db/query/sort_pattern.h"
 #include "mongo/db/s/collection_sharding_state.h"
@@ -95,6 +98,68 @@ using std::unique_ptr;
 using write_ops::InsertCommandRequest;
 
 namespace {
+/**
+ * Extracts a prefix of 'DocumentSourceGroup' stages from the given pipeline to prepare for
+ * pushdown of $group into the inner query layer so that it can be executed using SBE. Group stages
+ * are extracted from the pipeline under when all of the following conditions are met:
+ *    0. When the 'internalQueryForceClassicEngine' feature flag is 'false'.
+ *    1. When 'allowDiskUse' is false. We currently don't support spilling in the SBE HashAgg
+ *       stage. This will change once that is supported when SERVER-58436 is complete.
+ *    2. When there's only a single index other than the implicit '_id' index on the provided
+ *       collection. This case is necessary because we don't currently support extending the
+ *       QuerySolution with the 'postMultiPlan' QuerySolutionNode when the PlanCache is involved in
+ *       the query. This will be resolved when SERVER-58429 is complete.
+ *    3. $match stage does not have $or and thus, does not need subplanning.
+ *    4. When the DocumentSourceGroup has 'doingMerge=false', this will change when we implement
+ *       hash table spilling in SERVER-58436.
+ */
+std::vector<std::unique_ptr<InnerPipelineStageInterface>> extractSbeCompatibleGroupsForPushdown(
+    const intrusive_ptr<ExpressionContext>& expCtx,
+    const CollectionPtr& collection,
+    const CanonicalQuery* cq,
+    Pipeline* pipeline) {
+    // We will eventually use the extracted group stages to populate 'CanonicalQuery::pipeline'
+    // which requires stages to be wrapped in an interface.
+    std::vector<std::unique_ptr<InnerPipelineStageInterface>> groupsForPushdown;
+
+    // In case that we have a top $or for $match stage, it triggers the tripwire assertion 5842500
+    // because subplanning does not expect that the base query has pushed down $group stage(s) but
+    // it does when $group stage exist in pipeline.
+    // TODO SERVER-60197: Remove this check after supporting this scenario.
+    auto queryNeedsSubplanning = cq->getQueryObj().hasField("$or");
+
+    // This handles the case of unionWith against an unknown collection.
+    if (collection == nullptr) {
+        return {};
+    }
+
+    const auto indexCatalog = collection->getIndexCatalog();
+    const auto isSingleIndex =
+        indexCatalog ? indexCatalog->numIndexesTotal(expCtx->opCtx) == 1 : false;
+
+    if (!feature_flags::gFeatureFlagSBEGroupPushdown.isEnabled(
+            serverGlobalParams.featureCompatibility) ||
+        cq->getForceClassicEngine() || expCtx->allowDiskUse || !isSingleIndex ||
+        queryNeedsSubplanning) {
+        return {};
+    }
+
+    auto&& sources = pipeline->getSources();
+
+    for (auto itr = sources.begin(); itr != sources.end();) {
+        auto groupStage = dynamic_cast<DocumentSourceGroup*>(itr->get());
+        if (!(groupStage && groupStage->sbeCompatible()) || groupStage->doingMerge()) {
+            // Only pushdown a prefix of group stages that are supported by sbe.
+            // TODO: SERVER-59070 remove the 'doingMerge' check when we support merging.
+            break;
+        }
+        groupsForPushdown.push_back(std::make_unique<InnerPipelineStageImpl>(groupStage));
+        sources.erase(itr++);
+    }
+
+    return groupsForPushdown;
+}
+
 StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> attemptToGetExecutor(
     const intrusive_ptr<ExpressionContext>& expCtx,
     const CollectionPtr& collection,
@@ -107,7 +172,8 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> attemptToGetExe
     boost::optional<std::string> groupIdForDistinctScan,
     const AggregateCommandRequest* aggRequest,
     const size_t plannerOpts,
-    const MatchExpressionParser::AllowedFeatureSet& matcherFeatures) {
+    const MatchExpressionParser::AllowedFeatureSet& matcherFeatures,
+    Pipeline* pipeline) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     query_request_helper::setTailableMode(expCtx->tailableMode, findCommand.get());
     findCommand->setFilter(queryObj.getOwned());
@@ -185,9 +251,16 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> attemptToGetExe
         }
     }
 
-    bool permitYield = true;
-    return getExecutorFind(
-        expCtx->opCtx, &collection, std::move(cq.getValue()), permitYield, plannerOpts);
+    auto permitYield = true;
+    return getExecutorFind(expCtx->opCtx,
+                           &collection,
+                           std::move(cq.getValue()),
+                           [&](auto* canonicalQuery) {
+                               canonicalQuery->setPipeline(extractSbeCompatibleGroupsForPushdown(
+                                   expCtx, collection, canonicalQuery, pipeline));
+                           },
+                           permitYield,
+                           plannerOpts);
 }
 
 /**
@@ -713,9 +786,10 @@ auto buildProjectionForPushdown(const DepsTracker& deps,
         if (projStage->getType() == TransformerInterface::TransformerType::kInclusionProjection) {
             auto projObj =
                 projStage->getTransformer().serializeTransformation(boost::none).toBson();
-            auto projAst = projection_ast::parse(projStage->getContext(),
-                                                 projObj,
-                                                 ProjectionPolicies::aggregateProjectionPolicies());
+            auto projAst =
+                projection_ast::parseAndAnalyze(projStage->getContext(),
+                                                projObj,
+                                                ProjectionPolicies::aggregateProjectionPolicies());
             if (!projAst.hasExpressions() || allowExpressions) {
                 // If there is an inclusion projection at the front of the pipeline, we have case 1.
                 sources.pop_front();
@@ -903,7 +977,9 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PipelineD::prep
     // Any data returned from the inner executor must be owned.
     size_t plannerOpts = QueryPlannerParams::DEFAULT;
 
-    if (pipeline->peekFront() && pipeline->peekFront()->constraints().isChangeStreamStage()) {
+    bool isChangeStream =
+        pipeline->peekFront() && pipeline->peekFront()->constraints().isChangeStreamStage();
+    if (isChangeStream) {
         invariant(expCtx->tailableMode == TailableModeEnum::kTailableAndAwaitData);
         plannerOpts |= (QueryPlannerParams::TRACK_LATEST_OPLOG_TS |
                         QueryPlannerParams::ASSERT_MIN_TS_HAS_NOT_FALLEN_OFF_OPLOG);
@@ -987,7 +1063,8 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PipelineD::prep
                                                       rewrittenGroupStage->groupId(),
                                                       aggRequest,
                                                       plannerOpts,
-                                                      matcherFeatures);
+                                                      matcherFeatures,
+                                                      pipeline);
 
         if (swExecutorGrouped.isOK()) {
             // Any $limit stage before the $group stage should make the pipeline ineligible for this
@@ -1016,6 +1093,14 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PipelineD::prep
         }
     }
 
+    // If this pipeline is a change stream, then the cursor must use the simple collation, so we
+    // temporarily switch the collator on the ExpressionContext to nullptr. We do this here because
+    // by this point, all the necessary pipeline analyses and optimizations have already been
+    // performed. Note that 'collatorStash' restores the original collator when it leaves scope.
+    std::unique_ptr<CollatorInterface> collatorForCursor = nullptr;
+    auto collatorStash =
+        isChangeStream ? expCtx->temporarilyChangeCollator(std::move(collatorForCursor)) : nullptr;
+
     return attemptToGetExecutor(expCtx,
                                 collection,
                                 nss,
@@ -1027,7 +1112,8 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PipelineD::prep
                                 boost::none, /* groupIdForDistinctScan */
                                 aggRequest,
                                 plannerOpts,
-                                matcherFeatures);
+                                matcherFeatures,
+                                pipeline);
 }
 
 Timestamp PipelineD::getLatestOplogTimestamp(const Pipeline* pipeline) {
