@@ -52,7 +52,9 @@ public:
                                      bool includeBuildUUIDs) override;
 
     std::unique_ptr<Pipeline, PipelineDeleter> attachCursorSourceToPipeline(
-        Pipeline* pipeline, bool allowTargetingShards) override;
+        Pipeline* pipeline,
+        ShardTargetingPolicy shardTargetingPolicy = ShardTargetingPolicy::kAllowed,
+        boost::optional<BSONObj> readConcern = boost::none) override;
 
     BSONObj preparePipelineAndExplain(Pipeline* ownedPipeline, ExplainOptions::Verbosity verbosity);
 
@@ -79,6 +81,13 @@ public:
     std::vector<FieldPath> collectDocumentKeyFieldsActingAsRouter(
         OperationContext*, const NamespaceString&) const final;
 
+    boost::optional<Document> lookupSingleDocument(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        const NamespaceString& nss,
+        UUID collectionUUID,
+        const Document& documentKey,
+        boost::optional<BSONObj> readConcern) final;
+
     Status insert(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                   const NamespaceString& ns,
                   std::vector<BSONObj>&& objs,
@@ -110,6 +119,27 @@ public:
                                         const NamespaceString& ns,
                                         const std::vector<BSONObj>& indexSpecs) override;
 
+    void setExpectedShardVersion(OperationContext* opCtx,
+                                 const NamespaceString& nss,
+                                 boost::optional<ChunkVersion> chunkVersion) override {
+        // Do nothing on a non-shardsvr mongoD.
+    }
+
+    bool setExpectedDbVersion(OperationContext* opCtx,
+                              const NamespaceString& nss,
+                              DatabaseVersion dbVersion) override {
+        // Do nothing on a non-shardsvr mongoD.
+        return false;
+    }
+
+    void unsetExpectedDbVersion(OperationContext* opCtx, const NamespaceString& nss) override {
+        // Do nothing on a non-shardsvr mongoD.
+    }
+
+    void checkOnPrimaryShardForDb(OperationContext* opCtx, const NamespaceString& nss) override {
+        // Do nothing on a non-shardsvr mongoD.
+    }
+
 protected:
     // This constructor is marked as protected in order to prevent instantiation since this
     // interface is designed to have a concrete process interface for each possible
