@@ -44,6 +44,7 @@
 #include "mongo/db/pipeline/sharded_agg_helpers.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/rpc/get_status_from_command_result.h"
@@ -57,6 +58,12 @@ namespace mongo {
 
 using namespace fmt::literals;
 
+ShardServerProcessInterface::ShardServerProcessInterface(
+    OperationContext* opCtx, std::shared_ptr<executor::TaskExecutor> executor)
+    : CommonMongodProcessInterface(executor) {
+    _opIsVersioned = OperationShardingState::isOperationVersioned(opCtx);
+}
+
 bool ShardServerProcessInterface::isSharded(OperationContext* opCtx, const NamespaceString& nss) {
     const auto cm =
         uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss));
@@ -116,6 +123,21 @@ ShardServerProcessInterface::collectDocumentKeyFieldsForHostedCollection(Operati
     return {{"_id"}, false};
 }
 
+boost::optional<Document> ShardServerProcessInterface::lookupSingleDocument(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx,
+    const NamespaceString& nss,
+    UUID collectionUUID,
+    const Document& documentKey,
+    boost::optional<BSONObj> readConcern) {
+    // We only want to retrieve the one document that corresponds to 'documentKey', so we
+    // ignore collation when computing which shard to target.
+    MakePipelineOptions opts;
+    opts.shardTargetingPolicy = ShardTargetingPolicy::kForceTargetingWithSimpleCollation;
+    opts.readConcern = std::move(readConcern);
+
+    return doLookupSingleDocument(expCtx, nss, collectionUUID, documentKey, std::move(opts));
+}
+
 Status ShardServerProcessInterface::insert(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                            const NamespaceString& ns,
                                            std::vector<BSONObj>&& objs,
@@ -172,17 +194,12 @@ BSONObj ShardServerProcessInterface::preparePipelineAndExplain(
 
 std::unique_ptr<ShardFilterer> ShardServerProcessInterface::getShardFilterer(
     const boost::intrusive_ptr<ExpressionContext>& expCtx) const {
-    if (OperationShardingState::isOperationVersioned(expCtx->opCtx)) {
-        // We can only rely on the ownership filter if the operation is coming from the router
-        // (i.e. it is versioned).
-        auto collectionFilter =
-            CollectionShardingState::get(expCtx->opCtx, expCtx->ns)
-                ->getOwnershipFilter(
-                    expCtx->opCtx,
-                    CollectionShardingState::OrphanCleanupPolicy::kDisallowOrphanCleanup);
-        return std::make_unique<ShardFiltererImpl>(std::move(collectionFilter));
-    }
-    return nullptr;
+    auto collectionFilter =
+        CollectionShardingState::get(expCtx->opCtx, expCtx->ns)
+            ->getOwnershipFilter(
+                expCtx->opCtx,
+                CollectionShardingState::OrphanCleanupPolicy::kDisallowOrphanCleanup);
+    return std::make_unique<ShardFiltererImpl>(std::move(collectionFilter));
 }
 
 void ShardServerProcessInterface::renameIfOptionsAndIndexesHaveNotChanged(
@@ -199,14 +216,15 @@ void ShardServerProcessInterface::renameIfOptionsAndIndexesHaveNotChanged(
     newCmdWithWriteConcernBuilder.append(WriteConcernOptions::kWriteConcernField,
                                          opCtx->getWriteConcern().toBSON());
     newCmdObj = newCmdWithWriteConcernBuilder.done();
-    auto response =
-        executeCommandAgainstDatabasePrimary(opCtx,
-                                             // internalRenameIfOptionsAndIndexesMatch is adminOnly.
-                                             NamespaceString::kAdminDb,
-                                             std::move(cachedDbInfo),
-                                             newCmdObj,
-                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                             Shard::RetryPolicy::kNoRetry);
+    auto response = executeRawCommandAgainstDatabasePrimary(
+        opCtx,
+        // internalRenameIfOptionsAndIndexesMatch is adminOnly.
+        NamespaceString::kAdminDb,
+        cachedDbInfo,
+        // $out target collection must not exist or not be sharded.
+        _versionCommandIfAppropriate(newCmdObj, cachedDbInfo, ChunkVersion::UNSHARDED()),
+        ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+        Shard::RetryPolicy::kNoRetry);
     uassertStatusOKWithContext(response.swResponse,
                                str::stream() << "failed while running command " << newCmdObj);
     auto result = response.swResponse.getValue().data;
@@ -236,7 +254,7 @@ BSONObj ShardServerProcessInterface::getCollectionOptions(OperationContext* opCt
             shard->runExhaustiveCursorCommand(opCtx,
                                               ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                                               nss.db().toString(),
-                                              appendDbVersionIfPresent(cmdObj, cachedDbInfo),
+                                              _versionCommandIfAppropriate(cmdObj, cachedDbInfo),
                                               Milliseconds(-1)));
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         return BSONObj{};
@@ -282,7 +300,7 @@ std::list<BSONObj> ShardServerProcessInterface::getIndexSpecs(OperationContext*
             shard->runExhaustiveCursorCommand(opCtx,
                                               ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                                               ns.db().toString(),
-                                              appendDbVersionIfPresent(cmdObj, cachedDbInfo),
+                                              _versionCommandIfAppropriate(cmdObj, cachedDbInfo),
                                               Milliseconds(-1)));
     } catch (ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         return std::list<BSONObj>();
@@ -299,13 +317,13 @@ void ShardServerProcessInterface::createCollection(OperationContext* opCtx,
     finalCmdBuilder.append(WriteConcernOptions::kWriteConcernField,
                            opCtx->getWriteConcern().toBSON());
     BSONObj finalCmdObj = finalCmdBuilder.obj();
-    auto response =
-        executeCommandAgainstDatabasePrimary(opCtx,
-                                             dbName,
-                                             std::move(cachedDbInfo),
-                                             finalCmdObj,
-                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                             Shard::RetryPolicy::kIdempotent);
+    auto response = executeRawCommandAgainstDatabasePrimary(
+        opCtx,
+        dbName,
+        cachedDbInfo,
+        _versionCommandIfAppropriate(finalCmdObj, cachedDbInfo),
+        ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+        Shard::RetryPolicy::kIdempotent);
     uassertStatusOKWithContext(response.swResponse,
                                str::stream() << "failed while running command " << finalCmdObj);
     auto result = response.swResponse.getValue().data;
@@ -333,11 +351,11 @@ void ShardServerProcessInterface::createIndexesOnEmptyCollection(
         ns,
         "copying index for empty collection {}"_format(ns.ns()),
         [&] {
-            auto response = executeCommandAgainstDatabasePrimary(
+            auto response = executeRawCommandAgainstDatabasePrimary(
                 opCtx,
                 ns.db(),
-                std::move(cachedDbInfo),
-                cmdObj,
+                cachedDbInfo,
+                _versionCommandIfAppropriate(cmdObj, cachedDbInfo),
                 ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                 Shard::RetryPolicy::kIdempotent);
 
@@ -363,13 +381,14 @@ void ShardServerProcessInterface::dropCollection(OperationContext* opCtx,
     newCmdBuilder.append(WriteConcernOptions::kWriteConcernField,
                          opCtx->getWriteConcern().toBSON());
     auto cmdObj = newCmdBuilder.done();
-    auto response =
-        executeCommandAgainstDatabasePrimary(opCtx,
-                                             ns.db(),
-                                             std::move(cachedDbInfo),
-                                             cmdObj,
-                                             ReadPreferenceSetting(ReadPreference::PrimaryOnly),
-                                             Shard::RetryPolicy::kIdempotent);
+    auto response = executeRawCommandAgainstDatabasePrimary(
+        opCtx,
+        ns.db(),
+        cachedDbInfo,
+        // Only unsharded collections can be dropped.
+        _versionCommandIfAppropriate(cmdObj, cachedDbInfo, ChunkVersion::UNSHARDED()),
+        ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+        Shard::RetryPolicy::kIdempotent);
     uassertStatusOKWithContext(response.swResponse,
                                str::stream() << "failed while running command " << cmdObj);
     auto result = response.swResponse.getValue().data;
@@ -382,8 +401,33 @@ void ShardServerProcessInterface::dropCollection(OperationContext* opCtx,
 
 std::unique_ptr<Pipeline, PipelineDeleter>
 ShardServerProcessInterface::attachCursorSourceToPipeline(Pipeline* ownedPipeline,
-                                                          bool allowTargetingShards) {
-    return sharded_agg_helpers::attachCursorToPipeline(ownedPipeline, allowTargetingShards);
+                                                          ShardTargetingPolicy shardTargetingPolicy,
+                                                          boost::optional<BSONObj> readConcern) {
+    return sharded_agg_helpers::attachCursorToPipeline(
+        ownedPipeline, shardTargetingPolicy, std::move(readConcern));
+}
+
+void ShardServerProcessInterface::unsetExpectedDbVersion(OperationContext* opCtx,
+                                                         const NamespaceString& nss) {
+    auto& oss = OperationShardingState::get(opCtx);
+    oss.unsetExpectedDbVersion_Only_For_Aggregation_Local_Reads(nss.db());
+}
+
+bool ShardServerProcessInterface::setExpectedDbVersion(OperationContext* opCtx,
+                                                       const NamespaceString& nss,
+                                                       DatabaseVersion dbVersion) {
+    auto& oss = OperationShardingState::get(opCtx);
+
+    if (auto knownDBVersion = oss.getDbVersion(nss.db())) {
+        uassert(ErrorCodes::IllegalOperation,
+                "Expected db version must match known db version",
+                knownDBVersion == dbVersion);
+    } else if (_opIsVersioned) {
+        oss.initializeClientRoutingVersions(nss, boost::none, dbVersion);
+        return true;
+    }
+
+    return false;
 }
 
 void ShardServerProcessInterface::setExpectedShardVersion(
@@ -393,10 +437,25 @@ void ShardServerProcessInterface::setExpectedShardVersion(
     auto& oss = OperationShardingState::get(opCtx);
     if (oss.hasShardVersion(nss)) {
         invariant(oss.getShardVersion(nss) == chunkVersion);
-    } else {
-        OperationShardingState::get(opCtx).initializeClientRoutingVersions(
-            nss, chunkVersion, boost::none);
+    } else if (_opIsVersioned) {
+        oss.initializeClientRoutingVersions(nss, chunkVersion, boost::none);
     }
 }
 
+void ShardServerProcessInterface::checkOnPrimaryShardForDb(OperationContext* opCtx,
+                                                           const NamespaceString& nss) {
+    DatabaseShardingState::checkIsPrimaryShardForDb(opCtx, nss.db());
+}
+
+BSONObj ShardServerProcessInterface::_versionCommandIfAppropriate(
+    BSONObj cmdObj,
+    const CachedDatabaseInfo& cachedDbInfo,
+    boost::optional<ChunkVersion> shardVersion) {
+    if (!_opIsVersioned) {
+        return cmdObj;
+    }
+    return appendDbVersionIfPresent(
+        shardVersion ? appendShardVersion(cmdObj, *shardVersion) : cmdObj, cachedDbInfo);
+}
+
 }  // namespace mongo
