@@ -30,6 +30,8 @@
 #pragma once
 
 #include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/exec/delete.h"
+#include "mongo/db/exec/update_stage.h"
 #include "mongo/db/ops/delete_request_gen.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/ops/parsed_update.h"
@@ -99,6 +101,7 @@ bool isAnyComponentOfPathMultikey(const BSONObj& indexKeyPattern,
  * will be retrieved.
  */
 IndexEntry indexEntryFromIndexCatalogEntry(OperationContext* opCtx,
+                                           const CollectionPtr& collection,
                                            const IndexCatalogEntry& ice,
                                            const CanonicalQuery* canonicalQuery = nullptr);
 
@@ -117,11 +120,17 @@ bool shouldWaitForOplogVisibility(OperationContext* opCtx,
  * PlanExecutor.
  *
  * If the query cannot be executed, returns a Status indicating why.
+ *
+ * If the caller provides a 'extractAndAttachPipelineStages' function and the query is eligible for
+ * pushdown into the find layer this function will be invoked to extract pipeline stages and
+ * attach them to the provided 'CanonicalQuery'. This function should capture the Pipeline that
+ * stages should be extracted from.
  */
 StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> getExecutor(
     OperationContext* opCtx,
     const CollectionPtr* collection,
     std::unique_ptr<CanonicalQuery> canonicalQuery,
+    std::function<void(CanonicalQuery*)> extractAndAttachPipelineStages,
     PlanYieldPolicy::YieldPolicy yieldPolicy,
     size_t plannerOptions = 0);
 
@@ -134,22 +143,20 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> getExecutor(
  * PlanExecutor.
  *
  * If the query cannot be executed, returns a Status indicating why.
+ *
+ * If the caller provides a 'extractAndAttachPipelineStages' function and the query is eligible for
+ * pushdown into the find layer this function will be invoked to extract pipeline stages and
+ * attach them to the provided 'CanonicalQuery'. This function should capture the Pipeline that
+ * stages should be extracted from.
  */
 StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> getExecutorFind(
     OperationContext* opCtx,
     const CollectionPtr* collection,
     std::unique_ptr<CanonicalQuery> canonicalQuery,
+    std::function<void(CanonicalQuery*)> extractAndAttachPipelineStages,
     bool permitYield = false,
     size_t plannerOptions = QueryPlannerParams::DEFAULT);
 
-/**
- * Returns a plan executor for a legacy OP_QUERY find.
- */
-StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> getExecutorLegacyFind(
-    OperationContext* opCtx,
-    const CollectionPtr* collection,
-    std::unique_ptr<CanonicalQuery> canonicalQuery);
-
 /**
  * If possible, turn the provided QuerySolution into a QuerySolution that uses a DistinctNode
  * to provide results for the distinct command.
@@ -243,7 +250,8 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> getExecutorDele
     OpDebug* opDebug,
     const CollectionPtr* collection,
     ParsedDelete* parsedDelete,
-    boost::optional<ExplainOptions::Verbosity> verbosity);
+    boost::optional<ExplainOptions::Verbosity> verbosity,
+    DeleteStageParams::DocumentCounter&& documentCounter = nullptr);
 
 /**
  * Get a PlanExecutor for an update operation. 'parsedUpdate' describes the query predicate
@@ -269,5 +277,6 @@ StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> getExecutorUpda
     OpDebug* opDebug,
     const CollectionPtr* collection,
     ParsedUpdate* parsedUpdate,
-    boost::optional<ExplainOptions::Verbosity> verbosity);
+    boost::optional<ExplainOptions::Verbosity> verbosity,
+    UpdateStageParams::DocumentCounter&& documentCounter = nullptr);
 }  // namespace mongo
