@@ -36,9 +36,12 @@
 #include "mongo/db/fts/fts_query.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/matcher/expression.h"
+#include "mongo/db/pipeline/accumulation_statement.h"
+#include "mongo/db/query/classic_plan_cache.h"
 #include "mongo/db/query/index_bounds.h"
-#include "mongo/db/query/plan_cache.h"
+#include "mongo/db/query/plan_enumerator_explain_info.h"
 #include "mongo/db/query/stage_types.h"
+#include "mongo/util/id_generator.h"
 
 namespace mongo {
 
@@ -60,12 +63,63 @@ enum class FieldAvailability {
     kFullyProvided,
 };
 
+/**
+ * Represents the set of sort orders satisfied by the data returned from a particular
+ * QuerySolutionNode.
+ */
+class ProvidedSortSet {
+public:
+    ProvidedSortSet(BSONObj pattern, std::set<std::string> ignoreFields)
+        : _baseSortPattern(std::move(pattern)), _ignoredFields(std::move(ignoreFields)) {}
+    ProvidedSortSet() = default;
+
+    /**
+     * Returns true if the 'input' sort order is provided.
+     *
+     * Note: This function is sensitive to direction, i.e, if a pattern {a: 1} is provided, {a: -1}
+     * may not be provided.
+     */
+    bool contains(BSONObj input) const;
+    BSONObj getBaseSortPattern() const {
+        return _baseSortPattern;
+    }
+    const std::set<std::string>& getIgnoredFields() const {
+        return _ignoredFields;
+    }
+    std::string debugString() const {
+        str::stream ss;
+        ss << "baseSortPattern: " << _baseSortPattern << ", ignoredFields: [";
+        for (auto&& ignoreField : _ignoredFields) {
+            ss << ignoreField
+               << /* last element */ (ignoreField == *_ignoredFields.rbegin() ? "" : ", ");
+        }
+        ss << "]";
+        return ss;
+    }
+
+private:
+    // The base sort order that is used as a reference to generate all possible sort orders. It is
+    // also implied that all the prefixes of '_baseSortPattern' are provided.
+    BSONObj _baseSortPattern;
+
+    // Object to hold set of fields on which there is an equality predicate in the 'query' and
+    // doesn't contribute to the sort order. Note that this doesn't include multiKey fields or
+    // collations fields since they can contribute to the sort order.
+    std::set<std::string> _ignoredFields;
+};
+
+/**
+ * An empty ProvidedSortSet that can be used in QSNs that have no children and don't derive from
+ * QuerySolutionNodeWithSortSet.
+ */
+inline const static ProvidedSortSet kEmptySet;
+
 /**
  * This is an abstract representation of a query plan.  It can be transcribed into a tree of
  * PlanStages, which can then be handed to a PlanRunner for execution.
  */
 struct QuerySolutionNode {
-    QuerySolutionNode() {}
+    QuerySolutionNode() = default;
 
     /**
      * Constructs a QuerySolutionNode with a single child.
@@ -111,7 +165,7 @@ struct QuerySolutionNode {
     /**
      * If true, one of these are true:
      *          1. All outputs are already fetched, or
-     *          2. There is a projection in place and a fetch is not required.
+     *          2. There is a stage in place that makes a FETCH stage unnecessary.
      *
      * If false, a fetch needs to be placed above the root in order to provide results.
      *
@@ -147,15 +201,15 @@ struct QuerySolutionNode {
     virtual bool sortedByDiskLoc() const = 0;
 
     /**
-     * Return a BSONObjSet representing the possible sort orders of the data stream from this node.
-     * If the data is not sorted in any particular fashion, returns an empty set.
+     * Returns a 'ProvidedSortSet' object which can be used to determine the possible sort orders of
+     * the data returned from this node.
      *
      * Usage:
      * 1. If our plan gives us a sort order, we don't have to add a sort stage.
      * 2. If all the children of an OR have the same sort order, we can maintain that
      *    sort order with a STAGE_SORT_MERGE instead of STAGE_OR.
      */
-    virtual const BSONObjSet& getSort() const = 0;
+    virtual const ProvidedSortSet& providedSorts() const = 0;
 
     /**
      * Make a deep copy.
@@ -188,6 +242,35 @@ struct QuerySolutionNode {
                        [](auto& child) { return child.release(); });
     }
 
+    bool getScanLimit() {
+        if (hitScanLimit) {
+            return hitScanLimit;
+        }
+        for (const auto& child : children) {
+            if (child->getScanLimit()) {
+                hitScanLimit = true;
+                return true;
+            }
+        }
+        return false;
+    }
+
+    /**
+     * True, if this node, or any of it's children is of the given 'type'.
+     */
+    bool hasNode(StageType type) const;
+
+    /**
+     * Returns the id associated with this node. Each node in a 'QuerySolution' tree is assigned a
+     * unique identifier, which are assigned as sequential positive integers starting from 1.  An id
+     * of 0 means that no id was explicitly assigned during construction of the QuerySolution.
+     *
+     * The identifiers are unique within the tree, but not across trees.
+     */
+    PlanNodeId nodeId() const {
+        return _nodeId;
+    }
+
     // These are owned here.
     //
     // TODO SERVER-35512: Make this a vector of unique_ptr.
@@ -197,6 +280,8 @@ struct QuerySolutionNode {
     // filter.
     std::unique_ptr<MatchExpression> filter;
 
+    bool hitScanLimit = false;
+
 protected:
     /**
      * Formatting helper used by toString().
@@ -210,8 +295,34 @@ protected:
     void addCommon(str::stream* ss, int indent) const;
 
 private:
+    // Allows the QuerySolution constructor to set '_nodeId'.
+    friend class QuerySolution;
+
     QuerySolutionNode(const QuerySolutionNode&) = delete;
     QuerySolutionNode& operator=(const QuerySolutionNode&) = delete;
+
+    PlanNodeId _nodeId{0u};
+};
+
+struct QuerySolutionNodeWithSortSet : public QuerySolutionNode {
+    QuerySolutionNodeWithSortSet() = default;
+
+    /**
+     * This constructor is only useful for QuerySolutionNodes with a single child.
+     */
+    explicit QuerySolutionNodeWithSortSet(std::unique_ptr<QuerySolutionNode> child)
+        : QuerySolutionNode(std::move(child)) {}
+
+    const ProvidedSortSet& providedSorts() const final {
+        return sortSet;
+    }
+
+    void cloneBaseData(QuerySolutionNodeWithSortSet* other) const {
+        QuerySolutionNode::cloneBaseData(other);
+        other->sortSet = sortSet;
+    }
+
+    ProvidedSortSet sortSet;
 };
 
 /**
@@ -220,14 +331,55 @@ private:
  * A tree of stages may be built from a QuerySolution.  The QuerySolution must outlive the tree
  * of stages.
  */
-struct QuerySolution {
-    QuerySolution() : hasBlockingStage(false), indexFilterApplied(false) {}
+class QuerySolution {
+public:
+    QuerySolution() = default;
 
-    // Owned here.
-    std::unique_ptr<QuerySolutionNode> root;
+    /**
+     * Return true if this solution tree contains a node of the given 'type'.
+     */
+    bool hasNode(StageType type) const {
+        return _root && _root->hasNode(type);
+    }
 
-    // Any filters in root or below point into this object.  Must be owned.
-    BSONObj filterData;
+    /**
+     * Output a human-readable std::string representing the plan.
+     */
+    std::string toString() {
+        if (!_root) {
+            return "empty query solution";
+        }
+
+        str::stream ss;
+        _root->appendToString(&ss, 0);
+        return ss;
+    }
+
+    const QuerySolutionNode* root() const {
+        return _root.get();
+    }
+    QuerySolutionNode* root() {
+        return _root.get();
+    }
+
+    /**
+     * Extends the solution's tree by attaching it to the tree rooted at 'extensionRoot'. The
+     * extension tree must contain exactly one 'SentinelNode' node that denotes the attachment
+     * point. The sentinel node will be replaces with the '_root' node.
+     */
+    void extendWith(std::unique_ptr<QuerySolutionNode> extensionRoot);
+
+    /**
+     * Assigns the QuerySolutionNode rooted at 'root' to this QuerySolution. Also assigns a unique
+     * identifying integer to each node in the tree, which can subsequently be displayed in debug
+     * output (e.g. explain).
+     */
+    void setRoot(std::unique_ptr<QuerySolutionNode> root);
+
+    /**
+     * Extracts the root of the QuerySolutionNode rooted at `_root`.
+     */
+    std::unique_ptr<QuerySolutionNode> extractRoot();
 
     // There are two known scenarios in which a query solution might potentially block:
     //
@@ -238,80 +390,32 @@ struct QuerySolution {
     // Hashed AND stage:
     // The hashed AND stage buffers data from multiple index scans and could block. In that case,
     // we would want to fall back on an alternate non-blocking solution.
-    bool hasBlockingStage;
+    bool hasBlockingStage{false};
 
     // Runner executing this solution might be interested in knowing
     // if the planning process for this solution was based on filtered indices.
-    bool indexFilterApplied;
+    bool indexFilterApplied{false};
 
     // Owned here. Used by the plan cache.
     std::unique_ptr<SolutionCacheData> cacheData;
 
-    /**
-     * Output a human-readable std::string representing the plan.
-     */
-    std::string toString() {
-        if (nullptr == root) {
-            return "empty query solution";
-        }
+    PlanEnumeratorExplainInfo _enumeratorExplainInfo;
 
-        str::stream ss;
-        root->appendToString(&ss, 0);
-        return ss;
-    }
+    // Score calculated by PlanRanker. Only present if there are multiple candidate plans.
+    boost::optional<double> score;
 
 private:
+    using QsnIdGenerator = IdGenerator<PlanNodeId>;
+
     QuerySolution(const QuerySolution&) = delete;
     QuerySolution& operator=(const QuerySolution&) = delete;
+
+    void assignNodeIds(QsnIdGenerator& idGenerator, QuerySolutionNode& node);
+
+    std::unique_ptr<QuerySolutionNode> _root;
 };
 
-struct TextNode : public QuerySolutionNode {
-    TextNode(IndexEntry index)
-        : _sort(SimpleBSONObjComparator::kInstance.makeBSONObjSet()), index(std::move(index)) {}
-
-    virtual ~TextNode() {}
-
-    virtual StageType getType() const {
-        return STAGE_TEXT;
-    }
-
-    virtual void appendToString(str::stream* ss, int indent) const;
-
-    // Text's return is LOC_AND_OBJ so it's fetched and has all fields.
-    bool fetched() const {
-        return true;
-    }
-    FieldAvailability getFieldAvailability(const std::string& field) const {
-        return FieldAvailability::kFullyProvided;
-    }
-    bool sortedByDiskLoc() const {
-        return false;
-    }
-    const BSONObjSet& getSort() const {
-        return _sort;
-    }
-
-    QuerySolutionNode* clone() const;
-
-    BSONObjSet _sort;
-
-    IndexEntry index;
-    std::unique_ptr<fts::FTSQuery> ftsQuery;
-
-    // The number of fields in the prefix of the text index. For example, if the key pattern is
-    //
-    //   { a: 1, b: 1, _fts: "text", _ftsx: 1, c: 1 }
-    //
-    // then the number of prefix fields is 2, because of "a" and "b".
-    size_t numPrefixFields = 0u;
-
-    // "Prefix" fields of a text index can handle equality predicates.  We group them with the
-    // text node while creating the text leaf node and convert them into a BSONObj index prefix
-    // when we finish the text leaf node.
-    BSONObj indexPrefix;
-};
-
-struct CollectionScanNode : public QuerySolutionNode {
+struct CollectionScanNode : public QuerySolutionNodeWithSortSet {
     CollectionScanNode();
     virtual ~CollectionScanNode() {}
 
@@ -330,34 +434,26 @@ struct CollectionScanNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return _sort;
-    }
 
     QuerySolutionNode* clone() const;
 
-    BSONObjSet _sort;
-
     // Name of the namespace.
     std::string name;
 
-    // If present, the collection scan will seek directly to the RecordId of an oplog entry as
-    // close to 'minTs' as possible without going higher. Should only be set on forward oplog scans.
-    // This field cannot be used in conjunction with 'resumeAfterRecordId'.
-    boost::optional<Timestamp> minTs;
+    // If present, this parameter sets the start point of a forward scan or the end point of a
+    // reverse scan.
+    boost::optional<RecordId> minRecord;
 
-    // If present the collection scan will stop and return EOF the first time it sees a document
-    // that does not pass the filter and has 'ts' greater than 'maxTs'. Should only be set on
-    // forward oplog scans.
-    // This field cannot be used in conjunction with 'resumeAfterRecordId'.
-    boost::optional<Timestamp> maxTs;
+    // If present, this parameter sets the start point of a reverse scan or the end point of a
+    // forward scan.
+    boost::optional<RecordId> maxRecord;
 
     // If true, the collection scan will return a token that can be used to resume the scan.
     bool requestResumeToken = false;
 
     // If present, the collection scan will seek to the exact RecordId, or return KeyNotFound if it
     // does not exist. Must only be set on forward collection scans.
-    // This field cannot be used in conjunction with 'minTs' or 'maxTs'.
+    // This field cannot be used in conjunction with 'minRecord' or 'maxRecord'.
     boost::optional<RecordId> resumeAfterRecordId;
 
     // Should we make a tailable cursor?
@@ -368,6 +464,9 @@ struct CollectionScanNode : public QuerySolutionNode {
     // across a sharded cluster.
     bool shouldTrackLatestOplogTimestamp = false;
 
+    // Assert that the specified timestamp has not fallen off the oplog.
+    boost::optional<Timestamp> assertTsHasNotFallenOffOplog = boost::none;
+
     int direction{1};
 
     // Whether or not to wait for oplog visibility on oplog collection scans.
@@ -377,6 +476,66 @@ struct CollectionScanNode : public QuerySolutionNode {
     bool stopApplyingFilterAfterFirstMatch = false;
 };
 
+/**
+ * A VirtualScanNode is similar to a collection or an index scan except that it doesn't depend on an
+ * underlying storage implementation. It can be used to represent a virtual
+ * collection or an index scan in memory by using a backing vector of BSONArray.
+ */
+struct VirtualScanNode : public QuerySolutionNodeWithSortSet {
+    enum class ScanType { kCollScan, kIxscan };
+
+    VirtualScanNode(std::vector<BSONArray> docs,
+                    ScanType scanType,
+                    bool hasRecordId,
+                    BSONObj indexKeyPattern = {});
+
+    virtual StageType getType() const {
+        return STAGE_VIRTUAL_SCAN;
+    }
+
+    virtual void appendToString(str::stream* ss, int indent) const;
+
+    bool fetched() const {
+        return scanType == ScanType::kCollScan;
+    }
+    FieldAvailability getFieldAvailability(const std::string& field) const {
+        if (scanType == ScanType::kCollScan) {
+            return FieldAvailability::kFullyProvided;
+        } else {
+            return indexKeyPattern.hasField(field) ? FieldAvailability::kFullyProvided
+                                                   : FieldAvailability::kNotProvided;
+        }
+    }
+    bool sortedByDiskLoc() const {
+        return false;
+    }
+
+    QuerySolutionNode* clone() const;
+
+    // A representation of a collection's documents. Here we use a BSONArray so metadata like a
+    // RecordId can be stored alongside of the main document payload. The format of the data in
+    // BSONArray is entirely up to a client of this node, but if this node is to be used for
+    // consumption downstream by stage builder implementations it must conform to the format
+    // expected by those stage builders. That expected contract depends on the hasRecordId flag. If
+    // the hasRecordId flag is 'false' the BSONArray will have a single element that is a BSONObj
+    // representation of a document being produced from this node. If 'hasRecordId' is true, then
+    // each BSONArray in docs will carry a RecordId in the zeroth position of the array and a
+    // BSONObj in the first position of the array.
+    std::vector<BSONArray> docs;
+
+    // Indicates whether the scan is mimicking a collection scan or index scan.
+    const ScanType scanType;
+
+    // A flag to indicate the format of the BSONArray document payload in the above vector, docs. If
+    // hasRecordId is set to true, then both a RecordId and a BSONObj document are stored in that
+    // order for every BSONArray in docs. Otherwise, the RecordId is omitted and the BSONArray will
+    // only carry a BSONObj document.
+    bool hasRecordId;
+
+    // Set when 'scanType' is 'kIxscan'.
+    BSONObj indexKeyPattern;
+};
+
 struct AndHashNode : public QuerySolutionNode {
     AndHashNode();
     virtual ~AndHashNode();
@@ -392,16 +551,14 @@ struct AndHashNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return children.back()->getSort();
+    const ProvidedSortSet& providedSorts() const {
+        return children.back()->providedSorts();
     }
 
     QuerySolutionNode* clone() const;
-
-    BSONObjSet _sort;
 };
 
-struct AndSortedNode : public QuerySolutionNode {
+struct AndSortedNode : public QuerySolutionNodeWithSortSet {
     AndSortedNode();
     virtual ~AndSortedNode();
 
@@ -416,16 +573,11 @@ struct AndSortedNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return true;
     }
-    const BSONObjSet& getSort() const {
-        return _sort;
-    }
 
     QuerySolutionNode* clone() const;
-
-    BSONObjSet _sort;
 };
 
-struct OrNode : public QuerySolutionNode {
+struct OrNode : public QuerySolutionNodeWithSortSet {
     OrNode();
     virtual ~OrNode();
 
@@ -442,18 +594,13 @@ struct OrNode : public QuerySolutionNode {
         // any order on the output.
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return _sort;
-    }
 
     QuerySolutionNode* clone() const;
 
-    BSONObjSet _sort;
-
     bool dedup;
 };
 
-struct MergeSortNode : public QuerySolutionNode {
+struct MergeSortNode : public QuerySolutionNodeWithSortSet {
     MergeSortNode();
     virtual ~MergeSortNode();
 
@@ -469,28 +616,22 @@ struct MergeSortNode : public QuerySolutionNode {
         return false;
     }
 
-    const BSONObjSet& getSort() const {
-        return _sorts;
-    }
-
     QuerySolutionNode* clone() const;
 
     virtual void computeProperties() {
         for (size_t i = 0; i < children.size(); ++i) {
             children[i]->computeProperties();
         }
-        _sorts.clear();
-        _sorts.insert(sort);
+        sortSet = ProvidedSortSet(sort, std::set<std::string>());
     }
 
-    BSONObjSet _sorts;
-
     BSONObj sort;
     bool dedup;
 };
 
 struct FetchNode : public QuerySolutionNode {
-    FetchNode();
+    FetchNode() {}
+    FetchNode(std::unique_ptr<QuerySolutionNode> child) : QuerySolutionNode(std::move(child)) {}
     virtual ~FetchNode() {}
 
     virtual StageType getType() const {
@@ -508,16 +649,14 @@ struct FetchNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return children[0]->sortedByDiskLoc();
     }
-    const BSONObjSet& getSort() const {
-        return children[0]->getSort();
+    const ProvidedSortSet& providedSorts() const {
+        return children[0]->providedSorts();
     }
 
     QuerySolutionNode* clone() const;
-
-    BSONObjSet _sorts;
 };
 
-struct IndexScanNode : public QuerySolutionNode {
+struct IndexScanNode : public QuerySolutionNodeWithSortSet {
     IndexScanNode(IndexEntry index);
     virtual ~IndexScanNode() {}
 
@@ -534,9 +673,6 @@ struct IndexScanNode : public QuerySolutionNode {
     }
     FieldAvailability getFieldAvailability(const std::string& field) const;
     bool sortedByDiskLoc() const;
-    const BSONObjSet& getSort() const {
-        return _sorts;
-    }
 
     QuerySolutionNode* clone() const;
 
@@ -550,8 +686,6 @@ struct IndexScanNode : public QuerySolutionNode {
     static std::set<StringData> getFieldsWithStringBounds(const IndexBounds& bounds,
                                                           const BSONObj& indexKeyPattern);
 
-    BSONObjSet _sorts;
-
     IndexEntry index;
 
     int direction;
@@ -592,8 +726,8 @@ struct ReturnKeyNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const final {
         return children[0]->sortedByDiskLoc();
     }
-    const BSONObjSet& getSort() const final {
-        return children[0]->getSort();
+    const ProvidedSortSet& providedSorts() const final {
+        return children[0]->providedSorts();
     }
 
     QuerySolutionNode* clone() const final;
@@ -607,12 +741,11 @@ struct ReturnKeyNode : public QuerySolutionNode {
  * is much slower than the fast-path implementations. We only really have all the information
  * available to choose a projection implementation at planning time.
  */
-struct ProjectionNode : QuerySolutionNode {
+struct ProjectionNode : public QuerySolutionNodeWithSortSet {
     ProjectionNode(std::unique_ptr<QuerySolutionNode> child,
                    const MatchExpression& fullExpression,
                    projection_ast::Projection proj)
-        : QuerySolutionNode(std::move(child)),
-          _sorts(SimpleBSONObjComparator::kInstance.makeBSONObjSet()),
+        : QuerySolutionNodeWithSortSet(std::move(child)),
           fullExpression(fullExpression),
           proj(std::move(proj)) {}
 
@@ -646,10 +779,6 @@ struct ProjectionNode : QuerySolutionNode {
         return children[0]->sortedByDiskLoc();
     }
 
-    const BSONObjSet& getSort() const {
-        return _sorts;
-    }
-
 protected:
     void cloneProjectionData(ProjectionNode* copy) const;
 
@@ -659,8 +788,6 @@ public:
      */
     virtual StringData projectionImplementationTypeToString() const = 0;
 
-    BSONObjSet _sorts;
-
     // The full query tree.  Needed when we have positional operators.
     // Owned in the CanonicalQuery, not here.
     const MatchExpression& fullExpression;
@@ -745,8 +872,8 @@ struct SortKeyGeneratorNode : public QuerySolutionNode {
         return children[0]->sortedByDiskLoc();
     }
 
-    const BSONObjSet& getSort() const final {
-        return children[0]->getSort();
+    const ProvidedSortSet& providedSorts() const final {
+        return children[0]->providedSorts();
     }
 
     QuerySolutionNode* clone() const final;
@@ -757,8 +884,8 @@ struct SortKeyGeneratorNode : public QuerySolutionNode {
     BSONObj sortSpec;
 };
 
-struct SortNode : public QuerySolutionNode {
-    SortNode() : _sorts(SimpleBSONObjComparator::kInstance.makeBSONObjSet()), limit(0) {}
+struct SortNode : public QuerySolutionNodeWithSortSet {
+    SortNode() : limit(0) {}
 
     virtual ~SortNode() {}
 
@@ -774,20 +901,13 @@ struct SortNode : public QuerySolutionNode {
         return false;
     }
 
-    const BSONObjSet& getSort() const {
-        return _sorts;
-    }
-
     virtual void computeProperties() {
         for (size_t i = 0; i < children.size(); ++i) {
             children[i]->computeProperties();
         }
-        _sorts.clear();
-        _sorts.insert(pattern);
+        sortSet = ProvidedSortSet(pattern, std::set<std::string>());
     }
 
-    BSONObjSet _sorts;
-
     BSONObj pattern;
 
     // Sum of both limit and skip count in the parsed query.
@@ -795,6 +915,11 @@ struct SortNode : public QuerySolutionNode {
 
     bool addSortKeyMetadata = false;
 
+    // The maximum number of bytes of memory we're willing to use during execution of the sort. If
+    // this limit is exceeded and we're not allowed to spill to disk, the query will fail at
+    // execution time. Otherwise, the data will be spilled to disk.
+    uint64_t maxMemoryUsageBytes = internalQueryMaxBlockingSortMemoryUsageBytes.load();
+
 protected:
     void cloneSortData(SortNode* copy) const;
 
@@ -854,8 +979,8 @@ struct LimitNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return children[0]->sortedByDiskLoc();
     }
-    const BSONObjSet& getSort() const {
-        return children[0]->getSort();
+    const ProvidedSortSet& providedSorts() const {
+        return children[0]->providedSorts();
     }
 
     QuerySolutionNode* clone() const;
@@ -881,8 +1006,8 @@ struct SkipNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return children[0]->sortedByDiskLoc();
     }
-    const BSONObjSet& getSort() const {
-        return children[0]->getSort();
+    const ProvidedSortSet& providedSorts() const {
+        return children[0]->providedSorts();
     }
 
     QuerySolutionNode* clone() const;
@@ -890,13 +1015,9 @@ struct SkipNode : public QuerySolutionNode {
     long long skip;
 };
 
-// This is a standalone stage.
-struct GeoNear2DNode : public QuerySolutionNode {
+struct GeoNear2DNode : public QuerySolutionNodeWithSortSet {
     GeoNear2DNode(IndexEntry index)
-        : _sorts(SimpleBSONObjComparator::kInstance.makeBSONObjSet()),
-          index(std::move(index)),
-          addPointMeta(false),
-          addDistMeta(false) {}
+        : index(std::move(index)), addPointMeta(false), addDistMeta(false) {}
 
     virtual ~GeoNear2DNode() {}
 
@@ -914,14 +1035,9 @@ struct GeoNear2DNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return _sorts;
-    }
 
     QuerySolutionNode* clone() const;
 
-    BSONObjSet _sorts;
-
     // Not owned here
     const GeoNearExpression* nq;
     IndexBounds baseBounds;
@@ -931,13 +1047,9 @@ struct GeoNear2DNode : public QuerySolutionNode {
     bool addDistMeta;
 };
 
-// This is actually its own standalone stage.
-struct GeoNear2DSphereNode : public QuerySolutionNode {
+struct GeoNear2DSphereNode : public QuerySolutionNodeWithSortSet {
     GeoNear2DSphereNode(IndexEntry index)
-        : _sorts(SimpleBSONObjComparator::kInstance.makeBSONObjSet()),
-          index(std::move(index)),
-          addPointMeta(false),
-          addDistMeta(false) {}
+        : index(std::move(index)), addPointMeta(false), addDistMeta(false) {}
 
     virtual ~GeoNear2DSphereNode() {}
 
@@ -955,14 +1067,9 @@ struct GeoNear2DSphereNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return _sorts;
-    }
 
     QuerySolutionNode* clone() const;
 
-    BSONObjSet _sorts;
-
     // Not owned here
     const GeoNearExpression* nq;
     IndexBounds baseBounds;
@@ -1000,8 +1107,8 @@ struct ShardingFilterNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return children[0]->sortedByDiskLoc();
     }
-    const BSONObjSet& getSort() const {
-        return children[0]->getSort();
+    const ProvidedSortSet& providedSorts() const {
+        return children[0]->providedSorts();
     }
 
     QuerySolutionNode* clone() const;
@@ -1011,9 +1118,8 @@ struct ShardingFilterNode : public QuerySolutionNode {
  * Distinct queries only want one value for a given field.  We run an index scan but
  * *always* skip over the current key to the next key.
  */
-struct DistinctNode : public QuerySolutionNode {
-    DistinctNode(IndexEntry index)
-        : sorts(SimpleBSONObjComparator::kInstance.makeBSONObjSet()), index(std::move(index)) {}
+struct DistinctNode : public QuerySolutionNodeWithSortSet {
+    DistinctNode(IndexEntry index) : index(std::move(index)) {}
 
     virtual ~DistinctNode() {}
 
@@ -1038,16 +1144,11 @@ struct DistinctNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return sorts;
-    }
 
     QuerySolutionNode* clone() const;
 
     virtual void computeProperties();
 
-    BSONObjSet sorts;
-
     IndexEntry index;
     IndexBounds bounds;
 
@@ -1062,9 +1163,8 @@ struct DistinctNode : public QuerySolutionNode {
  * Some count queries reduce to counting how many keys are between two entries in a
  * Btree.
  */
-struct CountScanNode : public QuerySolutionNode {
-    CountScanNode(IndexEntry index)
-        : sorts(SimpleBSONObjComparator::kInstance.makeBSONObjSet()), index(std::move(index)) {}
+struct CountScanNode : public QuerySolutionNodeWithSortSet {
+    CountScanNode(IndexEntry index) : index(std::move(index)) {}
 
     virtual ~CountScanNode() {}
 
@@ -1082,14 +1182,9 @@ struct CountScanNode : public QuerySolutionNode {
     bool sortedByDiskLoc() const {
         return false;
     }
-    const BSONObjSet& getSort() const {
-        return sorts;
-    }
 
     QuerySolutionNode* clone() const;
 
-    BSONObjSet sorts;
-
     IndexEntry index;
 
     BSONObj startKey;
@@ -1099,36 +1194,165 @@ struct CountScanNode : public QuerySolutionNode {
     bool endKeyInclusive;
 };
 
-/**
- * This stage drops results that are out of sorted order.
- */
-struct EnsureSortedNode : public QuerySolutionNode {
-    EnsureSortedNode() {}
-    virtual ~EnsureSortedNode() {}
+struct EofNode : public QuerySolutionNodeWithSortSet {
+    EofNode() {}
 
     virtual StageType getType() const {
-        return STAGE_ENSURE_SORTED;
+        return STAGE_EOF;
     }
 
     virtual void appendToString(str::stream* ss, int indent) const;
 
     bool fetched() const {
-        return children[0]->fetched();
+        return false;
     }
+
     FieldAvailability getFieldAvailability(const std::string& field) const {
-        return children[0]->getFieldAvailability(field);
+        return FieldAvailability::kNotProvided;
     }
+
     bool sortedByDiskLoc() const {
-        return children[0]->sortedByDiskLoc();
-    }
-    const BSONObjSet& getSort() const {
-        return children[0]->getSort();
+        return false;
     }
 
     QuerySolutionNode* clone() const;
-
-    // The pattern that the results should be sorted by.
-    BSONObj pattern;
 };
 
+struct TextOrNode : public OrNode {
+    TextOrNode() {}
+
+    StageType getType() const override {
+        return STAGE_TEXT_OR;
+    }
+
+    void appendToString(str::stream* ss, int indent) const override;
+    QuerySolutionNode* clone() const override;
+};
+
+struct TextMatchNode : public QuerySolutionNodeWithSortSet {
+    TextMatchNode(IndexEntry index, std::unique_ptr<fts::FTSQuery> ftsQuery, bool wantTextScore)
+        : index(std::move(index)), ftsQuery(std::move(ftsQuery)), wantTextScore(wantTextScore) {}
+
+    StageType getType() const override {
+        return STAGE_TEXT_MATCH;
+    }
+
+    void appendToString(str::stream* ss, int indent) const override;
+
+    // Text's return is LOC_AND_OBJ so it's fetched and has all fields.
+    bool fetched() const {
+        return true;
+    }
+    FieldAvailability getFieldAvailability(const std::string& field) const {
+        return FieldAvailability::kFullyProvided;
+    }
+    bool sortedByDiskLoc() const override {
+        return false;
+    }
+
+    QuerySolutionNode* clone() const override;
+
+    IndexEntry index;
+    std::unique_ptr<fts::FTSQuery> ftsQuery;
+
+    // The number of fields in the prefix of the text index. For example, if the key pattern is
+    //
+    //   { a: 1, b: 1, _fts: "text", _ftsx: 1, c: 1 }
+    //
+    // then the number of prefix fields is 2, because of "a" and "b".
+    size_t numPrefixFields = 0u;
+
+    // "Prefix" fields of a text index can handle equality predicates.  We group them with the
+    // text node while creating the text leaf node and convert them into a BSONObj index prefix
+    // when we finish the text leaf node.
+    BSONObj indexPrefix;
+
+    // True, if we need to compute text scores.
+    bool wantTextScore;
+};
+
+struct GroupNode : public QuerySolutionNode {
+    GroupNode(std::unique_ptr<QuerySolutionNode> child,
+              StringMap<boost::intrusive_ptr<Expression>> groupByExprs,
+              std::vector<AccumulationStatement> accs,
+              bool merging)
+        : QuerySolutionNode(std::move(child)),
+          groupByExpressions(std::move(groupByExprs)),
+          accumulators(std::move(accs)),
+          doingMerge(merging) {
+        // Use the DepsTracker to extract the fields that the 'groupByExpressions' and accumulator
+        // expressions depend on.
+        for (auto&& [field, expr] : groupByExpressions) {
+            for (auto& groupByExprField : expr->getDependencies().fields) {
+                requiredFields.insert(groupByExprField);
+            }
+        }
+        for (auto&& acc : accumulators) {
+            auto argExpr = acc.expr.argument;
+            for (auto& argExprField : argExpr->getDependencies().fields) {
+                requiredFields.insert(argExprField);
+            }
+        }
+    }
+
+    StageType getType() const override {
+        return STAGE_GROUP;
+    }
+
+    void appendToString(str::stream* ss, int indent) const override;
+
+    bool fetched() const {
+        return true;
+    }
+
+    FieldAvailability getFieldAvailability(const std::string& field) const {
+        return FieldAvailability::kFullyProvided;
+    }
+    bool sortedByDiskLoc() const override {
+        return false;
+    }
+
+    const ProvidedSortSet& providedSorts() const final {
+        return children.back()->providedSorts();
+    }
+
+    QuerySolutionNode* clone() const override;
+
+    StringMap<boost::intrusive_ptr<Expression>> groupByExpressions;
+    std::vector<AccumulationStatement> accumulators;
+    bool doingMerge;
+
+    // Carries the fields this GroupNode depends on. Namely, 'requiredFields' contains the union of
+    // the fields in the 'groupByExpressions' and the fields in the input Expressions of the
+    // 'accumulators'.
+    StringSet requiredFields;
+};
+
+struct SentinelNode : public QuerySolutionNode {
+
+    SentinelNode() {}
+
+    StageType getType() const override {
+        return STAGE_SENTINEL;
+    }
+
+    void appendToString(str::stream* ss, int indent) const override;
+
+    bool fetched() const {
+        return true;
+    }
+
+    FieldAvailability getFieldAvailability(const std::string& field) const {
+        return FieldAvailability::kFullyProvided;
+    }
+    bool sortedByDiskLoc() const override {
+        return false;
+    }
+
+    const ProvidedSortSet& providedSorts() const final {
+        return kEmptySet;
+    }
+
+    QuerySolutionNode* clone() const override;
+};
 }  // namespace mongo
