@@ -263,8 +263,12 @@ OpTime OplogFetcher::getLastOpTimeFetched_forTest() const {
     return _getLastOpTimeFetched();
 }
 
-BSONObj OplogFetcher::getFindQuery_forTest(long long findTimeout) const {
-    return _makeFindQuery(findTimeout);
+BSONObj OplogFetcher::getFindQueryFilter_forTest() const {
+    return _makeFindQueryFilter();
+}
+
+Query OplogFetcher::getFindQuerySettings_forTest(long long findTimeout) const {
+    return _makeFindQuerySettings(findTimeout);
 }
 
 Milliseconds OplogFetcher::getAwaitDataTimeout_forTest() const {
@@ -396,17 +400,11 @@ void OplogFetcher::_runQuery(const executor::TaskExecutor::CallbackArgs& callbac
         auto batchResult = _getNextBatch();
         if (!batchResult.isOK()) {
             auto brStatus = batchResult.getStatus();
-            // Determine if we should stop syncing from our current sync source. If we're going
-            // to change sync sources anyway, do it immediately rather than checking if we can
-            // retry the error.
-            const bool stopFetching = _dataReplicatorExternalState->shouldStopFetchingOnError(
-                                          _config.source, _getLastOpTimeFetched()) !=
-                ChangeSyncSourceAction::kContinueSyncing;
 
             // Recreate a cursor if we have enough retries left.
             // If we are a TenantOplogFetcher, we never retry as we will always restart the
             // TenantMigrationRecipient state machine on failure. So instead, we just fail and exit.
-            if (!stopFetching && _oplogFetcherRestartDecision->shouldContinue(this, brStatus) &&
+            if (_oplogFetcherRestartDecision->shouldContinue(this, brStatus) &&
                 !_config.forTenantMigration) {
                 hangBeforeOplogFetcherRetries.pauseWhileSet();
                 _cursor.reset();
@@ -494,9 +492,6 @@ Status OplogFetcher::_connect() {
             }
         }();
     } while (!connectStatus.isOK() &&
-             _dataReplicatorExternalState->shouldStopFetchingOnError(_config.source,
-                                                                     _getLastOpTimeFetched()) ==
-                 ChangeSyncSourceAction::kContinueSyncing &&
              _oplogFetcherRestartDecision->shouldContinue(this, connectStatus));
 
     return connectStatus;
@@ -578,11 +573,11 @@ AggregateCommandRequest OplogFetcher::_makeAggregateCommandRequest(long long max
     return aggRequest;
 }
 
-BSONObj OplogFetcher::_makeFindQuery(long long findTimeout) const {
+BSONObj OplogFetcher::_makeFindQueryFilter() const {
     BSONObjBuilder queryBob;
 
     auto lastOpTimeFetched = _getLastOpTimeFetched();
-    BSONObjBuilder filterBob(queryBob.subobjStart("query"));
+    BSONObjBuilder filterBob;
     filterBob.append("ts", BSON("$gte" << lastOpTimeFetched.getTimestamp()));
     // Handle caller-provided filter.
     if (!_config.queryFilter.isEmpty()) {
@@ -590,34 +585,34 @@ BSONObj OplogFetcher::_makeFindQuery(long long findTimeout) const {
             "$or",
             BSON_ARRAY(_config.queryFilter << BSON("ts" << lastOpTimeFetched.getTimestamp())));
     }
-    filterBob.done();
+    return filterBob.obj();
+}
 
-    queryBob.append("$maxTimeMS", findTimeout);
+Query OplogFetcher::_makeFindQuerySettings(long long findTimeout) const {
+    Query query = Query().maxTimeMS(findTimeout);
     if (_config.requestResumeToken) {
-        queryBob.append("$hint", BSON("$natural" << 1));
-        queryBob.append("$_requestResumeToken", true);
+        query.hint(BSON("$natural" << 1)).requestResumeToken(true);
     }
 
     auto lastCommittedWithCurrentTerm =
         _dataReplicatorExternalState->getCurrentTermAndLastCommittedOpTime();
     auto term = lastCommittedWithCurrentTerm.value;
     if (term != OpTime::kUninitializedTerm) {
-        queryBob.append("term", term);
+        query.term(term);
     }
 
     if (_config.queryReadConcern.isEmpty()) {
         // This ensures that the sync source waits for all earlier oplog writes to be visible.
         // Since Timestamp(0, 0) isn't allowed, Timestamp(0, 1) is the minimal we can use.
-        queryBob.append("readConcern",
-                        BSON("level"
-                             << "local"
-                             << "afterClusterTime" << Timestamp(0, 1)));
+        query.readConcern(BSON("level"
+                               << "local"
+                               << "afterClusterTime" << Timestamp(0, 1)));
     } else {
         // Caller-provided read concern.
-        queryBob.appendElements(_config.queryReadConcern.toBSON());
+        query.appendElements(_config.queryReadConcern.toBSON());
     }
 
-    return queryBob.obj();
+    return query;
 }
 
 Status OplogFetcher::_createNewCursor(bool initialFind) {
@@ -648,7 +643,8 @@ Status OplogFetcher::_createNewCursor(bool initialFind) {
         _cursor = std::make_unique<DBClientCursor>(
             _conn.get(),
             _nss,
-            _makeFindQuery(maxTimeMs),
+            _makeFindQueryFilter(),
+            _makeFindQuerySettings(maxTimeMs),
             0 /* nToReturn */,
             0 /* nToSkip */,
             nullptr /* fieldsToReturn */,
@@ -892,7 +888,7 @@ Status OplogFetcher::_onSuccessfulBatch(const Documents& documents) {
     errMsg << " previous batch last fetched optime: " << previousOpTimeFetched.toString();
     errMsg << " current batch last fetched optime: " << lastDocOpTime.toString();
 
-    if (changeSyncSourceAction == ChangeSyncSourceAction::kStopSyncingAndDropLastBatchIfPresent) {
+    if (changeSyncSourceAction == ChangeSyncSourceAction::kStopSyncingAndDropLastBatch) {
         return Status(ErrorCodes::InvalidSyncSource, errMsg);
     }
 
@@ -1049,7 +1045,7 @@ Status OplogFetcher::_checkTooStaleToSyncFromSource(const OpTime lastFetched,
         auto query = Query().sort(BSON("$natural" << 1));
         // Since this function is called after the first batch, the exhaust stream has not been
         // started yet. As a result, using the same connection is safe.
-        remoteFirstOplogEntry = _conn->findOne(_nss.ns(), query);
+        remoteFirstOplogEntry = _conn->findOne(_nss.ns(), BSONObj{}, query);
     } catch (DBException& e) {
         // If an error occurs with the query, throw an error.
         return Status(ErrorCodes::TooStaleToSyncFromSource, e.reason());
