@@ -38,6 +38,7 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/repl/repl_client_info.h"
@@ -333,7 +334,7 @@ void PrimaryOnlyService::onStepUp(const OpTime& stepUpOpTime) {
     invariant(newTerm > _term,
               str::stream() << "term " << newTerm << " is not greater than " << _term);
     _term = newTerm;
-    _setState(State::kRebuilding, lk);
+    _state = State::kRebuilding;
     _source = CancellationSource();
 
     // Install a new executor, while moving the old one into 'newThenOldScopedExecutor' so it
@@ -438,7 +439,7 @@ void PrimaryOnlyService::onStepDown() {
                         {ErrorCodes::InterruptedDueToReplStateChange,
                          "PrimaryOnlyService interrupted due to stepdown"});
 
-    _setState(State::kPaused, lk);
+    _state = State::kPaused;
     _rebuildStatus = Status::OK();
 
     _afterStepDown();
@@ -475,7 +476,7 @@ void PrimaryOnlyService::shutdown() {
         // complete.
         std::swap(savedInstances, _activeInstances);
 
-        _setState(State::kShutdown, lk);
+        _state = State::kShutdown;
         // shutdown can race with startup, so access _hasExecutor in this critical section.
         hasExecutor = _getHasExecutor();
     }
@@ -509,7 +510,8 @@ PrimaryOnlyService::getOrCreateInstance(OperationContext* opCtx, BSONObj initial
     InstanceID instanceID = idElem.wrap().getOwned();
 
     stdx::unique_lock lk(_mutex);
-    _waitForStateNotRebuilding(opCtx, lk);
+    opCtx->waitForConditionOrInterrupt(
+        _rebuildCV, lk, [this]() { return _state != State::kRebuilding; });
     if (_state == State::kRebuildFailed) {
         uassertStatusOK(_rebuildStatus);
     }
@@ -536,7 +538,8 @@ boost::optional<std::shared_ptr<PrimaryOnlyService::Instance>> PrimaryOnlyServic
               opCtx->lockState()->wasGlobalLockTakenInModeConflictingWithWrites());
 
     stdx::unique_lock lk(_mutex);
-    _waitForStateNotRebuilding(opCtx, lk);
+    opCtx->waitForConditionOrInterrupt(
+        _rebuildCV, lk, [this]() { return _state != State::kRebuilding; });
 
     if (_state == State::kShutdown || _state == State::kPaused) {
         return boost::none;
@@ -565,7 +568,8 @@ std::vector<std::shared_ptr<PrimaryOnlyService::Instance>> PrimaryOnlyService::g
     std::vector<std::shared_ptr<PrimaryOnlyService::Instance>> instances;
 
     stdx::unique_lock lk(_mutex);
-    _waitForStateNotRebuilding(opCtx, lk);
+    opCtx->waitForConditionOrInterrupt(
+        _rebuildCV, lk, [this]() { return _state != State::kRebuilding; });
 
     if (_state == State::kShutdown || _state == State::kPaused) {
         return instances;
@@ -653,7 +657,7 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
                     Status(ErrorCodes::InternalError, "Querying state documents failed"));
             }
 
-            auto cursor = client.query(ns, Query());
+            auto cursor = client.query(ns, BSONObj{});
             while (cursor->more()) {
                 stateDocuments.push_back(cursor->nextSafe().getOwned());
             }
@@ -675,11 +679,12 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
 
             stdx::lock_guard lk(_mutex);
             if (_state != State::kRebuilding || _term != term) {
-                _stateChangeCV.notify_all();
+                _rebuildCV.notify_all();
                 return;
             }
-            _setState(State::kRebuildFailed, lk);
+            _state = State::kRebuildFailed;
             _rebuildStatus = std::move(status);
+            _rebuildCV.notify_all();
             return;
         }
     }
@@ -688,7 +693,7 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
         {
             stdx::lock_guard lk(_mutex);
             if (_state != State::kRebuilding || _term != term) {  // Node stepped down
-                _stateChangeCV.notify_all();
+                _rebuildCV.notify_all();
                 return;
             }
         }
@@ -698,7 +703,7 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
     stdx::lock_guard lk(_mutex);
     if (_state != State::kRebuilding || _term != term) {
         // Node stepped down before finishing rebuilding service from previous stepUp.
-        _stateChangeCV.notify_all();
+        _rebuildCV.notify_all();
         return;
     }
     invariant(_activeInstances.empty());
@@ -721,7 +726,8 @@ void PrimaryOnlyService::_rebuildInstances(long long term) noexcept {
         [[maybe_unused]] auto newInstance =
             _insertNewInstance(lk, std::move(instance), std::move(instanceID));
     }
-    _setState(State::kRunning, lk);
+    _state = State::kRunning;
+    _rebuildCV.notify_all();
 }
 
 std::shared_ptr<PrimaryOnlyService::Instance> PrimaryOnlyService::_insertNewInstance(
@@ -771,20 +777,6 @@ StringData PrimaryOnlyService::_getStateString(WithLock) const {
     }
 }
 
-void PrimaryOnlyService::_waitForStateNotRebuilding(OperationContext* opCtx,
-                                                    BasicLockableAdapter m) {
-
-    opCtx->waitForConditionOrInterrupt(
-        _stateChangeCV, m, [this]() { return _state != State::kRebuilding; });
-}
-
-void PrimaryOnlyService::_setState(State newState, WithLock) {
-    if (std::exchange(_state, newState) != newState) {
-        _stateChangeCV.notify_all();
-    }
-}
-
-
 PrimaryOnlyService::AllowOpCtxWhenServiceRebuildingBlock::AllowOpCtxWhenServiceRebuildingBlock(
     Client* client)
     : _client(client), _clientState(&primaryOnlyServiceStateForClient(_client)) {
