@@ -360,7 +360,7 @@ ReplicationCoordinator::ApplierState ReplicationCoordinatorMock::getApplierState
     return ApplierState::Running;
 }
 
-void ReplicationCoordinatorMock::signalDrainComplete(OperationContext*, long long) {}
+void ReplicationCoordinatorMock::signalDrainComplete(OperationContext*, long long) noexcept {}
 
 void ReplicationCoordinatorMock::signalUpstreamUpdater() {}
 
@@ -376,6 +376,67 @@ ReplSetConfig ReplicationCoordinatorMock::getConfig() const {
     return _getConfigReturnValue;
 }
 
+ConnectionString ReplicationCoordinatorMock::getConfigConnectionString() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getConnectionString();
+}
+
+Milliseconds ReplicationCoordinatorMock::getConfigElectionTimeoutPeriod() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getElectionTimeoutPeriod();
+}
+
+std::vector<MemberConfig> ReplicationCoordinatorMock::getConfigVotingMembers() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.votingMembers();
+}
+
+std::int64_t ReplicationCoordinatorMock::getConfigTerm() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getConfigTerm();
+}
+
+std::int64_t ReplicationCoordinatorMock::getConfigVersion() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getConfigVersion();
+}
+
+ConfigVersionAndTerm ReplicationCoordinatorMock::getConfigVersionAndTerm() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getConfigVersionAndTerm();
+}
+
+int ReplicationCoordinatorMock::getConfigNumMembers() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getNumMembers();
+}
+
+Milliseconds ReplicationCoordinatorMock::getConfigHeartbeatTimeoutPeriodMillis() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getHeartbeatTimeoutPeriodMillis();
+}
+
+BSONObj ReplicationCoordinatorMock::getConfigBSON() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.toBSON();
+}
+
+const MemberConfig* ReplicationCoordinatorMock::findConfigMemberByHostAndPort(
+    const HostAndPort& hap) const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.findMemberByHostAndPort(hap);
+}
+
+bool ReplicationCoordinatorMock::isConfigLocalHostAllowed() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.isLocalHostAllowed();
+}
+
+Milliseconds ReplicationCoordinatorMock::getConfigHeartbeatInterval() const {
+    stdx::lock_guard<Mutex> lock(_mutex);
+    return _getConfigReturnValue.getHeartbeatInterval();
+}
+
 void ReplicationCoordinatorMock::setGetConfigReturnValue(ReplSetConfig returnValue) {
     stdx::lock_guard<Mutex> lk(_mutex);
 
@@ -515,12 +576,7 @@ ChangeSyncSourceAction ReplicationCoordinatorMock::shouldChangeSyncSource(
     const rpc::ReplSetMetadata& replMetadata,
     const rpc::OplogQueryMetadata& oqMetadata,
     const OpTime& previousOpTimeFetched,
-    const OpTime& lastOpTimeFetched) const {
-    MONGO_UNREACHABLE;
-}
-
-ChangeSyncSourceAction ReplicationCoordinatorMock::shouldChangeSyncSourceOnError(
-    const HostAndPort& currentSource, const OpTime& lastOpTimeFetched) const {
+    const OpTime& lastOpTimeFetched) {
     MONGO_UNREACHABLE;
 }
 
@@ -621,10 +677,6 @@ bool ReplicationCoordinatorMock::setContainsArbiter() const {
     return false;
 }
 
-bool ReplicationCoordinatorMock::replSetContainsNewlyAddedMembers() const {
-    return false;
-}
-
 void ReplicationCoordinatorMock::attemptToAdvanceStableTimestamp() {
     return;
 }
