@@ -183,7 +183,7 @@ public:
 
     virtual ApplierState getApplierState();
 
-    virtual void signalDrainComplete(OperationContext*, long long);
+    virtual void signalDrainComplete(OperationContext*, long long) noexcept;
 
     virtual void signalUpstreamUpdater();
 
@@ -197,6 +197,31 @@ public:
 
     virtual ReplSetConfig getConfig() const;
 
+    virtual ConnectionString getConfigConnectionString() const override;
+
+    virtual Milliseconds getConfigElectionTimeoutPeriod() const override;
+
+    virtual std::vector<MemberConfig> getConfigVotingMembers() const override;
+
+    virtual std::int64_t getConfigTerm() const override;
+
+    virtual std::int64_t getConfigVersion() const override;
+
+    virtual ConfigVersionAndTerm getConfigVersionAndTerm() const override;
+
+    virtual int getConfigNumMembers() const override;
+
+    virtual Milliseconds getConfigHeartbeatTimeoutPeriodMillis() const override;
+
+    virtual BSONObj getConfigBSON() const override;
+
+    virtual const MemberConfig* findConfigMemberByHostAndPort(
+        const HostAndPort& hap) const override;
+
+    virtual bool isConfigLocalHostAllowed() const override;
+
+    virtual Milliseconds getConfigHeartbeatInterval() const override;
+
     virtual void processReplSetGetConfig(BSONObjBuilder* result,
                                          bool commitmentStatus = false,
                                          bool includeNewlyAdded = false);
@@ -328,16 +353,14 @@ public:
 
     virtual bool setContainsArbiter() const override;
 
-    virtual bool replSetContainsNewlyAddedMembers() const override;
-
     virtual void attemptToAdvanceStableTimestamp() override;
 
     virtual void finishRecoveryIfEligible(OperationContext* opCtx) override;
 
     virtual void updateAndLogStateTransitionMetrics(
-        const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
-        const size_t numOpsKilled,
-        const size_t numOpsRunning) const override;
+        ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
+        size_t numOpsKilled,
+        size_t numOpsRunning) const override;
 
     virtual void setCanAcceptNonLocalWrites(bool canAcceptNonLocalWrites);
 
