@@ -38,7 +38,6 @@
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
-#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/apply_ops.h"
@@ -46,7 +45,6 @@
 #include "mongo/db/repl/oplog_buffer.h"
 #include "mongo/db/repl/oplog_interface_local.h"
 #include "mongo/db/repl/repl_server_parameters_gen.h"
-#include "mongo/db/repl/replica_set_aware_service.h"
 #include "mongo/db/repl/replication_consistency_markers_impl.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/transaction_oplog_application.h"
@@ -146,10 +144,13 @@ public:
             ? BSON("$gte" << _oplogApplicationStartPoint << "$lte" << *_oplogApplicationEndPoint)
             : BSON("$gte" << _oplogApplicationStartPoint);
         _cursor = _client->query(NamespaceString::kRsOplogNamespace,
-                                 QUERY("ts" << predicate),
-                                 /*batchSize*/ 0,
+                                 BSON("ts" << predicate),
+                                 /*querySettings*/ Query(),
+                                 /*limit*/ 0,
                                  /*skip*/ 0,
-                                 /*projection*/ nullptr);
+                                 /*projection*/ nullptr,
+                                 /*options*/ 0,
+                                 /*batchSize*/ 0);
 
         // Check that the first document matches our appliedThrough point then skip it since it's
         // already been applied.
@@ -215,10 +216,7 @@ public:
     void clear(OperationContext*) final {
         MONGO_UNREACHABLE;
     }
-    bool waitForDataFor(Milliseconds, Interruptible*) final {
-        MONGO_UNREACHABLE;
-    }
-    bool waitForDataUntil(Date_t, Interruptible*) final {
+    bool waitForData(Seconds) final {
         MONGO_UNREACHABLE;
     }
     boost::optional<Value> lastObjectPushed(OperationContext*) const final {
@@ -361,14 +359,10 @@ void ReplicationRecoveryImpl::recoverFromOplogAsStandalone(OperationContext* opC
 
     reconstructPreparedTransactions(opCtx, OplogApplication::Mode::kRecovering);
 
-    // Two-phase index builds are built in the background, which may still be in-progress after
-    // recovering from the oplog. To prevent crashing the server, skip enabling read-only mode.
-    if (IndexBuildsCoordinator::get(opCtx)->noIndexBuildInProgress()) {
-        LOGV2_WARNING(21558,
-                      "Setting mongod to readOnly mode as a result of specifying "
-                      "'recoverFromOplogAsStandalone'");
-        storageGlobalParams.readOnly = true;
-    }
+    LOGV2_WARNING(21558,
+                  "Setting mongod to readOnly mode as a result of specifying "
+                  "'recoverFromOplogAsStandalone'");
+    storageGlobalParams.readOnly = true;
 }
 
 void ReplicationRecoveryImpl::recoverFromOplogUpTo(OperationContext* opCtx, Timestamp endPoint) {
@@ -466,12 +460,6 @@ void ReplicationRecoveryImpl::recoverFromOplog(OperationContext* opCtx,
 
     hangAfterOplogTruncationInRollback.pauseWhileSet();
 
-    // Truncation may need to adjust the initialDataTimestamp so we let it complete first.
-    if (!isRollbackRecovery) {
-        ReplicaSetAwareServiceRegistry::get(getGlobalServiceContext())
-            .onInitialDataAvailable(opCtx, true /* isMajorityDataAvailable */);
-    }
-
     auto topOfOplogSW = _getTopOfOplog(opCtx);
     if (topOfOplogSW.getStatus() == ErrorCodes::CollectionIsEmpty ||
         topOfOplogSW.getStatus() == ErrorCodes::NamespaceNotFound) {
