@@ -62,7 +62,7 @@ public:
     StatusWith<std::unique_ptr<CollectionBulkLoader>> createCollectionForBulkLoading(
         const NamespaceString& nss,
         const CollectionOptions& options,
-        const BSONObj idIndexSpec,
+        BSONObj idIndexSpec,
         const std::vector<BSONObj>& secondaryIndexSpecs) override;
 
     Status insertDocument(OperationContext* opCtx,
@@ -82,7 +82,7 @@ public:
     Status createCollection(OperationContext* opCtx,
                             const NamespaceString& nss,
                             const CollectionOptions& options,
-                            const bool createIdIndex = true,
+                            bool createIdIndex = true,
                             const BSONObj& idIndexSpec = BSONObj()) override;
 
     Status createIndexesOnEmptyCollection(OperationContext* opCtx,
@@ -100,6 +100,7 @@ public:
 
     Status setIndexIsMultikey(OperationContext* opCtx,
                               const NamespaceString& nss,
+                              const UUID& collectionUUID,
                               const std::string& indexName,
                               const KeyStringSet& multikeyMetadataKeys,
                               const MultikeyPaths& paths,
