@@ -76,7 +76,7 @@ StatusWith<DistributionStatus> createCollectionDistributionStatus(
 
     chunkMgr.forEachChunk([&](const auto& chunkEntry) {
         ChunkType chunk;
-        chunk.setNS(nss);
+        chunk.setCollectionUUID(*chunkMgr.getUUID());
         chunk.setMin(chunkEntry.getMin());
         chunk.setMax(chunkEntry.getMax());
         chunk.setJumbo(chunkEntry.isJumbo());
@@ -294,22 +294,8 @@ StatusWith<SplitInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToSpli
     std::shuffle(collections.begin(), collections.end(), _random);
 
     for (const auto& coll : collections) {
-        if (coll.getDropped()) {
-            continue;
-        }
-
         const NamespaceString& nss(coll.getNss());
 
-        if (coll.getTimeseriesFields()) {
-            LOGV2_DEBUG(5559200,
-                        1,
-                        "Not splitting collection {namespace}; explicitly disabled.",
-                        "Not splitting explicitly disabled collection",
-                        "namespace"_attr = nss,
-                        "timeseriesFields"_attr = coll.getTimeseriesFields());
-            continue;
-        }
-
         auto candidatesStatus = _getSplitCandidatesForCollection(opCtx, nss, shardStats);
         if (candidatesStatus == ErrorCodes::NamespaceNotFound) {
             // Namespace got dropped before we managed to get to it, so just skip it
@@ -377,13 +363,9 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
     std::shuffle(collections.begin(), collections.end(), _random);
 
     for (const auto& coll : collections) {
-        if (coll.getDropped()) {
-            continue;
-        }
-
         const NamespaceString& nss(coll.getNss());
 
-        if (!coll.getAllowBalance() || !coll.getAllowMigrations() || coll.getTimeseriesFields()) {
+        if (!coll.getAllowBalance() || !coll.getAllowMigrations() || !coll.getPermitMigrations()) {
             LOGV2_DEBUG(21851,
                         1,
                         "Not balancing collection {namespace}; explicitly disabled.",
@@ -426,11 +408,9 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
 
     const auto& shardStats = shardStatsStatus.getValue();
 
-    auto collection = Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
-    if (collection.getDropped()) {
-        return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << "collection " << nss.ns() << " not found");
-    }
+    // Used to check locally if the collection exists, it should trow NamespaceNotFound if it
+    // doesn't.
+    Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss);
 
     std::set<ShardId> usedShards;
 
@@ -444,6 +424,7 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
 
 StatusWith<boost::optional<MigrateInfo>>
 BalancerChunkSelectionPolicyImpl::selectSpecificChunkToMove(OperationContext* opCtx,
+                                                            const NamespaceString& nss,
                                                             const ChunkType& chunk) {
     auto shardStatsStatus = _clusterStats->getStats(opCtx);
     if (!shardStatsStatus.isOK()) {
@@ -452,8 +433,6 @@ BalancerChunkSelectionPolicyImpl::selectSpecificChunkToMove(OperationContext* op
 
     const auto& shardStats = shardStatsStatus.getValue();
 
-    const auto& nss = chunk.getNS();
-
     auto routingInfoStatus =
         Grid::get(opCtx)->catalogCache()->getShardedCollectionRoutingInfoWithRefresh(opCtx, nss);
     if (!routingInfoStatus.isOK()) {
@@ -480,7 +459,10 @@ Status BalancerChunkSelectionPolicyImpl::checkMoveAllowed(OperationContext* opCt
         return shardStatsStatus.getStatus();
     }
 
-    const auto& nss = chunk.getNS();
+    const CollectionType collection = Grid::get(opCtx)->catalogClient()->getCollection(
+        opCtx, chunk.getCollectionUUID(), repl::ReadConcernLevel::kLocalReadConcern);
+    const auto& nss = collection.getNss();
+
 
     auto shardStats = std::move(shardStatsStatus.getValue());
 
