@@ -34,7 +34,7 @@
 #include "mongo/db/s/balancer/cluster_statistics_impl.h"
 #include "mongo/db/s/balancer/migration_test_fixture.h"
 #include "mongo/platform/random.h"
-#include "mongo/s/type_collection_timeseries_fields_gen.h"
+#include "mongo/s/type_collection_common_types_gen.h"
 
 namespace mongo {
 namespace {
@@ -150,8 +150,8 @@ TEST_F(BalancerChunkSelectionTest, TagRangesOverlap) {
                 shardTargeterMock(opCtx.get(), kShardId0)->setFindHostReturnValue(kShardHost0);
                 shardTargeterMock(opCtx.get(), kShardId1)->setFindHostReturnValue(kShardHost1);
 
-                auto migrateInfoStatus =
-                    _chunkSelectionPolicy.get()->selectSpecificChunkToMove(opCtx.get(), chunk);
+                auto migrateInfoStatus = _chunkSelectionPolicy.get()->selectSpecificChunkToMove(
+                    opCtx.get(), kNamespace, chunk);
                 ASSERT_EQUALS(ErrorCodes::RangeOverlapConflict,
                               migrateInfoStatus.getStatus().code());
             });
@@ -222,7 +222,7 @@ TEST_F(BalancerChunkSelectionTest, TagRangeMaxNotAlignedWithChunkMax) {
 
         expectGetStatsCommands(2);
         future.default_timed_get();
-        removeAllChunks(kNamespace);
+        removeAllChunks(kNamespace, collUUID);
     };
 
     assertErrorWhenMoveChunk({{kKeyPattern.globalMin(), BSON(kPattern << -5)},
