@@ -51,6 +51,15 @@ namespace {
 
 class ConfigSvrCreateDatabaseCommand final : public TypedCommand<ConfigSvrCreateDatabaseCommand> {
 public:
+    /**
+     * We accept any apiVersion, apiStrict, and/or apiDeprecationErrors forwarded with this internal
+     * command.
+     */
+    bool skipApiVersionCheck() const override {
+        // Internal command (server to server).
+        return true;
+    }
+
     using Request = ConfigsvrCreateDatabase;
     using Response = ConfigsvrCreateDatabaseResponse;
 
