@@ -37,14 +37,12 @@
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/curop.h"
-#include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/pipeline/process_interface/shardsvr_process_interface.h"
 #include "mongo/db/pipeline/sharded_agg_helpers.h"
 #include "mongo/db/s/balancer/balancer_policy.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/vector_clock.h"
-#include "mongo/logv2/log.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/grid.h"
@@ -56,7 +54,6 @@ namespace {
 
 using ChunkDistributionMap = stdx::unordered_map<ShardId, size_t>;
 using ZoneShardMap = StringMap<std::vector<ShardId>>;
-using boost::intrusive_ptr;
 
 std::vector<ShardId> getAllShardIdsSorted(OperationContext* opCtx) {
     // Many tests assume that chunks will be placed on shards
@@ -77,47 +74,12 @@ void appendChunk(const SplitPolicyParams& params,
                  const Timestamp& creationTimestamp,
                  const ShardId& shardId,
                  std::vector<ChunkType>* chunks) {
-    switch (params.configFormat) {
-        case ChunkEntryFormat::kNamespaceOnlyNoTimestamps:
-            chunks->emplace_back(params.nss,
-                                 ChunkRange(min, max),
-                                 ChunkVersion(version->majorVersion(),
-                                              version->minorVersion(),
-                                              version->epoch(),
-                                              boost::none),
-                                 shardId);
-            break;
-        case ChunkEntryFormat::kNamespaceAndUUIDNoTimestamps:
-            chunks->emplace_back(params.nss,
-                                 params.collectionUUID,
-                                 ChunkRange(min, max),
-                                 ChunkVersion(version->majorVersion(),
-                                              version->minorVersion(),
-                                              version->epoch(),
-                                              boost::none),
-                                 shardId);
-            break;
-        case ChunkEntryFormat::kNamespaceAndUUIDWithTimestamps:
-            chunks->emplace_back(params.nss,
-                                 params.collectionUUID,
-                                 ChunkRange(min, max),
-                                 ChunkVersion(version->majorVersion(),
-                                              version->minorVersion(),
-                                              version->epoch(),
-                                              creationTimestamp),
-                                 shardId);
-            break;
-        case ChunkEntryFormat::kUUIDOnlyWithTimestamps:
-            chunks->emplace_back(params.collectionUUID,
-                                 ChunkRange(min, max),
-                                 ChunkVersion(version->majorVersion(),
-                                              version->minorVersion(),
-                                              version->epoch(),
-                                              creationTimestamp),
-                                 shardId);
-            break;
-    }
-
+    chunks->emplace_back(
+        params.collectionUUID,
+        ChunkRange(min, max),
+        ChunkVersion(
+            version->majorVersion(), version->minorVersion(), version->epoch(), creationTimestamp),
+        shardId);
     auto& chunk = chunks->back();
     chunk.setHistory({ChunkHistory(creationTimestamp, shardId)});
     version->incMinor();
@@ -380,18 +342,19 @@ InitialSplitPolicy::ShardCollectionConfig UnoptimizedSplitPolicy::createFirstChu
     std::vector<ShardId> shardIds{params.primaryShardId};
 
     // Refresh the balancer settings to ensure the chunk size setting, which is sent as part of
-    // the autoSplitVector command and affects the number of chunks returned, has been loaded.
+    // the splitVector command and affects the number of chunks returned, has been loaded.
     const auto balancerConfig = Grid::get(opCtx)->getBalancerConfiguration();
     uassertStatusOK(balancerConfig->refreshAndCheck(opCtx));
+    auto optNss = CollectionCatalog::get(opCtx)->lookupNSSByUUID(opCtx, params.collectionUUID);
+    invariant(optNss);
     const auto shardSelectedSplitPoints = uassertStatusOK(
         shardutil::selectChunkSplitPoints(opCtx,
                                           params.primaryShardId,
-                                          params.nss,
+                                          *optNss,
                                           shardKeyPattern,
                                           ChunkRange(shardKeyPattern.getKeyPattern().globalMin(),
                                                      shardKeyPattern.getKeyPattern().globalMax()),
-                                          balancerConfig->getMaxChunkSizeBytes(),
-                                          0));
+                                          balancerConfig->getMaxChunkSizeBytes()));
 
     const auto currentTime = VectorClock::get(opCtx)->getTime();
     return generateShardCollectionInitialChunks(params,
@@ -470,8 +433,7 @@ InitialSplitPolicy::ShardCollectionConfig AbstractTagsBasedSplitPolicy::createFi
         invariant(it != tagToShards.end());
         uassert(50973,
                 str::stream()
-                    << "Cannot shard collection " << params.nss.ns() << " due to zone "
-                    << tag.getTag()
+                    << "Cannot shard collection " << tag.getNS() << " due to zone " << tag.getTag()
                     << " which is not assigned to a shard. Please assign this zone to a shard.",
                 !it->second.empty());
 
@@ -688,29 +650,34 @@ std::vector<BSONObj> ReshardingSplitPolicy::createRawPipeline(const ShardKeyPatt
 
     std::vector<BSONObj> res;
     const auto& shardKeyFields = shardKey.getKeyPatternFields();
+
+    BSONObjBuilder projectValBuilder;
     BSONObjBuilder sortValBuilder;
-    using Doc = Document;
-    using Arr = std::vector<Value>;
-    using V = Value;
-    Arr arrayToObjectBuilder;
+
     for (auto&& fieldRef : shardKeyFields) {
         // If the shard key includes a hashed field and current fieldRef is the hashed field.
         if (shardKey.isHashedPattern() &&
             fieldRef->dottedField().compare(shardKey.getHashedField().fieldNameStringData()) == 0) {
-            arrayToObjectBuilder.emplace_back(
-                Doc{{"k", V{fieldRef->dottedField()}},
-                    {"v", Doc{{"$toHashedIndexKey", V{"$" + fieldRef->dottedField()}}}}});
+            projectValBuilder.append(fieldRef->dottedField(),
+                                     BSON("$toHashedIndexKey"
+                                          << "$" + fieldRef->dottedField()));
         } else {
-            arrayToObjectBuilder.emplace_back(Doc{
-                {"k", V{fieldRef->dottedField()}},
-                {"v", Doc{{"$ifNull", V{Arr{V{"$" + fieldRef->dottedField()}, V{BSONNULL}}}}}}});
+            projectValBuilder.append(
+                str::stream() << fieldRef->dottedField(),
+                BSON("$ifNull" << BSON_ARRAY("$" + fieldRef->dottedField() << BSONNULL)));
         }
+
         sortValBuilder.append(fieldRef->dottedField().toString(), 1);
     }
+
+    // Do not project _id if it's not part of the shard key.
+    if (!shardKey.hasId()) {
+        projectValBuilder.append("_id", 0);
+    }
+
     res.push_back(BSON("$sample" << BSON("size" << numSplitPoints * samplesPerChunk)));
+    res.push_back(BSON("$project" << projectValBuilder.obj()));
     res.push_back(BSON("$sort" << sortValBuilder.obj()));
-    res.push_back(
-        Doc{{"$replaceWith", Doc{{"$arrayToObject", Arr{V{arrayToObjectBuilder}}}}}}.toBson());
     return res;
 }
 
@@ -833,34 +800,26 @@ void ReshardingSplitPolicy::_appendSplitPointsFromSample(BSONObjSet* splitPoints
 
     while (nextKey && nRemaining > 0) {
         // if key is hashed, nextKey values are already hashed
-        auto result = splitPoints->insert(nextKey->getOwned());
+        auto result = splitPoints->insert(
+            dotted_path_support::extractElementsBasedOnTemplate(*nextKey, shardKey.toBSON())
+                .getOwned());
+
         if (result.second) {
             nRemaining--;
         }
+
         nextKey = _samples->getNext();
     }
 }
 
-std::unique_ptr<ReshardingSplitPolicy::SampleDocumentSource>
-ReshardingSplitPolicy::makePipelineDocumentSource_forTest(OperationContext* opCtx,
-                                                          const NamespaceString& ns,
-                                                          const ShardKeyPattern& shardKey,
-                                                          int numInitialChunks,
-                                                          int samplesPerChunk) {
-    MakePipelineOptions opts;
-    opts.attachCursorSource = false;
-    return _makePipelineDocumentSource(
-        opCtx, ns, shardKey, numInitialChunks, samplesPerChunk, std::move(opts));
-}
-
 std::unique_ptr<ReshardingSplitPolicy::SampleDocumentSource>
 ReshardingSplitPolicy::_makePipelineDocumentSource(OperationContext* opCtx,
                                                    const NamespaceString& ns,
                                                    const ShardKeyPattern& shardKey,
                                                    int numInitialChunks,
-                                                   int samplesPerChunk,
-                                                   MakePipelineOptions opts) {
+                                                   int samplesPerChunk) {
     auto rawPipeline = createRawPipeline(shardKey, numInitialChunks - 1, samplesPerChunk);
+
     StringMap<ExpressionContext::ResolvedNamespace> resolvedNamespaces;
     resolvedNamespaces[ns.coll()] = {ns, std::vector<BSONObj>{}};
 
@@ -868,13 +827,13 @@ ReshardingSplitPolicy::_makePipelineDocumentSource(OperationContext* opCtx,
     // ShardServerProcessInterface instead of getting it from the generic factory so the pipeline
     // can talk to the shards.
     auto pi = std::make_shared<ShardServerProcessInterface>(
-        Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor());
+        opCtx, Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor());
 
     auto expCtx = make_intrusive<ExpressionContext>(opCtx,
                                                     boost::none, /* explain */
                                                     false,       /* fromMongos */
                                                     false,       /* needsMerge */
-                                                    true,        /* allowDiskUse */
+                                                    false,       /* allowDiskUse */
                                                     true,        /* bypassDocumentValidation */
                                                     false,       /* isMapReduceCommand */
                                                     ns,
@@ -884,10 +843,8 @@ ReshardingSplitPolicy::_makePipelineDocumentSource(OperationContext* opCtx,
                                                     std::move(resolvedNamespaces),
                                                     boost::none); /* collUUID */
 
-    expCtx->tempDir = storageGlobalParams.dbpath + "/tmp";
-
-    return std::make_unique<PipelineDocumentSource>(
-        Pipeline::makePipeline(rawPipeline, expCtx, opts), samplesPerChunk - 1);
+    return std::make_unique<PipelineDocumentSource>(Pipeline::makePipeline(rawPipeline, expCtx, {}),
+                                                    samplesPerChunk - 1);
 }
 
 ReshardingSplitPolicy::PipelineDocumentSource::PipelineDocumentSource(
