@@ -173,12 +173,6 @@ public:
                                 const NamespaceString& nss,
                                 const ChunkRange& range);
 
-    /**
-     * Exposes the zone operations mutex to external callers in order to allow them to synchronize
-     * with any changes to the zones.
-     */
-    Lock::ExclusiveLock lockZoneMutex(OperationContext* opCtx);
-
     //
     // General utilities related to the ShardingCatalogManager
     //
@@ -193,10 +187,6 @@ public:
     static void withTransaction(OperationContext* opCtx,
                                 const NamespaceString& namespaceForInitialFind,
                                 unique_function<void(OperationContext*, TxnNumber)> func);
-    static void withTransaction(OperationContext* opCtx,
-                                const NamespaceString& namespaceForInitialFind,
-                                unique_function<void(OperationContext*, TxnNumber)> func,
-                                const WriteConcernOptions& writeConcern);
 
     /**
      * Runs the write 'request' on namespace 'nss' in a transaction with 'txnNumber'. Write must be
@@ -313,8 +303,7 @@ public:
      * that doesn't pass these extra fields.
      */
     void ensureChunkVersionIsGreaterThan(OperationContext* opCtx,
-                                         const boost::optional<NamespaceString>& nss,
-                                         const boost::optional<UUID>& collUUID,
+                                         const UUID& collUUID,
                                          const BSONObj& minKey,
                                          const BSONObj& maxKey,
                                          const ChunkVersion& version);
@@ -328,11 +317,6 @@ public:
         OperationContext* opCtx,
         const NamespaceString& nss,
         unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc);
-    void bumpCollectionVersionAndChangeMetadataInTxn(
-        OperationContext* opCtx,
-        const NamespaceString& nss,
-        unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc,
-        const WriteConcernOptions& writeConcern);
 
     /**
      * Same as bumpCollectionVersionAndChangeMetadataInTxn, but bumps the version for several
@@ -342,11 +326,6 @@ public:
         OperationContext* opCtx,
         const std::vector<NamespaceString>& collNames,
         unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc);
-    void bumpMultipleCollectionVersionsAndChangeMetadataInTxn(
-        OperationContext* opCtx,
-        const std::vector<NamespaceString>& collNames,
-        unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc,
-        const WriteConcernOptions& writeConcern);
 
     /**
      * Performs a split on the chunk with min value "minKey". If the split fails, it is marked as
@@ -365,6 +344,13 @@ public:
                                            const boost::optional<UUID>& collectionUUID,
                                            bool allowMigrations);
 
+    /**
+     * Bump the minor version of the newest chunk on each shard
+     */
+    void bumpCollectionMinorVersionInTxn(OperationContext* opCtx,
+                                         const NamespaceString& nss,
+                                         TxnNumber txnNumber) const;
+
     //
     // Database Operations
     //
@@ -379,13 +365,6 @@ public:
                                 const boost::optional<ShardId>& optPrimaryShard,
                                 bool enableSharding);
 
-    /**
-     * Updates metadata in config.databases collection to show the given primary database on its
-     * new shard.
-     */
-    // TODO SERVER-54879 throw out this method once 5.0 becomes last-LTS
-    Status commitMovePrimary(OperationContext* opCtx, const StringData nss, const ShardId& toShard);
-
     //
     // Collection Operations
     //
@@ -409,9 +388,15 @@ public:
     void updateShardingCatalogEntryForCollectionInTxn(OperationContext* opCtx,
                                                       const NamespaceString& nss,
                                                       const CollectionType& coll,
-                                                      const bool upsert,
+                                                      bool upsert,
                                                       TxnNumber txnNumber);
 
+
+    void configureCollectionAutoSplit(OperationContext* opCtx,
+                                      const NamespaceString& nss,
+                                      boost::optional<int64_t> maxChunkSizeBytes,
+                                      boost::optional<bool> enableAutoSplitter);
+
     //
     // Shard Operations
     //
@@ -433,7 +418,7 @@ public:
     StatusWith<std::string> addShard(OperationContext* opCtx,
                                      const std::string* shardProposedName,
                                      const ConnectionString& shardConnectionString,
-                                     const long long maxSize);
+                                     long long maxSize);
 
     /**
      * Tries to remove a shard. To completely remove a shard from a sharded cluster,
@@ -455,40 +440,18 @@ public:
     Status setFeatureCompatibilityVersionOnShards(OperationContext* opCtx, const BSONObj& cmdObj);
 
     /**
-     * Patches-up persistent metadata for 5.0.
+     * Upgrade the persistent metadata to FCV 5.1 (phase 2).
      *
-     * It shall be called when upgrading to 5.0 or newer versions, when shards are in phase-1 of the
-     * setFCV protocol.
-     * TODO SERVER-53283: Remove once 5.0 has been released.
+     * TODO: Remove once FCV 6.0 becomes last-lts
      */
-    void upgradeMetadataFor50Phase1(OperationContext* opCtx);
+    void upgradeMetadataTo51Phase2(OperationContext* opCtx);
 
     /**
-     * Patches-up persistent metadata for 5.0.
+     * Upgrade the persistent metadata from FCV 5.1 (phase 2).
      *
-     * It shall be called when upgrading to 5.0 or newer versions, when shards are in phase-2 of the
-     * setFCV protocol.
-     * TODO SERVER-53283: Remove once 5.0 has been released.
+     * TODO: Remove once FCV 6.0 becomes last-lts
      */
-    void upgradeMetadataFor50Phase2(OperationContext* opCtx);
-
-    /**
-     * Patches-up persistent metadata for downgrade from 5.0.
-     *
-     * It shall be called when downgrading from 5.0 to an earlier version, when shards are in
-     * phase-1 of the setFCV protocol.
-     * TODO SERVER-53283: Remove once 5.0 has been released.
-     */
-    void downgradeMetadataToPre50Phase1(OperationContext* opCtx);
-
-    /**
-     * Patches-up persistent metadata for downgrade from 5.0.
-     *
-     * It shall be called when downgrading from 5.0 to an earlier version, when shards are in
-     * phase-2 of the setFCV protocol.
-     * TODO SERVER-53283: Remove once 5.0 has been released.
-     */
-    void downgradeMetadataToPre50Phase2(OperationContext* opCtx);
+    void downgradeMetadataToPre51Phase2(OperationContext* opCtx);
 
     /*
      * Rename collection metadata as part of a renameCollection operation.
@@ -517,18 +480,6 @@ public:
      */
     static void clearForTests(ServiceContext* serviceContext);
 
-    //
-    // Upgrade/downgrade
-    //
-
-    /**
-     * Upgrade the chunk metadata to include the history field.
-     */
-    void upgradeChunksHistory(OperationContext* opCtx,
-                              const NamespaceString& nss,
-                              bool force,
-                              const Timestamp validAfter);
-
 private:
     /**
      * Performs the necessary checks for version compatibility and creates a new config.version
@@ -623,9 +574,9 @@ private:
      * Retrieve the full chunk description from the config.
      */
     StatusWith<ChunkType> _findChunkOnConfig(OperationContext* opCtx,
-                                             const NamespaceStringOrUUID& nsOrUUID,
+                                             const UUID& uuid,
                                              const OID& epoch,
-                                             const boost::optional<Timestamp>& timestamp,
+                                             const Timestamp& timestamp,
                                              const BSONObj& key);
 
     /**
@@ -637,79 +588,29 @@ private:
                                                       const std::string& shardName,
                                                       const std::string& zoneName);
 
-    /*
-     * Unset the specified fields from all documents in config.chunks.
+    /**
+     * Enable the support for long collection name for each entity in 'config.collections' for
+     * which the support is unset, then force the catalog cache refresh of updated collections on
+     * each shard.
      *
-     * The removal is happening in two steps:
-     * - "Unsafe" delete updating all the documents without affecting the balancer activity.
-     * - "Safe" delete holding the chunks lock and updating documents potentially written during the
-     * previous step.
+     * This metadata change is not bound to any specific setFCV phase, so it could be safely run in
+     * phase 1 or 2.
+     *
+     * TODO: Remove once FCV 6.0 becomes last-lts
      */
-    void _unsetFieldsFromChunkDocuments(OperationContext* opCtx, const BSONObj& fields);
+    void _enableSupportForLongCollectionName(OperationContext* opCtx);
 
     /**
-     * Removes all entries from the config server's config.collections where 'dropped' is true.
+     * Disable the support for long collection name for each entity in 'config.collections' for
+     * which the support is implicitely enabled, then force the catalog cache refresh of updated
+     * collections on each shard.
      *
-     * Before 5.0, when a collection was dropped, its entry in config.collections remained, tagged
-     * as 'dropped: true'. As those are no longer needed, this method cleans up the leftover
-     * metadata.
+     * This metadata change is not bound to any specific setFCV phase, so it could be safely run in
+     * phase 1 or 2.
      *
-     * It shall be called when upgrading to 5.0 or newer versions.
-     *
-     * TODO SERVER-53283: Remove once 5.0 has becomes last-lts.
+     * TODO: Remove once FCV 6.0 becomes last-lts
      */
-    void _removePre50LegacyMetadata(OperationContext* opCtx);
-
-    /**
-     * Creates a 'version.timestamp' for each one of the entries in the config server's
-     * config.databases where it didn't already exist before.
-     *
-     * TODO SERVER-53283: Remove once 5.0 becomes last-lts.
-     */
-    void _upgradeDatabasesEntriesTo50(OperationContext* opCtx);
-
-    /**
-     * Downgrades the config.databases entries to prior 4.9 version. More specifically, it removes
-     * the 'version.timestamp' field from all the documents in config.databases.
-     *
-     * TODO SERVER-53283: Remove once 5.0 becomes last-lts.
-     */
-    void _downgradeDatabasesEntriesToPre50(OperationContext* opCtx);
-
-    /**
-     * For each one of the entries in config.collections where there is no 'timestamp':
-     * - Patches-up the entries in config.chunks to set their 'collectionUUID' and 'timestamp'
-     * fields.
-     * - Creates a 'timestamp' in its entry in config.collections.
-     * , and builds the uuid_* indexes and drops the ns_* indexes on config.chunks.
-     *
-     * TODO SERVER-53283: Remove once 5.0 becomes last-lts.
-     */
-    void _upgradeCollectionsAndChunksEntriesTo50Phase1(OperationContext* opCtx);
-
-    /**
-     * Unsets the 'ns' field from all documents in config.chunks
-     *
-     * TODO SERVER-53283: Remove once 5.0 becomes last-lts.
-     */
-    void _upgradeCollectionsAndChunksEntriesTo50Phase2(OperationContext* opCtx);
-
-    /**
-     * For each one of the entries in config.collections where there is a 'timestamp':
-     * - Patches-up the entries in config.chunks to set their 'ns' field.
-     * - Unsets the 'timestamp' field from its entry in config.collections.
-     * , and builds the ns_* indexes and drops the uuid_* indexes on config.chunks.
-     *
-     * TODO SERVER-53283: Remove once 5.0 becomes last-lts.
-     */
-    void _downgradeCollectionsAndChunksEntriesToPre50Phase1(OperationContext* opCtx);
-
-    /**
-     * Unsets the 'collectionUUID' and 'timestamp' fields from all documents in config.chunks
-     *
-     * TODO SERVER-53283: Remove once 5.0 becomes last-lts.
-     */
-    void _downgradeCollectionsAndChunksEntriesToPre50Phase2(OperationContext* opCtx);
+    void _disableSupportForLongCollectionName(OperationContext* opCtx);
 
     // The owning service context
     ServiceContext* const _serviceContext;
@@ -763,16 +664,6 @@ private:
      * taking this.
      */
     Lock::ResourceMutex _kZoneOpLock;
-
-    /**
-     * Lock for local database operations. This should be acquired when executing
-     * 'commitMovePrimary' and 'setFeatureCompatibilityVersion' commands which affect the
-     * config.databases collection. No other locks should be held when locking this. If an operation
-     * needs to take database locks (for example to write to a local collection) those locks should
-     * be taken after taking this.
-     * TODO (SERVER-53283): Remove once version 5.0 has been released.
-     */
-    Lock::ResourceMutex _kDatabaseOpLock;
 };
 
 }  // namespace mongo
