@@ -33,11 +33,11 @@
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/client/read_preference.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/s/config/config_server_test_fixture.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/config_server_test_fixture.h"
 
 namespace mongo {
 namespace {
@@ -47,8 +47,13 @@ using unittest::assertGet;
 using CommitChunkMigrate = ConfigServerTestFixture;
 
 const NamespaceString kNamespace("TestDB.TestColl");
+const KeyPattern kKeyPattern(BSON("x" << 1));
+
+TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectly) {
+    const auto collUUID = UUID::gen();
+    const auto collEpoch = OID::gen();
+    const auto collTimestamp = Timestamp(42);
 
-TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithControlChunk) {
     ShardType shard0;
     shard0.setName("shard0");
     shard0.setHost("shard0:12");
@@ -61,9 +66,10 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithControlChunk) {
 
     ChunkType migratedChunk, controlChunk;
     {
-        ChunkVersion origVersion(12, 7, OID::gen());
+        ChunkVersion origVersion(12, 7, collEpoch, collTimestamp);
 
-        migratedChunk.setNS(kNamespace);
+        migratedChunk.setName(OID::gen());
+        migratedChunk.setCollectionUUID(collUUID);
         migratedChunk.setVersion(origVersion);
         migratedChunk.setShard(shard0.getName());
         migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
@@ -72,7 +78,8 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithControlChunk) {
 
         origVersion.incMinor();
 
-        controlChunk.setNS(kNamespace);
+        controlChunk.setName(OID::gen());
+        controlChunk.setCollectionUUID(collUUID);
         controlChunk.setVersion(origVersion);
         controlChunk.setShard(shard0.getName());
         controlChunk.setHistory({ChunkHistory(Timestamp(50, 0), shard0.getName())});
@@ -81,7 +88,7 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithControlChunk) {
         controlChunk.setJumbo(true);
     }
 
-    setupChunks({migratedChunk, controlChunk});
+    setupCollection(kNamespace, kKeyPattern, {migratedChunk, controlChunk});
 
     Timestamp validAfter{101, 0};
     BSONObj versions = assertGet(ShardingCatalogManager::get(operationContext())
@@ -94,30 +101,29 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithControlChunk) {
                                                             validAfter));
 
     // Verify the versions returned match expected values.
-    auto mver = assertGet(ChunkVersion::parseWithField(versions, "migratedChunkVersion"));
-    ASSERT_EQ(ChunkVersion(migratedChunk.getVersion().majorVersion() + 1,
-                           0,
-                           migratedChunk.getVersion().epoch()),
-              mver);
-
-    auto cver = assertGet(ChunkVersion::parseWithField(versions, "controlChunkVersion"));
+    auto mver = assertGet(ChunkVersion::parseWithField(versions, "shardVersion"));
     ASSERT_EQ(ChunkVersion(migratedChunk.getVersion().majorVersion() + 1,
                            1,
-                           migratedChunk.getVersion().epoch()),
-              cver);
+                           migratedChunk.getVersion().epoch(),
+                           migratedChunk.getVersion().getTimestamp()),
+              mver);
 
-    // Verify the chunks ended up in the right shards, and versions match the values returned.
-    auto chunkDoc0 = uassertStatusOK(getChunkDoc(operationContext(), migratedChunk.getMin()));
+    // Verify that a collection version is returned
+    auto cver = assertGet(ChunkVersion::parseWithField(versions, "collectionVersion"));
+    ASSERT_TRUE(mver.isOlderOrEqualThan(cver));
+
+    // Verify the chunks ended up in the right shards.
+    auto chunkDoc0 = uassertStatusOK(
+        getChunkDoc(operationContext(), migratedChunk.getMin(), collEpoch, collTimestamp));
     ASSERT_EQ("shard1", chunkDoc0.getShard().toString());
-    ASSERT_EQ(mver, chunkDoc0.getVersion());
 
     // The migrated chunk's history should be updated.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
     ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
 
-    auto chunkDoc1 = uassertStatusOK(getChunkDoc(operationContext(), controlChunk.getMin()));
+    auto chunkDoc1 = uassertStatusOK(
+        getChunkDoc(operationContext(), controlChunk.getMin(), collEpoch, collTimestamp));
     ASSERT_EQ("shard0", chunkDoc1.getShard().toString());
-    ASSERT_EQ(cver, chunkDoc1.getVersion());
 
     // The control chunk's history and jumbo status should be unchanged.
     ASSERT_EQ(1UL, chunkDoc1.getHistory().size());
@@ -129,6 +135,9 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithControlChunk) {
 }
 
 TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
+    const auto collUUID = UUID::gen();
+    const auto collEpoch = OID::gen();
+    const auto collTimestamp = Timestamp(42);
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -141,10 +150,11 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
     setupShards({shard0, shard1});
 
     int origMajorVersion = 15;
-    auto const origVersion = ChunkVersion(origMajorVersion, 4, OID::gen());
+    auto const origVersion = ChunkVersion(origMajorVersion, 4, collEpoch, collTimestamp);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
     chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
@@ -155,13 +165,13 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
     auto chunkMax = BSON("a" << 10);
     chunk0.setMax(chunkMax);
 
-    setupChunks({chunk0});
+    setupCollection(kNamespace, kKeyPattern, {chunk0});
 
     Timestamp validAfter{101, 0};
 
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 origVersion.epoch(),
                                                                 ShardId(shard0.getName()),
@@ -172,23 +182,27 @@ TEST_F(CommitChunkMigrate, ChunksUpdatedCorrectlyWithoutControlChunk) {
 
     // Verify the version returned matches expected value.
     BSONObj versions = resultBSON.getValue();
-    auto mver = ChunkVersion::parseWithField(versions, "migratedChunkVersion");
+    auto mver = ChunkVersion::parseWithField(versions, "shardVersion");
     ASSERT_OK(mver.getStatus());
-    ASSERT_EQ(ChunkVersion(origMajorVersion + 1, 0, origVersion.epoch()), mver.getValue());
+    ASSERT_EQ(ChunkVersion(0, 0, origVersion.epoch(), origVersion.getTimestamp()), mver.getValue());
 
-    auto cver = ChunkVersion::parseWithField(versions, "controlChunkVersion");
-    ASSERT_NOT_OK(cver.getStatus());
+    // Verify that a collection version is returned
+    auto cver = assertGet(ChunkVersion::parseWithField(versions, "collectionVersion"));
+    ASSERT_EQ(ChunkVersion(origMajorVersion + 1, 0, collEpoch, collTimestamp), cver);
 
-    // Verify the chunk ended up in the right shard, and version matches the value returned.
-    auto chunkDoc0 = uassertStatusOK(getChunkDoc(operationContext(), chunkMin));
+    // Verify the chunk ended up in the right shard.
+    auto chunkDoc0 =
+        uassertStatusOK(getChunkDoc(operationContext(), chunkMin, collEpoch, collTimestamp));
     ASSERT_EQ("shard1", chunkDoc0.getShard().toString());
-    ASSERT_EQ(mver.getValue(), chunkDoc0.getVersion());
     // The history should be updated.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
     ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
 }
 
 TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
+    const auto collUUID = UUID::gen();
+    const auto collEpoch = OID::gen();
+    const auto collTimestamp = Timestamp(42);
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -201,10 +215,11 @@ TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
     setupShards({shard0, shard1});
 
     int origMajorVersion = 15;
-    auto const origVersion = ChunkVersion(origMajorVersion, 4, OID::gen());
+    auto const origVersion = ChunkVersion(origMajorVersion, 4, collEpoch, collTimestamp);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
     chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
@@ -215,14 +230,14 @@ TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
     auto chunkMax = BSON("a" << 10);
     chunk0.setMax(chunkMax);
 
-    setupChunks({chunk0});
+    setupCollection(kNamespace, kKeyPattern, {chunk0});
 
     // Make the time distance between the last history element large enough.
     Timestamp validAfter{200, 0};
 
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 origVersion.epoch(),
                                                                 ShardId(shard0.getName()),
@@ -233,23 +248,22 @@ TEST_F(CommitChunkMigrate, CheckCorrectOpsCommandNoCtlTrimHistory) {
 
     // Verify the version returned matches expected value.
     BSONObj versions = resultBSON.getValue();
-    auto mver = ChunkVersion::parseWithField(versions, "migratedChunkVersion");
+    auto mver = ChunkVersion::parseWithField(versions, "shardVersion");
     ASSERT_OK(mver.getStatus());
-    ASSERT_EQ(ChunkVersion(origMajorVersion + 1, 0, origVersion.epoch()), mver.getValue());
+    ASSERT_EQ(ChunkVersion(0, 0, origVersion.epoch(), origVersion.getTimestamp()), mver.getValue());
 
-    auto cver = ChunkVersion::parseWithField(versions, "controlChunkVersion");
-    ASSERT_NOT_OK(cver.getStatus());
-
-    // Verify the chunk ended up in the right shard, and version matches the value returned.
-    auto chunkDoc0 = uassertStatusOK(getChunkDoc(operationContext(), chunkMin));
+    // Verify the chunk ended up in the right shard.
+    auto chunkDoc0 =
+        uassertStatusOK(getChunkDoc(operationContext(), chunkMin, collEpoch, collTimestamp));
     ASSERT_EQ("shard1", chunkDoc0.getShard().toString());
-    ASSERT_EQ(mver.getValue(), chunkDoc0.getVersion());
-    // The history should be updated.
-    ASSERT_EQ(1UL, chunkDoc0.getHistory().size());
+
+    // The new history entry should be added, but the old one preserved.
+    ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
     ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
 }
 
 TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
+    const auto collUUID = UUID::gen();
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -262,10 +276,12 @@ TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
     setupShards({shard0, shard1});
 
     int origMajorVersion = 15;
-    auto const origVersion = ChunkVersion(origMajorVersion, 4, OID::gen());
+    auto const origVersion =
+        ChunkVersion(origMajorVersion, 4, OID::gen(), Timestamp(42) /* timestamp */);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
     chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
@@ -276,14 +292,14 @@ TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
     auto chunkMax = BSON("a" << 10);
     chunk0.setMax(chunkMax);
 
-    setupChunks({chunk0});
+    setupCollection(kNamespace, kKeyPattern, {chunk0});
 
     // Make the time before the last change to trigger the failure.
     Timestamp validAfter{99, 0};
 
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 origVersion.epoch(),
                                                                 ShardId(shard0.getName()),
@@ -294,6 +310,7 @@ TEST_F(CommitChunkMigrate, RejectOutOfOrderHistory) {
 }
 
 TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
+    const auto collUUID = UUID::gen();
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -306,10 +323,12 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
     setupShards({shard0, shard1});
 
     int origMajorVersion = 12;
-    auto const origVersion = ChunkVersion(origMajorVersion, 7, OID::gen());
+    auto const origVersion =
+        ChunkVersion(origMajorVersion, 7, OID::gen(), Timestamp(42) /* timestamp */);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
 
@@ -320,7 +339,8 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
     chunk0.setMax(chunkMax);
 
     ChunkType chunk1;
-    chunk1.setNS(kNamespace);
+    chunk1.setName(OID::gen());
+    chunk1.setCollectionUUID(collUUID);
     chunk1.setVersion(origVersion);
     chunk1.setShard(shard0.getName());
 
@@ -328,13 +348,13 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
     auto chunkMaxax = BSON("a" << 20);
     chunk1.setMax(chunkMaxax);
 
-    setupChunks({chunk0, chunk1});
+    setupCollection(kNamespace, kKeyPattern, {chunk0, chunk1});
 
     Timestamp validAfter{1};
 
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 OID::gen(),
                                                                 ShardId(shard0.getName()),
@@ -345,6 +365,7 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch0) {
 }
 
 TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
+    const auto collUUID = UUID::gen();
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -357,11 +378,14 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
     setupShards({shard0, shard1});
 
     int origMajorVersion = 12;
-    auto const origVersion = ChunkVersion(origMajorVersion, 7, OID::gen());
-    auto const otherVersion = ChunkVersion(origMajorVersion, 7, OID::gen());
+    auto const origVersion =
+        ChunkVersion(origMajorVersion, 7, OID::gen(), Timestamp(42) /* timestamp */);
+    auto const otherVersion =
+        ChunkVersion(origMajorVersion, 7, OID::gen(), Timestamp(42) /* timestamp */);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
 
@@ -372,7 +396,8 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
     chunk0.setMax(chunkMax);
 
     ChunkType chunk1;
-    chunk1.setNS(kNamespace);
+    chunk1.setName(OID::gen());
+    chunk1.setCollectionUUID(collUUID);
     chunk1.setVersion(otherVersion);
     chunk1.setShard(shard0.getName());
 
@@ -381,13 +406,13 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
     chunk1.setMax(chunkMaxax);
 
     // get version from the control chunk this time
-    setupChunks({chunk1, chunk0});
+    setupCollection(kNamespace, kKeyPattern, {chunk1, chunk0});
 
     Timestamp validAfter{1};
 
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 origVersion.epoch(),
                                                                 ShardId(shard0.getName()),
@@ -398,6 +423,7 @@ TEST_F(CommitChunkMigrate, RejectWrongCollectionEpoch1) {
 }
 
 TEST_F(CommitChunkMigrate, RejectChunkMissing0) {
+    const auto collUUID = UUID::gen();
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -410,10 +436,12 @@ TEST_F(CommitChunkMigrate, RejectChunkMissing0) {
     setupShards({shard0, shard1});
 
     int origMajorVersion = 12;
-    auto const origVersion = ChunkVersion(origMajorVersion, 7, OID::gen());
+    auto const origVersion =
+        ChunkVersion(origMajorVersion, 7, OID::gen(), Timestamp(42) /* timestamp */);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
 
@@ -424,7 +452,8 @@ TEST_F(CommitChunkMigrate, RejectChunkMissing0) {
     chunk0.setMax(chunkMax);
 
     ChunkType chunk1;
-    chunk1.setNS(kNamespace);
+    chunk1.setName(OID::gen());
+    chunk1.setCollectionUUID(collUUID);
     chunk1.setVersion(origVersion);
     chunk1.setShard(shard0.getName());
 
@@ -432,13 +461,13 @@ TEST_F(CommitChunkMigrate, RejectChunkMissing0) {
     auto chunkMaxax = BSON("a" << 20);
     chunk1.setMax(chunkMaxax);
 
-    setupChunks({chunk1});
+    setupCollection(kNamespace, kKeyPattern, {chunk1});
 
     Timestamp validAfter{1};
 
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 origVersion.epoch(),
                                                                 ShardId(shard0.getName()),
@@ -449,6 +478,9 @@ TEST_F(CommitChunkMigrate, RejectChunkMissing0) {
 }
 
 TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks) {
+    const auto collUUID = UUID::gen();
+    const auto collEpoch = OID::gen();
+    const auto collTimestamp = Timestamp(42);
 
     ShardType shard0;
     shard0.setName("shard0");
@@ -461,10 +493,11 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     setupShards({shard0, shard1});
 
     int origMajorVersion = 12;
-    auto const origVersion = ChunkVersion(origMajorVersion, 7, OID::gen());
+    auto const origVersion = ChunkVersion(origMajorVersion, 7, collEpoch, collTimestamp);
 
     ChunkType chunk0;
-    chunk0.setNS(kNamespace);
+    chunk0.setName(OID::gen());
+    chunk0.setCollectionUUID(collUUID);
     chunk0.setVersion(origVersion);
     chunk0.setShard(shard0.getName());
     chunk0.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
@@ -476,7 +509,8 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     chunk0.setMax(chunkMax);
 
     ChunkType chunk1;
-    chunk1.setNS(kNamespace);
+    chunk1.setName(OID::gen());
+    chunk1.setCollectionUUID(collUUID);
     chunk1.setVersion(origVersion);
     chunk1.setShard(shard1.getName());
 
@@ -487,12 +521,12 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     Timestamp ctrlChunkValidAfter = Timestamp(50, 0);
     chunk1.setHistory({ChunkHistory(ctrlChunkValidAfter, shard1.getName())});
 
-    setupChunks({chunk0, chunk1});
+    setupCollection(kNamespace, kKeyPattern, {chunk0, chunk1});
 
     Timestamp validAfter{101, 0};
     StatusWith<BSONObj> resultBSON = ShardingCatalogManager::get(operationContext())
                                          ->commitChunkMigration(operationContext(),
-                                                                chunk0.getNS(),
+                                                                kNamespace,
                                                                 chunk0,
                                                                 origVersion.epoch(),
                                                                 ShardId(shard0.getName()),
@@ -503,22 +537,21 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
 
     // Verify the versions returned match expected values.
     BSONObj versions = resultBSON.getValue();
-    auto mver = ChunkVersion::parseWithField(versions, "migratedChunkVersion");
+    auto mver = ChunkVersion::parseWithField(versions, "shardVersion");
     ASSERT_OK(mver.getStatus());
-    ASSERT_EQ(ChunkVersion(origMajorVersion + 1, 0, origVersion.epoch()), mver.getValue());
+    ASSERT_EQ(ChunkVersion(0, 0, origVersion.epoch(), origVersion.getTimestamp()), mver.getValue());
 
-    ASSERT_TRUE(versions["controlChunkVersion"].eoo());
-
-    // Verify the chunks ended up in the right shards, and versions match the values returned.
-    auto chunkDoc0 = uassertStatusOK(getChunkDoc(operationContext(), chunkMin));
+    // Verify the chunks ended up in the right shards.
+    auto chunkDoc0 =
+        uassertStatusOK(getChunkDoc(operationContext(), chunkMin, collEpoch, collTimestamp));
     ASSERT_EQ(shard1.getName(), chunkDoc0.getShard().toString());
-    ASSERT_EQ(mver.getValue(), chunkDoc0.getVersion());
 
     // The migrated chunk's history should be updated.
     ASSERT_EQ(2UL, chunkDoc0.getHistory().size());
     ASSERT_EQ(validAfter, chunkDoc0.getHistory().front().getValidAfter());
 
-    auto chunkDoc1 = uassertStatusOK(getChunkDoc(operationContext(), chunkMax));
+    auto chunkDoc1 =
+        uassertStatusOK(getChunkDoc(operationContext(), chunkMax, collEpoch, collTimestamp));
     ASSERT_EQ(shard1.getName(), chunkDoc1.getShard().toString());
     ASSERT_EQ(chunk1.getVersion(), chunkDoc1.getVersion());
 
@@ -527,5 +560,156 @@ TEST_F(CommitChunkMigrate, CommitWithLastChunkOnShardShouldNotAffectOtherChunks)
     ASSERT_EQ(ctrlChunkValidAfter, chunkDoc1.getHistory().front().getValidAfter());
 }
 
+TEST_F(CommitChunkMigrate, RejectMissingChunkVersion) {
+    const auto collUUID = UUID::gen();
+
+    ShardType shard0;
+    shard0.setName("shard0");
+    shard0.setHost("shard0:12");
+
+    ShardType shard1;
+    shard1.setName("shard1");
+    shard1.setHost("shard1:12");
+
+    setupShards({shard0, shard1});
+
+    ChunkVersion origVersion(12, 7, OID::gen(), Timestamp(42) /* timestamp */);
+
+    // Create migrate chunk with no chunk version set.
+    ChunkType migratedChunk;
+    migratedChunk.setName(OID::gen());
+    migratedChunk.setCollectionUUID(collUUID);
+    migratedChunk.setShard(shard0.getName());
+    migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    migratedChunk.setMin(BSON("a" << 1));
+    migratedChunk.setMax(BSON("a" << 10));
+
+    ChunkType currentChunk;
+    currentChunk.setName(OID::gen());
+    currentChunk.setCollectionUUID(collUUID);
+    currentChunk.setVersion(origVersion);
+    currentChunk.setShard(shard0.getName());
+    currentChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    currentChunk.setMin(BSON("a" << 1));
+    currentChunk.setMax(BSON("a" << 10));
+
+    setupCollection(kNamespace, kKeyPattern, {currentChunk});
+
+    Timestamp validAfter{101, 0};
+    ASSERT_THROWS_CODE(ShardingCatalogManager::get(operationContext())
+                           ->commitChunkMigration(operationContext(),
+                                                  kNamespace,
+                                                  migratedChunk,
+                                                  origVersion.epoch(),
+                                                  ShardId(shard0.getName()),
+                                                  ShardId(shard1.getName()),
+                                                  validAfter),
+                       DBException,
+                       4683300);
+}
+
+TEST_F(CommitChunkMigrate, RejectOlderChunkVersion) {
+    const auto collUUID = UUID::gen();
+
+    ShardType shard0;
+    shard0.setName("shard0");
+    shard0.setHost("shard0:12");
+
+    ShardType shard1;
+    shard1.setName("shard1");
+    shard1.setHost("shard1:12");
+
+    setupShards({shard0, shard1});
+
+    auto epoch = OID::gen();
+    ChunkVersion origVersion(12, 7, epoch, Timestamp(42) /* timestamp */);
+
+    ChunkType migratedChunk;
+    migratedChunk.setName(OID::gen());
+    migratedChunk.setCollectionUUID(collUUID);
+    migratedChunk.setVersion(origVersion);
+    migratedChunk.setShard(shard0.getName());
+    migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    migratedChunk.setMin(BSON("a" << 1));
+    migratedChunk.setMax(BSON("a" << 10));
+
+    ChunkVersion currentChunkVersion(14, 7, epoch, Timestamp(42) /* timestamp */);
+
+    ChunkType currentChunk;
+    currentChunk.setName(OID::gen());
+    currentChunk.setCollectionUUID(collUUID);
+    currentChunk.setVersion(currentChunkVersion);
+    currentChunk.setShard(shard0.getName());
+    currentChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    currentChunk.setMin(BSON("a" << 1));
+    currentChunk.setMax(BSON("a" << 10));
+
+    setupCollection(kNamespace, kKeyPattern, {currentChunk});
+
+    Timestamp validAfter{101, 0};
+    auto result = ShardingCatalogManager::get(operationContext())
+                      ->commitChunkMigration(operationContext(),
+                                             kNamespace,
+                                             migratedChunk,
+                                             origVersion.epoch(),
+                                             ShardId(shard0.getName()),
+                                             ShardId(shard1.getName()),
+                                             validAfter);
+
+    ASSERT_NOT_OK(result);
+    ASSERT_EQ(result, ErrorCodes::ConflictingOperationInProgress);
+}
+
+TEST_F(CommitChunkMigrate, RejectMismatchedEpoch) {
+    const auto collUUID = UUID::gen();
+
+    ShardType shard0;
+    shard0.setName("shard0");
+    shard0.setHost("shard0:12");
+
+    ShardType shard1;
+    shard1.setName("shard1");
+    shard1.setHost("shard1:12");
+
+    setupShards({shard0, shard1});
+
+    ChunkVersion origVersion(12, 7, OID::gen(), Timestamp(42) /* timestamp */);
+
+    ChunkType migratedChunk;
+    migratedChunk.setName(OID::gen());
+    migratedChunk.setCollectionUUID(collUUID);
+    migratedChunk.setVersion(origVersion);
+    migratedChunk.setShard(shard0.getName());
+    migratedChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    migratedChunk.setMin(BSON("a" << 1));
+    migratedChunk.setMax(BSON("a" << 10));
+
+    ChunkVersion currentChunkVersion(12, 7, OID::gen(), Timestamp(42) /* timestamp */);
+
+    ChunkType currentChunk;
+    currentChunk.setName(OID::gen());
+    currentChunk.setCollectionUUID(collUUID);
+    currentChunk.setVersion(currentChunkVersion);
+    currentChunk.setShard(shard0.getName());
+    currentChunk.setHistory({ChunkHistory(Timestamp(100, 0), shard0.getName())});
+    currentChunk.setMin(BSON("a" << 1));
+    currentChunk.setMax(BSON("a" << 10));
+
+    setupCollection(kNamespace, kKeyPattern, {currentChunk});
+
+    Timestamp validAfter{101, 0};
+    auto result = ShardingCatalogManager::get(operationContext())
+                      ->commitChunkMigration(operationContext(),
+                                             kNamespace,
+                                             migratedChunk,
+                                             origVersion.epoch(),
+                                             ShardId(shard0.getName()),
+                                             ShardId(shard1.getName()),
+                                             validAfter);
+
+    ASSERT_NOT_OK(result);
+    ASSERT_EQ(result, ErrorCodes::StaleEpoch);
+}
+
 }  // namespace
 }  // namespace mongo
