@@ -38,7 +38,6 @@
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands/create_gen.h"
 #include "mongo/db/commands/feature_compatibility_version.h"
-#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/persistent_task_store.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
@@ -56,7 +55,7 @@
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/cluster_write.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/request_types/shard_collection_gen.h"
+#include "mongo/s/long_collection_names_gen.h"
 
 namespace mongo {
 namespace {
@@ -196,7 +195,7 @@ bool checkIfCollectionIsEmpty(OperationContext* opCtx, const NamespaceString& ns
     // command doesn't just consult the cached metadata, which may not always be
     // correct
     DBDirectClient localClient(opCtx);
-    return localClient.findOne(nss.ns(), Query()).isEmpty();
+    return localClient.findOne(nss.ns(), BSONObj{}).isEmpty();
 }
 
 int getNumShards(OperationContext* opCtx) {
@@ -287,11 +286,6 @@ void insertChunks(OperationContext* opCtx,
             entries.push_back(chunk.toConfigBSON());
         }
         insertOp.setDocuments(entries);
-        insertOp.setWriteCommandRequestBase([] {
-            write_ops::WriteCommandRequestBase wcb;
-            wcb.setOrdered(false);
-            return wcb;
-        }());
         return insertOp;
     }());
 
@@ -319,27 +313,52 @@ void insertChunks(OperationContext* opCtx,
     }
 }
 
-void insertCollectionEntry(OperationContext* opCtx,
-                           const NamespaceString& nss,
-                           CollectionType& coll,
-                           const OperationSessionInfo& osi) {
+void updateCatalogEntry(OperationContext* opCtx,
+                        const NamespaceString& nss,
+                        CollectionType& coll,
+                        const OperationSessionInfo& osi) {
     const auto configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
 
-    BatchedCommandRequest insertRequest(
-        write_ops::InsertCommandRequest(CollectionType::ConfigNS, {coll.toBSON()}));
-    insertRequest.setWriteConcern(ShardingCatalogClient::kMajorityWriteConcern.toBSON());
+    BatchedCommandRequest updateRequest([&]() {
+        write_ops::UpdateCommandRequest updateOp(CollectionType::ConfigNS);
+        updateOp.setUpdates({[&] {
+            write_ops::UpdateOpEntry entry;
+            entry.setQ(BSON(CollectionType::kNssFieldName << nss.ns()));
+            entry.setU(write_ops::UpdateModification::parseFromClassicUpdate(coll.toBSON()));
+            entry.setUpsert(true);
+            entry.setMulti(false);
+            return entry;
+        }()});
+        return updateOp;
+    }());
 
-    const BSONObj cmdObj = insertRequest.toBSON().addFields(osi.toBSON());
+    updateRequest.setWriteConcern(ShardingCatalogClient::kMajorityWriteConcern.toBSON());
+    const BSONObj cmdObj = updateRequest.toBSON().addFields(osi.toBSON());
 
-    BatchedCommandResponse unusedResponse;
-    uassertStatusOK(Shard::CommandResponse::processBatchWriteResponse(
-        configShard->runCommand(opCtx,
-                                ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-                                CollectionType::ConfigNS.db().toString(),
-                                cmdObj,
-                                Shard::kDefaultConfigCommandTimeout,
-                                Shard::RetryPolicy::kIdempotent),
-        &unusedResponse));
+    try {
+        BatchedCommandResponse batchResponse;
+        const auto response =
+            configShard->runCommand(opCtx,
+                                    ReadPreferenceSetting{ReadPreference::PrimaryOnly},
+                                    CollectionType::ConfigNS.db().toString(),
+                                    cmdObj,
+                                    Shard::kDefaultConfigCommandTimeout,
+                                    Shard::RetryPolicy::kIdempotent);
+
+        const auto writeStatus =
+            Shard::CommandResponse::processBatchWriteResponse(response, &batchResponse);
+
+        uassertStatusOK(batchResponse.toStatus());
+        uassertStatusOK(writeStatus);
+    } catch (const DBException&) {
+        // If an error happens when contacting the config server, we don't know if the update
+        // succeeded or not, which might cause the local shard version to differ from the config
+        // server, so we clear the metadata to allow another operation to refresh it.
+        UninterruptibleLockGuard noInterrupt(opCtx->lockState());
+        AutoGetCollection autoColl(opCtx, nss, MODE_IX);
+        CollectionShardingRuntime::get(opCtx, nss)->clearFilteringMetadata(opCtx);
+        throw;
+    }
 }
 
 void broadcastDropCollection(OperationContext* opCtx,
@@ -365,7 +384,6 @@ CreateCollectionCoordinator::CreateCollectionCoordinator(ShardingDDLCoordinatorS
     : ShardingDDLCoordinator(service, initialState),
       _doc(CreateCollectionCoordinatorDocument::parse(
           IDLParserErrorContext("CreateCollectionCoordinatorDocument"), initialState)),
-      _request(_doc.getCreateCollectionRequest()),
       _critSecReason(BSON("command"
                           << "createCollection"
                           << "ns" << nss().toString() << "request"
@@ -378,12 +396,7 @@ boost::optional<BSONObj> CreateCollectionCoordinator::reportForCurrentOp(
     if (const auto& optComment = getForwardableOpMetadata().getComment()) {
         cmdBob.append(optComment.get().firstElement());
     }
-    cmdBob.appendElements(_request.toBSON());
-
-    const auto currPhase = [&]() {
-        stdx::lock_guard l{_docMutex};
-        return _doc.getPhase();
-    }();
+    cmdBob.appendElements(_doc.getCreateCollectionRequest().toBSON());
 
     BSONObjBuilder bob;
     bob.append("type", "op");
@@ -391,7 +404,7 @@ boost::optional<BSONObj> CreateCollectionCoordinator::reportForCurrentOp(
     bob.append("op", "command");
     bob.append("ns", nss().toString());
     bob.append("command", cmdBob.obj());
-    bob.append("currentPhase", currPhase);
+    bob.append("currentPhase", _doc.getPhase());
     bob.append("active", true);
     return bob.obj();
 }
@@ -399,13 +412,14 @@ boost::optional<BSONObj> CreateCollectionCoordinator::reportForCurrentOp(
 void CreateCollectionCoordinator::checkIfOptionsConflict(const BSONObj& doc) const {
     // If we have two shard collections on the same namespace, then the arguments must be the same.
     const auto otherDoc = CreateCollectionCoordinatorDocument::parse(
-        IDLParserErrorContext("RenameCollectionCoordinatorDocument"), doc);
+        IDLParserErrorContext("CreateCollectionCoordinatorDocument"), doc);
 
     uassert(ErrorCodes::ConflictingOperationInProgress,
             "Another create collection with different arguments is already running for the same "
             "namespace",
             SimpleBSONObjComparator::kInstance.evaluate(
-                _request.toBSON() == otherDoc.getCreateCollectionRequest().toBSON()));
+                _doc.getCreateCollectionRequest().toBSON() ==
+                otherDoc.getCreateCollectionRequest().toBSON()));
 }
 
 ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
@@ -413,7 +427,7 @@ ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
     const CancellationToken& token) noexcept {
     return ExecutorFuture<void>(**executor)
         .then([this, anchor = shared_from_this()] {
-            _shardKeyPattern = ShardKeyPattern(*_request.getShardKey());
+            _shardKeyPattern = ShardKeyPattern(*_doc.getShardKey());
             if (_doc.getPhase() < Phase::kCommit) {
                 auto opCtxHolder = cc().makeOperationContext();
                 auto* opCtx = opCtxHolder.get();
@@ -433,13 +447,17 @@ ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
                     // Perform a noop write on the participants in order to advance the txnNumber
                     // for this coordinator's lsid so that requests with older txnNumbers can no
                     // longer execute.
-                    //
-                    // Additionally we want to perform a majority write on the CSRS to ensure that
-                    // all the subsequent reads will see all the writes performed from a previous
-                    // execution of this coordinator.
                     _performNoopRetryableWriteOnParticipants(opCtx, **executor);
                 }
 
+                if (_recoveredFromDisk) {
+                    // If a stedown happened it could've ocurred while waiting for majority when
+                    // writing config.collections. If the refresh happens before this write is
+                    // majority committed, we will only see the data on config.chunks but not on
+                    // config.collections, so we need to serialize the refresh with the collection
+                    // creation.
+                    sharding_ddl_util::linearizeCSRSReads(opCtx);
+                }
                 // Log the start of the event only if we're not recovering.
                 _logStartCreateCollection(opCtx);
 
@@ -450,20 +468,17 @@ ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
                             opCtx,
                             nss(),
                             _shardKeyPattern->getKeyPattern().toBSON(),
-
-                            getCollation(opCtx, nss(), _request.getCollation()).second,
-                            _request.getUnique().value_or(false))) {
-                    // The collection was already created and commited but there was a stepdown
-                    // after the commit.
-
+                            getCollation(opCtx, nss(), _doc.getCollation()).second,
+                            _doc.getUnique().value_or(false))) {
+                    _result = createCollectionResponseOpt;
+                    // The collection was already created and commited but there was a
+                    // stepdown after the commit.
                     RecoverableCriticalSectionService::get(opCtx)
                         ->releaseRecoverableCriticalSection(
                             opCtx,
                             nss(),
                             _critSecReason,
                             ShardingCatalogClient::kMajorityWriteConcern);
-
-                    _result = createCollectionResponseOpt;
                     return;
                 }
 
@@ -475,7 +490,7 @@ ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
                     ->acquireRecoverableCriticalSectionBlockWrites(
                         opCtx, nss(), _critSecReason, ShardingCatalogClient::kMajorityWriteConcern);
 
-                if (!_firstExecution) {
+                if (_recoveredFromDisk) {
                     auto uuid = sharding_ddl_util::getCollectionUUID(opCtx, nss());
                     // If the collection can be found locally, then we clean up the config.chunks
                     // collection.
@@ -497,15 +512,17 @@ ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
                 _createPolicy(opCtx);
                 _createCollectionAndIndexes(opCtx);
 
-                audit::logShardCollection(
-                    opCtx->getClient(), nss().ns(), *_request.getShardKey(), *_request.getUnique());
+                audit::logShardCollection(opCtx->getClient(),
+                                          nss().ns(),
+                                          *_doc.getShardKey(),
+                                          _doc.getUnique().value_or(false));
 
                 if (_splitPolicy->isOptimized()) {
                     _createChunks(opCtx);
-
-                    // Block reads/writes from here on if we need to create the collection on other
-                    // shards, this way we prevent reads/writes that should be redirected to another
-                    // shard
+                    // Block reads/writes from here on if we need to create
+                    // the collection on other shards, this way we prevent
+                    // reads/writes that should be redirected to another
+                    // shard.
                     RecoverableCriticalSectionService::get(opCtx)
                         ->promoteRecoverableCriticalSectionToBlockAlsoReads(
                             opCtx,
@@ -535,8 +552,11 @@ ExecutorFuture<void> CreateCollectionCoordinator::_runImpl(
                 // collections.
                 if (!_splitPolicy->isOptimized()) {
                     _createChunks(opCtx);
+
                     _commit(opCtx);
                 }
+
+                _finalize(opCtx);
             }))
         .then([this] {
             auto opCtxHolder = cc().makeOperationContext();
@@ -586,11 +606,6 @@ void CreateCollectionCoordinator::_checkCommandArguments(OperationContext* opCtx
             str::stream() << "sharding not enabled for db " << nss().db(),
             dbEnabledForSharding);
 
-    uassert(ErrorCodes::InvalidNamespace,
-            str::stream() << "Namespace too long. Namespace: " << nss()
-                          << " Max: " << NamespaceString::MaxNsShardedCollectionLen,
-            nss().size() <= NamespaceString::MaxNsShardedCollectionLen);
-
     if (nss().db() == NamespaceString::kConfigDb) {
         // Only allowlisted collections in config may be sharded (unless we are in test mode)
         uassert(ErrorCodes::IllegalOperation,
@@ -602,7 +617,7 @@ void CreateCollectionCoordinator::_checkCommandArguments(OperationContext* opCtx
     uassert(ErrorCodes::InvalidOptions,
             "Hashed shard keys cannot be declared unique. It's possible to ensure uniqueness on "
             "the hashed field by declaring an additional (non-hashed) unique index on the field.",
-            !_shardKeyPattern->isHashedPattern() || !_request.getUnique().value_or(false));
+            !_shardKeyPattern->isHashedPattern() || !_doc.getUnique().value_or(false));
 
     // Ensure that a time-series collection cannot be sharded unless the feature flag is enabled.
     if (nss().isTimeseriesBucketsCollection()) {
@@ -619,20 +634,20 @@ void CreateCollectionCoordinator::_checkCommandArguments(OperationContext* opCtx
             !nss().isSystem() || nss() == NamespaceString::kLogicalSessionsNamespace ||
                 nss().isTemporaryReshardingCollection() || nss().isTimeseriesBucketsCollection());
 
-    if (_request.getNumInitialChunks()) {
+    if (_doc.getNumInitialChunks()) {
         // Ensure numInitialChunks is within valid bounds.
-        // Cannot have more than kMaxSplitPoints initial chunks per shard. Setting a maximum of
-        // 1,000,000 chunks in total to limit the amount of memory this command consumes so there is
-        // less danger of an OOM error.
+        // Cannot have more than 8192 initial chunks per shard. Setting a maximum of 1,000,000
+        // chunks in total to limit the amount of memory this command consumes so there is less
+        // danger of an OOM error.
 
         const int maxNumInitialChunksForShards =
-            Grid::get(opCtx)->shardRegistry()->getNumShardsNoReload() * shardutil::kMaxSplitPoints;
+            Grid::get(opCtx)->shardRegistry()->getNumShardsNoReload() * 8192;
         const int maxNumInitialChunksTotal = 1000 * 1000;  // Arbitrary limit to memory consumption
-        int numChunks = _request.getNumInitialChunks().value();
+        int numChunks = _doc.getNumInitialChunks().value();
         uassert(ErrorCodes::InvalidOptions,
                 str::stream() << "numInitialChunks cannot be more than either: "
-                              << maxNumInitialChunksForShards << ", " << shardutil::kMaxSplitPoints
-                              << " * number of shards; or " << maxNumInitialChunksTotal,
+                              << maxNumInitialChunksForShards << ", 8192 * number of shards; or "
+                              << maxNumInitialChunksTotal,
                 numChunks >= 0 && numChunks <= maxNumInitialChunksForShards &&
                     numChunks <= maxNumInitialChunksTotal);
     }
@@ -663,12 +678,12 @@ void CreateCollectionCoordinator::_createCollectionAndIndexes(OperationContext*
         5277903, 2, "Create collection _createCollectionAndIndexes", "namespace"_attr = nss());
 
     boost::optional<Collation> collation;
-    std::tie(collation, _collationBSON) = getCollation(opCtx, nss(), _request.getCollation());
+    std::tie(collation, _collationBSON) = getCollation(opCtx, nss(), _doc.getCollation());
 
     // We need to implicitly create a timeseries view and underlying bucket collection.
-    if (_collectionEmpty && _request.getTimeseries()) {
+    if (_collectionEmpty && _doc.getTimeseries()) {
         const auto viewName = nss().getTimeseriesViewNamespace();
-        auto createCmd = makeCreateCommand(viewName, collation, _request.getTimeseries().get());
+        auto createCmd = makeCreateCommand(viewName, collation, _doc.getTimeseries().get());
 
         BSONObj createRes;
         DBDirectClient localClient(opCtx);
@@ -685,27 +700,13 @@ void CreateCollectionCoordinator::_createCollectionAndIndexes(OperationContext*
         }
     }
 
-    auto indexCreated = false;
-    if (_doc.getImplicitlyCreateIndex().value_or(true)) {
-        indexCreated = shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
-            opCtx,
-            nss(),
-            *_shardKeyPattern,
-            _collationBSON,
-            _doc.getUnique().value_or(false),
-            _doc.getEnforceUniquenessCheck().value_or(true),
-            shardkeyutil::ValidationBehaviorsShardCollection(opCtx));
-    } else {
-        uassert(6373200,
-                "Must have an index compatible with the proposed shard key",
-                validShardKeyIndexExists(opCtx,
-                                         nss(),
-                                         *_shardKeyPattern,
-                                         _collationBSON,
-                                         _doc.getUnique().value_or(false) &&
-                                             _doc.getEnforceUniquenessCheck().value_or(true),
-                                         shardkeyutil::ValidationBehaviorsShardCollection(opCtx)));
-    }
+    const auto indexCreated = shardkeyutil::validateShardKeyIndexExistsOrCreateIfPossible(
+        opCtx,
+        nss(),
+        *_shardKeyPattern,
+        _collationBSON,
+        _doc.getUnique().value_or(false),
+        shardkeyutil::ValidationBehaviorsShardCollection(opCtx));
 
     auto replClientInfo = repl::ReplClientInfo::forClient(opCtx->getClient());
 
@@ -731,9 +732,9 @@ void CreateCollectionCoordinator::_createPolicy(OperationContext* opCtx) {
     _splitPolicy = InitialSplitPolicy::calculateOptimizationStrategy(
         opCtx,
         *_shardKeyPattern,
-        _request.getNumInitialChunks() ? *_request.getNumInitialChunks() : 0,
-        _request.getPresplitHashedZones() ? *_request.getPresplitHashedZones() : false,
-        _request.getInitialSplitPoints(),
+        _doc.getNumInitialChunks() ? *_doc.getNumInitialChunks() : 0,
+        _doc.getPresplitHashedZones() ? *_doc.getPresplitHashedZones() : false,
+        _doc.getInitialSplitPoints(),
         getTagsAndValidate(opCtx, nss(), _shardKeyPattern->toBSON()),
         getNumShards(opCtx),
         *_collectionEmpty);
@@ -743,17 +744,12 @@ void CreateCollectionCoordinator::_createChunks(OperationContext* opCtx) {
     LOGV2_DEBUG(5277904, 2, "Create collection _createChunks", "namespace"_attr = nss());
 
     _initialChunks = _splitPolicy->createFirstChunks(
-        opCtx,
-        *_shardKeyPattern,
-        {nss(),
-         *_collectionUUID,
-         ShardingState::get(opCtx)->shardId(),
-         ChunkEntryFormat::getForVersionCallerGuaranteesFCVStability(
-             ServerGlobalParams::FeatureCompatibility::Version::kVersion50)});
+        opCtx, *_shardKeyPattern, {*_collectionUUID, ShardingState::get(opCtx)->shardId()});
 
     // There must be at least one chunk.
-    invariant(_initialChunks);
-    invariant(!_initialChunks->chunks.empty());
+    invariant(!_initialChunks.chunks.empty());
+
+    _numChunks = _initialChunks.chunks.size();
 }
 
 void CreateCollectionCoordinator::_createCollectionOnNonPrimaryShards(
@@ -770,7 +766,7 @@ void CreateCollectionCoordinator::_createCollectionOnNonPrimaryShards(
     NamespaceStringOrUUID nssOrUUID{nss().db().toString(), *_collectionUUID};
     auto [collOptions, indexes, idIndex] = getCollectionOptionsAndIndexes(opCtx, nssOrUUID);
 
-    for (const auto& chunk : _initialChunks->chunks) {
+    for (const auto& chunk : _initialChunks.chunks) {
         const auto& chunkShardId = chunk.getShard();
         if (chunkShardId == dbPrimaryShardId ||
             initializedShards.find(chunkShardId) != initializedShards.end()) {
@@ -824,19 +820,36 @@ void CreateCollectionCoordinator::_commit(OperationContext* opCtx) {
 
     // Upsert Chunks.
     _doc = _updateSession(opCtx, _doc);
-    insertChunks(opCtx, _initialChunks->chunks, getCurrentSession(_doc));
+    insertChunks(opCtx, _initialChunks.chunks, getCurrentSession(_doc));
 
     CollectionType coll(nss(),
-                        _initialChunks->collVersion().epoch(),
-                        _initialChunks->collVersion().getTimestamp(),
+                        _initialChunks.collVersion().epoch(),
+                        _initialChunks.collVersion().getTimestamp(),
                         Date_t::now(),
                         *_collectionUUID);
 
     coll.setKeyPattern(_shardKeyPattern->getKeyPattern());
 
-    if (_request.getTimeseries()) {
+    // Prevent the FCV from changing before committing the new collection to the config server.
+    // This ensures that the 'supportingLongName' field is properly set (and committed) based on
+    // the current shard's FCV.
+    //
+    // TODO: Remove once FCV 6.0 becomes last-lts
+    std::shared_ptr<FixedFCVRegion> currentFCV;
+
+    // TODO: Remove condition once FCV 6.0 becomes last-lts
+    if (feature_flags::gFeatureFlagLongCollectionNames.isEnabledAndIgnoreFCV()) {
+        currentFCV = std::make_shared<FixedFCVRegion>(opCtx);
+        if ((*currentFCV)
+                ->isGreaterThanOrEqualTo(
+                    multiversion::FeatureCompatibilityVersion::kUpgradingFrom_5_0_To_5_1)) {
+            coll.setSupportingLongName(SupportingLongNameStatusEnum::kImplicitlyEnabled);
+        }
+    }
+
+    if (_doc.getCreateCollectionRequest().getTimeseries()) {
         TypeCollectionTimeseriesFields timeseriesFields;
-        timeseriesFields.setTimeseriesOptions(*_request.getTimeseries());
+        timeseriesFields.setTimeseriesOptions(*_doc.getCreateCollectionRequest().getTimeseries());
         coll.setTimeseriesFields(std::move(timeseriesFields));
     }
 
@@ -844,31 +857,25 @@ void CreateCollectionCoordinator::_commit(OperationContext* opCtx) {
         coll.setDefaultCollation(_collationBSON.value());
     }
 
-    if (_request.getUnique()) {
-        coll.setUnique(*_request.getUnique());
+    if (_doc.getUnique()) {
+        coll.setUnique(*_doc.getUnique());
     }
 
     _doc = _updateSession(opCtx, _doc);
+    updateCatalogEntry(opCtx, nss(), coll, getCurrentSession(_doc));
+}
+
+void CreateCollectionCoordinator::_finalize(OperationContext* opCtx) {
+    LOGV2_DEBUG(5277907, 2, "Create collection _finalize", "namespace"_attr = nss());
 
     try {
-        insertCollectionEntry(opCtx, nss(), coll, getCurrentSession(_doc));
-
-        LOGV2_DEBUG(5277907, 2, "Collection successfully committed", "namespace"_attr = nss());
-
         forceShardFilteringMetadataRefresh(opCtx, nss());
-    } catch (const DBException& ex) {
-        LOGV2(5277908,
-              "Failed to obtain collection's shard version, so it will be recovered",
-              "namespace"_attr = nss(),
-              "error"_attr = redact(ex));
-
+    } catch (const DBException&) {
         // If the refresh fails, then set the shard version to UNKNOWN and let a future operation to
         // refresh the metadata.
         UninterruptibleLockGuard noInterrupt(opCtx->lockState());
         AutoGetCollection autoColl(opCtx, nss(), MODE_IX);
         CollectionShardingRuntime::get(opCtx, nss())->clearFilteringMetadata(opCtx);
-
-        throw;
     }
 
     // Best effort refresh to warm up cache of all involved shards so we can have a cluster ready to
@@ -877,9 +884,8 @@ void CreateCollectionCoordinator::_commit(OperationContext* opCtx) {
     auto dbPrimaryShardId = ShardingState::get(opCtx)->shardId();
 
     std::set<ShardId> shardsRefreshed;
-    for (const auto& chunk : _initialChunks->chunks) {
+    for (const auto& chunk : _initialChunks.chunks) {
         const auto& chunkShardId = chunk.getShard();
-
         if (chunkShardId == dbPrimaryShardId ||
             shardsRefreshed.find(chunkShardId) != shardsRefreshed.end()) {
             continue;
@@ -897,10 +903,11 @@ void CreateCollectionCoordinator::_commit(OperationContext* opCtx) {
     LOGV2(5277901,
           "Created initial chunk(s)",
           "namespace"_attr = nss(),
-          "numInitialChunks"_attr = _initialChunks->chunks.size(),
-          "initialCollectionVersion"_attr = _initialChunks->collVersion());
+          "numInitialChunks"_attr = _initialChunks.chunks.size(),
+          "initialCollectionVersion"_attr = _initialChunks.collVersion());
 
-    auto result = CreateCollectionResponse(_initialChunks->chunks.back().getVersion());
+    auto result = CreateCollectionResponse(
+        _initialChunks.chunks[_initialChunks.chunks.size() - 1].getVersion());
     result.setCollectionUUID(_collectionUUID);
     _result = std::move(result);
 
@@ -913,7 +920,7 @@ void CreateCollectionCoordinator::_commit(OperationContext* opCtx) {
 
 void CreateCollectionCoordinator::_logStartCreateCollection(OperationContext* opCtx) {
     BSONObjBuilder collectionDetail;
-    collectionDetail.append("shardKey", *_request.getShardKey());
+    collectionDetail.append("shardKey", *_doc.getCreateCollectionRequest().getShardKey());
     collectionDetail.append("collection", nss().ns());
     collectionDetail.append("primary", ShardingState::get(opCtx)->shardId().toString());
     ShardingLogging::get(opCtx)->logChange(
@@ -926,9 +933,8 @@ void CreateCollectionCoordinator::_logEndCreateCollection(OperationContext* opCt
     collectionDetail.append("version", _result->getCollectionVersion().toString());
     if (_collectionEmpty)
         collectionDetail.append("empty", *_collectionEmpty);
-    if (_initialChunks)
-        collectionDetail.appendNumber("numChunks",
-                                      static_cast<long long>(_initialChunks->chunks.size()));
+    if (_numChunks)
+        collectionDetail.appendNumber("numChunks", static_cast<long long>(*_numChunks));
     ShardingLogging::get(opCtx)->logChange(
         opCtx, "shardCollection.end", nss().ns(), collectionDetail.obj());
 }
@@ -961,15 +967,10 @@ void CreateCollectionCoordinator::_enterPhase(Phase newPhase) {
                 "oldPhase"_attr = CreateCollectionCoordinatorPhase_serializer(_doc.getPhase()));
 
     if (_doc.getPhase() == Phase::kUnset) {
-        newDoc = _insertStateDocument(std::move(newDoc));
-    } else {
-        newDoc = _updateStateDocument(cc().makeOperationContext().get(), std::move(newDoc));
-    }
-
-    {
-        stdx::unique_lock ul{_docMutex};
-        _doc = std::move(newDoc);
+        _doc = _insertStateDocument(std::move(newDoc));
+        return;
     }
+    _doc = _updateStateDocument(cc().makeOperationContext().get(), std::move(newDoc));
 }
 
 }  // namespace mongo
