@@ -38,6 +38,7 @@
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/op_observer.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/repl_client_info.h"
@@ -55,6 +56,11 @@ class FlushReshardingStateChangeCmd final : public TypedCommand<FlushReshardingS
 public:
     using Request = _flushReshardingStateChange;
 
+    bool skipApiVersionCheck() const override {
+        // Internal command (server to server).
+        return true;
+    }
+
     std::string help() const override {
         return "Internal command used by the resharding coordinator to flush state changes to the "
                "participant shards while the critical section is active.";
