@@ -56,6 +56,11 @@ namespace {
 template <typename Derived>
 class FlushRoutingTableCacheUpdatesCmdBase : public TypedCommand<Derived> {
 public:
+    bool skipApiVersionCheck() const override {
+        // Internal command (server to server).
+        return true;
+    }
+
     std::string help() const override {
         return "Internal command which waits for any pending routing table cache updates for a "
                "particular namespace to be written locally. The operationTime returned in the "
@@ -99,20 +104,17 @@ public:
             uassertStatusOK(shardingState->canAcceptShardedCommands());
 
             uassert(ErrorCodes::IllegalOperation,
-                    str::stream() << "Can't issue " << Derived::Request::kCommandName
-                                  << " from 'eval'",
+                    "Can't issue _flushRoutingTableCacheUpdates from 'eval'",
                     !opCtx->getClient()->isInDirectClient());
 
             uassert(ErrorCodes::IllegalOperation,
-                    str::stream() << "Can't call " << Derived::Request::kCommandName
-                                  << " if in read-only mode",
+                    "Can't call _flushRoutingTableCacheUpdates if in read-only mode",
                     !storageGlobalParams.readOnly);
 
             auto& oss = OperationShardingState::get(opCtx);
 
             {
-                AutoGetCollection autoColl(
-                    opCtx, ns(), MODE_IS, AutoGetCollectionViewMode::kViewsPermitted);
+                AutoGetCollection autoColl(opCtx, ns(), MODE_IS);
 
                 // If the primary is in the critical section, secondaries must wait for the commit
                 // to finish on the primary in case a secondary's caller has an afterClusterTime
@@ -148,7 +150,7 @@ public:
 class FlushRoutingTableCacheUpdatesCmd
     : public FlushRoutingTableCacheUpdatesCmdBase<FlushRoutingTableCacheUpdatesCmd> {
 public:
-    using Request = FlushRoutingTableCacheUpdates;
+    using Request = _flushRoutingTableCacheUpdates;
 
     static bool supportsWriteConcern() {
         return false;
@@ -160,7 +162,7 @@ class FlushRoutingTableCacheUpdatesCmdWithWriteConcern
     : public FlushRoutingTableCacheUpdatesCmdBase<
           FlushRoutingTableCacheUpdatesCmdWithWriteConcern> {
 public:
-    using Request = FlushRoutingTableCacheUpdatesWithWriteConcern;
+    using Request = _flushRoutingTableCacheUpdatesWithWriteConcern;
 
     static bool supportsWriteConcern() {
         return true;
