@@ -85,15 +85,15 @@ protected:
             nullptr,
             false,
             epoch,
-            boost::none /* timestamp */,
+            Timestamp(1, 1),
             boost::none /* timeseriesFields */,
             boost::none,
+            boost::none /* chunkSizeBytes */,
             true,
-            {ChunkType{
-                kNss, range, ChunkVersion(1, 0, epoch, boost::none /* timestamp */), kOtherShard}});
+            {ChunkType{uuid, range, ChunkVersion(1, 0, epoch, Timestamp(1, 1)), kOtherShard}});
 
         return CollectionMetadata(ChunkManager(kThisShard,
-                                               DatabaseVersion(UUID::gen()),
+                                               DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
                                                makeStandaloneRoutingTableHistory(std::move(rt)),
                                                boost::none),
                                   kThisShard);
@@ -125,18 +125,24 @@ protected:
 
         if (SimpleBSONObjComparator::kInstance.evaluate(chunkToSplit.getMin() < minKey)) {
             chunkVersion.incMajor();
-            splitChunks.emplace_back(
-                kNss, ChunkRange(chunkToSplit.getMin(), minKey), chunkVersion, kOtherShard);
+            splitChunks.emplace_back(*collMetadata.getUUID(),
+                                     ChunkRange(chunkToSplit.getMin(), minKey),
+                                     chunkVersion,
+                                     kOtherShard);
         }
 
         chunkVersion.incMajor();
-        splitChunks.emplace_back(kNss, ChunkRange(minKey, maxKey), chunkVersion, kThisShard);
+        splitChunks.emplace_back(
+            *collMetadata.getUUID(), ChunkRange(minKey, maxKey), chunkVersion, kThisShard);
 
         chunkVersion.incMajor();
-        splitChunks.emplace_back(
-            kNss, ChunkRange(maxKey, chunkToSplit.getMax()), chunkVersion, kOtherShard);
+        splitChunks.emplace_back(*collMetadata.getUUID(),
+                                 ChunkRange(maxKey, chunkToSplit.getMax()),
+                                 chunkVersion,
+                                 kOtherShard);
 
-        auto rt = cm->getRoutingTableHistory_ForTest().makeUpdated(boost::none, true, splitChunks);
+        auto rt = cm->getRoutingTableHistory_ForTest().makeUpdated(
+            boost::none /* timeseriesFields */, boost::none, boost::none, true, splitChunks);
 
         return CollectionMetadata(ChunkManager(cm->dbPrimary(),
                                                cm->dbVersion(),
@@ -161,9 +167,12 @@ protected:
         chunkVersion.incMajor();
 
         auto rt = cm->getRoutingTableHistory_ForTest().makeUpdated(
+            boost::none /* timeseriesFields */,
+            boost::none,
             boost::none,
             true,
-            {ChunkType(kNss, ChunkRange(minKey, maxKey), chunkVersion, kOtherShard)});
+            {ChunkType(
+                *metadata.getUUID(), ChunkRange(minKey, maxKey), chunkVersion, kOtherShard)});
 
         return CollectionMetadata(ChunkManager(cm->dbPrimary(),
                                                cm->dbVersion(),
