@@ -29,10 +29,8 @@
 
 #pragma once
 
-#include <deque>
 #include <list>
 #include <memory>
-#include <mutex>
 #include <set>
 
 #include "mongo/bson/bsonobj.h"
@@ -58,19 +56,22 @@ class CollectionPtr;
 class Database;
 class RecordId;
 
+// Overhead to prevent mods buffers from being too large
+const long long kFixedCommandOverhead = 32 * 1024;
+
 /**
  * Used to commit work for LogOpForSharding. Used to keep track of changes in documents that are
  * part of a chunk being migrated.
  */
 class LogTransactionOperationsForShardingHandler final : public RecoveryUnit::Change {
 public:
-    LogTransactionOperationsForShardingHandler(ServiceContext* svcCtx,
-                                               const std::vector<repl::OplogEntry>& stmts,
-                                               repl::OpTime prepareOrCommitOpTime);
-
+    /**
+     * Invariant: idObj should belong to a document that is part of the active chunk being migrated
+     */
     LogTransactionOperationsForShardingHandler(ServiceContext* svcCtx,
                                                const std::vector<repl::ReplOperation>& stmts,
-                                               repl::OpTime prepareOrCommitOpTime);
+                                               const repl::OpTime& prepareOrCommitOpTime)
+        : _svcCtx(svcCtx), _stmts(stmts), _prepareOrCommitOpTime(prepareOrCommitOpTime) {}
 
     void commit(boost::optional<Timestamp>) override;
 
@@ -78,8 +79,6 @@ public:
 
 private:
     ServiceContext* _svcCtx;
-    // Use to keep BSON obj alive for the lifetime of this object.
-    std::vector<BSONObj> _ownedReplBSONObj;
     std::vector<repl::ReplOperation> _stmts;
     const repl::OpTime _prepareOrCommitOpTime;
 };
@@ -210,168 +209,6 @@ private:
     friend class LogOpForShardingHandler;
     friend class LogTransactionOperationsForShardingHandler;
 
-    using RecordIdSet = std::set<RecordId>;
-
-    /**
-     * This is responsible for all the logic revolving around handling documents that needs to be
-     * cloned.
-     *
-     * This class is multithread-safe.
-     */
-    class CloneList {
-    public:
-        /**
-         * Simple container that increments the given counter when this is constructed and
-         * decrements it when it is destroyed. User of this class is responsible for holding
-         * necessary mutexes when counter is being modified.
-         */
-        class InProgressReadToken {
-        public:
-            InProgressReadToken(WithLock, CloneList& cloneList);
-            InProgressReadToken(const InProgressReadToken&) = delete;
-            InProgressReadToken(InProgressReadToken&&) = default;
-
-            ~InProgressReadToken();
-
-        private:
-            CloneList& _cloneList;
-        };
-
-        /**
-         * Container for a document that can be added to the nextCloneBatch call. As long as
-         * instances of this object exist, it will prevent getNextDoc from prematurely returning
-         * an empty response (which means there are no more docs left to clone).
-         *
-         * This assumes that _mutex is not being held when it is destroyed.
-         */
-        class DocumentInFlightWhileNotInLock {
-        public:
-            DocumentInFlightWhileNotInLock(std::unique_ptr<InProgressReadToken> inProgressReadToken,
-                                           boost::optional<Snapshotted<BSONObj>> doc);
-            DocumentInFlightWhileNotInLock(const DocumentInFlightWhileNotInLock&) = delete;
-            DocumentInFlightWhileNotInLock(DocumentInFlightWhileNotInLock&&) = default;
-
-            void setDoc(boost::optional<Snapshotted<BSONObj>> doc);
-            const boost::optional<Snapshotted<BSONObj>>& getDoc();
-
-        private:
-            std::unique_ptr<InProgressReadToken> _inProgressReadToken;
-            boost::optional<Snapshotted<BSONObj>> _doc;
-        };
-
-        /**
-         * A variant of the DocumentInFlightWhileNotInLock where the _mutex should be held while it
-         * has a document contained within it.
-         */
-        class DocumentInFlightWithLock {
-        public:
-            DocumentInFlightWithLock(WithLock, CloneList& clonerList);
-            DocumentInFlightWithLock(const DocumentInFlightWithLock&) = delete;
-            DocumentInFlightWithLock(DocumentInFlightWithLock&&) = default;
-
-            void setDoc(boost::optional<Snapshotted<BSONObj>> doc);
-
-            /**
-             * Releases the contained document. Can only be called once for the entire lifetime
-             * of this object.
-             */
-            std::unique_ptr<DocumentInFlightWhileNotInLock> release();
-
-        private:
-            std::unique_ptr<InProgressReadToken> _inProgressReadToken;
-            boost::optional<Snapshotted<BSONObj>> _doc;
-        };
-
-        CloneList();
-
-        /**
-         * Overwrites the list of record ids to clone.
-         */
-        void populateList(RecordIdSet recordIds);
-
-        /**
-         * Returns a document to clone. If there are no more documents left to clone,
-         * DocumentInFlightWhileNotInLock::getDoc will return boost::none.
-         *
-         * numRecordsNoLonger exists is an optional parameter that can be used to track
-         * the number of recordIds encountered that refers to a document that no longer
-         * exists.
-         */
-        std::unique_ptr<DocumentInFlightWhileNotInLock> getNextDoc(OperationContext* opCtx,
-                                                                   const CollectionPtr& collection,
-                                                                   int* numRecordsNoLongerExist);
-
-        /**
-         * Put back a document previously obtained from this CloneList instance to the overflow
-         * pool.
-         */
-        void insertOverflowDoc(Snapshotted<BSONObj> doc);
-
-        /**
-         * Returns true if there are more documents to clone.
-         */
-        bool hasMore() const;
-
-        /**
-         * Returns the size of the populated record ids.
-         */
-        size_t size() const;
-
-    private:
-        /**
-         * Increments the counter for inProgressReads.
-         */
-        void _startedOneInProgressRead(WithLock);
-
-        /**
-         * Decrements the counter for inProgressReads.
-         */
-        void _finishedOneInProgressRead();
-
-        mutable Mutex _mutex = MONGO_MAKE_LATCH("MigrationChunkClonerSource::CloneList::_mutex");
-
-        RecordIdSet _recordIds;
-
-        // This iterator is a pointer into the _recordIds set.  It allows concurrent access to
-        // the _recordIds set by allowing threads servicing _migrateClone requests to do the
-        // following:
-        //   1.  Acquire mutex "_mutex" above.
-        //   2.  Copy *_recordIdsIter into its local stack frame.
-        //   3.  Increment _recordIdsIter
-        //   4.  Unlock "_mutex."
-        //   5.  Do the I/O to fetch the document corresponding to this record Id.
-        //
-        // The purpose of this algorithm, is to allow different threads to concurrently start I/O
-        // jobs in order to more fully saturate the disk.
-        //
-        // One issue with this algorithm, is that only 16MB worth of documents can be returned in
-        // response to a _migrateClone request.  But, the thread does not know the size of a
-        // document until it does the I/O.  At which point, if the document does not fit in the
-        // response to _migrateClone request the document must be made available to a different
-        // thread servicing a _migrateClone request. To solve this problem, the thread adds the
-        // document to the below _overflowDocs deque.
-        RecordIdSet::iterator _recordIdsIter;
-
-        // This deque stores all documents that must be sent to the destination, but could not fit
-        // in the response to a particular _migrateClone request.
-        std::deque<Snapshotted<BSONObj>> _overflowDocs;
-
-        // This integer represents how many documents are being "held" by threads servicing
-        // _migrateClone requests. Any document that is "held" by a thread may be added to the
-        // _overflowDocs deque if it doesn't fit in the response to a _migrateClone request.
-        // This integer is necessary because it gives us a condition on when all documents to be
-        // sent to the destination have been exhausted.
-        //
-        // If (_recordIdsIter == _recordIds.end() && _overflowDocs.empty() &&
-        //     _inProgressReads == 0) then all documents have been returned to the destination.
-        RecordIdSet::size_type _inProgressReads = 0;
-
-        // This condition variable allows us to wait on the following condition:
-        //   Either we're done and the above condition is satisfied, or there is some document to
-        //   return.
-        stdx::condition_variable _moreDocsCV;
-    };
-
     // Represents the states in which the cloner can be
     enum State { kNew, kCloning, kDone };
 
@@ -427,7 +264,7 @@ private:
      * corresponding operation track request from the operation track requests queue.
      */
     void _addToTransferModsQueue(const BSONObj& idObj,
-                                 const char op,
+                                 char op,
                                  const repl::OpTime& opTime,
                                  const repl::OpTime& prePostImageOpTime);
 
@@ -467,50 +304,13 @@ private:
      */
     void _drainAllOutstandingOperationTrackRequests(stdx::unique_lock<Latch>& lk);
 
-    /**
-     * Appends to the builder the list of _id of documents that were deleted during migration.
-     * Entries appended to the builder are removed from the list.
-     * Returns the total size of the documents that were appended + initialSize.
-     */
-    long long _xferDeletes(BSONObjBuilder* builder,
-                           std::list<BSONObj>* removeList,
-                           long long initialSize);
-
-    /**
-     * Appends to the builder the list of full documents that were modified/inserted during the
-     * migration. Entries appended to the builder are removed from the list.
-     * Returns the total size of the documents that were appended + initialSize.
-     */
-    long long _xferUpdates(OperationContext* opCtx,
-                           Database* db,
-                           BSONObjBuilder* builder,
-                           std::list<BSONObj>* updateList,
-                           long long initialSize);
-
     /**
      * Sends _recvChunkStatus to the recipient shard until it receives 'steady' from the recipient,
      * an error has occurred, or a timeout is hit.
      */
     Status _checkRecipientCloningStatus(OperationContext* opCtx, Milliseconds maxTimeToWait);
 
-    /**
-     * Inspects the pre and post image document keys and determines which xferMods bucket to
-     * add a new entry. Returns false if neither pre or post image document keys fall into
-     * the chunk boundaries being migrated.
-     */
-    bool _processUpdateForXferMod(const BSONObj& preImageDocKey, const BSONObj& postImageDocKey);
-
-    /**
-     * Defer processing of update ops into xferMods entries to when nextModsBatch is called.
-     */
-    void _deferProcessingForXferMod(const BSONObj& preImageDocKey);
-
-    /**
-     * Converts all deferred update ops captured by the op observer into xferMods entries.
-     */
-    void _processDeferredXferMods(OperationContext* opCtx, Database* database);
-
-    // The original move range request
+    // The original move chunk request
     const MoveChunkRequest _args;
 
     // The shard key associated with the namespace
@@ -533,10 +333,8 @@ private:
     // The current state of the cloner
     State _state{kNew};
 
-    CloneList _cloneList;
-
-    RecordIdSet::size_type _numRecordsCloned{0};
-    RecordIdSet::size_type _numRecordsPassedOver{0};
+    // List of record ids that needs to be transferred (initial clone)
+    std::set<RecordId> _cloneLocs;
 
     // The estimated average object size during the clone phase. Used for buffer size
     // pre-allocation (initial clone).
@@ -568,13 +366,6 @@ private:
     // Amount of delete xfer mods that have not yet reached the recipient.
     size_t _untransferredDeletesCounter{0};
 
-    // Amount of ops that are yet to be converted to update/delete xferMods.
-    size_t _deferredUntransferredOpsCounter{0};
-
-    // Stores document keys of document that needs to be examined if we need to put in to xferMods
-    // list later.
-    std::vector<BSONObj> _deferredReloadOrDeletePreImageDocKeys;
-
     // Total bytes in _reload + _deleted (xfer mods)
     uint64_t _memoryUsed{0};
 
@@ -596,4 +387,14 @@ private:
     boost::optional<JumboChunkCloneState> _jumboChunkCloneState;
 };
 
+/**
+ * Appends to the builder the list of documents either deleted or modified during migration.
+ * Entries appended to the builder are removed from the list.
+ * Returns the total size of the documents that were appended + initialSize.
+ */
+long long xferMods(BSONArrayBuilder* arr,
+                   std::list<BSONObj>* modsList,
+                   long long initialSize,
+                   std::function<bool(BSONObj, BSONObj*)> extractDocToAppendFn);
+
 }  // namespace mongo
