@@ -29,7 +29,6 @@
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kShardingMigration
 
-#include "mongo/db/s/migration_batch_fetcher.h"
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/migration_destination_manager.h"
@@ -40,7 +39,7 @@
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/cancelable_operation_context.h"
 #include "mongo/db/catalog/document_validation.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/index/index_descriptor.h"
@@ -69,6 +68,7 @@
 #include "mongo/db/session_catalog_mongod.h"
 #include "mongo/db/storage/remove_saver.h"
 #include "mongo/db/transaction_participant.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/client/shard_registry.h"
@@ -108,7 +108,8 @@ void checkOutSessionAndVerifyTxnState(OperationContext* opCtx) {
     TransactionParticipant::get(opCtx).beginOrContinue(opCtx,
                                                        *opCtx->getTxnNumber(),
                                                        boost::none /* autocommit */,
-                                                       boost::none /* startTransaction */);
+                                                       boost::none /* startTransaction */,
+                                                       boost::none /* txnRetryCounter */);
 }
 
 template <typename Callable>
@@ -372,8 +373,8 @@ void MigrationDestinationManager::report(BSONObjBuilder& b,
     }
 
     BSONObjBuilder bb(b.subobjStart("counts"));
-    bb.append("cloned", _getNumCloned());
-    bb.append("clonedBytes", _getNumBytesCloned());
+    bb.append("cloned", _numCloned);
+    bb.append("clonedBytes", _clonedBytes);
     bb.append("catchup", _numCatchup);
     bb.append("steady", _numSteady);
     bb.done();
@@ -407,8 +408,6 @@ Status MigrationDestinationManager::start(OperationContext* opCtx,
     _lsid = cloneRequest.getLsid();
     _txnNumber = cloneRequest.getTxnNumber();
 
-    _parallelFetchersSupported = cloneRequest.parallelFetchingSupported();
-
     _nss = nss;
     _fromShard = cloneRequest.getFromShardId();
     _fromShardConnString =
@@ -425,8 +424,8 @@ Status MigrationDestinationManager::start(OperationContext* opCtx,
 
     _chunkMarkedPending = false;
 
-    _migrationCloningProgress = std::make_shared<MigrationCloningProgressSharedState>();
-
+    _numCloned = 0;
+    _clonedBytes = 0;
     _numCatchup = 0;
     _numSteady = 0;
 
@@ -443,19 +442,16 @@ Status MigrationDestinationManager::start(OperationContext* opCtx,
     _sessionMigration =
         std::make_unique<SessionCatalogMigrationDestination>(_nss, _fromShard, *_sessionId);
     ShardingStatistics::get(opCtx).countRecipientMoveChunkStarted.addAndFetch(1);
-    if (mongo::feature_flags::gConcurrencyInChunkMigration.isEnabledAndIgnoreFCV())
-        ShardingStatistics::get(opCtx).chunkMigrationConcurrencyCnt.store(
-            chunkMigrationConcurrency.load());
 
     _migrateThreadHandle = stdx::thread([this]() { _migrateThread(); });
 
     return Status::OK();
 }
 
-repl::OpTime MigrationDestinationManager::cloneDocumentsFromDonor(
+repl::OpTime MigrationDestinationManager::fetchAndApplyBatch(
     OperationContext* opCtx,
-    std::function<void(OperationContext*, BSONObj)> insertBatchFn,
-    std::function<BSONObj(OperationContext*)> fetchBatchFn) {
+    std::function<bool(OperationContext*, BSONObj)> applyBatchFn,
+    std::function<bool(OperationContext*, BSONObj*)> fetchBatchFn) {
 
     SingleProducerSingleConsumerQueue<BSONObj>::Options options;
     options.maxQueueDepth = 1;
@@ -463,8 +459,8 @@ repl::OpTime MigrationDestinationManager::cloneDocumentsFromDonor(
     SingleProducerSingleConsumerQueue<BSONObj> batches(options);
     repl::OpTime lastOpApplied;
 
-    stdx::thread inserterThread{[&] {
-        Client::initThread("chunkInserter", opCtx->getServiceContext(), nullptr);
+    stdx::thread applicationThread{[&] {
+        Client::initThread("batchApplier", opCtx->getServiceContext(), nullptr);
         auto client = Client::getCurrent();
         {
             stdx::lock_guard lk(*client);
@@ -472,46 +468,45 @@ repl::OpTime MigrationDestinationManager::cloneDocumentsFromDonor(
         }
         auto executor =
             Grid::get(opCtx->getServiceContext())->getExecutorPool()->getFixedExecutor();
-        auto inserterOpCtx = CancelableOperationContext(
+        auto applicationOpCtx = CancelableOperationContext(
             cc().makeOperationContext(), opCtx->getCancellationToken(), executor);
 
-        auto consumerGuard = makeGuard([&] {
+        ScopeGuard consumerGuard([&] {
             batches.closeConsumerEnd();
-            lastOpApplied = repl::ReplClientInfo::forClient(inserterOpCtx->getClient()).getLastOp();
+            lastOpApplied =
+                repl::ReplClientInfo::forClient(applicationOpCtx->getClient()).getLastOp();
         });
 
         try {
             while (true) {
-                auto nextBatch = batches.pop(inserterOpCtx.get());
-                auto arr = nextBatch["objects"].Obj();
-                if (arr.isEmpty()) {
+                auto nextBatch = batches.pop(applicationOpCtx.get());
+                if (!applyBatchFn(applicationOpCtx.get(), nextBatch)) {
                     return;
                 }
-                insertBatchFn(inserterOpCtx.get(), arr);
             }
         } catch (...) {
             stdx::lock_guard<Client> lk(*opCtx->getClient());
             opCtx->getServiceContext()->killOperation(lk, opCtx, ErrorCodes::Error(51008));
             LOGV2(21999,
-                  "Batch insertion failed: {error}",
-                  "Batch insertion failed",
+                  "Batch application failed: {error}",
+                  "Batch application failed",
                   "error"_attr = redact(exceptionToStatus()));
         }
     }};
 
 
     {
-        auto inserterThreadJoinGuard = makeGuard([&] {
+        ScopeGuard applicationThreadJoinGuard([&] {
             batches.closeProducerEnd();
-            inserterThread.join();
+            applicationThread.join();
         });
 
         while (true) {
-            auto res = fetchBatchFn(opCtx);
+            BSONObj nextBatch;
+            bool emptyBatch = fetchBatchFn(opCtx, &nextBatch);
             try {
-                batches.push(res.getOwned(), opCtx);
-                auto arr = res["objects"].Obj();
-                if (arr.isEmpty()) {
+                batches.push(nextBatch.getOwned(), opCtx);
+                if (emptyBatch) {
                     break;
                 }
             } catch (const ExceptionFor<ErrorCodes::ProducerConsumerQueueEndClosed>&) {
@@ -827,7 +822,7 @@ void MigrationDestinationManager::cloneCollectionIndexesAndOptions(
                             << "aborting, shard is missing " << indexSpecs.size() << " indexes and "
                             << "collection is not empty. Non-trivial "
                             << "index creation should be scheduled manually",
-                        collection->numRecords(opCtx) == 0);
+                        collection->isEmpty(opCtx));
             }
             return indexSpecs;
         };
@@ -927,7 +922,8 @@ void MigrationDestinationManager::_migrateThread() {
         txnParticipant.beginOrContinue(opCtx,
                                        *opCtx->getTxnNumber(),
                                        boost::none /* autocommit */,
-                                       boost::none /* startTransaction */);
+                                       boost::none /* startTransaction */,
+                                       boost::none /* txnRetryCounter */);
         _migrateDriver(opCtx);
     } catch (...) {
         _setStateFail(str::stream() << "migrate failed: " << redact(exceptionToStatus()));
@@ -981,8 +977,6 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
         return {uuid, indexes, idIndex, collOptions};
     }();
 
-    _collUuid = donorCollectionOptionsAndIndexes.uuid;
-
     auto fromShard =
         uassertStatusOK(Grid::get(outerOpCtx)->shardRegistry()->getShard(outerOpCtx, _fromShard));
 
@@ -1011,9 +1005,11 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
               "range"_attr = redact(range.toString()),
               "migrationId"_attr = _migrationId->toBSON());
 
-        auto waitTime = Milliseconds(receiveChunkWaitForRangeDeleterTimeoutMS.load());
-        auto status = CollectionShardingRuntime::waitForClean(
-            outerOpCtx, _nss, donorCollectionOptionsAndIndexes.uuid, range, waitTime);
+        auto status = CollectionShardingRuntime::waitForClean(outerOpCtx,
+                                                              _nss,
+                                                              donorCollectionOptionsAndIndexes.uuid,
+                                                              range,
+                                                              rangeDeletionWaitDeadline);
 
         if (!status.isOK()) {
             _setStateFail(redact(status.toString()));
@@ -1074,6 +1070,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                                                 range,
                                                 CleanWhenEnum::kNow);
         recipientDeletionTask.setPending(true);
+        const auto currentTime = VectorClock::get(outerOpCtx)->getTime();
+        recipientDeletionTask.setTimestamp(currentTime.clusterTime().asTimestamp());
 
         // It is illegal to wait for write concern with a session checked out, so persist the range
         // deletion task with an immediately satsifiable write concern and then wait for majority
@@ -1085,10 +1083,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
             WriteConcernResult ignoreResult;
             auto latestOpTime =
                 repl::ReplClientInfo::forClient(outerOpCtx->getClient()).getLastOp();
-            uassertStatusOK(waitForWriteConcern(outerOpCtx,
-                                                latestOpTime,
-                                                WriteConcerns::kMajorityWriteConcernShardingTimeout,
-                                                &ignoreResult));
+            uassertStatusOK(waitForWriteConcern(
+                outerOpCtx, latestOpTime, WriteConcerns::kMajorityWriteConcern, &ignoreResult));
         });
 
         timing.done(3);
@@ -1113,32 +1109,109 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
 
         _sessionMigration->start(opCtx->getServiceContext());
 
+        const BSONObj migrateCloneRequest = createMigrateCloneRequest(_nss, *_sessionId);
+
         _chunkMarkedPending = true;  // no lock needed, only the migrate thread looks.
 
-        {
-            // Destructor of MigrationBatchFetcher is non-trivial. Therefore,
-            // this scope has semantic significance.
-            MigrationBatchFetcher<MigrationBatchInserter> fetcher{outerOpCtx,
-                                                                  opCtx,
-                                                                  _nss,
-                                                                  *_sessionId,
-                                                                  _writeConcern,
-                                                                  _fromShard,
-                                                                  range,
-                                                                  *_migrationId,
-                                                                  *_collUuid,
-                                                                  _migrationCloningProgress,
-                                                                  _parallelFetchersSupported};
-            fetcher.fetchAndScheduleInsertion();
-        }
-        opCtx->checkForInterrupt();
-        lastOpApplied = _migrationCloningProgress->getMaxOptime();
+        auto assertNotAborted = [&](OperationContext* opCtx) {
+            opCtx->checkForInterrupt();
+            outerOpCtx->checkForInterrupt();
+            uassert(50748, "Migration aborted while copying documents", getState() != ABORT);
+        };
+
+        auto insertBatchFn = [&](OperationContext* opCtx, BSONObj nextBatch) {
+            auto arr = nextBatch["objects"].Obj();
+            if (arr.isEmpty()) {
+                return false;
+            }
+            auto it = arr.begin();
+            while (it != arr.end()) {
+                int batchNumCloned = 0;
+                int batchClonedBytes = 0;
+                const int batchMaxCloned = migrateCloneInsertionBatchSize.load();
+
+                assertNotAborted(opCtx);
+
+                write_ops::InsertCommandRequest insertOp(_nss);
+                insertOp.getWriteCommandRequestBase().setOrdered(true);
+                insertOp.setDocuments([&] {
+                    std::vector<BSONObj> toInsert;
+                    while (it != arr.end() &&
+                           (batchMaxCloned <= 0 || batchNumCloned < batchMaxCloned)) {
+                        const auto& doc = *it;
+                        BSONObj docToClone = doc.Obj();
+                        toInsert.push_back(docToClone);
+                        batchNumCloned++;
+                        batchClonedBytes += docToClone.objsize();
+                        ++it;
+                    }
+                    return toInsert;
+                }());
+
+                const auto reply =
+                    write_ops_exec::performInserts(opCtx, insertOp, OperationSource::kFromMigrate);
+
+                for (unsigned long i = 0; i < reply.results.size(); ++i) {
+                    uassertStatusOKWithContext(
+                        reply.results[i],
+                        str::stream() << "Insert of " << insertOp.getDocuments()[i] << " failed.");
+                }
+
+                {
+                    stdx::lock_guard<Latch> statsLock(_mutex);
+                    _numCloned += batchNumCloned;
+                    ShardingStatistics::get(opCtx).countDocsClonedOnRecipient.addAndFetch(
+                        batchNumCloned);
+                    _clonedBytes += batchClonedBytes;
+                }
+                if (_writeConcern.needToWaitForOtherNodes()) {
+                    runWithoutSession(outerOpCtx, [&] {
+                        repl::ReplicationCoordinator::StatusAndDuration replStatus =
+                            repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
+                                opCtx,
+                                repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp(),
+                                _writeConcern);
+                        if (replStatus.status.code() == ErrorCodes::WriteConcernFailed) {
+                            LOGV2_WARNING(
+                                22011,
+                                "secondaryThrottle on, but doc insert timed out; continuing",
+                                "migrationId"_attr = _migrationId->toBSON());
+                        } else {
+                            uassertStatusOK(replStatus.status);
+                        }
+                    });
+                }
+
+                sleepmillis(migrateCloneInsertionBatchDelayMS.load());
+            }
+            return true;
+        };
+
+        auto fetchBatchFn = [&](OperationContext* opCtx, BSONObj* nextBatch) {
+            auto commandResponse = uassertStatusOKWithContext(
+                fromShard->runCommand(opCtx,
+                                      ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+                                      "admin",
+                                      migrateCloneRequest,
+                                      Shard::RetryPolicy::kNoRetry),
+                "_migrateClone failed: ");
+
+            uassertStatusOKWithContext(Shard::CommandResponse::getEffectiveStatus(commandResponse),
+                                       "_migrateClone failed: ");
+
+            *nextBatch = commandResponse.response;
+            return nextBatch->getField("objects").Obj().isEmpty();
+        };
+
+        // If running on a replicated system, we'll need to flush the docs we cloned to the
+        // secondaries
+        lastOpApplied = fetchAndApplyBatch(opCtx, insertBatchFn, fetchBatchFn);
 
         timing.done(4);
         migrateThreadHangAtStep4.pauseWhileSet();
 
         if (MONGO_unlikely(failMigrationOnRecipient.shouldFail())) {
-            _setStateFail(str::stream() << "failing migration after cloning " << _getNumCloned()
+            _setStateFail(str::stream() << "failing migration after cloning " << _numCloned
                                         << " docs due to failMigrationOnRecipient failpoint");
             return;
         }
@@ -1150,8 +1223,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
         // 5. Do bulk of mods
         _setState(CATCHUP);
 
-        while (true) {
-            auto res = uassertStatusOKWithContext(
+        auto fetchBatchFn = [&](OperationContext* opCtx, BSONObj* nextBatch) {
+            auto commandResponse = uassertStatusOKWithContext(
                 fromShard->runCommand(opCtx,
                                       ReadPreferenceSetting(ReadPreference::PrimaryOnly),
                                       "admin",
@@ -1159,18 +1232,21 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                                       Shard::RetryPolicy::kNoRetry),
                 "_transferMods failed: ");
 
-            uassertStatusOKWithContext(Shard::CommandResponse::getEffectiveStatus(res),
+            uassertStatusOKWithContext(Shard::CommandResponse::getEffectiveStatus(commandResponse),
                                        "_transferMods failed: ");
 
-            const auto& mods = res.response;
+            *nextBatch = commandResponse.response;
+            return nextBatch->getField("size").number() == 0;
+        };
 
-            if (mods["size"].number() == 0) {
+        auto applyModsFn = [&](OperationContext* opCtx, BSONObj nextBatch) {
+            if (nextBatch["size"].number() == 0) {
                 // There are no more pending modifications to be applied. End the catchup phase
-                break;
+                return false;
             }
 
-            if (!_applyMigrateOp(opCtx, mods, &lastOpApplied)) {
-                continue;
+            if (!_applyMigrateOp(opCtx, nextBatch)) {
+                return true;
             }
 
             const int maxIterations = 3600 * 50;
@@ -1180,17 +1256,16 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                 opCtx->checkForInterrupt();
                 outerOpCtx->checkForInterrupt();
 
-                if (getState() == ABORT) {
-                    LOGV2(22002,
-                          "Migration aborted while waiting for replication at catch up stage",
-                          "migrationId"_attr = _migrationId->toBSON());
-                    return;
-                }
+                uassert(ErrorCodes::CommandFailed,
+                        str::stream()
+                            << "Migration aborted while waiting for replication at catch up stage, "
+                            << _migrationId->toBSON(),
+                        getState() != ABORT);
 
                 if (runWithoutSession(outerOpCtx, [&] {
                         return opReplicatedEnough(opCtx, lastOpApplied, _writeConcern);
                     })) {
-                    break;
+                    return true;
                 }
 
                 if (i > 100) {
@@ -1202,11 +1277,15 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
                 sleepmillis(20);
             }
 
-            if (i == maxIterations) {
-                _setStateFail("secondary can't keep up with migrate");
-                return;
-            }
-        }
+            uassert(ErrorCodes::CommandFailed,
+                    "Secondary can't keep up with migrate",
+                    i != maxIterations);
+
+            return true;
+        };
+
+        auto updatedTime = fetchAndApplyBatch(opCtx, applyModsFn, fetchBatchFn);
+        lastOpApplied = (updatedTime == repl::OpTime()) ? lastOpApplied : updatedTime;
 
         timing.done(5);
         migrateThreadHangAtStep5.pauseWhileSet();
@@ -1273,7 +1352,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
 
             auto mods = res.response;
 
-            if (mods["size"].number() > 0 && _applyMigrateOp(opCtx, mods, &lastOpApplied)) {
+            if (mods["size"].number() > 0 && _applyMigrateOp(opCtx, mods)) {
+                lastOpApplied = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
                 continue;
             }
 
@@ -1320,18 +1400,9 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* outerOpCtx) {
     migrateThreadHangAtStep7.pauseWhileSet();
 }
 
-bool MigrationDestinationManager::_applyMigrateOp(OperationContext* opCtx,
-                                                  const BSONObj& xfer,
-                                                  repl::OpTime* lastOpApplied) {
-    invariant(lastOpApplied);
-
+bool MigrationDestinationManager::_applyMigrateOp(OperationContext* opCtx, const BSONObj& xfer) {
     bool didAnything = false;
 
-    DisableDocumentValidation documentValidationDisabler(
-        opCtx,
-        DocumentValidationSettings::kDisableSchemaValidation |
-            DocumentValidationSettings::kDisableInternalValidation);
-
     // Deleted documents
     if (xfer["deleted"].isABSONObj()) {
         boost::optional<RemoveSaver> rs;
@@ -1374,7 +1445,6 @@ bool MigrationDestinationManager::_applyMigrateOp(OperationContext* opCtx,
                               true /* fromMigrate */);
             });
 
-            *lastOpApplied = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
             didAnything = true;
         }
     }
@@ -1426,7 +1496,6 @@ bool MigrationDestinationManager::_applyMigrateOp(OperationContext* opCtx,
                 Helpers::upsert(opCtx, _nss.ns(), updatedDoc, true);
             });
 
-            *lastOpApplied = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
             didAnything = true;
         }
     }
