@@ -40,6 +40,7 @@ namespace mongo {
  */
 class ReadOnlyCatalogCacheLoader final : public CatalogCacheLoader {
 public:
+    ReadOnlyCatalogCacheLoader() = default;
     ~ReadOnlyCatalogCacheLoader();
 
     void initializeReplicaSetRole(bool isPrimary) override {}
@@ -50,14 +51,9 @@ public:
     void waitForCollectionFlush(OperationContext* opCtx, const NamespaceString& nss) override;
     void waitForDatabaseFlush(OperationContext* opCtx, StringData dbName) override;
 
-    std::shared_ptr<Notification<void>> getChunksSince(
-        const NamespaceString& nss,
-        ChunkVersion version,
-        GetChunksSinceCallbackFn callbackFn) override;
-
-    void getDatabase(
-        StringData dbName,
-        stdx::function<void(OperationContext*, StatusWith<DatabaseType>)> callbackFn) override;
+    SemiFuture<CollectionAndChangedChunks> getChunksSince(const NamespaceString& nss,
+                                                          ChunkVersion version) override;
+    SemiFuture<DatabaseType> getDatabase(StringData dbName) override;
 
 private:
     ConfigServerCatalogCacheLoader _configServerLoader;
