@@ -171,8 +171,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> ReshardingCollectionCloner::makePipel
 }
 
 std::unique_ptr<Pipeline, PipelineDeleter> ReshardingCollectionCloner::_targetAggregationRequest(
-    const Pipeline& pipeline) {
-    auto opCtx = pipeline.getContext()->opCtx;
+    OperationContext* opCtx, const Pipeline& pipeline) {
     // We associate the aggregation cursors established on each donor shard with a logical session
     // to prevent them from killing the cursor when it is idle locally. Due to the cursor's merging
     // behavior across all donor shards, it is possible for the cursor to be active on one donor
@@ -194,11 +193,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> ReshardingCollectionCloner::_targetAg
                                 << repl::readConcernLevels::kSnapshotName
                                 << repl::ReadConcernArgs::kAtClusterTimeFieldName
                                 << _atClusterTime));
-    // The read preference on the request is merely informational (e.g. for profiler entries) -- the
-    // pipeline's opCtx setting is actually used when sending the request.
-    auto readPref = ReadPreferenceSetting{ReadPreference::Nearest};
-    request.setUnwrappedReadPref(readPref.toContainingBSON());
-    ReadPreferenceSetting::get(opCtx) = readPref;
+    request.setUnwrappedReadPref(ReadPreferenceSetting{ReadPreference::Nearest}.toContainingBSON());
 
     return shardVersionRetry(opCtx,
                              Grid::get(opCtx)->catalogCache(),
@@ -232,7 +227,7 @@ std::unique_ptr<Pipeline, PipelineDeleter> ReshardingCollectionCloner::_restartP
     ON_BLOCK_EXIT([curOp] { curOp->done(); });
 
     auto pipeline = _targetAggregationRequest(
-        *makePipeline(opCtx, MongoProcessInterface::create(opCtx), idToResumeFrom));
+        opCtx, *makePipeline(opCtx, MongoProcessInterface::create(opCtx), idToResumeFrom));
 
     if (!idToResumeFrom.missing()) {
         // Skip inserting the first document retrieved after resuming because $gte was used in the
@@ -310,7 +305,7 @@ SemiFuture<void> ReshardingCollectionCloner::run(
                }
 
                auto opCtx = factory.makeOperationContext(&cc());
-               auto guard = makeGuard([&] {
+               ScopeGuard guard([&] {
                    chainCtx->pipeline->dispose(opCtx.get());
                    chainCtx->pipeline.reset();
                });
