@@ -27,13 +27,15 @@
  *    it in the license file.
  */
 
+#include "mongo/base/string_data.h"
+#include "mongo/s/write_ops/batched_command_request.h"
+
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kResharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/resharding/resharding_coordinator_service.h"
 
-#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/json.h"
 #include "mongo/db/auth/authorization_session_impl.h"
@@ -41,8 +43,6 @@
 #include "mongo/db/logical_session_cache.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/repl/primary_only_service.h"
-#include "mongo/db/repl/repl_client_info.h"
-#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/balancer/balance_stats.h"
 #include "mongo/db/s/balancer/balancer_policy.h"
 #include "mongo/db/s/config/initial_split_policy.h"
@@ -69,7 +69,6 @@
 #include "mongo/s/request_types/flush_resharding_state_change_gen.h"
 #include "mongo/s/request_types/flush_routing_table_cache_updates_gen.h"
 #include "mongo/s/shard_id.h"
-#include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
@@ -83,7 +82,6 @@ namespace {
 using namespace fmt::literals;
 
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorAfterPreparingToDonate);
-MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeInitializing);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeCloning);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeBlockingWrites);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeDecisionPersisted);
@@ -93,7 +91,6 @@ MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeStartingErrorFlow);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforePersistingStateTransition);
 MONGO_FAIL_POINT_DEFINE(pauseBeforeTellDonorToRefresh);
 MONGO_FAIL_POINT_DEFINE(pauseBeforeInsertCoordinatorDoc);
-MONGO_FAIL_POINT_DEFINE(pauseBeforeCTHolderInitialization);
 
 const std::string kReshardingCoordinatorActiveIndexName = "ReshardingCoordinatorActiveIndex";
 const Backoff kExponentialBackoff(Seconds(1), Milliseconds::max());
@@ -519,21 +516,41 @@ void removeChunkAndTagsDocs(OperationContext* opCtx,
         opCtx, TagsType::ConfigNS, tagsQuery, kMajorityWriteConcern, tagDeleteOperationHint));
 }
 
+void updateChunkAndTagsDocsForTempNss(OperationContext* opCtx,
+                                      const ReshardingCoordinatorDocument& coordinatorDoc,
+                                      OID newCollectionEpoch,
+                                      TxnNumber txnNumber) {
+    auto hint = BSON("ns" << 1 << "min" << 1);
+    auto tagsRequest = BatchedCommandRequest::buildUpdateOp(
+        TagsType::ConfigNS,
+        BSON(TagsType::ns(coordinatorDoc.getTempReshardingNss().ns())),    // query
+        BSON("$set" << BSON("ns" << coordinatorDoc.getSourceNss().ns())),  // update
+        false,                                                             // upsert
+        true,                                                              // multi
+        hint                                                               // hint
+    );
+
+    // Update the 'ns' field to be the original collection namespace for all tags documents that
+    // currently have 'ns' as the temporary collection namespace
+    auto tagsRes = ShardingCatalogManager::get(opCtx)->writeToConfigDocumentInTxn(
+        opCtx, TagsType::ConfigNS, tagsRequest, txnNumber);
+}
+
 /**
  * Executes metadata changes in a transaction without bumping the collection version.
  */
 void executeMetadataChangesInTxn(
     OperationContext* opCtx,
     unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc) {
-    ShardingCatalogManager::withTransaction(
-        opCtx,
-        NamespaceString::kConfigReshardingOperationsNamespace,
-        [&](OperationContext* opCtx, TxnNumber txnNumber) { changeMetadataFunc(opCtx, txnNumber); },
-        ShardingCatalogClient::kLocalWriteConcern);
+    ShardingCatalogManager::withTransaction(opCtx,
+                                            NamespaceString::kConfigReshardingOperationsNamespace,
+                                            [&](OperationContext* opCtx, TxnNumber txnNumber) {
+                                                changeMetadataFunc(opCtx, txnNumber);
+                                            });
 }
 
 BSONObj makeFlushRoutingTableCacheUpdatesCmd(const NamespaceString& nss) {
-    auto cmd = FlushRoutingTableCacheUpdatesWithWriteConcern(nss);
+    auto cmd = _flushRoutingTableCacheUpdatesWithWriteConcern(nss);
     cmd.setSyncFromConfig(true);
     cmd.setDbName(nss.db());
     return cmd.toBSON(
@@ -609,35 +626,16 @@ void writeDecisionPersistedState(OperationContext* opCtx,
         // shard key, new epoch, and new UUID
         updateConfigCollectionsForOriginalNss(
             opCtx, coordinatorDoc, newCollectionEpoch, newCollectionTimestamp, txnNumber);
+
+        updateChunkAndTagsDocsForTempNss(opCtx, coordinatorDoc, newCollectionEpoch, txnNumber);
     });
 }
 
-void updateTagsDocsForTempNss(OperationContext* opCtx,
-                              const ReshardingCoordinatorDocument& coordinatorDoc) {
-    auto hint = BSON("ns" << 1 << "min" << 1);
-    auto tagsRequest = BatchedCommandRequest::buildUpdateOp(
-        TagsType::ConfigNS,
-        BSON(TagsType::ns(coordinatorDoc.getTempReshardingNss().ns())),    // query
-        BSON("$set" << BSON("ns" << coordinatorDoc.getSourceNss().ns())),  // update
-        false,                                                             // upsert
-        true,                                                              // multi
-        hint                                                               // hint
-    );
-
-    // Update the 'ns' field to be the original collection namespace for all tags documents that
-    // currently have 'ns' as the temporary collection namespace.
-    DBDirectClient client(opCtx);
-    BSONObj tagsRes;
-    client.runCommand(tagsRequest.getNS().db().toString(), tagsRequest.toBSON(), tagsRes);
-    uassertStatusOK(getStatusFromWriteCommandReply(tagsRes));
-}
-
 void insertCoordDocAndChangeOrigCollEntry(OperationContext* opCtx,
                                           const ReshardingCoordinatorDocument& coordinatorDoc) {
+
     ShardingCatalogManager::get(opCtx)->bumpCollectionVersionAndChangeMetadataInTxn(
-        opCtx,
-        coordinatorDoc.getSourceNss(),
-        [&](OperationContext* opCtx, TxnNumber txnNumber) {
+        opCtx, coordinatorDoc.getSourceNss(), [&](OperationContext* opCtx, TxnNumber txnNumber) {
             auto doc = ShardingCatalogManager::get(opCtx)->findOneConfigDocumentInTxn(
                 opCtx,
                 CollectionType::ConfigNS,
@@ -676,8 +674,7 @@ void insertCoordDocAndChangeOrigCollEntry(OperationContext* opCtx,
             // 'reshardingFields'
             updateConfigCollectionsForOriginalNss(
                 opCtx, coordinatorDoc, boost::none, boost::none, txnNumber);
-        },
-        ShardingCatalogClient::kLocalWriteConcern);
+        });
 }
 
 void writeParticipantShardsAndTempCollInfo(
@@ -696,6 +693,7 @@ void writeParticipantShardsAndTempCollInfo(
         [&](OperationContext* opCtx, TxnNumber txnNumber) {
             // Insert the config.collections entry for the temporary resharding collection. The
             // chunks all have the same epoch, so picking the last chunk here is arbitrary.
+            invariant(initialChunks.size() != 0);
             auto chunkVersion = initialChunks.back().getVersion();
             writeToConfigCollectionsForTempNss(
                 opCtx, updatedCoordinatorDoc, chunkVersion, CollationSpec::kSimpleSpec, txnNumber);
@@ -703,8 +701,7 @@ void writeParticipantShardsAndTempCollInfo(
             writeToCoordinatorStateNss(opCtx, updatedCoordinatorDoc, txnNumber);
             updateConfigCollectionsForOriginalNss(
                 opCtx, updatedCoordinatorDoc, boost::none, boost::none, txnNumber);
-        },
-        ShardingCatalogClient::kLocalWriteConcern);
+        });
 }
 
 void writeStateTransitionAndCatalogUpdatesThenBumpShardVersions(
@@ -718,9 +715,7 @@ void writeStateTransitionAndCatalogUpdatesThenBumpShardVersions(
     }
 
     ShardingCatalogManager::get(opCtx)->bumpMultipleCollectionVersionsAndChangeMetadataInTxn(
-        opCtx,
-        collNames,
-        [&](OperationContext* opCtx, TxnNumber txnNumber) {
+        opCtx, collNames, [&](OperationContext* opCtx, TxnNumber txnNumber) {
             // Update the config.reshardingOperations entry
             writeToCoordinatorStateNss(opCtx, coordinatorDoc, txnNumber);
 
@@ -736,8 +731,7 @@ void writeStateTransitionAndCatalogUpdatesThenBumpShardVersions(
                 writeToConfigCollectionsForTempNss(
                     opCtx, coordinatorDoc, boost::none, boost::none, txnNumber);
             }
-        },
-        ShardingCatalogClient::kLocalWriteConcern);
+        });
 }
 
 void removeCoordinatorDocAndReshardingFields(OperationContext* opCtx,
@@ -781,20 +775,14 @@ void removeCoordinatorDocAndReshardingFields(OperationContext* opCtx,
             // Remove the resharding fields from the config.collections entry
             updateConfigCollectionsForOriginalNss(
                 opCtx, updatedCoordinatorDoc, boost::none, boost::none, txnNumber);
-        },
-        ShardingCatalogClient::kLocalWriteConcern);
+        });
 }
 }  // namespace resharding
 
 ChunkVersion ReshardingCoordinatorExternalState::calculateChunkVersionForInitialChunks(
     OperationContext* opCtx) {
-    boost::optional<Timestamp> timestamp;
-    if (serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-            ServerGlobalParams::FeatureCompatibility::Version::kVersion50)) {
-        const auto now = VectorClock::get(opCtx)->getTime();
-        timestamp = now.clusterTime().asTimestamp();
-    }
-
+    const auto now = VectorClock::get(opCtx)->getTime();
+    const auto timestamp = now.clusterTime().asTimestamp();
     return ChunkVersion(1, 0, OID::gen(), timestamp);
 }
 
@@ -850,19 +838,10 @@ ReshardingCoordinatorExternalStateImpl::calculateParticipantShardsAndChunks(
         for (const auto& reshardedChunk : *chunks) {
             recipientShardIds.emplace(reshardedChunk.getRecipientShardId());
 
-            if (version.getTimestamp()) {
-                initialChunks.emplace_back(
-                    coordinatorDoc.getReshardingUUID(),
-                    ChunkRange{reshardedChunk.getMin(), reshardedChunk.getMax()},
-                    version,
-                    reshardedChunk.getRecipientShardId());
-            } else {
-                initialChunks.emplace_back(
-                    coordinatorDoc.getTempReshardingNss(),
-                    ChunkRange{reshardedChunk.getMin(), reshardedChunk.getMax()},
-                    version,
-                    reshardedChunk.getRecipientShardId());
-            }
+            initialChunks.emplace_back(coordinatorDoc.getReshardingUUID(),
+                                       ChunkRange{reshardedChunk.getMin(), reshardedChunk.getMax()},
+                                       version,
+                                       reshardedChunk.getRecipientShardId());
             version.incMinor();
         }
     } else {
@@ -897,12 +876,8 @@ ReshardingCoordinatorExternalStateImpl::calculateParticipantShardsAndChunks(
 
         // Note: The resharding initial split policy doesn't care about what is the real primary
         // shard, so just pass in a random shard.
-        const SplitPolicyParams splitParams{
-            tempNs,
-            coordinatorDoc.getReshardingUUID(),
-            *donorShardIds.begin(),
-            ChunkEntryFormat::getForVersionCallerGuaranteesFCVStability(
-                ServerGlobalParams::FeatureCompatibility::Version::kVersion50)};
+        const SplitPolicyParams splitParams{coordinatorDoc.getReshardingUUID(),
+                                            *donorShardIds.begin()};
         auto splitResult = initialSplitter.createFirstChunks(opCtx, shardKey, splitParams);
         initialChunks = std::move(splitResult.chunks);
 
@@ -1082,8 +1057,7 @@ ReshardingCoordinatorService::ReshardingCoordinator::_tellAllParticipantsReshard
                        _cancelableOpCtxFactory.emplace(_ctHolder->getStepdownToken(),
                                                        _markKilledExecutor);
                    })
-                   .then([this] { return _waitForMajority(_ctHolder->getStepdownToken()); })
-                   .then([this, executor]() {
+                   .then([this, executor] {
                        pauseBeforeTellDonorToRefresh.pauseWhileSet();
                        _establishAllDonorsAsParticipants(executor);
                    })
@@ -1253,13 +1227,8 @@ ReshardingCoordinatorService::ReshardingCoordinator::_commitAndFinishReshardOper
     return resharding::WithAutomaticRetry([this, executor, updatedCoordinatorDoc] {
                return ExecutorFuture<void>(**executor)
                    .then([this, executor, updatedCoordinatorDoc] {
-                       _commit(updatedCoordinatorDoc);
-
-                       auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
-                       resharding::updateTagsDocsForTempNss(opCtx.get(), updatedCoordinatorDoc);
+                       return _commit(updatedCoordinatorDoc);
                    })
-                   .then([this] { return _waitForMajority(_ctHolder->getStepdownToken()); })
-                   .thenRunOn(**executor)
                    .then([this, executor] {
                        _tellAllParticipantsToCommit(_coordinatorDoc.getSourceNss(), executor);
                    })
@@ -1307,18 +1276,7 @@ ReshardingCoordinatorService::ReshardingCoordinator::_commitAndFinishReshardOper
 SemiFuture<void> ReshardingCoordinatorService::ReshardingCoordinator::run(
     std::shared_ptr<executor::ScopedTaskExecutor> executor,
     const CancellationToken& stepdownToken) noexcept {
-    pauseBeforeCTHolderInitialization.pauseWhileSet();
-
-    auto abortCalled = [&] {
-        stdx::lock_guard<Latch> lk(_abortCalledMutex);
-        _ctHolder = std::make_unique<CoordinatorCancellationTokenHolder>(stepdownToken);
-        return _abortCalled;
-    }();
-
-    if (abortCalled) {
-        _ctHolder->abort();
-    }
-
+    _ctHolder = std::make_unique<CoordinatorCancellationTokenHolder>(stepdownToken);
     _markKilledExecutor->startup();
     _cancelableOpCtxFactory.emplace(_ctHolder->getAbortToken(), _markKilledExecutor);
 
@@ -1329,15 +1287,8 @@ SemiFuture<void> ReshardingCoordinatorService::ReshardingCoordinator::run(
         })
         .onCompletion([this, executor](Status status) {
             auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
-            reshardingPauseCoordinatorBeforeCompletion.executeIf(
-                [&](const BSONObj&) {
-                    reshardingPauseCoordinatorBeforeCompletion.pauseWhileSetAndNotCanceled(
-                        opCtx.get(), _ctHolder->getStepdownToken());
-                },
-                [&](const BSONObj& data) {
-                    auto ns = std::string(data.getStringField("sourceNamespace"));
-                    return ns.empty() ? true : ns == _coordinatorDoc.getSourceNss().ns();
-                });
+            reshardingPauseCoordinatorBeforeCompletion.pauseWhileSetAndNotCanceled(
+                opCtx.get(), _ctHolder->getStepdownToken());
 
             {
                 auto lg = stdx::lock_guard(_fulfillmentMutex);
@@ -1435,30 +1386,23 @@ ReshardingCoordinatorService::ReshardingCoordinator::_onAbortCoordinatorAndParti
     invariant(_coordinatorDoc.getState() >= CoordinatorStateEnum::kPreparingToDonate);
 
     return resharding::WithAutomaticRetry([this, executor, status] {
-               return ExecutorFuture<void>(**executor)
-                   .then([this, executor, status] {
-                       if (_coordinatorDoc.getState() != CoordinatorStateEnum::kAborting) {
-                           // The coordinator only transitions into kAborting if there are
-                           // participants to wait on before transitioning to kDone.
-                           _updateCoordinatorDocStateAndCatalogEntries(
-                               CoordinatorStateEnum::kAborting,
-                               _coordinatorDoc,
-                               boost::none,
-                               boost::none,
-                               status);
-                       }
-                   })
-                   .then([this] { return _waitForMajority(_ctHolder->getStepdownToken()); })
-                   .thenRunOn(**executor)
-                   .then([this, executor, status] {
-                       _tellAllParticipantsToAbort(executor,
-                                                   status == ErrorCodes::ReshardCollectionAborted);
+               if (_coordinatorDoc.getState() != CoordinatorStateEnum::kAborting) {
+                   // The coordinator only transitions into kAborting if there are participants to
+                   // wait on before transitioning to kDone.
+                   _updateCoordinatorDocStateAndCatalogEntries(CoordinatorStateEnum::kAborting,
+                                                               _coordinatorDoc,
+                                                               boost::none,
+                                                               boost::none,
+                                                               status);
+               }
 
-                       // Wait for all participants to acknowledge the operation reached an
-                       // unrecoverable error.
-                       return future_util::withCancellation(
-                           _awaitAllParticipantShardsDone(executor), _ctHolder->getStepdownToken());
-                   });
+               _tellAllParticipantsToAbort(executor,
+                                           status == ErrorCodes::ReshardCollectionAborted);
+
+               // Wait for all participants to acknowledge the operation reached an unrecoverable
+               // error.
+               return future_util::withCancellation(_awaitAllParticipantShardsDone(executor),
+                                                    _ctHolder->getStepdownToken());
            })
         .onTransientError([](const Status& retryStatus) {
             LOGV2(5093707,
@@ -1474,15 +1418,7 @@ ReshardingCoordinatorService::ReshardingCoordinator::_onAbortCoordinatorAndParti
 }
 
 void ReshardingCoordinatorService::ReshardingCoordinator::abort() {
-    auto ctHolderInitialized = [&] {
-        stdx::lock_guard<Latch> lk(_abortCalledMutex);
-        _abortCalled = true;
-        return !(_ctHolder == nullptr);
-    }();
-
-    if (ctHolderInitialized) {
-        _ctHolder->abort();
-    }
+    _ctHolder->abort();
 }
 
 boost::optional<BSONObj> ReshardingCoordinatorService::ReshardingCoordinator::reportForCurrentOp(
@@ -1519,16 +1455,6 @@ void ReshardingCoordinatorService::ReshardingCoordinator::_fulfillOkayToEnterCri
     }
 }
 
-SemiFuture<void> ReshardingCoordinatorService::ReshardingCoordinator::_waitForMajority(
-    const CancellationToken& token) {
-    auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
-    auto client = opCtx->getClient();
-    repl::ReplClientInfo::forClient(client).setLastOpToSystemLastOpTime(opCtx.get());
-    auto opTime = repl::ReplClientInfo::forClient(client).getLastOp();
-    return WaitForMajorityService::get(client->getServiceContext())
-        .waitUntilMajority(opTime, token);
-}
-
 void ReshardingCoordinatorService::ReshardingCoordinator::_insertCoordDocAndChangeOrigCollEntry() {
     if (_coordinatorDoc.getState() > CoordinatorStateEnum::kUnused) {
         if (!_coordinatorDocWrittenPromise.getFuture().isReady()) {
@@ -1545,8 +1471,6 @@ void ReshardingCoordinatorService::ReshardingCoordinator::_insertCoordDocAndChan
     }
 
     auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
-    reshardingPauseCoordinatorBeforeInitializing.pauseWhileSetAndNotCanceled(
-        opCtx.get(), _ctHolder->getStepdownToken());
     ReshardingCoordinatorDocument updatedCoordinatorDoc = _coordinatorDoc;
     updatedCoordinatorDoc.setState(CoordinatorStateEnum::kInitializing);
     resharding::insertCoordDocAndChangeOrigCollEntry(opCtx.get(), updatedCoordinatorDoc);
@@ -1649,8 +1573,7 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllDonorsReadyToDonat
                 coordinatorDocChangedOnDisk,
                 highestMinFetchTimestamp,
                 computeApproxCopySize(coordinatorDocChangedOnDisk));
-        })
-        .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); });
+        });
 }
 
 ExecutorFuture<void>
@@ -1667,8 +1590,7 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllRecipientsFinished
         .then([this](ReshardingCoordinatorDocument coordinatorDocChangedOnDisk) {
             this->_updateCoordinatorDocStateAndCatalogEntries(CoordinatorStateEnum::kApplying,
                                                               coordinatorDocChangedOnDisk);
-        })
-        .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); });
+        });
 }
 
 void ReshardingCoordinatorService::ReshardingCoordinator::_startCommitMonitor(
@@ -1704,22 +1626,9 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllRecipientsFinished
             _startCommitMonitor(executor);
 
             LOGV2(5391602, "Resharding operation waiting for an okay to enter critical section");
-
-            // The _reshardingCoordinatorObserver->awaitAllRecipientsInStrictConsistency() future is
-            // used for reporting recipient shard errors encountered during the Applying phase and
-            // in turn aborting the resharding operation.
-            // For all other cases, the _canEnterCritical.getFuture() resolves first and the
-            // operation can then proceed to entering the critical section depending on the status
-            // returned.
-            return future_util::withCancellation(
-                       whenAny(
-                           _canEnterCritical.getFuture().thenRunOn(**executor),
-                           _reshardingCoordinatorObserver->awaitAllRecipientsInStrictConsistency()
-                               .thenRunOn(**executor)
-                               .ignoreValue()),
-                       _ctHolder->getAbortToken())
+            return future_util::withCancellation(_canEnterCritical.getFuture(),
+                                                 _ctHolder->getAbortToken())
                 .thenRunOn(**executor)
-                .then([](auto result) { return result.result; })
                 .onCompletion([this](Status status) {
                     _ctHolder->cancelCommitMonitor();
                     if (status.isOK()) {
@@ -1737,10 +1646,6 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllRecipientsFinished
 
             this->_updateCoordinatorDocStateAndCatalogEntries(CoordinatorStateEnum::kBlockingWrites,
                                                               _coordinatorDoc);
-        })
-        .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); })
-        .thenRunOn(**executor)
-        .then([this, executor] {
             const auto criticalSectionTimeout =
                 Milliseconds(resharding::gReshardingCriticalSectionTimeoutMillis.load());
             const auto criticalSectionExpiresAt = (*executor)->now() + criticalSectionTimeout;
@@ -1778,11 +1683,11 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllRecipientsInStrict
         .thenRunOn(**executor);
 }
 
-void ReshardingCoordinatorService::ReshardingCoordinator::_commit(
+Future<void> ReshardingCoordinatorService::ReshardingCoordinator::_commit(
     const ReshardingCoordinatorDocument& coordinatorDoc) {
     if (_coordinatorDoc.getState() > CoordinatorStateEnum::kBlockingWrites) {
         invariant(_coordinatorDoc.getState() != CoordinatorStateEnum::kAborting);
-        return;
+        return Status::OK();
     }
 
     ReshardingCoordinatorDocument updatedCoordinatorDoc = coordinatorDoc;
@@ -1808,6 +1713,8 @@ void ReshardingCoordinatorService::ReshardingCoordinator::_commit(
 
     // Update the in memory state
     installCoordinatorDoc(opCtx.get(), updatedCoordinatorDoc);
+
+    return Status::OK();
 }
 
 ExecutorFuture<void>
