@@ -32,7 +32,6 @@
 #include "mongo/platform/basic.h"
 
 #include <boost/optional.hpp>
-#include <functional>
 
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/logical_session_cache_noop.h"
@@ -53,7 +52,6 @@
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_shard.h"
-#include "mongo/stdx/unordered_map.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/clock_source_mock.h"
 #include "mongo/util/fail_point.h"
@@ -90,19 +88,11 @@ class ExternalStateForTest : public ReshardingCoordinatorExternalState {
         // recipient list.
         if (const auto& chunks = coordinatorDoc.getPresetReshardedChunks()) {
             for (const auto& reshardedChunk : *chunks) {
-                if (version.getTimestamp()) {
-                    initialChunks.emplace_back(
-                        coordinatorDoc.getReshardingUUID(),
-                        ChunkRange{reshardedChunk.getMin(), reshardedChunk.getMax()},
-                        version,
-                        reshardedChunk.getRecipientShardId());
-                } else {
-                    initialChunks.emplace_back(
-                        coordinatorDoc.getTempReshardingNss(),
-                        ChunkRange{reshardedChunk.getMin(), reshardedChunk.getMax()},
-                        version,
-                        reshardedChunk.getRecipientShardId());
-                }
+                initialChunks.emplace_back(
+                    coordinatorDoc.getReshardingUUID(),
+                    ChunkRange{reshardedChunk.getMin(), reshardedChunk.getMax()},
+                    version,
+                    reshardedChunk.getRecipientShardId());
                 version.incMinor();
             }
         }
@@ -232,7 +222,8 @@ public:
     ReshardingCoordinatorDocument getCoordinatorDoc(OperationContext* opCtx) {
         DBDirectClient client(opCtx);
 
-        auto doc = client.findOne(NamespaceString::kConfigReshardingOperationsNamespace.ns(), {});
+        auto doc =
+            client.findOne(NamespaceString::kConfigReshardingOperationsNamespace.ns(), BSONObj{});
         IDLParserErrorContext errCtx("reshardingCoordFromTest");
         return ReshardingCoordinatorDocument::parse(errCtx, doc);
     }
@@ -356,8 +347,6 @@ public:
         if (reshardingFields)
             collType.setReshardingFields(std::move(reshardingFields.get()));
 
-        // TODO SERVER-53330: Evaluate whether or not we can include
-        // CoordinatorStateEnum::kInitializing in this if statement.
         if (coordinatorDoc.getState() == CoordinatorStateEnum::kDone ||
             coordinatorDoc.getState() == CoordinatorStateEnum::kAborting) {
             collType.setAllowMigrations(true);
@@ -393,7 +382,7 @@ public:
         DatabaseType dbDoc(coordinatorDoc.getSourceNss().db().toString(),
                            coordinatorDoc.getDonorShards().front().getId(),
                            true,
-                           DatabaseVersion{UUID::gen()});
+                           DatabaseVersion{UUID::gen(), Timestamp(1, 1)});
         client.insert(DatabaseType::ConfigNS.ns(), dbDoc.toBSON());
 
         return coordinatorDoc;
@@ -414,13 +403,14 @@ public:
 
     std::vector<ChunkType> makeChunks(const UUID& uuid,
                                       OID epoch,
+                                      const Timestamp& timestamp,
                                       const ShardKeyPattern& shardKey,
                                       std::vector<OID> ids) {
         auto chunkRanges =
             _newShardKey.isShardKey(shardKey.toBSON()) ? _newChunkRanges : _oldChunkRanges;
 
         // Create two chunks, one on each shard with the given namespace and epoch
-        ChunkVersion version(1, 0, epoch, boost::none /* timestamp */);
+        ChunkVersion version(1, 0, epoch, timestamp);
         ChunkType chunk1(uuid, chunkRanges[0], version, ShardId("shard0000"));
         chunk1.setName(ids[0]);
         version.incMinor();
@@ -433,9 +423,10 @@ public:
     // Returns the chunk for the donor shard.
     ChunkType makeAndInsertChunksForDonorShard(const UUID& uuid,
                                                OID epoch,
+                                               const Timestamp& timestamp,
                                                const ShardKeyPattern& shardKey,
                                                std::vector<OID> ids) {
-        auto chunks = makeChunks(uuid, epoch, shardKey, ids);
+        auto chunks = makeChunks(uuid, epoch, timestamp, shardKey, ids);
 
         // Only the chunk corresponding to shard0000 is stored as a donor in the coordinator state
         // document constructed.
@@ -507,87 +498,6 @@ public:
         }
     }
 
-    auto initializeAndGetCoordinator() {
-        auto doc = insertStateAndCatalogEntries(CoordinatorStateEnum::kUnused, _originalEpoch);
-        auto opCtx = operationContext();
-        auto donorChunk = makeAndInsertChunksForDonorShard(
-            _originalUUID, _originalEpoch, _oldShardKey, std::vector{OID::gen(), OID::gen()});
-
-        auto initialChunks = makeChunks(
-            _reshardingUUID, _tempEpoch, _newShardKey, std::vector{OID::gen(), OID::gen()});
-
-        std::vector<ReshardedChunk> presetReshardedChunks;
-        for (const auto& chunk : initialChunks) {
-            presetReshardedChunks.emplace_back(chunk.getShard(), chunk.getMin(), chunk.getMax());
-        }
-
-        doc.setPresetReshardedChunks(presetReshardedChunks);
-
-        return ReshardingCoordinator::getOrCreate(opCtx, _service, doc.toBSON());
-    }
-
-    using TransitionFunctionMap = stdx::unordered_map<CoordinatorStateEnum, std::function<void()>>;
-
-    void runReshardingToCompletion() {
-        runReshardingToCompletion(TransitionFunctionMap{});
-    }
-
-    void runReshardingToCompletion(const TransitionFunctionMap& transitionFunctions) {
-        auto runFunctionForState = [&](CoordinatorStateEnum state) {
-            auto it = transitionFunctions.find(state);
-            if (it == transitionFunctions.end()) {
-                return;
-            }
-            it->second();
-        };
-
-        const std::vector<CoordinatorStateEnum> states{CoordinatorStateEnum::kPreparingToDonate,
-                                                       CoordinatorStateEnum::kCloning,
-                                                       CoordinatorStateEnum::kApplying,
-                                                       CoordinatorStateEnum::kBlockingWrites,
-                                                       CoordinatorStateEnum::kCommitting};
-        PauseDuringStateTransitions stateTransitionsGuard{controller(), states};
-
-        auto opCtx = operationContext();
-        auto coordinator = initializeAndGetCoordinator();
-
-        stateTransitionsGuard.wait(CoordinatorStateEnum::kPreparingToDonate);
-        runFunctionForState(CoordinatorStateEnum::kPreparingToDonate);
-        stateTransitionsGuard.unset(CoordinatorStateEnum::kPreparingToDonate);
-        waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kPreparingToDonate);
-        makeDonorsReadyToDonate(opCtx);
-
-        stateTransitionsGuard.wait(CoordinatorStateEnum::kCloning);
-        runFunctionForState(CoordinatorStateEnum::kCloning);
-        stateTransitionsGuard.unset(CoordinatorStateEnum::kCloning);
-        waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kCloning);
-
-        makeRecipientsFinishedCloning(opCtx);
-        stateTransitionsGuard.wait(CoordinatorStateEnum::kApplying);
-        runFunctionForState(CoordinatorStateEnum::kApplying);
-        stateTransitionsGuard.unset(CoordinatorStateEnum::kApplying);
-        waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kApplying);
-
-        coordinator->onOkayToEnterCritical();
-        stateTransitionsGuard.wait(CoordinatorStateEnum::kBlockingWrites);
-        runFunctionForState(CoordinatorStateEnum::kBlockingWrites);
-        stateTransitionsGuard.unset(CoordinatorStateEnum::kBlockingWrites);
-        waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kBlockingWrites);
-
-        makeRecipientsBeInStrictConsistency(opCtx);
-
-        stateTransitionsGuard.wait(CoordinatorStateEnum::kCommitting);
-        runFunctionForState(CoordinatorStateEnum::kCommitting);
-        stateTransitionsGuard.unset(CoordinatorStateEnum::kCommitting);
-
-        waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kCommitting);
-
-        makeDonorsProceedToDone(opCtx);
-        makeRecipientsProceedToDone(opCtx);
-
-        coordinator->getCompletionFuture().get(opCtx);
-    }
-
     repl::PrimaryOnlyService* _service = nullptr;
 
     std::shared_ptr<CoordinatorStateTransitionController> _controller;
@@ -599,10 +509,12 @@ public:
     NamespaceString _originalNss = NamespaceString("db.foo");
     UUID _originalUUID = UUID::gen();
     OID _originalEpoch = OID::gen();
+    Timestamp _originalTimestamp = Timestamp(1);
 
     NamespaceString _tempNss = NamespaceString("db.system.resharding." + _originalUUID.toString());
     UUID _reshardingUUID = UUID::gen();
     OID _tempEpoch = OID::gen();
+    Timestamp _tempTimestamp = Timestamp(2);
     ShardKeyPattern _oldShardKey = ShardKeyPattern(BSON("oldShardKey" << 1));
     ShardKeyPattern _newShardKey = ShardKeyPattern(BSON("newShardKey" << 1));
 
@@ -626,36 +538,135 @@ public:
 };
 
 TEST_F(ReshardingCoordinatorServiceTest, ReshardingCoordinatorSuccessfullyTransitionsTokDone) {
-    runReshardingToCompletion();
+    const std::vector<CoordinatorStateEnum> coordinatorStates{
+        CoordinatorStateEnum::kPreparingToDonate,
+        CoordinatorStateEnum::kCloning,
+        CoordinatorStateEnum::kApplying,
+        CoordinatorStateEnum::kBlockingWrites,
+        CoordinatorStateEnum::kCommitting};
+    PauseDuringStateTransitions stateTransitionsGuard{controller(), coordinatorStates};
+
+    auto doc = insertStateAndCatalogEntries(CoordinatorStateEnum::kUnused, _originalEpoch);
+    auto opCtx = operationContext();
+    auto donorChunk = makeAndInsertChunksForDonorShard(_originalUUID,
+                                                       _originalEpoch,
+                                                       _originalTimestamp,
+                                                       _oldShardKey,
+                                                       std::vector{OID::gen(), OID::gen()});
+
+    auto initialChunks = makeChunks(_reshardingUUID,
+                                    _tempEpoch,
+                                    _tempTimestamp,
+                                    _newShardKey,
+                                    std::vector{OID::gen(), OID::gen()});
+
+    std::vector<ReshardedChunk> presetReshardedChunks;
+    for (const auto& chunk : initialChunks) {
+        presetReshardedChunks.emplace_back(chunk.getShard(), chunk.getMin(), chunk.getMax());
+    }
+
+    doc.setPresetReshardedChunks(presetReshardedChunks);
+
+    auto coordinator = ReshardingCoordinator::getOrCreate(opCtx, _service, doc.toBSON());
+
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kPreparingToDonate);
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kPreparingToDonate);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kPreparingToDonate);
+    makeDonorsReadyToDonate(opCtx);
+
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kCloning);
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kCloning);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kCloning);
+
+    makeRecipientsFinishedCloning(opCtx);
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kApplying);
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kApplying);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kApplying);
+
+    coordinator->onOkayToEnterCritical();
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kBlockingWrites);
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kBlockingWrites);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kBlockingWrites);
+
+    makeRecipientsBeInStrictConsistency(opCtx);
+
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kCommitting);
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kCommitting);
+
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kCommitting);
+
+    makeDonorsProceedToDone(opCtx);
+    makeRecipientsProceedToDone(opCtx);
+
+    coordinator->getCompletionFuture().get(opCtx);
 }
 
 TEST_F(ReshardingCoordinatorServiceTest, ReshardingCoordinatorTransitionsTokDoneWithInterrupt) {
-    const auto interrupt = [this] { killAllReshardingCoordinatorOps(); };
-    runReshardingToCompletion(
-        TransitionFunctionMap{{CoordinatorStateEnum::kPreparingToDonate, interrupt},
-                              {CoordinatorStateEnum::kCloning, interrupt},
-                              {CoordinatorStateEnum::kApplying, interrupt},
-                              {CoordinatorStateEnum::kBlockingWrites, interrupt}});
-}
+    const std::vector<CoordinatorStateEnum> coordinatorStates{
+        CoordinatorStateEnum::kPreparingToDonate,
+        CoordinatorStateEnum::kCloning,
+        CoordinatorStateEnum::kApplying,
+        CoordinatorStateEnum::kBlockingWrites,
+        CoordinatorStateEnum::kCommitting};
+    PauseDuringStateTransitions stateTransitionsGuard{controller(), coordinatorStates};
 
-TEST_F(ReshardingCoordinatorServiceTest,
-       ReshardingCoordinatorTransitionsTokDoneWithTransactionFailWC) {
-    const auto failNextTransaction = [] {
-        globalFailPointRegistry()
-            .find("shardingCatalogManagerWithTransactionFailWCAfterCommit")
-            ->setMode(FailPoint::nTimes, 1);
-    };
-    // PauseDuringStateTransitions relies on updates to the coordinator state document on disk to
-    // decide when to unpause. kInitializing is the initial state written to disk (i.e. not an
-    // update), but we still want to verify correct behavior if the transaction to transition to
-    // kInitializing fails, so call failNextTransaction() before calling
-    // runReshardingToCompletion().
-    failNextTransaction();
-    runReshardingToCompletion(
-        TransitionFunctionMap{{CoordinatorStateEnum::kPreparingToDonate, failNextTransaction},
-                              {CoordinatorStateEnum::kCloning, failNextTransaction},
-                              {CoordinatorStateEnum::kApplying, failNextTransaction},
-                              {CoordinatorStateEnum::kBlockingWrites, failNextTransaction}});
+    auto doc = insertStateAndCatalogEntries(CoordinatorStateEnum::kUnused, _originalEpoch);
+    auto opCtx = operationContext();
+    auto donorChunk = makeAndInsertChunksForDonorShard(_originalUUID,
+                                                       _originalEpoch,
+                                                       _originalTimestamp,
+                                                       _oldShardKey,
+                                                       std::vector{OID::gen(), OID::gen()});
+
+    auto initialChunks = makeChunks(_reshardingUUID,
+                                    _tempEpoch,
+                                    _tempTimestamp,
+                                    _newShardKey,
+                                    std::vector{OID::gen(), OID::gen()});
+
+    std::vector<ReshardedChunk> presetReshardedChunks;
+    for (const auto& chunk : initialChunks) {
+        presetReshardedChunks.emplace_back(chunk.getShard(), chunk.getMin(), chunk.getMax());
+    }
+
+    doc.setPresetReshardedChunks(presetReshardedChunks);
+
+    auto coordinator = ReshardingCoordinator::getOrCreate(opCtx, _service, doc.toBSON());
+
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kPreparingToDonate);
+    killAllReshardingCoordinatorOps();
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kPreparingToDonate);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kPreparingToDonate);
+    makeDonorsReadyToDonate(opCtx);
+
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kCloning);
+    killAllReshardingCoordinatorOps();
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kCloning);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kCloning);
+
+    makeRecipientsFinishedCloning(opCtx);
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kApplying);
+    killAllReshardingCoordinatorOps();
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kApplying);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kApplying);
+
+    coordinator->onOkayToEnterCritical();
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kBlockingWrites);
+    killAllReshardingCoordinatorOps();
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kBlockingWrites);
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kBlockingWrites);
+
+    makeRecipientsBeInStrictConsistency(opCtx);
+
+    stateTransitionsGuard.wait(CoordinatorStateEnum::kCommitting);
+    stateTransitionsGuard.unset(CoordinatorStateEnum::kCommitting);
+
+    waitUntilCommittedCoordinatorDocReach(opCtx, CoordinatorStateEnum::kCommitting);
+
+    makeDonorsProceedToDone(opCtx);
+    makeRecipientsProceedToDone(opCtx);
+
+    coordinator->getCompletionFuture().get(opCtx);
 }
 
 TEST_F(ReshardingCoordinatorServiceTest, StepDownStepUpDuringInitializing) {
@@ -671,11 +682,17 @@ TEST_F(ReshardingCoordinatorServiceTest, StepDownStepUpDuringInitializing) {
     doc.setRecipientShards({});
     doc.setDonorShards({});
 
-    auto donorChunk = makeAndInsertChunksForDonorShard(
-        _originalUUID, _originalEpoch, _oldShardKey, std::vector{OID::gen(), OID::gen()});
+    auto donorChunk = makeAndInsertChunksForDonorShard(_originalUUID,
+                                                       _originalEpoch,
+                                                       _originalTimestamp,
+                                                       _oldShardKey,
+                                                       std::vector{OID::gen(), OID::gen()});
 
-    auto initialChunks =
-        makeChunks(_reshardingUUID, _tempEpoch, _newShardKey, std::vector{OID::gen(), OID::gen()});
+    auto initialChunks = makeChunks(_reshardingUUID,
+                                    _tempEpoch,
+                                    _tempTimestamp,
+                                    _newShardKey,
+                                    std::vector{OID::gen(), OID::gen()});
 
     std::vector<ReshardedChunk> presetReshardedChunks;
     for (const auto& chunk : initialChunks) {
@@ -731,11 +748,17 @@ TEST_F(ReshardingCoordinatorServiceTest, StepDownStepUpEachTransition) {
 
     auto doc = insertStateAndCatalogEntries(CoordinatorStateEnum::kUnused, _originalEpoch);
     auto opCtx = operationContext();
-    auto donorChunk = makeAndInsertChunksForDonorShard(
-        _originalUUID, _originalEpoch, _oldShardKey, std::vector{OID::gen(), OID::gen()});
+    auto donorChunk = makeAndInsertChunksForDonorShard(_originalUUID,
+                                                       _originalEpoch,
+                                                       _originalTimestamp,
+                                                       _oldShardKey,
+                                                       std::vector{OID::gen(), OID::gen()});
 
-    auto initialChunks =
-        makeChunks(_reshardingUUID, _tempEpoch, _newShardKey, std::vector{OID::gen(), OID::gen()});
+    auto initialChunks = makeChunks(_reshardingUUID,
+                                    _tempEpoch,
+                                    _tempTimestamp,
+                                    _newShardKey,
+                                    std::vector{OID::gen(), OID::gen()});
 
     std::vector<ReshardedChunk> presetReshardedChunks;
     for (const auto& chunk : initialChunks) {
@@ -845,7 +868,7 @@ TEST_F(ReshardingCoordinatorServiceTest, StepDownStepUpEachTransition) {
             ChunkType::ConfigNS, BSON(ChunkType::collectionUUID() << doc.getReshardingUUID()));
         while (chunkCursor->more()) {
             auto d = uassertStatusOK(ChunkType::fromConfigBSON(
-                chunkCursor->nextSafe().getOwned(), _originalEpoch, boost::none));
+                chunkCursor->nextSafe().getOwned(), _originalEpoch, _originalTimestamp));
             foundChunks.push_back(d);
         }
         ASSERT_EQUALS(foundChunks.size(), initialChunks.size());
@@ -866,11 +889,17 @@ TEST_F(ReshardingCoordinatorServiceTest, StepDownStepUpEachTransition) {
 TEST_F(ReshardingCoordinatorServiceTest, ReshardingCoordinatorFailsIfMigrationNotAllowed) {
     auto doc = insertStateAndCatalogEntries(CoordinatorStateEnum::kUnused, _originalEpoch);
     auto opCtx = operationContext();
-    auto donorChunk = makeAndInsertChunksForDonorShard(
-        _originalUUID, _originalEpoch, _oldShardKey, std::vector{OID::gen(), OID::gen()});
+    auto donorChunk = makeAndInsertChunksForDonorShard(_originalUUID,
+                                                       _originalEpoch,
+                                                       _originalTimestamp,
+                                                       _oldShardKey,
+                                                       std::vector{OID::gen(), OID::gen()});
 
-    auto initialChunks =
-        makeChunks(_reshardingUUID, _tempEpoch, _newShardKey, std::vector{OID::gen(), OID::gen()});
+    auto initialChunks = makeChunks(_reshardingUUID,
+                                    _tempEpoch,
+                                    _tempTimestamp,
+                                    _newShardKey,
+                                    std::vector{OID::gen(), OID::gen()});
 
     std::vector<ReshardedChunk> presetReshardedChunks;
     for (const auto& chunk : initialChunks) {
@@ -899,16 +928,5 @@ TEST_F(ReshardingCoordinatorServiceTest, ReshardingCoordinatorFailsIfMigrationNo
     }
 }
 
-TEST_F(ReshardingCoordinatorServiceTest, SuccessfullyAbortReshardOperationImmediately) {
-    auto pauseBeforeCTHolderInitialization =
-        globalFailPointRegistry().find("pauseBeforeCTHolderInitialization");
-    auto timesEnteredFailPoint = pauseBeforeCTHolderInitialization->setMode(FailPoint::alwaysOn, 0);
-    auto coordinator = initializeAndGetCoordinator();
-    coordinator->abort();
-    pauseBeforeCTHolderInitialization->waitForTimesEntered(timesEnteredFailPoint + 1);
-    pauseBeforeCTHolderInitialization->setMode(FailPoint::off, 0);
-    coordinator->getCompletionFuture().wait();
-}
-
 }  // namespace
 }  // namespace mongo
