@@ -98,18 +98,19 @@ boost::optional<Timestamp> _calculatePin(OperationContext* opCtx) {
 
     // If the RecoveryUnit already had an open snapshot, keep the snapshot open. Otherwise abandon
     // the snapshot when exitting the function.
-    auto scopeGuard = makeGuard([&] { opCtx->recoveryUnit()->abandonSnapshot(); });
+    ScopeGuard scopeGuard([&] { opCtx->recoveryUnit()->abandonSnapshot(); });
     if (opCtx->recoveryUnit()->isActive()) {
         scopeGuard.dismiss();
     }
 
-    AutoGetCollectionForRead autoColl(opCtx, NamespaceString::kDonorReshardingOperationsNamespace);
-    if (!autoColl) {
+    auto collection = CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(
+        opCtx, NamespaceString::kDonorReshardingOperationsNamespace);
+    if (!collection) {
         return boost::none;
     }
 
     Timestamp ret = Timestamp::max();
-    auto cursor = autoColl->getCursor(opCtx);
+    auto cursor = collection->getCursor(opCtx);
     for (auto doc = cursor->next(); doc; doc = cursor->next()) {
         if (auto fetchTs = parseNewMinFetchTimestampValue(doc.get().data.toBson()); fetchTs) {
             ret = std::min(ret, fetchTs.get());
