@@ -86,7 +86,7 @@ void runWithTransaction(OperationContext* opCtx,
     MongoDOperationContextSession ocs(asr.opCtx());
     auto txnParticipant = TransactionParticipant::get(asr.opCtx());
 
-    auto guard = makeGuard([opCtx = asr.opCtx(), &txnParticipant] {
+    ScopeGuard guard([opCtx = asr.opCtx(), &txnParticipant] {
         try {
             txnParticipant.abortTransaction(opCtx);
         } catch (DBException& e) {
@@ -96,7 +96,11 @@ void runWithTransaction(OperationContext* opCtx,
         }
     });
 
-    txnParticipant.beginOrContinue(asr.opCtx(), txnNumber, false, true);
+    txnParticipant.beginOrContinue(asr.opCtx(),
+                                   txnNumber,
+                                   false /* autocommit */,
+                                   true /* startTransaction */,
+                                   boost::none /* txnRetryCounter */);
     txnParticipant.unstashTransactionResources(asr.opCtx(), "reshardingOplogApplication");
 
     func(asr.opCtx());
