@@ -158,6 +158,22 @@ public:
         _metrics->onStart(ReshardingMetrics::Role::kRecipient,
                           getServiceContext()->getFastClockSource()->now());
         _metrics->setRecipientState(RecipientStateEnum::kApplying);
+
+        _executor = makeTaskExecutorForApplier();
+        _executor->startup();
+
+        _cancelableOpCtxExecutor = makeExecutorForCancelableOpCtx();
+        _cancelableOpCtxExecutor->startup();
+    }
+
+    void tearDown() {
+        _executor->shutdown();
+        _executor->join();
+
+        _cancelableOpCtxExecutor->shutdown();
+        _cancelableOpCtxExecutor->join();
+
+        ShardingMongodTestFixture::tearDown();
     }
 
     ChunkManager createChunkManagerForOriginalColl() {
@@ -167,20 +183,20 @@ public:
         const OID epoch = OID::gen();
         std::vector<ChunkType> chunks = {
             ChunkType{
-                kCrudNs,
+                kCrudUUID,
                 ChunkRange{BSON(kOriginalShardKey << MINKEY),
                            BSON(kOriginalShardKey << -std::numeric_limits<double>::infinity())},
-                ChunkVersion(1, 0, epoch, boost::none /* timestamp */),
+                ChunkVersion(1, 0, epoch, Timestamp(1, 1)),
                 _sourceId.getShardId()},
             ChunkType{
-                kCrudNs,
+                kCrudUUID,
                 ChunkRange{BSON(kOriginalShardKey << -std::numeric_limits<double>::infinity()),
                            BSON(kOriginalShardKey << 0)},
-                ChunkVersion(1, 0, epoch, boost::none /* timestamp */),
+                ChunkVersion(1, 0, epoch, Timestamp(1, 1)),
                 kOtherShardId},
-            ChunkType{kCrudNs,
+            ChunkType{kCrudUUID,
                       ChunkRange{BSON(kOriginalShardKey << 0), BSON(kOriginalShardKey << MAXKEY)},
-                      ChunkVersion(1, 0, epoch, boost::none /* timestamp */),
+                      ChunkVersion(1, 0, epoch, Timestamp(1, 1)),
                       _sourceId.getShardId()}};
 
         auto rt = RoutingTableHistory::makeNew(kCrudNs,
@@ -189,14 +205,15 @@ public:
                                                nullptr,
                                                false,
                                                epoch,
-                                               boost::none /* timestamp */,
+                                               Timestamp(1, 1),
                                                boost::none /* timeseriesFields */,
                                                boost::none,
+                                               boost::none /* chunkSizeBytes */,
                                                false,
                                                chunks);
 
         return ChunkManager(_sourceId.getShardId(),
-                            DatabaseVersion(UUID::gen()),
+                            DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
                             makeStandaloneRoutingTableHistory(std::move(rt)),
                             boost::none);
     }
@@ -256,12 +273,27 @@ public:
         return kStashCollections;
     }
 
+    BSONObj getMetricsOpCounters() {
+        BSONObjBuilder bob;
+        _metrics->serializeCumulativeOpMetrics(&bob);
+        return bob.obj().getObjectField("opcounters").getOwned();
+    }
+
     long long metricsAppliedCount() const {
         BSONObjBuilder bob;
         _metrics->serializeCurrentOpMetrics(&bob, ReshardingMetrics::Role::kRecipient);
         return bob.obj()["oplogEntriesApplied"_sd].Long();
     }
 
+    std::shared_ptr<executor::ThreadPoolTaskExecutor> getExecutor() {
+        return _executor;
+    }
+
+
+    std::shared_ptr<ThreadPool> getCancelableOpCtxExecutor() {
+        return _cancelableOpCtxExecutor;
+    }
+
 protected:
     auto makeApplierEnv() {
         return std::make_unique<ReshardingOplogApplier::Env>(getServiceContext(), &*_metrics);
@@ -294,20 +326,17 @@ protected:
             executor::makeNetworkInterface(
                 "TestReshardOplogApplicationNetwork", nullptr, std::move(hookList)));
 
-        executor->startup();
         return executor;
     }
 
-    CancelableOperationContextFactory makeCancelableOpCtxForApplier(CancellationToken cancelToken) {
-        auto executor = std::make_shared<ThreadPool>([] {
+    std::shared_ptr<ThreadPool> makeExecutorForCancelableOpCtx() {
+        return std::make_shared<ThreadPool>([] {
             ThreadPool::Options options;
             options.poolName = "TestReshardOplogApplierCancelableOpCtxPool";
             options.minThreads = 1;
             options.maxThreads = 1;
             return options;
         }());
-
-        return CancelableOperationContextFactory(cancelToken, executor);
     }
 
     static constexpr int kWriterPoolSize = 4;
@@ -324,6 +353,9 @@ protected:
 
     const ReshardingSourceId _sourceId{UUID::gen(), kMyShardId};
     std::unique_ptr<ReshardingMetrics> _metrics;
+
+    std::shared_ptr<executor::ThreadPoolTaskExecutor> _executor;
+    std::shared_ptr<ThreadPool> _cancelableOpCtxExecutor;
 };
 
 TEST_F(ReshardingOplogApplierTest, NothingToIterate) {
@@ -331,7 +363,6 @@ TEST_F(ReshardingOplogApplierTest, NothingToIterate) {
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 2 /* batchSize */);
 
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
 
     applier.emplace(makeApplierEnv(),
                     sourceId(),
@@ -342,8 +373,8 @@ TEST_F(ReshardingOplogApplierTest, NothingToIterate) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_OK(future.getNoThrow());
 }
 
@@ -368,7 +399,6 @@ TEST_F(ReshardingOplogApplierTest, ApplyBasicCrud) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 2 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -378,8 +408,8 @@ TEST_F(ReshardingOplogApplierTest, ApplyBasicCrud) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_OK(future.getNoThrow());
 
     DBDirectClient client(operationContext());
@@ -413,7 +443,6 @@ TEST_F(ReshardingOplogApplierTest, CanceledApplyingBatch) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 2 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
 
     applier.emplace(makeApplierEnv(),
                     sourceId(),
@@ -426,9 +455,9 @@ TEST_F(ReshardingOplogApplierTest, CanceledApplyingBatch) {
     auto abortSource = CancellationSource();
     abortSource.cancel();
     auto cancelToken = abortSource.token();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
 
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::CallbackCanceled);
 }
 
@@ -444,7 +473,6 @@ TEST_F(ReshardingOplogApplierTest, InsertTypeOplogAppliedInMultipleBatches) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 3 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -454,8 +482,8 @@ TEST_F(ReshardingOplogApplierTest, InsertTypeOplogAppliedInMultipleBatches) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_OK(future.getNoThrow());
 
     DBDirectClient client(operationContext());
@@ -485,7 +513,6 @@ TEST_F(ReshardingOplogApplierTest, ErrorDuringFirstBatchApply) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 4 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -495,8 +522,8 @@ TEST_F(ReshardingOplogApplierTest, ErrorDuringFirstBatchApply) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::FailedToParse);
 
     DBDirectClient client(operationContext());
@@ -528,7 +555,6 @@ TEST_F(ReshardingOplogApplierTest, ErrorDuringSecondBatchApply) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 2 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -538,8 +564,8 @@ TEST_F(ReshardingOplogApplierTest, ErrorDuringSecondBatchApply) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::FailedToParse);
 
     DBDirectClient client(operationContext());
@@ -570,7 +596,6 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingFirstOplog) {
     iterator->setThrowWhenSingleItem();
 
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -580,8 +605,8 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingFirstOplog) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::InternalError);
 
     DBDirectClient client(operationContext());
@@ -607,7 +632,6 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingFirstBatch) {
     iterator->setThrowWhenSingleItem();
 
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -617,8 +641,8 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingFirstBatch) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::InternalError);
 
     DBDirectClient client(operationContext());
@@ -648,7 +672,6 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingSecondBatch) {
     iterator->setThrowWhenSingleItem();
 
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -658,8 +681,8 @@ TEST_F(ReshardingOplogApplierTest, ErrorWhileIteratingSecondBatch) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::InternalError);
 
     DBDirectClient client(operationContext());
@@ -688,7 +711,6 @@ TEST_F(ReshardingOplogApplierTest, ExecutorIsShutDown) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(crudOps), 4 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -697,11 +719,11 @@ TEST_F(ReshardingOplogApplierTest, ExecutorIsShutDown) {
                     chunkManager(),
                     std::move(iterator));
 
-    executor->shutdown();
+    getExecutor()->shutdown();
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::ShutdownInProgress);
 
     DBDirectClient client(operationContext());
@@ -730,7 +752,6 @@ TEST_F(ReshardingOplogApplierTest, UnsupportedCommandOpsShouldError) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(ops), 1 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -740,8 +761,8 @@ TEST_F(ReshardingOplogApplierTest, UnsupportedCommandOpsShouldError) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::OplogOperationUnsupported);
 
     DBDirectClient client(operationContext());
@@ -767,7 +788,6 @@ TEST_F(ReshardingOplogApplierTest, DropSourceCollectionCmdShouldError) {
 
     auto iterator = std::make_unique<OplogIteratorMock>(std::move(ops), 1 /* batchSize */);
     boost::optional<ReshardingOplogApplier> applier;
-    auto executor = makeTaskExecutorForApplier();
     applier.emplace(makeApplierEnv(),
                     sourceId(),
                     appliedToNs(),
@@ -777,8 +797,8 @@ TEST_F(ReshardingOplogApplierTest, DropSourceCollectionCmdShouldError) {
                     std::move(iterator));
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier->run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier->run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_EQ(future.getNoThrow(), ErrorCodes::OplogOperationUnsupported);
 
     auto progressDoc = ReshardingOplogApplier::checkStoredProgress(operationContext(), sourceId());
@@ -786,7 +806,6 @@ TEST_F(ReshardingOplogApplierTest, DropSourceCollectionCmdShouldError) {
 }
 
 TEST_F(ReshardingOplogApplierTest, MetricsAreReported) {
-    auto executor = makeTaskExecutorForApplier();
     // Compress the makeOplog syntax a little further for this special case.
     using OpT = repl::OpTypeEnum;
     auto easyOp = [this](auto ts, OpT opType, BSONObj obj1, boost::optional<BSONObj> obj2 = {}) {
@@ -811,10 +830,15 @@ TEST_F(ReshardingOplogApplierTest, MetricsAreReported) {
     ASSERT_EQ(metricsAppliedCount(), 0);
 
     auto cancelToken = operationContext()->getCancellationToken();
-    auto factory = makeCancelableOpCtxForApplier(cancelToken);
-    auto future = applier.run(executor, executor, cancelToken, factory);
+    CancelableOperationContextFactory factory(cancelToken, getCancelableOpCtxExecutor());
+    auto future = applier.run(getExecutor(), getExecutor(), cancelToken, factory);
     ASSERT_OK(future.getNoThrow());
 
+    auto opCountersObj = getMetricsOpCounters();
+    ASSERT_EQ(opCountersObj.getIntField("insert"), 2);
+    ASSERT_EQ(opCountersObj.getIntField("update"), 1);
+    ASSERT_EQ(opCountersObj.getIntField("delete"), 2);
+
     // The in-memory metrics should show the 5 ops above + the final oplog entry, but on disk should
     // not include the final entry in its count.
     ASSERT_EQ(metricsAppliedCount(), 6);
