@@ -50,10 +50,8 @@
 #include "mongo/db/s/resharding/resharding_service_test_helpers.h"
 #include "mongo/logv2/log.h"
 #include "mongo/unittest/death_test.h"
-#include "mongo/util/clock_source_mock.h"
 #include "mongo/util/fail_point.h"
 
-
 namespace mongo {
 namespace {
 
@@ -80,9 +78,9 @@ public:
 
         const OID epoch = OID::gen();
         std::vector<ChunkType> chunks = {ChunkType{
-            nss,
+            _sourceUUID,
             ChunkRange{BSON(_currentShardKey << MINKEY), BSON(_currentShardKey << MAXKEY)},
-            ChunkVersion(100, 0, epoch, boost::none /* timestamp */),
+            ChunkVersion(100, 0, epoch, Timestamp(1, 1)),
             _someDonorId}};
 
         auto rt = RoutingTableHistory::makeNew(_sourceNss,
@@ -91,14 +89,15 @@ public:
                                                nullptr /* defaultCollator */,
                                                false /* unique */,
                                                std::move(epoch),
-                                               boost::none /* timestamp */,
+                                               Timestamp(1, 1),
                                                boost::none /* timeseriesFields */,
                                                boost::none /* reshardingFields */,
+                                               boost::none /* chunkSizeBytes */,
                                                true /* allowMigrations */,
                                                chunks);
 
         return ChunkManager(_someDonorId,
-                            DatabaseVersion(UUID::gen()),
+                            DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
                             _makeStandaloneRoutingTableHistory(std::move(rt)),
                             boost::none /* clusterTime */);
     }
@@ -140,7 +139,8 @@ private:
     RoutingTableHistoryValueHandle _makeStandaloneRoutingTableHistory(RoutingTableHistory rt) {
         const auto version = rt.getVersion();
         return RoutingTableHistoryValueHandle(
-            std::move(rt), ComparableChunkVersion::makeComparableChunkVersion(version));
+            std::make_shared<RoutingTableHistory>(std::move(rt)),
+            ComparableChunkVersion::makeComparableChunkVersion(version));
     }
 
     const StringData _currentShardKey = "oldKey";
@@ -215,14 +215,13 @@ public:
 
     void setUp() override {
         repl::PrimaryOnlyServiceMongoDTest::setUp();
-        auto clockSource = std::make_unique<ClockSourceMock>();
-        clockSource->advance(Seconds(1));
-        getServiceContext()->setFastClockSource(std::move(clockSource));
+
         auto serviceContext = getServiceContext();
         auto storageMock = std::make_unique<repl::StorageInterfaceMock>();
         repl::DropPendingCollectionReaper::set(
             serviceContext, std::make_unique<repl::DropPendingCollectionReaper>(storageMock.get()));
         repl::StorageInterface::set(serviceContext, std::move(storageMock));
+
         _controller = std::make_shared<RecipientStateTransitionController>();
         _opObserverRegistry->addObserver(std::make_unique<RecipientOpObserverForTest>(_controller));
     }
@@ -479,12 +478,6 @@ TEST_F(ReshardingRecipientServiceTest, OpCtxKilledWhileRestoringMetrics) {
 }
 
 DEATH_TEST_REGEX_F(ReshardingRecipientServiceTest, CommitFn, "4457001.*tripwire") {
-    // TODO (SERVER-57194): enable lock-free reads.
-    bool disableLockFreeReadsOriginalValue = storageGlobalParams.disableLockFreeReads;
-    storageGlobalParams.disableLockFreeReads = true;
-    ON_BLOCK_EXIT(
-        [&] { storageGlobalParams.disableLockFreeReads = disableLockFreeReadsOriginalValue; });
-
     auto doc = makeStateDocument(false /* isAlsoDonor */);
     auto opCtx = makeOperationContext();
     RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
@@ -502,81 +495,69 @@ DEATH_TEST_REGEX_F(ReshardingRecipientServiceTest, CommitFn, "4457001.*tripwire"
 TEST_F(ReshardingRecipientServiceTest, DropsTemporaryReshardingCollectionOnAbort) {
     auto metrics = ReshardingRecipientServiceTest::metrics();
     for (bool isAlsoDonor : {false, true}) {
-        for (bool waitForMetricsInitialized : {false, true}) {
-            LOGV2(5551107,
-                  "Running case",
-                  "test"_attr = _agent.getTestName(),
-                  "isAlsoDonor"_attr = isAlsoDonor,
-                  "waitForMetricsInitialized"_attr = waitForMetricsInitialized);
+        LOGV2(5551107,
+              "Running case",
+              "test"_attr = _agent.getTestName(),
+              "isAlsoDonor"_attr = isAlsoDonor);
 
-            boost::optional<PauseDuringStateTransitions> doneTransitionGuard;
-            doneTransitionGuard.emplace(controller(), RecipientStateEnum::kDone);
+        boost::optional<PauseDuringStateTransitions> doneTransitionGuard;
+        doneTransitionGuard.emplace(controller(), RecipientStateEnum::kDone);
 
-            auto doc = makeStateDocument(isAlsoDonor);
-            auto instanceId = BSON(ReshardingRecipientDocument::kReshardingUUIDFieldName
-                                   << doc.getReshardingUUID());
+        auto doc = makeStateDocument(isAlsoDonor);
+        auto instanceId =
+            BSON(ReshardingRecipientDocument::kReshardingUUIDFieldName << doc.getReshardingUUID());
 
-            auto opCtx = makeOperationContext();
+        auto opCtx = makeOperationContext();
 
-            if (isAlsoDonor) {
-                // If the recipient is also a donor, the original collection should already exist on
-                // this shard.
-                createSourceCollection(opCtx.get(), doc);
-            }
+        if (isAlsoDonor) {
+            // If the recipient is also a donor, the original collection should already exist on
+            // this shard.
+            createSourceCollection(opCtx.get(), doc);
+        }
 
-            RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
-            auto recipient =
-                RecipientStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
+        RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
+        auto recipient = RecipientStateMachine::getOrCreate(opCtx.get(), _service, doc.toBSON());
 
-            notifyToStartCloning(opCtx.get(), *recipient, doc);
-            if (waitForMetricsInitialized) {
-                // Waiting for the metrics to be initialized here causes the second abort to occur
-                // before the metrics are initialized after the step up, thereby testing a different
-                // code path.
-                doneTransitionGuard->wait(RecipientStateEnum::kCreatingCollection);
-            }
+        notifyToStartCloning(opCtx.get(), *recipient, doc);
+        recipient->abort(false);
 
-            recipient->abort(false);
+        doneTransitionGuard->wait(RecipientStateEnum::kDone);
+        stepDown();
 
-            doneTransitionGuard->wait(RecipientStateEnum::kDone);
+        ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
+                  ErrorCodes::InterruptedDueToReplStateChange);
 
-            stepDown();
+        recipient.reset();
+        stepUp(opCtx.get());
 
-            ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
-                      ErrorCodes::InterruptedDueToReplStateChange);
+        auto maybeRecipient = RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
+        ASSERT_TRUE(bool(maybeRecipient));
+        recipient = *maybeRecipient;
 
-            recipient.reset();
-            stepUp(opCtx.get());
+        doneTransitionGuard.reset();
+        recipient->abort(false);
 
-            auto maybeRecipient = RecipientStateMachine::lookup(opCtx.get(), _service, instanceId);
-            ASSERT_TRUE(bool(maybeRecipient));
-            recipient = *maybeRecipient;
+        ASSERT_OK(recipient->getCompletionFuture().getNoThrow());
+        checkStateDocumentRemoved(opCtx.get());
 
-            doneTransitionGuard.reset();
-            recipient->abort(false);
+        if (isAlsoDonor) {
+            // Verify original collection still exists after aborting.
+            AutoGetCollection coll(opCtx.get(), doc.getSourceNss(), MODE_IS);
+            ASSERT_TRUE(bool(coll));
+            ASSERT_EQ(coll->uuid(), doc.getSourceUUID());
+        }
 
-            ASSERT_OK(recipient->getCompletionFuture().getNoThrow());
-            checkStateDocumentRemoved(opCtx.get());
-
-            if (isAlsoDonor) {
-                // Verify original collection still exists after aborting.
-                AutoGetCollection coll(opCtx.get(), doc.getSourceNss(), MODE_IS);
-                ASSERT_TRUE(bool(coll));
-                ASSERT_EQ(coll->uuid(), doc.getSourceUUID());
-            }
-
-            // Verify the temporary collection no longer exists.
-            {
-                AutoGetCollection coll(opCtx.get(), doc.getTempReshardingNss(), MODE_IS);
-                ASSERT_FALSE(bool(coll));
-            }
+        // Verify the temporary collection no longer exists.
+        {
+            AutoGetCollection coll(opCtx.get(), doc.getTempReshardingNss(), MODE_IS);
+            ASSERT_FALSE(bool(coll));
         }
     }
 
     BSONObjBuilder result;
     metrics->serializeCumulativeOpMetrics(&result);
 
-    ASSERT_LESS_THAN_OR_EQUALS(result.obj().getField("countReshardingFailures").numberLong(), 4);
+    ASSERT_EQ(result.obj().getField("countReshardingFailures").numberLong(), 2);
 }
 
 TEST_F(ReshardingRecipientServiceTest, RenamesTemporaryReshardingCollectionWhenDone) {
@@ -617,12 +598,6 @@ TEST_F(ReshardingRecipientServiceTest, RenamesTemporaryReshardingCollectionWhenD
 }
 
 TEST_F(ReshardingRecipientServiceTest, WritesNoopOplogEntryOnReshardDoneCatchUp) {
-    // TODO (SERVER-57194): enable lock-free reads.
-    bool disableLockFreeReadsOriginalValue = storageGlobalParams.disableLockFreeReads;
-    storageGlobalParams.disableLockFreeReads = true;
-    ON_BLOCK_EXIT(
-        [&] { storageGlobalParams.disableLockFreeReads = disableLockFreeReadsOriginalValue; });
-
     boost::optional<PauseDuringStateTransitions> doneTransitionGuard;
     doneTransitionGuard.emplace(controller(), RecipientStateEnum::kDone);
 
@@ -668,12 +643,7 @@ TEST_F(ReshardingRecipientServiceTest, WritesNoopOplogEntryOnReshardDoneCatchUp)
 }
 
 TEST_F(ReshardingRecipientServiceTest, TruncatesXLErrorOnRecipientDocument) {
-    // TODO (SERVER-57194): enable lock-free reads.
     auto metrics = ReshardingRecipientServiceTest::metrics();
-    bool disableLockFreeReadsOriginalValue = storageGlobalParams.disableLockFreeReads;
-    storageGlobalParams.disableLockFreeReads = true;
-    ON_BLOCK_EXIT(
-        [&] { storageGlobalParams.disableLockFreeReads = disableLockFreeReadsOriginalValue; });
 
     for (bool isAlsoDonor : {false, true}) {
         LOGV2(5568600,
@@ -703,8 +673,8 @@ TEST_F(ReshardingRecipientServiceTest, TruncatesXLErrorOnRecipientDocument) {
             PersistentTaskStore<ReshardingRecipientDocument> store(
                 NamespaceString::kRecipientReshardingOperationsNamespace);
             store.forEach(opCtx.get(),
-                          QUERY(ReshardingRecipientDocument::kReshardingUUIDFieldName
-                                << doc.getReshardingUUID()),
+                          BSON(ReshardingRecipientDocument::kReshardingUUIDFieldName
+                               << doc.getReshardingUUID()),
                           [&](const auto& recipientDocument) {
                               persistedRecipientDocument.emplace(recipientDocument);
                               return false;
@@ -734,14 +704,7 @@ TEST_F(ReshardingRecipientServiceTest, TruncatesXLErrorOnRecipientDocument) {
 }
 
 TEST_F(ReshardingRecipientServiceTest, MetricsSuccessfullyShutDownOnUserCancelation) {
-    // TODO (SERVER-57194): enable lock-free reads.
     auto metrics = ReshardingRecipientServiceTest::metrics();
-    bool disableLockFreeReadsOriginalValue = storageGlobalParams.disableLockFreeReads;
-    storageGlobalParams.disableLockFreeReads = true;
-    ON_BLOCK_EXIT(
-        [&] { storageGlobalParams.disableLockFreeReads = disableLockFreeReadsOriginalValue; });
-
-
     auto doc = makeStateDocument(false);
     auto opCtx = makeOperationContext();
     RecipientStateMachine::insertStateDocument(opCtx.get(), doc);
@@ -767,12 +730,6 @@ TEST_F(ReshardingRecipientServiceTest, RestoreMetricsAfterStepUp) {
                                                           RecipientStateEnum::kApplying,
                                                           RecipientStateEnum::kStrictConsistency,
                                                           RecipientStateEnum::kDone};
-    // TODO (SERVER-57194): enable lock-free reads.
-    bool disableLockFreeReadsOriginalValue = storageGlobalParams.disableLockFreeReads;
-    storageGlobalParams.disableLockFreeReads = true;
-    ON_BLOCK_EXIT(
-        [&] { storageGlobalParams.disableLockFreeReads = disableLockFreeReadsOriginalValue; });
-
     PauseDuringStateTransitions stateTransitionsGuard{controller(), recipientStates};
     auto doc = makeStateDocument(false);
     auto instanceId =
@@ -857,33 +814,22 @@ TEST_F(ReshardingRecipientServiceTest, RestoreMetricsAfterStepUp) {
         }
         // Step down before the transition to state can complete.
         stateTransitionsGuard.wait(state);
-
-        dynamic_cast<ClockSourceMock*>(getServiceContext()->getFastClockSource())
-            ->advance(Seconds(1));
-        auto currOp =
-            recipient
-                ->reportForCurrentOp(MongoProcessInterface::CurrentOpConnectionsMode::kExcludeIdle,
-                                     MongoProcessInterface::CurrentOpSessionsMode::kExcludeIdle)
-                .get();
-
-
-        if (state == RecipientStateEnum::kApplying) {
-            ASSERT_EQ(currOp.getField("totalApplyTimeElapsedSecs").Long(), 0);
-            ASSERT_EQ(currOp.getStringField("recipientState"),
-                      RecipientState_serializer(RecipientStateEnum::kCloning));
-            ASSERT_GT(currOp.getField("totalCopyTimeElapsedSecs").Long(), 0);
-            ASSERT_GT(currOp.getField("approxBytesToCopy").Long(), 0);
-
-        } else if (state == RecipientStateEnum::kStrictConsistency) {
+        if (state == RecipientStateEnum::kStrictConsistency) {
+            auto currOp = recipient
+                              ->reportForCurrentOp(
+                                  MongoProcessInterface::CurrentOpConnectionsMode::kExcludeIdle,
+                                  MongoProcessInterface::CurrentOpSessionsMode::kExcludeIdle)
+                              .get();
             ASSERT_EQ(currOp.getField("documentsCopied").Long(), 1L);
             ASSERT_EQ(currOp.getField("bytesCopied").Long(), (long)reshardedDoc.objsize());
             ASSERT_EQ(currOp.getStringField("recipientState"),
                       RecipientState_serializer(RecipientStateEnum::kApplying));
-            ASSERT_GT(currOp.getField("totalCopyTimeElapsedSecs").Long(), 0);
-            ASSERT_GT(currOp.getField("totalApplyTimeElapsedSecs").Long(), 0);
-            ASSERT_GT(currOp.getField("approxBytesToCopy").Long(), 0);
-
         } else if (state == RecipientStateEnum::kDone) {
+            auto currOp = recipient
+                              ->reportForCurrentOp(
+                                  MongoProcessInterface::CurrentOpConnectionsMode::kExcludeIdle,
+                                  MongoProcessInterface::CurrentOpSessionsMode::kExcludeIdle)
+                              .get();
             ASSERT_EQ(currOp.getField("documentsCopied").Long(), 1L);
             ASSERT_EQ(currOp.getField("bytesCopied").Long(), (long)reshardedDoc.objsize());
             ASSERT_EQ(currOp.getField("oplogEntriesFetched").Long(),
@@ -892,11 +838,7 @@ TEST_F(ReshardingRecipientServiceTest, RestoreMetricsAfterStepUp) {
                       oplogEntriesAppliedOnEachDonor * doc.getDonorShards().size());
             ASSERT_EQ(currOp.getStringField("recipientState"),
                       RecipientState_serializer(RecipientStateEnum::kStrictConsistency));
-            ASSERT_GT(currOp.getField("totalCopyTimeElapsedSecs").Long(), 0);
-            ASSERT_GT(currOp.getField("totalApplyTimeElapsedSecs").Long(), 0);
-            ASSERT_GT(currOp.getField("approxBytesToCopy").Long(), 0);
         }
-
         stepDown();
 
         ASSERT_EQ(recipient->getCompletionFuture().getNoThrow(),
