@@ -243,9 +243,17 @@ protected:
         auto txnParticipant = TransactionParticipant::get(opCtx);
         ASSERT(txnParticipant);
         if (multiDocTxn) {
-            txnParticipant.beginOrContinue(opCtx, txnNum, false, true);
+            txnParticipant.beginOrContinue(opCtx,
+                                           txnNum,
+                                           false /* autocommit */,
+                                           true /* startTransaction */,
+                                           boost::none /* txnRetryCounter */);
         } else {
-            txnParticipant.beginOrContinue(opCtx, txnNum, boost::none, boost::none);
+            txnParticipant.beginOrContinue(opCtx,
+                                           txnNum,
+                                           boost::none /* autocommit */,
+                                           boost::none /* startTransaction */,
+                                           boost::none /* txnRetryCounter */);
         }
     }
 
@@ -253,9 +261,10 @@ protected:
         DBDirectClient client(operationContext());
         // The same logical session entry may be inserted more than once by a test case, so use a
         // $natural sort to find the most recently inserted entry.
-        Query oplogQuery(BSON(repl::OplogEntryBase::kSessionIdFieldName << sessionId.toBSON()));
-        auto bsonOplog = client.findOne(NamespaceString::kRsOplogNamespace.ns(),
-                                        oplogQuery.sort(BSON("$natural" << -1)));
+        auto bsonOplog =
+            client.findOne(NamespaceString::kRsOplogNamespace.ns(),
+                           BSON(repl::OplogEntryBase::kSessionIdFieldName << sessionId.toBSON()),
+                           Query().sort(BSON("$natural" << -1)));
         ASSERT(!bsonOplog.isEmpty());
         auto oplogEntry = repl::MutableOplogEntry::parse(bsonOplog).getValue();
         ASSERT_EQ(oplogEntry.getTxnNumber().get(), txnNum);
@@ -265,7 +274,7 @@ protected:
 
         auto bsonTxn =
             client.findOne(NamespaceString::kSessionTransactionsTableNamespace.ns(),
-                           {BSON(SessionTxnRecord::kSessionIdFieldName << sessionId.toBSON())});
+                           BSON(SessionTxnRecord::kSessionIdFieldName << sessionId.toBSON()));
         ASSERT(!bsonTxn.isEmpty());
         auto txn = SessionTxnRecord::parse(
             IDLParserErrorContext("resharding config transactions cloning test"), bsonTxn);
@@ -342,6 +351,7 @@ protected:
 
     std::shared_ptr<MongoProcessInterface> makeMongoProcessInterface() {
         return std::make_shared<ShardServerProcessInterface>(
+            operationContext(),
             Grid::get(getServiceContext())->getExecutorPool()->getFixedExecutor());
     }
 
@@ -387,8 +397,11 @@ protected:
 
         MongoDOperationContextSession ocs(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
-        txnParticipant.beginOrContinue(
-            opCtx, txnNumber, false /* autocommit */, true /* startTransaction */);
+        txnParticipant.beginOrContinue(opCtx,
+                                       txnNumber,
+                                       false /* autocommit */,
+                                       true /* startTransaction */,
+                                       boost::none /* txnRetryCounter */);
 
         txnParticipant.unstashTransactionResources(opCtx, "prepareTransaction");
 
@@ -408,8 +421,11 @@ protected:
 
         MongoDOperationContextSession ocs(opCtx);
         auto txnParticipant = TransactionParticipant::get(opCtx);
-        txnParticipant.beginOrContinue(
-            opCtx, txnNumber, false /* autocommit */, boost::none /* startTransaction */);
+        txnParticipant.beginOrContinue(opCtx,
+                                       txnNumber,
+                                       false /* autocommit */,
+                                       boost::none /* startTransaction */,
+                                       boost::none /* txnRetryCounter */);
 
         txnParticipant.unstashTransactionResources(opCtx, "abortTransaction");
         txnParticipant.abortTransaction(opCtx);
@@ -421,7 +437,7 @@ protected:
         std::vector<repl::DurableOplogEntry> result;
 
         PersistentTaskStore<repl::OplogEntryBase> store(NamespaceString::kRsOplogNamespace);
-        store.forEach(opCtx, QUERY("ts" << BSON("$gt" << ts)), [&](const auto& oplogEntry) {
+        store.forEach(opCtx, BSON("ts" << BSON("$gt" << ts)), [&](const auto& oplogEntry) {
             result.emplace_back(
                 unittest::assertGet(repl::DurableOplogEntry::parse(oplogEntry.toBSON())));
             return true;
@@ -437,7 +453,7 @@ protected:
         PersistentTaskStore<SessionTxnRecord> store(
             NamespaceString::kSessionTransactionsTableNamespace);
         store.forEach(opCtx,
-                      QUERY(SessionTxnRecord::kSessionIdFieldName << lsid.toBSON()),
+                      BSON(SessionTxnRecord::kSessionIdFieldName << lsid.toBSON()),
                       [&](const auto& sessionTxnRecord) {
                           result.emplace(sessionTxnRecord);
                           return false;
