@@ -43,7 +43,6 @@
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_util.h"
 #include "mongo/db/vector_clock.h"
-#include "mongo/executor/scoped_task_executor.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/metadata/impersonated_user_metadata.h"
 #include "mongo/s/catalog/type_chunk.h"
@@ -147,92 +146,6 @@ write_ops::UpdateCommandRequest buildNoopWriteRequestCommand() {
     return updateOp;
 }
 
-// Check that the collection UUID is the same in every shard knowing the collection
-void checkCollectionUUIDConsistencyAcrossShards(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const UUID& collectionUuid,
-    const std::vector<mongo::ShardId>& shardIds,
-    std::shared_ptr<executor::ScopedTaskExecutor> executor) {
-    const BSONObj filterObj = BSON("name" << nss.coll());
-    BSONObj cmdObj = BSON("listCollections" << 1 << "filter" << filterObj);
-
-    auto responses = sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx, nss.db().toString(), cmdObj, shardIds, **executor);
-
-    struct MismatchedShard {
-        std::string shardId;
-        std::string uuid;
-    };
-
-    std::vector<MismatchedShard> mismatches;
-
-    for (auto cmdResponse : responses) {
-        auto responseData = uassertStatusOK(cmdResponse.swResponse);
-        auto collectionVector = responseData.data.firstElement()["firstBatch"].Array();
-        auto shardId = cmdResponse.shardId;
-
-        if (collectionVector.empty()) {
-            // Collection does not exist on the shard
-            continue;
-        }
-
-        auto bsonCollectionUuid = collectionVector.front()["info"]["uuid"];
-        if (collectionUuid.data() != bsonCollectionUuid.uuid()) {
-            mismatches.push_back({shardId.toString(), bsonCollectionUuid.toString()});
-        }
-    }
-
-    if (!mismatches.empty()) {
-        std::stringstream errorMessage;
-        errorMessage << "The collection " << nss.toString()
-                     << " with expected UUID: " << collectionUuid.toString()
-                     << " has different UUIDs on the following shards: [";
-
-        for (auto mismatch : mismatches) {
-            errorMessage << "{ " << mismatch.shardId << ":" << mismatch.uuid << " },";
-        }
-        errorMessage << "]";
-        uasserted(ErrorCodes::InvalidUUID, errorMessage.str());
-    }
-}
-
-
-// Check the collection does not exist in any shard when `dropTarget` is set to false
-void checkTargetCollectionDoesNotExistInCluster(
-    OperationContext* opCtx,
-    const NamespaceString& toNss,
-    const std::vector<mongo::ShardId>& shardIds,
-    std::shared_ptr<executor::ScopedTaskExecutor> executor) {
-    const BSONObj filterObj = BSON("name" << toNss.coll());
-    BSONObj cmdObj = BSON("listCollections" << 1 << "filter" << filterObj);
-
-    auto responses = sharding_ddl_util::sendAuthenticatedCommandToShards(
-        opCtx, toNss.db(), cmdObj, shardIds, **executor);
-
-    std::vector<std::string> shardsContainingTargetCollection;
-    for (auto cmdResponse : responses) {
-        uassertStatusOK(cmdResponse.swResponse);
-        auto responseData = uassertStatusOK(cmdResponse.swResponse);
-        auto collectionVector = responseData.data.firstElement()["firstBatch"].Array();
-
-        if (!collectionVector.empty()) {
-            shardsContainingTargetCollection.push_back(cmdResponse.shardId.toString());
-        }
-    }
-
-    if (!shardsContainingTargetCollection.empty()) {
-        std::stringstream errorMessage;
-        errorMessage << "The collection " << toNss.toString()
-                     << " already exists in the following shards: [";
-        std::move(shardsContainingTargetCollection.begin(),
-                  shardsContainingTargetCollection.end(),
-                  std::ostream_iterator<std::string>(errorMessage, ", "));
-        errorMessage << "]";
-        uasserted(ErrorCodes::NamespaceExists, errorMessage.str());
-    }
-}
-
 }  // namespace
 
 void linearizeCSRSReads(OperationContext* opCtx) {
@@ -247,20 +160,27 @@ void linearizeCSRSReads(OperationContext* opCtx) {
         ShardingCatalogClient::kMajorityWriteConcern));
 }
 
-std::vector<AsyncRequestsSender::Response> sendAuthenticatedCommandToShards(
-    OperationContext* opCtx,
-    StringData dbName,
-    const BSONObj& command,
-    const std::vector<ShardId>& shardIds,
-    const std::shared_ptr<executor::TaskExecutor>& executor) {
+void sendAuthenticatedCommandToShards(OperationContext* opCtx,
+                                      StringData dbName,
+                                      const BSONObj& command,
+                                      const std::vector<ShardId>& shardIds,
+                                      const std::shared_ptr<executor::TaskExecutor>& executor) {
+    // TODO SERVER-57519: remove the following scope
+    {
+        // Ensure ShardRegistry is initialized before using the AsyncRequestsSender that relies on
+        // unsafe functions (SERVER-57280)
+        auto shardRegistry = Grid::get(opCtx)->shardRegistry();
+        if (!shardRegistry->isUp()) {
+            shardRegistry->reload(opCtx);
+        }
+    }
 
     // The AsyncRequestsSender ignore impersonation metadata so we need to manually attach them to
     // the command
     BSONObjBuilder bob(command);
     rpc::writeAuthDataToImpersonatedUserMetadata(opCtx, &bob);
     auto authenticatedCommand = bob.obj();
-    return sharding_util::sendCommandToShards(
-        opCtx, dbName, authenticatedCommand, shardIds, executor);
+    sharding_util::sendCommandToShards(opCtx, dbName, authenticatedCommand, shardIds, executor);
 }
 
 void removeTagsMetadataFromConfig(OperationContext* opCtx,
@@ -390,7 +310,7 @@ void shardedRenameMetadata(OperationContext* opCtx,
         // Only bump the epoch if the whole cluster is in FCV 5.0, so chunks do not contain epochs.
         FixedFCVRegion fixedFCVRegion(opCtx);
         if (serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-                ServerGlobalParams::FeatureCompatibility::Version::kVersion50)) {
+                multiversion::FeatureCompatibilityVersion::kFullyDowngradedTo_5_0)) {
             fromCollType.setEpoch(OID::gen());
         }
     }
@@ -400,35 +320,9 @@ void shardedRenameMetadata(OperationContext* opCtx,
         opCtx, CollectionType::ConfigNS, fromCollType.toBSON(), writeConcern));
 }
 
-void checkCatalogConsistencyAcrossShardsForRename(
-    OperationContext* opCtx,
-    const NamespaceString& fromNss,
-    const NamespaceString& toNss,
-    const bool dropTarget,
-    std::shared_ptr<executor::ScopedTaskExecutor> executor) {
-
-    auto participants = Grid::get(opCtx)->shardRegistry()->getAllShardIds(opCtx);
-
-    auto sourceCollUuid = *getCollectionUUID(opCtx, fromNss);
-    checkCollectionUUIDConsistencyAcrossShards(
-        opCtx, fromNss, sourceCollUuid, participants, executor);
-
-    if (!dropTarget) {
-        checkTargetCollectionDoesNotExistInCluster(opCtx, toNss, participants, executor);
-    }
-}
-
-void checkRenamePreconditions(OperationContext* opCtx,
-                              bool sourceIsSharded,
-                              const NamespaceString& toNss,
-                              bool dropTarget) {
-    if (sourceIsSharded) {
-        uassert(ErrorCodes::InvalidNamespace,
-                str::stream() << "Namespace of target collection too long. Namespace: " << toNss
-                              << " Max: " << NamespaceString::MaxNsShardedCollectionLen,
-                toNss.size() <= NamespaceString::MaxNsShardedCollectionLen);
-    }
-
+void checkShardedRenamePreconditions(OperationContext* opCtx,
+                                     const NamespaceString& toNss,
+                                     const bool dropTarget) {
     auto catalogClient = Grid::get(opCtx)->catalogClient();
     if (!dropTarget) {
         // Check that the sharded target collection doesn't exist
@@ -594,11 +488,5 @@ void sendDropCollectionParticipantCommandToShards(OperationContext* opCtx,
     }
 }
 
-BSONObj getCriticalSectionReasonForRename(const NamespaceString& from, const NamespaceString& to) {
-    return BSON("command"
-                << "rename"
-                << "from" << from.toString() << "to" << to.toString());
-}
-
 }  // namespace sharding_ddl_util
 }  // namespace mongo
