@@ -90,14 +90,14 @@ bool checkIfSingleDoc(OperationContext* opCtx,
 
 /**
  * Checks the collection's metadata for a successful split on the specified chunkRange using the
- * specified split points. Returns false if the metadata's chunks don't match the new chunk
+ * specified splitKeys. Returns false if the metadata's chunks don't match the new chunk
  * boundaries exactly.
  */
 bool checkMetadataForSuccessfulSplitChunk(OperationContext* opCtx,
                                           const NamespaceString& nss,
                                           const OID& epoch,
                                           const ChunkRange& chunkRange,
-                                          const std::vector<BSONObj>& splitPoints) {
+                                          const std::vector<BSONObj>& splitKeys) {
     AutoGetCollection autoColl(opCtx, nss, MODE_IS);
     const auto metadataAfterSplit =
         CollectionShardingRuntime::get(opCtx, nss)->getCurrentMetadataIfKnown();
@@ -106,21 +106,19 @@ bool checkMetadataForSuccessfulSplitChunk(OperationContext* opCtx,
             str::stream() << "Collection " << nss.ns() << " changed since split start",
             metadataAfterSplit && metadataAfterSplit->getShardVersion().epoch() == epoch);
 
+    auto newChunkBounds(splitKeys);
+    auto startKey = chunkRange.getMin();
+    newChunkBounds.push_back(chunkRange.getMax());
+
     ChunkType nextChunk;
-    for (auto it = splitPoints.begin(); it != splitPoints.end(); ++it) {
+    for (const auto& endKey : newChunkBounds) {
         // Check that all new chunks fit the new chunk boundaries
-        const auto& currentChunkMinKey =
-            it == splitPoints.begin() ? chunkRange.getMin() : *std::prev(it);
-        const auto& currentChunkMaxKey = *it;
-        if (!metadataAfterSplit->getNextChunk(currentChunkMinKey, &nextChunk) ||
-            nextChunk.getMax().woCompare(currentChunkMaxKey)) {
+        if (!metadataAfterSplit->getNextChunk(startKey, &nextChunk) ||
+            nextChunk.getMax().woCompare(endKey)) {
             return false;
         }
-    }
-    // Special check for the last chunk produced.
-    if (!metadataAfterSplit->getNextChunk(splitPoints.back(), &nextChunk) ||
-        nextChunk.getMax().woCompare(chunkRange.getMax())) {
-        return false;
+
+        startKey = endKey;
     }
 
     return true;
@@ -132,7 +130,7 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
                                                    const NamespaceString& nss,
                                                    const BSONObj& keyPatternObj,
                                                    const ChunkRange& chunkRange,
-                                                   std::vector<BSONObj>&& splitPoints,
+                                                   const std::vector<BSONObj>& splitKeys,
                                                    const std::string& shardName,
                                                    const OID& expectedCollectionEpoch) {
     auto scopedSplitOrMergeChunk(uassertStatusOK(
@@ -142,8 +140,8 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     // data types.
     const auto hashedField = ShardKeyPattern::extractHashedField(keyPatternObj);
     if (hashedField) {
-        for (const auto& splitPoint : splitPoints) {
-            auto hashedSplitElement = splitPoint[hashedField.fieldName()];
+        for (BSONObj splitKey : splitKeys) {
+            auto hashedSplitElement = splitKey[hashedField.fieldName()];
             if (!ShardKeyPattern::isValidHashedValue(hashedSplitElement)) {
                 return {ErrorCodes::CannotSplit,
                         str::stream() << "splitChunk cannot split chunk " << chunkRange.toString()
@@ -155,8 +153,8 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     }
 
     // Commit the split to the config server.
-    auto request = SplitChunkRequest(
-        nss, shardName, expectedCollectionEpoch, chunkRange, std::move(splitPoints));
+    auto request =
+        SplitChunkRequest(nss, shardName, expectedCollectionEpoch, chunkRange, splitKeys);
 
     auto configCmdObj =
         request.toConfigCommandBSON(ShardingCatalogClient::kMajorityWriteConcern.toBSON());
@@ -206,7 +204,7 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
     if (!commandStatus.isOK() || !writeConcernStatus.isOK()) {
 
         if (checkMetadataForSuccessfulSplitChunk(
-                opCtx, nss, expectedCollectionEpoch, chunkRange, request.getSplitPoints())) {
+                opCtx, nss, expectedCollectionEpoch, chunkRange, splitKeys)) {
             // Split was committed.
         } else if (!commandStatus.isOK()) {
             return commandStatus;
@@ -226,26 +224,27 @@ StatusWith<boost::optional<ChunkRange>> splitChunk(OperationContext* opCtx,
 
     // Allow multiKey based on the invariant that shard keys must be single-valued. Therefore,
     // any multi-key index prefixed by shard key cannot be multikey over the shard key fields.
-    const IndexDescriptor* idx =
-        collection->getIndexCatalog()->findShardKeyPrefixedIndex(opCtx, keyPatternObj, false);
-    if (!idx) {
+    auto catalog = collection->getIndexCatalog();
+    auto shardKeyIdx = catalog->findShardKeyPrefixedIndex(
+        opCtx, *collection, keyPatternObj, /*requireSingleKey=*/false);
+    if (!shardKeyIdx) {
         return boost::optional<ChunkRange>(boost::none);
     }
 
     auto backChunk = ChunkType();
-    backChunk.setMin(request.getSplitPoints().back());
+    backChunk.setMin(splitKeys.back());
     backChunk.setMax(chunkRange.getMax());
 
     auto frontChunk = ChunkType();
     frontChunk.setMin(chunkRange.getMin());
-    frontChunk.setMax(request.getSplitPoints().front());
+    frontChunk.setMax(splitKeys.front());
 
     KeyPattern shardKeyPattern(keyPatternObj);
     if (shardKeyPattern.globalMax().woCompare(backChunk.getMax()) == 0 &&
-        checkIfSingleDoc(opCtx, collection.getCollection(), idx, &backChunk)) {
+        checkIfSingleDoc(opCtx, collection.getCollection(), shardKeyIdx, &backChunk)) {
         return boost::optional<ChunkRange>(ChunkRange(backChunk.getMin(), backChunk.getMax()));
     } else if (shardKeyPattern.globalMin().woCompare(frontChunk.getMin()) == 0 &&
-               checkIfSingleDoc(opCtx, collection.getCollection(), idx, &frontChunk)) {
+               checkIfSingleDoc(opCtx, collection.getCollection(), shardKeyIdx, &frontChunk)) {
         return boost::optional<ChunkRange>(ChunkRange(frontChunk.getMin(), frontChunk.getMax()));
     }
     return boost::optional<ChunkRange>(boost::none);
