@@ -71,7 +71,7 @@ public:
      *
      * If no coordinator for the (lsid, txnNumber) exists, returns boost::none.
      */
-    boost::optional<Future<txn::CommitDecision>> coordinateCommit(
+    boost::optional<SharedSemiFuture<txn::CommitDecision>> coordinateCommit(
         OperationContext* opCtx,
         LogicalSessionId lsid,
         TxnNumber txnNumber,
@@ -83,9 +83,9 @@ public:
      *
      * If no coordinator for the (lsid, txnNumber) exists, returns boost::none.
      */
-    boost::optional<Future<txn::CommitDecision>> recoverCommit(OperationContext* opCtx,
-                                                               LogicalSessionId lsid,
-                                                               TxnNumber txnNumber);
+    boost::optional<SharedSemiFuture<txn::CommitDecision>> recoverCommit(OperationContext* opCtx,
+                                                                         LogicalSessionId lsid,
+                                                                         TxnNumber txnNumber);
 
     /**
      * Marks the coordinator catalog as stepping up, which blocks all incoming requests for
@@ -102,6 +102,11 @@ public:
     void onStepUp(OperationContext* opCtx, Milliseconds recoveryDelayForTesting = Milliseconds(0));
     void onStepDown();
 
+    /**
+     * Shuts down this service. This will no longer be usable once shutdown is called.
+     */
+    void shutdown();
+
     /**
      * Called when an already established replica set is added as a shard to a cluster. Ensures that
      * the TransactionCoordinator service is started up if the replica set is currently primary.
@@ -135,8 +140,8 @@ private:
     };
 
     /**
-     * Returns the current catalog + scheduler if stepUp has started, otherwise throws a NotMaster
-     * exception.
+     * Returns the current catalog + scheduler if stepUp has started, otherwise throws a
+     * NotWritablePrimary exception.
      */
     std::shared_ptr<CatalogAndScheduler> _getCatalogAndScheduler(OperationContext* opCtx);
 
@@ -151,6 +156,9 @@ private:
     // The catalog + scheduler instantiated at the last step-up attempt. When nullptr, it means
     // onStepUp has not been called yet after the last stepDown (or construction).
     std::shared_ptr<CatalogAndScheduler> _catalogAndScheduler;
+
+    // Sets to false once shutdown was called at least once.
+    bool _isShuttingDown{false};
 };
 
 }  // namespace mongo
