@@ -337,13 +337,6 @@ NetworkCounter networkCounter;
 AuthCounter authCounter;
 AggStageCounters aggStageCounters;
 DotsAndDollarsFieldsCounters dotsAndDollarsFieldsCounters;
-OperatorCountersAggExpressions operatorCountersAggExpressions;
+OperatorCountersExpressions operatorCountersExpressions;
 OperatorCountersMatchExpressions operatorCountersMatchExpressions;
-
-Counter64 updateManyCount;
-ServerStatusMetricField<Counter64> displayUpdateManyCount("query.updateManyCount",
-                                                          &updateManyCount);
-Counter64 deleteManyCount;
-ServerStatusMetricField<Counter64> displayeleteManyCount("query.deleteManyCount", &deleteManyCount);
-
 }  // namespace mongo
