@@ -32,8 +32,8 @@
 #include <algorithm>
 #include <numeric>
 
-#include "mongo/db/catalog/disable_index_spec_namespace_generation_gen.h"
 #include "mongo/db/field_ref.h"
+#include "mongo/db/server_options.h"
 
 namespace mongo {
 
@@ -57,7 +57,7 @@ void appendMultikeyPathsAsBytes(BSONObj keyPattern,
     char multikeyPathsEncodedAsBytes[kMaxKeyPatternPathLength];
 
     size_t i = 0;
-    for (const auto keyElem : keyPattern) {
+    for (const auto& keyElem : keyPattern) {
         StringData keyName = keyElem.fieldNameStringData();
         size_t numParts = FieldRef{keyName}.numParts();
         invariant(numParts > 0);
@@ -84,7 +84,7 @@ void appendMultikeyPathsAsBytes(BSONObj keyPattern,
 void parseMultikeyPathsFromBytes(BSONObj multikeyPathsObj, MultikeyPaths* multikeyPaths) {
     invariant(multikeyPaths);
     for (auto elem : multikeyPathsObj) {
-        std::set<size_t> multikeyComponents;
+        MultikeyComponents multikeyComponents;
         int len;
         const char* data = elem.binData(len);
         invariant(len > 0);
@@ -101,9 +101,6 @@ void parseMultikeyPathsFromBytes(BSONObj multikeyPathsObj, MultikeyPaths* multik
 
 }  // namespace
 
-const StringData BSONCollectionCatalogEntry::kIndexBuildScanning = "scanning"_sd;
-const StringData BSONCollectionCatalogEntry::kIndexBuildDraining = "draining"_sd;
-
 // --------------------------
 
 void BSONCollectionCatalogEntry::IndexMetaData::updateTTLSetting(long long newExpireSeconds) {
@@ -120,15 +117,53 @@ void BSONCollectionCatalogEntry::IndexMetaData::updateTTLSetting(long long newEx
     spec = b.obj();
 }
 
+
+void BSONCollectionCatalogEntry::IndexMetaData::updateHiddenSetting(bool hidden) {
+    // If hidden == false, we remove this field from catalog rather than add a field with false.
+    // or else, the old binary can't startup due to the unknown field.
+    BSONObjBuilder b;
+    for (BSONObjIterator bi(spec); bi.more();) {
+        BSONElement e = bi.next();
+        if (e.fieldNameStringData() == "hidden") {
+            continue;
+        }
+        b.append(e);
+    }
+
+    if (hidden) {
+        b.append("hidden", hidden);
+    }
+    spec = b.obj();
+}
+
+
 // --------------------------
 
+int BSONCollectionCatalogEntry::MetaData::getTotalIndexCount() const {
+    return std::count_if(
+        indexes.cbegin(), indexes.cend(), [](const auto& index) { return index.isPresent(); });
+}
+
 int BSONCollectionCatalogEntry::MetaData::findIndexOffset(StringData name) const {
     for (unsigned i = 0; i < indexes.size(); i++)
-        if (indexes[i].name() == name)
+        if (indexes[i].nameStringData() == name)
             return i;
     return -1;
 }
 
+void BSONCollectionCatalogEntry::MetaData::insertIndex(IndexMetaData indexMetaData) {
+    int indexOffset = findIndexOffset(indexMetaData.nameStringData());
+
+    if (indexOffset < 0) {
+        indexes.push_back(std::move(indexMetaData));
+        return;
+    }
+
+    // We have an unused element, was invalidated due to an index drop, that can be reused
+    // for this new index.
+    indexes[indexOffset] = std::move(indexMetaData);
+}
+
 bool BSONCollectionCatalogEntry::MetaData::eraseIndex(StringData name) {
     int indexOffset = findIndexOffset(name);
 
@@ -136,76 +171,53 @@ bool BSONCollectionCatalogEntry::MetaData::eraseIndex(StringData name) {
         return false;
     }
 
-    indexes.erase(indexes.begin() + indexOffset);
+    // Zero out the index metadata to be reused later and to keep the indexes of other indexes
+    // stable.
+    indexes[indexOffset] = {};
+
     return true;
 }
 
-void BSONCollectionCatalogEntry::MetaData::rename(StringData toNS) {
-    ns = toNS.toString();
-    for (size_t i = 0; i < indexes.size(); i++) {
-        BSONObj spec = indexes[i].spec;
-        BSONObjBuilder b;
-        // Add the fields in the same order they were in the original specification.
-        for (auto&& elem : spec) {
-            if (elem.fieldNameStringData() == "ns") {
-                b.append("ns", toNS);
-            } else {
-                b.append(elem);
-            }
-        }
-        indexes[i].spec = b.obj();
-    }
-}
-
-KVPrefix BSONCollectionCatalogEntry::MetaData::getMaxPrefix() const {
-    // Use the collection prefix as the initial max value seen. Then compare it with each index
-    // prefix. Note the oplog has no indexes so the vector of 'IndexMetaData' may be empty.
-    return std::accumulate(
-        indexes.begin(), indexes.end(), prefix, [](KVPrefix max, IndexMetaData index) {
-            return max < index.prefix ? index.prefix : max;
-        });
-}
-
-BSONObj BSONCollectionCatalogEntry::MetaData::toBSON() const {
+BSONObj BSONCollectionCatalogEntry::MetaData::toBSON(bool hasExclusiveAccess) const {
     BSONObjBuilder b;
     b.append("ns", ns);
     b.append("options", options.toBSON());
     {
         BSONArrayBuilder arr(b.subarrayStart("indexes"));
         for (unsigned i = 0; i < indexes.size(); i++) {
+            if (!indexes[i].isPresent()) {
+                continue;
+            }
+
             BSONObjBuilder sub(arr.subobjStart());
             sub.append("spec", indexes[i].spec);
             sub.appendBool("ready", indexes[i].ready);
-            sub.appendBool("multikey", indexes[i].multikey);
+            {
+                stdx::unique_lock lock(indexes[i].multikeyMutex, stdx::defer_lock_t{});
+                if (!hasExclusiveAccess) {
+                    lock.lock();
+                }
+                sub.appendBool("multikey", indexes[i].multikey);
 
-            if (!indexes[i].multikeyPaths.empty()) {
-                BSONObjBuilder subMultikeyPaths(sub.subobjStart("multikeyPaths"));
-                appendMultikeyPathsAsBytes(indexes[i].spec.getObjectField("key"),
-                                           indexes[i].multikeyPaths,
-                                           &subMultikeyPaths);
-                subMultikeyPaths.doneFast();
+                if (!indexes[i].multikeyPaths.empty()) {
+                    BSONObjBuilder subMultikeyPaths(sub.subobjStart("multikeyPaths"));
+                    appendMultikeyPathsAsBytes(indexes[i].spec.getObjectField("key"),
+                                               indexes[i].multikeyPaths,
+                                               &subMultikeyPaths);
+                    subMultikeyPaths.doneFast();
+                }
             }
 
             sub.append("head", 0ll);  // For backward compatibility with 4.0
-            sub.append("prefix", indexes[i].prefix.toBSONValue());
             sub.append("backgroundSecondary", indexes[i].isBackgroundSecondaryBuild);
 
-            sub.append("runTwoPhaseBuild", indexes[i].runTwoPhaseBuild);
-            sub.append("versionOfBuild", static_cast<long long>(indexes[i].versionOfBuild));
-            if (indexes[i].buildPhase) {
-                sub.append("buildPhase", *indexes[i].buildPhase);
-            }
-            if (indexes[i].constraintViolationsIdent) {
-                sub.append("constraintViolationsIdent", *indexes[i].constraintViolationsIdent);
-            }
-            if (indexes[i].sideWritesIdent) {
-                sub.append("sideWritesIdent", *indexes[i].sideWritesIdent);
+            if (indexes[i].buildUUID) {
+                indexes[i].buildUUID->appendToBuilder(&sub, "buildUUID");
             }
             sub.doneFast();
         }
         arr.doneFast();
     }
-    b.append("prefix", prefix.toBSONValue());
     return b.obj();
 }
 
@@ -213,8 +225,8 @@ void BSONCollectionCatalogEntry::MetaData::parse(const BSONObj& obj) {
     ns = obj["ns"].valuestrsafe();
 
     if (obj["options"].isABSONObj()) {
-        options.parse(obj["options"].Obj(), CollectionOptions::parseForStorage)
-            .transitional_ignore();
+        options = uassertStatusOK(
+            CollectionOptions::parse(obj["options"].Obj(), CollectionOptions::parseForStorage));
     }
 
     BSONElement indexList = obj["indexes"];
@@ -231,28 +243,23 @@ void BSONCollectionCatalogEntry::MetaData::parse(const BSONObj& obj) {
                 parseMultikeyPathsFromBytes(multikeyPathsElem.Obj(), &imd.multikeyPaths);
             }
 
-            imd.prefix = KVPrefix::fromBSONElement(idx["prefix"]);
             auto bgSecondary = BSONElement(idx["backgroundSecondary"]);
             // Opt-in to rebuilding behavior for old-format index catalog objects.
             imd.isBackgroundSecondaryBuild = bgSecondary.eoo() || bgSecondary.trueValue();
 
-            imd.runTwoPhaseBuild = idx["runTwoPhaseBuild"].trueValue();
-            if (idx.hasField("versionOfBuild")) {
-                imd.versionOfBuild = idx["versionOfBuild"].numberLong();
+            if (idx["buildUUID"]) {
+                imd.buildUUID = fassert(31353, UUID::parse(idx["buildUUID"]));
             }
-            if (idx["buildPhase"]) {
-                imd.buildPhase = idx["buildPhase"].str();
-            }
-            if (idx["constraintViolationsIdent"]) {
-                imd.constraintViolationsIdent = idx["constraintViolationsIdent"].str();
-            }
-            if (idx["sideWritesIdent"]) {
-                imd.sideWritesIdent = idx["sideWritesIdent"].str();
+
+            if (!imd.isPresent()) {
+                fassertFailedWithStatus(
+                    5738500,
+                    Status(ErrorCodes::FailedToParse,
+                           str::stream() << "invalid index in collection metadata: " << obj));
             }
+
             indexes.push_back(imd);
         }
     }
-
-    prefix = KVPrefix::fromBSONElement(obj["prefix"]);
 }
 }  // namespace mongo
