@@ -128,9 +128,7 @@ public:
         // Make sure we have a valid IndexSpec for the type requested
         IndexSpec spec;
         spec.version(1).name(indexName).addKeys(keyPattern);
-        if (indexType == IndexNames::GEO_HAYSTACK) {
-            spec.geoHaystackBucketSize(1.0);
-        } else if (indexType == IndexNames::TEXT) {
+        if (indexType == IndexNames::TEXT) {
             spec.textWeights(BSON("a" << 1));
             spec.textIndexVersion(2);
             spec.textDefaultLanguage("swedish");
@@ -396,8 +394,7 @@ TEST_F(DurableCatalogTest, PathLevelMultikeyTrackingIsSupportedBy2dsphereIndexes
 }
 
 TEST_F(DurableCatalogTest, PathLevelMultikeyTrackingIsNotSupportedByAllIndexTypes) {
-    std::string indexTypes[] = {
-        IndexNames::GEO_2D, IndexNames::GEO_HAYSTACK, IndexNames::TEXT, IndexNames::HASHED};
+    std::string indexTypes[] = {IndexNames::GEO_2D, IndexNames::TEXT, IndexNames::HASHED};
 
     for (auto&& indexType : indexTypes) {
         auto indexEntry = createIndex(BSON("a" << indexType << "b" << 1), indexType);
@@ -465,134 +462,6 @@ TEST_F(DurableCatalogTest, NoOpWhenEntireIndexAlreadySetAsMultikey) {
     }
 }
 
-class ConcurrentMultikeyTest : public DurableCatalogTest {
-public:
-    void testConcurrentMultikey(BSONObj keyPattern,
-                                const MultikeyPaths& first,
-                                const MultikeyPaths& second,
-                                const MultikeyPaths& expected) {
-        /*
-         * This test verifies that we can set multikey on two threads concurrently with the
-         * following interleaving that do not cause a WCE from the storage engine:
-         *
-         * T1: open storage snapshot
-         *
-         * T1: set multikey paths to {first}
-         *
-         * T1: commit
-         *
-         * T2: open storage snapshot
-         *
-         * T2: set multikey paths to {second}
-         *
-         * T2: commit
-         *
-         * T1: onCommit handler
-         *
-         * T2: onCommit handler
-         *
-         */
-
-        auto indexEntry = createIndex(keyPattern);
-        auto collection = getCollection();
-
-        mongo::Mutex mutex;
-        stdx::condition_variable cv;
-        int numMultikeyCalls = 0;
-
-        // Start a thread that will set multikey paths to 'first'. It will commit the change to the
-        // storage engine but block before running the onCommit handler that updates the in-memory
-        // state in the Collection instance.
-        stdx::thread t([svcCtx = getServiceContext(),
-                        &collection,
-                        &indexEntry,
-                        &first,
-                        &mutex,
-                        &cv,
-                        &numMultikeyCalls] {
-            ThreadClient client(svcCtx);
-            auto opCtx = client->makeOperationContext();
-
-            Lock::GlobalLock globalLock{opCtx.get(), MODE_IX};
-            WriteUnitOfWork wuow(opCtx.get());
-
-            // Register a onCommit that will block until the main thread has committed its multikey
-            // write. This onCommit handler is registered before any writes and will thus be
-            // performed first, blocking all other onCommit handlers.
-            opCtx->recoveryUnit()->onCommit(
-                [&mutex, &cv, &numMultikeyCalls](boost::optional<Timestamp> commitTime) {
-                    stdx::unique_lock lock(mutex);
-
-                    // Let the main thread now we have committed to the storage engine
-                    numMultikeyCalls = 1;
-                    cv.notify_all();
-
-                    // Wait until the main thread has committed its multikey write
-                    cv.wait(lock, [&numMultikeyCalls]() { return numMultikeyCalls == 2; });
-                });
-
-            // Set the index to multikey with 'first' as paths.
-            collection->setIndexIsMultikey(
-                opCtx.get(), indexEntry->descriptor()->indexName(), first);
-            wuow.commit();
-        });
-
-        // Wait for the thread above to commit its multikey write to the storage engine
-        {
-            stdx::unique_lock lock(mutex);
-            cv.wait(lock, [&numMultikeyCalls]() { return numMultikeyCalls == 1; });
-        }
-
-        // Set the index to multikey with 'second' as paths. This will not cause a WCE as the write
-        // in the thread is fully committed to the storage engine.
-        {
-            Lock::GlobalLock globalLock{operationContext(), MODE_IX};
-            // First confirm that we can observe the multikey write set by the other thread.
-            MultikeyPaths paths;
-            ASSERT_TRUE(collection->isIndexMultikey(
-                operationContext(), indexEntry->descriptor()->indexName(), &paths));
-            assertMultikeyPathsAreEqual(paths, first);
-
-            // Then perform our own multikey write.
-            WriteUnitOfWork wuow(operationContext());
-            collection->setIndexIsMultikey(
-                operationContext(), indexEntry->descriptor()->indexName(), second);
-            wuow.commit();
-        }
-
-        // Notify the thread that our multikey write is committed
-        {
-            stdx::unique_lock lock(mutex);
-            numMultikeyCalls = 2;
-            cv.notify_all();
-        }
-        t.join();
-
-        // Verify that our Collection instance has 'expected' as multikey paths for this index
-        {
-            MultikeyPaths multikeyPaths;
-            ASSERT(collection->isIndexMultikey(
-                operationContext(), indexEntry->descriptor()->indexName(), &multikeyPaths));
-            assertMultikeyPathsAreEqual(multikeyPaths, expected);
-        }
-
-        // Verify that the durable catalog has 'expected' as multikey paths for this index
-        Lock::GlobalLock globalLock{operationContext(), MODE_IS};
-        auto md = getCatalog()->getMetaData(operationContext(), collection->getCatalogId());
-
-        auto indexOffset = md->findIndexOffset(indexEntry->descriptor()->indexName());
-        assertMultikeyPathsAreEqual(md->indexes[indexOffset].multikeyPaths, expected);
-    }
-};
-
-TEST_F(ConcurrentMultikeyTest, MultikeyPathsConcurrentSecondSubset) {
-    testConcurrentMultikey(BSON("a.b" << 1), {{0U, 1U}}, {{0U}}, {{0U, 1U}});
-}
-
-TEST_F(ConcurrentMultikeyTest, MultikeyPathsConcurrentDistinct) {
-    testConcurrentMultikey(BSON("a.b" << 1), {{0U}}, {{1U}}, {{0U, 1U}});
-}
-
 TEST_F(DurableCatalogTest, SinglePhaseIndexBuild) {
     auto indexEntry = createIndex(BSON("a" << 1));
     auto collection = getCollection();
