@@ -43,7 +43,7 @@
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/server_options.h"
-#include "mongo/db/storage/durable_catalog_feature_tracker.h"
+#include "mongo/db/storage/durable_catalog_impl.h"
 #include "mongo/db/storage/durable_history_pin.h"
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/kv/temporary_kv_record_store.h"
@@ -149,18 +149,8 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx, LastShutdownState l
         _catalogRecordStore.get(), _options.directoryPerDB, _options.directoryForIndexes, this));
     _catalog->init(opCtx);
 
-    // We populate 'identsKnownToStorageEngine' only if:
-    // - doing repair; or
-    // - or asked to recover orphaned idents, which is the case when loading after an unclean
-    //   shutdown.
-    auto loadingFromUncleanShutdownOrRepair =
-        lastShutdownState == LastShutdownState::kUnclean || _options.forRepair;
-
-    std::vector<std::string> identsKnownToStorageEngine;
-    if (loadingFromUncleanShutdownOrRepair) {
-        identsKnownToStorageEngine = _engine->getAllIdents(opCtx);
-        std::sort(identsKnownToStorageEngine.begin(), identsKnownToStorageEngine.end());
-    }
+    std::vector<std::string> identsKnownToStorageEngine = _engine->getAllIdents(opCtx);
+    std::sort(identsKnownToStorageEngine.begin(), identsKnownToStorageEngine.end());
 
     std::vector<DurableCatalog::Entry> catalogEntries = _catalog->getAllCatalogEntries(opCtx);
 
@@ -236,7 +226,8 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx, LastShutdownState l
         }
     }
 
-    BatchedCollectionCatalogWriter catalogBatchWriter{opCtx};
+    const auto loadingFromUncleanShutdownOrRepair =
+        lastShutdownState == LastShutdownState::kUnclean || _options.forRepair;
     for (DurableCatalog::Entry entry : catalogEntries) {
         if (loadingFromUncleanShutdownOrRepair) {
             // If we are loading the catalog after an unclean shutdown or during repair, it's
@@ -275,6 +266,26 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx, LastShutdownState l
             }
         }
 
+        if (!entry.nss.isReplicated() &&
+            !std::binary_search(identsKnownToStorageEngine.begin(),
+                                identsKnownToStorageEngine.end(),
+                                entry.ident)) {
+            // All collection drops are non-transactional and unreplicated collections are dropped
+            // immediately as they do not use two-phase drops. It's possible to run into a situation
+            // where there are collections in the catalog that are unknown to the storage engine
+            // after restoring from backed up data files. See SERVER-55552.
+            WriteUnitOfWork wuow(opCtx);
+            fassert(5555200, _catalog->_removeEntry(opCtx, entry.catalogId));
+            wuow.commit();
+
+            LOGV2_INFO(5555201,
+                       "Removed unknown unreplicated collection from the catalog",
+                       "catalogId"_attr = entry.catalogId,
+                       logAttrs(entry.nss),
+                       "ident"_attr = entry.ident);
+            continue;
+        }
+
         Timestamp minVisibleTs = Timestamp::min();
         // If there's no recovery timestamp, every collection is available.
         if (boost::optional<Timestamp> recoveryTs = _engine->getRecoveryTimestamp()) {
@@ -303,10 +314,7 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx, LastShutdownState l
         _initCollection(opCtx, entry.catalogId, entry.nss, _options.forRepair, minVisibleTs);
 
         if (entry.nss.isOrphanCollection()) {
-            LOGV2(22248,
-                  "Orphaned collection found: {namespace}",
-                  "Orphaned collection found",
-                  "namespace"_attr = entry.nss);
+            LOGV2(22248, "Orphaned collection found", "namespace"_attr = entry.nss);
         }
     }
 
@@ -366,8 +374,6 @@ Status StorageEngineImpl::_recoverOrphanedCollection(OperationContext* opCtx,
         return {ErrorCodes::IllegalOperation, "Orphan recovery only supported in repair"};
     }
     LOGV2(22249,
-          "Storage engine is missing collection '{namespace}' from its metadata. Attempting "
-          "to locate and recover the data for {ident}",
           "Storage engine is missing collection from its metadata. Attempting to locate and "
           "recover the data",
           "namespace"_attr = collectionName,
@@ -535,7 +541,6 @@ StatusWith<StorageEngine::ReconcileResult> StorageEngineImpl::reconcileCatalogAn
         // checkpoint.
         if (dropPendingIdents.find(it) != dropPendingIdents.cend()) {
             LOGV2(22250,
-                  "Not removing ident for uncheckpointed collection or index drop: {ident}",
                   "Not removing ident for uncheckpointed collection or index drop",
                   "ident"_attr = it);
             continue;
@@ -576,8 +581,8 @@ StatusWith<StorageEngine::ReconcileResult> StorageEngineImpl::reconcileCatalogAn
         // Batch up the indexes to remove them from `metaData` outside of the iterator.
         std::vector<std::string> indexesToDrop;
         for (const auto& indexMetaData : metaData->indexes) {
-            const std::string& indexName = indexMetaData.name();
-            std::string indexIdent = _catalog->getIndexIdent(opCtx, entry.catalogId, indexName);
+            auto indexName = indexMetaData.nameStringData();
+            auto indexIdent = _catalog->getIndexIdent(opCtx, entry.catalogId, indexName);
 
             // Warn in case of incorrect "multikeyPath" information in catalog documents. This is
             // the result of a concurrency bug which has since been fixed, but may persist in
@@ -630,8 +635,6 @@ StatusWith<StorageEngine::ReconcileResult> StorageEngineImpl::reconcileCatalogAn
                 auto buildUUID = *indexMetaData.buildUUID;
 
                 LOGV2(22253,
-                      "Found index from unfinished build. Collection: {coll} ({uuid}), index: "
-                      "{indexName}, build UUID: {buildUUID}",
                       "Found index from unfinished build",
                       "namespace"_attr = coll,
                       "uuid"_attr = *collUUID,
@@ -659,7 +662,8 @@ StatusWith<StorageEngine::ReconcileResult> StorageEngineImpl::reconcileCatalogAn
                       "- see SERVER-43097",
                       "namespace"_attr = coll,
                       "index"_attr = indexName);
-                reconcileResult.indexesToRebuild.push_back({entry.catalogId, coll, indexName});
+                reconcileResult.indexesToRebuild.push_back(
+                    {entry.catalogId, coll, indexName.toString()});
                 continue;
             }
 
@@ -676,7 +680,7 @@ StatusWith<StorageEngine::ReconcileResult> StorageEngineImpl::reconcileCatalogAn
                       "index"_attr = indexName);
                 // Ensure the `ident` is dropped while we have the `indexIdent` value.
                 fassert(50713, _engine->dropIdent(opCtx->recoveryUnit(), indexIdent));
-                indexesToDrop.push_back(indexName);
+                indexesToDrop.push_back(indexName.toString());
                 continue;
             }
         }
@@ -718,11 +722,10 @@ std::string StorageEngineImpl::getFilesystemPathForDb(const std::string& dbName)
 
 void StorageEngineImpl::cleanShutdown() {
     if (_timestampMonitor) {
-        _timestampMonitor->removeListener(&_minOfCheckpointAndOldestTimestampListener);
+        _timestampMonitor->clearListeners();
     }
 
     CollectionCatalog::write(getGlobalServiceContext(), [](CollectionCatalog& catalog) {
-        catalog.onCloseCatalog();
         catalog.deregisterAllCollectionsAndViews();
     });
 
@@ -739,10 +742,10 @@ StorageEngineImpl::~StorageEngineImpl() {}
 
 void StorageEngineImpl::finishInit() {
     if (_engine->supportsRecoveryTimestamp()) {
-        _timestampMonitor = std::make_unique<TimestampMonitor>(
-            _engine.get(), getGlobalServiceContext()->getPeriodicRunner());
-        _timestampMonitor->startup();
-        _timestampMonitor->addListener(&_minOfCheckpointAndOldestTimestampListener);
+        _timestampMonitor =
+            std::make_unique<TimestampMonitor>(_engine.get(),
+                                               &_minOfCheckpointAndOldestTimestampListener,
+                                               getGlobalServiceContext()->getPeriodicRunner());
     }
 }
 
@@ -803,7 +806,7 @@ Status StorageEngineImpl::_dropCollectionsNoTimestamp(OperationContext* opCtx,
     }
 
     // Ensure the method exits with the same "commit timestamp" state that it was called with.
-    auto addCommitTimestamp = makeGuard([&opCtx, commitTs] {
+    ScopeGuard addCommitTimestamp([&opCtx, commitTs] {
         if (!commitTs.isNull()) {
             opCtx->recoveryUnit()->setCommitTimestamp(commitTs);
         }
@@ -879,7 +882,8 @@ void StorageEngineImpl::endNonBlockingBackup(OperationContext* opCtx) {
     return _engine->endNonBlockingBackup(opCtx);
 }
 
-StatusWith<std::deque<std::string>> StorageEngineImpl::extendBackupCursor(OperationContext* opCtx) {
+StatusWith<std::vector<std::string>> StorageEngineImpl::extendBackupCursor(
+    OperationContext* opCtx) {
     return _engine->extendBackupCursor(opCtx);
 }
 
@@ -999,18 +1003,15 @@ bool StorageEngineImpl::supportsRecoveryTimestamp() const {
 StatusWith<Timestamp> StorageEngineImpl::recoverToStableTimestamp(OperationContext* opCtx) {
     invariant(opCtx->lockState()->isW());
 
-    // The "feature document" should not be rolled back. Perform a non-timestamped update to the
-    // feature document to lock in the current state.
-    DurableCatalogImpl::FeatureTracker::FeatureBits featureInfo;
-    {
-        WriteUnitOfWork wuow(opCtx);
-        featureInfo = _catalog->getFeatureTracker()->getInfo(opCtx);
-        _catalog->getFeatureTracker()->putInfo(opCtx, featureInfo);
-        wuow.commit();
-    }
-
     auto state = catalog::closeCatalog(opCtx);
 
+    // SERVER-58311: Reset the recovery unit to unposition storage engine cursors. This allows WT to
+    // assert it has sole access when performing rollback_to_stable().
+    auto recovUnit = opCtx->releaseRecoveryUnit();
+    recovUnit.reset();
+    opCtx->setRecoveryUnit(std::unique_ptr<RecoveryUnit>(newRecoveryUnit()),
+                           WriteUnitOfWork::RecoveryUnitState::kNotInUnitOfWork);
+
     StatusWith<Timestamp> swTimestamp = _engine->recoverToStableTimestamp(opCtx);
     if (!swTimestamp.isOK()) {
         return swTimestamp;
@@ -1046,10 +1047,7 @@ bool StorageEngineImpl::supportsOplogStones() const {
 }
 
 bool StorageEngineImpl::supportsResumableIndexBuilds() const {
-    return enableResumableIndexBuilds && supportsReadConcernMajority() && !isEphemeral() &&
-        serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-        serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-            ServerGlobalParams::FeatureCompatibility::Version::kVersion47) &&
+    return supportsReadConcernMajority() && !isEphemeral() &&
         !repl::ReplSettings::shouldRecoverFromOplogAsStandalone();
 }
 
@@ -1118,16 +1116,12 @@ void StorageEngineImpl::_onMinOfCheckpointAndOldestTimestampChanged(const Timest
     }
 }
 
-StorageEngineImpl::TimestampMonitor::TimestampMonitor(KVEngine* engine, PeriodicRunner* runner)
+StorageEngineImpl::TimestampMonitor::TimestampMonitor(KVEngine* engine,
+                                                      TimestampListener* listener,
+                                                      PeriodicRunner* runner)
     : _engine(engine), _running(false), _periodicRunner(runner) {
-    _currentTimestamps.checkpoint = _engine->getCheckpointTimestamp();
-    _currentTimestamps.oldest = _engine->getOldestTimestamp();
-    _currentTimestamps.stable = _engine->getStableTimestamp();
-    _currentTimestamps.minOfCheckpointAndOldest =
-        (_currentTimestamps.checkpoint.isNull() ||
-         (_currentTimestamps.checkpoint > _currentTimestamps.oldest))
-        ? _currentTimestamps.oldest
-        : _currentTimestamps.checkpoint;
+    _listeners.push_back(listener);
+    _startup();
 }
 
 StorageEngineImpl::TimestampMonitor::~TimestampMonitor() {
@@ -1136,7 +1130,7 @@ StorageEngineImpl::TimestampMonitor::~TimestampMonitor() {
     invariant(_listeners.empty());
 }
 
-void StorageEngineImpl::TimestampMonitor::startup() {
+void StorageEngineImpl::TimestampMonitor::_startup() {
     invariant(!_running);
 
     LOGV2(22262, "Timestamp monitor starting");
@@ -1150,11 +1144,6 @@ void StorageEngineImpl::TimestampMonitor::startup() {
                 }
             }
 
-            Timestamp checkpoint = _currentTimestamps.checkpoint;
-            Timestamp oldest = _currentTimestamps.oldest;
-            Timestamp stable = _currentTimestamps.stable;
-            Timestamp minOfCheckpointAndOldest = _currentTimestamps.minOfCheckpointAndOldest;
-
             try {
                 auto opCtx = client->getOperationContext();
                 mongo::ServiceContext::UniqueOperationContext uOpCtx;
@@ -1163,6 +1152,11 @@ void StorageEngineImpl::TimestampMonitor::startup() {
                     opCtx = uOpCtx.get();
                 }
 
+                Timestamp checkpoint;
+                Timestamp oldest;
+                Timestamp stable;
+                Timestamp minOfCheckpointAndOldest;
+
                 {
                     // Take a global lock in MODE_IS while fetching timestamps to guarantee that
                     // rollback-to-stable isn't running concurrently.
@@ -1182,20 +1176,14 @@ void StorageEngineImpl::TimestampMonitor::startup() {
                 {
                     stdx::lock_guard<Latch> lock(_monitorMutex);
                     for (const auto& listener : _listeners) {
-                        // Notify the listener if the timestamp changed.
-                        if (listener->getType() == TimestampType::kCheckpoint &&
-                            _currentTimestamps.checkpoint != checkpoint) {
+                        if (listener->getType() == TimestampType::kCheckpoint) {
                             listener->notify(checkpoint);
-                        } else if (listener->getType() == TimestampType::kOldest &&
-                                   _currentTimestamps.oldest != oldest) {
+                        } else if (listener->getType() == TimestampType::kOldest) {
                             listener->notify(oldest);
-                        } else if (listener->getType() == TimestampType::kStable &&
-                                   _currentTimestamps.stable != stable) {
+                        } else if (listener->getType() == TimestampType::kStable) {
                             listener->notify(stable);
                         } else if (listener->getType() ==
-                                       TimestampType::kMinOfCheckpointAndOldest &&
-                                   _currentTimestamps.minOfCheckpointAndOldest !=
-                                       minOfCheckpointAndOldest) {
+                                   TimestampType::kMinOfCheckpointAndOldest) {
                             listener->notify(minOfCheckpointAndOldest);
                         } else if (stable == Timestamp::min()) {
                             // Special case notification of all listeners when writes do not have
@@ -1205,20 +1193,17 @@ void StorageEngineImpl::TimestampMonitor::startup() {
                     }
                 }
 
-                _currentTimestamps.checkpoint = checkpoint;
-                _currentTimestamps.oldest = oldest;
-                _currentTimestamps.stable = stable;
-                _currentTimestamps.minOfCheckpointAndOldest = minOfCheckpointAndOldest;
             } catch (const ExceptionForCat<ErrorCategory::Interruption>& ex) {
                 if (!ErrorCodes::isCancellationError(ex))
                     throw;
                 // If we're interrupted at shutdown or after PeriodicRunner's client has been
                 // killed, it's fine to give up on future notifications.
-                LOGV2(22263,
-                      "Timestamp monitor is stopping. {reason}",
-                      "Timestamp monitor is stopping",
-                      "error"_attr = ex.reason());
+                LOGV2(22263, "Timestamp monitor is stopping", "error"_attr = ex);
                 return;
+            } catch (const DBException& ex) {
+                // Logs and rethrows the exceptions of other types.
+                LOGV2_ERROR(5802500, "Timestamp monitor throws an exception", "error"_attr = ex);
+                throw;
             }
         },
         Seconds(1));
@@ -1228,7 +1213,7 @@ void StorageEngineImpl::TimestampMonitor::startup() {
     _running = true;
 }
 
-void StorageEngineImpl::TimestampMonitor::addListener(TimestampListener* listener) {
+void StorageEngineImpl::TimestampMonitor::addListener_forTestOnly(TimestampListener* listener) {
     stdx::lock_guard<Latch> lock(_monitorMutex);
     if (std::find(_listeners.begin(), _listeners.end(), listener) != _listeners.end()) {
         bool listenerAlreadyRegistered = true;
@@ -1237,13 +1222,9 @@ void StorageEngineImpl::TimestampMonitor::addListener(TimestampListener* listene
     _listeners.push_back(listener);
 }
 
-void StorageEngineImpl::TimestampMonitor::removeListener(TimestampListener* listener) {
+void StorageEngineImpl::TimestampMonitor::clearListeners() {
     stdx::lock_guard<Latch> lock(_monitorMutex);
-    if (std::find(_listeners.begin(), _listeners.end(), listener) == _listeners.end()) {
-        bool listenerNotRegistered = true;
-        invariant(!listenerNotRegistered);
-    }
-    _listeners.erase(std::remove(_listeners.begin(), _listeners.end(), listener));
+    _listeners.clear();
 }
 
 int64_t StorageEngineImpl::sizeOnDiskForDb(OperationContext* opCtx, StringData dbName) {
@@ -1280,8 +1261,12 @@ void StorageEngineImpl::setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp
     _engine->setPinnedOplogTimestamp(pinnedTimestamp);
 }
 
-StatusWith<BSONObj> StorageEngineImpl::getSanitizedStorageOptionsForSecondaryReplication(
-    const BSONObj& options) const {
-    return _engine->getSanitizedStorageOptionsForSecondaryReplication(options);
+DurableCatalog* StorageEngineImpl::getCatalog() {
+    return _catalog.get();
 }
+
+const DurableCatalog* StorageEngineImpl::getCatalog() const {
+    return _catalog.get();
+}
+
 }  // namespace mongo
