@@ -38,7 +38,7 @@
 #include "mongo/base/init.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/json.h"
 #include "mongo/db/operation_context_noop.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
@@ -47,7 +47,6 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_size_storer.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
-#include "mongo/unittest/barrier.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/fail_point.h"
@@ -204,7 +203,6 @@ TEST(WiredTigerRecordStoreTest, Isolation2) {
     }
 }
 
-
 StatusWith<RecordId> insertBSON(ServiceContext::UniqueOperationContext& opCtx,
                                 unique_ptr<RecordStore>& rs,
                                 const Timestamp& opTime) {
@@ -856,7 +854,7 @@ TEST(WiredTigerRecordStoreTest, OplogStones_AscendingOrder) {
 
 // Ensure that if we sample and create duplicate oplog stones, perform truncation correctly, and
 // with no crashing behavior. This scenario may be possible if the same record is sampled multiple
-// times during startup, which can be very likely if the size storer is very innacurate.
+// times during startup, which can be very likely if the size storer is very inaccurate.
 TEST(WiredTigerRecordStoreTest, OplogStones_Duplicates) {
     std::unique_ptr<RecordStoreHarnessHelper> harnessHelper = newRecordStoreHarnessHelper();
     auto wtHarnessHelper = dynamic_cast<WiredTigerHarnessHelper*>(harnessHelper.get());
@@ -879,7 +877,7 @@ TEST(WiredTigerRecordStoreTest, OplogStones_Duplicates) {
     wtKvEngine->getOplogManager()->setOplogReadTimestamp(Timestamp(4, 0));
 
     {
-        // Force initialize the oplog stones to use sampling by providing very large, innacurate
+        // Force initialize the oplog stones to use sampling by providing very large, inaccurate
         // sizes. This should cause us to oversample the records in the oplog.
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
         wtrs->setNumRecords(1024 * 1024);
@@ -1047,54 +1045,5 @@ TEST(WiredTigerRecordStoreTest, CursorInActiveTxnAfterSeek) {
     }
 }
 
-// Make sure numRecords is accurate after a delete rolls back and some other transaction deletes the
-// same rows before we have a chance of patching up the metadata.
-TEST(WiredTigerRecordStoreTest, NumRecordsAccurateAfterRollbackWithDelete) {
-    const auto harnessHelper(newRecordStoreHarnessHelper());
-    unique_ptr<RecordStore> rs(harnessHelper->newNonCappedRecordStore());
-
-    RecordId rid;  // This record will be deleted by two transactions.
-
-    ServiceContext::UniqueOperationContext ctx(harnessHelper->newOperationContext());
-    {
-        WriteUnitOfWork uow(ctx.get());
-        rid = rs->insertRecord(ctx.get(), "a", 2, Timestamp()).getValue();
-        uow.commit();
-    }
-
-    ASSERT_EQ(1, rs->numRecords(ctx.get()));
-
-    WriteUnitOfWork uow(ctx.get());
-
-    auto aborted = std::make_shared<unittest::Barrier>(2);
-    auto deleted = std::make_shared<unittest::Barrier>(2);
-
-    // This thread will delete the record and then rollback. We'll block the roll back process after
-    // rolling back the WT transaction and before running the rest of the registered changes,
-    // allowing the main thread to delete the same rows again.
-    stdx::thread abortedThread([&harnessHelper, &rs, &rid, aborted, deleted]() {
-        auto client = harnessHelper->serviceContext()->makeClient("c1");
-        auto ctx = harnessHelper->newOperationContext(client.get());
-        WriteUnitOfWork txn(ctx.get());
-        // Registered changes are executed in reverse order.
-        rs->deleteRecord(ctx.get(), rid);
-        ctx.get()->recoveryUnit()->onRollback([&]() { deleted->countDownAndWait(); });
-        ctx.get()->recoveryUnit()->onRollback([&]() { aborted->countDownAndWait(); });
-    });
-
-    // Wait for the other thread to abort.
-    aborted->countDownAndWait();
-
-    rs->deleteRecord(ctx.get(), rid);
-
-    // Notify the other thread we have deleted, let it complete the rollback.
-    deleted->countDownAndWait();
-
-    uow.commit();
-
-    abortedThread.join();
-    ASSERT_EQ(0, rs->numRecords(ctx.get()));
-}
-
 }  // namespace
 }  // namespace mongo
