@@ -38,6 +38,7 @@
 #include "mongo/base/init.h"
 #include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/json.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
@@ -97,8 +98,7 @@ TEST(WiredTigerRecordStoreTest, SizeStorer1) {
     rs.reset(nullptr);
 
     {
-        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        auto& info = *ss.load(opCtx.get(), uri);
+        auto& info = *ss.load(uri);
         ASSERT_EQUALS(N, info.numRecords.load());
     }
 
@@ -135,7 +135,7 @@ TEST(WiredTigerRecordStoreTest, SizeStorer1) {
         {
             WriteUnitOfWork uow(opCtx.get());
             WT_SESSION* s = ru->getSession()->getSession();
-            invariantWTOK(s->create(s, indexUri.c_str(), ""), s);
+            invariantWTOK(s->create(s, indexUri.c_str(), ""));
             uow.commit();
         }
 
@@ -146,7 +146,7 @@ TEST(WiredTigerRecordStoreTest, SizeStorer1) {
         ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
         const bool enableWtLogging = false;
         WiredTigerSizeStorer ss2(harnessHelper->conn(), indexUri, enableWtLogging);
-        auto info = ss2.load(opCtx.get(), uri);
+        auto info = ss2.load(uri);
         ASSERT_EQUALS(N, info->numRecords.load());
     }
 
@@ -176,12 +176,12 @@ private:
     }
 
 protected:
-    long long getNumRecords(OperationContext* opCtx) const {
-        return sizeStorer->load(opCtx, uri)->numRecords.load();
+    long long getNumRecords() const {
+        return sizeStorer->load(uri)->numRecords.load();
     }
 
-    long long getDataSize(OperationContext* opCtx) const {
-        return sizeStorer->load(opCtx, uri)->dataSize.load();
+    long long getDataSize() const {
+        return sizeStorer->load(uri)->dataSize.load();
     }
 
     std::unique_ptr<WiredTigerHarnessHelper> harnessHelper;
@@ -196,8 +196,8 @@ TEST_F(SizeStorerUpdateTest, Basic) {
     ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
     long long val = 5;
     rs->updateStatsAfterRepair(opCtx.get(), val, val);
-    ASSERT_EQUALS(getNumRecords(opCtx.get()), val);
-    ASSERT_EQUALS(getDataSize(opCtx.get()), val);
+    ASSERT_EQUALS(getNumRecords(), val);
+    ASSERT_EQUALS(getDataSize(), val);
 }
 
 }  // namespace
