@@ -36,7 +36,7 @@
 
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/commands/server_status.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/timeseries/timeseries_options.h"
 #include "mongo/db/views/view_catalog.h"
@@ -52,7 +52,6 @@ void normalizeObject(BSONObjBuilder* builder, const BSONObj& obj);
 
 const auto getBucketCatalog = ServiceContext::declareDecoration<BucketCatalog>();
 MONGO_FAIL_POINT_DEFINE(hangTimeseriesDirectModificationBeforeWriteConflict);
-MONGO_FAIL_POINT_DEFINE(hangWaitingForConflictingPreparedBatch);
 
 uint8_t numDigits(uint32_t num) {
     uint8_t numDigits = 0;
@@ -156,45 +155,6 @@ BSONObj buildControlMinTimestampDoc(StringData timeField, Date_t roundedTime) {
     builder.append(timeField, roundedTime);
     return builder.obj();
 }
-
-std::pair<OID, Date_t> generateBucketId(const Date_t& time, const TimeseriesOptions& options) {
-    OID bucketId = OID::gen();
-
-    // We round the measurement timestamp down to the nearest minute, hour, or day depending on the
-    // granularity. We do this for two reasons. The first is so that if measurements come in
-    // slightly out of order, we don't have to close the current bucket due to going backwards in
-    // time. The second, and more important reason, is so that we reliably group measurements
-    // together into predictable chunks for sharding. This way we know from a measurement timestamp
-    // what the bucket timestamp will be, so we can route measurements to the right shard chunk.
-    auto roundedTime = timeseries::roundTimestampToGranularity(time, options.getGranularity());
-    int64_t const roundedSeconds = durationCount<Seconds>(roundedTime.toDurationSinceEpoch());
-    bucketId.setTimestamp(roundedSeconds);
-
-    // Now, if we stopped here we could end up with bucket OID collisions. Consider the case where
-    // we have the granularity set to 'Hours'. This means we will round down to the nearest day, so
-    // any bucket generated on the same machine on the same day will have the same timestamp portion
-    // and unique instance portion of the OID. Only the increment will differ. Since we only use 3
-    // bytes for the increment portion, we run a serious risk of overflow if we are generating lots
-    // of buckets.
-    //
-    // To address this, we'll take the difference between the actual timestamp and the rounded
-    // timestamp and add it to the instance portion of the OID to ensure we can't have a collision.
-    // for timestamps generated on the same machine.
-    //
-    // This leaves open the possibility that in the case of step-down/step-up, we could get a
-    // collision if the old primary and the new primary have unique instance bits that differ by
-    // less than the maximum rounding difference. This is quite unlikely though, and can be resolved
-    // by restarting the new primary. It remains an open question whether we can fix this in a
-    // better way.
-    // TODO (SERVER-61412): Avoid time-series bucket OID collisions after election
-    auto instance = bucketId.getInstanceUnique();
-    uint32_t sum = DataView(reinterpret_cast<char*>(instance.bytes)).read<uint32_t>(1) +
-        (durationCount<Seconds>(time.toDurationSinceEpoch()) - roundedSeconds);
-    DataView(reinterpret_cast<char*>(instance.bytes)).write<uint32_t>(sum, 1);
-    bucketId.setInstanceUnique(instance);
-
-    return {bucketId, roundedTime};
-}
 }  // namespace
 
 const std::shared_ptr<BucketCatalog::ExecutionStats> BucketCatalog::kEmptyStats{
@@ -217,7 +177,7 @@ BSONObj BucketCatalog::getMetadata(const OID& bucketId) const {
     return bucket->_metadata.toBSON();
 }
 
-StatusWith<std::shared_ptr<BucketCatalog::WriteBatch>> BucketCatalog::insert(
+StatusWith<BucketCatalog::InsertResult> BucketCatalog::insert(
     OperationContext* opCtx,
     const NamespaceString& ns,
     const StringData::ComparatorInterface* comparator,
@@ -244,7 +204,8 @@ StatusWith<std::shared_ptr<BucketCatalog::WriteBatch>> BucketCatalog::insert(
 
     auto time = timeElem.Date();
 
-    BucketAccess bucket{this, key, options, stats.get(), time};
+    ClosedBuckets closedBuckets;
+    BucketAccess bucket{this, key, options, stats.get(), &closedBuckets, time};
     invariant(bucket);
 
     NewFieldNames newFieldNamesToBeInserted;
@@ -266,7 +227,7 @@ StatusWith<std::shared_ptr<BucketCatalog::WriteBatch>> BucketCatalog::insert(
             stats->numBucketsClosedDueToSize.fetchAndAddRelaxed(1);
             return true;
         }
-        auto bucketTime = (*bucket)->getTime();
+        auto bucketTime = (*bucket).getTime();
         if (time - bucketTime >= Seconds(*options.getBucketMaxSpanSeconds())) {
             stats->numBucketsClosedDueToTimeForward.fetchAndAddRelaxed(1);
             return true;
@@ -279,7 +240,8 @@ StatusWith<std::shared_ptr<BucketCatalog::WriteBatch>> BucketCatalog::insert(
     };
 
     if (!bucket->_ns.isEmpty() && isBucketFull(&bucket)) {
-        bucket.rollover(isBucketFull);
+        bucket.rollover(isBucketFull, &closedBuckets);
+
         bucket->_calculateBucketFieldsAndSizeChange(doc,
                                                     options.getMetaField(),
                                                     &newFieldNamesToBeInserted,
@@ -313,7 +275,7 @@ StatusWith<std::shared_ptr<BucketCatalog::WriteBatch>> BucketCatalog::insert(
     }
     _memoryUsage.fetchAndAdd(bucket->_memoryUsage);
 
-    return batch;
+    return InsertResult{batch, closedBuckets};
 }
 
 bool BucketCatalog::prepareCommit(std::shared_ptr<WriteBatch> batch) {
@@ -326,19 +288,7 @@ bool BucketCatalog::prepareCommit(std::shared_ptr<WriteBatch> batch) {
 
     BucketAccess bucket(this, batch->bucketId(), BucketState::kPrepared);
     if (batch->finished()) {
-        // Someone may have aborted it while we were waiting. Since we have the prepared batch, we
-        // should now be able to fully abort the bucket.
-        if (bucket) {
-            bucket.release();
-            auto lk = _lockExclusive();
-            auto it = _allBuckets.find(batch->bucketId());
-            if (it != _allBuckets.end()) {
-                auto bucket = it->second.get();
-                stdx::unique_lock blk{bucket->_mutex};
-                bucket->_preparedBatch.reset();
-                _abort(blk, bucket, nullptr, boost::none);
-            }
-        }
+        // Someone may have aborted it while we were waiting.
         return false;
     } else if (!bucket) {
         abort(batch);
@@ -352,10 +302,13 @@ bool BucketCatalog::prepareCommit(std::shared_ptr<WriteBatch> batch) {
     return true;
 }
 
-void BucketCatalog::finish(std::shared_ptr<WriteBatch> batch, const CommitInfo& info) {
+boost::optional<BucketCatalog::ClosedBucket> BucketCatalog::finish(
+    std::shared_ptr<WriteBatch> batch, const CommitInfo& info) {
     invariant(!batch->finished());
     invariant(!batch->active());
 
+    boost::optional<ClosedBucket> closedBucket;
+
     batch->_finish(info);
 
     BucketAccess bucket(this, batch->bucketId(), BucketState::kNormal);
@@ -402,6 +355,9 @@ void BucketCatalog::finish(std::shared_ptr<WriteBatch> batch, const CommitInfo&
                 Bucket* ptr = it->second.get();
                 _verifyBucketIsUnused(ptr);
 
+                closedBucket = ClosedBucket{
+                    batch->bucketId(), ptr->getTimeField().toString(), ptr->numMeasurements()};
+
                 // Only remove from _allBuckets and _idleBuckets. If it was marked full, we know
                 // that happened in BucketAccess::rollover, and that there is already a new open
                 // bucket for this metadata.
@@ -417,6 +373,7 @@ void BucketCatalog::finish(std::shared_ptr<WriteBatch> batch, const CommitInfo&
             _markBucketIdle(bucket);
         }
     }
+    return closedBucket;
 }
 
 void BucketCatalog::abort(std::shared_ptr<WriteBatch> batch,
@@ -425,6 +382,13 @@ void BucketCatalog::abort(std::shared_ptr<WriteBatch> batch,
     invariant(batch->_commitRights.load());
 
     if (batch->finished()) {
+        auto batchStatus = batch->getResult().getStatus();
+        tassert(5916403,
+                str::stream() << "Unexpected error when aborting time-series batch: "
+                              << batchStatus,
+                batchStatus == ErrorCodes::TimeseriesBucketCleared ||
+                    batchStatus.isA<ErrorCategory::Interruption>() ||
+                    batchStatus.isA<ErrorCategory::StaleShardVersionError>());
         return;
     }
 
@@ -542,11 +506,6 @@ void BucketCatalog::_waitToCommitBatch(const std::shared_ptr<WriteBatch>& batch)
 
         // We have to wait for someone else to finish.
         bucket.release();
-
-        // We only hit this failpoint when there are conflicting prepared batches on the same
-        // bucket.
-        hangWaitingForConflictingPreparedBatch.pauseWhileSet();
-
         current->getResult().getStatus().ignore();  // We don't care about the result.
     }
 }
@@ -608,8 +567,6 @@ void BucketCatalog::_abort(stdx::unique_lock<Mutex>& lk,
     lk.unlock();
     if (doRemove) {
         [[maybe_unused]] bool removed = _removeBucket(bucket, false /* expiringBuckets */);
-    } else {
-        _setBucketState(bucket->id(), BucketState::kCleared);
     }
 }
 
@@ -639,14 +596,17 @@ void BucketCatalog::_verifyBucketIsUnused(Bucket* bucket) const {
     stdx::lock_guard<Mutex> lk{bucket->_mutex};
 }
 
-void BucketCatalog::_expireIdleBuckets(ExecutionStats* stats) {
+void BucketCatalog::_expireIdleBuckets(ExecutionStats* stats,
+                                       std::vector<BucketCatalog::ClosedBucket>* closedBuckets) {
     // Must hold an exclusive lock on _bucketMutex from outside.
     stdx::unique_lock lk{_idleMutex};
 
-    // As long as we still need space and have entries, close idle buckets.
+    // As long as we still need space and have entries and remaining attempts, close idle buckets.
+    int32_t numClosed = 0;
     while (!_idleBuckets.empty() &&
-           _memoryUsage.load() > static_cast<std::uint64_t>(
-                                     gTimeseriesIdleBucketExpiryMemoryUsageThresholdBytes.load())) {
+           _memoryUsage.load() >
+               static_cast<std::uint64_t>(gTimeseriesIdleBucketExpiryMemoryUsageThreshold.load()) &&
+           numClosed <= gTimeseriesIdleBucketExpiryMaxCountPerAttempt) {
         Bucket* bucket = _idleBuckets.back();
 
         lk.unlock();
@@ -658,8 +618,12 @@ void BucketCatalog::_expireIdleBuckets(ExecutionStats* stats) {
             continue;
         }
 
+        ClosedBucket closed{
+            bucket->id(), bucket->getTimeField().toString(), bucket->numMeasurements()};
         if (_removeBucket(bucket, true /* expiringBuckets */)) {
             stats->numBucketsClosedDueToMemoryThreshold.fetchAndAddRelaxed(1);
+            closedBuckets->push_back(closed);
+            ++numClosed;
         }
     }
 }
@@ -673,10 +637,15 @@ BucketCatalog::Bucket* BucketCatalog::_allocateBucket(const BucketKey& key,
                                                       const Date_t& time,
                                                       const TimeseriesOptions& options,
                                                       ExecutionStats* stats,
+                                                      ClosedBuckets* closedBuckets,
                                                       bool openedDuetoMetadata) {
-    _expireIdleBuckets(stats);
+    _expireIdleBuckets(stats, closedBuckets);
 
-    auto [bucketId, roundedTime] = generateBucketId(time, options);
+    OID bucketId = OID::gen();
+
+    auto roundedTime = timeseries::roundTimestampToGranularity(time, options.getGranularity());
+    auto const roundedSeconds = durationCount<Seconds>(roundedTime.toDurationSinceEpoch());
+    bucketId.setTimestamp(roundedSeconds);
 
     auto [it, inserted] = _allBuckets.try_emplace(bucketId, std::make_unique<Bucket>(bucketId));
     tassert(6130900, "Expected bucket to be inserted", inserted);
@@ -691,7 +660,7 @@ BucketCatalog::Bucket* BucketCatalog::_allocateBucket(const BucketKey& key,
         stats->numBucketsOpenedDueToMetadata.fetchAndAddRelaxed(1);
     }
 
-    bucket->_minTime = roundedTime;
+    bucket->_timeField = options.getTimeField().toString();
 
     // Make sure we set the control.min time field to match the rounded _id timestamp.
     auto controlDoc = buildControlMinTimestampDoc(options.getTimeField(), roundedTime);
@@ -827,6 +796,10 @@ const OID& BucketCatalog::Bucket::id() const {
     return _id;
 }
 
+StringData BucketCatalog::Bucket::getTimeField() {
+    return _timeField;
+}
+
 void BucketCatalog::Bucket::_calculateBucketFieldsAndSizeChange(
     const BSONObj& doc,
     boost::optional<StringData> metaField,
@@ -873,8 +846,8 @@ bool BucketCatalog::Bucket::allCommitted() const {
     return _batches.empty() && !_preparedBatch;
 }
 
-Date_t BucketCatalog::Bucket::getTime() const {
-    return _minTime;
+uint32_t BucketCatalog::Bucket::numMeasurements() const {
+    return _numMeasurements;
 }
 
 std::shared_ptr<BucketCatalog::WriteBatch> BucketCatalog::Bucket::_activeBatch(
@@ -890,6 +863,7 @@ BucketCatalog::BucketAccess::BucketAccess(BucketCatalog* catalog,
                                           BucketKey& key,
                                           const TimeseriesOptions& options,
                                           ExecutionStats* stats,
+                                          ClosedBuckets* closedBuckets,
                                           const Date_t& time)
     : _catalog(catalog), _key(&key), _options(&options), _stats(stats), _time(&time) {
 
@@ -946,7 +920,7 @@ BucketCatalog::BucketAccess::BucketAccess(BucketCatalog* catalog,
         : key.withCopiedMetadata(BSONObj());
     hashedKey.key = &originalBucketKey;
     auto lk = _catalog->_lockExclusive();
-    _findOrCreateOpenBucketThenLock(hashedNormalizedKey, hashedKey);
+    _findOrCreateOpenBucketThenLock(hashedNormalizedKey, hashedKey, closedBuckets);
 }
 
 BucketCatalog::BucketAccess::BucketAccess(BucketCatalog* catalog,
@@ -1048,11 +1022,13 @@ BucketCatalog::BucketState BucketCatalog::BucketAccess::_confirmStateForAcquired
 }
 
 void BucketCatalog::BucketAccess::_findOrCreateOpenBucketThenLock(
-    const HashedBucketKey& normalizedKey, const HashedBucketKey& nonNormalizedKey) {
+    const HashedBucketKey& normalizedKey,
+    const HashedBucketKey& nonNormalizedKey,
+    ClosedBuckets* closedBuckets) {
     auto it = _catalog->_openBuckets.find(normalizedKey);
     if (it == _catalog->_openBuckets.end()) {
         // No open bucket for this metadata.
-        _create(normalizedKey, nonNormalizedKey);
+        _create(normalizedKey, nonNormalizedKey, closedBuckets);
         return;
     }
 
@@ -1066,7 +1042,7 @@ void BucketCatalog::BucketAccess::_findOrCreateOpenBucketThenLock(
     }
 
     _catalog->_abort(_guard, _bucket, nullptr, boost::none);
-    _create(normalizedKey, nonNormalizedKey);
+    _create(normalizedKey, nonNormalizedKey, closedBuckets);
 }
 
 void BucketCatalog::BucketAccess::_acquire() {
@@ -1076,10 +1052,11 @@ void BucketCatalog::BucketAccess::_acquire() {
 
 void BucketCatalog::BucketAccess::_create(const HashedBucketKey& normalizedKey,
                                           const HashedBucketKey& nonNormalizedKey,
+                                          ClosedBuckets* closedBuckets,
                                           bool openedDuetoMetadata) {
     invariant(_options);
-    _bucket =
-        _catalog->_allocateBucket(normalizedKey, *_time, *_options, _stats, openedDuetoMetadata);
+    _bucket = _catalog->_allocateBucket(
+        normalizedKey, *_time, *_options, _stats, closedBuckets, openedDuetoMetadata);
     _catalog->_openBuckets[nonNormalizedKey] = _bucket;
     _bucket->_nonNormalizedKeyMetadatas.push_back(nonNormalizedKey.key->metadata.toBSON());
     _acquire();
@@ -1108,7 +1085,8 @@ BucketCatalog::BucketAccess::operator BucketCatalog::Bucket*() const {
     return _bucket;
 }
 
-void BucketCatalog::BucketAccess::rollover(const std::function<bool(BucketAccess*)>& isBucketFull) {
+void BucketCatalog::BucketAccess::rollover(const std::function<bool(BucketAccess*)>& isBucketFull,
+                                           ClosedBuckets* closedBuckets) {
     invariant(isLocked());
     invariant(_key);
     invariant(_time);
@@ -1125,7 +1103,7 @@ void BucketCatalog::BucketAccess::rollover(const std::function<bool(BucketAccess
     auto hashedKey = BucketHasher{}.hashed_key(prevBucketKey);
 
     auto lk = _catalog->_lockExclusive();
-    _findOrCreateOpenBucketThenLock(hashedNormalizedKey, hashedKey);
+    _findOrCreateOpenBucketThenLock(hashedNormalizedKey, hashedKey, closedBuckets);
 
     // Recheck if still full now that we've reacquired the bucket.
     bool sameBucket =
@@ -1135,6 +1113,9 @@ void BucketCatalog::BucketAccess::rollover(const std::function<bool(BucketAccess
         if (_bucket->allCommitted()) {
             // The bucket does not contain any measurements that are yet to be committed, so we can
             // remove it now. Otherwise, we must keep the bucket around until it is committed.
+            closedBuckets->push_back(ClosedBucket{
+                _bucket->id(), _bucket->getTimeField().toString(), _bucket->numMeasurements()});
+
             Bucket* oldBucket = _bucket;
             release();
             bool removed = _catalog->_removeBucket(oldBucket, false /* expiringBuckets */);
@@ -1148,10 +1129,14 @@ void BucketCatalog::BucketAccess::rollover(const std::function<bool(BucketAccess
             release();
         }
 
-        _create(hashedNormalizedKey, hashedKey, false /* openedDueToMetadata */);
+        _create(hashedNormalizedKey, hashedKey, closedBuckets, false /* openedDueToMetadata */);
     }
 }
 
+Date_t BucketCatalog::BucketAccess::getTime() const {
+    return _bucket->id().asDateT();
+}
+
 BucketCatalog::WriteBatch::WriteBatch(const OID& bucketId,
                                       OperationId opId,
                                       const std::shared_ptr<ExecutionStats>& stats)
