@@ -65,6 +65,16 @@ public:
         boost::optional<OID> electionId;
     };
 
+    /**
+     * Information of a Bucket that got closed when performing an operation on this BucketCatalog.
+     */
+    struct ClosedBucket {
+        OID bucketId;
+        std::string timeField;
+        uint32_t numMeasurements;
+    };
+    using ClosedBuckets = std::vector<ClosedBucket>;
+
     /**
      * The basic unit of work for a bucket. Each insert will return a shared_ptr to a WriteBatch.
      * When a writer is finished with all their insertions, they should then take steps to ensure
@@ -165,6 +175,15 @@ public:
         SharedPromise<CommitInfo> _promise;
     };
 
+    /**
+     * Return type for the insert function.
+     * See comment above insert() for more information.
+     */
+    struct InsertResult {
+        std::shared_ptr<WriteBatch> batch;
+        ClosedBuckets closedBuckets;
+    };
+
     static BucketCatalog& get(ServiceContext* svcCtx);
     static BucketCatalog& get(OperationContext* opCtx);
 
@@ -184,17 +203,16 @@ public:
     BSONObj getMetadata(const OID& bucketId) const;
 
     /**
-     * Returns the WriteBatch into which the document was inserted. Any caller who receives the same
-     * batch may commit or abort the batch after claiming commit rights. See WriteBatch for more
-     * details.
+     * Returns the WriteBatch into which the document was inserted and optional information about a
+     * bucket if one was closed. Any caller who receives the same batch may commit or abort the
+     * batch after claiming commit rights. See WriteBatch for more details.
      */
-    StatusWith<std::shared_ptr<WriteBatch>> insert(
-        OperationContext* opCtx,
-        const NamespaceString& ns,
-        const StringData::ComparatorInterface* comparator,
-        const TimeseriesOptions& options,
-        const BSONObj& doc,
-        CombineWithInsertsFromOtherClients combine);
+    StatusWith<InsertResult> insert(OperationContext* opCtx,
+                                    const NamespaceString& ns,
+                                    const StringData::ComparatorInterface* comparator,
+                                    const TimeseriesOptions& options,
+                                    const BSONObj& doc,
+                                    CombineWithInsertsFromOtherClients combine);
 
     /**
      * Prepares a batch for commit, transitioning it to an inactive state. Caller must already have
@@ -206,8 +224,10 @@ public:
     /**
      * Records the result of a batch commit. Caller must already have commit rights on batch, and
      * batch must have been previously prepared.
+     *
+     * Returns bucket information of a bucket if one was closed.
      */
-    void finish(std::shared_ptr<WriteBatch> batch, const CommitInfo& info);
+    boost::optional<ClosedBucket> finish(std::shared_ptr<WriteBatch> batch, const CommitInfo& info);
 
     /**
      * Aborts the given write batch and any other outstanding batches on the same bucket. Caller
@@ -344,15 +364,20 @@ public:
          */
         const OID& id() const;
 
+        /**
+         * Returns the timefield for the underlying bucket.
+         */
+        StringData getTimeField();
+
         /**
          * Returns whether all measurements have been committed.
          */
         bool allCommitted() const;
 
         /**
-         * Retrieve the time associated with the bucket
+         * Returns total number of measurements in the bucket.
          */
-        Date_t getTime() const;
+        uint32_t numMeasurements() const;
 
     private:
         /**
@@ -398,8 +423,8 @@ public:
         // Top-level field names of the measurements that have been inserted into the bucket.
         StringSet _fieldNames;
 
-        // Minimum timestamp over contained measurements
-        Date_t _minTime;
+        // Time field for the measurements that have been inserted into the bucket.
+        std::string _timeField;
 
         // The minimum and maximum values for each field in the bucket.
         timeseries::MinMax _minmax;
@@ -561,6 +586,7 @@ private:
                      BucketKey& key,
                      const TimeseriesOptions& options,
                      ExecutionStats* stats,
+                     ClosedBuckets* closedBuckets,
                      const Date_t& time);
         BucketAccess(BucketCatalog* catalog,
                      const OID& bucketId,
@@ -580,8 +606,14 @@ private:
          * Parameter is a function which should check that the bucket is indeed still full after
          * reacquiring the necessary locks. The first parameter will give the function access to
          * this BucketAccess instance, with the bucket locked.
+         *
+         * Returns bucket information of a bucket if one was closed.
          */
-        void rollover(const std::function<bool(BucketAccess*)>& isBucketFull);
+        void rollover(const std::function<bool(BucketAccess*)>& isBucketFull,
+                      ClosedBuckets* closedBuckets);
+
+        // Retrieve the time associated with the bucket (id)
+        Date_t getTime() const;
 
     private:
         /**
@@ -616,7 +648,8 @@ private:
         // Helper to find an open bucket for the given metadata if it exists, create it if it
         // doesn't, and lock it. Requires an exclusive lock on the catalog.
         void _findOrCreateOpenBucketThenLock(const HashedBucketKey& normalizedKey,
-                                             const HashedBucketKey& key);
+                                             const HashedBucketKey& key,
+                                             ClosedBuckets* closedBuckets);
 
         // Lock _bucket.
         void _acquire();
@@ -625,6 +658,7 @@ private:
         // a lock on it.
         void _create(const HashedBucketKey& normalizedKey,
                      const HashedBucketKey& key,
+                     ClosedBuckets* closedBuckets,
                      bool openedDuetoMetadata = true);
 
         BucketCatalog* _catalog;
@@ -684,7 +718,7 @@ private:
     /**
      * Expires idle buckets until the bucket catalog's memory usage is below the expiry threshold.
      */
-    void _expireIdleBuckets(ExecutionStats* stats);
+    void _expireIdleBuckets(ExecutionStats* stats, ClosedBuckets* closedBuckets);
 
     std::size_t _numberOfIdleBuckets() const;
 
@@ -693,6 +727,7 @@ private:
                             const Date_t& time,
                             const TimeseriesOptions& options,
                             ExecutionStats* stats,
+                            ClosedBuckets* closedBuckets,
                             bool openedDuetoMetadata);
 
     std::shared_ptr<ExecutionStats> _getExecutionStats(const NamespaceString& ns);
