@@ -32,7 +32,7 @@
 #include "mongo/db/catalog/catalog_test_fixture.h"
 #include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog_raii.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/timeseries/bucket_catalog.h"
 #include "mongo/db/views/view_catalog.h"
 #include "mongo/stdx/future.h"
@@ -44,13 +44,17 @@ namespace mongo {
 namespace {
 class BucketCatalogTest : public CatalogTestFixture {
 protected:
-    class RunBackgroundTaskAndWaitForFailpoint {
+    class Task {
+        AtomicWord<bool> _running{false};
+        stdx::packaged_task<void()> _task;
+        stdx::future<void> _future;
         stdx::thread _taskThread;
 
     public:
-        RunBackgroundTaskAndWaitForFailpoint(const std::string& failpointName,
-                                             std::function<void()>&& fn);
-        ~RunBackgroundTaskAndWaitForFailpoint();
+        Task(std::function<void()>&& fn);
+        ~Task();
+
+        const stdx::future<void>& future();
     };
 
     void setUp() override;
@@ -87,6 +91,25 @@ protected:
     BSONObj _makeTimeseriesOptionsForCreate() const override;
 };
 
+BucketCatalogTest::Task::Task(std::function<void()>&& fn)
+    : _task{[this, fn = std::move(fn)]() {
+          _running.store(true);
+          fn();
+      }},
+      _future{_task.get_future()},
+      _taskThread{std::move(_task)} {
+    while (!_running.load()) {
+        stdx::this_thread::yield();
+    }
+}
+BucketCatalogTest::Task::~Task() {
+    _taskThread.join();
+}
+
+const stdx::future<void>& BucketCatalogTest::Task::future() {
+    return _future;
+}
+
 void BucketCatalogTest::setUp() {
     CatalogTestFixture::setUp();
 
@@ -101,23 +124,6 @@ void BucketCatalogTest::setUp() {
     }
 }
 
-BucketCatalogTest::RunBackgroundTaskAndWaitForFailpoint::RunBackgroundTaskAndWaitForFailpoint(
-    const std::string& failpointName, std::function<void()>&& fn) {
-    auto fp = globalFailPointRegistry().find(failpointName);
-    auto timesEntered = fp->setMode(FailPoint::alwaysOn, 0);
-
-    // Start background job.
-    _taskThread = stdx::thread(std::move(fn));
-
-    // Once we hit the failpoint once, turn it off.
-    fp->waitForTimesEntered(timesEntered + 1);
-    fp->setMode(FailPoint::off, 0);
-}
-
-BucketCatalogTest::RunBackgroundTaskAndWaitForFailpoint::~RunBackgroundTaskAndWaitForFailpoint() {
-    _taskThread.join();
-}
-
 std::pair<ServiceContext::UniqueClient, ServiceContext::UniqueOperationContext>
 BucketCatalogTest::_makeOperationContext() {
     auto client = getServiceContext()->makeClient("BucketCatalogTest");
@@ -162,7 +168,7 @@ void BucketCatalogTest::_insertOneAndCommit(const NamespaceString& ns,
                                          _getTimeseriesOptions(ns),
                                          BSON(_timeField << Date_t::now()),
                                          BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    auto& batch = result.getValue();
+    auto& batch = result.getValue().batch;
     _commit(batch, numPreviouslyCommittedMeasurements);
 }
 
@@ -181,7 +187,7 @@ TEST_F(BucketCatalogTest, InsertIntoSameBucket) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    auto batch1 = result1.getValue();
+    auto batch1 = result1.getValue().batch;
     ASSERT(batch1->claimCommitRights());
     ASSERT(batch1->active());
 
@@ -194,7 +200,7 @@ TEST_F(BucketCatalogTest, InsertIntoSameBucket) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    auto batch2 = result2.getValue();
+    auto batch2 = result2.getValue().batch;
     ASSERT_EQ(batch1, batch2);
     ASSERT(!batch2->claimCommitRights());
 
@@ -227,7 +233,8 @@ TEST_F(BucketCatalogTest, GetMetadataReturnsEmptyDocOnMissingBucket) {
                               _getTimeseriesOptions(_ns1),
                               BSON(_timeField << Date_t::now()),
                               BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                     .getValue();
+                     .getValue()
+                     .batch;
     ASSERT(batch->claimCommitRights());
     auto bucketId = batch->bucketId();
     _bucketCatalog->abort(batch);
@@ -258,20 +265,21 @@ TEST_F(BucketCatalogTest, InsertIntoDifferentBuckets) {
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
 
     // Inserts should all be into three distinct buckets (and therefore batches).
-    ASSERT_NE(result1.getValue(), result2.getValue());
-    ASSERT_NE(result1.getValue(), result3.getValue());
-    ASSERT_NE(result2.getValue(), result3.getValue());
+    ASSERT_NE(result1.getValue().batch, result2.getValue().batch);
+    ASSERT_NE(result1.getValue().batch, result3.getValue().batch);
+    ASSERT_NE(result2.getValue().batch, result3.getValue().batch);
 
     // Check metadata in buckets.
     ASSERT_BSONOBJ_EQ(BSON(_metaField << "123"),
-                      _bucketCatalog->getMetadata(result1.getValue()->bucketId()));
+                      _bucketCatalog->getMetadata(result1.getValue().batch->bucketId()));
     ASSERT_BSONOBJ_EQ(BSON(_metaField << BSONObj()),
-                      _bucketCatalog->getMetadata(result2.getValue()->bucketId()));
-    ASSERT(_bucketCatalog->getMetadata(result3.getValue()->bucketId()).isEmpty());
+                      _bucketCatalog->getMetadata(result2.getValue().batch->bucketId()));
+    ASSERT(_bucketCatalog->getMetadata(result3.getValue().batch->bucketId()).isEmpty());
 
     // Committing one bucket should only return the one document in that bucket and should not
     // affect the other bucket.
-    for (const auto& batch : {result1.getValue(), result2.getValue(), result3.getValue()}) {
+    for (const auto& batch :
+         {result1.getValue().batch, result2.getValue().batch, result3.getValue().batch}) {
         _commit(batch, 0);
     }
 }
@@ -292,13 +300,13 @@ TEST_F(BucketCatalogTest, InsertIntoSameBucketArray) {
         BSON(_timeField << Date_t::now() << _metaField << BSON_ARRAY(BSON("b" << 1 << "a" << 0))),
         BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
 
-    ASSERT_EQ(result1.getValue(), result2.getValue());
+    ASSERT_EQ(result1.getValue().batch, result2.getValue().batch);
 
     // Check metadata in buckets.
     ASSERT_BSONOBJ_EQ(BSON(_metaField << BSON_ARRAY(BSON("a" << 0 << "b" << 1))),
-                      _bucketCatalog->getMetadata(result1.getValue()->bucketId()));
+                      _bucketCatalog->getMetadata(result1.getValue().batch->bucketId()));
     ASSERT_BSONOBJ_EQ(BSON(_metaField << BSON_ARRAY(BSON("a" << 0 << "b" << 1))),
-                      _bucketCatalog->getMetadata(result2.getValue()->bucketId()));
+                      _bucketCatalog->getMetadata(result2.getValue().batch->bucketId()));
 }
 
 TEST_F(BucketCatalogTest, InsertIntoSameBucketObjArray) {
@@ -321,17 +329,17 @@ TEST_F(BucketCatalogTest, InsertIntoSameBucketObjArray) {
                                                           << BSON("g" << 0 << "f" << 1))))),
         BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
 
-    ASSERT_EQ(result1.getValue(), result2.getValue());
+    ASSERT_EQ(result1.getValue().batch, result2.getValue().batch);
 
     // Check metadata in buckets.
     ASSERT_BSONOBJ_EQ(
         BSON(_metaField << BSONObj(BSON(
                  "c" << BSON_ARRAY(BSON("a" << 0 << "b" << 1) << BSON("f" << 1 << "g" << 0))))),
-        _bucketCatalog->getMetadata(result1.getValue()->bucketId()));
+        _bucketCatalog->getMetadata(result1.getValue().batch->bucketId()));
     ASSERT_BSONOBJ_EQ(
         BSON(_metaField << BSONObj(BSON(
                  "c" << BSON_ARRAY(BSON("a" << 0 << "b" << 1) << BSON("f" << 1 << "g" << 0))))),
-        _bucketCatalog->getMetadata(result2.getValue()->bucketId()));
+        _bucketCatalog->getMetadata(result2.getValue().batch->bucketId()));
 }
 
 
@@ -357,17 +365,17 @@ TEST_F(BucketCatalogTest, InsertIntoSameBucketNestedArray) {
                                                                         << "456"))))),
         BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
 
-    ASSERT_EQ(result1.getValue(), result2.getValue());
+    ASSERT_EQ(result1.getValue().batch, result2.getValue().batch);
 
     // Check metadata in buckets.
     ASSERT_BSONOBJ_EQ(BSON(_metaField << BSONObj(BSON("c" << BSON_ARRAY(BSON("a" << 0 << "b" << 1)
                                                                         << BSON_ARRAY("123"
                                                                                       << "456"))))),
-                      _bucketCatalog->getMetadata(result1.getValue()->bucketId()));
+                      _bucketCatalog->getMetadata(result1.getValue().batch->bucketId()));
     ASSERT_BSONOBJ_EQ(BSON(_metaField << BSONObj(BSON("c" << BSON_ARRAY(BSON("a" << 0 << "b" << 1)
                                                                         << BSON_ARRAY("123"
                                                                                       << "456"))))),
-                      _bucketCatalog->getMetadata(result2.getValue()->bucketId()));
+                      _bucketCatalog->getMetadata(result2.getValue().batch->bucketId()));
 }
 
 TEST_F(BucketCatalogTest, InsertNullAndMissingMetaFieldIntoDifferentBuckets) {
@@ -387,16 +395,16 @@ TEST_F(BucketCatalogTest, InsertNullAndMissingMetaFieldIntoDifferentBuckets) {
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
 
     // Inserts should all be into three distinct buckets (and therefore batches).
-    ASSERT_NE(result1.getValue(), result2.getValue());
+    ASSERT_NE(result1.getValue().batch, result2.getValue().batch);
 
     // Check metadata in buckets.
     ASSERT_BSONOBJ_EQ(BSON(_metaField << BSONNULL),
-                      _bucketCatalog->getMetadata(result1.getValue()->bucketId()));
-    ASSERT(_bucketCatalog->getMetadata(result2.getValue()->bucketId()).isEmpty());
+                      _bucketCatalog->getMetadata(result1.getValue().batch->bucketId()));
+    ASSERT(_bucketCatalog->getMetadata(result2.getValue().batch->bucketId()).isEmpty());
 
     // Committing one bucket should only return the one document in that bucket and should not
     // affect the other bucket.
-    for (const auto& batch : {result1.getValue(), result2.getValue()}) {
+    for (const auto& batch : {result1.getValue().batch, result2.getValue().batch}) {
         _commit(batch, 0);
     }
 }
@@ -438,7 +446,8 @@ TEST_F(BucketCatalogTest, InsertBetweenPrepareAndFinish) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT(batch1->claimCommitRights());
     _bucketCatalog->prepareCommit(batch1);
     ASSERT_EQ(batch1->measurements().size(), 1);
@@ -452,7 +461,8 @@ TEST_F(BucketCatalogTest, InsertBetweenPrepareAndFinish) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT_NE(batch1, batch2);
 
     _bucketCatalog->finish(batch1, {});
@@ -469,7 +479,7 @@ DEATH_TEST_F(BucketCatalogTest, CannotCommitWithoutRights, "invariant") {
                                          _getTimeseriesOptions(_ns1),
                                          BSON(_timeField << Date_t::now()),
                                          BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    auto& batch = result.getValue();
+    auto& batch = result.getValue().batch;
     _bucketCatalog->prepareCommit(batch);
 }
 
@@ -480,7 +490,7 @@ DEATH_TEST_F(BucketCatalogTest, CannotFinishUnpreparedBatch, "invariant") {
                                          _getTimeseriesOptions(_ns1),
                                          BSON(_timeField << Date_t::now()),
                                          BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    auto& batch = result.getValue();
+    auto& batch = result.getValue().batch;
     ASSERT(batch->claimCommitRights());
     _bucketCatalog->finish(batch, {});
 }
@@ -493,7 +503,8 @@ TEST_F(BucketCatalogWithoutMetadataTest, GetMetadataReturnsEmptyDoc) {
                               _getTimeseriesOptions(_ns1),
                               BSON(_timeField << Date_t::now()),
                               BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                     .getValue();
+                     .getValue()
+                     .batch;
 
     ASSERT_BSONOBJ_EQ(BSONObj(), _bucketCatalog->getMetadata(batch->bucketId()));
 
@@ -509,7 +520,7 @@ TEST_F(BucketCatalogWithoutMetadataTest, CommitReturnsNewFields) {
                                          BSON(_timeField << Date_t::now() << "a" << 0),
                                          BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
     ASSERT(result.isOK());
-    auto batch = result.getValue();
+    auto batch = result.getValue().batch;
     auto oldId = batch->bucketId();
     _commit(batch, 0);
     ASSERT_EQ(2U, batch->newFieldNamesToBeInserted().size()) << batch->toBSON();
@@ -524,8 +535,8 @@ TEST_F(BucketCatalogWithoutMetadataTest, CommitReturnsNewFields) {
                                     _getTimeseriesOptions(_ns1),
                                     BSON(_timeField << Date_t::now() << "a" << 1),
                                     BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    ASSERT_OK(result);
-    batch = result.getValue();
+    ASSERT(result.isOK());
+    batch = result.getValue().batch;
     _commit(batch, 1);
     ASSERT_EQ(0U, batch->newFieldNamesToBeInserted().size()) << batch->toBSON();
 
@@ -536,8 +547,8 @@ TEST_F(BucketCatalogWithoutMetadataTest, CommitReturnsNewFields) {
                                     _getTimeseriesOptions(_ns1),
                                     BSON(_timeField << Date_t::now() << "a" << 2 << "b" << 2),
                                     BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    ASSERT_OK(result);
-    batch = result.getValue();
+    ASSERT(result.isOK());
+    batch = result.getValue().batch;
     _commit(batch, 2);
     ASSERT_EQ(1U, batch->newFieldNamesToBeInserted().size()) << batch->toBSON();
     ASSERT(batch->newFieldNamesToBeInserted().count("b")) << batch->toBSON();
@@ -550,8 +561,8 @@ TEST_F(BucketCatalogWithoutMetadataTest, CommitReturnsNewFields) {
                                         _getTimeseriesOptions(_ns1),
                                         BSON(_timeField << Date_t::now() << "a" << i),
                                         BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-        ASSERT_OK(result);
-        batch = result.getValue();
+        ASSERT(result.isOK());
+        batch = result.getValue().batch;
         _commit(batch, i);
         ASSERT_EQ(0U, batch->newFieldNamesToBeInserted().size()) << i << ":" << batch->toBSON();
     }
@@ -565,7 +576,7 @@ TEST_F(BucketCatalogWithoutMetadataTest, CommitReturnsNewFields) {
         _getTimeseriesOptions(_ns1),
         BSON(_timeField << Date_t::now() << "a" << gTimeseriesBucketMaxCount),
         BucketCatalog::CombineWithInsertsFromOtherClients::kAllow);
-    auto& batch2 = result2.getValue();
+    auto& batch2 = result2.getValue().batch;
     ASSERT_NE(oldId, batch2->bucketId());
     _commit(batch2, 0);
     ASSERT_EQ(2U, batch2->newFieldNamesToBeInserted().size()) << batch2->toBSON();
@@ -581,7 +592,8 @@ TEST_F(BucketCatalogTest, AbortBatchOnBucketWithPreparedCommit) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT(batch1->claimCommitRights());
     _bucketCatalog->prepareCommit(batch1);
     ASSERT_EQ(batch1->measurements().size(), 1);
@@ -595,7 +607,8 @@ TEST_F(BucketCatalogTest, AbortBatchOnBucketWithPreparedCommit) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT_NE(batch1, batch2);
 
     ASSERT(batch2->claimCommitRights());
@@ -616,7 +629,8 @@ TEST_F(BucketCatalogTest, ClearNamespaceWithConcurrentWrites) {
                               _getTimeseriesOptions(_ns1),
                               BSON(_timeField << Date_t::now()),
                               BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                     .getValue();
+                     .getValue()
+                     .batch;
     ASSERT(batch->claimCommitRights());
 
     _bucketCatalog->clear(_ns1);
@@ -633,7 +647,8 @@ TEST_F(BucketCatalogTest, ClearNamespaceWithConcurrentWrites) {
                          _getTimeseriesOptions(_ns1),
                          BSON(_timeField << Date_t::now()),
                          BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                .getValue();
+                .getValue()
+                .batch;
     ASSERT(batch->claimCommitRights());
     _bucketCatalog->prepareCommit(batch);
     ASSERT_EQ(batch->measurements().size(), 1);
@@ -660,7 +675,8 @@ TEST_F(BucketCatalogTest, ClearBucketWithPreparedBatchThrowsConflict) {
                               _getTimeseriesOptions(_ns1),
                               BSON(_timeField << Date_t::now()),
                               BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                     .getValue();
+                     .getValue()
+                     .batch;
     ASSERT(batch->claimCommitRights());
     _bucketCatalog->prepareCommit(batch);
     ASSERT_EQ(batch->measurements().size(), 1);
@@ -681,7 +697,8 @@ TEST_F(BucketCatalogTest, PrepareCommitOnClearedBatchWithAlreadyPreparedBatch) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT(batch1->claimCommitRights());
     _bucketCatalog->prepareCommit(batch1);
     ASSERT_EQ(batch1->measurements().size(), 1);
@@ -695,7 +712,8 @@ TEST_F(BucketCatalogTest, PrepareCommitOnClearedBatchWithAlreadyPreparedBatch) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT_NE(batch1, batch2);
     ASSERT_EQ(batch1->bucketId(), batch2->bucketId());
 
@@ -720,7 +738,8 @@ TEST_F(BucketCatalogTest, PrepareCommitOnClearedBatchWithAlreadyPreparedBatch) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
     ASSERT_NE(batch1, batch3);
     ASSERT_NE(batch2, batch3);
     ASSERT_NE(batch1->bucketId(), batch3->bucketId());
@@ -742,7 +761,8 @@ TEST_F(BucketCatalogTest, PrepareCommitOnAlreadyAbortedBatch) {
                               _getTimeseriesOptions(_ns1),
                               BSON(_timeField << Date_t::now()),
                               BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                     .getValue();
+                     .getValue()
+                     .batch;
     ASSERT(batch->claimCommitRights());
 
     _bucketCatalog->abort(batch);
@@ -763,7 +783,8 @@ TEST_F(BucketCatalogTest, CombiningWithInsertsFromOtherClients) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     auto batch2 = _bucketCatalog
                       ->insert(_makeOperationContext().second.get(),
@@ -772,7 +793,8 @@ TEST_F(BucketCatalogTest, CombiningWithInsertsFromOtherClients) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     auto batch3 = _bucketCatalog
                       ->insert(_makeOperationContext().second.get(),
@@ -781,7 +803,8 @@ TEST_F(BucketCatalogTest, CombiningWithInsertsFromOtherClients) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     auto batch4 = _bucketCatalog
                       ->insert(_makeOperationContext().second.get(),
@@ -790,7 +813,8 @@ TEST_F(BucketCatalogTest, CombiningWithInsertsFromOtherClients) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kAllow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     ASSERT_NE(batch1, batch2);
     ASSERT_NE(batch1, batch3);
@@ -810,7 +834,8 @@ TEST_F(BucketCatalogTest, CannotConcurrentlyCommitBatchesForSameBucket) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     auto batch2 = _bucketCatalog
                       ->insert(_makeOperationContext().second.get(),
@@ -819,137 +844,24 @@ TEST_F(BucketCatalogTest, CannotConcurrentlyCommitBatchesForSameBucket) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     ASSERT(batch1->claimCommitRights());
     ASSERT(batch2->claimCommitRights());
 
     // Batch 2 will not be able to commit until batch 1 has finished.
-    ASSERT_TRUE(_bucketCatalog->prepareCommit(batch1));
+    _bucketCatalog->prepareCommit(batch1);
+    auto task = Task{[&]() { _bucketCatalog->prepareCommit(batch2); }};
+    // Add a little extra wait to make sure prepareCommit actually gets to the blocking point.
+    stdx::this_thread::sleep_for(stdx::chrono::milliseconds(10));
+    ASSERT(task.future().valid());
+    ASSERT(stdx::future_status::timeout == task.future().wait_for(stdx::chrono::microseconds(1)))
+        << "prepareCommit finished before expected";
 
-    {
-        auto task = RunBackgroundTaskAndWaitForFailpoint{
-            "hangWaitingForConflictingPreparedBatch",
-            [&]() { ASSERT_TRUE(_bucketCatalog->prepareCommit(batch2)); }};
-
-        // Finish the first batch.
-        _bucketCatalog->finish(batch1, {});
-        ASSERT(batch1->finished());
-    }
-
-    _bucketCatalog->finish(batch2, {});
-    ASSERT(batch2->finished());
-}
-
-TEST_F(BucketCatalogTest, AbortingBatchEnsuresBucketIsEventuallyClosed) {
-    auto batch1 = _bucketCatalog
-                      ->insert(_opCtx,
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-
-    auto batch2 = _bucketCatalog
-                      ->insert(_makeOperationContext().second.get(),
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-    auto batch3 = _bucketCatalog
-                      ->insert(_makeOperationContext().second.get(),
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-    ASSERT_EQ(batch1->bucketId(), batch2->bucketId());
-    ASSERT_EQ(batch1->bucketId(), batch3->bucketId());
-
-    ASSERT(batch1->claimCommitRights());
-    ASSERT(batch2->claimCommitRights());
-    ASSERT(batch3->claimCommitRights());
-
-    // Batch 2 will not be able to commit until batch 1 has finished.
-    ASSERT_TRUE(_bucketCatalog->prepareCommit(batch1));
-
-    {
-        auto task = RunBackgroundTaskAndWaitForFailpoint{
-            "hangWaitingForConflictingPreparedBatch",
-            [&]() { ASSERT_FALSE(_bucketCatalog->prepareCommit(batch2)); }};
-
-        // If we abort the third batch, it should abort the second one too, as it isn't prepared.
-        // However, since the first batch is prepared, we can't abort it or clean up the bucket. We
-        // can then finish the first batch, which will allow the second batch to proceed. It should
-        // recognize it has been aborted and clean up the bucket.
-        _bucketCatalog->abort(batch3, Status{ErrorCodes::TimeseriesBucketCleared, "cleared"});
-        _bucketCatalog->finish(batch1, {});
-        ASSERT(batch1->finished());
-    }
-    // Wait for the batch 2 task to finish preparing commit. Since batch 1 finished, batch 2 should
-    // be unblocked. Note that after aborting batch 3, batch 2 was not in a prepared state, so we
-    // expect the prepareCommit() call to fail.
-    ASSERT(batch2->finished());
-
-    // Make sure a new batch ends up in a new bucket.
-    auto batch4 = _bucketCatalog
-                      ->insert(_opCtx,
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-    ASSERT_NE(batch2->bucketId(), batch4->bucketId());
-}
-
-TEST_F(BucketCatalogTest, AbortingBatchEnsuresNewInsertsGoToNewBucket) {
-    auto batch1 = _bucketCatalog
-                      ->insert(_opCtx,
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-
-    auto batch2 = _bucketCatalog
-                      ->insert(_makeOperationContext().second.get(),
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-
-    // Batch 1 and 2 use the same bucket.
-    ASSERT_EQ(batch1->bucketId(), batch2->bucketId());
-    ASSERT(batch1->claimCommitRights());
-    ASSERT(batch2->claimCommitRights());
-    ASSERT_TRUE(_bucketCatalog->prepareCommit(batch1));
-
-    // Batch 1 will be in a prepared state now. Abort the second batch so that bucket 1 will be
-    // closed after batch 1 finishes.
-    _bucketCatalog->abort(batch2, Status{ErrorCodes::TimeseriesBucketCleared, "cleared"});
     _bucketCatalog->finish(batch1, {});
-    ASSERT(batch1->finished());
-    ASSERT(batch2->finished());
-
-    // Ensure a batch started after batch 2 aborts, does not insert future measurements into the
-    // aborted batch/bucket.
-    auto batch3 = _bucketCatalog
-                      ->insert(_opCtx,
-                               _ns1,
-                               _getCollator(_ns1),
-                               _getTimeseriesOptions(_ns1),
-                               BSON(_timeField << Date_t::now()),
-                               BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
-    ASSERT_NE(batch1->bucketId(), batch3->bucketId());
+    task.future().wait();
+    _bucketCatalog->finish(batch2, {});
 }
 
 TEST_F(BucketCatalogTest, DuplicateNewFieldNamesAcrossConcurrentBatches) {
@@ -960,7 +872,8 @@ TEST_F(BucketCatalogTest, DuplicateNewFieldNamesAcrossConcurrentBatches) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     auto batch2 = _bucketCatalog
                       ->insert(_makeOperationContext().second.get(),
@@ -969,7 +882,8 @@ TEST_F(BucketCatalogTest, DuplicateNewFieldNamesAcrossConcurrentBatches) {
                                _getTimeseriesOptions(_ns1),
                                BSON(_timeField << Date_t::now()),
                                BucketCatalog::CombineWithInsertsFromOtherClients::kDisallow)
-                      .getValue();
+                      .getValue()
+                      .batch;
 
     // Batch 2 is the first batch to commit the time field.
     ASSERT(batch2->claimCommitRights());
