@@ -167,47 +167,9 @@ bool MinMaxStore::ConstIterator::operator!=(const MinMaxStore::ConstIterator& rh
     return !operator==(rhs);
 }
 
-MinMaxStore::ObjView::ObjView(const MinMaxStore::Entries& entries,
-                              MinMaxStore::Entries::const_iterator pos)
-    : _entries(entries), _pos(pos) {}
-
-MinMaxStore::ObjView& MinMaxStore::ObjView::operator=(const MinMaxStore::ObjView& rhs) {
-    if (this != &rhs) {
-        _pos = rhs._pos;
-    }
-    return *this;
-}
-
-MinMaxStore::ObjView MinMaxStore::ObjView::object(MinMaxStore::ConstIterator pos) const {
-    return {_entries, pos._pos};
-}
-
-MinMaxStore::ObjView MinMaxStore::ObjView::parent() const {
-    return {_entries, _pos - _pos->_offsetParent};
-}
-
-const MinMaxStore::Element& MinMaxStore::ObjView::element() const {
-    return _pos->_element;
-}
-
-MinMaxStore::ConstIterator MinMaxStore::ObjView::iterator() const {
-    return {_pos};
-}
-
-MinMaxStore::ConstIterator MinMaxStore::ObjView::begin() const {
-    return {_pos + 1};
-}
-
-MinMaxStore::ConstIterator MinMaxStore::ObjView::end() const {
-    return {_pos + _pos->_offsetEnd};
-}
-
 MinMaxStore::Obj::Obj(MinMaxStore::Entries& entries, MinMaxStore::Entries::iterator pos)
     : _entries(entries), _pos(pos) {}
 
-MinMaxStore::Obj::operator MinMaxStore::ObjView() {
-    return {_entries, _pos};
-}
 
 MinMaxStore::Obj& MinMaxStore::Obj::operator=(const MinMaxStore::Obj& rhs) {
     if (this != &rhs) {
@@ -455,12 +417,11 @@ std::pair<MinMaxStore::Iterator, MinMaxStore::Iterator> MinMax::_update(
     };
 
     if (elem.type() == Object) {
-        auto shouldUpdateObject = [&](MinMaxStore::Data& data, auto compare) {
+        auto shouldUpdateObject = [&](MinMaxStore::Data& data, auto comp) {
             return data.type() == MinMaxStore::Type::kObject ||
                 data.type() == MinMaxStore::Type::kUnset ||
-                (data.type() == MinMaxStore::Type::kArray && compare(typeComp(Array), 0)) ||
-                (data.type() == MinMaxStore::Type::kValue &&
-                 compare(typeComp(data.valueType()), 0));
+                (data.type() == MinMaxStore::Type::kArray && comp(typeComp(Array), 0)) ||
+                (data.type() == MinMaxStore::Type::kValue && comp(typeComp(data.valueType()), 0));
         };
         bool updateMin =
             updateMinValues && shouldUpdateObject(obj.element().min(), std::less<int>{});
@@ -484,12 +445,11 @@ std::pair<MinMaxStore::Iterator, MinMaxStore::Iterator> MinMax::_update(
     }
 
     if (elem.type() == Array) {
-        auto shouldUpdateArray = [&](MinMaxStore::Data& data, auto compare) {
+        auto shouldUpdateArray = [&](MinMaxStore::Data& data, auto comp) {
             return data.type() == MinMaxStore::Type::kArray ||
                 data.type() == MinMaxStore::Type::kUnset ||
-                (data.type() == MinMaxStore::Type::kObject && compare(typeComp(Object), 0)) ||
-                (data.type() == MinMaxStore::Type::kValue &&
-                 compare(typeComp(data.valueType()), 0));
+                (data.type() == MinMaxStore::Type::kObject && comp(typeComp(Object), 0)) ||
+                (data.type() == MinMaxStore::Type::kValue && comp(typeComp(data.valueType()), 0));
         };
         bool updateMin =
             updateMinValues && shouldUpdateArray(obj.element().min(), std::less<int>{});
@@ -523,40 +483,40 @@ std::pair<MinMaxStore::Iterator, MinMaxStore::Iterator> MinMax::_update(
         return {obj.iterator(), obj.parent().end()};
     }
 
-    auto maybeUpdateValue = [&](MinMaxStore::Data& data, auto compare) {
+    auto maybeUpdateValue = [&](MinMaxStore::Data& data, auto comp) {
         if (data.type() == MinMaxStore::Type::kUnset ||
-            (data.type() == MinMaxStore::Type::kObject && compare(typeComp(Object), 0)) ||
-            (data.type() == MinMaxStore::Type::kArray && compare(typeComp(Array), 0)) ||
+            (data.type() == MinMaxStore::Type::kObject && comp(typeComp(Object), 0)) ||
+            (data.type() == MinMaxStore::Type::kArray && comp(typeComp(Array), 0)) ||
             (data.type() == MinMaxStore::Type::kValue &&
-             compare(elem.woCompare(data.value(), false, stringComparator), 0))) {
+             comp(elem.woCompare(data.value(), false, stringComparator), 0))) {
             data.setValue(elem);
         }
     };
     if (updateMinValues) {
-        maybeUpdateValue(obj.element().min(), std::less<int>{});
+        maybeUpdateValue(obj.element().min(), std::less<>{});
     }
 
     if (updateMaxValues) {
-        maybeUpdateValue(obj.element().max(), std::greater<int>{});
+        maybeUpdateValue(obj.element().max(), std::greater<>{});
     }
 
     return {obj.iterator(), obj.parent().end()};
 }
 
-BSONObj MinMax::min() const {
+BSONObj MinMax::min() {
     BSONObjBuilder builder;
     _append(_store.root(), &builder, GetMin());
     return builder.obj();
 }
 
-BSONObj MinMax::max() const {
+BSONObj MinMax::max() {
     BSONObjBuilder builder;
     _append(_store.root(), &builder, GetMax());
     return builder.obj();
 }
 
 template <typename GetDataFn>
-void MinMax::_append(MinMaxStore::ObjView obj, BSONObjBuilder* builder, GetDataFn getData) const {
+void MinMax::_append(MinMaxStore::Obj obj, BSONObjBuilder* builder, GetDataFn getData) {
     for (auto it = obj.begin(); it != obj.end(); ++it) {
         const auto& data = getData(*it);
         if (data.type() == MinMaxStore::Type::kValue) {
@@ -568,11 +528,13 @@ void MinMax::_append(MinMaxStore::ObjView obj, BSONObjBuilder* builder, GetDataF
             BSONArrayBuilder subArr(builder->subarrayStart(it->fieldName()));
             _append(obj.object(it), &subArr, getData);
         }
+        if (data.updated())
+            _clearUpdated(it, getData);
     }
 }
 
 template <typename GetDataFn>
-void MinMax::_append(MinMaxStore::ObjView obj, BSONArrayBuilder* builder, GetDataFn getData) const {
+void MinMax::_append(MinMaxStore::Obj obj, BSONArrayBuilder* builder, GetDataFn getData) {
     for (auto it = obj.begin(); it != obj.end(); ++it) {
         const auto& data = getData(*it);
         if (data.type() == MinMaxStore::Type::kValue) {
@@ -584,6 +546,8 @@ void MinMax::_append(MinMaxStore::ObjView obj, BSONArrayBuilder* builder, GetDat
             BSONArrayBuilder subArr(builder->subarrayStart());
             _append(obj.object(it), &subArr, getData);
         }
+        if (data.updated())
+            _clearUpdated(it, getData);
     }
 }
 
@@ -625,7 +589,8 @@ bool MinMax::_appendUpdates(MinMaxStore::Obj obj, BSONObjBuilder* builder, GetDa
                 _clearUpdated(it, getData);
                 appended = true;
                 hasUpdateSection = true;
-            } else if (subdata.type() != MinMaxStore::Type::kValue) {
+            } else if (subdata.type() != MinMaxStore::Type::kValue &&
+                       subdata.type() != MinMaxStore::Type::kUnset) {
                 BSONObjBuilder subDiff;
                 if (_appendUpdates(obj.object(it), &subDiff, getData)) {
                     // An update occurred at a lower level, so append the sub diff.
@@ -662,7 +627,8 @@ bool MinMax::_appendUpdates(MinMaxStore::Obj obj, BSONObjBuilder* builder, GetDa
                 }
                 _clearUpdated(it, getData);
                 appended = true;
-            } else if (subdata.type() != MinMaxStore::Type::kValue) {
+            } else if (subdata.type() != MinMaxStore::Type::kValue &&
+                       subdata.type() != MinMaxStore::Type::kUnset) {
                 BSONObjBuilder subDiff;
                 if (_appendUpdates(obj.object(it), &subDiff, getData)) {
                     // An update occurred at a lower level, so append the sub diff.
