@@ -31,7 +31,9 @@
 
 #include "mongo/db/timeseries/timeseries_options.h"
 
-#include "mongo/db/catalog/collection_catalog.h"
+#include "mongo/bson/bsonobj.h"
+#include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 
 namespace mongo {
 
@@ -76,17 +78,6 @@ bool isValidTimeseriesGranularityTransition(BucketGranularityEnum current,
 
 }  // namespace
 
-boost::optional<TimeseriesOptions> getTimeseriesOptions(OperationContext* opCtx,
-                                                        const NamespaceString& nss) {
-    auto bucketsNs = nss.makeTimeseriesBucketsNamespace();
-    auto bucketsColl =
-        CollectionCatalog::get(opCtx)->lookupCollectionByNamespaceForRead(opCtx, bucketsNs);
-    if (!bucketsColl) {
-        return boost::none;
-    }
-    return bucketsColl->getTimeseriesOptions();
-}
-
 int getMaxSpanSecondsFromGranularity(BucketGranularityEnum granularity) {
     switch (granularity) {
         case BucketGranularityEnum::Seconds:
@@ -103,21 +94,6 @@ int getMaxSpanSecondsFromGranularity(BucketGranularityEnum granularity) {
     MONGO_UNREACHABLE;
 }
 
-int getBucketRoundingSecondsFromGranularity(BucketGranularityEnum granularity) {
-    switch (granularity) {
-        case BucketGranularityEnum::Seconds:
-            // Round down to nearest minute.
-            return 60;
-        case BucketGranularityEnum::Minutes:
-            // Round down to nearest hour.
-            return 60 * 60;
-        case BucketGranularityEnum::Hours:
-            // Round down to hearest day.
-            return 60 * 60 * 24;
-    }
-    MONGO_UNREACHABLE;
-}
-
 StatusWith<std::pair<TimeseriesOptions, bool>> applyTimeseriesOptionsModifications(
     const TimeseriesOptions& currentOptions, const BSONObj& mod) {
     TimeseriesOptions newOptions = currentOptions;
@@ -149,15 +125,52 @@ BSONObj generateViewPipeline(const TimeseriesOptions& options, bool asArray) {
             asArray,
             BSON("$_internalUnpackBucket" << BSON(
                      "timeField" << options.getTimeField() << "metaField" << *options.getMetaField()
-                                 << "bucketMaxSpanSeconds" << *options.getBucketMaxSpanSeconds()
-                                 << "exclude" << BSONArray())));
+                                 << "bucketMaxSpanSeconds" << *options.getBucketMaxSpanSeconds())));
     }
-    return wrapInArrayIf(
-        asArray,
-        BSON("$_internalUnpackBucket" << BSON(
-                 "timeField" << options.getTimeField() << "bucketMaxSpanSeconds"
-                             << *options.getBucketMaxSpanSeconds() << "exclude" << BSONArray())));
+    return wrapInArrayIf(asArray,
+                         BSON("$_internalUnpackBucket" << BSON(
+                                  "timeField" << options.getTimeField() << "bucketMaxSpanSeconds"
+                                              << *options.getBucketMaxSpanSeconds())));
 }
 
+bool optionsAreEqual(const TimeseriesOptions& option1, const TimeseriesOptions& option2) {
+    const auto option1BucketSpan = option1.getBucketMaxSpanSeconds()
+        ? *option1.getBucketMaxSpanSeconds()
+        : getMaxSpanSecondsFromGranularity(option1.getGranularity());
+    const auto option2BucketSpan = option2.getBucketMaxSpanSeconds()
+        ? *option2.getBucketMaxSpanSeconds()
+        : getMaxSpanSecondsFromGranularity(option2.getGranularity());
+    return option1.getTimeField() == option1.getTimeField() &&
+        option1.getMetaField() == option2.getMetaField() &&
+        option1.getGranularity() == option2.getGranularity() &&
+        option1BucketSpan == option2BucketSpan;
+}
+
+namespace {
+/**
+ * Returns the number of seconds used to round down the bucket ID and control.min timestamp.
+ */
+int getBucketRoundingSecondsFromGranularity(BucketGranularityEnum granularity) {
+    switch (granularity) {
+        case BucketGranularityEnum::Seconds:
+            // Round down to nearest minute.
+            return 60;
+        case BucketGranularityEnum::Minutes:
+            // Round down to nearest hour.
+            return 60 * 60;
+        case BucketGranularityEnum::Hours:
+            // Round down to nearest day.
+            return 60 * 60 * 24;
+    }
+    MONGO_UNREACHABLE;
+}
+}  // namespace
+
+Date_t roundTimestampToGranularity(const Date_t& time, BucketGranularityEnum granularity) {
+    int roundingSeconds = getBucketRoundingSecondsFromGranularity(granularity);
+    long long timeSeconds = durationCount<Seconds>(time.toDurationSinceEpoch());
+    long long roundedTimeSeconds = (timeSeconds - (timeSeconds % roundingSeconds));
+    return Date_t::fromDurationSinceEpoch(Seconds{roundedTimeSeconds});
+}
 }  // namespace timeseries
 }  // namespace mongo
