@@ -43,6 +43,7 @@
 #include "mongo/db/update/delta_executor.h"
 #include "mongo/db/update/modifier_table.h"
 #include "mongo/db/update/object_replace_executor.h"
+#include "mongo/db/update/object_transform_executor.h"
 #include "mongo/db/update/path_support.h"
 #include "mongo/db/update/storage_validation.h"
 #include "mongo/db/update/update_oplog_entry_version.h"
@@ -153,18 +154,30 @@ void UpdateDriver::parse(
     uassert(51198, "Constant values may only be specified for pipeline updates", !constants);
 
     // Check if the update expression is a full object replacement.
-    if (updateMod.type() == write_ops::UpdateModification::Type::kReplacement) {
+    if (isDocReplacement(updateMod)) {
         uassert(ErrorCodes::FailedToParse,
                 "multi update is not supported for replacement-style update",
                 !multi);
 
-        _updateExecutor = std::make_unique<ObjectReplaceExecutor>(updateMod.getUpdateReplacement());
+        _updateExecutor = std::make_unique<ObjectReplaceExecutor>(updateMod.getUpdateClassic());
 
         // Register the fact that this driver will only do full object replacements.
         _updateType = UpdateType::kReplacement;
         return;
     }
 
+    if (updateMod.type() == write_ops::UpdateModification::Type::kTransform) {
+        uassert(5857811, "multi update is not supported for transform-style update", !multi);
+
+        uassert(5857812,
+                "arrayFilters may not be specified for transform-syle updates",
+                arrayFilters.empty());
+
+        _updateType = UpdateType::kTransform;
+        _updateExecutor = std::make_unique<ObjectTransformExecutor>(updateMod.getTransform());
+        return;
+    }
+
     invariant(_updateType == UpdateType::kOperator);
 
     // By this point we are expecting a "classic" update. This version of mongod only supports $v:
@@ -172,7 +185,7 @@ void UpdateDriver::parse(
     // checked whether this is a delta update so we check that the $v field isn't present, or has a
     // value of 1.
 
-    auto updateExpr = updateMod.getUpdateModifier();
+    auto updateExpr = updateMod.getUpdateClassic();
     BSONElement versionElement = updateExpr[kUpdateOplogEntryVersionFieldName];
     if (versionElement) {
         uassert(ErrorCodes::FailedToParse,
@@ -258,6 +271,7 @@ Status UpdateDriver::update(OperationContext* opCtx,
     applyParams.matchedField = matchedField;
     applyParams.insert = isInsert;
     applyParams.fromOplogApplication = _fromOplogApplication;
+    applyParams.skipDotsDollarsCheck = _skipDotsDollarsCheck;
     applyParams.validateForStorage = validateForStorage;
     applyParams.indexData = _indexedFields;
     applyParams.modifiedPaths = modifiedPaths;
@@ -265,18 +279,7 @@ Status UpdateDriver::update(OperationContext* opCtx,
     invariant(!modifiedPaths || modifiedPaths->empty());
 
     if (_logOp && logOpRec) {
-        const auto& fcvState = serverGlobalParams.featureCompatibility;
-
-        // Updates may be run as part of the startup sequence, before the global FCV state has been
-        // initialized. We conservatively do not permit the use of $v:2 oplog entries in these
-        // situations.
-
-        // TODO SERVER-51075: Remove FCV check for $v:2 delta oplog entries.
-        const bool fcvAllowsV2Entries = fcvState.isVersionInitialized() &&
-            fcvState.isGreaterThanOrEqualTo(
-                ServerGlobalParams::FeatureCompatibility::Version::kVersion47);
-
-        applyParams.logMode = fcvAllowsV2Entries && internalQueryEnableLoggingV2OplogEntries.load()
+        applyParams.logMode = internalQueryEnableLoggingV2OplogEntries.load()
             ? ApplyParams::LogMode::kGenerateOplogEntry
             : ApplyParams::LogMode::kGenerateOnlyV1OplogEntry;
 
@@ -313,4 +316,10 @@ void UpdateDriver::setCollator(const CollatorInterface* collator) {
     }
 }
 
+bool UpdateDriver::isDocReplacement(const write_ops::UpdateModification& updateMod) {
+    return (updateMod.type() == write_ops::UpdateModification::Type::kClassic &&
+            *updateMod.getUpdateClassic().firstElementFieldName() != '$') ||
+        updateMod.type() == write_ops::UpdateModification::Type::kPipeline;
+}
+
 }  // namespace mongo
