@@ -69,7 +69,10 @@ private:
         return !replicationCoordinator ||
             (replicationCoordinator->getReplicationMode() ==
                  repl::ReplicationCoordinator::modeReplSet &&
-             replicationCoordinator->getMemberState().readable());
+             // Check repl status without locks to prevent deadlocks. This is a best effort check
+             // as the repl state can change right after this check even when inspected under a
+             // lock or mutex.
+             replicationCoordinator->isInPrimaryOrSecondaryState_UNSAFE());
     }
 
     bool _permitRefreshDuringGossipOut() const override {
@@ -81,7 +84,7 @@ private:
     SharedSemiFuture<void> waitForDurableConfigTime() override;
     SharedSemiFuture<void> waitForDurableTopologyTime() override;
     SharedSemiFuture<void> waitForDurable() override;
-    VectorClock::VectorTime recoverDirect(OperationContext* opCtx) override;
+    SharedSemiFuture<void> recover() override;
 
     LogicalTime _tick(Component component, uint64_t nTicks) override;
     void _tickTo(Component component, LogicalTime newTime) override;
@@ -89,8 +92,6 @@ private:
     // ReplicaSetAwareService methods implementation
 
     void onStartup(OperationContext* opCtx) override {}
-    void onStartupRecoveryComplete(OperationContext* opCtx) override {}
-    void onInitialSyncComplete(OperationContext* opCtx) override {}
     void onShutdown() override {}
     void onStepUpBegin(OperationContext* opCtx, long long term) override;
     void onStepUpComplete(OperationContext* opCtx, long long term) override {}
@@ -183,27 +184,6 @@ VectorClockMongoD::~VectorClockMongoD() = default;
 void VectorClockMongoD::onStepUpBegin(OperationContext* opCtx, long long term) {
     stdx::lock_guard lg(_mutex);
     _durableTime.reset();
-
-    // Initialize the config server's topology time to the maximum topology time from
-    // `config.shards` collection instead of using `Timestamp(0 ,0)`.
-    if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
-        const auto maxTopologyTime{[&opCtx]() -> boost::optional<Timestamp> {
-            DBDirectClient client{opCtx};
-            const auto query{Query().sort(BSON(ShardType::topologyTime << -1))};
-            const auto collEntry{client.findOne(ShardType::ConfigNS.ns(), query)};
-            if (collEntry.isEmpty()) {
-                // No shards are available yet.
-                return boost::none;
-            }
-
-            const auto shardEntry{uassertStatusOK(ShardType::fromBSON(collEntry))};
-            return shardEntry.getTopologyTime();
-        }()};
-
-        if (maxTopologyTime) {
-            _advanceComponentTimeTo(Component::TopologyTime, LogicalTime(*maxTopologyTime));
-        }
-    }
 }
 
 void VectorClockMongoD::onStepDown() {
@@ -251,27 +231,12 @@ SharedSemiFuture<void> VectorClockMongoD::waitForDurable() {
     return _enqueueWaiterAndScheduleLoopIfNeeded(std::move(ul), std::move(time));
 }
 
-VectorClock::VectorTime VectorClockMongoD::recoverDirect(OperationContext* opCtx) {
-    VectorClockDocument durableVectorClock;
+SharedSemiFuture<void> VectorClockMongoD::recover() {
+    stdx::unique_lock ul(_mutex);
+    if (_durableTime)
+        return SharedSemiFuture<void>();
 
-    PersistentTaskStore<VectorClockDocument> store(NamespaceString::kVectorClockNamespace);
-    store.forEach(opCtx,
-                  BSON(VectorClockDocument::k_idFieldName << durableVectorClock.get_id()),
-                  [&, numDocsFound = 0](const auto& doc) mutable {
-                      invariant(++numDocsFound == 1);
-                      durableVectorClock = doc;
-                      return true;
-                  });
-
-    const auto newDurableTime = VectorTime({VectorClock::kInitialComponentTime,
-                                            LogicalTime(durableVectorClock.getConfigTime()),
-                                            LogicalTime(durableVectorClock.getTopologyTime())});
-
-    // Make sure the VectorClock advances at least up to the just recovered durable time
-    _advanceTime(
-        {newDurableTime.clusterTime(), newDurableTime.configTime(), newDurableTime.topologyTime()});
-
-    return newDurableTime;
+    return _enqueueWaiterAndScheduleLoopIfNeeded(std::move(ul), VectorTime());
 }
 
 SharedSemiFuture<void> VectorClockMongoD::_enqueueWaiterAndScheduleLoopIfNeeded(
@@ -311,6 +276,12 @@ Future<void> VectorClockMongoD::_doWhileQueueNotEmptyOrError(ServiceContext* ser
                           }
                           ul.unlock();
 
+                          // Make sure the VectorClock advances at least up to the just recovered
+                          // durable time
+                          _advanceTime({newDurableTime.clusterTime(),
+                                        newDurableTime.configTime(),
+                                        newDurableTime.topologyTime()});
+
                           for (auto& p : promises)
                               p->emplaceValue();
                       })
@@ -356,21 +327,33 @@ Future<void> VectorClockMongoD::_doWhileQueueNotEmptyOrError(ServiceContext* ser
             auto* const opCtx = opCtxHolder.get();
 
             if (mustRecoverDurableTime) {
-                return recoverDirect(opCtx);
+                VectorClockDocument durableVectorClock;
+
+                PersistentTaskStore<VectorClockDocument> store(
+                    NamespaceString::kVectorClockNamespace);
+                store.forEach(
+                    opCtx,
+                    BSON(VectorClockDocument::k_idFieldName << durableVectorClock.get_id()),
+                    [&, numDocsFound = 0](const auto& doc) mutable {
+                        invariant(++numDocsFound == 1);
+                        durableVectorClock = doc;
+                        return true;
+                    });
+
+                return VectorTime({LogicalTime(Timestamp(0)),
+                                   LogicalTime(durableVectorClock.getConfigTime()),
+                                   LogicalTime(durableVectorClock.getTopologyTime())});
             }
 
             auto vectorTime = getTime();
-
-            VectorClockDocument vcd;
-            vcd.setConfigTime(vectorTime.configTime().asTimestamp());
-            vcd.setTopologyTime(vectorTime.topologyTime().asTimestamp());
+            const VectorClockDocument vcd(vectorTime.configTime().asTimestamp(),
+                                          vectorTime.topologyTime().asTimestamp());
 
             PersistentTaskStore<VectorClockDocument> store(NamespaceString::kVectorClockNamespace);
             store.upsert(opCtx,
-                         QUERY(VectorClockDocument::k_idFieldName << vcd.get_id()),
+                         BSON(VectorClockDocument::k_idFieldName << vcd.get_id()),
                          vcd.toBSON(),
-                         WriteConcerns::kMajorityWriteConcernNoTimeout);
-
+                         WriteConcerns::kMajorityWriteConcern);
             return vectorTime;
         })
         .getAsync([this, promise = std::move(p)](StatusWith<VectorTime> swResult) mutable {
@@ -427,8 +410,10 @@ void VectorClockMongoD::_tickTo(Component component, LogicalTime newTime) {
         return;
     }
 
-    if (component == Component::ConfigTime &&
-        serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
+    if (component == Component::ConfigTime) {
+        // The ConfigTime is allowed to be tickTo on the ConfigServer and on the shard only when
+        // called from ShardingStateRecovery
+        // TODO SERVER-55032 re-add clusterRole == configServer condition like for TopologyTime
         _advanceComponentTimeTo(component, std::move(newTime));
         return;
     }
