@@ -169,11 +169,8 @@ Status ViewCatalog::reload(OperationContext* opCtx,
                            const Database* db,
                            ViewCatalogLookupBehavior lookupBehavior) {
     auto catalog = getViewCatalog(db).writer();
-    Lock::CollectionLock systemViewsLock(
-        opCtx,
-        NamespaceString(catalog->_durable->getName(),
-                        NamespaceString::kSystemDotViewsCollectionName),
-        MODE_IS);
+    invariant(opCtx->lockState()->isCollectionLockedForMode(
+        NamespaceString(db->name(), NamespaceString::kSystemDotViewsCollectionName), MODE_IS));
     auto result =
         catalog.writable()->_reload(opCtx, ViewCatalogLookupBehavior::kValidateDurableViews, true);
     catalog.commit();
@@ -458,10 +455,10 @@ StatusWith<stdx::unordered_set<NamespaceString>> ViewCatalog::validatePipeline(
     // primary, ban the use of new agg features introduced in kLatest to prevent them from being
     // persisted in the catalog.
     // (Generic FCV reference): This FCV check should exist across LTS binary versions.
-    ServerGlobalParams::FeatureCompatibility::Version fcv;
+    multiversion::FeatureCompatibilityVersion fcv;
     if (serverGlobalParams.validateFeaturesAsPrimary.load() &&
-        serverGlobalParams.featureCompatibility.isLessThan(
-            ServerGlobalParams::FeatureCompatibility::kLatest, &fcv)) {
+        serverGlobalParams.featureCompatibility.isLessThan(multiversion::GenericFCV::kLatest,
+                                                           &fcv)) {
         expCtx->maxFeatureCompatibilityVersion = fcv;
     }
 
@@ -511,7 +508,7 @@ Status ViewCatalog::_validateCollation(OperationContext* opCtx,
                                        const std::vector<NamespaceString>& refs) const {
     for (auto&& potentialViewNss : refs) {
         auto otherView =
-            _lookup(opCtx, potentialViewNss.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+            _lookup(opCtx, potentialViewNss, ViewCatalogLookupBehavior::kValidateDurableViews);
         if (otherView &&
             !CollatorInterface::collatorsMatch(view.defaultCollator(),
                                                otherView->defaultCollator())) {
@@ -542,8 +539,7 @@ Status ViewCatalog::createView(OperationContext* opCtx,
         return Status(ErrorCodes::BadValue,
                       "View must be created on a view or collection in the same database");
 
-    if (catalog->_lookup(
-            opCtx, StringData(viewName.ns()), ViewCatalogLookupBehavior::kValidateDurableViews))
+    if (catalog->_lookup(opCtx, viewName, ViewCatalogLookupBehavior::kValidateDurableViews))
         return Status(ErrorCodes::NamespaceExists, "Namespace already exists");
 
     if (!NamespaceString::validCollectionName(viewOn.coll()))
@@ -585,7 +581,7 @@ Status ViewCatalog::modifyView(OperationContext* opCtx,
                       "View must be created on a view or collection in the same database");
 
     auto viewPtr =
-        catalog->_lookup(opCtx, viewName.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+        catalog->_lookup(opCtx, viewName, ViewCatalogLookupBehavior::kValidateDurableViews);
     if (!viewPtr)
         return Status(ErrorCodes::NamespaceNotFound,
                       str::stream() << "cannot modify missing view " << viewName.ns());
@@ -642,8 +638,8 @@ Status ViewCatalog::dropView(OperationContext* opCtx,
         catalogStorage.setIgnoreExternalChange(true);
 
         // Save a copy of the view definition in case we need to roll back.
-        auto viewPtr = catalog->_lookup(
-            opCtx, viewName.ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+        auto viewPtr =
+            catalog->_lookup(opCtx, viewName, ViewCatalogLookupBehavior::kValidateDurableViews);
         if (!viewPtr) {
             return {ErrorCodes::NamespaceNotFound,
                     str::stream() << "cannot drop missing view: " << viewName.ns()};
@@ -679,8 +675,10 @@ Status ViewCatalog::dropView(OperationContext* opCtx,
 }
 
 std::shared_ptr<const ViewDefinition> ViewCatalog::_lookup(
-    OperationContext* opCtx, StringData ns, ViewCatalogLookupBehavior lookupBehavior) const {
-    ViewMap::const_iterator it = _viewMap.find(ns);
+    OperationContext* opCtx,
+    const NamespaceString& ns,
+    ViewCatalogLookupBehavior lookupBehavior) const {
+    ViewMap::const_iterator it = _viewMap.find(ns.ns());
     if (it != _viewMap.end()) {
         return it->second;
     }
@@ -688,17 +686,17 @@ std::shared_ptr<const ViewDefinition> ViewCatalog::_lookup(
 }
 
 std::shared_ptr<ViewDefinition> ViewCatalog::_lookup(OperationContext* opCtx,
-                                                     StringData ns,
+                                                     const NamespaceString& ns,
                                                      ViewCatalogLookupBehavior lookupBehavior) {
     return std::const_pointer_cast<ViewDefinition>(
         std::as_const(*this)._lookup(opCtx, ns, lookupBehavior));
 }
 
 std::shared_ptr<const ViewDefinition> ViewCatalog::lookup(OperationContext* opCtx,
-                                                          StringData ns) const {
+                                                          const NamespaceString& ns) const {
     if (!_valid && opCtx->getClient()->isFromUserConnection()) {
         // We want to avoid lookups on invalid collection names.
-        if (!NamespaceString::validCollectionName(ns)) {
+        if (!NamespaceString::validCollectionName(ns.ns())) {
             return nullptr;
         }
 
@@ -712,12 +710,14 @@ std::shared_ptr<const ViewDefinition> ViewCatalog::lookup(OperationContext* opCt
 }
 
 std::shared_ptr<const ViewDefinition> ViewCatalog::lookupWithoutValidatingDurableViews(
-    OperationContext* opCtx, StringData ns) const {
+    OperationContext* opCtx, const NamespaceString& ns) const {
     return _lookup(opCtx, ns, ViewCatalogLookupBehavior::kAllowInvalidDurableViews);
 }
 
-StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
-                                                  const NamespaceString& nss) const {
+StatusWith<ResolvedView> ViewCatalog::resolveView(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    boost::optional<BSONObj> timeSeriesCollator) const {
     _requireValidCatalog();
 
     // Keep looping until the resolution completes. If the catalog is invalidated during the
@@ -742,7 +742,7 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
         int depth = 0;
         for (; depth < ViewGraph::kMaxViewDepth; depth++) {
             auto view =
-                _lookup(opCtx, resolvedNss->ns(), ViewCatalogLookupBehavior::kValidateDurableViews);
+                _lookup(opCtx, *resolvedNss, ViewCatalogLookupBehavior::kValidateDurableViews);
             if (!view) {
                 // Return error status if pipeline is too large.
                 int pipelineSize = 0;
@@ -767,8 +767,13 @@ StatusWith<ResolvedView> ViewCatalog::resolveView(OperationContext* opCtx,
             resolvedNss = &view->viewOn();
             dependencyChain.push_back(*resolvedNss);
             if (!collation) {
-                collation = view->defaultCollator() ? view->defaultCollator()->getSpec().toBSON()
-                                                    : CollationSpec::kSimpleSpec;
+                if (timeSeriesCollator) {
+                    collation = *timeSeriesCollator;
+                } else {
+                    collation = view->defaultCollator()
+                        ? view->defaultCollator()->getSpec().toBSON()
+                        : CollationSpec::kSimpleSpec;
+                }
             }
 
             // Prepend the underlying view's pipeline to the current working pipeline.
