@@ -34,7 +34,6 @@
 #include "mongo/db/client.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/json.h"
-#include "mongo/db/lasterror.h"
 #include "mongo/dbtests/dbtests.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/util/timer.h"
@@ -44,52 +43,9 @@ namespace DirectClientTests {
 using std::unique_ptr;
 using std::vector;
 
-class ClientBase {
-public:
-    ClientBase() {
-        mongo::LastError::get(cc()).reset();
-    }
-    virtual ~ClientBase() {
-        mongo::LastError::get(cc()).reset();
-    }
-};
-
 const char* ns = "a.b";
 
-class Capped : public ClientBase {
-public:
-    virtual void run() {
-        // Skip the test if the storage engine doesn't support capped collections.
-        if (!getGlobalServiceContext()->getStorageEngine()->supportsCappedCollections()) {
-            return;
-        }
-
-        const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
-        OperationContext& opCtx = *opCtxPtr;
-        DBDirectClient client(&opCtx);
-        for (int pass = 0; pass < 3; pass++) {
-            client.createCollection(ns, 1024 * 1024, true, 999);
-            for (int j = 0; j < pass * 3; j++)
-                client.insert(ns, BSON("x" << j));
-
-            // test truncation of a capped collection
-            if (pass) {
-                BSONObj info;
-                BSONObj cmd = BSON("captrunc"
-                                   << "b"
-                                   << "n" << 1 << "inc" << true);
-                // cout << cmd.toString() << endl;
-                bool ok = client.runCommand("a", cmd, info);
-                // cout << info.toString() << endl;
-                verify(ok);
-            }
-
-            verify(client.dropCollection(ns));
-        }
-    }
-};
-
-class InsertMany : ClientBase {
+class InsertMany {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
@@ -101,20 +57,21 @@ public:
         objs.push_back(BSON("_id" << 1));
         objs.push_back(BSON("_id" << 2));
 
-
         client.dropCollection(ns);
-        client.insert(ns, objs);
-        ASSERT_EQUALS(client.getLastErrorDetailed()["code"].numberInt(), 11000);
+
+        auto response = client.insertAcknowledged(ns, objs);
+        ASSERT_EQUALS(ErrorCodes::DuplicateKey, getStatusFromWriteCommandReply(response));
         ASSERT_EQUALS((int)client.count(NamespaceString(ns)), 1);
 
         client.dropCollection(ns);
-        client.insert(ns, objs, InsertOption_ContinueOnError);
-        ASSERT_EQUALS(client.getLastErrorDetailed()["code"].numberInt(), 11000);
+
+        response = client.insertAcknowledged(ns, objs, false /*ordered*/);
+        ASSERT_EQUALS(ErrorCodes::DuplicateKey, getStatusFromWriteCommandReply(response));
         ASSERT_EQUALS((int)client.count(NamespaceString(ns)), 2);
     }
 };
 
-class BadNSCmd : ClientBase {
+class BadNSCmd {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
@@ -129,20 +86,20 @@ public:
     }
 };
 
-class BadNSQuery : ClientBase {
+class BadNSQuery {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         DBDirectClient client(&opCtx);
 
-        ASSERT_THROWS_CODE(client.query(NamespaceString(), Query(), 1)->nextSafe(),
+        ASSERT_THROWS_CODE(client.query(NamespaceString(), BSONObj{}, Query(), 1)->nextSafe(),
                            AssertionException,
                            ErrorCodes::InvalidNamespace);
     }
 };
 
-class BadNSGetMore : ClientBase {
+class BadNSGetMore {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
@@ -150,43 +107,44 @@ public:
         DBDirectClient client(&opCtx);
 
         ASSERT_THROWS_CODE(
-            client.getMore("", 1, 1)->nextSafe(), AssertionException, ErrorCodes::InvalidNamespace);
+            client.getMore("", 1)->nextSafe(), AssertionException, ErrorCodes::InvalidNamespace);
     }
 };
 
-class BadNSInsert : ClientBase {
+class BadNSInsert {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         DBDirectClient client(&opCtx);
 
-        client.insert("", BSONObj(), 0);
-        ASSERT(!client.getLastError().empty());
+        auto response = client.insertAcknowledged("", {BSONObj()});
+        ASSERT_EQ(ErrorCodes::InvalidNamespace, getStatusFromCommandResult(response));
     }
 };
 
-class BadNSUpdate : ClientBase {
+class BadNSUpdate {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         DBDirectClient client(&opCtx);
 
-        client.update("", Query(), BSON("$set" << BSON("x" << 1)));
-        ASSERT(!client.getLastError().empty());
+        auto response =
+            client.updateAcknowledged("", BSONObj{} /*filter*/, BSON("$set" << BSON("x" << 1)));
+        ASSERT_EQ(ErrorCodes::InvalidNamespace, getStatusFromCommandResult(response));
     }
 };
 
-class BadNSRemove : ClientBase {
+class BadNSRemove {
 public:
     virtual void run() {
         const ServiceContext::UniqueOperationContext opCtxPtr = cc().makeOperationContext();
         OperationContext& opCtx = *opCtxPtr;
         DBDirectClient client(&opCtx);
 
-        client.remove("", Query());
-        ASSERT(!client.getLastError().empty());
+        auto response = client.removeAcknowledged("", BSONObj{} /*filter*/);
+        ASSERT_EQ(ErrorCodes::InvalidNamespace, getStatusFromCommandResult(response));
     }
 };
 
@@ -194,7 +152,6 @@ class All : public OldStyleSuiteSpecification {
 public:
     All() : OldStyleSuiteSpecification("directclient") {}
     void setupTests() {
-        add<Capped>();
         add<InsertMany>();
         add<BadNSCmd>();
         add<BadNSQuery>();
