@@ -49,6 +49,7 @@
 #include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/query/get_executor.h"
 #include "mongo/db/query/mock_yield_policies.h"
+#include "mongo/db/query/plan_cache_key_factory.h"
 #include "mongo/db/query/plan_executor_factory.h"
 #include "mongo/db/query/plan_executor_impl.h"
 #include "mongo/db/query/plan_summary_stats.h"
@@ -148,7 +149,7 @@ unique_ptr<PlanStage> getIxScanPlan(ExpressionContext* expCtx,
         expCtx->opCtx, BSON("foo" << 1), false, &indexes);
     ASSERT_EQ(indexes.size(), 1U);
 
-    IndexScanParams ixparams(expCtx->opCtx, indexes[0]);
+    IndexScanParams ixparams(expCtx->opCtx, coll, indexes[0]);
     ixparams.bounds.isSimpleRange = true;
     ixparams.bounds.startKey = BSON("" << desiredFooValue);
     ixparams.bounds.endKey = BSON("" << desiredFooValue);
@@ -181,11 +182,6 @@ unique_ptr<PlanStage> getCollScanPlan(ExpressionContext* expCtx,
     return root;
 }
 
-const PlanStage* getBestPlanRoot(const MultiPlanStage* mps) {
-    auto bestPlanIdx = mps->bestPlanIdx();
-    return bestPlanIdx ? mps->getChildren()[bestPlanIdx.get()].get() : nullptr;
-}
-
 std::unique_ptr<MultiPlanStage> runMultiPlanner(ExpressionContext* expCtx,
                                                 const NamespaceString& nss,
                                                 const CollectionPtr& coll,
@@ -195,7 +191,6 @@ std::unique_ptr<MultiPlanStage> runMultiPlanner(ExpressionContext* expCtx,
     // at least).
     unique_ptr<WorkingSet> sharedWs(new WorkingSet());
     unique_ptr<PlanStage> ixScanRoot = getIxScanPlan(expCtx, coll, sharedWs.get(), desiredFooValue);
-    const auto* ixScanRootPtr = ixScanRoot.get();
 
     // Plan 1: CollScan.
     BSONObj filterObj = BSON("foo" << desiredFooValue);
@@ -214,7 +209,7 @@ std::unique_ptr<MultiPlanStage> runMultiPlanner(ExpressionContext* expCtx,
     NoopYieldPolicy yieldPolicy(expCtx->opCtx->getServiceContext()->getFastClockSource());
     ASSERT_OK(mps->pickBestPlan(&yieldPolicy));
     ASSERT(mps->bestPlanChosen());
-    ASSERT_EQUALS(getBestPlanRoot(mps.get()), ixScanRootPtr);
+    ASSERT_EQUALS(0, *mps->bestPlanIdx());
 
     return mps;
 }
@@ -247,8 +242,6 @@ TEST_F(QueryStageMultiPlanTest, MPSCollectionScanVsHighlySelectiveIXScan) {
     unique_ptr<WorkingSet> sharedWs(new WorkingSet());
     unique_ptr<PlanStage> ixScanRoot = getIxScanPlan(_expCtx.get(), coll, sharedWs.get(), 7);
 
-    const auto* ixScanRootPtr = ixScanRoot.get();
-
     // Plan 1: CollScan with matcher.
     BSONObj filterObj = BSON("foo" << 7);
     unique_ptr<MatchExpression> filter = makeMatchExpressionFromFilter(_expCtx.get(), filterObj);
@@ -263,7 +256,11 @@ TEST_F(QueryStageMultiPlanTest, MPSCollectionScanVsHighlySelectiveIXScan) {
     mps->addPlan(createQuerySolution(), std::move(ixScanRoot), sharedWs.get());
     mps->addPlan(createQuerySolution(), std::move(collScanRoot), sharedWs.get());
 
-    const auto* mpsPtr = mps.get();
+    // Plan 0 aka the first plan aka the index scan should be the best.
+    NoopYieldPolicy yieldPolicy(_clock);
+    ASSERT_OK(mps->pickBestPlan(&yieldPolicy));
+    ASSERT(mps->bestPlanChosen());
+    ASSERT_EQUALS(0, *mps->bestPlanIdx());
 
     // Takes ownership of arguments other than 'collection'.
     auto statusWithPlanExecutor =
@@ -276,9 +273,6 @@ TEST_F(QueryStageMultiPlanTest, MPSCollectionScanVsHighlySelectiveIXScan) {
     ASSERT_OK(statusWithPlanExecutor.getStatus());
     auto exec = std::move(statusWithPlanExecutor.getValue());
 
-    ASSERT_TRUE(mpsPtr->bestPlanChosen());
-    ASSERT_EQUALS(mpsPtr->getChildren()[mpsPtr->bestPlanIdx().get()].get(), ixScanRootPtr);
-
     // Get all our results out.
     int results = 0;
     BSONObj obj;
@@ -305,6 +299,7 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     const CollectionPtr& coll = ctx.getCollection();
 
     const auto cq = makeCanonicalQuery(_opCtx.get(), nss, BSON("foo" << 7));
+    auto key = plan_cache_key_factory::make<PlanCacheKey>(*cq, coll);
 
     // Run an index scan and collection scan, searching for {foo: 7}.
     auto mps = runMultiPlanner(_expCtx.get(), nss, coll, 7);
@@ -312,7 +307,7 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     // Be sure that an inactive cache entry was added.
     PlanCache* cache = CollectionQueryInfo::get(coll).getPlanCache();
     ASSERT_EQ(cache->size(), 1U);
-    auto entry = assertGet(cache->getEntry(*cq));
+    auto entry = assertGet(cache->getEntry(key));
     ASSERT_FALSE(entry->isActive);
     const size_t firstQueryWorks = getBestPlanWorks(mps.get());
     ASSERT_EQ(firstQueryWorks, entry->works);
@@ -324,7 +319,7 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     // The last plan run should have required far more works than the previous plan. This means
     // that the 'works' in the cache entry should have doubled.
     ASSERT_EQ(cache->size(), 1U);
-    entry = assertGet(cache->getEntry(*cq));
+    entry = assertGet(cache->getEntry(key));
     ASSERT_FALSE(entry->isActive);
     ASSERT_EQ(firstQueryWorks * 2, entry->works);
 
@@ -332,14 +327,14 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesNotCreateActiveCacheEntryImmediately) {
     // should cause the cache entry's 'works' to be doubled again.
     mps = runMultiPlanner(_expCtx.get(), nss, coll, 5);
     ASSERT_EQ(cache->size(), 1U);
-    entry = assertGet(cache->getEntry(*cq));
+    entry = assertGet(cache->getEntry(key));
     ASSERT_FALSE(entry->isActive);
     ASSERT_EQ(firstQueryWorks * 2 * 2, entry->works);
 
     // Run the query yet again. This time, an active cache entry should be created.
     mps = runMultiPlanner(_expCtx.get(), nss, coll, 5);
     ASSERT_EQ(cache->size(), 1U);
-    entry = assertGet(cache->getEntry(*cq));
+    entry = assertGet(cache->getEntry(key));
     ASSERT_TRUE(entry->isActive);
     ASSERT_EQ(getBestPlanWorks(mps.get()), entry->works);
 }
@@ -360,18 +355,19 @@ TEST_F(QueryStageMultiPlanTest, MPSDoesCreatesActiveEntryWhenInactiveEntriesDisa
     const CollectionPtr& coll = ctx.getCollection();
 
     const auto cq = makeCanonicalQuery(_opCtx.get(), nss, BSON("foo" << 7));
+    auto key = plan_cache_key_factory::make<PlanCacheKey>(*cq, coll);
 
     // Run an index scan and collection scan, searching for {foo: 7}.
     auto mps = runMultiPlanner(_expCtx.get(), nss, coll, 7);
 
     // Be sure that an _active_ cache entry was added.
     PlanCache* cache = CollectionQueryInfo::get(coll).getPlanCache();
-    ASSERT_EQ(cache->get(*cq).state, PlanCache::CacheEntryState::kPresentActive);
+    ASSERT_EQ(cache->get(key).state, PlanCache::CacheEntryState::kPresentActive);
 
     // Run the multi-planner again. The entry should still be active.
     mps = runMultiPlanner(_expCtx.get(), nss, coll, 5);
 
-    ASSERT_EQ(cache->get(*cq).state, PlanCache::CacheEntryState::kPresentActive);
+    ASSERT_EQ(cache->get(key).state, PlanCache::CacheEntryState::kPresentActive);
 }
 
 // Case in which we select a blocking plan as the winner, and a non-blocking plan
@@ -394,6 +390,7 @@ TEST_F(QueryStageMultiPlanTest, MPSBackupPlan) {
     verify(statusWithCQ.isOK());
     unique_ptr<CanonicalQuery> cq = std::move(statusWithCQ.getValue());
     ASSERT(nullptr != cq.get());
+    auto key = plan_cache_key_factory::make<PlanCacheKey>(*cq, collection.getCollection());
 
     // Force index intersection.
     bool forceIxisectOldValue = internalQueryForceIntersectionPlans.load();
@@ -404,9 +401,9 @@ TEST_F(QueryStageMultiPlanTest, MPSBackupPlan) {
     fillOutPlannerParams(_opCtx.get(), collection.getCollection(), cq.get(), &plannerParams);
 
     // Plan.
-    auto statusWithSolutions = QueryPlanner::plan(*cq, plannerParams);
-    ASSERT_OK(statusWithSolutions.getStatus());
-    auto solutions = std::move(statusWithSolutions.getValue());
+    auto&& [statusWithMultiPlanSolns, _] = QueryPlanner::plan(*cq, plannerParams);
+    ASSERT_OK(statusWithMultiPlanSolns.getStatus());
+    auto solutions = std::move(statusWithMultiPlanSolns.getValue());
 
     // We expect a plan using index {a: 1} and plan using index {b: 1} and
     // an index intersection plan.
@@ -431,12 +428,12 @@ TEST_F(QueryStageMultiPlanTest, MPSBackupPlan) {
 
     // We should have picked the index intersection plan due to forcing ixisect.
     auto soln = static_cast<const MultiPlanStage*>(mps.get())->bestSolution();
-    ASSERT(
-        QueryPlannerTestLib::solutionMatches("{sort: {pattern: {b: 1}, limit: 0, node:"
-                                             "{fetch: {node: {andSorted: {nodes: ["
-                                             "{ixscan: {filter: null, pattern: {a:1}}},"
-                                             "{ixscan: {filter: null, pattern: {b:1}}}]}}}}}}",
-                                             soln->root()));
+    ASSERT(QueryPlannerTestLib::solutionMatches("{sort: {pattern: {b: 1}, limit: 0, node:"
+                                                "{fetch: {node: {andSorted: {nodes: ["
+                                                "{ixscan: {filter: null, pattern: {a:1}}},"
+                                                "{ixscan: {filter: null, pattern: {b:1}}}]}}}}}}",
+                                                soln->root())
+               .isOK());
 
     // Get the resulting document.
     PlanStage::StageState state = PlanStage::NEED_TIME;
@@ -455,12 +452,12 @@ TEST_F(QueryStageMultiPlanTest, MPSBackupPlan) {
     // and the winning plan should still be the index intersection one.
     ASSERT(!mps->hasBackupPlan());
     soln = static_cast<const MultiPlanStage*>(mps.get())->bestSolution();
-    ASSERT(
-        QueryPlannerTestLib::solutionMatches("{sort: {pattern: {b: 1}, limit: 0, node:"
-                                             "{fetch: {node: {andSorted: {nodes: ["
-                                             "{ixscan: {filter: null, pattern: {a:1}}},"
-                                             "{ixscan: {filter: null, pattern: {b:1}}}]}}}}}}",
-                                             soln->root()));
+    ASSERT(QueryPlannerTestLib::solutionMatches("{sort: {pattern: {b: 1}, limit: 0, node:"
+                                                "{fetch: {node: {andSorted: {nodes: ["
+                                                "{ixscan: {filter: null, pattern: {a:1}}},"
+                                                "{ixscan: {filter: null, pattern: {b:1}}}]}}}}}}",
+                                                soln->root())
+               .isOK());
 
     // Restore index intersection force parameter.
     internalQueryForceIntersectionPlans.store(forceIxisectOldValue);
@@ -487,8 +484,6 @@ TEST_F(QueryStageMultiPlanTest, MPSExplainAllPlans) {
 
     auto ws = std::make_unique<WorkingSet>();
     auto firstPlan = std::make_unique<MockStage>(_expCtx.get(), ws.get());
-    const auto* firstPlanPtr = firstPlan.get();
-
     auto secondPlan = std::make_unique<MockStage>(_expCtx.get(), ws.get());
 
     for (int i = 0; i < nDocs; ++i) {
@@ -524,7 +519,7 @@ TEST_F(QueryStageMultiPlanTest, MPSExplainAllPlans) {
     auto root = static_cast<MultiPlanStage*>(execImpl->getRootStage());
     ASSERT_TRUE(root->bestPlanChosen());
     // The first candidate plan should have won.
-    ASSERT_EQ(getBestPlanRoot(root), firstPlanPtr);
+    ASSERT_EQ(*root->bestPlanIdx(), 0);
 
     BSONObjBuilder bob;
     Explain::explainStages(exec.get(),
@@ -556,8 +551,7 @@ TEST_F(QueryStageMultiPlanTest, MPSExplainAllPlans) {
 //
 // This is a regression test for SERVER-20111.
 TEST_F(QueryStageMultiPlanTest, MPSSummaryStats) {
-    RAIIServerParameterControllerForTest controller("internalQueryEnableSlotBasedExecutionEngine",
-                                                    false);
+    RAIIServerParameterControllerForTest controller("internalQueryForceClassicEngine", true);
 
     const int N = 5000;
     for (int i = 0; i < N; ++i) {
@@ -575,8 +569,12 @@ TEST_F(QueryStageMultiPlanTest, MPSSummaryStats) {
     auto findCommand = std::make_unique<FindCommandRequest>(nss);
     findCommand->setFilter(BSON("foo" << BSON("$gte" << 0)));
     auto cq = uassertStatusOK(CanonicalQuery::canonicalize(opCtx(), std::move(findCommand)));
-    auto exec = uassertStatusOK(
-        getExecutor(opCtx(), &coll, std::move(cq), PlanYieldPolicy::YieldPolicy::NO_YIELD, 0));
+    auto exec = uassertStatusOK(getExecutor(opCtx(),
+                                            &coll,
+                                            std::move(cq),
+                                            nullptr /* extractAndAttachPipelineStages */,
+                                            PlanYieldPolicy::YieldPolicy::NO_YIELD,
+                                            0));
 
     auto execImpl = dynamic_cast<PlanExecutorImpl*>(exec.get());
     ASSERT(execImpl);
