@@ -170,7 +170,7 @@ public:
 
     ApplierState getApplierState() override;
 
-    void signalDrainComplete(OperationContext*, long long) override;
+    void signalDrainComplete(OperationContext*, long long) noexcept override;
 
     void signalUpstreamUpdater() override;
 
@@ -182,6 +182,30 @@ public:
 
     repl::ReplSetConfig getConfig() const override;
 
+    ConnectionString getConfigConnectionString() const override;
+
+    Milliseconds getConfigElectionTimeoutPeriod() const override;
+
+    std::vector<repl::MemberConfig> getConfigVotingMembers() const override;
+
+    std::int64_t getConfigTerm() const override;
+
+    std::int64_t getConfigVersion() const override;
+
+    repl::ConfigVersionAndTerm getConfigVersionAndTerm() const override;
+
+    int getConfigNumMembers() const override;
+
+    Milliseconds getConfigHeartbeatTimeoutPeriodMillis() const override;
+
+    BSONObj getConfigBSON() const override;
+
+    const repl::MemberConfig* findConfigMemberByHostAndPort(const HostAndPort& hap) const override;
+
+    bool isConfigLocalHostAllowed() const override;
+
+    Milliseconds getConfigHeartbeatInterval() const override;
+
     void processReplSetGetConfig(BSONObjBuilder*,
                                  bool commitmentStatus = false,
                                  bool includeNewlyAdded = false) override;
@@ -274,16 +298,14 @@ public:
 
     bool setContainsArbiter() const override;
 
-    bool replSetContainsNewlyAddedMembers() const override;
-
     void attemptToAdvanceStableTimestamp() override;
 
     void finishRecoveryIfEligible(OperationContext* opCtx) override;
 
     void updateAndLogStateTransitionMetrics(
-        const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
-        const size_t numOpsKilled,
-        const size_t numOpsRunning) const override;
+        ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
+        size_t numOpsKilled,
+        size_t numOpsRunning) const override;
 
     TopologyVersion getTopologyVersion() const override;
 
