@@ -31,7 +31,8 @@
 
 #include "mongo/executor/connection_pool_test_fixture.h"
 
-#include "mongo/stdx/memory.h"
+#include <memory>
+
 
 namespace mongo {
 namespace executor {
@@ -180,7 +181,7 @@ void ConnectionImpl::cancelTimeout() {
     _timer.cancelTimeout();
 }
 
-void ConnectionImpl::setup(Milliseconds timeout, SetupCallback cb) {
+void ConnectionImpl::setup(Milliseconds timeout, SetupCallback cb, std::string) {
     _setupCallback = std::move(cb);
 
     _timer.setTimeout(timeout, [this] {
@@ -222,7 +223,7 @@ std::shared_ptr<ConnectionPool::ConnectionInterface> PoolImpl::makeConnection(
 }
 
 std::shared_ptr<ConnectionPool::TimerInterface> PoolImpl::makeTimer() {
-    return stdx::make_unique<TimerImpl>(this);
+    return std::make_unique<TimerImpl>(this);
 }
 
 const std::shared_ptr<OutOfLineExecutor>& PoolImpl::getExecutor() {
