@@ -171,7 +171,7 @@ public:
     explicit TLConnectionSetupHook(executor::NetworkConnectionHook* hookToWrap, bool x509AuthOnly)
         : _wrappedHook(hookToWrap), _x509AuthOnly(x509AuthOnly) {}
 
-    BSONObj augmentIsMasterRequest(BSONObj cmdObj) override {
+    BSONObj augmentIsMasterRequest(const HostAndPort& remoteHost, BSONObj cmdObj) override {
         BSONObjBuilder bob(std::move(cmdObj));
         bob.append("hangUpOnStepDown", false);
         if (internalSecurity.user) {
@@ -181,7 +181,7 @@ public:
         if (_x509AuthOnly) {
             _speculativeAuthType = auth::SpeculativeAuthType::kAuthenticate;
         } else {
-            _speculativeAuthType = auth::speculateInternalAuth(&bob, &_session);
+            _speculativeAuthType = auth::speculateInternalAuth(remoteHost, &bob, &_session);
         }
 
         return bob.obj();
@@ -292,7 +292,7 @@ private:
 
 }  // namespace
 
-void TLConnection::setup(Milliseconds timeout, SetupCallback cb) {
+void TLConnection::setup(Milliseconds timeout, SetupCallback cb, std::string instanceName) {
     auto anchor = shared_from_this();
 
     auto pf = makePromiseFuture<void>();
@@ -333,9 +333,10 @@ void TLConnection::setup(Milliseconds timeout, SetupCallback cb) {
         .onError([](StatusWith<AsyncDBClient::Handle> swc) -> StatusWith<AsyncDBClient::Handle> {
             return Status(ErrorCodes::HostUnreachable, swc.getStatus().reason());
         })
-        .then([this, isMasterHook](AsyncDBClient::Handle client) {
+        .then([this, isMasterHook, instanceName = std::move(instanceName)](
+                  AsyncDBClient::Handle client) {
             _client = std::move(client);
-            return _client->initWireVersion("NetworkInterfaceTL", isMasterHook.get());
+            return _client->initWireVersion(instanceName, isMasterHook.get());
         })
         .then([this, isMasterHook]() -> Future<bool> {
             if (_skipAuth) {
