@@ -45,7 +45,6 @@ using ReshardingFields = TypeCollectionReshardingFields;
  *      "_id" : "foo.bar",
  *      "lastmodEpoch" : ObjectId("58b6fd76132358839e409e47"),
  *      "lastmod" : ISODate("1970-02-19T17:02:47.296Z"),
- *      "dropped" : false,
  *      "key" : {
  *          "_id" : 1
  *      },
@@ -84,14 +83,19 @@ public:
     static constexpr auto kKeyPatternFieldName = kPre50CompatibleKeyPatternFieldName;
     static constexpr auto kUuidFieldName = kPre50CompatibleUuidFieldName;
     static constexpr auto kAllowMigrationsFieldName = kPre50CompatibleAllowMigrationsFieldName;
+
+    using CollectionTypeBase::kMaxChunkSizeBytesFieldName;
+    using CollectionTypeBase::kNoAutoSplitFieldName;
     using CollectionTypeBase::kNssFieldName;
     using CollectionTypeBase::kReshardingFieldsFieldName;
+    using CollectionTypeBase::kSupportingLongNameFieldName;
     using CollectionTypeBase::kTimeseriesFieldsFieldName;
     using CollectionTypeBase::kTimestampFieldName;
     using CollectionTypeBase::kUniqueFieldName;
     using CollectionTypeBase::kUpdatedAtFieldName;
 
     // Make getters and setters accessible.
+    using CollectionTypeBase::getMaxChunkSizeBytes;
     using CollectionTypeBase::getNss;
     using CollectionTypeBase::getReshardingFields;
     using CollectionTypeBase::getTimeseriesFields;
@@ -109,13 +113,8 @@ public:
     // Name of the collections collection in the config server.
     static const NamespaceString ConfigNS;
 
-    CollectionType(NamespaceString nss, OID epoch, Date_t updatedAt, UUID uuid);
-
-    CollectionType(NamespaceString nss,
-                   OID epoch,
-                   boost::optional<Timestamp> creationTime,
-                   Date_t updatedAt,
-                   UUID uuid);
+    CollectionType(
+        NamespaceString nss, OID epoch, Timestamp creationTime, Date_t updatedAt, UUID uuid);
 
     explicit CollectionType(const BSONObj& obj);
 
@@ -133,10 +132,6 @@ public:
     }
     void setUuid(UUID uuid);
 
-    bool getDropped() const {
-        return getPre50CompatibleDropped().get_value_or(false);
-    }
-
     const KeyPattern& getKeyPattern() const {
         return *getPre50CompatibleKeyPattern();
     }
@@ -147,6 +142,12 @@ public:
     }
     void setDefaultCollation(const BSONObj& defaultCollation);
 
+    void setMaxChunkSizeBytes(int64_t value);
+
+    bool getAllowAutoSplit() const {
+        return !getNoAutoSplit().get_value_or(false);
+    }
+
     bool getAllowBalance() const {
         return !getNoBalance();
     }
@@ -166,6 +167,10 @@ public:
     bool getPermitMigrations() const {
         return CollectionTypeBase::getPermitMigrations().get_value_or(true);
     }
+
+    SupportingLongNameStatusEnum getSupportingLongName() const;
+
+    void setSupportingLongName(SupportingLongNameStatusEnum value);
 };
 
 }  // namespace mongo
