@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -48,19 +47,20 @@ TEST(DatabaseType, Empty) {
 
 TEST(DatabaseType, Basic) {
     UUID uuid = UUID::gen();
+    Timestamp timestamp = Timestamp(1, 1);
     StatusWith<DatabaseType> status = DatabaseType::fromBSON(
         BSON(DatabaseType::name("mydb")
-             << DatabaseType::primary("shard")
-             << DatabaseType::sharded(true)
-             << DatabaseType::version(BSON("uuid" << uuid << "lastMod" << 0))));
+             << DatabaseType::primary("shard") << DatabaseType::sharded(true)
+             << DatabaseType::version(
+                    BSON("uuid" << uuid << "lastMod" << 0 << "timestamp" << timestamp))));
     ASSERT_TRUE(status.isOK());
 
     DatabaseType db = status.getValue();
     ASSERT_EQUALS(db.getName(), "mydb");
     ASSERT_EQUALS(db.getPrimary(), "shard");
     ASSERT_TRUE(db.getSharded());
-    ASSERT_EQUALS(db.getVersion()->getUuid(), uuid);
-    ASSERT_EQUALS(db.getVersion()->getLastMod(), 0);
+    ASSERT_EQUALS(db.getVersion().getUuid(), uuid);
+    ASSERT_EQUALS(db.getVersion().getLastMod(), 0);
 }
 
 TEST(DatabaseType, BadType) {
