@@ -308,8 +308,7 @@ ChunkMap::_overlappingBounds(const BSONObj& min, const BSONObj& max, bool isMaxI
     return {itMin, itMax};
 }
 
-ShardVersionTargetingInfo::ShardVersionTargetingInfo(const OID& epoch,
-                                                     const boost::optional<Timestamp>& timestamp)
+ShardVersionTargetingInfo::ShardVersionTargetingInfo(const OID& epoch, const Timestamp& timestamp)
     : shardVersion(0, 0, epoch, timestamp) {}
 
 RoutingTableHistory::RoutingTableHistory(
@@ -320,6 +319,7 @@ RoutingTableHistory::RoutingTableHistory(
     bool unique,
     boost::optional<TypeCollectionTimeseriesFields> timeseriesFields,
     boost::optional<TypeCollectionReshardingFields> reshardingFields,
+    boost::optional<uint64_t> maxChunkSizeBytes,
     bool allowMigrations,
     ChunkMap chunkMap)
     : _nss(std::move(nss)),
@@ -329,6 +329,7 @@ RoutingTableHistory::RoutingTableHistory(
       _unique(unique),
       _timeseriesFields(std::move(timeseriesFields)),
       _reshardingFields(std::move(reshardingFields)),
+      _maxChunkSizeBytes(maxChunkSizeBytes),
       _allowMigrations(allowMigrations),
       _chunkMap(std::move(chunkMap)),
       _shardVersions(_chunkMap.constructShardVersionMap()) {}
@@ -599,9 +600,10 @@ IndexBounds ChunkManager::getIndexBoundsForQuery(const BSONObj& key,
                           nullptr /* projExec */);
     plannerParams.indices.push_back(std::move(indexEntry));
 
-    auto plannerResult = QueryPlanner::plan(canonicalQuery, plannerParams);
-    if (plannerResult.getStatus().code() != ErrorCodes::NoQueryExecutionPlans) {
-        auto solutions = uassertStatusOK(std::move(plannerResult));
+    auto statusWithMultiPlanSolns =
+        QueryPlanner::planForMultiPlanner(canonicalQuery, plannerParams);
+    if (statusWithMultiPlanSolns.getStatus().code() != ErrorCodes::NoQueryExecutionPlans) {
+        auto solutions = uassertStatusOK(std::move(statusWithMultiPlanSolns));
 
         // Pick any solution that has non-trivial IndexBounds. bounds.size() == 0 represents a
         // trivial IndexBounds where none of the fields' values are bounded.
@@ -691,6 +693,20 @@ bool ChunkManager::allowMigrations() const {
     return _rt->optRt->allowMigrations();
 }
 
+bool ChunkManager::allowAutoSplit() const {
+    const auto maxChunkSize = maxChunkSizeBytes();
+    if (!maxChunkSize)
+        return true;
+
+    return *maxChunkSize != 0;
+}
+
+boost::optional<uint64_t> ChunkManager::maxChunkSizeBytes() const {
+    if (!_rt->optRt)
+        return boost::none;
+    return _rt->optRt->maxChunkSizeBytes();
+}
+
 std::string ChunkManager::toString() const {
     return _rt->optRt ? _rt->optRt->toString() : "UNSHARDED";
 }
@@ -754,28 +770,33 @@ RoutingTableHistory RoutingTableHistory::makeNew(
     std::unique_ptr<CollatorInterface> defaultCollator,
     bool unique,
     OID epoch,
-    const boost::optional<Timestamp>& timestamp,
+    const Timestamp& timestamp,
     boost::optional<TypeCollectionTimeseriesFields> timeseriesFields,
     boost::optional<TypeCollectionReshardingFields> reshardingFields,
+    boost::optional<uint64_t> maxChunkSizeBytes,
     bool allowMigrations,
     const std::vector<ChunkType>& chunks) {
+
+    auto changedChunkInfos = flatten(chunks);
     return RoutingTableHistory(std::move(nss),
                                std::move(uuid),
                                std::move(shardKeyPattern),
                                std::move(defaultCollator),
                                std::move(unique),
                                std::move(timeseriesFields),
-                               boost::none,
+                               std::move(reshardingFields),
+                               maxChunkSizeBytes,
                                allowMigrations,
-                               ChunkMap{epoch, timestamp})
-        .makeUpdated(std::move(reshardingFields), allowMigrations, chunks);
+                               ChunkMap{epoch, timestamp}.createMerged(changedChunkInfos));
 }
 
 // Note that any new parameters added to RoutingTableHistory::makeUpdated() must also be added to
 // ShardServerCatalogCacheLoader::_getLoaderMetadata() and copied into the persisted metadata when
 // it may overlap with the enqueued metadata.
 RoutingTableHistory RoutingTableHistory::makeUpdated(
+    boost::optional<TypeCollectionTimeseriesFields> timeseriesFields,
     boost::optional<TypeCollectionReshardingFields> reshardingFields,
+    boost::optional<uint64_t> maxChunkSizeBytes,
     bool allowMigrations,
     const std::vector<ChunkType>& changedChunks) const {
     auto changedChunkInfos = flatten(changedChunks);
@@ -789,43 +810,13 @@ RoutingTableHistory RoutingTableHistory::makeUpdated(
                                getShardKeyPattern().getKeyPattern(),
                                CollatorInterface::cloneCollator(getDefaultCollator()),
                                isUnique(),
-                               _timeseriesFields,
+                               std::move(timeseriesFields),
                                std::move(reshardingFields),
+                               maxChunkSizeBytes,
                                allowMigrations,
                                std::move(chunkMap));
 }
 
-RoutingTableHistory RoutingTableHistory::makeUpdatedReplacingTimestamp(
-    const boost::optional<Timestamp>& timestamp) const {
-    invariant(getVersion().getTimestamp().is_initialized() != timestamp.is_initialized());
-
-    ChunkMap newMap(getVersion().epoch(), timestamp, _chunkMap.size());
-    _chunkMap.forEach([&](const std::shared_ptr<ChunkInfo>& chunkInfo) {
-        const ChunkVersion oldVersion = chunkInfo->getLastmod();
-        newMap.appendChunk(std::make_shared<ChunkInfo>(chunkInfo->getRange(),
-                                                       chunkInfo->getMaxKeyString(),
-                                                       chunkInfo->getShardId(),
-                                                       ChunkVersion(oldVersion.majorVersion(),
-                                                                    oldVersion.minorVersion(),
-                                                                    oldVersion.epoch(),
-                                                                    timestamp),
-                                                       chunkInfo->getHistory(),
-                                                       chunkInfo->isJumbo(),
-                                                       chunkInfo->getWritesTracker()));
-        return true;
-    });
-
-    return RoutingTableHistory(_nss,
-                               _uuid,
-                               getShardKeyPattern().getKeyPattern(),
-                               CollatorInterface::cloneCollator(getDefaultCollator()),
-                               _unique,
-                               _timeseriesFields,
-                               _reshardingFields,
-                               _allowMigrations,
-                               std::move(newMap));
-}
-
 AtomicWord<uint64_t> ComparableChunkVersion::_epochDisambiguatingSequenceNumSource{1ULL};
 AtomicWord<uint64_t> ComparableChunkVersion::_forcedRefreshSequenceNumSource{1ULL};
 
@@ -898,29 +889,17 @@ bool ComparableChunkVersion::operator<(const ComparableChunkVersion& other) cons
                                                     // _epochDisambiguatingSequenceNum to see which
                                                     // one is more recent.
 
-    const boost::optional<Timestamp> timestamp = _chunkVersion->getTimestamp();
-    const boost::optional<Timestamp> otherTimestamp = other._chunkVersion->getTimestamp();
-    if (timestamp && otherTimestamp) {
-        if (_chunkVersion->isSet() && other._chunkVersion->isSet()) {
-            if (*timestamp == *otherTimestamp)
-                return _chunkVersion->majorVersion() < other._chunkVersion->majorVersion() ||
-                    (_chunkVersion->majorVersion() == other._chunkVersion->majorVersion() &&
-                     _chunkVersion->minorVersion() < other._chunkVersion->minorVersion());
-            else
-                return *timestamp < *otherTimestamp;
-        } else if (!_chunkVersion->isSet() && !other._chunkVersion->isSet())
-            return false;  // Both sides are the "no chunks on the shard version"
-    } else if (sameEpoch(other)) {
-        if (_chunkVersion->isSet() && other._chunkVersion->isSet())
+    if (_chunkVersion->getTimestamp() == other._chunkVersion->getTimestamp()) {
+        if (!_chunkVersion->isSet() && !other._chunkVersion->isSet()) {
+            return false;
+        } else if (_chunkVersion->isSet() && other._chunkVersion->isSet()) {
             return _chunkVersion->majorVersion() < other._chunkVersion->majorVersion() ||
                 (_chunkVersion->majorVersion() == other._chunkVersion->majorVersion() &&
                  _chunkVersion->minorVersion() < other._chunkVersion->minorVersion());
-        else if (!_chunkVersion->isSet() && !other._chunkVersion->isSet())
-            return false;  // Both sides are the "no chunks on the shard version"
+        }
+    } else if (_chunkVersion->isSet() && other._chunkVersion->isSet()) {
+        return _chunkVersion->getTimestamp() < other._chunkVersion->getTimestamp();
     }
-
-    // If the epochs are different, or if they match, but one of the versions is the "no chunks"
-    // version, use the _epochDisambiguatingSequenceNum to disambiguate
     return _epochDisambiguatingSequenceNum < other._epochDisambiguatingSequenceNum;
 }
 
