@@ -46,9 +46,7 @@
 #include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/catalog/type_shard.h"
 #include "mongo/s/grid.h"
-#include "mongo/util/future_util.h"
 #include "mongo/util/str.h"
-#include "mongo/util/testing_proctor.h"
 
 namespace mongo {
 
@@ -56,17 +54,6 @@ namespace {
 
 const Seconds kRefreshPeriod(30);
 
-/**
- * Whether or not the actual topologyTime should be used.  When this is false, the
- * topologyTime part of the cache's Time will stay fixed and not advance.
- */
-bool useActualTopologyTime() {
-    return serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-        serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-            ServerGlobalParams::FeatureCompatibility::Version::kVersion47);
-}
-const Backoff kExponentialBackoff(Seconds(1), Milliseconds::max());
-
 }  // namespace
 
 using CallbackArgs = executor::TaskExecutor::CallbackArgs;
@@ -139,29 +126,28 @@ ShardRegistry::Cache::LookupResult ShardRegistry::_lookup(OperationContext* opCt
     // Check if we need to refresh from the configsvrs.  If so, then do that and get the results,
     // otherwise (this is a lookup only to incorporate updated connection strings from the RSM),
     // then get the equivalent values from the previously cached data.
-    auto [returnData, returnTopologyTime, returnForceReloadIncrement, removedShards] =
-        [&]() -> std::tuple<ShardRegistryData, Timestamp, Increment, ShardRegistryData::ShardMap> {
+    auto [returnData,
+          returnTopologyTime,
+          returnForceReloadIncrement,
+          removedShards,
+          fetchedFromConfigServers] = [&]()
+        -> std::tuple<ShardRegistryData, Timestamp, Increment, ShardRegistryData::ShardMap, bool> {
         if (timeInStore.topologyTime > cachedData.getTime().topologyTime ||
             timeInStore.forceReloadIncrement > cachedData.getTime().forceReloadIncrement) {
             auto [reloadedData, maxTopologyTime] =
                 ShardRegistryData::createFromCatalogClient(opCtx, _shardFactory.get());
-            if (!useActualTopologyTime()) {
-                // If not using the actual topology time, then just use the topologyTime currently
-                // in the cache, instead of the maximum topologyTime value from config.shards.  This
-                // is necessary during upgrade/downgrade when topologyTime might not be gossiped by
-                // all nodes (and so isn't being used).
-                maxTopologyTime = cachedData.getTime().topologyTime;
-            }
 
             auto [mergedData, removedShards] =
                 ShardRegistryData::mergeExisting(*cachedData, reloadedData);
 
-            return {mergedData, maxTopologyTime, timeInStore.forceReloadIncrement, removedShards};
+            return {
+                mergedData, maxTopologyTime, timeInStore.forceReloadIncrement, removedShards, true};
         } else {
             return {*cachedData,
                     cachedData.getTime().topologyTime,
                     cachedData.getTime().forceReloadIncrement,
-                    {}};
+                    {},
+                    false};
         }
     }();
 
@@ -196,6 +182,11 @@ ShardRegistry::Cache::LookupResult ShardRegistry::_lookup(OperationContext* opCt
         }
     }
 
+    // The registry is "up" once there has been a successful lookup from the config servers.
+    if (fetchedFromConfigServers) {
+        _isUp.store(true);
+    }
+
     Time returnTime{returnTopologyTime, rsmIncrementForConnStrings, returnForceReloadIncrement};
     LOGV2_DEBUG(4620251,
                 2,
@@ -216,33 +207,26 @@ void ShardRegistry::startupPeriodicReloader(OperationContext* opCtx) {
     // construct task executor
     auto net = executor::makeNetworkInterface("ShardRegistryUpdater", nullptr, std::move(hookList));
     auto netPtr = net.get();
-    _executor = std::make_shared<executor::ThreadPoolTaskExecutor>(
+    _executor = std::make_unique<executor::ThreadPoolTaskExecutor>(
         std::make_unique<executor::NetworkInterfaceThreadPool>(netPtr), std::move(net));
     LOGV2_DEBUG(22724, 1, "Starting up task executor for periodic reloading of ShardRegistry");
     _executor->startup();
 
-    AsyncTry([this] {
-        LOGV2_DEBUG(22726, 1, "Reloading shardRegistry");
-        return _reloadAsyncNoRetry();
-    })
-        .until([](auto&& sw) {
-            if (!sw.isOK()) {
-                LOGV2(22727,
-                      "Error running periodic reload of shard registry",
-                      "error"_attr = redact(sw.getStatus()),
-                      "shardRegistryReloadInterval"_attr = kRefreshPeriod);
-            }
-            // Continue until the _executor will shutdown
-            return false;
-        })
-        .withDelayBetweenIterations(kRefreshPeriod)  // This call is optional.
-        .on(_executor, CancellationToken::uncancelable())
-        .getAsync([](auto&& sw) {
-            LOGV2_DEBUG(22725,
-                        1,
-                        "Exiting periodic shard registry reloader",
-                        "reason"_attr = redact(sw.getStatus()));
-        });
+    auto status =
+        _executor->scheduleWork([this](const CallbackArgs& cbArgs) { _periodicReload(cbArgs); });
+
+    if (status.getStatus() == ErrorCodes::ShutdownInProgress) {
+        LOGV2_DEBUG(
+            22725, 1, "Can't schedule Shard Registry reload. Executor shutdown in progress");
+        return;
+    }
+
+    if (!status.isOK()) {
+        LOGV2_FATAL(40252,
+                    "Error scheduling shard registry reload caused by {error}",
+                    "Error scheduling shard registry reload",
+                    "error"_attr = redact(status.getStatus()));
+    }
 }
 
 void ShardRegistry::shutdownPeriodicReloader() {
@@ -255,17 +239,65 @@ void ShardRegistry::shutdownPeriodicReloader() {
 }
 
 void ShardRegistry::shutdown() {
+    shutdownPeriodicReloader();
+
     if (!_isShutdown.load()) {
         LOGV2_DEBUG(4620235, 1, "Shutting down shard registry");
         _threadPool.shutdown();
-
-        shutdownPeriodicReloader();
-
         _threadPool.join();
         _isShutdown.store(true);
     }
 }
 
+void ShardRegistry::_periodicReload(const CallbackArgs& cbArgs) {
+    LOGV2_DEBUG(22726, 1, "Reloading shardRegistry");
+    if (!cbArgs.status.isOK()) {
+        LOGV2_WARNING(22734,
+                      "Error reloading shard registry caused by {error}",
+                      "Error reloading shard registry",
+                      "error"_attr = redact(cbArgs.status));
+        return;
+    }
+
+    ThreadClient tc("shard-registry-reload", getGlobalServiceContext());
+
+    auto opCtx = tc->makeOperationContext();
+
+    auto refreshPeriod = kRefreshPeriod;
+
+    try {
+        reload(opCtx.get());
+    } catch (const DBException& e) {
+        if (e.code() == ErrorCodes::ReadConcernMajorityNotAvailableYet) {
+            refreshPeriod = Seconds(1);
+        }
+        LOGV2(22727,
+              "Error running periodic reload of shard registry caused by {error}; will retry after "
+              "{shardRegistryReloadInterval}",
+              "Error running periodic reload of shard registry",
+              "error"_attr = redact(e),
+              "shardRegistryReloadInterval"_attr = refreshPeriod);
+    }
+
+    // reschedule itself
+    auto status =
+        _executor->scheduleWorkAt(_executor->now() + refreshPeriod,
+                                  [this](const CallbackArgs& cbArgs) { _periodicReload(cbArgs); });
+
+    if (status.getStatus() == ErrorCodes::ShutdownInProgress) {
+        LOGV2_DEBUG(
+            22728, 1, "Error scheduling shard registry reload. Executor shutdown in progress");
+        return;
+    }
+
+    if (!status.isOK()) {
+        LOGV2_FATAL(40253,
+                    "Error scheduling shard registry reload caused by {error}",
+                    "Error scheduling shard registry reload",
+                    "error"_attr = redact(status.getStatus()));
+    }
+}
+
 ConnectionString ShardRegistry::getConfigServerConnectionString() const {
     return getConfigShard()->getConnString();
 }
@@ -300,49 +332,6 @@ StatusWith<std::shared_ptr<Shard>> ShardRegistry::getShard(OperationContext* opC
     return {ErrorCodes::ShardNotFound, str::stream() << "Shard " << shardId << " not found"};
 }
 
-SemiFuture<std::shared_ptr<Shard>> ShardRegistry::getShard(ExecutorPtr executor,
-                                                           const ShardId& shardId) noexcept {
-
-    // Fetch the shard registry data associated to the latest known topology time
-    return _getDataAsync()
-        .thenRunOn(executor)
-        .then([this, executor, shardId](auto&& cachedData) {
-            // First check if this is a non config shard lookup
-            if (auto shard = cachedData->findShard(shardId)) {
-                return SemiFuture<std::shared_ptr<Shard>>::makeReady(std::move(shard));
-            }
-
-            // then check if this is a config shard (this call is blocking in any case)
-            {
-                stdx::lock_guard<Latch> lk(_mutex);
-                if (auto shard = _configShardData.findShard(shardId)) {
-                    return SemiFuture<std::shared_ptr<Shard>>::makeReady(std::move(shard));
-                }
-            }
-
-            // If the shard was not found, force reload the shard regitry data and try again.
-            //
-            // This is to cover the following scenario:
-            // 1. Primary of the replicaset fetch the list of shards and store it on disk
-            // 2. Primary crash before the latest VectorClock topology time is majority written to
-            //    disk
-            // 3. A new primary with a stale ShardRegistry is elected and read the set of shards
-            //    from disk and calls ShardRegistry::getShard
-
-            return _reloadAsync()
-                .thenRunOn(executor)
-                .then([this, executor, shardId](auto&& cachedData) -> std::shared_ptr<Shard> {
-                    auto shard = cachedData->findShard(shardId);
-                    uassert(ErrorCodes::ShardNotFound,
-                            str::stream() << "Shard " << shardId << " not found",
-                            shard);
-                    return shard;
-                })
-                .semi();
-        })
-        .semi();
-}
-
 std::vector<ShardId> ShardRegistry::getAllShardIds(OperationContext* opCtx) {
     auto shardIds = _getData(opCtx)->getAllShardIds();
     if (shardIds.empty()) {
@@ -428,18 +417,8 @@ std::unique_ptr<Shard> ShardRegistry::createConnection(const ConnectionString& c
     return _shardFactory->createUniqueShard(ShardId("<unnamed>"), connStr);
 }
 
-bool ShardRegistry::isUp() {
-    if (_isUp.load())
-        return true;
-
-    // Before the first lookup is completed, the latest cached value is either empty or it is
-    // associated to the default constructed time
-    const auto latestCached = _cache->peekLatestCached(_kSingleton);
-    if (latestCached && latestCached.getTime() != Time()) {
-        _isUp.store(true);
-        return true;
-    }
-    return false;
+bool ShardRegistry::isUp() const {
+    return _isUp.load();
 }
 
 void ShardRegistry::toBSON(BSONObjBuilder* result) const {
@@ -458,33 +437,12 @@ void ShardRegistry::toBSON(BSONObjBuilder* result) const {
 }
 
 void ShardRegistry::reload(OperationContext* opCtx) {
-    _reloadAsync().get(opCtx);
-}
-
-SharedSemiFuture<ShardRegistry::Cache::ValueHandle> ShardRegistry::_reloadAsync() {
-    if (MONGO_unlikely(TestingProctor::instance().isEnabled())) {
-        // TODO SERVER-62152 investigate hang on reload in unit tests
-        // Some unit tests don't support running the reload's AsyncTry on the fixed executor.
-        return _reloadAsyncNoRetry();
-    } else {
-        return AsyncTry([=]() mutable { return _reloadAsyncNoRetry(); })
-            .until([](auto sw) mutable {
-                return sw.getStatus() != ErrorCodes::ReadConcernMajorityNotAvailableYet;
-            })
-            .withBackoffBetweenIterations(kExponentialBackoff)
-            .on(Grid::get(getGlobalServiceContext())->getExecutorPool()->getFixedExecutor(),
-                CancellationToken::uncancelable())
-            .share();
-    }
-}
-
-SharedSemiFuture<ShardRegistry::Cache::ValueHandle> ShardRegistry::_reloadAsyncNoRetry() {
     // Make the next acquire do a lookup.
     auto value = _forceReloadIncrement.addAndFetch(1);
     LOGV2_DEBUG(4620253, 2, "Forcing ShardRegistry reload", "newForceReloadIncrement"_attr = value);
 
     // Force it to actually happen now.
-    return _getDataAsync();
+    _getData(opCtx);
 }
 
 void ShardRegistry::clearEntries() {
@@ -553,12 +511,8 @@ SharedSemiFuture<ShardRegistry::Cache::ValueHandle> ShardRegistry::_getDataAsync
 
     // Update the time the cache should be aiming for.
     auto now = VectorClock::get(_service)->getTime();
-    // The topologyTime should be advanced to either the actual topologyTime (if it is being
-    // gossiped), or else the previously cached topologyTime value (so that this part of the cache's
-    // time doesn't advance, if topologyTime isn't being gossiped).
-    Timestamp topologyTime = useActualTopologyTime()
-        ? now.topologyTime().asTimestamp()
-        : _cache->peekLatestCached(_kSingleton).getTime().topologyTime;
+    // The topologyTime should be advanced to the gossiped topologyTime.
+    Timestamp topologyTime = now.topologyTime().asTimestamp();
     _cache->advanceTimeInStore(
         _kSingleton, Time(topologyTime, _rsmIncrement.load(), _forceReloadIncrement.load()));
 
