@@ -64,6 +64,18 @@ void appendWriteConcernErrorToCmdResponse(const ShardId& shardID,
  */
 std::unique_ptr<WriteConcernErrorDetail> getWriteConcernErrorDetailFromBSONObj(const BSONObj& obj);
 
+/**
+ * Makes an expression context suitable for canonicalization of queries that contain let parameters
+ * and runtimeConstants on mongos.
+ */
+boost::intrusive_ptr<ExpressionContext> makeExpressionContextWithDefaultsForTargeter(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const BSONObj& collation,
+    const boost::optional<ExplainOptions::Verbosity>& verbosity,
+    const boost::optional<BSONObj>& letParameters,
+    const boost::optional<LegacyRuntimeConstants>& runtimeConstants);
+
 /**
  * Consults the routing info to build requests for:
  * 1. If sharded, shards that own chunks for the namespace, or
@@ -75,7 +87,7 @@ std::unique_ptr<WriteConcernErrorDetail> getWriteConcernErrorDetailFromBSONObj(c
 std::vector<AsyncRequestsSender::Request> buildVersionedRequestsForTargetedShards(
     OperationContext* opCtx,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const std::set<ShardId>& shardsToSkip,
     const BSONObj& cmdObj,
     const BSONObj& query,
@@ -120,11 +132,6 @@ BSONObj appendDbVersionIfPresent(BSONObj cmdObj, DatabaseVersion dbVersion);
  */
 BSONObj appendShardVersion(BSONObj cmdObj, ChunkVersion version);
 
-/**
- * Returns a copy of 'cmdObj' with 'allowImplicitCollectionCreation' appended.
- */
-BSONObj appendAllowImplicitCreate(BSONObj cmdObj, bool allow);
-
 /**
  * Returns a copy of 'cmdObj' with the read/writeConcern from the OpCtx appended, unless the
  * cmdObj explicitly specifies read/writeConcern.
@@ -136,12 +143,15 @@ BSONObj applyReadWriteConcern(OperationContext* opCtx,
 
 /**
  * Convenience versions of applyReadWriteConcern() for calling from within
- * CommandInvocation or BasicCommand.
+ * CommandInvocation, BasicCommand or BasicCommandWithRequestParser.
  */
 BSONObj applyReadWriteConcern(OperationContext* opCtx,
                               CommandInvocation* invocation,
                               const BSONObj& cmdObj);
-BSONObj applyReadWriteConcern(OperationContext* opCtx, BasicCommand* cmd, const BSONObj& cmdObj);
+
+BSONObj applyReadWriteConcern(OperationContext* opCtx,
+                              BasicCommandWithReplyBuilderInterface* cmd,
+                              const BSONObj& cmdObj);
 
 /**
  * Returns a copy of 'cmdObj' with the writeConcern removed.
@@ -180,7 +190,7 @@ std::vector<AsyncRequestsSender::Response> scatterGatherVersionedTargetByRouting
     OperationContext* opCtx,
     StringData dbName,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy,
@@ -203,7 +213,7 @@ scatterGatherVersionedTargetByRoutingTableNoThrowOnStaleShardVersionErrors(
     OperationContext* opCtx,
     StringData dbName,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const std::set<ShardId>& shardsToSkip,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
@@ -232,7 +242,7 @@ std::vector<AsyncRequestsSender::Response> scatterGatherOnlyVersionIfUnsharded(
 
 /**
  * Utility for dispatching commands against the primary of a database and attaching the appropriate
- * database version. Does not retry on StaleDbVersion.
+ * database version. Also attaches UNSHARDED to the command. Does not retry on stale version.
  */
 AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
     OperationContext* opCtx,
@@ -263,7 +273,7 @@ AsyncRequestsSender::Response executeRawCommandAgainstDatabasePrimary(
 AsyncRequestsSender::Response executeCommandAgainstShardWithMinKeyChunk(
     OperationContext* opCtx,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const BSONObj& cmdObj,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy);
@@ -305,14 +315,6 @@ BSONObj extractQuery(const BSONObj& cmdObj);
  */
 BSONObj extractCollation(const BSONObj& cmdObj);
 
-/**
- * Utility function to compute a single error code from a vector of command results.
- *
- * @return If there is an error code common to all of the error results, returns that error
- *          code; otherwise, returns 0.
- */
-int getUniqueCodeFromCommandResults(const std::vector<Strategy::CommandResult>& results);
-
 /**
  * Utility function to return an empty result set from a command.
  */
@@ -321,18 +323,12 @@ bool appendEmptyResultSet(OperationContext* opCtx,
                           Status status,
                           const std::string& ns);
 
-/**
- * If the specified database exists already, loads it in the cache (if not already there).
- * Otherwise, if it does not exist, this call will implicitly create it as non-sharded.
- */
-void createShardDatabase(OperationContext* opCtx, StringData dbName);
-
 /**
  * Returns the shards that would be targeted for the given query according to the given routing
  * info.
  */
-std::set<ShardId> getTargetedShardsForQuery(OperationContext* opCtx,
-                                            const CachedCollectionRoutingInfo& routingInfo,
+std::set<ShardId> getTargetedShardsForQuery(boost::intrusive_ptr<ExpressionContext> expCtx,
+                                            const ChunkManager& cm,
                                             const BSONObj& query,
                                             const BSONObj& collation);
 
@@ -343,7 +339,7 @@ std::set<ShardId> getTargetedShardsForQuery(OperationContext* opCtx,
 std::vector<std::pair<ShardId, BSONObj>> getVersionedRequestsForTargetedShards(
     OperationContext* opCtx,
     const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
+    const ChunkManager& cm,
     const BSONObj& cmdObj,
     const BSONObj& query,
     const BSONObj& collation);
@@ -356,8 +352,8 @@ std::vector<std::pair<ShardId, BSONObj>> getVersionedRequestsForTargetedShards(
  *
  * Should be used by all router commands that can be run in a transaction when targeting shards.
  */
-StatusWith<CachedCollectionRoutingInfo> getCollectionRoutingInfoForTxnCmd(
-    OperationContext* opCtx, const NamespaceString& nss);
+StatusWith<ChunkManager> getCollectionRoutingInfoForTxnCmd(OperationContext* opCtx,
+                                                           const NamespaceString& nss);
 
 /**
  * Loads all of the indexes for the given namespace from the appropriate shard. For unsharded
