@@ -45,7 +45,6 @@
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/query/cursor_response.h"
 #include "mongo/db/query/explain_common.h"
-#include "mongo/db/query/getmore_request.h"
 #include "mongo/db/query/map_reduce_output_format.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog_cache.h"
@@ -187,9 +186,8 @@ bool runAggregationMapReduce(OperationContext* opCtx,
                                                                pipelineBuilder,
                                                                cm,
                                                                involvedNamespaces,
-                                                               false,   // hasChangeStream
-                                                               true,    // allowedToPassthrough
-                                                               false);  // perShardCursor
+                                                               false,  // hasChangeStream
+                                                               true);  // allowedToPassthrough
     try {
         switch (targeter.policy) {
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::kPassthrough: {
@@ -211,7 +209,8 @@ bool runAggregationMapReduce(OperationContext* opCtx,
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::
                 kMongosRequired: {
                 // Pipelines generated from mapReduce should never be required to run on mongos.
-                MONGO_UNREACHABLE_TASSERT(31291);
+                uasserted(31291, "Internal error during mapReduce translation");
+                break;
             }
 
             case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::kAnyShard: {
@@ -234,12 +233,6 @@ bool runAggregationMapReduce(OperationContext* opCtx,
                     false));  // hasChangeStream
                 break;
             }
-
-            case cluster_aggregation_planner::AggregationTargeter::TargetingPolicy::
-                kSpecificShardOnly: {
-                // It should not be possible to pass $_passthroughToShard to a map reduce command.
-                MONGO_UNREACHABLE_TASSERT(6273803);
-            }
         }
     } catch (DBException& e) {
         uassert(ErrorCodes::CommandNotSupportedOnView,
