@@ -49,11 +49,11 @@
 #include "mongo/db/error_labels.h"
 #include "mongo/db/initialize_api_parameters.h"
 #include "mongo/db/initialize_operation_session_info.h"
-#include "mongo/db/lasterror.h"
 #include "mongo/db/logical_session_id_helpers.h"
 #include "mongo/db/logical_time_validator.h"
 #include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/not_primary_error_tracker.h"
 #include "mongo/db/operation_time_tracker.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/query/find_common.h"
@@ -81,7 +81,6 @@
 #include "mongo/s/cluster_commands_helpers.h"
 #include "mongo/s/commands/cluster_explain.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/load_balancer_support.h"
 #include "mongo/s/mongos_topology_coordinator.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
 #include "mongo/s/query/cluster_find.h"
@@ -102,7 +101,7 @@ using namespace fmt::literals;
 
 namespace mongo {
 namespace {
-MONGO_FAIL_POINT_DEFINE(hangBeforeCheckingMongosShutdownInterrupt);
+
 const auto kOperationTime = "operationTime"_sd;
 
 /**
@@ -154,14 +153,14 @@ void appendRequiredFieldsToResponse(OperationContext* opCtx, BSONObjBuilder* res
     // Ensure that either both operationTime and $clusterTime are output, or neither.
     if (clusterTimeWasOutput) {
         auto operationTime = OperationTimeTracker::get(opCtx)->getMaxOperationTime();
-        if (VectorClock::isValidComponentTime(operationTime)) {
+        if (operationTime != LogicalTime::kUninitialized) {
             LOGV2_DEBUG(22764,
                         5,
                         "Appending operationTime: {operationTime}",
                         "Appending operationTime",
                         "operationTime"_attr = operationTime.asTimestamp());
             operationTime.appendAsOperationTime(responseBuilder);
-        } else if (VectorClock::isValidComponentTime(clusterTime)) {
+        } else if (clusterTime != LogicalTime::kUninitialized) {
             // If we don't know the actual operation time, use the cluster time instead. This is
             // safe but not optimal because we can always return a later operation time than
             // actual.
@@ -399,9 +398,6 @@ private:
     // invocation and extracting read/write concerns).
     void _parseCommand();
 
-    // updates statistics and applies labels if an error occurs.
-    void _updateStatsAndApplyErrorLabels(const Status& status);
-
     const std::shared_ptr<RequestExecutionContext> _rec;
     const std::shared_ptr<BSONObjBuilder> _errorBuilder;
     const NetworkOp _opType;
@@ -437,6 +433,9 @@ public:
 private:
     Status _setup();
 
+    // Logs and updates statistics if an error occurs.
+    void _tapOnError(const Status& status);
+
     ParseAndRunCommand* const _parc;
 
     boost::optional<RouterOperationContextSession> _routerSession;
@@ -478,34 +477,7 @@ private:
 
     int _tries = 0;
 };
-void ParseAndRunCommand::_updateStatsAndApplyErrorLabels(const Status& status) {
-    auto opCtx = _rec->getOpCtx();
-    const auto command = _rec->getCommand();
 
-    if (command)
-        command->incrementCommandsFailed();
-    LastError::get(opCtx->getClient()).setLastError(status.code(), status.reason());
-    // NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(status.code());
-    // WriteConcern error (wcCode) is set to boost::none because:
-    // 1. TransientTransaction error label handling for commitTransaction command in mongos is
-    //    delegated to the shards. Mongos simply propagates the shard's response up to the client.
-    // 2. For other commands in a transaction, they shouldn't get a writeConcern error so this
-    //    setting doesn't apply.
-
-    if (_osi.has_value()) {
-
-        auto errorLabels = getErrorLabels(opCtx,
-                                          *_osi,
-                                          command->getName(),
-                                          status.code(),
-                                          boost::none,
-                                          false /* isInternalClient */,
-                                          true /* isMongos */);
-
-
-        _errorBuilder->appendElements(errorLabels);
-    }
-}
 void ParseAndRunCommand::_parseCommand() {
     auto opCtx = _rec->getOpCtx();
     const auto& m = _rec->getMessage();
@@ -528,7 +500,8 @@ void ParseAndRunCommand::_parseCommand() {
     _isHello.emplace(command->getName() == "hello"_sd || command->getName() == "isMaster"_sd);
 
     opCtx->setExhaust(OpMsg::isFlagSet(m, OpMsg::kExhaustSupported));
-    const auto session = opCtx->getClient()->session();
+    Client* client = opCtx->getClient();
+    const auto session = client->session();
     if (session) {
         if (!opCtx->isExhaust() || !_isHello.get()) {
             InExhaustHello::get(session.get())->setInExhaust(false, _commandName);
@@ -548,13 +521,19 @@ void ParseAndRunCommand::_parseCommand() {
     uassert(ErrorCodes::InvalidOptions,
             "no such command option $maxTimeMs; use maxTimeMS instead",
             request.body[query_request_helper::queryOptionMaxTimeMS].eoo());
+    const int maxTimeMS =
+        uassertStatusOK(parseMaxTimeMS(request.body[query_request_helper::cmdOptionMaxTimeMS]));
+    if (maxTimeMS > 0 && command->getLogicalOp() != LogicalOp::opGetMore) {
+        opCtx->setDeadlineAfterNowBy(Milliseconds{maxTimeMS}, ErrorCodes::MaxTimeMSExpired);
+    }
+    opCtx->checkForInterrupt();  // May trigger maxTimeAlwaysTimeOut fail point.
 
     // If the command includes a 'comment' field, set it on the current OpCtx.
     if (auto commentField = request.body["comment"]) {
+        stdx::lock_guard<Client> lk(*client);
         opCtx->setComment(commentField.wrap());
     }
 
-    Client* client = opCtx->getClient();
     auto const apiParamsFromClient = initializeAPIParameters(request.body, command);
 
     {
@@ -613,25 +592,6 @@ Status ParseAndRunCommand::RunInvocation::_setup() {
     const auto& request = _parc->_rec->getRequest();
     auto replyBuilder = _parc->_rec->getReplyBuilder();
 
-    const int maxTimeMS =
-        uassertStatusOK(parseMaxTimeMS(request.body[query_request_helper::cmdOptionMaxTimeMS]));
-    if (maxTimeMS > 0 && command->getLogicalOp() != LogicalOp::opGetMore) {
-        opCtx->setDeadlineAfterNowBy(Milliseconds{maxTimeMS}, ErrorCodes::MaxTimeMSExpired);
-    }
-    if (MONGO_unlikely(
-            hangBeforeCheckingMongosShutdownInterrupt.shouldFail([&](const BSONObj& data) {
-                if (data.hasField("cmdName") && data.hasField("ns")) {
-                    std::string cmdNS = _parc->_ns.get();
-                    return ((data.getStringField("cmdName") == _parc->_commandName) &&
-                            (data.getStringField("ns") == cmdNS));
-                }
-                return false;
-            }))) {
-
-        LOGV2(6217501, "Hanging before hangBeforeCheckingMongosShutdownInterrupt is cancelled");
-        hangBeforeCheckingMongosShutdownInterrupt.pauseWhileSet();
-    }
-    opCtx->checkForInterrupt();  // May trigger maxTimeAlwaysTimeOut fail point.
     auto appendStatusToReplyAndSkipCommandExecution = [replyBuilder](Status status) -> Status {
         auto responseBuilder = replyBuilder->getBodyBuilder();
         CommandHelpers::appendCommandStatusNoThrow(responseBuilder, status);
@@ -654,21 +614,22 @@ Status ParseAndRunCommand::RunInvocation::_setup() {
         apiVersionMetrics.update(appName, apiParams);
     }
 
-    rpc::readRequestMetadata(opCtx, request.body, command->requiresAuth());
+    rpc::readRequestMetadata(opCtx, request, command->requiresAuth());
 
     CommandHelpers::evaluateFailCommandFailPoint(opCtx, invocation.get());
     bool startTransaction = false;
     if (_parc->_osi->getAutocommit()) {
         _routerSession.emplace(opCtx);
 
-        load_balancer_support::setMruSession(opCtx->getClient(), *opCtx->getLogicalSessionId());
-
         auto txnRouter = TransactionRouter::get(opCtx);
         invariant(txnRouter);
 
         auto txnNumber = opCtx->getTxnNumber();
         invariant(txnNumber);
 
+        auto txnRetryCounter = opCtx->getTxnRetryCounter();
+        invariant(txnRetryCounter);
+
         auto transactionAction = ([&] {
             auto startTxnSetting = _parc->_osi->getStartTransaction();
             if (startTxnSetting && *startTxnSetting) {
@@ -683,7 +644,7 @@ Status ParseAndRunCommand::RunInvocation::_setup() {
         })();
 
         startTransaction = (transactionAction == TransactionRouter::TransactionActions::kStart);
-        txnRouter.beginOrContinueTxn(opCtx, *txnNumber, transactionAction);
+        txnRouter.beginOrContinueTxn(opCtx, *txnNumber, transactionAction, *txnRetryCounter);
     }
 
     bool supportsWriteConcern = invocation->supportsWriteConcern();
@@ -718,15 +679,9 @@ Status ParseAndRunCommand::RunInvocation::_setup() {
                 ReadWriteConcernDefaults::get(opCtx->getServiceContext()).getDefault(opCtx);
             if (const auto wcDefault = rwcDefaults.getDefaultWriteConcern()) {
                 _parc->_wc = *wcDefault;
-                if (repl::feature_flags::gDefaultWCMajority.isEnabled(
-                        serverGlobalParams.featureCompatibility)) {
-                    const auto defaultWriteConcernSource =
-                        rwcDefaults.getDefaultWriteConcernSource();
-                    customDefaultWriteConcernWasApplied = defaultWriteConcernSource &&
-                        defaultWriteConcernSource == DefaultWriteConcernSourceEnum::kGlobal;
-                } else {
-                    customDefaultWriteConcernWasApplied = true;
-                }
+                const auto defaultWriteConcernSource = rwcDefaults.getDefaultWriteConcernSource();
+                customDefaultWriteConcernWasApplied = defaultWriteConcernSource &&
+                    defaultWriteConcernSource == DefaultWriteConcernSourceEnum::kGlobal;
                 LOGV2_DEBUG(22766,
                             2,
                             "Applying default writeConcern on {command} of {writeConcern}",
@@ -805,12 +760,8 @@ Status ParseAndRunCommand::RunInvocation::_setup() {
                 ReadWriteConcernDefaults::get(opCtx->getServiceContext()).getDefault(opCtx);
             const auto rcDefault = rwcDefaults.getDefaultReadConcern();
             if (rcDefault) {
-                const bool isDefaultRCLocalFeatureFlagEnabled =
-                    serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-                    repl::feature_flags::gDefaultRCLocal.isEnabled(
-                        serverGlobalParams.featureCompatibility);
                 const auto readConcernSource = rwcDefaults.getDefaultReadConcernSource();
-                customDefaultReadConcernWasApplied = !isDefaultRCLocalFeatureFlagEnabled ||
+                customDefaultReadConcernWasApplied =
                     (readConcernSource &&
                      readConcernSource.get() == DefaultReadConcernSourceEnum::kGlobal);
 
@@ -906,11 +857,6 @@ Status ParseAndRunCommand::RunInvocation::_setup() {
         _shouldAffectCommandCounter = true;
     }
 
-    if (_parc->_opType == dbQuery && !_parc->_isHello.get()) {
-        warnDeprecation(*opCtx->getClient(), networkOpToString(dbQuery));
-        globalOpCounters.gotQueryDeprecated();
-    }
-
     return Status::OK();
 }
 
@@ -968,7 +914,7 @@ void ParseAndRunCommand::RunAndRetry::_checkRetryForTransaction(Status& status)
     if (!txnRouter)
         return;
 
-    auto abortGuard = makeGuard([&] { txnRouter.implicitlyAbortTransaction(opCtx, status); });
+    ScopeGuard abortGuard([&] { txnRouter.implicitlyAbortTransaction(opCtx, status); });
 
     if (!_canRetry()) {
         addContextForTransactionAbortingError(
@@ -1094,6 +1040,27 @@ void ParseAndRunCommand::RunAndRetry::_onShardCannotRefreshDueToLocksHeldError(S
         iassert(status);
 }
 
+void ParseAndRunCommand::RunInvocation::_tapOnError(const Status& status) {
+    auto opCtx = _parc->_rec->getOpCtx();
+    const auto command = _parc->_rec->getCommand();
+
+    command->incrementCommandsFailed();
+    NotPrimaryErrorTracker::get(opCtx->getClient()).recordError(status.code());
+    // WriteConcern error (wcCode) is set to boost::none because:
+    // 1. TransientTransaction error label handling for commitTransaction command in mongos is
+    //    delegated to the shards. Mongos simply propagates the shard's response up to the client.
+    // 2. For other commands in a transaction, they shouldn't get a writeConcern error so this
+    //    setting doesn't apply.
+    auto errorLabels = getErrorLabels(opCtx,
+                                      *_parc->_osi,
+                                      command->getName(),
+                                      status.code(),
+                                      boost::none,
+                                      false /* isInternalClient */,
+                                      true /* isMongos */);
+    _parc->_errorBuilder->appendElements(errorLabels);
+}
+
 Future<void> ParseAndRunCommand::RunAndRetry::run() {
     return makeReadyFutureWith([&] {
                // Try kMaxNumStaleVersionRetries times. On the last try, exceptions are rethrown.
@@ -1126,10 +1093,11 @@ Future<void> ParseAndRunCommand::RunAndRetry::run() {
 
 Future<void> ParseAndRunCommand::RunInvocation::run() {
     return makeReadyFutureWith([&] {
-        iassert(_setup());
-        return future_util::makeState<RunAndRetry>(_parc).thenWithState(
-            [](auto* runner) { return runner->run(); });
-    });
+               iassert(_setup());
+               return future_util::makeState<RunAndRetry>(_parc).thenWithState(
+                   [](auto* runner) { return runner->run(); });
+           })
+        .tapError([this](Status status) { _tapOnError(status); });
 }
 
 Future<void> ParseAndRunCommand::run() {
@@ -1138,7 +1106,6 @@ Future<void> ParseAndRunCommand::run() {
                return future_util::makeState<RunInvocation>(this).thenWithState(
                    [](auto* runner) { return runner->run(); });
            })
-        .tapError([this](Status status) { _updateStatsAndApplyErrorLabels(status); })
         .onError<ErrorCodes::SkipCommandExecution>([this](Status status) {
             // We've already skipped execution, so no other action is required.
             return Status::OK();
@@ -1147,125 +1114,6 @@ Future<void> ParseAndRunCommand::run() {
 
 }  // namespace
 
-DbResponse Strategy::queryOp(OperationContext* opCtx, const NamespaceString& nss, DbMessage* dbm) {
-    globalOpCounters.gotQuery();
-    globalOpCounters.gotQueryDeprecated();
-
-    ON_BLOCK_EXIT([opCtx] {
-        Grid::get(opCtx)->catalogCache()->checkAndRecordOperationBlockedByRefresh(
-            opCtx, mongo::LogicalOp::opQuery);
-    });
-
-    const QueryMessage q(*dbm);
-
-    const auto upconvertedQuery = upconvertQueryEntry(q.query, nss, q.ntoreturn, q.ntoskip);
-
-    // Set the upconverted query as the CurOp command object.
-    CurOp::get(opCtx)->setGenericOpRequestDetails(
-        opCtx, nss, nullptr, upconvertedQuery, dbm->msg().operation());
-
-    Client* const client = opCtx->getClient();
-    AuthorizationSession* const authSession = AuthorizationSession::get(client);
-
-    // The legacy '$comment' operator gets converted to 'comment' by upconvertQueryEntry(). We
-    // set the comment in 'opCtx' so that it can be passed on to the respective shards.
-    if (auto commentField = upconvertedQuery["comment"]) {
-        opCtx->setComment(commentField.wrap());
-    }
-
-    Status status = auth::checkAuthForFind(authSession, nss, false);
-    audit::logQueryAuthzCheck(client, nss, q.query, status.code());
-    uassertStatusOK(status);
-
-    LOGV2_DEBUG(22768,
-                3,
-                "Query: {namespace} {query} ntoreturn: {ntoreturn} options: {queryOptions}",
-                "Query",
-                "namespace"_attr = q.ns,
-                "query"_attr = redact(q.query),
-                "ntoreturn"_attr = q.ntoreturn,
-                "queryOptions"_attr = q.queryOptions);
-
-    if (q.queryOptions & QueryOption_Exhaust) {
-        uasserted(18526,
-                  str::stream() << "The 'exhaust' query option is invalid for mongos queries: "
-                                << nss.ns() << " " << q.query.toString());
-    }
-
-    // Determine the default read preference mode based on the value of the slaveOk flag.
-    const auto defaultReadPref = q.queryOptions & QueryOption_SecondaryOk
-        ? ReadPreference::SecondaryPreferred
-        : ReadPreference::PrimaryOnly;
-    ReadPreferenceSetting::get(opCtx) =
-        uassertStatusOK(ReadPreferenceSetting::fromContainingBSON(q.query, defaultReadPref));
-
-    const boost::intrusive_ptr<ExpressionContext> expCtx;
-    auto canonicalQuery = uassertStatusOK(
-        CanonicalQuery::canonicalize(opCtx,
-                                     q,
-                                     expCtx,
-                                     ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures));
-
-    const FindCommandRequest& findCommand = canonicalQuery->getFindCommandRequest();
-    // Handle query option $maxTimeMS (not used with commands).
-    if (findCommand.getMaxTimeMS().value_or(0) > 0) {
-        uassert(50749,
-                "Illegal attempt to set operation deadline within DBDirectClient",
-                !opCtx->getClient()->isInDirectClient());
-        opCtx->setDeadlineAfterNowBy(Milliseconds{findCommand.getMaxTimeMS().value_or(0)},
-                                     ErrorCodes::MaxTimeMSExpired);
-    }
-    opCtx->checkForInterrupt();  // May trigger maxTimeAlwaysTimeOut fail point.
-
-    // If the $explain flag was set, we must run the operation on the shards as an explain command
-    // rather than a find command.
-    if (canonicalQuery->getExplain()) {
-        const BSONObj findCommandObj = findCommand.toBSON(BSONObj());
-
-        // We default to allPlansExecution verbosity.
-        const auto verbosity = ExplainOptions::Verbosity::kExecAllPlans;
-
-        BSONObjBuilder explainBuilder;
-        Strategy::explainFind(opCtx,
-                              findCommandObj,
-                              findCommand,
-                              verbosity,
-                              ReadPreferenceSetting::get(opCtx),
-                              &explainBuilder);
-
-        BSONObj explainObj = explainBuilder.done();
-        return replyToQuery(explainObj);
-    }
-
-    // Do the work to generate the first batch of results. This blocks waiting to get responses from
-    // the shard(s).
-    std::vector<BSONObj> batch;
-
-    // 0 means the cursor is exhausted. Otherwise we assume that a cursor with the returned id can
-    // be retrieved via the ClusterCursorManager.
-    CursorId cursorId;
-    try {
-        cursorId = ClusterFind::runQuery(
-            opCtx, *canonicalQuery, ReadPreferenceSetting::get(opCtx), &batch);
-    } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>&) {
-        uasserted(40247, "OP_QUERY not supported on views");
-    }
-
-    // Fill out the response buffer.
-    int numResults = 0;
-    OpQueryReplyBuilder reply;
-    for (auto&& obj : batch) {
-        obj.appendSelfToBufBuilder(reply.bufBuilderForResults());
-        numResults++;
-    }
-
-    return DbResponse{reply.toQueryReply(0,  // query result flags
-                                         numResults,
-                                         0,  // startingFrom
-                                         cursorId)};
-}
-
 // Maintains the state required to execute client commands, and provides the interface to construct
 // a future-chain that runs the command against the database.
 class ClientCommand final {
@@ -1298,7 +1146,11 @@ private:
 void ClientCommand::_parseMessage() try {
     const auto& msg = _rec->getMessage();
     _rec->setReplyBuilder(rpc::makeReplyBuilder(rpc::protocolForMessage(msg)));
-    _rec->setRequest(rpc::opMsgRequestFromAnyProtocol(msg));
+    auto opMsgReq = rpc::opMsgRequestFromAnyProtocol(msg);
+    if (msg.operation() == dbQuery) {
+        checkAllowedOpQueryCommand(*(_rec->getOpCtx()->getClient()), opMsgReq.getCommandName());
+    }
+    _rec->setRequest(opMsgReq);
 } catch (const DBException& ex) {
     // If this error needs to fail the connection, propagate it out.
     if (ErrorCodes::isConnectionFatalMessageParseError(ex.code()))
@@ -1414,241 +1266,4 @@ Future<DbResponse> Strategy::clientCommand(std::shared_ptr<RequestExecutionConte
         return runner->run();
     });
 }
-
-DbResponse Strategy::getMore(OperationContext* opCtx, const NamespaceString& nss, DbMessage* dbm) {
-    const int ntoreturn = dbm->pullInt();
-    uassert(
-        34424, str::stream() << "Invalid ntoreturn for OP_GET_MORE: " << ntoreturn, ntoreturn >= 0);
-    const long long cursorId = dbm->pullInt64();
-
-    globalOpCounters.gotGetMore();
-    globalOpCounters.gotGetMoreDeprecated();
-
-    // TODO: Handle stale config exceptions here from coll being dropped or sharded during op for
-    // now has same semantics as legacy request.
-
-    auto statusGetDb = Grid::get(opCtx)->catalogCache()->getDatabase(opCtx, nss.db());
-    if (statusGetDb == ErrorCodes::NamespaceNotFound) {
-        return replyToQuery(ResultFlag_CursorNotFound, nullptr, 0, 0);
-    }
-    uassertStatusOK(statusGetDb);
-
-    GetMoreCommandRequest getMoreCmd(cursorId, nss.coll().toString());
-    getMoreCmd.setDbName(nss.db());
-    if (ntoreturn) {
-        getMoreCmd.setBatchSize(ntoreturn);
-    }
-
-    // Set the upconverted getMore as the CurOp command object.
-    CurOp::get(opCtx)->setGenericOpRequestDetails(
-        opCtx, nss, nullptr, getMoreCmd.toBSON({}), dbm->msg().operation());
-
-    auto cursorResponse = ClusterFind::runGetMore(opCtx, getMoreCmd);
-    if (cursorResponse == ErrorCodes::CursorNotFound) {
-        return replyToQuery(ResultFlag_CursorNotFound, nullptr, 0, 0);
-    }
-    uassertStatusOK(cursorResponse.getStatus());
-
-    // Build the response document.
-    BufBuilder buffer(FindCommon::kInitReplyBufferSize);
-
-    int numResults = 0;
-    for (const auto& obj : cursorResponse.getValue().getBatch()) {
-        buffer.appendBuf((void*)obj.objdata(), obj.objsize());
-        ++numResults;
-    }
-
-    return replyToQuery(0,
-                        buffer.buf(),
-                        buffer.len(),
-                        numResults,
-                        cursorResponse.getValue().getNumReturnedSoFar().value_or(0),
-                        cursorResponse.getValue().getCursorId());
-}
-
-void Strategy::killCursors(OperationContext* opCtx, DbMessage* dbm) {
-    const int numCursors = dbm->pullInt();
-    massert(34425,
-            str::stream() << "Invalid killCursors message. numCursors: " << numCursors
-                          << ", message size: " << dbm->msg().dataSize() << ".",
-            dbm->msg().dataSize() == 8 + (8 * numCursors));
-    uassert(28794,
-            str::stream() << "numCursors must be between 1 and 29999.  numCursors: " << numCursors
-                          << ".",
-            numCursors >= 1 && numCursors < 30000);
-
-    globalOpCounters.gotKillCursorsDeprecated();
-
-    ConstDataCursor cursors(dbm->getArray(numCursors));
-
-    Client* const client = opCtx->getClient();
-    ClusterCursorManager* const manager = Grid::get(opCtx)->getCursorManager();
-
-    for (int i = 0; i < numCursors; ++i) {
-        const CursorId cursorId = cursors.readAndAdvance<LittleEndian<int64_t>>();
-
-        boost::optional<NamespaceString> nss = manager->getNamespaceForCursorId(cursorId);
-        if (!nss) {
-            LOGV2_DEBUG(22773,
-                        3,
-                        "Can't find cursor to kill, no namespace found. Cursor id: {cursorId}",
-                        "Can't find cursor to kill, no namespace found",
-                        "cursorId"_attr = cursorId);
-            continue;
-        }
-
-        auto authzSession = AuthorizationSession::get(client);
-        auto authChecker = [&authzSession, &nss](UserNameIterator userNames) -> Status {
-            return auth::checkAuthForKillCursors(authzSession, *nss, userNames);
-        };
-        auto authzStatus = manager->checkAuthForKillCursors(opCtx, *nss, cursorId, authChecker);
-        audit::logKillCursorsAuthzCheck(client, *nss, cursorId, authzStatus.code());
-        if (!authzStatus.isOK()) {
-            LOGV2_DEBUG(
-                22774,
-                3,
-                "Not authorized to kill cursor. Namespace: '{namespace}', cursor id: {cursorId}",
-                "Not authorized to kill cursor",
-                "namespace"_attr = *nss,
-                "cursorId"_attr = cursorId);
-            continue;
-        }
-
-        Status killCursorStatus = manager->killCursor(opCtx, *nss, cursorId);
-        if (!killCursorStatus.isOK()) {
-            LOGV2_DEBUG(
-                22775,
-                3,
-                "Can't find cursor to kill. Namespace: '{namespace}', cursor id: {cursorId}",
-                "Can't find cursor to kill",
-                "namespace"_attr = *nss,
-                "cursorId"_attr = cursorId);
-            continue;
-        }
-
-        LOGV2_DEBUG(22776,
-                    3,
-                    "Killed cursor. Namespace: '{namespace}', cursor id: {cursorId}",
-                    "Killed cursor",
-                    "namespace"_attr = *nss,
-                    "cursorId"_attr = cursorId);
-    }
-}
-
-void Strategy::writeOp(std::shared_ptr<RequestExecutionContext> rec) {
-    rec->setRequest([msg = rec->getMessage()]() {
-        switch (msg.operation()) {
-            case dbInsert: {
-                auto op = InsertOp::parseLegacy(msg);
-                globalOpCounters.gotInsertsDeprecated(op.getDocuments().size());
-                return op.serialize({});
-            }
-            case dbUpdate: {
-                globalOpCounters.gotUpdateDeprecated();
-                return UpdateOp::parseLegacy(msg).serialize({});
-            }
-            case dbDelete: {
-                globalOpCounters.gotDeleteDeprecated();
-                return DeleteOp::parseLegacy(msg).serialize({});
-            }
-            default:
-                MONGO_UNREACHABLE;
-        }
-    }());
-
-    rec->setReplyBuilder(std::make_unique<rpc::OpMsgReplyBuilder>());
-    auto bob = std::make_shared<BSONObjBuilder>();  // built objects are ignorned.
-    future_util::makeState<ParseAndRunCommand>(std::move(rec), std::move(bob))
-        .thenWithState([](auto* runner) { return runner->run(); })
-        .get();
-}
-
-void Strategy::explainFind(OperationContext* opCtx,
-                           const BSONObj& findCommandObj,
-                           const FindCommandRequest& findCommand,
-                           ExplainOptions::Verbosity verbosity,
-                           const ReadPreferenceSetting& readPref,
-                           BSONObjBuilder* out) {
-    const auto explainCmd = ClusterExplain::wrapAsExplain(findCommandObj, verbosity);
-
-    long long millisElapsed;
-    std::vector<AsyncRequestsSender::Response> shardResponses;
-
-    for (int tries = 0;; ++tries) {
-        bool canRetry = tries < 4;  // Fifth try (i.e. try #4) is the last one.
-
-        // We will time how long it takes to run the commands on the shards.
-        Timer timer;
-        try {
-            invariant(findCommand.getNamespaceOrUUID().nss());
-            const auto routingInfo =
-                uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(
-                    opCtx, *findCommand.getNamespaceOrUUID().nss()));
-            shardResponses = scatterGatherVersionedTargetByRoutingTable(
-                opCtx,
-                findCommand.getNamespaceOrUUID().nss()->db(),
-                *findCommand.getNamespaceOrUUID().nss(),
-                routingInfo,
-                explainCmd,
-                readPref,
-                Shard::RetryPolicy::kIdempotent,
-                findCommand.getFilter(),
-                findCommand.getCollation());
-            millisElapsed = timer.millis();
-            break;
-        } catch (ExceptionFor<ErrorCodes::ShardInvalidatedForTargeting>&) {
-            Grid::get(opCtx)->catalogCache()->setOperationShouldBlockBehindCatalogCacheRefresh(
-                opCtx, true);
-
-            if (canRetry) {
-                continue;
-            }
-            throw;
-        } catch (const ExceptionForCat<ErrorCategory::NeedRetargettingError>& ex) {
-            const auto staleNs = [&] {
-                if (auto staleInfo = ex.extraInfo<StaleConfigInfo>()) {
-                    return staleInfo->getNss();
-                }
-                throw;
-            }();
-
-            if (auto staleInfo = ex.extraInfo<StaleConfigInfo>()) {
-                Grid::get(opCtx)
-                    ->catalogCache()
-                    ->invalidateShardOrEntireCollectionEntryForShardedCollection(
-                        staleNs, staleInfo->getVersionWanted(), staleInfo->getShardId());
-            } else {
-                // If we don't have the stale config info and therefore don't know the shard's id,
-                // we have to force all further targetting requests for the namespace to block on
-                // a refresh.
-                Grid::get(opCtx)->catalogCache()->invalidateCollectionEntry_LINEARIZABLE(staleNs);
-            }
-
-            if (canRetry) {
-                continue;
-            }
-            throw;
-        } catch (const ExceptionFor<ErrorCodes::StaleDbVersion>& ex) {
-            // Mark database entry in cache as stale.
-            Grid::get(opCtx)->catalogCache()->onStaleDatabaseVersion(ex->getDb(),
-                                                                     ex->getVersionWanted());
-            if (canRetry) {
-                continue;
-            }
-            throw;
-        } catch (const ExceptionForCat<ErrorCategory::SnapshotError>&) {
-            // Simple retry on any type of snapshot error.
-            if (canRetry) {
-                continue;
-            }
-            throw;
-        }
-    }
-
-    const char* mongosStageName =
-        ClusterExplain::getStageNameForReadOp(shardResponses.size(), findCommandObj);
-
-    uassertStatusOK(ClusterExplain::buildExplainResult(
-        opCtx, shardResponses, mongosStageName, millisElapsed, findCommandObj, out));
-}
 }  // namespace mongo
