@@ -31,8 +31,6 @@
 
 #include <functional>
 
-#include "mongo/db/repl/optime.h"
-#include "mongo/db/vector_clock.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
@@ -44,8 +42,6 @@ namespace mongo {
 
 class BalancerConfiguration;
 class ClusterCursorManager;
-class OperationContext;
-class ServiceContext;
 
 namespace executor {
 class NetworkInterface;
@@ -102,20 +98,6 @@ public:
     CustomConnectionPoolStatsFn getCustomConnectionPoolStatsFn() const;
     void setCustomConnectionPoolStatsFn(CustomConnectionPoolStatsFn statsFn);
 
-    /**
-     * Deprecated. This is only used on mongos, and once addShard is solely handled by the configs,
-     * it can be deleted.
-     * @return true if shards and config servers are allowed to use 'localhost' in address
-     */
-    bool allowLocalHost() const;
-
-    /**
-     * Deprecated. This is only used on mongos, and once addShard is solely handled by the configs,
-     * it can be deleted.
-     * @param whether to allow shards and config servers to use 'localhost' in address
-     */
-    void setAllowLocalHost(bool allow);
-
     ShardingCatalogClient* catalogClient() const {
         return _catalogClient.get();
     }
@@ -144,35 +126,6 @@ public:
         return _balancerConfig.get();
     }
 
-    /**
-     * Returns a readConcern at the specified level for reading after the current ConfigTime.
-     */
-    repl::ReadConcernArgs readConcernWithConfigTime(repl::ReadConcernLevel readConcernLevel) const;
-
-    /**
-     * Returns a readPreference (based on the given one) for targeting a config server that is at or
-     * after the current ConfigTime.
-     */
-    ReadPreferenceSetting readPreferenceWithConfigTime(
-        const ReadPreferenceSetting& readPreference) const;
-
-    /**
-     * Returns the the last optime that a shard or config server has reported as the current
-     * committed optime on the config server.
-     * NOTE: This is not valid to call on a config server instance.
-     */
-    repl::OpTime configOpTime() const;
-
-    /**
-     * Called whenever a mongos or shard gets a response from a config server or shard and updates
-     * what we've seen as the last config server optime.
-     * If the config optime was updated, returns the previous value.
-     * NOTE: This is not valid to call on a config server instance.
-     */
-    boost::optional<repl::OpTime> advanceConfigOpTime(OperationContext* opCtx,
-                                                      repl::OpTime opTime,
-                                                      StringData what);
-
     /**
      * Clears the grid object so that it can be reused between test executions. This will not
      * be necessary if grid is hanging off the global ServiceContext and each test gets its
@@ -199,29 +152,11 @@ private:
     // questions about the network configuration, such as getting the current server's hostname.
     executor::NetworkInterface* _network{nullptr};
 
-    CustomConnectionPoolStatsFn _customConnectionPoolStatsFn;
-
     AtomicWord<bool> _shardingInitialized{false};
 
-    // Protects _configOpTime.
     mutable Mutex _mutex = MONGO_MAKE_LATCH(HierarchicalAcquisitionLevel(0), "Grid::_mutex");
 
-    // Last known highest opTime from the config server that should be used when doing reads.
-    // This value is updated any time a shard or mongos talks to a config server or a shard.
-    repl::OpTime _configOpTime{VectorClock::kInitialComponentTime.asTimestamp(),
-                               repl::OpTime::kUninitializedTerm};
-
-    /**
-     * Called to update what we've seen as the last config server optime.
-     * If the config optime was updated, returns the previous value.
-     * NOTE: This is not valid to call on a config server instance.
-     */
-    boost::optional<repl::OpTime> _advanceConfigOpTime(const repl::OpTime& opTime);
-
-    // Deprecated. This is only used on mongos, and once addShard is solely handled by the configs,
-    // it can be deleted.
-    // Can 'localhost' be used in shard addresses?
-    bool _allowLocalShard{true};
+    CustomConnectionPoolStatsFn _customConnectionPoolStatsFn;
 };
 
 }  // namespace mongo
