@@ -36,9 +36,11 @@
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/client/remote_command_targeter.h"
 #include "mongo/db/pipeline/change_stream_constants.h"
+#include "mongo/db/pipeline/change_stream_invalidation_info.h"
 #include "mongo/db/query/cursor_response.h"
-#include "mongo/db/query/getmore_request.h"
-#include "mongo/db/query/killcursors_request.h"
+#include "mongo/db/query/getmore_command_gen.h"
+#include "mongo/db/query/kill_cursors_gen.h"
+#include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/remote_command_response.h"
 #include "mongo/s/catalog/type_shard.h"
@@ -55,8 +57,22 @@ namespace {
 const int kMaxNumFailedHostRetryAttempts = 3;
 
 /**
- * Returns the sort key out of the $sortKey metadata field in 'obj'. This object is of the form
- * {'': 'firstSortKey', '': 'secondSortKey', ...}.
+ * Returns the sort key out of the $sortKey metadata field in 'obj'. The sort key should be
+ * formatted as an array with one value per field of the sort pattern:
+ *  {..., $sortKey: [<firstSortKeyComponent>, <secondSortKeyComponent>, ...], ...}
+ *
+ * This function returns the sort key not as an array, but as the equivalent BSONObj:
+ *   {"0": <firstSortKeyComponent>, "1": <secondSortKeyComponent>}
+ *
+ * The return value is allowed to omit the key names, so the caller should not rely on the key names
+ * being present. That is, the return value could consist of an object such as
+ *   {"": <firstSortKeyComponent>, "": <secondSortKeyComponent>}
+ *
+ * If 'compareWholeSortKey' is true, then the value inside the $sortKey is directly interpreted as a
+ * single-element sort key. For example, given the document
+ *   {..., $sortKey: <value>, ...}
+ * and 'compareWholeSortKey'=true, this function will return
+ *   {"": <value>}
  */
 BSONObj extractSortKey(BSONObj obj, bool compareWholeSortKey) {
     auto key = obj[AsyncResultsMerger::kSortKeyField];
@@ -64,12 +80,8 @@ BSONObj extractSortKey(BSONObj obj, bool compareWholeSortKey) {
     if (compareWholeSortKey) {
         return key.wrap();
     }
-    // TODO (SERVER-43361): We expect the sort key to be an array, but if the sort key originated
-    // from a 4.2 mongod, it will be a document, instead. Either way, 'isABSONObj()' will return
-    // true, and 'compareSortKeys()' will behave the same way. After branching for 4.5, we can
-    // tighten this invariant to specifically require a 'BSONArray' type.
-    invariant(key.isABSONObj());
-    return key.Obj();
+    invariant(key.type() == BSONType::Array);
+    return key.embeddedObject();
 }
 
 /**
@@ -112,6 +124,12 @@ AsyncResultsMerger::AsyncResultsMerger(OperationContext* opCtx,
         // A remote cannot be flagged as 'partialResultsReturned' if 'allowPartialResults' is false.
         invariant(!(_remotes.back().partialResultsReturned && !_params.getAllowPartialResults()));
 
+        // For the first batch, cursor should never be invalidated.
+        tassert(
+            5493704, "Found invalidated cursor on the first batch", !_remotes.back().invalidated);
+
+        _remotes.back().shardId = remote.getShardId().toString();
+
         // We don't check the return value of _addBatchToBuffer here; if there was an error,
         // it will be stored in the remote and the first call to ready() will return true.
         _addBatchToBuffer(WithLock::withoutLock(), remoteIndex, remote.getCursorResponse());
@@ -425,13 +443,13 @@ Status AsyncResultsMerger::_askForNextBatch(WithLock, size_t remoteIndex) {
         adjustedBatchSize = *_params.getBatchSize() - remote.fetchedCount;
     }
 
-    BSONObj cmdObj = GetMoreRequest(remote.cursorNss,
-                                    remote.cursorId,
-                                    adjustedBatchSize,
-                                    _awaitDataTimeout,
-                                    boost::none,
-                                    boost::none)
-                         .toBSON();
+    GetMoreCommandRequest getMoreRequest(remote.cursorId, remote.cursorNss.coll().toString());
+    getMoreRequest.setBatchSize(adjustedBatchSize);
+    if (_awaitDataTimeout) {
+        getMoreRequest.setMaxTimeMS(
+            static_cast<std::int64_t>(durationCount<Milliseconds>(*_awaitDataTimeout)));
+    }
+    BSONObj cmdObj = getMoreRequest.toBSON({});
 
     if (_params.getSessionId()) {
         BSONObjBuilder newCmdBob(std::move(cmdObj));
@@ -475,6 +493,11 @@ Status AsyncResultsMerger::scheduleGetMores() {
 }
 
 Status AsyncResultsMerger::_scheduleGetMores(WithLock lk) {
+    // Before scheduling more work, check whether the cursor has been invalidated.
+    if (feature_flags::gFeatureFlagChangeStreamsOptimization.isEnabledAndIgnoreFCV()) {
+        _assertNotInvalidated(lk);
+    }
+
     // Schedule remote work on hosts for which we need more results.
     for (size_t i = 0; i < _remotes.size(); ++i) {
         auto& remote = _remotes[i];
@@ -539,6 +562,15 @@ StatusWith<executor::TaskExecutor::EventHandle> AsyncResultsMerger::nextEvent()
     return eventToReturn;
 }
 
+void AsyncResultsMerger::_assertNotInvalidated(WithLock lk) {
+    if (auto minPromisedSortKey = _getMinPromisedSortKey(lk)) {
+        const auto& minRemote = _remotes[minPromisedSortKey->second];
+        uassert(ChangeStreamInvalidationInfo{minPromisedSortKey->first.firstElement().Obj()},
+                "Change stream invalidated",
+                !(minRemote.invalidated && !_ready(lk)));
+    }
+}
+
 StatusWith<CursorResponse> AsyncResultsMerger::_parseCursorResponse(
     const BSONObj& responseObj, const RemoteCursorData& remote) {
 
@@ -566,6 +598,14 @@ void AsyncResultsMerger::_updateRemoteMetadata(WithLock lk,
     // Update the cursorId; it is sent as '0' when the cursor has been exhausted on the shard.
     auto& remote = _remotes[remoteIndex];
     remote.cursorId = response.getCursorId();
+
+    // If the response indicates that the cursor has been invalidated, mark the corresponding
+    // remote as invalidated. This also signifies that the shard cursor has been closed.
+    remote.invalidated = response.getInvalidated();
+    tassert(5493705,
+            "Unexpectedly encountered invalidated cursor with non-zero ID",
+            !(remote.invalidated && remote.cursorId > 0));
+
     if (response.getPostBatchResumeToken()) {
         // We only expect to see this for change streams.
         invariant(_params.getSort());
@@ -672,12 +712,10 @@ void AsyncResultsMerger::_cleanUpKilledBatch(WithLock lk) {
     invariant(_lifecycleState == kKillStarted);
 
     // If this is the last callback to run then we are ready to free the ARM. We signal the
-    // '_killCompleteEvent', which the caller of kill() may be waiting on.
+    // '_killCompleteInfo', which the caller of kill() may be waiting on.
     if (!_haveOutstandingBatchRequests(lk)) {
-        // If the event is invalid then '_executor' is in shutdown, so we cannot signal events.
-        if (_killCompleteEvent.isValid()) {
-            _executor->signalEvent(_killCompleteEvent);
-        }
+        invariant(_killCompleteInfo);
+        _killCompleteInfo->signalFutures();
 
         _lifecycleState = kKillComplete;
     }
@@ -805,11 +843,12 @@ bool AsyncResultsMerger::_haveOutstandingBatchRequests(WithLock) {
 }
 
 void AsyncResultsMerger::_scheduleKillCursors(WithLock, OperationContext* opCtx) {
-    invariant(_killCompleteEvent.isValid());
+    invariant(_killCompleteInfo);
 
     for (const auto& remote : _remotes) {
         if (remote.status.isOK() && remote.cursorId && !remote.exhausted()) {
-            BSONObj cmdObj = KillCursorsRequest(_params.getNss(), {remote.cursorId}).toBSON();
+            BSONObj cmdObj =
+                KillCursorsCommandRequest(_params.getNss(), {remote.cursorId}).toBSON(BSONObj{});
 
             executor::RemoteCommandRequest request(
                 remote.getTargetHost(), _params.getNss().db().toString(), cmdObj, opCtx);
@@ -820,48 +859,37 @@ void AsyncResultsMerger::_scheduleKillCursors(WithLock, OperationContext* opCtx)
     }
 }
 
-executor::TaskExecutor::EventHandle AsyncResultsMerger::kill(OperationContext* opCtx) {
+stdx::shared_future<void> AsyncResultsMerger::kill(OperationContext* opCtx) {
     stdx::lock_guard<Latch> lk(_mutex);
 
-    if (_killCompleteEvent.isValid()) {
+    if (_killCompleteInfo) {
         invariant(_lifecycleState != kAlive);
-        return _killCompleteEvent;
+        return _killCompleteInfo->getFuture();
     }
 
     invariant(_lifecycleState == kAlive);
     _lifecycleState = kKillStarted;
 
-    // Make '_killCompleteEvent', which we will signal as soon as all of our callbacks
-    // have finished running.
-    auto statusWithEvent = _executor->makeEvent();
-    if (ErrorCodes::isShutdownError(statusWithEvent.getStatus().code())) {
-        // The underlying task executor is shutting down.
-        if (!_haveOutstandingBatchRequests(lk)) {
-            _lifecycleState = kKillComplete;
-        }
-        return executor::TaskExecutor::EventHandle();
-    }
-    fassert(28716, statusWithEvent);
-    _killCompleteEvent = statusWithEvent.getValue();
+    // Create "_killCompleteInfo", which we will signal as soon as all of our callbacks have
+    // finished running.
+    _killCompleteInfo.emplace();
 
     _scheduleKillCursors(lk, opCtx);
 
     if (!_haveOutstandingBatchRequests(lk)) {
         _lifecycleState = kKillComplete;
-        // Signal the event right now, as there's nothing to wait for.
-        _executor->signalEvent(_killCompleteEvent);
-        return _killCompleteEvent;
+        // Signal the future right now, as there's nothing to wait for.
+        _killCompleteInfo->signalFutures();
+        return _killCompleteInfo->getFuture();
     }
 
-    _lifecycleState = kKillStarted;
-
     // Cancel all of our callbacks. Once they all complete, the event will be signaled.
     for (const auto& remote : _remotes) {
         if (remote.cbHandle.isValid()) {
             _executor->cancel(remote.cbHandle);
         }
     }
-    return _killCompleteEvent;
+    return _killCompleteInfo->getFuture();
 }
 
 //
