@@ -35,7 +35,6 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/connpool.h"
 #include "mongo/db/auth/authorization_session.h"
-#include "mongo/db/pipeline/change_stream_constants.h"
 #include "mongo/db/pipeline/change_stream_invalidation_info.h"
 #include "mongo/db/pipeline/document_source_limit.h"
 #include "mongo/db/pipeline/document_source_skip.h"
@@ -102,9 +101,7 @@ AsyncRequestsSender::Response establishMergingShardCursor(OperationContext* opCt
         ReadPreferenceSetting::get(opCtx),
         sharded_agg_helpers::getDesiredRetryPolicy(opCtx));
     const auto response = ars.next();
-    tassert(6273807,
-            "requested and received data from just one shard, but results are still pending",
-            ars.done());
+    invariant(ars.done());
     return response;
 }
 
@@ -151,7 +148,8 @@ BSONObj createCommandForMergingShard(Document serializedCommand,
     }
 
     // Attach the IGNORED chunk version to the command. On the shard, this will skip the actual
-    // version check but will nonetheless mark the operation as versioned.
+    // version check but will nonetheless mark the operation as versioned, indicating that any
+    // internal operations executed by the pipeline should also be appropriately versioned.
     auto mergeCmdObj = appendShardVersion(mergeCmd.freeze().toBson(), ChunkVersion::IGNORED());
 
     // Attach the read and write concerns if needed, and return the final command object.
@@ -286,7 +284,7 @@ BSONObj establishMergingMongosCursor(OperationContext* opCtx,
     for (long long objCount = 0; objCount < batchSize; ++objCount) {
         ClusterQueryResult next;
         try {
-            next = uassertStatusOK(ccc->next(RouterExecStage::ExecContext::kInitialFind));
+            next = uassertStatusOK(ccc->next());
         } catch (const ExceptionFor<ErrorCodes::CloseChangeStream>&) {
             // This exception is thrown when a $changeStream stage encounters an event
             // that invalidates the cursor. We should close the cursor and return without
@@ -298,8 +296,7 @@ BSONObj establishMergingMongosCursor(OperationContext* opCtx,
             // to the resume token of the invalidating event, and mark the cursor response as
             // invalidated. We always expect to have ExtraInfo for this error code.
             const auto extraInfo = ex.extraInfo<ChangeStreamInvalidationInfo>();
-            tassert(
-                5493706, "Missing ChangeStreamInvalidationInfo on exception", extraInfo != nullptr);
+            tassert(5493706, "Missing ChangeStreamInvalidationInfo on exception", extraInfo);
 
             responseBuilder.setPostBatchResumeToken(extraInfo->getInvalidateResumeToken());
             responseBuilder.setInvalidated();
@@ -571,11 +568,7 @@ AggregationTargeter AggregationTargeter::make(
     boost::optional<ChunkManager> cm,
     stdx::unordered_set<NamespaceString> involvedNamespaces,
     bool hasChangeStream,
-    bool allowedToPassthrough,
-    bool perShardCursor) {
-    if (perShardCursor) {
-        return {TargetingPolicy::kSpecificShardOnly, nullptr, cm};
-    }
+    bool allowedToPassthrough) {
 
     // Check if any of the involved collections are sharded.
     bool involvesShardedCollections = [&]() {
@@ -624,15 +617,68 @@ Status runPipelineOnPrimaryShard(const boost::intrusive_ptr<ExpressionContext>&
                                  Document serializedCommand,
                                  const PrivilegeVector& privileges,
                                  BSONObjBuilder* out) {
-    return runPipelineOnSpecificShardOnly(expCtx,
-                                          namespaces,
-                                          boost::optional<DatabaseVersion>(cm.dbVersion()),
-                                          explain,
-                                          serializedCommand,
-                                          privileges,
-                                          cm.dbPrimary(),
-                                          false,
-                                          out);
+    auto opCtx = expCtx->opCtx;
+
+    // Format the command for the shard. This adds the 'fromMongos' field, wraps the command as an
+    // explain if necessary, and rewrites the result into a format safe to forward to shards.
+    BSONObj cmdObj = sharded_agg_helpers::createPassthroughCommandForShard(expCtx,
+                                                                           serializedCommand,
+                                                                           explain,
+                                                                           nullptr, /* pipeline */
+                                                                           BSONObj(),
+                                                                           boost::none);
+
+    const auto shardId = cm.dbPrimary();
+    const auto cmdObjWithShardVersion = (shardId != ShardId::kConfigServerId)
+        ? appendShardVersion(std::move(cmdObj), ChunkVersion::UNSHARDED())
+        : std::move(cmdObj);
+
+    MultiStatementTransactionRequestsSender ars(
+        opCtx,
+        Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
+        namespaces.executionNss.db().toString(),
+        {{shardId, appendDbVersionIfPresent(cmdObjWithShardVersion, cm.dbVersion())}},
+        ReadPreferenceSetting::get(opCtx),
+        Shard::RetryPolicy::kIdempotent);
+    auto response = ars.next();
+    invariant(ars.done());
+
+    uassertStatusOK(response.swResponse);
+    auto commandStatus = getStatusFromCommandResult(response.swResponse.getValue().data);
+
+    if (ErrorCodes::isStaleShardVersionError(commandStatus.code())) {
+        uassertStatusOK(commandStatus.withContext("command failed because of stale config"));
+    } else if (ErrorCodes::isSnapshotError(commandStatus.code())) {
+        uassertStatusOK(
+            commandStatus.withContext("command failed because can not establish a snapshot"));
+    }
+
+    BSONObj result;
+    if (explain) {
+        // If this was an explain, then we get back an explain result object rather than a cursor.
+        result = response.swResponse.getValue().data;
+    } else {
+        result = uassertStatusOK(
+            storePossibleCursor(opCtx,
+                                shardId,
+                                *response.shardHostAndPort,
+                                response.swResponse.getValue().data,
+                                namespaces.requestedNss,
+                                Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
+                                Grid::get(opCtx)->getCursorManager(),
+                                privileges,
+                                TailableModeEnum::kNormal));
+    }
+
+    // First append the properly constructed writeConcernError. It will then be skipped
+    // in appendElementsUnique.
+    if (auto wcErrorElem = result["writeConcernError"]) {
+        appendWriteConcernErrorToCmdResponse(shardId, wcErrorElem, *out);
+    }
+
+    out->appendElementsUnique(CommandHelpers::filterCommandReplyForPassthrough(result));
+
+    return getStatusFromCommandResult(out->asTempObj());
 }
 
 Status runPipelineOnMongoS(const ClusterAggregate::Namespaces& namespaces,
@@ -775,96 +821,5 @@ std::pair<BSONObj, boost::optional<UUID>> getCollationAndUUID(
     return {collation.isEmpty() ? getCollation() : collation, getUUID()};
 }
 
-Status runPipelineOnSpecificShardOnly(const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                                      const ClusterAggregate::Namespaces& namespaces,
-                                      boost::optional<DatabaseVersion> dbVersion,
-                                      boost::optional<ExplainOptions::Verbosity> explain,
-                                      Document serializedCommand,
-                                      const PrivilegeVector& privileges,
-                                      ShardId shardId,
-                                      bool forPerShardCursor,
-                                      BSONObjBuilder* out) {
-    auto opCtx = expCtx->opCtx;
-
-    boost::optional<int> overrideBatchSize;
-    if (forPerShardCursor) {
-        tassert(6273804,
-                "Per shard cursors are supposed to pass fromMongos: false to shards",
-                !expCtx->inMongos);
-        // By using an initial batchSize of zero all of the events will get returned through
-        // the getMore path and have metadata stripped out.
-        overrideBatchSize = 0;
-    }
-
-    // Format the command for the shard. This wraps the command as an explain if necessary, and
-    // rewrites the result into a format safe to forward to shards.
-    BSONObj cmdObj = sharded_agg_helpers::createPassthroughCommandForShard(expCtx,
-                                                                           serializedCommand,
-                                                                           explain,
-                                                                           nullptr, /* pipeline */
-                                                                           BSONObj(),
-                                                                           overrideBatchSize);
-
-    if (!forPerShardCursor && shardId != ShardId::kConfigServerId) {
-        cmdObj = appendShardVersion(std::move(cmdObj), ChunkVersion::UNSHARDED());
-    }
-    if (!forPerShardCursor) {
-        // Unless this is a per shard cursor, we need to send shard version info.
-        tassert(6377400, "Missing shard versioning information", dbVersion.has_value());
-        cmdObj = appendDbVersionIfPresent(std::move(cmdObj), *dbVersion);
-    }
-
-    MultiStatementTransactionRequestsSender ars(
-        opCtx,
-        Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
-        namespaces.executionNss.db().toString(),
-        {{shardId, cmdObj}},
-        ReadPreferenceSetting::get(opCtx),
-        Shard::RetryPolicy::kIdempotent);
-    auto response = ars.next();
-    tassert(6273806,
-            "requested and received data from just one shard, but results are still pending",
-            ars.done());
-
-    uassertStatusOK(response.swResponse);
-    auto commandStatus = getStatusFromCommandResult(response.swResponse.getValue().data);
-
-    if (ErrorCodes::isStaleShardVersionError(commandStatus.code())) {
-        uassertStatusOK(commandStatus.withContext("command failed because of stale config"));
-    } else if (ErrorCodes::isSnapshotError(commandStatus.code())) {
-        uassertStatusOK(
-            commandStatus.withContext("command failed because can not establish a snapshot"));
-    }
-
-    BSONObj result;
-    if (explain) {
-        // If this was an explain, then we get back an explain result object rather than a cursor.
-        result = response.swResponse.getValue().data;
-    } else {
-        result = uassertStatusOK(storePossibleCursor(
-            opCtx,
-            shardId,
-            *response.shardHostAndPort,
-            response.swResponse.getValue().data,
-            namespaces.requestedNss,
-            Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
-            Grid::get(opCtx)->getCursorManager(),
-            privileges,
-            expCtx->tailableMode,
-            forPerShardCursor ? boost::optional<BSONObj>(change_stream_constants::kSortSpec)
-                              : boost::none));
-    }
-
-    // First append the properly constructed writeConcernError. It will then be skipped
-    // in appendElementsUnique.
-    if (auto wcErrorElem = result["writeConcernError"]) {
-        appendWriteConcernErrorToCmdResponse(shardId, wcErrorElem, *out);
-    }
-
-    out->appendElementsUnique(CommandHelpers::filterCommandReplyForPassthrough(result));
-
-    return getStatusFromCommandResult(out->asTempObj());
-}
-
 }  // namespace cluster_aggregation_planner
 }  // namespace mongo
