@@ -35,6 +35,7 @@
 #include "mongo/db/logical_session_cache_noop.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/service_context_test_fixture.h"
+#include "mongo/s/concurrency/locker_mongos_client_observer.h"
 #include "mongo/s/query/cluster_client_cursor_mock.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
 #include "mongo/unittest/unittest.h"
@@ -48,7 +49,10 @@ const NamespaceString nss("test.collection");
 
 class ClusterCursorManagerTest : public ServiceContextTest {
 protected:
-    ClusterCursorManagerTest() : _opCtx(makeOperationContext()), _manager(&_clockSourceMock) {
+    ClusterCursorManagerTest() : _manager(&_clockSourceMock) {
+        auto service = getServiceContext();
+        service->registerClientObserver(std::make_unique<LockerMongosClientObserver>());
+        _opCtx = makeOperationContext();
         LogicalSessionCache::set(getServiceContext(), std::make_unique<LogicalSessionCacheNoop>());
     }
 
@@ -143,11 +147,11 @@ TEST_F(ClusterCursorManagerTest, RegisterCursor) {
     auto pinnedCursor =
         getManager()->checkOutCursor(nss, cursorId, _opCtx.get(), successAuthChecker);
     ASSERT_OK(pinnedCursor.getStatus());
-    auto nextResult = pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind);
+    auto nextResult = pinnedCursor.getValue()->next();
     ASSERT_OK(nextResult.getStatus());
     ASSERT(nextResult.getValue().getResult());
     ASSERT_BSONOBJ_EQ(BSON("a" << 1), *nextResult.getValue().getResult());
-    nextResult = pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind);
+    nextResult = pinnedCursor.getValue()->next();
     ASSERT_OK(nextResult.getStatus());
     ASSERT_TRUE(nextResult.getValue().isEOF());
 }
@@ -179,11 +183,11 @@ TEST_F(ClusterCursorManagerTest, CheckOutCursorBasic) {
         getManager()->checkOutCursor(nss, cursorId, _opCtx.get(), successAuthChecker);
     ASSERT_OK(checkedOutCursor.getStatus());
     ASSERT_EQ(cursorId, checkedOutCursor.getValue().getCursorId());
-    auto nextResult = checkedOutCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind);
+    auto nextResult = checkedOutCursor.getValue()->next();
     ASSERT_OK(nextResult.getStatus());
     ASSERT(nextResult.getValue().getResult());
     ASSERT_BSONOBJ_EQ(BSON("a" << 1), *nextResult.getValue().getResult());
-    nextResult = checkedOutCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind);
+    nextResult = checkedOutCursor.getValue()->next();
     ASSERT_OK(nextResult.getStatus());
     ASSERT_TRUE(nextResult.getValue().isEOF());
 }
@@ -208,11 +212,11 @@ TEST_F(ClusterCursorManagerTest, CheckOutCursorMultipleCursors) {
         auto pinnedCursor =
             getManager()->checkOutCursor(nss, cursorIds[i], _opCtx.get(), successAuthChecker);
         ASSERT_OK(pinnedCursor.getStatus());
-        auto nextResult = pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind);
+        auto nextResult = pinnedCursor.getValue()->next();
         ASSERT_OK(nextResult.getStatus());
         ASSERT(nextResult.getValue().getResult());
         ASSERT_BSONOBJ_EQ(BSON("a" << i), *nextResult.getValue().getResult());
-        nextResult = pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind);
+        nextResult = pinnedCursor.getValue()->next();
         ASSERT_OK(nextResult.getStatus());
         ASSERT_TRUE(nextResult.getValue().isEOF());
     }
@@ -668,8 +672,7 @@ TEST_F(ClusterCursorManagerTest, StatsExhaustShardedCursor) {
     auto pinnedCursor =
         getManager()->checkOutCursor(nss, cursorId, _opCtx.get(), successAuthChecker);
     ASSERT_OK(pinnedCursor.getStatus());
-    ASSERT_OK(
-        pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind).getStatus());
+    ASSERT_OK(pinnedCursor.getValue()->next().getStatus());
     ASSERT_EQ(1U, getManager()->stats().cursorsMultiTarget);
     pinnedCursor.getValue().returnCursor(ClusterCursorManager::CursorState::Exhausted);
     ASSERT_EQ(0U, getManager()->stats().cursorsMultiTarget);
@@ -687,8 +690,7 @@ TEST_F(ClusterCursorManagerTest, StatsExhaustNotShardedCursor) {
     auto pinnedCursor =
         getManager()->checkOutCursor(nss, cursorId, _opCtx.get(), successAuthChecker);
     ASSERT_OK(pinnedCursor.getStatus());
-    ASSERT_OK(
-        pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind).getStatus());
+    ASSERT_OK(pinnedCursor.getValue()->next().getStatus());
     ASSERT_EQ(1U, getManager()->stats().cursorsSingleTarget);
     pinnedCursor.getValue().returnCursor(ClusterCursorManager::CursorState::Exhausted);
     ASSERT_EQ(0U, getManager()->stats().cursorsSingleTarget);
@@ -707,8 +709,7 @@ TEST_F(ClusterCursorManagerTest, StatsExhaustPinnedCursor) {
     auto pinnedCursor =
         getManager()->checkOutCursor(nss, cursorId, _opCtx.get(), successAuthChecker);
     ASSERT_OK(pinnedCursor.getStatus());
-    ASSERT_OK(
-        pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind).getStatus());
+    ASSERT_OK(pinnedCursor.getValue()->next().getStatus());
     ASSERT_EQ(1U, getManager()->stats().cursorsPinned);
     pinnedCursor.getValue().returnCursor(ClusterCursorManager::CursorState::Exhausted);
     ASSERT_EQ(0U, getManager()->stats().cursorsPinned);
@@ -727,8 +728,7 @@ TEST_F(ClusterCursorManagerTest, StatsCheckInWithoutExhaustingPinnedCursor) {
     auto pinnedCursor =
         getManager()->checkOutCursor(nss, cursorId, _opCtx.get(), successAuthChecker);
     ASSERT_OK(pinnedCursor.getStatus());
-    ASSERT_OK(
-        pinnedCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind).getStatus());
+    ASSERT_OK(pinnedCursor.getValue()->next().getStatus());
     ASSERT_EQ(1U, getManager()->stats().cursorsPinned);
     pinnedCursor.getValue().returnCursor(ClusterCursorManager::CursorState::NotExhausted);
     ASSERT_EQ(0U, getManager()->stats().cursorsPinned);
@@ -844,8 +844,7 @@ TEST_F(ClusterCursorManagerTest, PinnedCursorReturnCursorExhausted) {
     ASSERT_OK(registeredCursor.getStatus());
     ASSERT_EQ(cursorId, registeredCursor.getValue().getCursorId());
     ASSERT_NE(0, cursorId);
-    ASSERT_OK(
-        registeredCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind).getStatus());
+    ASSERT_OK(registeredCursor.getValue()->next().getStatus());
     registeredCursor.getValue().returnCursor(ClusterCursorManager::CursorState::Exhausted);
     ASSERT_EQ(0, registeredCursor.getValue().getCursorId());
 
@@ -872,8 +871,7 @@ TEST_F(ClusterCursorManagerTest, PinnedCursorReturnCursorExhaustedWithNonExhaust
     ASSERT_OK(registeredCursor.getStatus());
     ASSERT_EQ(cursorId, registeredCursor.getValue().getCursorId());
     ASSERT_NE(0, cursorId);
-    ASSERT_OK(
-        registeredCursor.getValue()->next(RouterExecStage::ExecContext::kInitialFind).getStatus());
+    ASSERT_OK(registeredCursor.getValue()->next().getStatus());
     registeredCursor.getValue().returnCursor(ClusterCursorManager::CursorState::Exhausted);
     ASSERT_EQ(0, registeredCursor.getValue().getCursorId());
 
