@@ -49,8 +49,9 @@
 #include "mongo/db/curop_failpoint_helpers.h"
 #include "mongo/db/pipeline/change_stream_invalidation_info.h"
 #include "mongo/db/query/canonical_query.h"
+#include "mongo/db/query/canonical_query_encoder.h"
 #include "mongo/db/query/find_common.h"
-#include "mongo/db/query/getmore_request.h"
+#include "mongo/db/query/getmore_command_gen.h"
 #include "mongo/db/query/query_planner_common.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
@@ -109,40 +110,6 @@ StatusWith<std::unique_ptr<FindCommandRequest>> transformQueryForShards(
         newLimit = newLimitValue;
     }
 
-    // Similarly, if nToReturn is set, we forward the sum of nToReturn and the skip.
-    boost::optional<int64_t> newNToReturn;
-    if (findCommand.getNtoreturn()) {
-        // 'singleBatch' and ntoreturn mean the same as 'singleBatch' and limit, so perform the
-        // conversion.
-        if (findCommand.getSingleBatch()) {
-            int64_t newLimitValue;
-            if (overflow::add(*findCommand.getNtoreturn(),
-                              findCommand.getSkip().value_or(0),
-                              &newLimitValue)) {
-                return Status(ErrorCodes::Overflow,
-                              str::stream()
-                                  << "sum of ntoreturn and skip cannot be represented as a 64-bit "
-                                     "integer, ntoreturn: "
-                                  << *findCommand.getNtoreturn()
-                                  << ", skip: " << findCommand.getSkip().value_or(0));
-            }
-            newLimit = newLimitValue;
-        } else {
-            int64_t newNToReturnValue;
-            if (overflow::add(*findCommand.getNtoreturn(),
-                              findCommand.getSkip().value_or(0),
-                              &newNToReturnValue)) {
-                return Status(ErrorCodes::Overflow,
-                              str::stream()
-                                  << "sum of ntoreturn and skip cannot be represented as a 64-bit "
-                                     "integer, ntoreturn: "
-                                  << *findCommand.getNtoreturn()
-                                  << ", skip: " << findCommand.getSkip().value_or(0));
-            }
-            newNToReturn = newNToReturnValue;
-        }
-    }
-
     // If there is a sort other than $natural, we send a sortKey meta-projection to the remote node.
     BSONObj newProjection = findCommand.getProjection();
     if (!findCommand.getSort().isEmpty() &&
@@ -165,7 +132,6 @@ StatusWith<std::unique_ptr<FindCommandRequest>> transformQueryForShards(
     newQR->setProjection(newProjection);
     newQR->setSkip(boost::none);
     newQR->setLimit(newLimit);
-    newQR->setNtoreturn(newNToReturn);
 
     // Even if the client sends us singleBatch=true, we may need to retrieve
     // multiple batches from a shard in order to return the single requested batch to the client.
@@ -263,8 +229,7 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
     ClusterClientCursorParams params(
         query.nss(), APIParameters::get(opCtx), readPref, ReadConcernArgs::get(opCtx));
     params.originatingCommandObj = CurOp::get(opCtx)->opDescription().getOwned();
-    params.batchSize =
-        findCommand.getBatchSize() ? findCommand.getBatchSize() : findCommand.getNtoreturn();
+    params.batchSize = findCommand.getBatchSize();
     params.tailableMode = query_request_helper::getTailableMode(findCommand);
     params.isAllowPartialResults = findCommand.getAllowPartialResults();
     params.lsid = opCtx->getLogicalSessionId();
@@ -362,7 +327,7 @@ CursorId runQueryWithoutRetrying(OperationContext* opCtx,
     // results from the initial batches (that were obtained while establishing cursors) into
     // 'results'.
     while (!FindCommon::enoughForFirstBatch(findCommand, results->size())) {
-        auto next = uassertStatusOK(ccc->next(RouterExecStage::ExecContext::kInitialFind));
+        auto next = uassertStatusOK(ccc->next());
 
         if (next.isEOF()) {
             // We reached end-of-stream. If the cursor is not tailable, then we mark it as
@@ -465,8 +430,10 @@ Status setUpOperationContextStateForGetMore(OperationContext* opCtx,
     // that if the 'getMore' command itself has a 'comment' field, we give precedence to it.
     auto comment = cursor->getOriginatingCommand()["comment"];
     if (!opCtx->getComment() && comment) {
+        stdx::lock_guard<Client> lk(*opCtx->getClient());
         opCtx->setComment(comment.wrap());
     }
+
     if (cursor->isTailableAndAwaitData()) {
         // For tailable + awaitData cursors, the request may have indicated a maximum amount of time
         // to wait for new data. If not, default it to 1 second.  We track the deadline instead via
@@ -474,7 +441,7 @@ Status setUpOperationContextStateForGetMore(OperationContext* opCtx,
         auto timeout = Milliseconds{cmd.getMaxTimeMS().value_or(1000)};
         awaitDataState(opCtx).waitForInsertsDeadline =
             opCtx->getServiceContext()->getPreciseClockSource()->now() + timeout;
-
+        awaitDataState(opCtx).shouldWaitForInserts = true;
         invariant(cursor->setAwaitDataTimeout(timeout).isOK());
     } else if (cmd.getMaxTimeMS()) {
         return {ErrorCodes::BadValue,
@@ -496,6 +463,8 @@ CursorId ClusterFind::runQuery(OperationContext* opCtx,
                                const ReadPreferenceSetting& readPref,
                                std::vector<BSONObj>* results,
                                bool* partialResultsReturned) {
+    CurOp::get(opCtx)->debug().queryHash = canonical_query_encoder::computeHash(query.encodeKey());
+
     // If the user supplied a 'partialResultsReturned' out-parameter, default it to false here.
     if (partialResultsReturned) {
         *partialResultsReturned = false;
@@ -696,7 +665,7 @@ void validateTxnNumber(OperationContext* opCtx,
 void validateOperationSessionInfo(OperationContext* opCtx,
                                   int64_t cursorId,
                                   ClusterCursorManager::PinnedCursor* cursor) {
-    auto returnCursorGuard = makeGuard(
+    ScopeGuard returnCursorGuard(
         [cursor] { cursor->returnCursor(ClusterCursorManager::CursorState::NotExhausted); });
     validateLSID(opCtx, cursorId, *cursor);
     validateTxnNumber(opCtx, cursorId, *cursor);
@@ -790,14 +759,10 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
     }
 
     while (!FindCommon::enoughForGetMore(batchSize, batch.size())) {
-        auto context = batch.empty()
-            ? RouterExecStage::ExecContext::kGetMoreNoResultsYet
-            : RouterExecStage::ExecContext::kGetMoreWithAtLeastOneResultInBatch;
-
         StatusWith<ClusterQueryResult> next =
             Status{ErrorCodes::InternalError, "uninitialized cluster query result"};
         try {
-            next = pinnedCursor.getValue()->next(context);
+            next = pinnedCursor.getValue()->next();
         } catch (const ExceptionFor<ErrorCodes::CloseChangeStream>&) {
             // This exception is thrown when a $changeStream stage encounters an event
             // that invalidates the cursor. We should close the cursor and return without
@@ -809,8 +774,7 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
             // to the resume token of the invalidating event, and mark the cursor response as
             // invalidated. We always expect to have ExtraInfo for this error code.
             const auto extraInfo = ex.extraInfo<ChangeStreamInvalidationInfo>();
-            tassert(
-                5493707, "Missing ChangeStreamInvalidationInfo on exception", extraInfo != nullptr);
+            tassert(5493707, "Missing ChangeStreamInvalidationInfo on exception", extraInfo);
 
             postBatchResumeToken = extraInfo->getInvalidateResumeToken();
             cursorState = ClusterCursorManager::CursorState::Exhausted;
@@ -840,6 +804,9 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
             break;
         }
 
+        // As soon as we get a result, this operation no longer waits.
+        awaitDataState(opCtx).shouldWaitForInserts = false;
+
         // Add doc to the batch. Account for the space overhead associated with returning this doc
         // inside a BSON array.
         bytesBuffered +=
