@@ -33,7 +33,6 @@
 
 #include <vector>
 
-#include "mongo/bson/simple_bsonelement_comparator.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/field_ref_set.h"
 #include "mongo/db/hasher.h"
@@ -289,7 +288,7 @@ bool ShardKeyPattern::isShardKey(const BSONObj& shardKey) const {
 }
 
 bool ShardKeyPattern::isExtendedBy(const ShardKeyPattern& newShardKeyPattern) const {
-    return toBSON().isPrefixOf(newShardKeyPattern.toBSON(), SimpleBSONElementComparator::kInstance);
+    return toBSON().isFieldNamePrefixOf(newShardKeyPattern.toBSON());
 }
 
 BSONObj ShardKeyPattern::normalizeShardKey(const BSONObj& shardKey) const {
@@ -430,20 +429,6 @@ BSONObj ShardKeyPattern::extractShardKeyFromDocThrows(const BSONObj& doc) const
     return shardKey;
 }
 
-BSONObj ShardKeyPattern::extractShardKeyFromOplogEntry(const repl::OplogEntry& entry) const {
-    if (!entry.isCrudOpType()) {
-        return BSONObj();
-    }
-
-    auto objWithDocumentKey = entry.getObjectContainingDocumentKey();
-
-    if (!entry.isUpdateOrDelete()) {
-        return extractShardKeyFromDoc(objWithDocumentKey);
-    }
-
-    return extractShardKeyFromDocumentKey(objWithDocumentKey);
-}
-
 BSONObj ShardKeyPattern::emplaceMissingShardKeyValuesForDocument(const BSONObj doc) const {
     BSONObjBuilder fullDocBuilder(doc);
     for (const auto& skField : _keyPattern.toBSON()) {
