@@ -33,7 +33,6 @@
 #include "mongo/client/remote_command_targeter_factory_mock.h"
 #include "mongo/client/remote_command_targeter_mock.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/concurrency/exception_util.h"
 #include "mongo/db/logical_session_id.h"
 #include "mongo/db/vector_clock.h"
 #include "mongo/s/catalog/type_shard.h"
@@ -86,6 +85,7 @@ BSONObj expectInsertsReturnStaleVersionErrorsBase(const NamespaceString& nss,
     staleResponse.setN(0);
 
     auto epoch = OID::gen();
+    Timestamp timestamp(1);
 
     // Report a stale version error for each write in the batch.
     int i = 0;
@@ -94,8 +94,8 @@ BSONObj expectInsertsReturnStaleVersionErrorsBase(const NamespaceString& nss,
         error->setStatus({ErrorCodes::StaleShardVersion, ""});
         error->setErrInfo([&] {
             StaleConfigInfo sci(nss,
-                                ChunkVersion(1, 0, epoch, boost::none /* timestamp */),
-                                ChunkVersion(2, 0, epoch, boost::none /* timestamp */),
+                                ChunkVersion(1, 0, epoch, timestamp),
+                                ChunkVersion(2, 0, epoch, timestamp),
                                 ShardId(kShardName1));
             BSONObjBuilder builder;
             sci.serialize(&builder);
@@ -141,7 +141,7 @@ BSONObj expectInsertsReturnStaleDbVersionErrorsBase(const NamespaceString& nss,
         errorBuilder.append("index", i);
         errorBuilder.append("code", int(ErrorCodes::StaleDbVersion));
 
-        auto dbVersion = DatabaseVersion(UUID::gen());
+        auto dbVersion = DatabaseVersion(UUID::gen(), Timestamp(1, 1));
         errorBuilder.append("db", nss.db());
         errorBuilder.append("vReceived", dbVersion.toBSON());
         errorBuilder.append("vWanted", dbVersion.makeUpdated().toBSON());
@@ -282,7 +282,7 @@ public:
     MockNSTargeter singleShardNSTargeter{
         nss,
         {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, OID::gen(), boost::none /* timestamp */),
+                                 ChunkVersion(100, 200, OID::gen(), Timestamp(1, 1)),
                                  boost::none),
                    BSON("x" << MINKEY),
                    BSON("x" << MAXKEY))}};
@@ -345,6 +345,7 @@ TEST_F(BatchWriteExecTest, SingleUpdateTargetsShardWithLet) {
     updateRequest.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     class MultiShardTargeter : public MockNSTargeter {
     public:
@@ -353,24 +354,20 @@ TEST_F(BatchWriteExecTest, SingleUpdateTargetsShardWithLet) {
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << MINKEY),
-                   BSON("x" << 0)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << 0),
-                   BSON("x" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("x" << MINKEY),
+             BSON("x" << 0)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("x" << 0),
+             BSON("x" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -442,23 +439,19 @@ TEST_F(BatchWriteExecTest, SingleDeleteTargetsShardWithLet) {
     protected:
         std::vector<ShardEndpoint> targetDelete(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
-            return std::vector{
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+            return std::vector{ShardEndpoint(
+                kShardName2, ChunkVersion(101, 200, epoch, Timestamp(1, 1)), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
+        {MockRange(ShardEndpoint(
+                       kShardName1, ChunkVersion(100, 200, epoch, Timestamp(1, 1)), boost::none),
                    BSON("x" << MINKEY),
                    BSON("x" << 0)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
+         MockRange(ShardEndpoint(
+                       kShardName2, ChunkVersion(101, 200, epoch, Timestamp(1, 1)), boost::none),
                    BSON("x" << 0),
                    BSON("x" << MAXKEY))});
 
@@ -607,9 +600,9 @@ TEST_F(BatchWriteExecTest, MultiOpLargeUnorderedWithStaleShardVersionError) {
         ASSERT_EQ(kNumDocsToInsert, response.getN());
     });
 
-    expectInsertsReturnStaleVersionErrors({docsToInsert.begin(), docsToInsert.begin() + 63791});
-    expectInsertsReturnSuccess({docsToInsert.begin(), docsToInsert.begin() + 63791});
-    expectInsertsReturnSuccess({docsToInsert.begin() + 63791, docsToInsert.end()});
+    expectInsertsReturnStaleVersionErrors({docsToInsert.begin(), docsToInsert.begin() + 60133});
+    expectInsertsReturnSuccess({docsToInsert.begin(), docsToInsert.begin() + 60133});
+    expectInsertsReturnSuccess({docsToInsert.begin() + 60133, docsToInsert.end()});
 
     future.default_timed_get();
 }
@@ -630,6 +623,7 @@ TEST_F(BatchWriteExecTest, StaleShardVersionReturnedFromBatchWithSingleMultiWrit
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     class MultiShardTargeter : public MockNSTargeter {
     public:
@@ -638,27 +632,21 @@ TEST_F(BatchWriteExecTest, StaleShardVersionReturnedFromBatchWithSingleMultiWrit
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << MINKEY),
-                   BSON("x" << 0)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << 0),
-                   BSON("x" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("x" << MINKEY),
+             BSON("x" << 0)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("x" << 0),
+             BSON("x" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -690,12 +678,11 @@ TEST_F(BatchWriteExecTest, StaleShardVersionReturnedFromBatchWithSingleMultiWrit
             errDetail->setIndex(0);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName2)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName2)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -742,6 +729,7 @@ TEST_F(BatchWriteExecTest,
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     // This allows the batch to target each write operation
     // to a specific shard (kShardName2), to perform this test
@@ -752,27 +740,21 @@ TEST_F(BatchWriteExecTest,
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << MINKEY),
-                   BSON("sk" << 10)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << 10),
-                   BSON("sk" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("sk" << MINKEY),
+             BSON("sk" << 10)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("sk" << 10),
+             BSON("sk" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -804,12 +786,11 @@ TEST_F(BatchWriteExecTest,
             errDetail->setIndex(0);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName2)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName2)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -821,12 +802,11 @@ TEST_F(BatchWriteExecTest,
             errDetail->setIndex(1);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName2)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName2)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -872,6 +852,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     // This allows the batch to target each write operation
     // to a specific shard (kShardName2), to perform this test
@@ -882,27 +863,21 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << MINKEY),
-                   BSON("sk" << 10)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << 10),
-                   BSON("sk" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("sk" << MINKEY),
+             BSON("sk" << 10)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("sk" << 10),
+             BSON("sk" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -924,12 +899,11 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
             errDetail->setIndex(1);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName1)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName1)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -951,12 +925,11 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1Firs) {
             errDetail->setIndex(0);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName2)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName2)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -1013,6 +986,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     // This allows the batch to target each write operation
     // to a specific shard (kShardName2), to perform this test
@@ -1023,27 +997,21 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << MINKEY),
-                   BSON("sk" << 10)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << 10),
-                   BSON("sk" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("sk" << MINKEY),
+             BSON("sk" << 10)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("sk" << 10),
+             BSON("sk" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -1065,12 +1033,11 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
             errDetail->setIndex(1);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName1)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName1)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -1092,12 +1059,11 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromMultiWriteWithShard1FirstOK
             errDetail->setIndex(1);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName2)),
-                    "Stale shard version");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName2)),
+                                 "Stale shard version");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -1150,6 +1116,7 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     // This allows the batch to target each write operation to perform this test
     class MultiShardTargeter : public MockNSTargeter {
@@ -1160,17 +1127,13 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
                                                 const BatchItemRef& itemRef) const override {
             if (targetAll) {
                 return std::vector{
-                    ShardEndpoint(kShardName1,
-                                  ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                  boost::none),
-                    ShardEndpoint(kShardName2,
-                                  ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                  boost::none)};
+                    ShardEndpoint(
+                        kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                    ShardEndpoint(
+                        kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
             } else {
-                return std::vector{
-                    ShardEndpoint(kShardName2,
-                                  ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                  boost::none)};
+                return std::vector{ShardEndpoint(
+                    kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
             }
         }
 
@@ -1179,16 +1142,14 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << MINKEY),
-                   BSON("sk" << 10)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("sk" << 10),
-                   BSON("sk" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("sk" << MINKEY),
+             BSON("sk" << 10)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("sk" << 10),
+             BSON("sk" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -1211,12 +1172,11 @@ TEST_F(BatchWriteExecTest, RetryableErrorReturnedFromWriteWithShard1SSVShard2OK)
             errDetail->setIndex(0);
             errDetail->setStatus({ErrorCodes::StaleShardVersion, "Stale shard version"});
             errDetail->setErrInfo([&] {
-                Status ssvStatus(
-                    StaleConfigInfo(nss,
-                                    ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                    ChunkVersion(105, 200, epoch, boost::none /* timestamp */),
-                                    ShardId(kShardName1)),
-                    "Migration happened");
+                Status ssvStatus(StaleConfigInfo(nss,
+                                                 ChunkVersion(101, 200, epoch, timestamp),
+                                                 ChunkVersion(105, 200, epoch, timestamp),
+                                                 ShardId(kShardName1)),
+                                 "Migration happened");
                 BSONObjBuilder builder;
                 ssvStatus.serializeErrorToBSON(&builder);
                 return builder.obj();
@@ -1732,6 +1692,7 @@ TEST_F(BatchWriteExecTargeterErrorTest, TargetedFailedAndErrorResponse) {
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     class MultiShardTargeter : public MockNSTargeter {
     public:
@@ -1740,27 +1701,21 @@ TEST_F(BatchWriteExecTargeterErrorTest, TargetedFailedAndErrorResponse) {
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << MINKEY),
-                   BSON("x" << 0)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << 0),
-                   BSON("x" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("x" << MINKEY),
+             BSON("x" << 0)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("x" << 0),
+             BSON("x" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -1844,8 +1799,10 @@ public:
         _scopedSession.emplace(operationContext());
 
         auto txnRouter = TransactionRouter::get(operationContext());
-        txnRouter.beginOrContinueTxn(
-            operationContext(), kTxnNumber, TransactionRouter::TransactionActions::kStart);
+        txnRouter.beginOrContinueTxn(operationContext(),
+                                     kTxnNumber,
+                                     TransactionRouter::TransactionActions::kStart,
+                                     0 /* txnRetryCounter */);
         txnRouter.setDefaultAtClusterTime(operationContext());
     }
 
@@ -1878,6 +1835,7 @@ TEST_F(BatchWriteExecTransactionTargeterErrorTest, TargetedFailedAndErrorRespons
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     class MultiShardTargeter : public MockNSTargeter {
     public:
@@ -1886,27 +1844,21 @@ TEST_F(BatchWriteExecTransactionTargeterErrorTest, TargetedFailedAndErrorRespons
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << MINKEY),
-                   BSON("x" << 0)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << 0),
-                   BSON("x" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("x" << MINKEY),
+             BSON("x" << 0)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("x" << 0),
+             BSON("x" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -1995,8 +1947,10 @@ public:
         _scopedSession.emplace(operationContext());
 
         auto txnRouter = TransactionRouter::get(operationContext());
-        txnRouter.beginOrContinueTxn(
-            operationContext(), kTxnNumber, TransactionRouter::TransactionActions::kStart);
+        txnRouter.beginOrContinueTxn(operationContext(),
+                                     kTxnNumber,
+                                     TransactionRouter::TransactionActions::kStart,
+                                     0 /* txnRetryCounter */);
         txnRouter.setDefaultAtClusterTime(operationContext());
     }
 
@@ -2027,6 +1981,7 @@ TEST_F(BatchWriteExecTransactionMultiShardTest, TargetedSucceededAndErrorRespons
     request.setWriteConcern(BSONObj());
 
     const static auto epoch = OID::gen();
+    const static Timestamp timestamp(2);
 
     class MultiShardTargeter : public MockNSTargeter {
     public:
@@ -2035,27 +1990,21 @@ TEST_F(BatchWriteExecTransactionMultiShardTest, TargetedSucceededAndErrorRespons
         std::vector<ShardEndpoint> targetUpdate(OperationContext* opCtx,
                                                 const BatchItemRef& itemRef) const override {
             return std::vector{
-                ShardEndpoint(kShardName1,
-                              ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                              boost::none),
-                ShardEndpoint(kShardName2,
-                              ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                              boost::none)};
+                ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+                ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none)};
         }
     };
 
     MultiShardTargeter multiShardNSTargeter(
         nss,
-        {MockRange(ShardEndpoint(kShardName1,
-                                 ChunkVersion(100, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << MINKEY),
-                   BSON("x" << 0)),
-         MockRange(ShardEndpoint(kShardName2,
-                                 ChunkVersion(101, 200, epoch, boost::none /* timestamp */),
-                                 boost::none),
-                   BSON("x" << 0),
-                   BSON("x" << MAXKEY))});
+        {MockRange(
+             ShardEndpoint(kShardName1, ChunkVersion(100, 200, epoch, timestamp), boost::none),
+             BSON("x" << MINKEY),
+             BSON("x" << 0)),
+         MockRange(
+             ShardEndpoint(kShardName2, ChunkVersion(101, 200, epoch, timestamp), boost::none),
+             BSON("x" << 0),
+             BSON("x" << MAXKEY))});
 
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
@@ -2127,8 +2076,10 @@ public:
         _scopedSession.emplace(operationContext());
 
         auto txnRouter = TransactionRouter::get(operationContext());
-        txnRouter.beginOrContinueTxn(
-            operationContext(), kTxnNumber, TransactionRouter::TransactionActions::kStart);
+        txnRouter.beginOrContinueTxn(operationContext(),
+                                     kTxnNumber,
+                                     TransactionRouter::TransactionActions::kStart,
+                                     0 /* txnRetryCounter */);
         txnRouter.setDefaultAtClusterTime(operationContext());
     }
 
@@ -2301,9 +2252,11 @@ TEST_F(BatchWriteExecTransactionTest, ErrorInBatchSets_TransientTxnError) {
     auto future = launchAsync([&] {
         BatchedCommandResponse response;
         BatchWriteExecStats stats;
-        ASSERT_THROWS(BatchWriteExec::executeBatch(
-                          operationContext(), singleShardNSTargeter, request, &response, &stats),
-                      WriteConflictException);
+        ASSERT_THROWS_CODE(
+            BatchWriteExec::executeBatch(
+                operationContext(), singleShardNSTargeter, request, &response, &stats),
+            AssertionException,
+            ErrorCodes::WriteConflict);
     });
 
     expectInsertsReturnTransientTxnErrors({BSON("x" << 1), BSON("x" << 2)});
