@@ -612,28 +612,6 @@ assert = (function() {
         return res;
     };
 
-    assert.dropExceptionsWithCode = function(func, dropCodes, onDrop) {
-        if (typeof func !== "function") {
-            doassert('assert.dropExceptionsWithCode 1st argument must be a function');
-        }
-        if (typeof onDrop !== "function") {
-            doassert('assert.dropExceptionsWithCode 3rd argument must be a function');
-        }
-        if (!Array.isArray(dropCodes)) {
-            dropCodes = [dropCodes];
-        }
-
-        try {
-            return func();
-        } catch (e) {
-            if (dropCodes.some((ec) => e.code === ec)) {
-                return onDrop(e);
-            } else {
-                throw e;
-            }
-        }
-    };
-
     assert.throws.automsg = function(func, params) {
         if (arguments.length === 1)
             params = [];
@@ -1107,65 +1085,6 @@ assert = (function() {
         doassert(_buildAssertionMessage(msg, msgPrefix));
     };
 
-    assert.gleOK = function(res, msg) {
-        var errMsg = null;
-
-        if (!res) {
-            errMsg = "missing first argument, no response to check";
-        } else if (!res.ok) {
-            errMsg = "getLastError failed: " + tojson(res);
-        } else if ('code' in res || 'errmsg' in res || ('err' in res && res['err'] != null)) {
-            errMsg = "write or write concern failed: " + tojson(res);
-        }
-
-        if (errMsg) {
-            doassert(_buildAssertionMessage(msg, errMsg), res);
-        }
-
-        return res;
-    };
-
-    assert.gleSuccess = function(dbOrGLEDoc, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (gle.err) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(msg, "getLastError not null: " + tojson(gle)), gle);
-        }
-        return gle;
-    };
-
-    assert.gleError = function(dbOrGLEDoc, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (!gle.err) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(msg, "getLastError is null: " + tojson(gle)));
-        }
-    };
-
-    assert.gleErrorCode = function(dbOrGLEDoc, code, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (!gle.err || gle.code != code) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(
-                msg,
-                "getLastError is null or has code other than \"" + code + "\": " + tojson(gle)));
-        }
-    };
-
-    assert.gleErrorRegex = function(dbOrGLEDoc, regex, msg) {
-        var gle = dbOrGLEDoc instanceof DB ? dbOrGLEDoc.getLastErrorObj() : dbOrGLEDoc;
-        if (!gle.err || !regex.test(gle.err)) {
-            if (typeof (msg) == "function")
-                msg = msg(gle);
-            doassert(_buildAssertionMessage(
-                msg,
-                "getLastError is null or doesn't match regex (" + regex + "): " + tojson(gle)));
-        }
-    };
-
     assert.includes = function(haystack, needle, msg) {
         if (!haystack.includes(needle)) {
             var assertMsg = "string [" + haystack + "] does not include [" + needle + "]";
