@@ -40,7 +40,7 @@
 #include "mongo/client/dbclient_cursor.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/cursor_response.h"
-#include "mongo/db/query/getmore_request.h"
+#include "mongo/db/query/getmore_command_gen.h"
 #include "mongo/db/query/query_request_helper.h"
 #include "mongo/logv2/log.h"
 #include "mongo/scripting/bson_template_evaluator.h"
@@ -133,16 +133,6 @@ BSONObj fixQuery(const BSONObj& obj, BsonTemplateEvaluator& btl) {
     return b.obj();
 }
 
-/**
- * Adds a '$orderby' to the query document. Useful when running on the legacy reads path.
- */
-BSONObj makeQueryLegacyCompatible(const BSONObj& query, const BSONObj& sortSpec) {
-    BSONObjBuilder bob;
-    bob.append("$query", query);
-    bob.append("$orderby", sortSpec);
-    return bob.obj();
-}
-
 bool runCommandWithSession(DBClientBase* conn,
                            const std::string& dbname,
                            const BSONObj& cmdObj,
@@ -269,21 +259,16 @@ int runQueryWithReadCommands(DBClientBase* conn,
     while (cursorResponse.getCursorId() != 0) {
         sleepFor(delayBeforeGetMore);
 
-        GetMoreRequest getMoreRequest(
-            findCommand->getNamespaceOrUUID().nss().value_or(NamespaceString()),
+        GetMoreCommandRequest getMoreRequest(
             cursorResponse.getCursorId(),
-            findCommand->getBatchSize() ? boost::optional<std::int64_t>(static_cast<std::int64_t>(
-                                              *findCommand->getBatchSize()))
-                                        : boost::none,
-            boost::none,   // maxTimeMS
-            boost::none,   // term
-            boost::none);  // lastKnownCommittedOpTime
+            findCommand->getNamespaceOrUUID().nss().value_or(NamespaceString()).coll().toString());
+        getMoreRequest.setBatchSize(findCommand->getBatchSize());
         BSONObj getMoreCommandResult;
         uassert(ErrorCodes::CommandFailed,
                 str::stream() << "getMore command failed; reply was: " << getMoreCommandResult,
                 runCommandWithSession(conn,
                                       dbName,
-                                      getMoreRequest.toBSON(),
+                                      getMoreRequest.toBSON({}),
                                       // read command with txnNumber implies performing reads in a
                                       // multi-statement transaction
                                       txnNumber ? kMultiStatementTransactionOption : kNoOptions,
@@ -384,7 +369,6 @@ void BenchRunConfig::initializeToDefaults() {
 
     parallel = 1;
     seconds = 1.0;
-    hideResults = true;
     handleErrors = false;
     hideErrors = false;
 
@@ -551,8 +535,6 @@ BenchRunOp opFromBson(const BSONObj& op) {
                         (opType == "find" || (opType == "update") || (opType == "delete") ||
                          (opType == "remove")));
             myOp.query = arg.Obj();
-        } else if (name == "safe") {
-            myOp.safe = arg.trueValue();
         } else if (name == "skip") {
             uassert(ErrorCodes::BadValue,
                     str::stream() << "Field 'skip' should be a number, instead it's type: "
@@ -575,8 +557,6 @@ BenchRunOp opFromBson(const BSONObj& op) {
             myOp.sort = arg.Obj();
         } else if (name == "showError") {
             myOp.showError = arg.trueValue();
-        } else if (name == "showResult") {
-            myOp.showResult = arg.trueValue();
         } else if (name == "target") {
             uassert(ErrorCodes::BadValue,
                     str::stream() << "Field 'target' should be a string. It's type: "
@@ -674,8 +654,17 @@ BenchRunOp opFromBson(const BSONObj& op) {
         }
     }
 
+    uassert(5751400,
+            "readCmd: true must be specified for read ops",
+            !isReadOp(myOp.op) || myOp.useReadCmd);
+    uassert(5751401,
+            "writeCmd: true must be specified for write ops",
+            !isWriteOp(myOp.op) || myOp.useWriteCmd);
+    uassert(5751402, "Exhaust queries are not supported", !(myOp.options & QueryOption_Exhaust));
+
     uassert(34395, "Benchrun op has an zero length ns", !myOp.ns.empty());
     uassert(34396, "Benchrun op doesn't have an optype set", myOp.op != OpType::NONE);
+
     return myOp;
 }
 
@@ -750,8 +739,6 @@ void BenchRunConfig::initializeFromBson(const BSONObj& args) {
                                   << typeName(arg.type()),
                     arg.isNumber());
             delayMillisOnFailedOperation = Milliseconds(arg.numberInt());
-        } else if (name == "hideResults") {
-            hideResults = arg.trueValue();
         } else if (name == "handleErrors") {
             handleErrors = arg.trueValue();
         } else if (name == "hideErrors") {
@@ -1005,16 +992,10 @@ void BenchRunWorker::generateLoadOnConnection(DBClientBase* conn) {
                 ++opState.stats->errCount;
             }
 
-            if (++count % 100 == 0 && !op.useWriteCmd) {
-                conn->getLastError();
-            }
-
             if (op.delay > 0)
                 sleepmillis(op.delay);
         }
     }
-
-    conn->getLastError();
 }
 
 void BenchRunOp::executeOnce(DBClientBase* conn,
@@ -1041,42 +1022,32 @@ void BenchRunOp::executeOnce(DBClientBase* conn,
         case OpType::FINDONE: {
             BSONObj fixedQuery = fixQuery(this->query, *state->bsonTemplateEvaluator);
             BSONObj result;
-            if (this->useReadCmd) {
-                auto findCommand = std::make_unique<FindCommandRequest>(NamespaceString(this->ns));
-                findCommand->setFilter(fixedQuery);
-                findCommand->setProjection(this->projection);
-                findCommand->setLimit(1LL);
-                findCommand->setSingleBatch(true);
-                findCommand->setSort(this->sort);
-                if (config.useSnapshotReads) {
-                    findCommand->setReadConcern(readConcernSnapshot);
-                }
-                invariant(query_request_helper::validateFindCommandRequest(*findCommand));
-
-                BenchRunEventTrace _bret(&state->stats->findOneCounter);
-                boost::optional<TxnNumber> txnNumberForOp;
-                if (config.useSnapshotReads) {
-                    ++state->txnNumber;
-                    txnNumberForOp = state->txnNumber;
-                    state->inProgressMultiStatementTxn = true;
-                }
-                runQueryWithReadCommands(conn,
-                                         lsid,
-                                         txnNumberForOp,
-                                         std::move(findCommand),
-                                         Milliseconds(0),
-                                         readPrefObj,
-                                         &result);
-            } else {
-                if (!this->sort.isEmpty()) {
-                    fixedQuery = makeQueryLegacyCompatible(std::move(fixedQuery), this->sort);
-                }
-                BenchRunEventTrace _bret(&state->stats->findOneCounter);
-                result = conn->findOne(
-                    this->ns, fixedQuery, nullptr, DBClientCursor::QueryOptionLocal_forceOpQuery);
+            auto findCommand = std::make_unique<FindCommandRequest>(NamespaceString(this->ns));
+            findCommand->setFilter(fixedQuery);
+            findCommand->setProjection(this->projection);
+            findCommand->setLimit(1LL);
+            findCommand->setSingleBatch(true);
+            findCommand->setSort(this->sort);
+            if (config.useSnapshotReads) {
+                findCommand->setReadConcern(readConcernSnapshot);
             }
-            if (!config.hideResults || this->showResult)
-                LOGV2_INFO(22796, "Result from benchRun thread [findOne]", "result"_attr = result);
+            invariant(query_request_helper::validateFindCommandRequest(*findCommand));
+
+            BenchRunEventTrace _bret(&state->stats->findOneCounter);
+            boost::optional<TxnNumber> txnNumberForOp;
+            if (config.useSnapshotReads) {
+                ++state->txnNumber;
+                txnNumberForOp = state->txnNumber;
+                state->inProgressMultiStatementTxn = true;
+            }
+            runQueryWithReadCommands(conn,
+                                     lsid,
+                                     txnNumberForOp,
+                                     std::move(findCommand),
+                                     Milliseconds(0),
+                                     readPrefObj,
+                                     &result);
+            LOGV2_DEBUG(22796, 5, "Result from benchRun thread [findOne]", "result"_attr = result);
         } break;
         case OpType::COMMAND: {
             bool ok;
@@ -1099,19 +1070,15 @@ void BenchRunOp::executeOnce(DBClientBase* conn,
                 auto cursorResponse = uassertStatusOK(CursorResponse::parseFromBSON(result));
                 int count = cursorResponse.getBatch().size();
                 while (cursorResponse.getCursorId() != 0) {
-                    GetMoreRequest getMoreRequest(cursorResponse.getNSS(),
-                                                  cursorResponse.getCursorId(),
-                                                  boost::none,   // batchSize
-                                                  boost::none,   // maxTimeMS
-                                                  boost::none,   // term
-                                                  boost::none);  // lastKnownCommittedOpTime
+                    GetMoreCommandRequest getMoreRequest(cursorResponse.getCursorId(),
+                                                         cursorResponse.getNSS().coll().toString());
                     BSONObj getMoreCommandResult;
                     uassert(ErrorCodes::CommandFailed,
                             str::stream()
                                 << "getMore command failed; reply was: " << getMoreCommandResult,
                             runCommandWithSession(conn,
                                                   this->ns,
-                                                  getMoreRequest.toBSON(),
+                                                  getMoreRequest.toBSON({}),
                                                   kNoOptions,
                                                   lsid,
                                                   &getMoreCommandResult));
@@ -1128,88 +1095,59 @@ void BenchRunOp::executeOnce(DBClientBase* conn,
 
             BSONObj fixedQuery = fixQuery(this->query, *state->bsonTemplateEvaluator);
 
-            if (this->useReadCmd) {
-                uassert(28824,
-                        "cannot use 'options' in combination with read commands",
-                        !this->options);
+            uassert(
+                28824, "cannot use 'options' in combination with read commands", !this->options);
 
-                auto findCommand = std::make_unique<FindCommandRequest>(NamespaceString(this->ns));
-                findCommand->setFilter(fixedQuery);
-                findCommand->setProjection(this->projection);
-                if (this->skip) {
-                    findCommand->setSkip(this->skip);
-                }
-                if (this->limit) {
-                    findCommand->setLimit(this->limit);
-                }
-                if (this->batchSize) {
-                    findCommand->setBatchSize(this->batchSize);
-                }
-                if (!this->sort.isEmpty()) {
-                    findCommand->setSort(this->sort);
-                }
-                BSONObjBuilder readConcernBuilder;
-                if (config.useSnapshotReads) {
-                    readConcernBuilder.append("level", "snapshot");
-                }
-                if (this->useAClusterTimeWithinPastSeconds > 0) {
-                    invariant(config.useSnapshotReads);
-                    // Get a random cluster time between the latest time and
-                    // 'useAClusterTimeWithinPastSeconds' in the past.
-                    Timestamp atClusterTime = getAClusterTimeSecondsInThePast(
-                        conn, state->rng->nextInt32(this->useAClusterTimeWithinPastSeconds));
-                    readConcernBuilder.append("atClusterTime", atClusterTime);
-                }
-                findCommand->setReadConcern(readConcernBuilder.obj());
-
-                invariant(query_request_helper::validateFindCommandRequest(*findCommand));
-
-                BenchRunEventTrace _bret(&state->stats->queryCounter);
-                boost::optional<TxnNumber> txnNumberForOp;
-                if (config.useSnapshotReads) {
-                    ++state->txnNumber;
-                    txnNumberForOp = state->txnNumber;
-                    state->inProgressMultiStatementTxn = true;
-                }
-
-                int delayBeforeGetMore = this->maxRandomMillisecondDelayBeforeGetMore
-                    ? state->rng->nextInt32(this->maxRandomMillisecondDelayBeforeGetMore)
-                    : 0;
-
-                count = runQueryWithReadCommands(conn,
-                                                 lsid,
-                                                 txnNumberForOp,
-                                                 std::move(findCommand),
-                                                 Milliseconds(delayBeforeGetMore),
-                                                 readPrefObj,
-                                                 nullptr);
-            } else {
-                if (!this->sort.isEmpty()) {
-                    fixedQuery = makeQueryLegacyCompatible(std::move(fixedQuery), this->sort);
-                }
-                // Use special query function for exhaust query option.
-                if (this->options & QueryOption_Exhaust) {
-                    BenchRunEventTrace _bret(&state->stats->queryCounter);
-                    std::function<void(const BSONObj&)> castedDoNothing(doNothing);
-                    count =
-                        conn->query(castedDoNothing,
-                                    NamespaceString(this->ns),
-                                    fixedQuery,
-                                    &this->projection,
-                                    this->options | DBClientCursor::QueryOptionLocal_forceOpQuery);
-                } else {
-                    BenchRunEventTrace _bret(&state->stats->queryCounter);
-                    std::unique_ptr<DBClientCursor> cursor(
-                        conn->query(NamespaceString(this->ns),
-                                    fixedQuery,
-                                    this->limit,
-                                    this->skip,
-                                    &this->projection,
-                                    this->options | DBClientCursor::QueryOptionLocal_forceOpQuery,
-                                    this->batchSize));
-                    count = cursor->itcount();
-                }
+            auto findCommand = std::make_unique<FindCommandRequest>(NamespaceString(this->ns));
+            findCommand->setFilter(fixedQuery);
+            findCommand->setProjection(this->projection);
+            if (this->skip) {
+                findCommand->setSkip(this->skip);
             }
+            if (this->limit) {
+                findCommand->setLimit(this->limit);
+            }
+            if (this->batchSize) {
+                findCommand->setBatchSize(this->batchSize);
+            }
+            if (!this->sort.isEmpty()) {
+                findCommand->setSort(this->sort);
+            }
+            BSONObjBuilder readConcernBuilder;
+            if (config.useSnapshotReads) {
+                readConcernBuilder.append("level", "snapshot");
+            }
+            if (this->useAClusterTimeWithinPastSeconds > 0) {
+                invariant(config.useSnapshotReads);
+                // Get a random cluster time between the latest time and
+                // 'useAClusterTimeWithinPastSeconds' in the past.
+                Timestamp atClusterTime = getAClusterTimeSecondsInThePast(
+                    conn, state->rng->nextInt32(this->useAClusterTimeWithinPastSeconds));
+                readConcernBuilder.append("atClusterTime", atClusterTime);
+            }
+            findCommand->setReadConcern(readConcernBuilder.obj());
+
+            invariant(query_request_helper::validateFindCommandRequest(*findCommand));
+
+            BenchRunEventTrace _bret(&state->stats->queryCounter);
+            boost::optional<TxnNumber> txnNumberForOp;
+            if (config.useSnapshotReads) {
+                ++state->txnNumber;
+                txnNumberForOp = state->txnNumber;
+                state->inProgressMultiStatementTxn = true;
+            }
+
+            int delayBeforeGetMore = this->maxRandomMillisecondDelayBeforeGetMore
+                ? state->rng->nextInt32(this->maxRandomMillisecondDelayBeforeGetMore)
+                : 0;
+
+            count = runQueryWithReadCommands(conn,
+                                             lsid,
+                                             txnNumberForOp,
+                                             std::move(findCommand),
+                                             Milliseconds(delayBeforeGetMore),
+                                             readPrefObj,
+                                             nullptr);
 
             if (this->expected >= 0 && count != this->expected) {
                 LOGV2_INFO(22797,
@@ -1220,9 +1158,7 @@ void BenchRunOp::executeOnce(DBClientBase* conn,
                            "got"_attr = count);
                 verify(false);
             }
-
-            if (!config.hideResults || this->showResult)
-                LOGV2_INFO(22798, "Result from benchRun thread [query]", "count"_attr = count);
+            LOGV2_DEBUG(22798, 5, "Result from benchRun thread [query]", "count"_attr = count);
         } break;
         case OpType::UPDATE: {
             BSONObj result;
@@ -1230,81 +1166,59 @@ void BenchRunOp::executeOnce(DBClientBase* conn,
                 BenchRunEventTrace _bret(&state->stats->updateCounter);
                 BSONObj query = fixQuery(this->query, *state->bsonTemplateEvaluator);
 
-                if (this->useWriteCmd) {
-                    BSONObjBuilder builder;
-                    builder.append("update", nsToCollectionSubstring(this->ns));
-                    BSONArrayBuilder updateArray(builder.subarrayStart("updates"));
-                    {
-                        BSONObjBuilder singleUpdate;
-                        singleUpdate.append("q", query);
-                        switch (this->update.type()) {
-                            case write_ops::UpdateModification::Type::kClassic: {
-                                singleUpdate.append("u",
-                                                    fixQuery(this->update.getUpdateClassic(),
-                                                             *state->bsonTemplateEvaluator));
-                                break;
-                            }
-                            case write_ops::UpdateModification::Type::kPipeline: {
-                                BSONArrayBuilder pipelineBuilder(singleUpdate.subarrayStart("u"));
-                                for (auto&& stage : this->update.getUpdatePipeline()) {
-                                    pipelineBuilder.append(
-                                        fixQuery(stage, *state->bsonTemplateEvaluator));
-                                }
-                                pipelineBuilder.doneFast();
-                                break;
-                            }
-                            case write_ops::UpdateModification::Type::kDelta:
-                                // It is not possible to run a delta update directly from a client.
-                                // Delta updates are only executed on secondaries as part of oplog
-                                // application.
-                                MONGO_UNREACHABLE;
+                BSONObjBuilder builder;
+                builder.append("update", nsToCollectionSubstring(this->ns));
+                BSONArrayBuilder updateArray(builder.subarrayStart("updates"));
+                {
+                    BSONObjBuilder singleUpdate;
+                    singleUpdate.append("q", query);
+                    switch (this->update.type()) {
+                        case write_ops::UpdateModification::Type::kClassic: {
+                            singleUpdate.append("u",
+                                                fixQuery(this->update.getUpdateClassic(),
+                                                         *state->bsonTemplateEvaluator));
+                            break;
                         }
-                        singleUpdate.append("multi", this->multi);
-                        singleUpdate.append("upsert", this->upsert);
-                        updateArray.append(singleUpdate.done());
+                        case write_ops::UpdateModification::Type::kPipeline: {
+                            BSONArrayBuilder pipelineBuilder(singleUpdate.subarrayStart("u"));
+                            for (auto&& stage : this->update.getUpdatePipeline()) {
+                                pipelineBuilder.append(
+                                    fixQuery(stage, *state->bsonTemplateEvaluator));
+                            }
+                            pipelineBuilder.doneFast();
+                            break;
+                        }
+                        case write_ops::UpdateModification::Type::kDelta:
+                            // It is not possible to run a delta update directly from a client.
+                            // Delta updates are only executed on secondaries as part of oplog
+                            // application.
+                            MONGO_UNREACHABLE;
+                        case write_ops::UpdateModification::Type::kTransform:
+                            // It is not possible to run a transform update directly from a client.
+                            MONGO_UNREACHABLE;
                     }
-                    updateArray.doneFast();
-                    builder.append("writeConcern", this->writeConcern);
-
-                    boost::optional<TxnNumber> txnNumberForOp;
-                    if (config.useIdempotentWrites) {
-                        ++state->txnNumber;
-                        txnNumberForOp = state->txnNumber;
-                    }
-                    runCommandWithSession(conn,
-                                          nsToDatabaseSubstring(this->ns).toString(),
-                                          builder.done(),
-                                          kNoOptions,
-                                          lsid,
-                                          txnNumberForOp,
-                                          &result);
-                } else {
-                    uassert(
-                        30015,
-                        "cannot use legacy write protocol for anything but classic style updates",
-                        this->update.type() == write_ops::UpdateModification::Type::kClassic);
-                    auto toSend = makeUpdateMessage(
-                        this->ns,
-                        query,
-                        fixQuery(this->update.getUpdateClassic(), *state->bsonTemplateEvaluator),
-                        (this->upsert ? UpdateOption_Upsert : 0) |
-                            (this->multi ? UpdateOption_Multi : 0));
-                    conn->say(toSend);
-                    if (this->safe)
-                        result = conn->getLastErrorDetailed();
+                    singleUpdate.append("multi", this->multi);
+                    singleUpdate.append("upsert", this->upsert);
+                    updateArray.append(singleUpdate.done());
                 }
-            }
+                updateArray.doneFast();
+                builder.append("writeConcern", this->writeConcern);
 
-            if (this->safe) {
-                if (!config.hideResults || this->showResult)
-                    LOGV2_INFO(
-                        22799, "Result from benchRun thread [safe update]", "result"_attr = result);
-
-                if (!result["err"].eoo() && result["err"].type() == String &&
-                    (config.throwGLE || this->throwGLE))
-                    uasserted(result["code"].eoo() ? 0 : result["code"].Int(),
-                              (std::string) "From benchRun GLE" + causedBy(result["err"].String()));
+                boost::optional<TxnNumber> txnNumberForOp;
+                if (config.useIdempotentWrites) {
+                    ++state->txnNumber;
+                    txnNumberForOp = state->txnNumber;
+                }
+                runCommandWithSession(conn,
+                                      nsToDatabaseSubstring(this->ns).toString(),
+                                      builder.done(),
+                                      kNoOptions,
+                                      lsid,
+                                      txnNumberForOp,
+                                      &result);
             }
+            LOGV2_DEBUG(
+                22799, 5, "Result from benchRun thread [safe update]", "result"_attr = result);
         } break;
         case OpType::INSERT: {
             BSONObj result;
@@ -1312,111 +1226,65 @@ void BenchRunOp::executeOnce(DBClientBase* conn,
                 BenchRunEventTrace _bret(&state->stats->insertCounter);
 
                 BSONObj insertDoc;
-                if (this->useWriteCmd) {
-                    BSONObjBuilder builder;
-                    builder.append("insert", nsToCollectionSubstring(this->ns));
-                    BSONArrayBuilder docBuilder(builder.subarrayStart("documents"));
-                    if (this->isDocAnArray) {
-                        for (const auto& element : this->doc) {
-                            insertDoc = fixQuery(element.Obj(), *state->bsonTemplateEvaluator);
-                            docBuilder.append(insertDoc);
-                        }
-                    } else {
-                        insertDoc = fixQuery(this->doc, *state->bsonTemplateEvaluator);
+                BSONObjBuilder builder;
+                builder.append("insert", nsToCollectionSubstring(this->ns));
+                BSONArrayBuilder docBuilder(builder.subarrayStart("documents"));
+                if (this->isDocAnArray) {
+                    for (const auto& element : this->doc) {
+                        insertDoc = fixQuery(element.Obj(), *state->bsonTemplateEvaluator);
                         docBuilder.append(insertDoc);
                     }
-                    docBuilder.done();
-                    builder.append("writeConcern", this->writeConcern);
-
-                    boost::optional<TxnNumber> txnNumberForOp;
-                    if (config.useIdempotentWrites) {
-                        ++state->txnNumber;
-                        txnNumberForOp = state->txnNumber;
-                    }
-                    runCommandWithSession(conn,
-                                          nsToDatabaseSubstring(this->ns).toString(),
-                                          builder.done(),
-                                          kNoOptions,
-                                          lsid,
-                                          txnNumberForOp,
-                                          &result);
                 } else {
-                    std::vector<BSONObj> insertArray;
-                    if (this->isDocAnArray) {
-                        for (const auto& element : this->doc) {
-                            BSONObj e = fixQuery(element.Obj(), *state->bsonTemplateEvaluator);
-                            insertArray.push_back(e);
-                        }
-                    } else {
-                        insertArray.push_back(fixQuery(this->doc, *state->bsonTemplateEvaluator));
-                    }
-
-                    auto toSend =
-                        makeInsertMessage(this->ns, insertArray.data(), insertArray.size());
-                    conn->say(toSend);
-
-                    if (this->safe)
-                        result = conn->getLastErrorDetailed();
+                    insertDoc = fixQuery(this->doc, *state->bsonTemplateEvaluator);
+                    docBuilder.append(insertDoc);
                 }
-            }
+                docBuilder.done();
+                builder.append("writeConcern", this->writeConcern);
 
-            if (this->safe) {
-                if (!config.hideResults || this->showResult)
-                    LOGV2_INFO(
-                        22800, "Result from benchRun thread [safe insert]", "result"_attr = result);
-
-                if (!result["err"].eoo() && result["err"].type() == String &&
-                    (config.throwGLE || this->throwGLE))
-                    uasserted(result["code"].eoo() ? 0 : result["code"].Int(),
-                              (std::string) "From benchRun GLE" + causedBy(result["err"].String()));
+                boost::optional<TxnNumber> txnNumberForOp;
+                if (config.useIdempotentWrites) {
+                    ++state->txnNumber;
+                    txnNumberForOp = state->txnNumber;
+                }
+                runCommandWithSession(conn,
+                                      nsToDatabaseSubstring(this->ns).toString(),
+                                      builder.done(),
+                                      kNoOptions,
+                                      lsid,
+                                      txnNumberForOp,
+                                      &result);
             }
+            LOGV2_DEBUG(
+                22800, 5, "Result from benchRun thread [safe insert]", "result"_attr = result);
         } break;
         case OpType::REMOVE: {
             BSONObj result;
             {
                 BenchRunEventTrace _bret(&state->stats->deleteCounter);
                 BSONObj predicate = fixQuery(this->query, *state->bsonTemplateEvaluator);
-                if (this->useWriteCmd) {
-                    BSONObjBuilder builder;
-                    builder.append("delete", nsToCollectionSubstring(this->ns));
-                    BSONArrayBuilder docBuilder(builder.subarrayStart("deletes"));
-                    int limit = (this->multi == true) ? 0 : 1;
-                    docBuilder.append(BSON("q" << predicate << "limit" << limit));
-                    docBuilder.done();
-                    builder.append("writeConcern", this->writeConcern);
+                BSONObjBuilder builder;
+                builder.append("delete", nsToCollectionSubstring(this->ns));
+                BSONArrayBuilder docBuilder(builder.subarrayStart("deletes"));
+                int limit = (this->multi == true) ? 0 : 1;
+                docBuilder.append(BSON("q" << predicate << "limit" << limit));
+                docBuilder.done();
+                builder.append("writeConcern", this->writeConcern);
 
-                    boost::optional<TxnNumber> txnNumberForOp;
-                    if (config.useIdempotentWrites) {
-                        ++state->txnNumber;
-                        txnNumberForOp = state->txnNumber;
-                    }
-                    runCommandWithSession(conn,
-                                          nsToDatabaseSubstring(this->ns).toString(),
-                                          builder.done(),
-                                          kNoOptions,
-                                          lsid,
-                                          txnNumberForOp,
-                                          &result);
-                } else {
-                    auto toSend = makeRemoveMessage(
-                        this->ns, predicate, this->multi ? 0 : RemoveOption_JustOne);
-                    conn->say(toSend);
-                    if (this->safe)
-                        result = conn->getLastErrorDetailed();
+                boost::optional<TxnNumber> txnNumberForOp;
+                if (config.useIdempotentWrites) {
+                    ++state->txnNumber;
+                    txnNumberForOp = state->txnNumber;
                 }
+                runCommandWithSession(conn,
+                                      nsToDatabaseSubstring(this->ns).toString(),
+                                      builder.done(),
+                                      kNoOptions,
+                                      lsid,
+                                      txnNumberForOp,
+                                      &result);
             }
-
-            if (this->safe) {
-                if (!config.hideResults || this->showResult)
-                    LOGV2_INFO(
-                        22801, "Result from benchRun thread [safe remove]", "result"_attr = result);
-
-                if (!result["err"].eoo() && result["err"].type() == String &&
-                    (config.throwGLE || this->throwGLE))
-                    uasserted(result["code"].eoo() ? 0 : result["code"].Int(),
-                              (std::string) "From benchRun GLE " +
-                                  causedBy(result["err"].String()));
-            }
+            LOGV2_DEBUG(
+                22801, 5, "Result from benchRun thread [safe remove]", "result"_attr = result);
         } break;
         case OpType::CREATEINDEX:
             conn->createIndex(this->ns, this->key);
