@@ -33,7 +33,6 @@
 
 #include "mongo/transport/transport_layer_asio.h"
 
-#include <fmt/format.h>
 #include <fstream>
 
 #include <asio.hpp>
@@ -51,16 +50,13 @@
 #include "mongo/transport/asio_utils.h"
 #include "mongo/transport/service_entry_point.h"
 #include "mongo/transport/transport_options_gen.h"
-#include "mongo/util/clock_source.h"
 #include "mongo/util/errno_util.h"
-#include "mongo/util/executor_stats.h"
 #include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/net/sockaddr.h"
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/options_parser/startup_options.h"
-#include "mongo/util/strong_weak_finish_line.h"
 
 #ifdef MONGO_CONFIG_SSL
 #include "mongo/util/net/ssl.hpp"
@@ -103,7 +99,6 @@ boost::optional<Status> maybeTcpFastOpenStatus;
 }  // namespace
 
 MONGO_FAIL_POINT_DEFINE(transportLayerASIOasyncConnectTimesOut);
-MONGO_FAIL_POINT_DEFINE(transportLayerASIOhangBeforeAccept);
 
 #ifdef MONGO_CONFIG_SSL
 SSLConnectionContext::~SSLConnectionContext() = default;
@@ -184,7 +179,7 @@ private:
 
 class TransportLayerASIO::ASIOReactor final : public Reactor {
 public:
-    ASIOReactor() : _clkSource(this), _stats(&_clkSource), _ioContext() {}
+    ASIOReactor() : _ioContext() {}
 
     void run() noexcept override {
         ThreadIdGuard threadIdGuard(this);
@@ -220,12 +215,11 @@ public:
     }
 
     void schedule(Task task) override {
-        asio::post(_ioContext, [task = _stats.wrapTask(std::move(task))] { task(Status::OK()); });
+        asio::post(_ioContext, [task = std::move(task)] { task(Status::OK()); });
     }
 
     void dispatch(Task task) override {
-        asio::dispatch(_ioContext,
-                       [task = _stats.wrapTask(std::move(task))] { task(Status::OK()); });
+        asio::dispatch(_ioContext, [task = std::move(task)] { task(Status::OK()); });
     }
 
     bool onReactorThread() const override {
@@ -236,29 +230,7 @@ public:
         return _ioContext;
     }
 
-    void appendStats(BSONObjBuilder& bob) const override {
-        _stats.serialize(&bob);
-    }
-
 private:
-    // Provides `ClockSource` API for the reactor's clock source.
-    class ReactorClockSource final : public ClockSource {
-    public:
-        explicit ReactorClockSource(ASIOReactor* reactor) : _reactor(reactor) {}
-        ~ReactorClockSource() = default;
-
-        Milliseconds getPrecision() override {
-            MONGO_UNREACHABLE;
-        }
-
-        Date_t now() override {
-            return _reactor->now();
-        }
-
-    private:
-        ASIOReactor* const _reactor;
-    };
-
     class ThreadIdGuard {
     public:
         ThreadIdGuard(TransportLayerASIO::ASIOReactor* reactor) {
@@ -274,20 +246,14 @@ private:
 
     static thread_local ASIOReactor* _reactorForThread;
 
-    ReactorClockSource _clkSource;
-
-    ExecutorStats _stats;
-
     asio::io_context _ioContext;
 };
 
 thread_local TransportLayerASIO::ASIOReactor* TransportLayerASIO::ASIOReactor::_reactorForThread =
     nullptr;
 
-TransportLayerASIO::Options::Options(const ServerGlobalParams* params,
-                                     boost::optional<int> loadBalancerPort)
+TransportLayerASIO::Options::Options(const ServerGlobalParams* params)
     : port(params->port),
-      loadBalancerPort(loadBalancerPort),
       ipList(params->bind_ips),
 #ifndef _WIN32
       useUnixSockets(!params->noUnixSocket),
@@ -296,63 +262,6 @@ TransportLayerASIO::Options::Options(const ServerGlobalParams* params,
       maxConns(params->maxConns) {
 }
 
-TransportLayerASIO::TimerService::TimerService(Options opt)
-    : _reactor(std::make_shared<TransportLayerASIO::ASIOReactor>()) {
-    if (opt.spawn)
-        _spawn = std::move(opt.spawn);
-}
-
-TransportLayerASIO::TimerService::~TimerService() {
-    stop();
-}
-
-void TransportLayerASIO::TimerService::start() {
-    // Skip the expensive lock acquisition and `compareAndSwap` in the common path.
-    if (MONGO_likely(_state.load() != State::kInitialized))
-        return;
-
-    // The following ensures only one thread continues to spawn a thread to run the reactor. It also
-    // ensures concurrent `start()` and `stop()` invocations are serialized. Holding the lock
-    // guarantees that the following runs either before or after running `stop()`. Note that using
-    // `compareAndSwap` while holding the lock is for simplicity and not necessary.
-    auto lk = stdx::lock_guard(_mutex);
-    auto precondition = State::kInitialized;
-    if (_state.compareAndSwap(&precondition, State::kStarted)) {
-        _thread = _spawn([reactor = _reactor] {
-            LOGV2_INFO(5490002, "Started a new thread for the timer service");
-            reactor->run();
-            LOGV2_INFO(5490003, "Returning from the timer service thread");
-        });
-    }
-}
-
-void TransportLayerASIO::TimerService::stop() {
-    // It's possible for `stop()` to be called without `start()` having been called (or for them to
-    // be called concurrently), so we only proceed with stopping the reactor and joining the thread
-    // if we've already transitioned to the `kStarted` state.
-    auto lk = stdx::lock_guard(_mutex);
-    if (_state.swap(State::kStopped) != State::kStarted)
-        return;
-
-    _reactor->stop();
-    _thread.join();
-}
-
-std::unique_ptr<ReactorTimer> TransportLayerASIO::TimerService::makeTimer() {
-    return _getReactor()->makeTimer();
-}
-
-Date_t TransportLayerASIO::TimerService::now() {
-    return _getReactor()->now();
-}
-
-Reactor* TransportLayerASIO::TimerService::_getReactor() {
-    // TODO SERVER-57253 We can start this service as part of starting `TransportLayerASIO`.
-    // Then, we can remove the following invocation of `start()`.
-    start();
-    return _reactor.get();
-}
-
 TransportLayerASIO::TransportLayerASIO(const TransportLayerASIO::Options& opts,
                                        ServiceEntryPoint* sep,
                                        const WireSpec& wireSpec)
@@ -361,8 +270,7 @@ TransportLayerASIO::TransportLayerASIO(const TransportLayerASIO::Options& opts,
       _egressReactor(std::make_shared<ASIOReactor>()),
       _acceptorReactor(std::make_shared<ASIOReactor>()),
       _sep(sep),
-      _listenerOptions(opts),
-      _timerService(std::make_unique<TimerService>()) {}
+      _listenerOptions(opts) {}
 
 TransportLayerASIO::~TransportLayerASIO() = default;
 
@@ -599,47 +507,13 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(
         (sslMode == kGlobalSSLMode &&
          ((globalSSLMode == SSLParams::SSLMode_preferSSL) ||
           (globalSSLMode == SSLParams::SSLMode_requireSSL)))) {
-        // The handshake is complete once either of the following passes the finish line:
-        // - The thread running the handshake returns from `handshakeSSLForEgress`.
-        // - The thread running `TimerService` cancels the handshake due to a timeout.
-        auto finishLine = std::make_shared<StrongWeakFinishLine>(2);
-
-        // Schedules a task to cancel the synchronous handshake if it does not complete before the
-        // specified timeout.
-        auto timer = _timerService->makeTimer();
-#ifndef _WIN32
-        // TODO SERVER-62035: enable the following on Windows.
-        if (timeout > Milliseconds(0)) {
-            timer->waitUntil(_timerService->now() + timeout)
-                .getAsync([finishLine, session](Status status) {
-                    if (status.isOK() && finishLine->arriveStrongly())
-                        session->end();
-                });
-        }
-#endif
-
         Date_t timeBefore = Date_t::now();
         auto sslStatus = session->handshakeSSLForEgress(peer).getNoThrow();
         Date_t timeAfter = Date_t::now();
-
         if (timeAfter - timeBefore > kSlowOperationThreshold) {
             networkCounter.incrementNumSlowSSLOperations();
         }
 
-        if (finishLine->arriveStrongly()) {
-            timer->cancel();
-        } else if (!sslStatus.isOK()) {
-            // We only take this path if the handshake times out. Overwrite the socket exception
-            // with a network timeout.
-            auto errMsg = fmt::format("SSL handshake timed out after {}",
-                                      (timeAfter - timeBefore).toString());
-            sslStatus = Status(ErrorCodes::NetworkTimeout, errMsg);
-            LOGV2(5490001,
-                  "Timed out while running handshake",
-                  "peer"_attr = peer,
-                  "timeout"_attr = timeout);
-        }
-
         if (!sslStatus.isOK()) {
             return sslStatus;
         }
@@ -664,11 +538,8 @@ StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncCon
 #ifdef TCP_FASTOPEN_CONNECT
     const auto family = protocol.family();
     if ((family == AF_INET) || (family == AF_INET6)) {
-        setSocketOption(sock,
-                        TCPFastOpenConnect(gTCPFastOpenClient),
-                        "connect (sync) TCP fast open",
-                        logv2::LogSeverity::Info(),
-                        ec);
+        setSocketOption(
+            sock, TCPFastOpenConnect(gTCPFastOpenClient), ec, "connect (sync) TCP fast open");
         if (tcpFastOpenIsConfigured) {
             return errorCodeToStatus(ec);
         }
@@ -716,8 +587,6 @@ StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncCon
 #endif
         return std::make_shared<ASIOSession>(
             this, std::move(sock), false, *endpoint, transientSSLContext);
-    } catch (const asio::system_error& e) {
-        return errorCodeToStatus(e.code());
     } catch (const DBException& e) {
         return e.toStatus();
     }
@@ -823,9 +692,8 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(
             std::error_code ec;
             setSocketOption(connector->socket,
                             TCPFastOpenConnect(gTCPFastOpenClient),
-                            "connect (async) TCP fast open",
-                            logv2::LogSeverity::Info(),
-                            ec);
+                            ec,
+                            "connect (async) TCP fast open");
             if (tcpFastOpenIsConfigured) {
                 return futurize(ec);
             }
@@ -834,17 +702,11 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(
         })
         .then([this, connector, sslMode, transientSSLContext]() -> Future<void> {
             stdx::unique_lock<Latch> lk(connector->mutex);
-            connector->session = [&] {
-                try {
-                    return std::make_shared<ASIOSession>(this,
-                                                         std::move(connector->socket),
-                                                         false,
-                                                         *connector->resolvedEndpoint,
-                                                         transientSSLContext);
-                } catch (const asio::system_error& e) {
-                    iasserted(errorCodeToStatus(e.code()));
-                }
-            }();
+            connector->session = std::make_shared<ASIOSession>(this,
+                                                               std::move(connector->socket),
+                                                               false,
+                                                               *connector->resolvedEndpoint,
+                                                               transientSSLContext);
             connector->session->ensureAsync();
 
 #ifndef MONGO_CONFIG_SSL
@@ -1053,11 +915,7 @@ Status TransportLayerASIO::setup() {
 
 #ifndef _WIN32
     if (_listenerOptions.useUnixSockets && _listenerOptions.isIngress()) {
-        listenAddrs.push_back(makeUnixSockPath(_listenerOptions.port));
-
-        if (_listenerOptions.loadBalancerPort) {
-            listenAddrs.push_back(makeUnixSockPath(*_listenerOptions.loadBalancerPort));
-        }
+        listenAddrs.emplace_back(makeUnixSockPath(_listenerOptions.port));
     }
 #endif
 
@@ -1073,35 +931,28 @@ Status TransportLayerASIO::setup() {
     _listenerPort = _listenerOptions.port;
     WrappedResolver resolver(*_acceptorReactor);
 
-    std::vector<int> ports = {_listenerPort};
-    if (_listenerOptions.loadBalancerPort) {
-        ports.push_back(*_listenerOptions.loadBalancerPort);
-    }
-
     // Self-deduplicating list of unique endpoint addresses.
     std::set<WrappedEndpoint> endpoints;
-    for (const auto& port : ports) {
-        for (const auto& listenAddr : listenAddrs) {
-            if (listenAddr.empty()) {
-                LOGV2_WARNING(23020, "Skipping empty bind address");
-                continue;
-            }
-
-            const auto& swAddrs =
-                resolver.resolve(HostAndPort(listenAddr, port), _listenerOptions.enableIPv6);
-            if (!swAddrs.isOK()) {
-                LOGV2_WARNING(23021,
-                              "Found no addresses for {peer}",
-                              "Found no addresses for peer",
-                              "peer"_attr = swAddrs.getStatus());
-                continue;
-            }
-            const auto& addrs = swAddrs.getValue();
-            endpoints.insert(addrs.begin(), addrs.end());
+    for (auto& ip : listenAddrs) {
+        if (ip.empty()) {
+            LOGV2_WARNING(23020, "Skipping empty bind address");
+            continue;
         }
+
+        auto swAddrs =
+            resolver.resolve(HostAndPort(ip, _listenerPort), _listenerOptions.enableIPv6);
+        if (!swAddrs.isOK()) {
+            LOGV2_WARNING(23021,
+                          "Found no addresses for {peer}",
+                          "Found no addresses for peer",
+                          "peer"_attr = swAddrs.getStatus());
+            continue;
+        }
+        auto& addrs = swAddrs.getValue();
+        endpoints.insert(addrs.begin(), addrs.end());
     }
 
-    for (const auto& addr : endpoints) {
+    for (auto& addr : endpoints) {
 #ifndef _WIN32
         if (addr.family() == AF_UNIX) {
             if (::unlink(addr.toString().c_str()) == -1 && errno != ENOENT) {
@@ -1125,15 +976,9 @@ Status TransportLayerASIO::setup() {
         } catch (std::exception&) {
             // Allow the server to start when "ipv6: true" and "bindIpAll: true", but the platform
             // does not support ipv6 (e.g., ipv6 kernel module is not loaded in Linux).
-            auto bindAllFmt = [](auto p) { return fmt::format(":::{}", p); };
-            bool addrIsBindAll = addr.toString() == bindAllFmt(_listenerPort);
-
-            if (!addrIsBindAll && _listenerOptions.loadBalancerPort) {
-                addrIsBindAll = (addr.toString() == bindAllFmt(*_listenerOptions.loadBalancerPort));
-            }
-
+            const auto bindAllIPv6Addr = ":::"_sd + std::to_string(_listenerPort);
             if (errno == EAFNOSUPPORT && _listenerOptions.enableIPv6 && addr.family() == AF_INET6 &&
-                addrIsBindAll) {
+                addr.toString() == bindAllIPv6Addr) {
                 LOGV2_WARNING(4206501,
                               "Failed to bind to address as the platform does not support ipv6",
                               "Failed to bind to {address} as the platform does not support ipv6",
@@ -1143,19 +988,13 @@ Status TransportLayerASIO::setup() {
 
             throw;
         }
-        setSocketOption(acceptor,
-                        GenericAcceptor::reuse_address(true),
-                        "acceptor reuse address",
-                        logv2::LogSeverity::Info());
+        setSocketOption(acceptor, GenericAcceptor::reuse_address(true), "acceptor reuse address");
 
         std::error_code ec;
 #ifdef TCP_FASTOPEN
         if (gTCPFastOpenServer && ((addr.family() == AF_INET) || (addr.family() == AF_INET6))) {
-            setSocketOption(acceptor,
-                            TCPFastOpen(gTCPFastOpenQueueSize),
-                            "acceptor TCP fast open",
-                            logv2::LogSeverity::Info(),
-                            ec);
+            setSocketOption(
+                acceptor, TCPFastOpen(gTCPFastOpenQueueSize), ec, "acceptor TCP fast open");
             if (tcpFastOpenIsConfigured) {
                 return errorCodeToStatus(ec);
             }
@@ -1163,8 +1002,7 @@ Status TransportLayerASIO::setup() {
         }
 #endif
         if (addr.family() == AF_INET6) {
-            setSocketOption(
-                acceptor, asio::ip::v6_only(true), "acceptor v6 only", logv2::LogSeverity::Info());
+            setSocketOption(acceptor, asio::ip::v6_only(true), "acceptor v6 only");
         }
 
         acceptor.non_blocking(true, ec);
@@ -1288,10 +1126,9 @@ void TransportLayerASIO::_runListener() noexcept {
 
 Status TransportLayerASIO::start() {
     stdx::unique_lock lk(_mutex);
-    if (_isShutdown) {
-        LOGV2(6986801, "Cannot start an already shutdown TransportLayer");
-        return ShutdownStatus;
-    }
+
+    // Make sure we haven't shutdown already
+    invariant(!_isShutdown);
 
     if (_listenerOptions.isIngress()) {
         _listener.thread = stdx::thread([this] { _runListener(); });
@@ -1311,10 +1148,6 @@ void TransportLayerASIO::shutdown() {
         return;
     }
 
-    lk.unlock();
-    _timerService->stop();
-    lk.lock();
-
     if (!_listenerOptions.isIngress()) {
         // Egress only reactors never start a listener
         return;
@@ -1354,8 +1187,6 @@ ReactorHandle TransportLayerASIO::getReactor(WhichReactor which) {
 void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
     auto acceptCb = [this, &acceptor](const std::error_code& ec,
                                       ASIOSession::GenericSocket peerSocket) mutable {
-        transportLayerASIOhangBeforeAccept.pauseWhileSet();
-
         if (auto lk = stdx::lock_guard(_mutex); _isShutdown) {
             return;
         }
@@ -1382,26 +1213,7 @@ void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
         try {
             std::shared_ptr<ASIOSession> session(
                 new ASIOSession(this, std::move(peerSocket), true));
-            if (session->isFromLoadBalancer()) {
-                session->parseProxyProtocolHeader(_acceptorReactor)
-                    .getAsync([this, session = std::move(session)](Status s) {
-                        if (s.isOK()) {
-                            _sep->startSession(std::move(session));
-                        }
-                    });
-            } else {
-                _sep->startSession(std::move(session));
-            }
-        } catch (const asio::system_error& e) {
-            // Swallow connection reset errors. Connection reset errors classically present as
-            // asio::error::eof, but can bubble up as asio::error::invalid_argument when calling
-            // into socket.set_option().
-            if (e.code() != asio::error::eof && e.code() != asio::error::invalid_argument) {
-                LOGV2_WARNING(5746600,
-                              "Error accepting new connection: {error}",
-                              "Error accepting new connection",
-                              "error"_attr = e.code().message());
-            }
+            _sep->startSession(std::move(session));
         } catch (const DBException& e) {
             LOGV2_WARNING(23023,
                           "Error accepting new connection: {error}",
