@@ -30,31 +30,40 @@
 #pragma once
 
 #include <type_traits>
+#include <utility>
 
 #include "mongo/platform/compiler.h"
 
 namespace mongo {
 
+/**
+ * Stores a callback to be invoked upon destruction.
+ *
+ * `F` is normally deduced from the initializer.
+ * Examples:
+ *
+ *     ScopeGuard cleanup([&] { ... });
+ *     auto cleanup = ScopeGuard([&] { ... });
+ *     ScopeGuard cleanup = [&] { ... };
+ */
 template <typename F>
 class [[nodiscard]] ScopeGuard {
 public:
-    explicit ScopeGuard(const F& f) : _func(f) {}
-    explicit ScopeGuard(F && f) : _func(std::move(f)) {}
+    template <typename FuncArg>
+    ScopeGuard(FuncArg && f) : _func(std::forward<FuncArg>(f)) {}
 
-    // Can only be move-constructed, to support being returned from a function.
+    // Remove all move and copy, MCE (mandatory copy elision) covers us here.
     ScopeGuard(const ScopeGuard&) = delete;
-    ScopeGuard(ScopeGuard && o) : _func(std::move(o._func)), _dismissed(o._dismissed) {
-        o._dismissed = true;
-    }
+    ScopeGuard(ScopeGuard &&) = delete;
     ScopeGuard& operator=(const ScopeGuard&) = delete;
     ScopeGuard& operator=(ScopeGuard&&) = delete;
 
-    ~ScopeGuard() {
-        if (_dismissed)
-            return;
-        _func();
+    ~ScopeGuard() noexcept {
+        if (!_dismissed)
+            _func();
     }
 
+    /** A dismissed ScopeGuard does not invoke its callback. */
     void dismiss() noexcept {
         _dismissed = true;
     }
@@ -64,6 +73,10 @@ private:
     bool _dismissed = false;
 };
 
+template <typename F>
+ScopeGuard(F &&)->ScopeGuard<std::decay_t<F>>;
+
+/** Obsolete: `ScopeGuard<F>` can deduce the `F` type. */
 template <typename F>
 auto makeGuard(F&& fun) {
     return ScopeGuard<std::decay_t<F>>(std::forward<F>(fun));
@@ -75,4 +88,9 @@ auto makeGuard(F&& fun) {
 #define MONGO_SCOPEGUARD_CAT(s1, s2) MONGO_SCOPEGUARD_CAT2(s1, s2)
 #define MONGO_SCOPEGUARD_ANON(str) MONGO_SCOPEGUARD_CAT(str, __LINE__)
 
-#define ON_BLOCK_EXIT(...) auto MONGO_SCOPEGUARD_ANON(onBlockExit) = makeGuard(__VA_ARGS__)
+/**
+ * Declares a ScopeGuard having a variable name based on line number.
+ * Example:
+ *     ON_BLOCK_EXIT([&] { ... });
+ */
+#define ON_BLOCK_EXIT ScopeGuard MONGO_SCOPEGUARD_ANON(onBlockExit)
